Package: copBasic
Title: General Bivariate Copula Theory and Many Utility Functions
Version: 2.2.14
Date: 2026-05-21
Authors@R: person(given = "William",
                  family = "Asquith",
                  role = c("aut", "cre"),
                  email = "william.asquith@ttu.edu",
		  comment = c(ORCID = "0000-0002-7400-1861"))
Description: Extensive functions for bivariate copula (bicopula) computations and related operations
 for bicopula theory. The lower, upper, product, and select other bicopula are implemented along
 with operations including the diagonal, survival copula, dual of a copula, co-copula, and
 numerical bicopula density. Level sets, horizontal and vertical sections are supported. Numerical
 derivatives and inverses of a bicopula are provided through which simulation is implemented.
 Bicopula composition, convex combination, asymmetry extension, and products also are provided.
 Support extends to the Kendall Function as well as the Lmoments thereof. Kendall Tau,
 Spearman Rho and Footrule, Gini Gamma, Blomqvist Beta, Hoeffding Phi, Schweizer-
 Wolff Sigma, tail dependency, tail order, skewness, and bivariate Lmoments are implemented, and
 positive/negative quadrant dependency, left (right) increasing (decreasing) are available.
 Other features include Kullback-Leibler Divergence, Vuong Procedure, spectral measure, and
 Lcomoments for fit and inference, Lcomoment ratio diagrams, maximum likelihood,
 and AIC, BIC, and RMSE for goodness-of-fit.
Maintainer: William Asquith <william.asquith@ttu.edu>
Repository: CRAN
Depends: R (>= 4.0.0)
Imports: lmomco, mvtnorm, randtoolbox
Suggests: copula
License: GPL-2
NeedsCompilation: no
Packaged: 2026-05-22 00:48:07 UTC; wasquith
Author: William Asquith [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7400-1861>)
Date/Publication: 2026-05-22 06:20:03 UTC
Built: R 4.5.2; ; 2026-05-22 11:19:31 UTC; unix

Package: copBasic
Title: General Bivariate Copula Theory and Many Utility Functions
Version: 2.2.14
Date: 2026-05-21
Authors@R: person(given = "William",
                  family = "Asquith",
                  role = c("aut", "cre"),
                  email = "william.asquith@ttu.edu",
		  comment = c(ORCID = "0000-0002-7400-1861"))
Description: Extensive functions for bivariate copula (bicopula) computations and related operations
 for bicopula theory. The lower, upper, product, and select other bicopula are implemented along
 with operations including the diagonal, survival copula, dual of a copula, co-copula, and
 numerical bicopula density. Level sets, horizontal and vertical sections are supported. Numerical
 derivatives and inverses of a bicopula are provided through which simulation is implemented.
 Bicopula composition, convex combination, asymmetry extension, and products also are provided.
 Support extends to the Kendall Function as well as the Lmoments thereof. Kendall Tau,
 Spearman Rho and Footrule, Gini Gamma, Blomqvist Beta, Hoeffding Phi, Schweizer-
 Wolff Sigma, tail dependency, tail order, skewness, and bivariate Lmoments are implemented, and
 positive/negative quadrant dependency, left (right) increasing (decreasing) are available.
 Other features include Kullback-Leibler Divergence, Vuong Procedure, spectral measure, and
 Lcomoments for fit and inference, Lcomoment ratio diagrams, maximum likelihood,
 and AIC, BIC, and RMSE for goodness-of-fit.
Maintainer: William Asquith <william.asquith@ttu.edu>
Repository: CRAN
Depends: R (>= 4.0.0)
Imports: lmomco, mvtnorm, randtoolbox
Suggests: copula
License: GPL-2
NeedsCompilation: no
Packaged: 2026-05-22 00:48:07 UTC; wasquith
Author: William Asquith [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7400-1861>)
Date/Publication: 2026-05-22 06:20:03 UTC
Built: R 4.6.0; ; 2026-05-25 03:24:02 UTC; unix

 importFrom("grDevices", "dev.list", "rgb", "grey")
 importFrom("graphics", "abline", "axTicks", "axis", "contour", "legend", "lines",
                       "mtext", "par", "plot", "points", "rug")
 importFrom("stats", "approx", "complete.cases", "cor", "dnorm",
            "integrate", "median", "qnorm", "quantile", "runif", "var",
            "uniroot", "optimise", "optim", "pnorm", "pt", "qt", "sd",
            "dbeta", "pbeta", "pgamma", "qgamma")
 importFrom("mvtnorm", "GenzBretz", "Miwa", "TVPACK", "pmvnorm", "pmvt")
 importFrom("randtoolbox", "halton", "sobol", "torus")
 importFrom("utils", "installed.packages", "str")
 importFrom("lmomco", "disfitqua", "lmr2par", "lmrloc", "lmoms", "lmom2par", "par2qua", "pp",
                      "rlmomco", "qlmomco", "vec2par")
# --------------------------------------------------------------------------------------------------
export(copBasic.fitpara.beta)
export(AMHcop)
export(CIRCcop)
export(CLcop)
export(COP)
export(COPinv)
export(COPinv2)
export(EMPIRcop)
export(EMPIRcopdf)
export(EMPIRgrid)
export(EMPIRgrid_fast)
export(EMPIRgrid_lkup)
export(EMPIRgridder)
export(EMPIRgridder2)
export(EMPIRgridderinv)
export(EMPIRgridderinv2)
export(EMPIRmed.regress)
export(EMPIRmed.regress2)
export(EMPIRqua.regress)
export(EMPIRqua.regress2)
export(EMPIRsim)
export(EMPIRsimv)
export(EuvCOP)
export(EvuCOP)
export(FGMcop)
export(FGMicop)
export(FRcop)
export(FRECHETcop)
export(GHcop)
export(GLcop)
export(GLEVcop)
export(GLPMcop)
export(HRcop)
export(LpCOP)
export(LpCOPradsym)
export(LpCOPpermsym)
export(LzCOPpermsym)
export(M)
export(M_N5p12b)
export(MOcop)
export(N4212cop)
export(N4220cop)
export(rN4220cop)
export(ORDSUMcop)
export(ORDSUWcop)
export(P)
export(PARETOcop)
export(PAcop)
export(PLACKETTcop)
export(PLcop)
export(PLACKETTpar)
export(PLpar)
export(PLACKETTsim)
export(PSP)
export(RAYcop)
export(RFcop)
export(W)
export(W_N5p12a)
export(aicCOP)
export(asCOP)
export(bilmoms)
export(bicCOP)
export(bicoploc)
export(blomCOP)
export(blomCOPss)
export(blomatrixCOPdec)
export(blomatrixCOPiqr)
export(breveCOP)
export(coCOP)
export(composite1COP)
export(khoudraji1COP)
export(khoudrajiPCOP)
export(composite2COP)
export(khoudraji2COP)
export(composite3COP)
export(concordCOP)
export(convexCOP)
export(convex2COP)
export(densityCOP)
export(densityCOPplot)
export(derCOP)
export(derCOP2)
export(derCOPinv)
export(derCOPinv2)
export(diagCOP)
export(diagCOPatf)
export(diagCOPinv)
export(duCOP)
export(footCOP)
export(gEVcop)
export(giniCOP)
export(glueCOP)
export(gridCOP)
export(HAIRPINcop)
export(hoefCOP)
export(JOcopB5)
export(JOcopBB4)
export(JOMAcop)
export(joeskewCOP)
export(jointCOP)
export(joint.curvesCOP)
export(joint.curvesCOP2)
export(isfuncCOP)
export(isCOP.permsym)
export(isCOP.radsym)
export(isCOP.LTD)
export(isCOP.RTI)
export(isCOP.PQD)
export(kmeasCOP)
export(kfuncCOP)
export(kfuncCOPinv)
export(kfuncCOPlmom)
export(kfuncCOPlmoms)
export(kullCOP)
export(kullCOPint)
export(kfuncCOP_Pd)
export(lcomoms2.ABcop2parameter)
export(lcomoms2.ABKGcop2parameter)
export(lcomCOP)
export(lcomCOPpv)
export(level.curvesCOP)
export(level.curvesCOP2)
export(level.setCOP)
export(level.setCOP2)
export(med.regressCOP)
export(med.regressCOP2)
export(mleCOP)
export(NORMcop)
export(nuskewCOP)
export(nustarCOP)
export(psepolar)
export(prod2COP)
export(qua.regressCOP)
export(qua.regressCOP.draw)
export(qua.regressCOP2)
export(rCOP)
export(rhoCOP)
export(rhobevCOP)
export(rmseCOP)
export(sectionCOP)
export(semicorCOP)
export(simCOP)
export(simCOPmicro)
export(simCOPv)
export(simcomposite3COP)
export(simcompositeCOP)
export(spectralmeas)
export(statTn)
export(surCOP)
export(surfuncCOP)
export(stabtaildepf)
export(tailconCOP)
export(taildepCOP)
export(tailordCOP)
export(tauCOP)
export(tEVcop)
export(Tcop)
export(uvskew)
export(uvlmoms)
export(vuongCOP)
export(wolfCOP)
export(wolfCOPsamc)
export(wolfCOPtest)

This directory contains code for generation of 3D copula figures


The perl script shown by example in the next line

3dcopula.pl 3dcopula_config_example.txt 

drives R and LaTeX to create 3dcopula.pdf. The program creates a bunch of TMP*.txt files and creates a file called 3dcopula_fromperl.tex, which provides a sort of core communication mechansim from the program to LaTeX. The file 3dcopula_ancillary.tex (if exists) is read just before\begin{document} is executed in the 3dcopula.tex file.


3dcopula.pl --clean

cleans up temporary files and exits, nothing else is done



The examples directory contains additional example configuration files. Let us run a few simple ones:

3dcopula.pl 3dconfig_examples/circ.txt
3dcopula.pl 3dconfig_examples/PI.txt
3dcopula.pl --nsimcop=0  3dconfig_examples/M.txt
3dcopula.pl --nsimcop=20 3dconfig_examples/W.txt

library(copBasic)
library(lmomco)
library(quantreg)

lo <- .Machine$double.eps; hi <- 1 - lo
xpara <- lmomco::vec2par(c(3, 0.6, +0), type="pe3")
ypara <- lmomco::vec2par(c(3, 0.4, -0), type="pe3")

ff    <- c(0.00001, 0.0001, 0.001, seq(0.01, 0.99, by=0.01), 0.999, 0.9999, 0.99999)
cpara <- list(cop=COP, para=list(cop=CLcop, para=8), reflect=3, alpha=0, beta=0.3)
UVt <- simCOP(200, cop=composite1COP, para=cpara, graphics=FALSE, resamv01=TRUE)
xsam <- qlmomco(UVt[,1], xpara)
ysam <- qlmomco(UVt[,2], ypara)
xsap <- lmom2par(lmoms(xsam), type="pe3")
ysap <- lmom2par(lmoms(ysam), type="pe3")


xthe  <- qlmomco(ff, xpara)
xldc  <- qlmomco(ff, xsap)

if(sign(cor(xsam, ysam, method="spearman")) < 0) {
  ythe  <- qlmomco(1-ff, ypara)
  yldc  <- qlmomco(1-ff, ysap )
} else {
  ythe  <- qlmomco(  ff, ypara)
  yldc  <- qlmomco(  ff, ysap )
}

xout  <- qlmomco(ff, xsap)
yout  <- qlmomco(ff, ysap)
ffx   <- plmomco(xout, xsap)
ffy   <- plmomco(yout, ysap)
ffyu  <- EvuCOP(ffx, cop=composite1COP, para=cpara, delu=.Machine$double.eps^0.25)
ffxv  <- EuvCOP(ffy, cop=composite1COP, para=cpara, delv=.Machine$double.eps^0.25)
mmyu  <- med.regressCOP( ffx, cop=composite1COP, para=cpara)
mmxv  <- med.regressCOP2(ffy, cop=composite1COP, para=cpara)
youtu <- qlmomco(ffyu, ysap)
xoutv <- qlmomco(ffxv, xsap)
ymmtu <- qlmomco(mmyu$V, ysap)
xmmtv <- qlmomco(mmxv$U, xsap)

plot(xthe, ythe, type="l", lwd=3)
points(xsam, ysam, pch=16, col=grey(0, 0.1))
#points(qlmomco(pp(xsam, sort=FALSE, a=0), xsap),
#       qlmomco(pp(ysam, sort=FALSE, a=0), ysap), pch=16, col=grey(0, 0.5))



lines(xout,  youtu, col="palegreen1", lwd=2, lty=1, type="b")
lines(xoutv, yout,  col="palegreen3", lwd=2, lty=1, type="b")
lines(xout,  ymmtu, col="purple1",    lwd=2, lty=2)
lines(xmmtv, yout,  col="purple4",    lwd=2, lty=2)


lines(xldc, yldc, col="cyan")

print(sqrt(mean((xout-xthe)^2 + (yout-ythe)^2)))

stop()

medss <- quantreg::rqss(ysam~qss(xsam, constraint="D"), tau=0.5)
lines(sort(xsam), predict(medss, newdata=data.frame(xsam=sort(xsam))), col="orange", lwd=2)
medss <- quantreg::rqss(xsam~qss(ysam, constraint="D"), tau=0.5)
lines(sort(ysam), predict(medss, newdata=data.frame(ysam=sort(ysam))), col="orange", lwd=2)


gamss <- mgcv::gam(y~s(x), gamma=1)
lines(sort(x), predict(gamss, newdata=data.frame(x=sort(x))), col="blue")

gamss <- mgcv::gam(y~s(x), gamma=2)
lines(sort(x), predict(gamss, newdata=data.frame(x=sort(x))), col="blue", lty=2)

gamss <- mgcv::gam(y~s(x), gamma=3)
lines(sort(x), predict(gamss, newdata=data.frame(x=sort(x))), col="blue", lty=3)


zz <- bicoploc(x,y, xout=xout, xpara=xpara, ypara=ypara, plotuv=FALSE, plotxy=FALSE)
 lines(zz$organic$xout, zz$organic$bicoploc_emp, col="red", lwd=2)
 lines(zz$organic$xout, zz$organic$bicoploc, col="red", lwd=2, lty=2)
 lines(zz$organic$xout, zz$organic$locpair, col="lightblue", lwd=2, lty=2)

"qua.regressCOP" <-
function(f=0.5, u=seq(0.01,0.99, by=0.01), cop=NULL, para=NULL, ...) {
  if(is.null(cop)) {
     warning("must have copula argument specified, returning NULL")
     return(NULL)
  }
  V <- sapply(1:length(u), function(i) {
          v <- derCOPinv(cop=cop, u[i], f, para=para, ...)
          if(is.na(v)) {
             warning("could not uniroot in derCOPinv, skipping i=",i," with U=", u[i])
             return(NA)
          }
          return(v) } )
  z <- data.frame(U=u,V=V)
  return(z)
}

"EMPIRgrid" <-
function(para=NULL, deluv=0.05, gridonly=FALSE, verbose=FALSE, ...) {

  if(is.null(para)) {
    warning("parameters (the observed u and v) for empirical copula are NULL")
    return(NULL)
  }
  if(is.list(para)) {
    if(exists("para", para)) {
      para$para <- as.data.frame(para$para)
      if(ncol(para$para) != 2) {
        warning("a data.frame having only two columns is required in the 'para$para' argument")
        return(NULL)
      }
    }
  } else if(ncol(para) != 2) {
     warning("a data.frame having only two columns is required in the 'para' argument")
     return(NULL)
  } else {
    para <- as.data.frame(para)
  }

  us <- seq(0,1, by=deluv);
  vs <- seq(0,1, by=deluv);
  nu <- length(us);
  nv <- length(vs);
  cop <- matrix(nrow=nu, ncol=nv)
  if(verbose) message("Index ")
  for(i in 1:nu) {
     if(verbose) message(nu - i,"-", appendLF=FALSE)
     cop[i,] <- EMPIRcop(rep(us[i], nv), vs, para=para, ...)
  }
  if(verbose) message("done")

  rownames(cop) <- as.character(us)
  colnames(cop) <- as.character(vs)
  if(gridonly) return(cop)
  zzz <- list(u=us, v=us, empcop=cop, deluv=deluv)
  zzz$ctype  <- "unknown"
  dots <- list(...)
  if(exists("ctype", dots)) zzz$ctype <- dots$ctype
  class(zzz) <- c("empirical.copula.grid", class(zzz))
  return(zzz)
}

"kullCOP" <-
function(cop1=NULL, cop2=NULL, para1=NULL, para2=NULL, alpha=0.05,
         del=0, n=1E5, verbose=TRUE, sobol=FALSE, scrambling=0, ...) {

    if(del > 0) {
      lo <- del; hi <- 1 - del # integration limits
    } else {
      lo <- 0; hi <- 1
    }

    UV <- NULL
    if(sobol) {
       if(! exists(".Random.seed")) tmp <- runif(1) # insures definition
       seed <- sample(.Random.seed, 1)
       UV <- randtoolbox::sobol(n=n, dim=2, seed=seed, scrambling=scrambling)
    } else {
       UV <- matrix(data=runif(2*n), ncol=2)
    }
    if(verbose) message("kullCOP: Computing 'f' density values---",
                        appendLF=FALSE)
    f <- densityCOP(UV[,1],UV[,2], cop=cop1, para=para1, ...)
    f[f == 0] <- .Machine$double.eps
    if(verbose) message("done")

    if(verbose) message("kullCOP: Computing 'g' density values---",
                        appendLF=FALSE)
    g <- densityCOP(UV[,1],UV[,2], cop=cop2, para=para2, ...)
    g[g == 0] <- .Machine$double.eps
    if(verbose) message("done")

    h <- g*(log(g/f))
    h <- h[is.finite(h)]
    h <- h[!  is.nan(h)]
    KLdivergence.fg    <- mean(h)
    KLdivergence.fg.sd <-   sd(h)/sqrt(n)

    h <- f*(log(f/g))
    h <- h[is.finite(h)]
    h <- h[!  is.nan(h)]
    KLdivergence.gf    <- mean(h)
    KLdivergence.gf.sd <-   sd(h)/sqrt(n)

    JEFF.divergence <- KLdivergence.fg + KLdivergence.gf
    names(JEFF.divergence) <- "Jeffrey Divergence"

    h <- g*log(g/f)^2
    h <- h[is.finite(h)]
    h <- h[!  is.nan(h)]
    KLvar.fg <- mean(h)
    KLvar.fg.sd <- sd(h)/sqrt(n)

    h <- f*log(f/g)^2
    h <- h[is.finite(h)]
    h <- h[!  is.nan(h)]
    KLvar.gf <- mean(h)
    KLvar.gf.sd <- sd(h)/sqrt(n)

    #    Warning messages:
    #1: In sqrt(KLvar.fg - KLdivergence.fg^2) : NaNs produced
    #2: In kullCOP(cop1 = EMPIRcop, para1 = UVaem, cop2 = EMPIRcop, para2 = UVnoaem) :
    #  NAs introduced by coercion to integer range

    suppressWarnings(sigmaKL.fg <- sqrt(KLvar.fg - KLdivergence.fg^2))
    if(is.nan(sigmaKL.fg)) {
      sigmaKL.fg <- 0
    }
    # message("KLvar.fg=",        KLvar.fg)
    # message("KLdivergence.fg=", KLdivergence.fg)

    sigmaKL.gf <- sqrt(KLvar.gf - KLdivergence.gf^2)
    if(is.nan(sigmaKL.gf)) {
      sigmaKL.gf <- 0
    }
    # message("KLvar.gf=",KLvar.gf)
    # message("KLdivergence.gt=",KLdivergence.gf)

    tmp <- c(sigmaKL.fg/KLdivergence.fg, sigmaKL.gf/KLdivergence.gf)
    tmp <- max(tmp[! is.nan(tmp)]) # even need with other protections above?

    KL.sample.size <- (qnorm(1-alpha) * tmp)^2

    diverge   <- c(KLdivergence.fg, sigmaKL.fg,
                   KLdivergence.gf, sigmaKL.gf)
    divergesd <- c(KLdivergence.fg.sd, KLvar.fg.sd,
                   KLdivergence.gf.sd, KLvar.gf.sd)
    names(diverge)   <- c("KL-diverge.fg", "sigmaKL-diverge.fg",
                          "KL-diverge.gf", "sigmaKL-diverge.gf")
    names(divergesd) <- c("StdDev_KL-diverge.fg", "StdDev_KL-variance.fg",
                          "StdDev_KL-diverge.gf", "StdDev_KL-variance.gf")
    SS <- as.integer(KL.sample.size)
    names(SS) <- "Kullback-Leibler sample size"
    zz <- list(MonteCarlo.sim.size = n,
               divergences         = diverge,
               stdev.divergences   = divergesd,
               Jeffrey.divergence  = JEFF.divergence,
               KL.sample.size      = SS)
    return(zz)
}


"kullCOPint" <-
 function(cop1=NULL, cop2=NULL, para1=NULL, para2=NULL, alpha=0.05,
          del=.Machine$double.eps^0.25, verbose=TRUE, ...) {

    if(del > 0) {
      lo <- del; hi <- 1 - del # integration limits
    } else {
      lo <- 0; hi <- 1
    }

    # Solve equation 5.7 in Joe (2015)
    if(verbose) message("  CPU on Kullback-Leibler, double integrations for divergence ",
                        "(f relative to g)")
    KLfg <- NULL
    try(KLfg <- integrate(function(u) {
        sapply(u, function(u) {
            integrate(function(v) {
              f <- densityCOP(u,v, cop=cop1, para=para1, ...)
              g <- densityCOP(u,v, cop=cop2, para=para2, ...)
              return(g*(log(g/f)))
            }, lo, hi)$value
        })
    }, lo, hi))
    #print(KLfg);stop()
    if(is.null(KLfg)) {
       warning("could not numerically integrate, Kullback-Leibler divergence, ",
               "(f relative to g)")
       KLfg            <- NA
       KLdivergence.fg <- NA
    } else {
       KLdivergence.fg <- KLfg$value
    }

    # Solve equation 5.8 in Joe (2015)
    if(verbose) message("  CPU on Kullback-Leibler, double integrations for divergence ",
                        "(g relative to f)")
    KLgf <- NULL
    try(KLgf <- integrate(function(u) {
        sapply(u, function(u) {
            integrate(function(v) {
              f <- densityCOP(u,v, cop=cop1, para=para1, ...)
              g <- densityCOP(u,v, cop=cop2, para=para2, ...)
              return(f*log(f/g))
            }, lo, hi)$value
        })
    }, lo, hi))
    if(is.null(KLgf)) {
       warning("could not numerically integrate, Kullback-Leibler divergence ",
               "(g relative to f)")
       KLgf            <- NA
       KLdivergence.gf <- NA
    } else {
       KLdivergence.gf <- KLgf$value
    }

    # Solve Jeffrey's Divergence Joe (2015)
    JEFF.divergence <- KLdivergence.fg + KLdivergence.gf
    names(JEFF.divergence) <- "Jeffrey Divergence"
    #JEFF <- NULL
    #try(JEFF <- integrate(function(u) {
    #    sapply(u, function(u) {
    #        integrate(function(v) {
    #          f <- densityCOP(u,v, cop=cop1, para=para1, ...)
    #          g <- densityCOP(u,v, cop=cop2, para=para2, ...)
    #          return((g-f)*log(g/f))
    #        }, lo, hi)$value
    #    })
    #}, lo, hi))
    #print(JEFF)


    # Solve equation 5.9 in Joe (2015)
    if(verbose) message("  CPU on Kullback-Leibler, double integrations for variance ",
                        "(f relative to g)")
    KLvar.fg <- NULL
    try(KLvar.fg <- integrate(function(u) {
        sapply(u, function(u) {
            integrate(function(v) {
              f <- densityCOP(u,v, cop=cop1, para=para1, ...)
              g <- densityCOP(u,v, cop=cop2, para=para2, ...)
              return(g*log(g/f)^2)
            }, lo, hi)$value
        })
    }, lo, hi))
    if(is.null(KLvar.fg)) {
       warning("could not numerically integrate, Kullback-Leibler variance ",
               "(f relative to g)")
       KLvar.fg   <- NA
       sigmaKL.fg <- NA
    } else {
       sigmaKL.fg <- sqrt(KLvar.fg$value - KLdivergence.fg^2)
    }

    if(verbose) message("  CPU on Kullback-Leibler, double integrations for variance ",
                        "(g relative to f)")
    KLvar.gf <- NULL
    try(KLvar.gf <- integrate(function(u) {
        sapply(u, function(u) {
            integrate(function(v) {
              f <- densityCOP(u,v, cop=cop1, para=para1, ...)
              g <- densityCOP(u,v, cop=cop2, para=para2, ...)
              return(f*log(f/g)^2)
            }, lo, hi)$value
        })
    }, lo, hi))
    if(is.null(KLvar.gf)) {
       warning("could not numerically integrate, Kullback-Leibler variance ",
               "(g relative to f)")
       KLvar.gf   <- NA
       sigmaKL.gf <- NA
    } else {
       sigmaKL.gf <- sqrt(KLvar.gf$value - KLdivergence.gf^2)
    }

    alpha <- alpha
    tmp <- max(c(sigmaKL.fg/KLdivergence.fg, sigmaKL.gf/KLdivergence.gf))
    KL.sample.size <- (qnorm(1-alpha) * tmp)^2

    yy <- list(KLintegrate.fg    = KLfg,
               KLintegrate.gf    = KLgf,
               KLvarintegrate.fg = KLvar.fg,
               KLvarintegrate.gf = KLvar.gf)
    diverge <- c(KLdivergence.fg, sigmaKL.fg,
                 KLdivergence.gf, sigmaKL.gf)
    names(diverge) <- c("KL-diverge.fg", "sigmaKL-diverge.fg",
                        "KL-diverge.gf", "sigmaKL-diverge.gf")
    SS <- as.integer(KL.sample.size)
    names(SS) <- "Kullback-Leibler sample size"
    zz <- list(divergences = diverge,
               Jeffrey.divergence=JEFF.divergence,
               KL.sample.size=SS,
               integrations = yy)
    return(zz)
}

"kmeasCOP" <-
function(z, cop=NULL, para=NULL, wrtV=FALSE, as.sample=FALSE, verbose=FALSE,
            subdivisions=100L, rel.tol=.Machine$double.eps^0.25, abs.tol=rel.tol, ...) {
   kfuncCOP(z, cop=cop, para=para, wrtV=wrtV, as.sample=as.sample, verbose=verbose, ...)
}

"kfuncCOP" <-
function(z, cop=NULL, para=NULL, wrtV=FALSE, as.sample=FALSE, verbose=FALSE,
            subdivisions=100L, rel.tol=.Machine$double.eps^0.25, abs.tol=rel.tol, ...) {
   as.sample <- as.character(as.sample)
   if(as.sample != "FALSE") {
      if(length(para[1,]) != 2) {
         warning("ambiguous UV provided by 'para', need only two columns")
         return(NA)
      }

      if(as.sample=="genest" | as.sample=="Genest" ) {
         n <- nrow(para); ns <- seq_len(n)
         R <- rank(para[,1]); S <- rank(para[,2])
         "VIN" <- function(i) sum(as.numeric(R <= R[i] & S <= S[i])) / n
         FKin <- sapply(z, function(t) {
                sum(sapply(ns, function(j) as.numeric(VIN(j) <= t) )) / n })
         return(FKin)
      } else if(as.sample == "charpentier" | as.sample == "Charpentier") {
        # Charpentier, Arthur, 2012, Kendall's function for copulas:
        # http://freakonometrics.hypotheses.org/1126  (12/09/2012)
        n  <- nrow(para);               ns <- seq_len(n)
        i  <- rep(ns, each=n); j <- rep(ns,n)
        S  <- (para[i,1] > para[j,1]) & (para[i,2] > para[j,2])
        Z  <- tapply(S, i, sum) / (n-1)
        zz <- data.frame(t=sort(Z), Kc=(ns)/n)
        return(zz)
      } else {
         n     <- nrow(para)
         FKin  <- sort(EMPIRcop(para[,1], para[,2], para=para, ...))
         Zin   <- ( rank(FKin) - 0.5 ) / n
         suppressWarnings( empkc <- approx(c(0, FKin, 1), c(0, Zin, 1), xout=z)$y )
         return(empkc)
      }

   }

    if(is.null(cop)) {
      warning("must have copula argument specified, returning NULL")
      return(NULL)
   }

   "KCfunc1" <- function(the.z) {
       if(the.z == 0) return(0)
       myint <- NULL
       try( myint <- integrate(function(u) {
                     cvt <- sapply(1:length(u), function(i) {
                             COPinv(u=u[i], t=the.z, cop=cop, para=para, ...)})
                     derCOP(u, cvt, cop=cop, para=para, ...)  }, the.z, 1,
                     subdivisions=subdivisions, rel.tol=rel.tol, abs.tol=abs.tol),
            silent=! verbose)
       if(is.null(myint)) {
          if(verbose) warning("error on integration encountered ",
                              "(some copula singularity?)")
          return(0)
       }
       KC <- the.z + myint$value
       if(KC >  1) KC <- 1 # assume rounding errors just breaking through
       if(KC <  0) KC <- 0 # again for rounding errors
       return(KC)
   }
   "KCfunc2" <- function(the.z) {
       if(the.z == 0) return(0)
       myint <- NULL
       try( myint <- integrate(function(v) {
                     cut <- sapply(1:length(v), function(i) {
                             COPinv2(v=v[i],t=the.z, cop=cop, para=para, ...)})
                     derCOP2(u=cut, v=v, cop=cop, para=para, ...)  }, the.z, 1,
                     subdivisions=subdivisions, rel.tol=rel.tol, abs.tol=abs.tol),
            silent=! verbose)
       if(is.null(myint)) {
          if(verbose) warning("error on integration encountered ",
                              "(some copula singularity?)")
          return(0)
       }
       KC <- the.z + myint$value
       if(KC >  1) KC <- 1 # assume rounding errors just breaking through
       if(KC <  0) KC <- 0 # again for rounding errors
       return(KC)
   }
   KCfunc <- KCfunc1
   if(wrtV) KCfunc <- KCfunc2
   kc <- sapply(1:length(z), function(i) { KCfunc(z[i]) } )
   return(kc)
}

"NORMcop" <-
function(u,v, para=NULL, rho=NULL, tau=NULL,
              fit=c('rho', 'tau'), ...) {

  if(is.null(para)) {
    fit <- match.arg(fit)
    zz  <- list(para=c(NA, NA), rho=NA, tau=NA)
    if(is.null(tau) & is.null(rho)) {
      if(fit == "rho") {
        zz$rho <- cor(u,v, method="spearman")
        p <- 2 * sin(pi/6 * zz$rho)
      } else {
        zz$tau <- cor(u,v, method="kendall")
        p <-     sin(pi/2 * zz$tau)
      }
    } else {
      if(fit == "rho") {
        p <- 2 * sin(pi/6 *    rho)
        zz$rho <- rho
      } else {
        p <-     sin(pi/2 *    tau)
        zz$tau <- tau
      }
    }
    zz$para <- p
    names(zz$para) <- c("theta")
    return(zz)
  }

  p  <- para[1]

  if(p < -1 | p > 1) {
    warning("Parameter must be -1 <= Theta <= 1")
    return(NULL)
  }

  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
    warning("length u = ", length(u), " and length v = ", length(v))
    warning("longer object length is not a multiple of shorter object length, ",
            "no recycling")
    return(NA)
  }

  if(length(u) == 1) {
    u <- rep(u, length(v))
  } else if (length(v) == 1) {
    v <- rep(v, length(u))
  }

  # Mimic of copula::normalCopula.R:::pmvnormAlgo
  mimic_pmvnormAlgo <- function(dim, x, checkCorr=FALSE, ...) {
    if(dim <= 3 && ! anyNA(x) && (! any(xI <- x == Inf) || all(xI))) {
      mvtnorm::TVPACK(...)
    } else if(dim <= 5) {
      mvtnorm::Miwa(checkCorr=checkCorr, ...)
    } else {
      mvtnorm::GenzBretz(...)
    }
  }

  upru  <- stats::qnorm(u)
  uprv  <- stats::qnorm(v)
  lwr   <- c(-Inf, -Inf)
  sigma <- matrix(c(1, p, p, 1), nrow=2)

  cop <- sapply(seq_len(length(u)), function(i) { # Mimic of copula::ptCopula
                         x <- c(upru[i], uprv[i]); algo <- mimic_pmvnormAlgo(2, x)
                  mvtnorm::pmvnorm(lower=lwr, upper=x, sigma=sigma, algorithm=algo) })

  # See the comments in Tcop.R for some details about the algorithm therein that do not appear
  # as critical for the NORMcop.R but the mimic_pmvnormAlgo() is used anyway herein to parallel
  # the general nature of the Tcop.R.

  return(cop)
}

"COPinv2" <-
function(cop=NULL, v, t, para=NULL, silent=TRUE, ...) {
    if(t == 0) return(0)
    if(t == 1) return(v)
    if(v == t) return(1)
    #str(cop)
    func <- function(x, v, LHS, cop, para=para, ...) {
            return(LHS - COP(cop=cop, v=v, u=x, para=para, ...))
    }
    lo <- COP(0,0, cop=cop, para=para, ...)
    if(is.na(lo) | ! is.finite(lo)) lo <- .Machine$double.eps
    my.rt <- NULL
    try(my.rt <- uniroot(func, interval=c(lo,1),
                         v=v, LHS=t, cop=cop, para=para, ...), silent=silent)

    if(is.null(my.rt)) return(NA)
    if(length(my.rt$root) != 0) {
      u <- my.rt$root
      return(u)
    } else {
      message(c("COPinv2: v=", v, ", t=", t, "\n"))
      return(NA)
    }
}

"prod2COP" <-
function(u,v, cop1=NULL, para1=NULL, cop2=NULL, para2=NULL,
              para=NULL, pinterval=NULL, ...) {

   if(! is.null(para)) {
      cop1  <- para$cop1;    cop2  <- para$cop2
      para1 <- para$para1;   para2 <- para$para2
      if(exists("pinterval", para)) pinterval <- get("pinterval", para)
   }

   if(is.null(cop1)) {
        warning("must have first copula specified, returning NULL")
        return(NULL)
   }
   if(is.null(cop2)) {
        warning("must have second copula specified, returning NULL")
        return(NULL)
   }

   if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
      warning("length u = ", length(u), " and length v = ", length(v))
      warning("longer object length is not a multiple of shorter object length, no recycling")
      return(NA)
   }
   if(length(u) == 1) {
      u <- rep(u, length(v))
   }
   else if(length(v) == 1) {
      v <- rep(v, length(u))
   }

   # d/du derCOP and d/dv derCOP2  (Nelsen, 2006, eq. 6.4.2)
   "afunc" <- function(t, U=NA, V=NA, ...) derCOP( t,V, cop=cop2, para=para2, ...) *
                                           derCOP2(U,t, cop=cop1, para=para1, ...)
   lo <- 0; hi <- 1
   if(! is.null(pinterval)) { lo <- pinterval[1]; hi <- pinterval[2] }
   return(sapply(1:length(u), function(i) integrate(afunc, lo,hi, U=u[i], V=v[i],
                           cop1=cop1, cop2=cop2, para1=para1, para2=para2, ...)$value))
}

"qua.regressCOP.draw" <-
function(f=seq(0.1, 0.9, by=0.1),
         fs=0.5, cop=NULL, para=NULL, ploton=TRUE,
         wrtV=FALSE, col=c(4,2), lwd=c(1,2), lty=1, ...) {
  if(ploton) {
    plot(c(0,1),c(0,1), type="n", lwd=3,
         xlab="U, NONEXCEEDANCE PROBABILITY", ylab="V, NONEXCEEDANCE PROBABILITY")
  }
  Fs <- sort(c(f,fs))
  for(af in Fs) {
    mycol <- col[1]; mylwd <- lwd[1]
    if(af == fs) { mycol <- col[2]; mylwd <- lwd[2] }
    if(wrtV == FALSE) {
      R <- qua.regressCOP(f=af, cop=cop, para=para, ...)
    } else {
      R <- qua.regressCOP2(f=af, cop=cop, para=para, ...)
    }
    lines(R$U,R$V, col=mycol, lty=lty, lwd=mylwd)
  }
}

"EvuCOP" <-
function(u=seq(0.01, 0.99, by=0.01), cop=NULL, para=NULL, asuv=FALSE, nsim=1E5,
         subdivisions=200L, rel.tol=.Machine$double.eps^0.25, abs.tol=rel.tol, ...) {

   v <- sapply(u, function(t) {
          E <- NULL
          try(E <- integrate(function(k) { 1 - derCOP( cop=cop, para=para, t, k, ...) },
                    lower=0, upper=1, subdivisions=subdivisions,
                    rel.tol=rel.tol, abs.tol=abs.tol)$value, silent=TRUE)
          if(is.null(E)) {
            E <- mean( 1 - derCOP(cop=cop, para=para, t, runif(nsim), ...) )
            names(E) <- "sim"
          }
          return(E)
        })
   v[v <= 0] <- 0
   v[v >= 1] <- 1
   if(asuv) return(data.frame(U=u, V=v))
   return(v)
}

"simCOPv" <-
function(u, cop=NULL, para=NULL,
            reflect=c("cop", "surv", "acute", "grave",
                        "1",    "2",     "3",     "4"), ...) {
   simCOPmicro(u, cop=cop, para=para, reflect=reflect, ...)
}


"simCOPmicro" <-
function(u, cop=NULL, para=NULL, seed=NULL,
            reflect=c("cop", "surv", "acute", "grave",
                        "1",    "2",     "3",     "4"), ...) {
  reflect <- match.arg(reflect)

  if(! is.null(seed)) set.seed(seed)

  n <- length(u); t <- runif(n); v <- vector(mode="numeric", length=n)
  v <- switch(reflect,
    cop   = sapply(1:n, function(i) {     derCOPinv(cop=cop,   u[i],   t[i], para=para, ...) }),
    surv  = sapply(1:n, function(i) { 1 - derCOPinv(cop=cop, 1-u[i], 1-t[i], para=para, ...) }),
    acute = sapply(1:n, function(i) {     derCOPinv(cop=cop, 1-u[i],   t[i], para=para, ...) }),
    grave = sapply(1:n, function(i) { 1 - derCOPinv(cop=cop,   u[i], 1-t[i], para=para, ...) }),
    "1"   = sapply(1:n, function(i) {     derCOPinv(cop=cop,   u[i],   t[i], para=para, ...) }),
    "2"   = sapply(1:n, function(i) { 1 - derCOPinv(cop=cop, 1-u[i], 1-t[i], para=para, ...) }),
    "3"   = sapply(1:n, function(i) {     derCOPinv(cop=cop, 1-u[i],   t[i], para=para, ...) }),
    "4"   = sapply(1:n, function(i) { 1 - derCOPinv(cop=cop,   u[i], 1-t[i], para=para, ...) }))
  if(any(is.na(v))) warning("could not uniroot at least for one element in derCOPinv")
  return(v)
}

# surv  is a reflection on the horizontal AND vertical axes
# acute is a reflection on the horizontal axis
# grave is a reflection on the verical axis

"FGMcop" <- function(u, v, para=c(NA, 1, 1), ...) {
    if(is.null(para)) {
      warning("self parameter estimation not supported")
      return(NULL)
    }
    if(length(para) == 1) {
      para <- c(para, 1,1)
    }
    para <- as.numeric(para)
    if(length(para) != 3) {
      warning("copula requires three parameters")
      return(NULL)
    }
    d <- para[1]; a <- para[2]
    n <- as.integer(para[3])
    lwr <- -min(c(1,1/(n*a^2))); upr <- 1/(n*a)
    if(is.na(d)) {
      warning("parameter Theta is NA")
      return(NULL)
    } else if(d < lwr) {
      warning("parameter Theta < ", lwr)
      return(NULL)
    } else if(d > upr) {
      warning("parameter Theta > ", upr)
      return(NULL)
    }
    if(a <= 0) {
      warning("parameter Alpha <= 0")
      return(NULL)
    }

    if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
      warning("length u = ", length(u), " and length v = ", length(v))
      warning("longer object length is not a multiple of shorter object length, ",
              "no recycling")
      return(NA)
    }
    if(length(u) == 1) {
       u <- rep(u, length(v))
    } else if(length(v) == 1) {
       v <- rep(v, length(u))
    }

    return(u*v*(1+d*(1-u^a)*(1-v^a))^n)
}


FGMicop <- function(u,v, para=NULL, ...) {
    if(is.null(para)) {
      warning("self parameter estimation not supported")
      return(NULL)
    }
    if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
      warning("length u = ", length(u), " and length v = ", length(v))
      warning("longer object length is not a multiple of shorter object length, ",
              "no recycling")
      return(NA)
    }
    if(length(u) == 1) {
      u <- rep(u, length(v))
    }
    else if(length(v) == 1) {
      v <- rep(v, length(u))
    }

    b <- para; r <- length(b)
    if(any(abs(b) > 1)) {
      warning("one of the Beta parameter is |beta| > 1")
      return(NULL)
    }
    uv <- u*v; uvp <- (1-u)*(1-v)
    tmp <- sapply(1:length(u), function(i) {
               sum(sapply(1:r, function(j) {
                k <- as.integer( j   /2) + 1
                l <- as.integer((j+1)/2)
                b[j] * uv[i]^k * uvp[i]^l })) })
    cop <- uv + tmp
    return(cop)
}

"diagCOP" <-
function(cop=NULL, para=NULL, secondary=FALSE,
         ploton=TRUE, lines=TRUE, delt=0.005, ...) {
  if(ploton) {
    plot(c(0,1), c(0,1), type="n",
         xlab="U=V, NONEXCEEDANCE PROBABILITY",
         ylab="H, NONEXCEEDANCE PROBABILITY")
  }
  TT <- seq(0+delt, 1-delt, delt)
  C <- sapply(TT, function(x) {
          	        y <- x
          	        if(secondary) y <- 1 - y
          	        return( cop(x,y, para=para, ...)) } )
  if(lines & ! is.null(dev.list())) lines(TT, C, ...)
  txt <- "primary"
  if(secondary) txt <- "secondary"
  return(list(t=TT, diagcop=C, diagtype=txt))
}

"HAIRPINcop" <-
function(u, v, para=c(1, 2, 1), ...) {
  if(is.null(para)) {
    warning("para can not be NULL to HAIRPINcop")
    return(NULL)
  }
  if(length(para) != 3) {
    warning("para must be length 3 in HAIRPINcop")
    return(NULL)
  }
  a <- as.numeric(para[1])
  p <- as.numeric(para[2])
  reflect <- as.character(as.integer(para[3]))
  if(a < 1) {
    warning("para[1] must be >= 1 in HAIRPINcop")
    return(NULL)
  }
  if(p < 1 | p > 2) {
    warning("para[2] must be 1 <= para[2] <= 2 in HAIRPINcop")
    return(NULL)
  }

  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
    warning("length u = ",length(u), " and length v = ",length(v))
    warning("longer object length is not a multiple of shorter object length, ",
            "no recycling")
    return(NA)
  }

         if(length(u) == 1) {
    u <- rep(u, length(v))
  } else if(length(v) == 1) {
    v <- rep(v, length(u))
  }

  cdiag <- function(x) a*x^p

  HPcop <- function(myu, myv) {
            sapply(seq_len(length(myu)), function(i) {
               min(c(myu[i], myv[i], mean(cdiag(c(myu[i], myv[i]))) ))
             }) }
  zz <- switch(reflect,
               `1`=            HPcop(    u,     v),
               `2`=u + v - 1 + HPcop(1 - u, 1 - v),
               `3`=    v -     HPcop(1 - u,     v),
               `4`=u -         HPcop(    u, 1 - v))
  return(zz)
}

# Durante, F., Fernandez-Sanchez, J., and Trutschnig, W., 2014, Multivariate copulas with
# hairpin support: Journal of Multivariate Analysis, v. 130, pp. 323-334, \doi{10.1016/j.jmva.2014.06.009}.

"wolfCOP" <-
function(cop=NULL, para=NULL, as.sample=FALSE, brute=FALSE, delta=0.002,
         nlarge=500, usefastgrid=TRUE, ...) {

    if(as.sample) {
      if(is.null(para)) {
         warning("Sample Schweizer-Wolff Sigma desired but para is NULL, ",
                 "returning NULL")
         return(NULL)
      }
      if(length(names(para)) != 2) {
        warning("para argument must be data.frame having only two columns, ",
                "returning NULL")
        return(NULL)
      }
      n <- nrow(para)
      if(usefastgrid & n > nlarge) {
        ef <- EMPIRgrid_fast(para=para, gridonly=FALSE, ...)
        pf <- ef$empcop
        # gu <- gv <- as.numeric( rownames(ef$empcop) ) # a bit more speed
        gu <- as.numeric( rownames(ef$empcop) ) # to use only a single vector of u = v
        for(i in seq_len(nrow(pf))) pf[i,] <- gu * gu[i] # in lieu of gv * gu[i] for instance
               samSIG <- 12 * sum(abs(ef$empcop - pf)) / (n^2 - 1)
        names(samSIG) <- "Sigma:EMPIRgrid_fast"
        return(samSIG)
      } else {
         if(as.sample == -1) message("Sample Schweizer-Wolff Sigma",
                                     "---CPU intensive!")
         # https://www.cs.cmu.edu/~bapoczos/articles/poczos11nipscopula.pdf
         #                                                (August 11, 2015)
         nn <- n^2; ns <- seq_len(n)
         R <- rank(para[,1]); S <- rank(para[,2])
         samSIG <- sum(sapply(ns, function(i) {
                   sum(sapply(ns, function(j) {
                   abs((sum(as.numeric(R <= i & S <= j))/n) - (i*j/nn))
             } )) } ))
               samSIG <- (12/(nn - 1)) * samSIG
        names(samSIG) <- "Sigma:Poczos et al."
        return(samSIG)
      }
   }

   if(is.null(cop)) {
      warning("must have copula argument specified, returning NULL")
      return(NULL)
   }

   if(brute) {
      us <- vs <- seq(.Machine$double.eps, 1-.Machine$double.eps, delta)
      wolf <- sum(sapply(us, function(u) {
                 sum(sapply(vs, function(v) {
                    abs(cop(u,v, para=para, ...) - u*v) })) }))
      wolf <- 12*wolf*delta^2
      #names(wolf) <- "Sigma:brute"
      return(wolf)
   }

   myint <- NULL
   try(myint <- integrate(function(u) {
            sapply(u,function(u) {
                      integrate(function(v) {
                          abs(COP( u, v, cop=cop, para=para, ...) - u*v)},
                      0, 1)$value
            })}, 0, 1) )
   wolf <- ifelse(is.null(myint), NA, 12*myint$value)
   #names(wolf) <- "Sigma:integrate()"
   return(wolf)
}

"semicorCOP" <-
function(cop=NULL, para=NULL, truncation=0, n=0, as.sample=FALSE, ...) {
   rhoNs <- list(cor.normal.scores=NA,
                 botleft.semicor=NA,   topright.semicor=NA,
                 topleft.semicor=NA,   botright.semicor=NA,
                 type="TO BE FILLED IN BY CODE", source="semicorCOP")
   if(truncation < 0) {
      warning("inconsistent truncation argument, returning NULL")
   }
   method <- "pearson"
   a <- truncation
   if(as.sample) {
      if(is.null(para)) {
         warning("Sample semi-correlations are desired by 'para' but it is NULL, ",
                 "returning NULL")
         return(NULL)
      }
      if(length(names(para)) != 2) {
         warning("para argument must be data.frame having only two columns, ",
                 "returning NULL")
         return(NULL)
      }
      para <- para[para[,1] > 0 & para[,1] < 1 & para[,2] > 0 & para[,2] < 1, ]
      n <- length(para[,1])  # Hazen plotting positions
      para[,1] <- (rank(para[,1]) - 0.5) / n
      para[,2] <- (rank(para[,2]) - 0.5) / n
            qu <- qnorm(para[,1])
            qv <- qnorm(para[,2])
      rhoNs$cor.normal.scores <- cor(qu,qv, method=method)
      rhoNs$topright.semicor      <- cor(qu[qu >  a & qv >  a],
                                     qv[qu >  a & qv >  a], method=method)
      rhoNs$botleft.semicor     <- cor(qu[qu < -a & qv < -a],
                                     qv[qu < -a & qv < -a], method=method)
      rhoNs$botright.semicor <- cor(qu[qu >  a & qv < -a],
                                     qv[qu >  a & qv < -a], method=method)
      rhoNs$topleft.semicor <- cor(qu[qu < -a & qv >  a],
                                     qv[qu < -a & qv >  a], method=method)
      rhoNs$type <- "cor()'s on columns in 'para'"
   } else {
      if(is.null(cop)) {
         warning("must have copula argument specified, returning NULL")
         return(NULL)
      }
      if(n == 0) {
         warning("must have sample size argument specified, a section of code ",
                 "using integration is not implemented, returning NULL")
         return(NULL)
      } else {
         UV <- simCOP(n=n, cop=cop, para=para, graphics=FALSE, ...)
         para <- UV[UV[,1] > 0 & UV[,1] < 1 & UV[,2] > 0 & UV[,2] < 1, ]
         n <- length(para[,1]) # Hazen plotting positions
         para[,1] <- (rank(para[,1]) - 0.5) / n
         para[,2] <- (rank(para[,2]) - 0.5) / n
               qu <- qnorm(para[,1]);
               qv <- qnorm(para[,2])
         rhoNs$cor.normal.scores <- cor(qu,qv, method=method)
         rhoNs$topright.semicor      <- cor(qu[qu >  a & qv >  a],
                                        qv[qu >  a & qv >  a], method=method)
         rhoNs$botleft.semicor     <- cor(qu[qu < -a & qv < -a],
                                        qv[qu < -a & qv < -a], method=method)
         rhoNs$botright.semicor <- cor(qu[qu >  a & qv < -a],
                                        qv[qu >  a & qv < -a], method=method)
         rhoNs$topleft.semicor <- cor(qu[qu < -a & qv >  a],
                                        qv[qu < -a & qv >  a], method=method)
         rhoNs$type <- "simulated the copula, then computed cor()'s"
      }
   }
   return(rhoNs)
}

"MOcop" <- function(u,v, para=NULL, lcomCOP=NULL, ...) { # Marshall-Olkin copula
   if(! is.null(lcomCOP)) {
     ofunc <- function(par, srho=NA, T3_12=NA, T3_21=NA) {
       lcm <- lcomCOP(cop=MOcop, para=pnorm(par))
       return((lcm$lcomUV[2] - srho )^2 + # look carefully, the 2, 3, 3 index
              (lcm$lcomUV[3] - T3_12)^2 + # use on the lcm list are correct, so do not
              (lcm$lcomVU[3] - T3_21)^2)  # expect to see 1, 2, 3 or 2, 3, 4.
     }
     rt <- NULL
     try(rt <- optim(c(0,0), ofunc, srho=lcomCOP[1], T3_12=lcomCOP[2], T3_21=lcomCOP[3]), silent=TRUE)
     if(is.null(rt)) {
       warning("could not optim the alpha, beta given the rhotau")
       return(NULL)
     }
     rt$para <- pnorm(rt$par)
     return(rt)
   }

   if(length(u) == 1) {
     u <- rep(u, length(v))
   } else if (length(v) == 1) {
     v <- rep(v, length(u))
   }
   alpha <- para[1]; beta <- para[2]
   f <- vector(mode="numeric", length(u))
   wnt <- u^alpha >= v^beta
   f[  wnt] <- u[  wnt]^(1-alpha) * v[  wnt]
   f[! wnt] <- u[! wnt]           * v[! wnt]^(1-beta)
   return(f)
}

"EMPIRgridder" <-
function(empgrid=NULL, ...) {

  if(is.null(empgrid)) {
    warning("The gridded empirical copula (say from EMPIRgrid) is NULL")
    return(NULL)
  }
  if(! is.list(empgrid)) {
    warning("The gridded empirical copula is expected as a list from EMPIRgrid")
    return(NULL)
  }

  deluv <- empgrid$deluv
  empcop <- empgrid$empcop
  rc <- dim(empcop)

  n <- rc[1]
  if(n != rc[2]) {
     warning("grid is not square!")
     return(NA)
  }
  if(deluv != 1/(n-1)) {
     warning("concerns over value of deluv, not congruent with matrix size")
  }

  the.deriv <- matrix(nrow=n, ncol=n)
  for(i in 1:n) {
     section <- empcop[,i]
     diff.section  <- diff(section)
     derivative    <- c(0, diff.section/deluv)
     the.deriv[,i] <- derivative
  }

  for(i in 2:n) {
     #denominator <- the.deriv[i,n]
     #print(c(i,the.deriv[i,n]))
     #print(c(i,denominator))
     the.deriv[i,] <- the.deriv[i,]/the.deriv[i,n]
     #print(c(i,sum(the.deriv[i,])))
     #if(is.nan(the.deriv[i,1])) stop(sum(the.deriv[i,]))
  }
  the.deriv[1,] <- rep(NA, n)

  for(i in 2:n) {
    if(length(the.deriv[! is.finite(the.deriv[i,])]) > 0) {
      #print(empcop[,i]) # this would have been the second
      warning("found nonfinite values on row=",i," in grid derivative")
      next
    }
  }

  attributes(the.deriv) <- list(dim=dim(empcop),
                                rownames=empgrid$u,
                                colnames=empgrid$v,
                                message="use the rows!, wrt U")

  return(the.deriv)
}

"tailconCOP" <- function(t, cop=NULL, para=NULL, ...) {
   if(any(t < 0)) {
      warning("at least one t < 0, returning NULL")
      return(NULL)
   }
   if(any(t > 1)) {
      warning("at least one t > 1, returning NULL")
      return(NULL)
   }
   if (is.null(cop)) {
      warning("must have copula argument specified, returning NULL")
      return(NULL)
   }
   I <- ifelse(t < 0.5, 1, 0)
   delC <- COP(t, t, cop=cop, para=para, ...)
   ( delC/t * I) + ((1 - 2*t + delC)/(1-t) * ! I)
}

"stabtaildepf" <-
function(uv=NULL, xy=NULL, k=function(n) as.integer(0.5*n),
             levelset=TRUE, ploton=TRUE, title=TRUE, delu=0.01,
             smooth=FALSE, ...) {
   if(is.null(uv)) {
      warning(" 'uv' is NULL, returning NULL"); return(NULL)
   }
   if(is.null(xy) & ! levelset) {
      warning(" 'xy' is NULL and levelset=FALSE, returning NULL"); return(NULL)
   }
   n <- length(uv[,1])
   if(is.function(k)) k <- k(n)
   if(levelset) {
      if(ploton) {
         plot(c(0,1), c(0,1), type="n",
              xlab="RELATIVE DISTANCE TO X END POINT",
              ylab="RELATIVE DISTANCE TO Y END POINT")
         if(title) mtext("Stable Tail Dependence Function")
      }
      if(! smooth) {
         afunc <- function(y, c=NULL, x=NULL, k=NULL, ...) {
          abs(c - stabtaildepf(xy=c(x,y), uv=uv, k=k, levelset=FALSE, ...)) }
         ZZ <- new.env()
         for(c in seq(0.1,1,by=0.1)) {
            x <- seq(0,c, by=delu)
            y <- sapply(x, function(j) {
                      optimise(afunc, interval=c(0,1), c=c, x=j, k=k)$minimum })
            x <- c(x,c); y <- c(y,0)
            lines(x,y, ...)
            assign(as.character(c), data.frame(x=x,y=y), envir=ZZ)
         }
         return(as.list(ZZ))
      } else {
         Hlis <- stabtaildepf(xy=NA, uv=uv, smooth=TRUE, levelset=FALSE, k=k)
         bfunc <- function(y, c=NULL, x=NULL, ...) {
                                  sol <- sapply(y, function(g) { 2*sum(Hlis$p3 *
                                         sapply(1:Hlis$Nn, function(i) {
                                  max(c(Hlis$Wn[i]*x, (1-Hlis$Wn[i])*g)) })) })
                                  abs(c - sol) }
         ZZ <- new.env()
         for(c in seq(0.1,1,by=0.1)) {
            x <- seq(0,c,by=delu)
            y <- sapply(x, function(j) {
                           optimise(bfunc, interval=c(0,1), c=c, x=j)$minimum })
            x <- c(x,c); y <- c(y,0)
            lines(x,y, ...)
            assign(as.character(c), data.frame(x=x,y=y), envir=ZZ)
         }
         return(as.list(ZZ))
      }
   } else {
      if(smooth) {
         WS <- psepolar(uv, ...)$table
         Sf <- sort(WS$Shat, decreasing=TRUE)[k+1]
         n <- length(WS$Shat); ix <- 1:n
         In <- ix[WS$Shat > Sf]; Nn <- length(In)
         Wn <- WS$What[In]
         Wbar <- mean(Wn); Sw <- 1/var(Wn)
         p3 <- (1 - (Wbar - 0.5)*Sw*(Wn - Wbar))/Nn
         zz <- list(Nn=Nn, Wn=Wn, p3=p3)
         return(zz)
      } else {
         R1 <- rank(uv[,1]); R2 <- rank(uv[,2])
         lhat <- (1/k)*sum((R1 > n + 1 - k*xy[1] | R2 > n + 1 - k*xy[2]))
         return(lhat)
      }
   }
}

"FRECHETcop" <- function(u,v, para=NULL, rho=NULL, tau=NULL, par2rhotau=FALSE, ...) {
   if(is.null(para)) {
      if(! is.null(tau) & ! is.null(rho)) {
         if(tau < -1 | tau > 1) {
            warning("tau not in [-1,1], returning NULL")
            return(NULL)
         }
         if(rho < -1 | rho > 1) {
            warning("tau not in [-1,1], returning NULL")
            return(NULL)
         }
         b <- (3*tau - rho^2 - 2*rho)/(2*rho); a <- rho + b
         if(is.nan(b)) a <- b <- 0
         if(a >= 0 & b >= 0 & (a + b) <= 1) {
            para <- c(a,b); names(para) <- c("alpha", "beta")
            return(list(para=para, source="FRECHETcop"))
         } else {
            warning("incompatible rho and tau: (",rho,", ",
                    tau,"), parameters a and b not a,b >= 0 & a + b <= 1")
            para <- c(NA,NA); names(para) <- c("alpha", "beta")
            return(list(para=para, source="FRECHETcop"))
         }
      } else {
         warning("could not determine parameters by rho and tau, returning NULL")
         return(NULL)
      }
   }
   para <- as.numeric(para); a <- para[1]; b <- para[2]
   if(par2rhotau) {
      if(a >= 0 & b >= 0 & (a + b) <= 1) {
         tau <- ((a-b)*(a+b+2))/3; rho <- a - b
         return(list(rho=rho, tau=tau, para=para, source="FRECHETcop"))
      } else {
         warning("parameters a and b not a,b >= 0 & a + b <= 1, returning NULL")
         return(NULL)
      }
   }
   if(a >= 0 & b >= 0 & (a + b) <= 1) {
      return(a*M(u,v) + (1-a-b)*P(u,v) + b*W(u,v))
   }
   warning("parameters a and b not a,b >= 0 & a + b <= 1, returning NULL")
   return(NULL)
}

"convexCOP" <- function(u,v, para, ...) {
  nm <- names(para)
  cops.idx <- grep("cop\\d+",  nm)
  pars.idx <- grep("para\\d+", nm)
  n <- length(cops.idx)
  p <- length(pars.idx)
  if(n != p) {
     warning("number of copulas and parameters are not equal")
     return()
  }
  if(length(grep("weights", nm)) == 0) {
     weights <- rep(1/n, n)
  } else {
     weights <- get("weights", para)
  }

  weights   <- weights/sum(weights)
  cop.names <- nm[cops.idx]; para.names <- nm[pars.idx]

  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
   warning("length u = ", length(u), " and length v = ", 
           length(v))
   warning("longer object length is not a multiple of shorter object length, ", 
           "no recycling")
   return(NA)
  }
  if(length(u) == 1) {
    u <- rep(u, length(v))
  } else if (length(v) == 1) {
    v <- rep(v, length(u))
  }

  HH <- sapply(1:length(u), function(k) {
          gg <- sapply(1:n, function(i) {
                    the.cop  <- get(cop.names[i],  para)
                    the.para <- get(para.names[i], para)
                    weights[i]*the.cop(u[k],v[k], para=the.para) })
          return(sum(gg)) })
  return(HH)
}

"Tcop" <-
function(u,v, para=NULL, rho=NULL, tau=NULL, taildep=NULL,
              fit=c('rho', 'tau'), ...) {

  if(is.null(para)) {
    lamUL <- function(p,nu) 2 * stats::pt(-(sqrt(nu+1)*sqrt(1-p)) / sqrt(1+p), df=nu+1)
    fit   <- match.arg(fit)
    zz    <- list(para=c(NA, NA), rho=NA, tau=NA, lambdaUL=NA, message=NA, lambdaUL_for_nueqone=NA)
    if(is.null(tau) & is.null(rho)) {
      if(fit == "rho") {
        zz$rho <- cor(u,v, method="spearman")
        p <- 2 * sin(pi/6 * zz$rho)
      } else {
        zz$tau <- cor(u,v, method="kendall")
        p <-     sin(pi/2 * zz$tau)
      }
    } else {
      if(fit == "rho") {
        p <- 2 * sin(pi/6 *    rho)
        zz$rho <- rho
      } else {
        p <-     sin(pi/2 *    tau)
        zz$tau <- tau
      }
    }
    lamMAX <- lamUL(p, 1)
    if(lamMAX < taildep) {
      txt <- paste0("computed maximum tail dependence (", lamMAX, ") for Theta=", p, " with nu=1",
                    " is less than given as argument taildep=", taildep,
                    " and therefore uniroot() for the nu will not be possible")
      warning(txt)
      zz$message <- txt
      zz$para <- c(p, NA)
      names(zz$para) <- c("theta", "nu")
      zz$lambdaUL_if_nu_were_eqone <- lamUL(p, 1)
      return(zz)
    }
    rt <- NULL
    try(rt <- uniroot(function(nu, taildep=0, p=NA) {
                        err <- taildep - lamUL(p, nu); return(err) },
                          lower=.Machine$double.eps, upper=1E6, p=p, taildep=taildep), silent=TRUE )
    if(is.null(rt)) {
      txt <-  paste0("computed rho = ", zz$rho, " or computed tau = ", zz$tau, " but unable to",
                     " uniroot() solution for nu for the given tail dependence = ", taildep)
      zz$message <- txt
      return(zz)
    }
    zz$message <- "OK"
    nu <- as.integer(round(rt$root, digits=0))
    zz$para <- c(p, nu)
    names(zz$para) <- c("theta", "nu")
    zz$lambdaUL <- lamUL(p, nu)
    zz$lambdaUL_if_nu_were_eqone <- lamUL(p, 1)
    return(zz)
  }

  p  <- para[1]
  nu <- as.integer(round(para[2], digits=0))

  if(p < -1 | p > 1) {
    warning("Parameter must be -1 <= Theta <= 1")
    return(NULL)
  }
  if(nu < 1) {
    warning("Parameter must be nu >= 1 (can be noninteger)")
    return(NULL)
  }


  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
    warning("length u = ", length(u), " and length v = ", length(v))
    warning("longer object length is not a multiple of shorter object length, ",
            "no recycling")
    return(NA)
  }

  if(length(u) == 1) {
    u <- rep(u, length(v))
  } else if (length(v) == 1) {
    v <- rep(v, length(u))
  }

  # Mimic of copula::tCopula.R:::pmvtAlgo
  mimic_pmvtAlgo <- function(dim, x, ...) {
    if(dim <= 3 && ! anyNA(x) && (! any(xI <- x == Inf) || all(xI))) {
      mvtnorm::TVPACK(...)
    } else {
      mvtnorm::GenzBretz(...)
    }
  }

  upru  <- stats::qt(u, df=nu)
  uprv  <- stats::qt(v, df=nu)
  lwr   <- c(-Inf, -Inf)
  sigma <- matrix(c(1, p, p, 1), nrow=2)

  cop <- sapply(seq_len(length(u)), function(i) { # Mimic of copula::ptCopula
                         x <- c(upru[i], uprv[i]); algo <- mimic_pmvtAlgo(2, x)
                  mvtnorm::pmvt(lower=lwr, upper=x, sigma=sigma, df=nu, algorithm=algo) })

  # UVo <- simCOP(1000, cop=Tcop, para=c(0.99, 1), seed=10)  # algorithm=TVPACK fixed
  # cop[u == 1] <- v # Note that if TVPACK is always the algorithm that we can seemingly intercept or
  # cop[v == 1] <- u # hack
  # Warning message: In simCOP(1000, cop = Tcop, para = c(0.99, 1)) :
  # user requested n=1000 sims but only m=750 could be made without
  # NA from derCOPinv (uniroot failure therein)

  # UVn <- simCOP(1000, cop=Tcop, para=c(0.99, 1), seed=10) # algorithm=mimic_pmvtAlgo
  # plot(UVo$V-UVn$V) # differences are very small with the
  #   TVPACK & cop[u == 1] <- v & cop[u == 1] <- u
  # that then is evidence that the blowups in the numerics can seemingly be trapped by enforcing
  # fundamental property of grounding the copula by cop[u == 1] <- v & cop[u == 1] <- u

  return(cop)
}

"EMPIRsimv" <-
function(u, n=1, empgrid=NULL, kumaraswamy=FALSE, ...) {

  empinv <- EMPIRgridderinv(empgrid=empgrid, kumaraswamy=kumaraswamy)
  rows <- as.numeric(attributes(empinv)$rownames)
  ix <- 1:length(rows)
  cols <- attributes(empinv)$colnames
  V <- vector(mode="numeric", length=n)
  for(i in 1:n) {
    t <- runif(1)
    ix.needed1 <- max(ix[rows <= u])
    ix.needed2 <- min(ix[rows >= u])
    if(ix.needed1 == 1) ix.needed1 <- 2
    if(ix.needed1 == ix.needed2) {
      v.available <- empinv[ix.needed1,]
      v <- approx(cols, y=v.available, xout=t, rule=2)$y
    } else {
      v.available1 <- empinv[ix.needed1,]
      v1 <- approx(cols, y=v.available1, xout=t, rule=2)$y
      v.available2 <- empinv[ix.needed2,]
      v2 <- approx(cols, y=v.available2, xout=t, rule=2)$y
      w1 <- u - rows[ix.needed1]
      w2 <- rows[ix.needed2] - u
      tw <- 1/w1 + 1/w2
      v <- (v1/w1 + v2/w2)/tw
    }
    V[i] <- v
  }
  return(V)
}

"kfuncCOPlmoms" <-
function(cop=NULL, para=NULL, nmom=5, begin.mom=1, ...) {
   if(begin.mom > nmom) {
       warning("begin.mom can not be greater than number of L-moments ",
               "requested by nmom, resetting to nmom")
       begin.mom <- nmom
   }

   if(is.null(cop)) {
      warning("must have copula argument specified, returning NULL")
      return(NULL)
   }

   L <- R <- vector(mode="numeric", length=nmom)
   L[1:nmom] <- R[1:nmom] <- NA
   for(r in begin.mom:nmom) L[r] <- kfuncCOPlmom(r, cop=cop, para=para, ...)
   R[2] <- L[2]/L[1]
   begr <- ifelse(begin.mom > 3, begin.mom, 3)
   R[begr:nmom] <- sapply(begr:nmom,
              function(r) { ifelse(is.na(L[2]), return(NA), return(L[r]/L[2]))})
   z <- list(lambdas=L, ratios=R,
             source="kfuncCOPlmoms")
   return(z)
}


"kfuncCOPlmom" <-
function(r, cop=NULL, para=NULL, ...) {
   if(is.null(cop)) {
      warning("must have copula argument specified, returning NULL")
      return(NULL)
   }
   if(r < 1) { warning("r < 1, returning NA"); return(NA) }
   if(r == 1) {
      tmpA <- NULL
      try(tmpA <- integrate(function(t) { 1-kfuncCOP(t,cop=cop,para=para,...) }, 0,1))
      if(is.null(tmpA)) {
         warning("could not integrate for the mean of the Kendall Function")
         return(NA)
      }
      return(tmpA$value)
   }
   # Technically, R appears to operate correctly to get the mean even when sapply'ing
   # the 0:(r-2) for r == 1. However, it is about 100 percent slower then just integrating
   # the above condition for r == 1.
   "sfunc" <- function(j) {
      "afunc" <- function(z, j) { Fz <- kfuncCOP(z, cop=cop, para=para, ...)
                                  return( Fz^(r-j-1) * (1-Fz)^(j+1) ) }
      tmpA <- NULL
      try(tmpA <- integrate(afunc, 0, 1, j=j))
      if(is.null(tmpA)) {
         warning("could not integrate for j=",j," of the Kendall Function")
         return(NA)
      }
      tmpA <- tmpA$value
      tmpB <- (-1)^j * exp(lchoose(r-2,j) + lchoose(r, j+1))
      return(tmpA*tmpB)
   }
   tmp <- sum(sapply(0:(r-2), sfunc)) / r
   return(tmp)
}

"rCOP" <-
function(n, cop=NULL, para=NULL, na.rm=TRUE, seed=NULL,
            resamv01=FALSE, showresamv01=FALSE, ...) {
   if(! is.null(seed)) set.seed(seed)
   u <- runif(n)
   v <- simCOPmicro(u, cop=cop, para=para, ...)
   nn <- length(v[is.na(v) | v <= 0 | v >= 1]) # a smaller infinite break than say n
   runn <- 5; tckr <- NULL
   if(nn != 0 & resamv01) { # only do this block if actually needed
     kk <- 0             # supporting infrastructure to prevent infinite looping
     while(any(is.na(v))) {
       if(showresamv01) message("rCOP() sim'd ", length(v[is.na(v)]), " of v == NA, ", "resampling those")
       v[is.na(v)] <- simCOPmicro(runif(length(v[v <= 0])), cop=cop, para=para, ...)
       kk <- kk + 1      # supporting infrastructure to prevent infinite looping
       if(kk > nn) break # supporting infrastructure to prevent infinite looping
     }
     ii <- 0             # supporting infrastructure to prevent infinite looping
     while(any(is.na(v)) | any(v <= 0)) {
       if(showresamv01) message("rCOP() sim'd ", length(v[v <= 0]), " of v <= 0, ", "resampling those")
       v[is.na(v) | v <= 0] <- simCOPmicro(runif(length(v[v <= 0])), cop=cop, para=para, ...)
       ii <- ii + 1      # supporting infrastructure to prevent infinite looping
       if(ii > nn) break # supporting infrastructure to prevent infinite looping
     }
     jj <- 0             # supporting infrastructure to prevent infinite looping
     while(any(is.na(v)) | any(v >= 1)) {
       if(showresamv01) message("rCOP() sim'd ", length(v[v >= 1]), " of v >= 1, ", "resampling those")
       v[is.na(v) | v >= 1] <- simCOPmicro(runif(length(v[v >= 1])), cop=cop, para=para, ...)
       jj <- jj + 1      # supporting infrastructure to prevent infinite looping
       if(jj > nn) break # supporting infrastructure to prevent infinite looping
     }
     hh <- 0             # supporting infrastructure to prevent infinite looping
     while(any(is.na(v))) {
       if(showresamv01) message("rCOP() still sim'd ", length(v[is.na(v)]), " of v == NA, ", "resampling u,t")
       v[is.na(v)] <- simCOPmicro(runif(length(v[is.na(v)])), cop=cop, para=para, ...)
       hh <- hh + 1      # supporting infrastructure to prevent infinite looping
       if(hh > nn) break # supporting infrastructure to prevent infinite looping
     }
     # still not a perfect solution as pathological situations could at this point now have
     # v <= 0 or v >= 1. But we assured the user that 0 and 1 would not be coming back, so now
     # finally fix by being very close to the edges. But there remains a risk of NAs on v.
     v[v <= 0] <-     .Machine$double.eps^0.50
     v[v >= 1] <- 1 - .Machine$double.eps^0.50
   }
   data.frame(U=u, V=v)
}


"simCOP" <-
function(n=100, cop=NULL, para=NULL, na.rm=TRUE, seed=NULL, keept=FALSE,
                graphics=TRUE, ploton=TRUE, points=TRUE, snv=FALSE,
                infsnv.rm=TRUE, trapinfsnv=.Machine$double.eps,
                resamv01=FALSE, showresamv01=FALSE, ...) {

  if(is.null(cop)) {
     warning("must have copula argument specified, returning NULL")
     return(NULL)
  }

  if(! graphics) {
     ploton <- FALSE
     points <- FALSE
  }

  if(! is.null(seed)) set.seed(seed)

  n <- as.integer(n)

  u <- runif(n); t <- runif(n)
  v <- sapply(seq_len(n), function(i) { derCOPinv(cop=cop, u[i], t[i], para=para, ...) })
  nn <- length(v[is.na(v) | v <= 0 | v >= 1]) # a smaller infinite break than say n
  if(nn != 0 & resamv01) { # only do this block if actually needed
    kk <- 0             # supporting infrastructure to prevent infinite looping
    runn <- 5; tckr <- NULL
    while(any(is.na(v))) {
      if(showresamv01) message("simCOP() sim'd ", length(v[is.na(v)]), " of v == NA, ", "resampling those")
      v[is.na(v)] <- sapply(seq_len(length(v[is.na(v)])),
                     function(i) { derCOPinv(cop=cop, u[i], t[i], para=para, ...) })
      kk <- kk + 1      # supporting infrastructure to prevent infinite looping
      if(kk > nn) break # supporting infrastructure to prevent infinite looping
      tckr <- c(tckr, length(v[is.na(v)]))
      if(length(tckr) > runn) {
        tckr <- tckr[(length(tckr)-runn+1):length(tckr)]
        if(length(unique(tckr)) == 1) break
      }
    }
    ii <- 0             # supporting infrastructure to prevent infinite looping
    tckr <- NULL
    while(any(is.na(v)) | any(v <= 0)) {
      if(showresamv01) message("simCOP() sim'd ", length(v[v <= 0]), " of v <= 0, ", "resampling those")
      v[is.na(v) | v <= 0] <- sapply(seq_len(length(v[v <= 0])),
                            function(i) { derCOPinv(cop=cop, u[i], t[i], para=para, ...) })
      ii <- ii + 1      # supporting infrastructure to prevent infinite looping
      if(ii > nn) break # supporting infrastructure to prevent infinite looping
      tckr <- c(tckr, length(v[is.na(v)]))
      if(length(tckr) > runn) {
        tckr <- tckr[(length(tckr)-runn+1):length(tckr)]
        if(length(unique(tckr)) == 1) break
      }
    }
    jj <- 0             # supporting infrastructure to prevent infinite looping
    tckr <- NULL
    while(any(is.na(v)) | any(v >= 1)) {
      if(showresamv01) message("simCOP() sim'd ", length(v[v >= 1]), " of v >= 1, ", "resampling those")
      v[is.na(v) | v >= 1] <- sapply(seq_len(length(v[v >= 1])),
                            function(i) { derCOPinv(cop=cop, u[i], t[i], para=para, ...) })
      jj <- jj + 1      # supporting infrastructure to prevent infinite looping
      if(jj > nn) break # supporting infrastructure to prevent infinite looping
      tckr <- c(tckr, length(v[is.na(v)]))
      if(length(tckr) > runn) {
        tckr <- tckr[(length(tckr)-runn+1):length(tckr)]
        if(length(unique(tckr)) == 1) break
      }
    }
    hh <- 0             # supporting infrastructure to prevent infinite looping
    tckr <- NULL
    while(any(is.na(v))) {
      if(showresamv01) message("simCOP() still sim'd ", length(v[is.na(v)]), " of v == NA, ", "resampling u,t")
      u[is.na(v)] <- runif(length(v[is.na(v)]))
      t[is.na(v)] <- runif(length(v[is.na(v)]))
      v[is.na(v)] <- sapply(seq_len(length(v[is.na(v)])),
                     function(i) { derCOPinv(cop=cop, u[i], t[i], para=para, ...) })
      hh <- hh + 1      # supporting infrastructure to prevent infinite looping
      if(hh > nn) break # supporting infrastructure to prevent infinite looping
      tckr <- c(tckr, length(v[is.na(v)]))
      if(length(tckr) > runn) {
        tckr <- tckr[(length(tckr)-runn+1):length(tckr)]
        if(length(unique(tckr)) == 1) break
      }
    }
    # still not a perfect solution as pathological situations could at this point now have
    # v <= 0 or v >= 1. But we assured the user that 0 and 1 would not be coming back, so now
    # finally fix by being very close to the edges. But there remains a risk of NAs on v.
    v[v <= 0] <-     .Machine$double.eps^0.50
    v[v >= 1] <- 1 - .Machine$double.eps^0.50
  }

  # stripping content from the ... so that we don't later get the stupid warnings of such and such
  # is not a plotting parameter or versions like that.
  dots <- list(...)
  ditches <- c("delu", "derdir", "trace")
  ditches <- c("pinterval") # for the prod2COP() function
  ditches <- c("breve")     # breveCOP()
  for(d in ditches) {
    if(d %in% names(dots)) dots <- dots[ - which(names(dots) == d)]
  }

  # Because z is a data.frame, it must be assigned within the ifelse()
  ifelse(keept, z <- data.frame(U=u, V=v, T=t), z <- data.frame(U=u, V=v))
  if(na.rm) {
     z <- z[complete.cases(z), ]
     m <- length(z[,1])
     if(m != n) {
        warning("user requested n=", n, " sims but only m=", m,
                " could be made without\nNA from derCOPinv (uniroot failure therein)")
        row.names(z) <- NULL # reset the rows to "1:m"
     }
  }
  if(snv) {
     if(infsnv.rm) {
        z <- z[z$U != 0, ]; z <- z[z$U != 0, ]
        z <- z[z$V != 1, ]; z <- z[z$V != 1, ]
        row.names(z) <- NULL
     } else if(trapinfsnv) {
        z$U[z$U == 0] <-   trapinfsnv; z$V[z$V == 0] <-   trapinfsnv
        z$U[z$U == 1] <- 1-trapinfsnv; z$V[z$V == 1] <- 1-trapinfsnv
     }
     z$U <- qnorm(z$U)
     z$V <- qnorm(z$V)
  }
  if(ploton) {
     if(snv) {
        plot(z$U, z$V, type="n",
             xlab="STANDARD NORMAL VARIATE OF U",
             ylab="STANDARD NORMAL VARIATE OF V")
     } else {
        plot(NA, NA, type="n", xlim=c(0,1), ylim=c(0,1),
             xlab="U, NONEXCEEDANCE PROBABILITY",
             ylab="V, NONEXCEEDANCE PROBABILITY")
     }
  }
  dots$x <- z$U; dots$y <- z$V
  if(points & ! is.null(dev.list())) do.call("points", dots)

  return(z)
}

"gridCOP" <- function(cop=NULL, para=NULL, delta=0.05, transpose=TRUE, ...) {
   u <- seq(0,1, by=delta); v <- seq(0,1, by=delta)
   h <- sapply(u, function(au) {
        sapply(v, function(av) { return(cop(au,av, para=para, ...)) } ) })
   # The transpose is critical so that image() function will work correctly.
   if(transpose) h <- t(h)
   return(h)
}

"JOcopB5" <- function(u, v, para=NULL, tau=NULL, ...) {
    parameter_large <- 1000
    if(is.null(para)) {
      if(is.null(tau)) tau <- cor(u,v, method="kendall")
      "ktau" <- function(d) { a <- 2/(2-d)
         if(! is.finite(a)) return(tau - (1 - trigamma(2)))
         tau - (1 + 2/(2-d)*(digamma(2)-digamma(1 + 2/d)))
      }
      rt <- NULL
      try(rt <- uniroot(ktau, interval=c(1, parameter_large)))
      if(! is.null(rt)) {
         para <- rt$root
         names(para) <- "theta"
         names(tau)  <- "Kendall Tau"
         return(list(para=para, tau=tau))
      } else {
         warning("could not solve for parameter Theta")
         return(NULL)
      }
    }
    if(length(para) != 1) {
       warning("copula only uses one parameter")
       return(NULL)
    }
    if(para[1] < 1) {
      warning("parameter Theta < 1")
      return(NULL)
    }

    if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
       warning("length u = ", length(u), " and length v = ", length(v))
       warning("longer object length is not a multiple of shorter object length, ",
               "no recycling")
       return(NA)
    }
    if(length(u) == 1) {
       u <- rep(u, length(v))
    } else if(length(v) == 1) {
       v <- rep(v, length(u))
    }

    d <- para[1]
    if(d >= parameter_large) return(pmin(u,v)) # M(u,v)
    cop <- 1 - ( (1-u)^d + (1-v)^d -
                ((1-u)^d)*((1-v)^d) )^(1/d)
    if(d >= 1000) {
       wnt <- u > 0.45     | v > 0.45
       cop[wnt] <- pmin(u[wnt], v[wnt]) # M(u,v)
    } else if(d >= 900) {
       wnt <- u > 0.50     | v > 0.50
       cop[wnt] <- pmin(u[wnt], v[wnt]) # M(u,v)
    } else if(d >= 800) {
       wnt <- u > 0.55     | v > 0.55
       cop[wnt] <- pmin(u[wnt], v[wnt]) # M(u,v)
    } else if(d >= 700) {
       wnt <- u > 0.60     | v > 0.60
       cop[wnt] <- pmin(u[wnt], v[wnt]) # M(u,v)
    } else if(d >= 600) {
       wnt <- u > 0.65     | v > 0.65
       cop[wnt] <- pmin(u[wnt], v[wnt]) # M(u,v)
    } else if(d >= 500) {
       wnt <- u > 0.70     | v > 0.70
       cop[wnt] <- pmin(u[wnt], v[wnt]) # M(u,v)
    } else if(d >= 400) {
       wnt <- u > 0.75     | v > 0.75
       cop[wnt] <- pmin(u[wnt], v[wnt]) # M(u,v)
    } else if(d >= 300) {
       wnt <- u > 0.90     | v > 0.90
       cop[wnt] <- pmin(u[wnt], v[wnt]) # M(u,v)
    } else if(d >= 200) {
       wnt <- u > 0.92     | v > 0.92
       cop[wnt] <- pmin(u[wnt], v[wnt]) # M(u,v)
    } else if(d >= 100) {
       wnt <- u > 0.97     | v > 0.97
       cop[wnt] <- pmin(u[wnt], v[wnt]) # M(u,v)
    } else if(d >= 90)  {
       wnt <- u > 0.998    | v > 0.998
       cop[wnt] <- pmin(u[wnt], v[wnt]) # M(u,v)
    } else if(d >= 85)  {
       wnt <- u > 0.999    | v > 0.999
       cop[wnt] <- pmin(u[wnt], v[wnt]) # M(u,v)
    } else if(d >= 80)  {
       wnt <- u > 0.9995   | v > 0.9995
       cop[wnt] <- pmin(u[wnt], v[wnt]) # M(u,v)
    } else if(d >= 70)  {
       wnt <- u > 0.9999   | v > 0.9999
       cop[wnt] <- pmin(u[wnt], v[wnt]) # M(u,v)
    } else if(d >= 60)  {
       wnt <- u > 0.99999  | v > 0.99999
       cop[wnt] <- pmin(u[wnt], v[wnt]) # M(u,v)
    }
    return(cop)
}

"lcomCOPpv" <-
function(n, lcom, cop=NULL, para=NULL, repcoe=5E3, type="gno",
                  mcn=1E4, mcrep=10, usemcmu=FALSE, digits=5, ...) {

   if(! any(type == lmomco::dist.list())) {
      warning("the 'type' argument is providing a distribution not available ",
              "in the lmomco package, resetting to the 'gno'")
      type <- "gno"
   }
   if(usemcmu & mcn == 0) {
       warning("argument 'usemcmu' is TRUE but 'mcn == 0', incompatible, ",
               "resetting 'usemcmu' to FALSE")
       usemcmu <- FALSE
   }
   NAs <- rep(NA, 3)
   T2b <- T3b <- T4b <- data.frame(LcomType=NAs, N=NAs, Nrep=NAs,
                                   Mean=NAs, Lscale=NAs, Lskew=NAs, Lkurt=NAs)
   if(mcn > 0) {
      if(mcrep < 4) {
         warning("mcrep is too small to compute first four L-moments, ",
                 "resetting to four")
         mcrep <- 4
      }
      T2COPb <- vector(mode="numeric", mcrep)
      T3COPb <- T4COPb <- T2COPb12 <- T3COPb12 <- T4COPb12 <- T2COPb
      T2COPb21 <- T3COPb21 <- T4COPb21 <- T2COPb
      message(" STATUS: Performing ", mcrep,
              " replicate(s) of Tau_(234)[12:21] with N=",
                                      mcn,
              " Monte Carlo integration")
      message("         Replication ", appendLF=FALSE)
      for(i in 1:mcrep) {
         message(i,"-", appendLF=FALSE)
         lmcr <- bilmoms(n=mcn, cop=cop, para=para, ...)$bilcomoms
         T2COPb12[i] <- lmcr$T2[1,2];  T2COPb21[i] <- lmcr$T2[2,1]
         T3COPb12[i] <- lmcr$T3[1,2];  T3COPb21[i] <- lmcr$T3[2,1]
         T4COPb12[i] <- lmcr$T4[1,2];  T4COPb21[i] <- lmcr$T4[2,1]
         T2COPb[i]   <- (T2COPb12[i] + T2COPb21[i]) / 2
         T3COPb[i]   <- (T3COPb12[i] + T3COPb21[i]) / 2
         T4COPb[i]   <- (T4COPb12[i] + T4COPb21[i]) / 2
      }
      message("done")
      message("         Computing univariate L-moments-", appendLF=FALSE)
      lmrT2b12 <- lmomco::lmoms(T2COPb12, nmom=4, no.stop=TRUE)
      lmrT2b21 <- lmomco::lmoms(T2COPb21, nmom=4, no.stop=TRUE)
      lmrT3b12 <- lmomco::lmoms(T3COPb12, nmom=4, no.stop=TRUE)
      lmrT3b21 <- lmomco::lmoms(T3COPb21, nmom=4, no.stop=TRUE)
      lmrT4b12 <- lmomco::lmoms(T4COPb12, nmom=4, no.stop=TRUE)
      lmrT4b21 <- lmomco::lmoms(T4COPb21, nmom=4, no.stop=TRUE)
      lmrT2b   <- lmomco::lmoms(T2COPb,   nmom=4, no.stop=TRUE)
      lmrT3b   <- lmomco::lmoms(T3COPb,   nmom=4, no.stop=TRUE)
      lmrT4b   <- lmomco::lmoms(T3COPb,   nmom=4, no.stop=TRUE)
      T2b <- data.frame(LcomType=c("Tau2[12]", "Tau2[21]", "Tau2[12:21]"),
          N=rep(mcn, 3), Nrep=rep(mcrep, 3),
          Mean=  c(lmrT2b12$lambdas[1], lmrT2b21$lambdas[1], lmrT2b$lambdas[1]),
          Lscale=c(lmrT2b12$lambdas[2], lmrT2b21$lambdas[2], lmrT2b$lambdas[2]),
          Lskew= c(lmrT2b12$ratios[3],  lmrT2b21$ratios[3],  lmrT2b$ratios[3]),
          Lkurt= c(lmrT2b12$ratios[4],  lmrT2b21$ratios[4],  lmrT2b$ratios[4]))
      T3b <- data.frame(LcomType=c("Tau3[12]", "Tau3[21]", "Tau3[12:21]"),
          N=rep(mcn, 3), Nrep=rep(mcrep, 3),
          Mean=  c(lmrT3b12$lambdas[1], lmrT3b21$lambdas[1], lmrT3b$lambdas[1]),
          Lscale=c(lmrT3b12$lambdas[2], lmrT3b21$lambdas[2], lmrT3b$lambdas[2]),
          Lskew= c(lmrT3b12$ratios[3],  lmrT3b21$ratios[3],  lmrT3b$ratios[3]),
          Lkurt= c(lmrT3b12$ratios[4],  lmrT3b21$ratios[4],  lmrT3b$ratios[4]))
      T4b <- data.frame(LcomType=c("Tau4[12]", "Tau4[21]", "Tau4[12:21]"),
          N=rep(mcn, 3), Nrep=rep(mcrep, 3),
          Mean=  c(lmrT4b12$lambdas[1], lmrT4b21$lambdas[1], lmrT4b$lambdas[1]),
          Lscale=c(lmrT4b12$lambdas[2], lmrT4b21$lambdas[2], lmrT4b$lambdas[2]),
          Lskew= c(lmrT4b12$ratios[3],  lmrT4b21$ratios[3],  lmrT4b$ratios[3]),
          Lkurt= c(lmrT4b12$ratios[4],  lmrT4b21$ratios[4],  lmrT4b$ratios[4]))
      message("done")
   } else {
      message(" STATUS: Skipping large sample sim. because sample size is zero.")
   }

   T2table <- data.frame(LcomType=NAs, n=NAs, nrep=NAs, Mean=NAs, Lscale=NAs,
                                                        Lskew=NAs, Lkurt=NAs,
                                     sample.est=NAs, p.value=NAs, signif=NAs)
   T4table <- T3table <- T2table
   if(n > 0) {
      nrep <- as.integer(repcoe/sqrt(n))
      if(nrep == 0) nrep <- 1
      message(" STATUS: Performing ", nrep, " replicate(s) for ",n,
              " sample size [small sam. sim.]")
      message("           ", appendLF=FALSE)
      T2COP <- vector(mode="numeric", length=nrep)
      T3COP <- T4COP <- T2COP12 <- T3COP12 <- T4COP12 <- T2COP
      T2COP21 <- T3COP21 <- T4COP21 <- T2COP
      for(i in 1:nrep) {
         if(length(grep('00+$', i, perl=TRUE)) == 1) message(i,"-",appendLF=FALSE)
         UV <- simCOP(n=n, para=para, cop=cop, graphics=FALSE)
         lmcr <- lmomco::lcomoms2(data.frame(U=UV$U, V=UV$V), nmom=4)
         T2COP12[i] <- lmcr$T2[1,2]; T2COP21[i] <- lmcr$T2[2,1]
         T3COP12[i] <- lmcr$T3[1,2]; T3COP21[i] <- lmcr$T3[2,1]
         T4COP12[i] <- lmcr$T4[1,2]; T4COP21[i] <- lmcr$T4[2,1]
           T2COP[i] <- (T2COP12[i] + T2COP21[i]) / 2
           T3COP[i] <- (T3COP12[i] + T3COP21[i]) / 2
           T4COP[i] <- (T4COP12[i] + T4COP21[i]) / 2
      }
      message("done")

      message("         Computing univariate L-moments for L-comoments 'r[12]'")
      # Work on [12]
      lmrT2 <- lmomco::lmoms(T2COP12, no.stop=TRUE); lmrT2t <- lmrT2
      lmrT3 <- lmomco::lmoms(T3COP12, no.stop=TRUE); lmrT3t <- lmrT3
      lmrT4 <- lmomco::lmoms(T4COP12, no.stop=TRUE); lmrT4t <- lmrT4
      if(usemcmu) {
         message("         Using mean of Monte Carlo integrations and rescaling ",
                 "L-scale by small simulation LCV")
         lmrT2t$lambdas[1] <- T2b[1,4]
         lmrT3t$lambdas[1] <- T3b[1,4]
         lmrT4t$lambdas[1] <- T4b[1,4]
         lmrT2t$lambdas[2] <- abs(lmrT2$ratios[2]*T2b[1,4]) # abs for protection
         lmrT3t$lambdas[2] <- abs(lmrT3$ratios[2]*T3b[1,4]) # against the mean
         lmrT4t$lambdas[2] <- abs(lmrT4$ratios[2]*T4b[1,4]) # being a negative
         lmrT2t$ratios[2] <- lmrT2$ratios[2]
         lmrT3t$ratios[2] <- lmrT3$ratios[2]
         lmrT4t$ratios[2] <- lmrT4$ratios[2]
      }
      par2      <- lmomco::lmom2par(lmrT2t, type=type);# print(par2)
      par3      <- lmomco::lmom2par(lmrT3t, type=type);# print(par3)
      par4      <- lmomco::lmom2par(lmrT4t, type=type);# print(par4)
      T2p.value <- lmomco::plmomco(lcom$T2[1,2], par2);# print(T2p.value)
      T3p.value <- lmomco::plmomco(lcom$T3[1,2], par3);# message("PARA",T3p.value)
      T4p.value <- lmomco::plmomco(lcom$T4[1,2], par4);# print(T4p.value)
      if(T2p.value > 0.5) T2p.value <- 1 - T2p.value
      if(T3p.value > 0.5) T3p.value <- 1 - T3p.value
      if(T4p.value > 0.5) T4p.value <- 1 - T4p.value
      T2table[1,] <- c("Tau2[12]", n, nrep, lmrT2$lambdas[1], lmrT2$lambdas[2],
                                lmrT2$ratios[3:4], lcom$T2[1,2], T2p.value, NA)
      T3table[1,] <- c("Tau3[12]", n, nrep, lmrT3$lambdas[1], lmrT3$lambdas[2],
                                lmrT3$ratios[3:4], lcom$T3[1,2], T3p.value, NA)
      T4table[1,] <- c("Tau4[12]", n, nrep, lmrT4$lambdas[1], lmrT4$lambdas[2],
                                lmrT4$ratios[3:4], lcom$T4[1,2], T4p.value, NA)

      message("         Computing univariate L-moments for L-comoments 'r[21]'")
      # Work on [21]
      lmrT2 <- lmomco::lmoms(T2COP21, no.stop=TRUE); lmrT2t <- lmrT2
      lmrT3 <- lmomco::lmoms(T3COP21, no.stop=TRUE); lmrT3t <- lmrT3
      lmrT4 <- lmomco::lmoms(T4COP21, no.stop=TRUE); lmrT4t <- lmrT4
      if(usemcmu) {
         message("         Using mean of Monte Carlo integrations and rescaling ",
                 "L-scale by small simulation LCV")
         lmrT2t$lambdas[1] <- T2b[2,4]
         lmrT3t$lambdas[1] <- T3b[2,4]
         lmrT4t$lambdas[1] <- T4b[2,4]
         lmrT2t$lambdas[2] <- abs(lmrT2$ratios[2]*T2b[2,4]) # abs for protection
         lmrT3t$lambdas[2] <- abs(lmrT3$ratios[2]*T3b[2,4]) # against the mean
         lmrT4t$lambdas[2] <- abs(lmrT4$ratios[2]*T4b[2,4]) # being a negative
         lmrT2t$ratios[2] <- lmrT2$ratios[2]
         lmrT3t$ratios[2] <- lmrT3$ratios[2]
         lmrT4t$ratios[2] <- lmrT4$ratios[2]
      }
      par2        <- lmomco::lmom2par(lmrT2t, type=type)
      par3        <- lmomco::lmom2par(lmrT3t, type=type)
      par4        <- lmomco::lmom2par(lmrT4t, type=type)
      T2p.value   <- lmomco::plmomco(lcom$T2[2,1], par2)
      T3p.value   <- lmomco::plmomco(lcom$T3[2,1], par3)
      T4p.value   <- lmomco::plmomco(lcom$T4[2,1], par4)
      if(T2p.value > 0.5) T2p.value <- 1 - T2p.value
      if(T3p.value > 0.5) T3p.value <- 1 - T3p.value
      if(T4p.value > 0.5) T4p.value <- 1 - T4p.value
      T2table[2,] <- c("Tau2[21]", n, nrep, lmrT2$lambdas[1], lmrT2$lambdas[2],
                               lmrT2$ratios[3:4], lcom$T2[2,1], T2p.value, NA)
      T3table[2,] <- c("Tau3[21]", n, nrep, lmrT3$lambdas[1], lmrT3$lambdas[2],
                               lmrT3$ratios[3:4], lcom$T3[2,1], T3p.value, NA)
      T4table[2,] <- c("Tau4[21]", n, nrep, lmrT4$lambdas[1], lmrT4$lambdas[2],
                               lmrT4$ratios[3:4], lcom$T4[2,1], T4p.value, NA)

      message("         Computing univariate L-moments for L-comoments 'r[12|21]'")
      # Working on [12|21]
      T2 <- (lcom$T2[1,2] + lcom$T2[2,1]) / 2
      T3 <- (lcom$T3[1,2] + lcom$T3[2,1]) / 2
      T4 <- (lcom$T4[1,2] + lcom$T4[2,1]) / 2
      lmrT2 <- lmomco::lmoms(T2COP, no.stop=TRUE); lmrT2t <- lmrT2 # Create temporary copies that we can do
      lmrT3 <- lmomco::lmoms(T3COP, no.stop=TRUE); lmrT3t <- lmrT3 # substitution on the mean and L-scale dependng
      lmrT4 <- lmomco::lmoms(T4COP, no.stop=TRUE); lmrT4t <- lmrT4 # on setting of 'usemcmu' for the p-value computation.
      if(usemcmu) {
         message("         Using mean of Monte Carlo integrations and rescaling ",
                 "L-scale by small simulation LCV")
         lmrT2t$lambdas[1] <- T2b[3,4]
         lmrT3t$lambdas[1] <- T3b[3,4]
         lmrT4t$lambdas[1] <- T4b[3,4]
         lmrT2t$lambdas[2] <- abs(lmrT2$ratios[2]*T2b[3,4]) # abs for protection
         lmrT3t$lambdas[2] <- abs(lmrT3$ratios[2]*T3b[3,4]) # against the mean
         lmrT4t$lambdas[2] <- abs(lmrT4$ratios[2]*T4b[3,4]) # being a negative
         lmrT2t$ratios[2] <- lmrT2$ratios[2]
         lmrT3t$ratios[2] <- lmrT3$ratios[2]
         lmrT4t$ratios[2] <- lmrT4$ratios[2]
      }
      par2        <- lmomco::lmom2par(lmrT2t, type=type)
      par3        <- lmomco::lmom2par(lmrT3t, type=type)
      par4        <- lmomco::lmom2par(lmrT4t, type=type)
      T2p.value   <- lmomco::plmomco(T2, par2)
      T3p.value   <- lmomco::plmomco(T3, par3)
      T4p.value   <- lmomco::plmomco(T4, par4)
      if(T2p.value > 0.5) T2p.value <- 1 - T2p.value
      if(T3p.value > 0.5) T3p.value <- 1 - T3p.value
      if(T4p.value > 0.5) T4p.value <- 1 - T4p.value
      T2table[3,] <- c("Tau2[12:21]", n, nrep,
       lmrT2$lambdas[1], lmrT2$lambdas[2], lmrT2$ratios[3:4], T2, T2p.value, NA)
      T3table[3,] <- c("Tau3[12:21]", n, nrep,
       lmrT3$lambdas[1], lmrT3$lambdas[2], lmrT3$ratios[3:4], T3, T3p.value, NA)
      T4table[3,] <- c("Tau4[12:21]", n, nrep,
       lmrT4$lambdas[1], lmrT4$lambdas[2], lmrT4$ratios[3:4], T4, T4p.value, NA)
      for(i in 2:9) { # annoying part of my logic flow, having to recast into numerics
         T2table[,i] <- as.numeric(T2table[,i])
         T3table[,i] <- as.numeric(T3table[,i])
         T4table[,i] <- as.numeric(T4table[,i])
      }
      sig.codes <- c("-", ".", "*", "**", "***")
      sig.val  <- c(2, 0.2, 0.1, 0.02, 0.002) # Field [1] is purposelfully present and not used
      sig.val <- sig.val/2
      sig.valc <- 1 - sig.val
      for(i in 1:3) {
         T2table[i,10] <- ifelse(T2table[i,9] <=  sig.val[5]  |
                                 T2table[i,9] >= sig.valc[5],  sig.codes[5],
                          ifelse(T2table[i,9] <=  sig.val[4]  |
                                 T2table[i,9] >= sig.valc[4],  sig.codes[4],
                          ifelse(T2table[i,9] <=  sig.val[3]  |
                                 T2table[i,9] >= sig.valc[3],  sig.codes[3],
                          ifelse(T2table[i,9] <=  sig.val[2]  |
                                 T2table[i,9] >= sig.valc[2],  sig.codes[2],
                                                               sig.codes[1]))))
         T3table[i,10] <- ifelse(T3table[i,9] <=  sig.val[5]  |
                                 T3table[i,9] >= sig.valc[5],  sig.codes[5],
                          ifelse(T3table[i,9] <=  sig.val[4]  |
                                 T3table[i,9] >= sig.valc[4],  sig.codes[4],
                          ifelse(T3table[i,9] <=  sig.val[3]  |
                                 T3table[i,9] >= sig.valc[3],  sig.codes[3],
                          ifelse(T3table[i,9] <=  sig.val[2]  |
                                 T3table[i,9] >= sig.valc[2],  sig.codes[2],
                                                               sig.codes[1]))))
         T4table[i,10] <- ifelse(T4table[i,9] <=  sig.val[5]  |
                                 T4table[i,9] >= sig.valc[5],  sig.codes[5],
                          ifelse(T4table[i,9] <=  sig.val[4]  |
                                 T4table[i,9] >= sig.valc[4],  sig.codes[4],
                          ifelse(T4table[i,9] <=  sig.val[3]  |
                                 T4table[i,9] >= sig.valc[3],  sig.codes[3],
                          ifelse(T4table[i,9] <=  sig.val[2]  |
                                 T4table[i,9] >= sig.valc[2],  sig.codes[2],
                                                               sig.codes[1]))))
      }
   } else {
      message(" STATUS: Skipping Monte Carlo integration because sample size is zero.")
   }
   message("STATUS: Rounding digits")
   if(! is.na(digits)) {
      if(mcn > 0) {
          T2b[,4:7]    <- round(T2b[,4:7],     digits=digits)
          T3b[,4:7]    <- round(T3b[,4:7],     digits=digits)
          T4b[,4:7]    <- round(T4b[,4:7],     digits=digits)
      }
      if(n > 0) {
         T2table[,4:9] <- round(T2table[,4:9], digits=digits)
         T3table[,4:9] <- round(T3table[,4:9], digits=digits)
         T4table[,4:9] <- round(T4table[,4:9], digits=digits)
      }
   }
   txt <- ifelse(usemcmu,
    "Monte Carlo integration means in 'Ntable' were used for p.value computation",
    "Small sample size simulated means in 'ntable' were used for p.value computation")
   zz <- list(text="Sample L-comoment Hypothesis Testing",
              p.value.how=txt,
              Ntable=list(Lcocorr=T2b,     Lcoskew=T3b,     Lcokurt=T4b),
              ntable=list(Lcocorr=T2table, Lcoskew=T3table, Lcokurt=T4table))
   return(zz)
}

"FRcop" <- # Frank copula, Joe (2015, p. 165, sec. 4.5.1)
function(u, v, para=NULL, rhotau=NULL, userhotau_chk=TRUE,
               cortype=c("kendall", "spearman", "tau", "rho"), ...) {
  cortype <-   tolower(cortype)
  cortype <- match.arg(cortype)
  D1 <- function(x) { # Joe (2015, p. 166). This integral needed for Kendall's tau.
    integrate(function(t) 1*x^(-1) * (t^1)*(expm1(t))^(-1), lower=0, upper=x,
              subdivisions=200, stop.on.error=FALSE) # expm1(t) = exp(t) - 1
    # set stop.on.error=TRUE for deeper testing when exploring limits of FRcop() implementation
  }
  D2 <- function(x) { # Joe (2015, p. 166). This integral needed for Kendall's tau.
    integrate(function(t) 2*x^(-2) * (t^2)*(expm1(t))^(-1), lower=0, upper=x,
              subdivisions=200, stop.on.error=FALSE) # expm1(t) = exp(t) - 1
    # set stop.on.error=TRUE for deeper testing when exploring limits of FRcop() implementation
  }

  big.lwr <- -3500
  big.upr <- +3500

  para.small <- 1.082793e-06 # from lines of code below and abs() will be used.
  # uniroot(ofunc_tau, interval=c(big.lwr, big.upr), target_tau=0)$root # [1] +1.082793e-06
  # uniroot(ofunc_rho, interval=c(big.lwr, big.upr), target_rho=0)$root # [1] -2.097237e-10

  # it is in the copula computations themselves, so we need another heuristic limit.
  large.para <- 354 # based on copBasic::simCOP behavior and handling of reflection with positive parameters
  large.rho  <- 0.9999479
  large.tau  <- 0.9887531

  if(is.null(para)) {
    if(cortype == "spearman" | cortype == "rho") {
      if(is.null(rhotau)) {
        rho <- cor(u,v, method="spearman")
      } else {
        rho <- rhotau
      }
      ofunc_rho <- function(trial_para, target_rho=NA) {
        rho <- 1 + (12/trial_para) * (D2(trial_para)$value - D1(trial_para)$value) # Joe (2015, p. 166)
        return(target_rho - rho)
      }
      para <- uniroot(ofunc_rho, interval=c(big.lwr, big.upr), target_rho=rho)$root
      names(para) <- "theta"
      names(rho ) <- "Spearman Rho"
      return(list(para=para, rho=rho))
    } else if(cortype == "kendall" | cortype == "tau") {
      if(is.null(rhotau)) {
        tau <- cor(u,v, method="kendall")
      } else {
        tau <- rhotau
      }
      ofunc_tau <- function(trial_para, target_tau=NA) {
        tau <- 1 + (4/trial_para) * (D1(trial_para)$value - 1) # Joe (2015, p. 166)
        return(target_tau - tau)
      }
      # uniroot(ofunc, interval=c(-3000, +3000), target_tau=0.998)$root
      # Error in integrate(function(t, k = 1) x^(-k) * (t^k) * (exp(t) - 1)^(-1), :
      # the integral is probably divergent
      para <- NULL
      try(para <- uniroot(ofunc_tau, interval=c(big.lwr, big.upr), target_tau=tau)$root)
      if(is.null(para)) {
        try(para <- uniroot(ofunc_tau, interval=c(big.lwr, big.upr), target_tau=tau)$root)
      }
      names(para) <- "theta"
      names(tau ) <- "Kendall Tau"
      return(list(para=para, tau=tau))
    } else {
      stop("should not be here in logic")
    }
  }
  if(length(para) == 1) {
    if(abs(para) < para.small) {
      tau <- 0
    } else {
      if(userhotau_chk) {
        # design choice for lesson is to compute tau again for purposes of outer limits of operation
        # but would be better to accept a precomputed tau just once for given parameter and pass
        # as another argument to this function to avoid the D1() call on the next line.
        d1 <- D1(para)$value
        tau <- 1 + ( 4/para) * (d1             -  1) # Joe (2015, p. 166)
        rho <- 1 + (12/para) * (D2(para)$value - d1) # Joe (2015, p. 166)
        # print(c(rho, tau))
      }
    }
  } else {
    warning("Parameter Theta can not be a vector")
    return(NULL)
  }
  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {   # standard syntax in copBasic style
    warning("length u = ", length(u), " and length v = ", length(v))
    warning("longer object length is not a multiple of shorter object length, ",
            "no recycling")
    return(NA)
  }
  if(length(u) == 1) {                                           # standard syntax in copBasic style
    u <- rep(u, length(v))
  } else if(length(v) == 1) {
    v <- rep(v, length(u))
  }

  if(abs(para[1]) < para.small) return(u*v) # independence

  if(userhotau_chk) {
    if( tau > +large.tau ) return( M(u,v) ) # upper bound copula, perfect 1:+1, limiting condition
    if( tau < -large.tau ) return( W(u,v) ) # lower bound copula, perfect 1:-1, limiting condition
    if( rho > +large.rho ) return( M(u,v) ) # upper bound copula, perfect 1:+1, limiting condition
    if( rho < -large.rho ) return( W(u,v) ) # lower bound copula, perfect 1:-1, limiting condition
  } else {
    if(para > +large.para) return( M(u,v) ) # upper bound copula, perfect 1:+1, limiting condition
    if(para < -large.para) return( W(u,v) ) # lower bound copula, perfect 1:-1, limiting condition
  }
  # Parameter reflection behavior of partial derivative or its inversion by copBasic::derCOP()
  # and copBasic::derCOPinv() for the positive parameters breaks down as log(0) encounter, but the
  # mirror image of the parameter in the negative domains does not see break down, so reverse
  # the parameter for the copula computations but reflect the v.
  p <- para
  if(para > 0) { v <- 1 - v; p <- -p } # reflection PART A, like "grave" operation in copBasic::COP()

  #a <- exp(-p); b <- exp(-p*u); c <- exp(-p*v)         # THE FRANK COPULA
  #cop <- (1 - a - (1 - b)*(1 - c) ) / (1 - a)          # THE FRANK COPULA
  #cop <- -para[1]^(-1) * log(cop)                      # THE FRANK COPULA

  # see ?expm1
  a <- -expm1(-p); b <- -expm1(-p*u); c <- -expm1(-p*v) # THE FRANK COPULA
  cop <- (a - b*c ) / a                                 # THE FRANK COPULA
  lcop <- log(cop)                                      # THE FRANK COPULA
  lcop[lcop == -Inf] <- log(.Machine$double.eps)        # THE FRANK COPULA
  cop <- -p^(-1) * lcop                                 # THE FRANK COPULA
  # The parameter reversing might protect us from the lcop == -Inf meaning that
  # (a - b*c) ---> 0 but let us use one last insurance policy of sorts and make zero
  # become .Machine$double.eps. Likely, real protection is the large.para setting at function top.
  if(para > 0) cop <- u - cop # reflection PART B, like "grave" operation in copBasic::COP()
  return(cop)
}

"EMPIRqua.regress" <-
function(f=0.5, u=seq(0.01,0.99, by=0.01), empinv=NULL,
         lowess=FALSE, f.lowess=1/5, ...) {
   cols <- attributes(empinv)$colnames
   ix <- 1:length(cols)
   ix.needed <- ix[as.character(cols) == as.character(f)]
   if(length(ix.needed) != 1) {
      warning("f=",f," value does not match against column names in empinv, ",
              "likely source of this is a real number with too many digits ",
              "relative to a keyed entry")
      return(data.frame(U=NA, V=NA))
   }
   U.available <- attributes(empinv)$rownames
   V.available <- empinv[,ix.needed]
   UVdf <- data.frame(U=U.available, V=V.available)
   UVdf <- UVdf[complete.cases(UVdf),]
   # we know that the x are given in ordered seqeuence to so avoid
   # the warning
   # In regularize.values(x, y, ties, missing(ties)) :
   # collapsing to unique 'x' values
   V <- approx(U.available, y=V.available, xout=u, rule=2, ties="ordered")$y
   z <- data.frame(U=u,V=V)
   if(lowess) {
      lws <- lowess(z$U, y=z$V, f=f.lowess)
      z <- data.frame(U=lws$x, V=lws$y)
      z$V[z$V < 0] <- 0; z$V[z$V > 1] <- 1 # -WHA 2019/06/28 (found a case)
   }
   return(z)
}

"simcomposite3COP" <-
function(nsim=100, compositor=composite3COP,
         parents=NULL, ploton=FALSE, points=FALSE,
         showpar=FALSE, showresults=FALSE, digits=6,
         ...) {

  if(nsim == Inf) {
    warning("can not have an infinite length (row) matrix")
    return(NA)
  }

  to.get.a.width1 <- parents$para1gen()
  w1 <- length(to.get.a.width1)
  to.get.a.width2 <- parents$para2gen()
  w2 <- length(to.get.a.width2)

  vals <- matrix(nrow=nsim, ncol=(w1 + w2 + 10))
  colnames(vals) <- c("alpha", "beta",
                      "kappa", "gamma",
                      "T2.12", "T2.21",
                      "T3.12", "T3.21",
                      "T4.12", "T4.21",
                      rep("Cop1Thetas",w1),
                      rep("Cop2Thetas",w2))
  i <- 0
  while(i < nsim) {
    Theta1 <- parents$para1gen()
    Theta2 <- parents$para2gen()
    alpha  <- runif(1)
    beta   <- runif(1)
    kappa  <- runif(1)
    gamma  <- runif(1)
    para   <- list(cop1=parents$cop1,
                   cop2=parents$cop2,
                   alpha=alpha,
                   beta=beta,
                   kappa=kappa,
                   gamma=gamma,
                   para1=Theta1,
                   para2=Theta2)

    if(showpar) print(para)

    #S <- simCOP(n=n, cop=compositor,
    #            ploton=ploton, points=points,
    #            para=para, col=rgb(0,0,0,0.1), pch=16)
    #abline(0,1); abline(1,-1)
    #mtext(paste(c("Theta1 = ",round(Theta1,digits=3), "   ",
    #              "Theta2 = ",round(Theta2,digits=3), "   ",
    #              "Alpha = ", round(alpha,digits=3),  "   ",
    #              "Beta = ",  round(beta,digits=3), "   ",
    #              "Kappa= ",  round(kappa,digits=3), "   ",
    #              "Gamma= ",  round(gamma,digits=3)), sep="",
    #                 collapse=" "))
    #cat(c("Sleeping to continue\n"))

    #z <- lmomco::lcomoms2(S, nmom=4, opdiag=TRUE)
    z <- lcomCOP(compositor, para, orders=2:4)
    results <- c(alpha, beta, kappa, gamma,
                 z$lcomUV[2], z$lcomVU[2],
                 z$lcomUV[3], z$lcomVU[3],
                 z$lcomUV[4], z$lcomVU[4], Theta1, Theta2)
    if(showresults) cat(c(round(results, digits=digits), "\n"))
    i <- i + 1
    vals[i,] <- results
  }
  return(vals)
}

"composite3COP" <-
function(u,v,para,...) {
  if(is.null(para$kappa)) {
    warning("no kappa parameter given")
    return(NULL)
  }
  if(is.null(para$gamma)) {
    warning("no gamma parameter given")
    return(NULL)
  }
  kappa <- para$kappa; kappa.p <- 1 - kappa
  gamma <- para$gamma; gamma.p <- 1 - gamma
  return(u^kappa * v^gamma * composite2COP(u^kappa.p, v^gamma.p, para, ...))
}

"derCOP" <-
function(cop=NULL, u, v,
         delu=.Machine$double.eps^0.50,
         derdir=c("left", "right", "center"), ...) {

    derdir <- match.arg(derdir)
    if(length(u) == 1) {
       if(u - delu < 0) derdir <- "left"
       if(u + delu > 1) derdir <- "right"
       #str(cop)
       if(derdir == "left") {
          return((cop(u+delu,v,...) - cop(u,v, ...))/delu)
       } else if(derdir == "right") {
          return((cop(u,v,...)      - cop(u-delu,v, ...))/delu)
       } else {
          return((cop(u+delu,v,...) - cop(u-delu,v, ...))/(2*delu))
       }
    } else {
      if(length(u) != length(v)) {
        #warning("length of u and v are not equal, so using only first element of v")
        v <- rep(v[1], length(u))
      }
      der <- vector(mode="numeric", length(u))
      for(i in 1:length(u)) {
         tmpdir <- derdir; au <- u[i]; av <- v[i]
         if(au - delu < 0) tmpdir <- "left"
         if(au + delu > 1) tmpdir <- "right"
         if(tmpdir == "left") {
           der[i] <- (cop(au+delu,av,...) - cop(au,av, ...))/delu
         } else if(tmpdir == "right") {
           der[i] <- (cop(au,av,...)      - cop(au-delu,av, ...))/delu
         } else {
           der[i] <- (cop(au+delu,av,...) - cop(au-delu,av, ...))/(2*delu)
         }
      }
      return(der)
    }
}

#gradCOP <- function(cop=NULL, u, v, para=NULL) {
#  func <- function(u, v=NA, para=NULL) cop(u, v, para=para)
#  if(length(u) == 1) {
#    return(sapply(v, function(v) numDeriv::grad(func, u, v=v, para=para)))
#  }
#  if(length(u) != length(v)) {
#    #warning("length of u and v are not equal, so using only first element of v")
#    v <- rep(v[1], length(u))
#  }
#  der <- vector(mode="numeric", length(u))
#  for(i in 1:length(u)) {
#    der[i] <- numDeriv::grad(func, u[i], method="complex", v=v[i], para=para)
#  }
#  return(der)
#}

"taildepCOP" <-
function(cop=NULL, para=NULL, tol=1e-6, divisor=2,
         plot=FALSE, ylim=NULL, verbose=FALSE, ...) {
  resolution <- abs(log10(tol)) - 1
  lamu.tmp <- lamu <- u <- 0
  LAMU1 <- LAMU2 <- vector(mode="numeric")
  if(verbose) message("Upper-Tail Dependency")
  t <- 0.5
  while(t < 1 - .Machine$double.eps) {
    lamu.tmp <- 2 - (1-cop(t,t, para=para, ...)) / (1-t)
    if(is.nan(lamu.tmp)) break
    u <- u + 1
    if(verbose) message("  iter.", u, "  t=", t,"  lambdaU=", lamu)
    LAMU1[u] <- t; LAMU2[u] <- lamu
    if(abs(lamu - lamu.tmp) < tol) break
    t <- t + (1-t)/divisor
    lamu <- lamu.tmp
  }
  if(verbose) message("\nLower-Tail Dependency")
  laml.tmp <- laml <- l <- 0
  LAML1 <- LAML2 <- vector(mode="numeric")
  t <- 0.5
  while(t > .Machine$double.eps) {
    laml.tmp <- cop(t,t, para=para, ...) / t
    if(is.nan(laml.tmp)) break
    l <- l + 1
    if(verbose) message("  iter.", l, "  t=", t, "  lambdaL=", laml)
    LAML1[l] <- t; LAML2[l] <- laml
    if(abs(laml - laml.tmp) < tol) break
    t <- t - t/divisor
    laml <- laml.tmp
  }
  if(verbose) message("---done.")
  if(plot) {
    xmin <- qnorm( min(LAMU1, LAML1) )
    xmax <- qnorm( max(LAMU1, LAML1) )
    # print(c(xmin, xmax))
    xlim <- c(-1, +1) * max( abs(c(xmin, xmax)) )
    ymin <- min(LAMU2,LAML2)
    ymax <- max(LAMU2,LAML2)
    if(is.null(ylim)) ylim <- c(ymin, ymax)
    plot(qnorm(LAMU1), LAMU2, type="l", col="blue", xlim=xlim, ylim=ylim,
           xlab="STANDARD NORMAL VARIATES",
           ylab="UPPER- AND LOWER-TAIL DEPENDENCY PARAMETER")
    lines(qnorm(LAML1), LAML2, col="red")
    points(qnorm(LAMU1[u]), LAMU2[u], cex=0.7, pch=16, col="blue")
    points(qnorm(LAMU1[u]), LAMU2[u], cex=1.3, pch="U")
    points(qnorm(LAML1[l]), LAML2[l], cex=0.7, pch=16, col="red")
    points(qnorm(LAML1[l]), LAML2[l], cex=1.3, pch="L")
  }
  return(list(lambdaL = round(LAML2[l], digits=resolution),
              lambdaU = round(LAMU2[u], digits=resolution),
              source = "taildepCOP"))
}

"uvlmoms" <- function(u,v=NULL, umv=TRUE, p=NA, type="gno", getlmoms=TRUE, ...) {
   if(is.null(v)) {
      if(length(names(u)) != 2) {
         warning("a data.frame having only two columns is required")
         return(NULL)
      } else {
         v <- u[,2]
         u <- u[,1]
      }
   }
   if(length(u) != length(v)) {
      warning("argument(s) or implied arguments u and v are unequal in length, ",
              "returning NULL")
      return(NULL)
   }

   ifelse(umv, psi <- u-v, psi <- u+v-1)
   ifelse(is.na(p), getlmoms <- TRUE, getlmoms <- FALSE)

   if(getlmoms) { # L-moments desired
     lmr <- lmomco::lmoms(psi, ...)
     lmr$ratios[2] <- NA # although numerically computable, theory
     # requires that the LCV for a distribution required to have
     # mean zero is NaN.
     return(lmr)
   }

   if(p < 0 | p >= 1/2) {
      warning("argument p is not in 0 < p < 1/2, returning NULL")
      return(NULL)
   }

   if(any(lmomco::dist.list() == type)) {
      # Fitting a univariate distribution by method of L-moments
   	  lmr  <- lmomco::lmoms(psi, ...)
   	  para <- lmomco::lmom2par(lmr, type)
         A <- lmomco::qlmomco(1-p, para)
         B <- lmomco::qlmomco(  p, para)
         C <- lmomco::qlmomco(1/2, para)
      skewness <- A - 2*C + B / (A - B)
      return(skewness)
   } else {
   	  # Using the empirical distribution function
   	  type <- as.integer(type)
      if(is.na(type) | type < 1 | type > 9) {
         warning("argument 'type' as an integer is not in [1, 9] as quantile() will ",
                 "require, returning NULL")
         return(NULL)
      }
      A <- quantile(psi, probs=1-p, names=FALSE, type=type)
      B <- quantile(psi, probs=  p, names=FALSE, type=type)
      C <- quantile(psi, probs=1/2, names=FALSE, type=type)
      skewness <- A - 2*C + B / (A - B)
      return(skewness)
   }
}


"uvskew" <- function(u,v=NULL, umv=TRUE, p=0.05, type=6, getlmoms=FALSE, ...) {
   return(uvlmoms(u,v=v, umv=umv, p=p, type=type, getlmoms=FALSE, ...))
}

"W_N5p12a" <-
function(u,v, para=1, ...) {
  para <- as.integer(para[1])
  if(para < 1) {
    warning("para must be a positive integer")
    return(NA)
  }
  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
    warning("length u = ",length(u), " and length v = ",length(v))
    warning("longer object length is not a multiple of shorter object length, no recycling in M()")
    return(NA)
  }
  # The extra hassle of vectorization made here is to handle situations
  # in which nested integrals are used where uneven vectors can be passed
  if(length(u) == 1) {
     u <- rep(u, length(v))
  } else if(length(v) == 1) {
     v <- rep(v, length(u))
  }
  # Nelsen(2006,p173)
  n <- para
  zz <- sapply(seq_len(length(u)), function(i) {
           for(k in seq_len(n)) {
             x1 <- y1 <- (k-1)/n
             x2 <- y2 <-  k   /n
             if(u[i] < x1 | u[i] > x2 | v[i] < y1 | v[i] > y2) next
             return(pmax((k-1)/n, u[i]+v[i]-k/n))
           }; return(pmin(u[i], v[i])) })
  return(zz)
  #zz <- pmin(u,v) # otherwise
  #for(i in seq_len(m)) {
  #  for(k in seq_len(n)) {
  #    x1 <- y1 <- (k-1)/n
  #    x2 <- y2 <-  k   /n
  #    if(u[i] < x1 | u[i] > x2 | v[i] < y1 | v[i] > y2) next
  #    zz[i] <- pmax((k-1)/n, u[i]+v[i]-k/n)
  #  }
  #}
  # (u,v) \in [(k-1)/n, k/n] x [(k-1)/n, k/n] for k=1,2,..para
  # max((k-1)/n, u+v - k/n)
  return(zz)
}

#UV <- simCOP(n=100, cop=Wshuf, para=2)

"JOMAcop" <- # Joe (2014, p. 177)
function(u, v, para=NULL, rhotau=NULL, cortype=c("kendall", "spearman", "tau", "rho"), ...) {
  cortype <-   tolower(cortype)
  cortype <- match.arg(cortype)
  if(is.null(para)) {
    if(cortype == "spearman" | cortype == "rho") {
      if(is.null(rhotau)) {
        rho <- cor(u,v, method="spearman")
      } else {
        rho <- rhotau
      }
      ofunc_rho <- function(trial_para, target_rho=NA) {
        rho <- rhoCOP(cop=JOMAcop, para=trial_para)
        return(target_rho - rho)
      }
      para <- uniroot(ofunc_rho, interval=c(-1, +1), target_rho=rho)$root
      names(para) <- "theta"
      names(rho ) <- "Spearman Rho"
      return(list(para=para, rho=rho))
    } else if(cortype == "kendall" | cortype == "tau") {
      if(is.null(rhotau)) {
        tau <- cor(u,v, method="kendall")
      } else {
        tau <- rhotau
      }
      ofunc_tau <- function(trial_para, target_tau=NA) {
        tau <- tauCOP(cop=JOMAcop, para=trial_para)
        return(target_tau - tau)
      }
      para <- uniroot(ofunc_tau, interval=c(-1, +1), target_tau=tau)$root
      names(para) <- "theta"
      names(tau ) <- "Kendall Tau"
      return(list(para=para, tau=tau))
    } else {
      stop("should not be here in logic")
    }
    }

    if(para[1] < -1) {
      warning("Parameter Delta < -1")
      return(NULL)
    } else if(para[1] > +1) {
      warning("Parameter Delta > +1")
      return(NULL)
    }

    if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
       warning("length u = ", length(u), " and length v = ", length(v))
       warning("longer object length is not a multiple of shorter object length, ",
               "no recycling")
       return(NA)
    }
    if(length(u) == 1) {
       u <- rep(u, length(v))
    } else if(length(v) == 1) {
       v <- rep(v, length(u))
    }

    d <- para[1]; t <- sign(d)  # This is departure from Joe and Ma parameterization as I bend the
    d <- 1 - abs(d); k <- 1 / d # rules to make the copula in (-1, +1) as part of extensive testing.

    # These break points come from simulation testing of the numerical derivative and the behavior
    # of tauCOP() on this copula. The Gamma distribution can be very difficult as shape parameter
    # goes "small" and I have not found a work around other than the two thresholds here and the
    # whole business of the d in (-1, +1) for this parameterization is that the d > 1 + d --> infinity
    # breaks down far to early in the derCOPinv() [including also Joe's direct inversion formula]
    # for suitability to copBasic. I remark that the R code base coming with Joe's book does not
    # appear to go this far in study of the numerical limitations of the Gamma distribution and
    # asymmetry whether the relation is negative associated or positive associated.
    if(d < 0.02 & d > 0.013) {
      Hcdf <- function(s, d=d, k=k) pgamma(s^k, d)    # Joe (2014, p. 179) "lower bounds"
      Hinv <- function(p, d=d     ) qgamma(p,   d)^d  # Joe (2014, p. 179) "lower bounds"
      if(t == -1) {  # Native to Joe--Ma definition for negative association
        return(      1 - Hcdf(Hinv(1-u, d) + Hinv(1-v, d), d=d, k=k)  )
      } else {       # Rotation 90 degrees counter-clockwise (see COP(..., reflect=4 or "grave") )
        v <- 1 - v   # note manipulation of u but "v -" against the copula formula
        return( u - (1 - Hcdf(Hinv(1-u, d) + Hinv(1-v, d), d=d, k=k)) )
      }
    } else if(d < 0.013) {
      if(t == -1) {  # Native to Joe--Ma definition for negative association
        return(    pmax(u + v - 1, 0))
      } else {       # Rotation 90 degrees counter-clockwise (see COP(..., reflect=4 or "grave") )
        v <- 1 - v   # note manipulation of u but "v -" against the copula formula
        return(u - pmax(u + v - 1, 0))
      }
    }
    if(t == -1) {    # Native to Joe--Ma definition for negative association
      return(      1 - pgamma( (qgamma(1-u, d)^d + qgamma(1-v, d)^d )^k, d)  )
    } else {         # Rotation 90 degrees counter-clockwise (see COP(..., reflect=4 or "grave") )
      v <- 1 - v     # note manipulation of u but "v -" against the copula formula
      return( u - (1 - pgamma( (qgamma(1-u, d)^d + qgamma(1-v, d)^d )^k, d)) )
    }
}


# "JOMAcop.derCOP" <- function(u, v, para=1) { # Joe (2014, p. 178)
#   u <- u[1]
#   d <- para[1]; k <- 1/d
#   x <- qgamma(1-u, d)
#   exp(-(x^d + qgamma(1-v, d)^d)^k ) / exp(-x)
# }

# "JOMAcop.derCOPinv" <- function(u, t, para=1) { # Joe (2014, p. 178)
#   u <- u[1]
#   d <- para[1]; k <- 1/d
#   x <- qgamma(1-u, d)
#   y <- ((x - log(t))^d - x^d)^k
#   1 - pgamma(y, d)
#}

"RFcop" <- function(u, v, para=NULL, rho=NULL, tau=NULL,
                           fit=c('rho', 'tau'), ...) {
    if(is.null(para)) {
      fit <- match.arg(fit)
      if(is.null(tau) & is.null(rho)) {
        if(fit == "rho") {
          rho <- cor(u,v, method="spearman")
        } else {
          tau <- cor(u,v, method="kendall")
        }
      }
      rt <- NULL
      if(is.null(rho)) {
        try(rt <- uniroot(function(t) { 2*t/(3-t) - tau}, interval=c(0,1)), silent=TRUE)
        if(is.null(rt)) {
          warning("could not uniroot the Theta from the Tau")
          return(NULL)
        }
        para <- rt$root
        names(para) <- "theta"
        names(tau)  <- "Kendall Tau"
        return(list(para=para, tau=tau))
      } else {
        try(rt <- uniroot(function(t) { t*(4-3*t)/(2-t)^2 - rho}, interval=c(0,1)), silent=TRUE)
        if(is.null(rt)) {
          warning("could not uniroot the Theta from the Rho")
          return(NULL)
        }
        para <- rt$root
        names(para) <- "theta"
        names(rho)  <- "Spearman Rho"
        return(list(para=para, rho=rho))
      }
    }
    if(length(para) == 1) {
       if(para < 0 | para > 1) {
         warning("Parameter must be 0 <= Theta <= 1")
         return(NULL)
       }
       tau <- 2*para/(  3-para)
       rho <-   para*(4-3*para)/(2-para)^2
    } else {
       warning("Parameter Theta can not be a vector")
       return(NULL)
    }
    if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
       warning("length u = ", length(u), " and length v = ", length(v))
       warning("longer object length is not a multiple of shorter object length, ",
               "no recycling")
       return(NA)
    }
    if(length(u) == 1) {
       u <- rep(u, length(v))
    } else if(length(v) == 1) {
       v <- rep(v, length(u))
    }
    m <- 1 - para; p <- 1 + para; g <- 1:length(u)
    rng <- sapply(g, function(i) range(c(u[i], v[i])))
    mx <- sapply(g, function(i) max(c(u[i],v[i])))
    mn <- sapply(g, function(i) min(c(u[i],v[i]))) # M(u,v)
    cop <- mn + (m/p)*(u*v)^(1/m)*(1 - mx^(-p/m))
    # Test one is more obvious, but test two requires the simulation
    # testing on derCOPinv shown below and commented out.
    if(any(is.nan(cop))) cop[is.nan(cop)] <- mn[is.nan(cop)]
    # Test two. The derCOPinv will end up trying to return the maximum,
    # which seems as mx[! is.finite(cop)] **Note mx** but in simulation
    # that produces little kicks away from M(u,v). It appears the proper
    # interception then is the mn **Note not mx** used in the reassignment below
    if(any(! is.finite(cop))) cop[! is.finite(cop)] <- mn[! is.finite(cop)]
    #print(c(u,v, cop, (u*v)^(1/m), (1 - mx^(-p/m))))
    return(cop)
}

#\note{
# The additive quantity to the right of the
# \eqn{\mathbf{M}(u,v)} in the definition will head to
# \code{NaN} as \eqn{\Theta \rightarrow 1^{-}}. The
# \code{NaN} but the quantity can be replaced with zero,
# which is done in the source code. Testing indicates
# that for large \eqn{\Theta} that \code{uniroot()} in
# \code{\link{derCOPinv}} will occassionally trigger this
# warning message:
#  \preformatted{
#   In uniroot(func, interval = c(0, 1), u = u, LHS = t,
# cop = cop,  ... :
#               NA/Inf replaced by maximum positive value
#  }
# This warning appears harmless. For example, a simulation
# test of \eqn{n=1{,}000} for \eqn{\Theta = 0.99},
# produces 10 warnings. A \eqn{\Theta = 1} itself does
# not because equivalent \code{\link{M}} does not do so
# when it is simulated. Perhaps some subtle algorithmic
# change in \code{RFcop} sources could be made in the
# future.
#}

# The change hinted at is the Test two in the copula,
# for which an example of the trigger is in the comments
# below.

#while(1) { warning(u, " ", t);
# u <- runif(1); t <- runif(1);
# derCOPinv(cop=RFcop, u, t, para=0.999);
# print(last.warning) }

#u <- 0.688409020658582
#t <- 0.508630372118205
#derCOPinv(cop=RFcop, u, t, para=0.999)

"lcomoms2.ABKGcop2parameter" <-
function(solutionenvir=NULL,
         T2.12=NULL, T2.21=NULL,
         T3.12=NULL, T3.21=NULL,
         T4.12=NULL, T4.21=NULL,
         t2eps=0.1, t3eps=0.1, t4eps=0.1,
         compt2erruv=TRUE, compt2errvu=TRUE,
         compt3erruv=TRUE, compt3errvu=TRUE,
         compt4erruv=TRUE, compt4errvu=TRUE,
         uset3err=TRUE, uset4err=FALSE,
         setreturn=1, maxtokeep=1e5) {

  if(is.null(T2.12)) {
    warning("T2.12 is NULL")
    return()
  }
  if(is.null(T2.21)) {
    warning("T3.21 is NULL")
    return()
  }
  if(is.null(T3.12)) {
    warning("T3.12 is NULL")
    return()
  }
  if(is.null(T3.21)) {
    warning("T3.21 is NULL")
    return()
  }
  if(is.null(T4.12)) {
    warning("T4.12 is NULL")
    return()
  }
  if(is.null(T4.21)) {
    warning("T4.21 is NULL")
    return()
  }

  if(is.null(solutionenvir)) {
    warning("Need the solution environment")
    return()
  }

  SOLUTIONS <- matrix(ncol=12, nrow=maxtokeep)

  eachfile <- ls(solutionenvir)
  eachfile <- sample(eachfile, length(eachfile), replace=FALSE)
  n <- 0; i <- 0

  # need the column names for a much later attributing
  tmp.lcom <- get(eachfile[1], envir=solutionenvir)
  if(is.data.frame(tmp.lcom)) {
    the.names <- names(tmp.lcom)
  } else {
    the.names <- unlist(attributes(tmp.lcom)$dimnames[2])
  }

  # For each "file" of solutions in the given environment
  # extract those arbitrary solutions meeting the *eps values
  # to build a much smaller suite of solutions from which we
  # will extract the solution with the smallest error
  for(file in eachfile) {
    if(i >= maxtokeep) next;
    message("Searching in ", file, appendLF=FALSE)
    # lcom contains the simulated L-comoments from known parameters
    lcom <- get(file, envir=solutionenvir)
    #print(lcom)
    if(is.data.frame(lcom)) lcom <- as.matrix(lcom)
    ix <- 1:attributes(lcom)$dim[1]
    ixn <- length(ix)

    n <- n + length(ix)


    # abs sum of the two offsets of 1 wrt 2 and 2 wrt 1
    ifelse(compt2erruv, t2erruv <- abs(T2.12 - lcom[,5]), t2erruv <- rep(0, ixn))
    ifelse(compt2errvu, t2errvu <- abs(T2.21 - lcom[,6]), t2errvu <- rep(0, ixn))
    t2err <- t2erruv + t2errvu
    t2errmin <- min(t2err) # capture the minimum for filtering

    # now create sub index of only those small errors in Tau2
    ixt2  <- ix[abs(t2err - t2errmin) < t2eps]
    ixt2n <- length(ixt2)

    # compute another abs sum of the two offsets of 1 wrt 2 and 2 wrt 1
    ifelse(compt3erruv, t3erruv <- abs(T3.12 - lcom[ixt2,7]), t3erruv <- rep(0, ixt2n))
    ifelse(compt3errvu, t3errvu <- abs(T3.21 - lcom[ixt2,8]), t3errvu <- rep(0, ixt2n))

    t3err <- t3erruv + t3errvu
    t3errmin <- min(t3err) # capture the minimum for filtering

    # now create sub index of only those small errors in Tau3
    ixt3  <- ixt2[abs(t3err - t3errmin) < t3eps]
    ixt3n <- length(ixt3)
    if(ixt3n == 0) {
       warning("No potential solutions in L-coskew, resetting counts to L-correlation")
       ixt3  <- ixt2
       ixt3n <- ixt2n
    }
    # compute another abs sum of the two offsets of 1 wrt 2 and 2 wrt 1
    ifelse(compt4erruv, t4erruv <- abs(T4.12 - lcom[ixt3,9]), t4erruv <- rep(0, ixt3n))
    ifelse(compt4errvu, t4errvu <- abs(T4.21 - lcom[ixt3,10]), t4errvu <- rep(0, ixt3n))
    t4err <- t4erruv + t4errvu
    t4errmin <- min(t4err) # capture the minimum for filtering

    # now create sub index of only those small errors in Tau4
    ixt4 <- ixt3[abs(t4err - t4errmin) < t4eps]
    ixt4n <- length(ixt4)
    if(ixt4n == 0) {
       warning("No potential solutions in L-cokurtosis, resetting counts to L-coskew")
       ixt4  <- ixt3
       ixt4n <- ixt3n
    }
    # Now pass through each of the potential solutions from this
    # file and store them in the SOLUTIONS matrix, which will be used
    # later to extract the minimum solution
    k <- length(lcom[ixt4,1])
    message(" found incremental solutions = ",k)

    if(i+k > maxtokeep) {
       k <- maxtokeep - i
    }
    for(j in 1:k) {
       if(i+j > maxtokeep) next;
       SOLUTIONS[i+j, ] <- sapply(1:12, function(x) { return(lcom[ixt4[j],x]) } )
    }
    i <- i + k # increment the file number
    #print(SOLUTIONS)
  }

  #message("Previous simulations checked: ",n)
  S <- SOLUTIONS[complete.cases(SOLUTIONS),] # safety check on NA, but
  #message("Truncating solutions in case of NAs")
  S <- as.matrix(S)
  #message("Solutions converted by as.matrix(S), just in case")
  if(attributes(S)$dim[2] == 1) S <- t(S)

  #print(attributes(S))
  # should have already been done by the preprocessing of the solutionenvir
  ns <- attributes(S)$dim[1] # number of incoming potential solutions
  nc <- attributes(S)$dim[2] # number of columns

  if(ns == 0) {
     warning("No solutions found")
  }

  message("Potential solutions found: ",ns)
  S <- as.data.frame(S)
  names(S) <- the.names
  S <- as.list(S)

  # Compute the 6 residuals
  S$t2.12res <- S$T2.12 - T2.12
  S$t2.21res <- S$T2.21 - T2.21
  S$t3.12res <- S$T3.12 - T3.12
  S$t3.21res <- S$T3.21 - T3.21
  S$t4.12res <- S$T4.12 - T4.12
  S$t4.21res <- S$T4.21 - T4.21

  # Rescale the abs of the residuals to put them all on the
  # same relative unit footing for combining
  ifelse(compt2erruv, t2erruv <- scale(abs(S$t2.12res), center=FALSE), t2erruv <- rep(0, ns))
  ifelse(compt2errvu, t2errvu <- scale(abs(S$t2.21res), center=FALSE), t2errvu <- rep(0, ns))
  S$t2err <- t2erruv + t2errvu

  ifelse(compt3erruv, t3erruv <- scale(abs(S$t3.12res), center=FALSE), t3erruv <- rep(0, ns))
  ifelse(compt3errvu, t3errvu <- scale(abs(S$t3.21res), center=FALSE), t3errvu <- rep(0, ns))
  S$t3err <- t3erruv + t3errvu

  ifelse(compt4erruv, t4erruv <- scale(abs(S$t4.12res), center=FALSE), t4erruv <- rep(0, ns))
  ifelse(compt4errvu, t4errvu <- scale(abs(S$t4.21res), center=FALSE), t4errvu <- rep(0, ns))
  S$t4err <- t4erruv + t4errvu

  S$jointerr <- S$t2err # L-correlation is always desired in minimization

  nb <- 2 # this is a counter used in a later sum() to return the
  # error of the fit back into the native units of the L-comoments
  # the value is 2 because of 1 wrt 2 and 2 wrt 1 with L-comoments
  if(uset3err) {
    nb <- nb + 2
    S$jointerr <- S$jointerr + S$t3err
  }
  if(uset4err) {
    nb <- nb + 2
    S$jointerr <- S$jointerr + S$t4err
  }

  S  <- as.data.frame(S)
  ix <- order(S$jointerr) # ordering indices of the solutions
  S  <- S[ix,] # order the entire data frame
  row.names(S) <- 1:length(S[,1]) # reset the row numbers
  i <- setreturn # this permits the user to "see"
  # other nearby solutions with the ith index used in the next line
  z <- as.list(S[i,1:nc]) # preparing the list object for the return()

  # Create a table for attachment to the return'ed list of real residuals
  # ** not rescaled units **
  resid <- data.frame(resT2.12=z$T2.12 - T2.12,
                      resT2.21=z$T2.12 - T2.12,
                      resT3.12=z$T3.12 - T3.12,
                      resT3.21=z$T3.21 - T3.21,
                      resT4.12=z$T4.12 - T4.12,
                      resT4.21=z$T4.21 - T4.21)
  my.names <- names(resid)
  resid[1,7] <- sum(abs(resid[1,1:nb])) # compute the true native unit
  # error of the L-comoments to the fitted copula
  names(resid) <- c(my.names, "SUM_ABS_USED_RESIDUALS")
  z$residuals <- resid

  z$solutions <- new.env() # the environment to house ALL of the
  # potential solutions meeting the *eps requirements
  assign("solutions", S, envir=z$solutions) # store the solutions

  return(z)
}

"jointCOP" <-
function(t, cop=NULL, para=NULL, type=c("and", "or"), ...) {

  type <- match.arg(type)

  if(length(t) > 1) {
     warning("only the first value in 't' will be used")
     t <- t[1]
  }

  "orfunc" <- function(v, LHS=NULL, cop=NULL, para=NULL, ...) {
      u <- v # duplication is made so that the heredity of the next line is seen
      RHS <- duCOP(u, v, cop = cop, para = para, ...)
      #message("u=",u,",  v=",v, ",  LHS=",LHS, ",  RHS=",RHS, ", DELTA=", LHS-RHS)
      return(LHS - RHS)
  }
  my.rt <- NULL
  if(type == "and") {
     u <- v <- diagCOPatf(t, cop=cop, para=para)
     zz <- c(u, v, t)
     names(zz) <- c("U", "V", "jointANDprob")
     return(zz)
  } else if(type == "or") {
     lo <- duCOP(0,0, cop=cop, para=para, ...)
     if(is.na(lo) | ! is.finite(lo)) lo <- .Machine$double.eps
     try(my.rt <- uniroot(orfunc, interval=c(lo,t), LHS=t,
              cop=cop, para=para, tol=.Machine$double.eps/10, ...), silent=FALSE)
     if(is.null(my.rt)) {
        u <- v <- NA
     } else if(length(my.rt$root) != 0) {
        u <- v <- my.rt$root
     } else {
        u <- v <- NA
     }
     zz <- c(u, v, t)
     names(zz) <- c("U", "V", "jointORprob")
     return(zz)
  } else {
     stop("should not be here in logic")
  }
}

"copBasic.fitpara.beta" <-
function(uv=NULL, popstat=NULL, statf=NULL, cop=NULL,
         paradim=1, interval=NULL, par.init=NULL, ...) {
   if(is.null(cop)) {
      warning("must have copula argument specified, returning NULL")
      return(NULL)
   }
   if(paradim == 1) {
      if(is.null(interval)) {
         warning("must have parameter interval specified, returning NULL")
         return(NULL)
      }
   } else {
      if(is.null(par.init)) {
         warning("must have initial guesses specified, returning NULL")
         return(NULL)
      }
   }
   if(is.null(statf)) {
      warning("must have measure of association function specified, ",
              "returning NULL")
      return(NULL)
   }

   #target.stat <- NULL
   ifelse(is.null(uv), target.stat <- popstat,
                       target.stat <- statf(para=uv, as.sample=TRUE))

   if(paradim == 1) {
      "objfuni" <- function(p, stat=NULL, cop=NULL, ...) {
         stat - statf(cop=cop, para=p, ...)
      }
   } else {
      "objfmulti" <- function(p, cop=cop, ...) statf(p, ...)
   }

   rt <- NULL
   if(paradim == 1) {
      try(rt <- uniroot(objfuni, interval, stat=target.stat, cop=cop))
   } else {
      try(rt <- optim(par.init, objfmulti, stat=target.stat, cop=cop))
   }
   if(is.null(rt)) {
      warning("could not find parameter(s) numerically, returning NA")
      return(NA)
   }

   ifelse(paradim > 1, para <- rt$par, para <- rt$root)
   return(para)
}

"qua.regressCOP2" <-
function(f=0.5, v=seq(0.01, 0.99, by=0.01), cop=NULL, para=NULL, ...) {
  if(is.null(cop)) {
     warning("must have copula argument specified, returning NULL")
     return(NULL)
  }
  U <- sapply(1:length(v), function(i) {
          u <- derCOPinv2(cop=cop, v[i], f, para=para, ...)
          if(is.na(u)) {
             warning("could not uniroot in derCOPinv2, skipping i=", i," with V=", v[i])
             return(NA)
          }
          return(u) } )
  z <- data.frame(U=U,V=v)
  return(z)
}

"concordCOP" <- function(cop=NULL, para=NULL, cop2=NULL, para2=NULL, ...) {
   if(is.null(cop)) {
      warning("must have copula argument specified, returning NULL")
      return(NULL)
   }
   if(is.null(cop2)) {
      warning("must provide the second copula [and optional parameters]")
      return()
   }
   tauCOP(cop=cop, para=para, cop2=cop2, para2=para2, ...)
}

"tauCOP" <-
function(cop=NULL,  para=NULL,
         cop2=NULL, para2=NULL, as.sample=FALSE,  brute=FALSE, delta=0.002, ...) {
   if(as.sample) {
      if(is.null(para)) {
         warning("Sample Kendall's Tau desired but para is NULL, ",
                 "returning NULL")
         return(NULL)
      }
      if(length(names(para)) != 2) {
        warning("para argument must be data.frame having only two columns, ",
                "returning NULL")
        return(NULL)
      }
      return(cor(para[,1], para[,2], method="kendall"))
   }

   if(is.null(cop)) {
      warning("must have copula argument specified, returning NULL")
      return(NULL)
   }

   if(is.null(cop2)) {
     cop2  <- cop  # This is the expected operation
     para2 <- para # as Kendall's Tau gets returned
   }

   us <- vs <- seq(.Machine$double.eps, 1-.Machine$double.eps, delta)
   if(brute) {
     Q <- sum(sapply(us, function(u) {
             sum(sapply(vs, function(v) {
                return( derCOP(u,v, cop=cop,  para=para,  ...) *
                        derCOP2(u,v, cop=cop2, para=para2, ...))
             }))
          }))
     Q <- 4*(0.5 - Q*delta^2) - 1 # SEE P 164 of NELSON 2006!
     if(Q >  1) Q <-  1 # assume rounding errors just breaking through
     if(Q < -1) Q <- -1 # again for rounding errors
     return(Q)
   }

   myint <- NULL
   try(myint <- integrate(function(u) {
               sapply(u,function(u) {
                 integrate(function(v) {
                  derCOP( u,v, cop=cop,  para=para,  ...) *
                 derCOP2( u,v, cop=cop2, para=para2, ...)}, 0,1)$value
             })}, 0,1))
    if(is.null(myint)) {
        warning("error on dual partial derivative integration ",
                "(copula singularity?), swapping copulas (Nelsen corollary ",
                "5.1.2)")
        try(myint <- integrate(function(u) {
               sapply(u,function(u) {
                 integrate(function(v) {
                  derCOP( u,v, cop=cop2, para=para2, ...)*
                 derCOP2( u,v, cop=cop,  para=para,  ...)}, 0,1)$value
             })}, 0,1))
        if(is.null(myint)) {
           warning("another error on dual partial derivative integration ",
                   "(again copula singularity?), kicking over to integration ",
                   "of the Kendall Function")
           tau<-3-4*integrate(function(f) kfuncCOP(f,cop=cop,para=para),0,1)$value
           return(tau)
        }
    }
    Q <- 4*(0.5 - myint$value) - 1
    if(Q >  1) Q <-  1 # assume rounding errors just breaking through
    if(Q < -1) Q <- -1 # again for rounding errors
    return(Q)
}

"composite1COP" <-
function(u, v, para, ...) {
  if(is.null(para$alpha)) {
    warning("no alpha parameter given")
    return(NULL)
  }
  if(is.null(para$beta)) {
    warning("no beta parameter given")
    return(NULL)
  }
  alpha <- para$alpha; alpha.p <- 1 - alpha
  beta  <- para$beta;   beta.p <- 1 - beta

  if(is.null(para$para)) para$para <- para$para1
  if(is.null(para$cop )) para$cop  <- para$cop1

  return(u^alpha * v^beta * COP(u^alpha.p, v^beta.p, para=para, ...))
}

"khoudraji1COP" <-
function(u, v, para, ...) {
  if(is.null(para$alpha)) {
    warning("no alpha parameter given")
    return(NULL)
  }
  if(is.null(para$beta)) {
    warning("no beta parameter given")
    return(NULL)
  }
  alpha <- para$alpha; alpha.p <- 1 - alpha
  beta  <- para$beta;   beta.p <- 1 - beta

  if(is.null(para$para)) para$para <- para$para1
  if(is.null(para$cop )) para$cop  <- para$cop1

  return(u^alpha * v^beta * COP(u^alpha.p, v^beta.p, para=para, ...))
}


"khoudrajiPCOP" <-
function(u, v, para, ...) {
  if(is.null(para$alpha)) {
    warning("no alpha parameter given")
    return(NULL)
  }
  if(is.null(para$beta)) {
    warning("no beta parameter given")
    return(NULL)
  }
  alpha <- para$alpha; alpha.p <- 1 - alpha
  beta  <- para$beta;   beta.p <- 1 - beta

  if(is.null(para$para)) para$para <- para$para1
  if(is.null(para$cop )) para$cop  <- para$cop1

  return(u^alpha * v^beta * COP(u^alpha.p, v^beta.p, para=para, ...))
}

"EMPIRgrid_lkup" <-
function(u,v, para=NULL, ...) {
  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
    warning("length u = ", length(u), " and length v = ", length(v))
    warning("longer object length is not a multiple of shorter object length, ",
            "no recycling")
    return(NA)
  }
  if(           length(u) == 1) {
    u <- rep(u, length(v))
  } else if(    length(v) == 1) {
    v <- rep(v, length(u))
  }
  if(is.null(para)) {
    para <- EMPIRgrid_fast(cbind(u,v), gridonly=FALSE, ...)
  }
  if(! any(class(para) == "empirical.copula.grid")) {
    warning("expecting class to be 'empirical.copula.grid'")
  }
  if(! exists('empcop', para)) {
    warning("para does not contain in para$empcop for matrix of the gridded empirical copula")
  }
  # Extract the u and v probabilities of the gridded empirical copula and then make a vector of the
  ui <- para$u; vi <- para$v; ix <- seq_len(length(ui)) # indices for look up purposes to come.

  u[u < 0] <- 0; u[u > 1] <- 1
  v[v < 0] <- 0; v[v > 1] <- 1

  # https://en.wikipedia.org/wiki/Bilinear_interpolation
  x1i <- sapply(u, function(u) max( ix[ui <= u]) ) # lower indices of x  --->  x1
  y1i <- sapply(v, function(v) max( ix[vi <= v]) ) # lower indices of y  --->  y1
  x2i <- sapply(u, function(u) min( ix[ui >= u]) ) # upper indices of x  --->  y1
  y2i <- sapply(v, function(v) min( ix[vi >= v]) ) # upper indices of y  --->  y2

  x1 <- ui[x1i]; x2 <- ui[x2i]; y1 <- vi[y1i]; y2 <- vi[y2i]     # vectors of probs. from indices

  d   <- ( (x2 - x1) * (y2 - y1) )                               # vectorized denominator
  w11 <- ( (x2 - u ) * (y2 - v ) ) / d                           # vectorized weights
  w12 <- ( (x2 - u ) * (v  - y1) ) / d                           # vectorized weights
  w21 <- ( (u  - x1) * (y2 - v ) ) / d                           # vectorized weights
  w22 <- ( (u  - x1) * (v  - y1) ) / d                           # vectorized weights

  six  <- seq_len(length(u))  # indices of the sample values
  f11  <- sapply(six, function(k) para$empcop[ x1i[k], y1i[k] ])   # vectorized function values
  f12  <- sapply(six, function(k) para$empcop[ x1i[k], y2i[k] ])   # vectorized function values
  f21  <- sapply(six, function(k) para$empcop[ x2i[k], y1i[k] ])   # vectorized function values
  f22  <- sapply(six, function(k) para$empcop[ x2i[k], y2i[k] ])   # vectorized function values

  cop <- w11*f11 + w12*f12 + w21*f21 + w22*f22         # vectorized weighted mean computation
  cop[d == 0] <- f11[d == 0] # all denominators == 0 mean that the coordinates are on a single cell
  return(cop)
}

"LMRuvCOP" <-
function(v=seq(0.01, 0.99, by=0.01), cop=NULL, para=NULL, nsim=1E5,
         subdivisions=200L, rel.tol=.Machine$double.eps^0.25, abs.tol=rel.tol, ...) {
  # Jones (2004, eq. 15b) and Asquith (2011, 6.10 is wrong with 1/(1-r) and needs 1/(r-1))
  #                                  refer to lmomco R package inst/ERRATA_FOR_ISBN9781463508418.txt
  Lrv <- function(v, r=2) {
    sapply(v, function(v) { sum(sapply(0:(r-2), function(j) {
                     (-1)^j * exp(lchoose(r-1,j)+lchoose(r-1,j+1))*v^(r-2-j)*(1-v)^j })) })/(r-1)
  }

  medvatv <- med.regressCOP2(v, cop=COP, para=para)[,1]
  lam1atv <- EuvCOP(v, cop=COP, para=para, rel.tol=rel.tol, abs.tol=abs.tol,
                                           subdivisions=subdivisions, ...)
  lam2atv <- sapply(v, function(v) { E <- NULL
                                 try(E <- integrate(function(u, para=para) {
                t <- derCOP2(cop=COP, u, v, para=para); return(t*(1-t)*Lrv(t, r=2)) }, 0, +1,
                                           para=para, subdivisions=200)$value, silent=TRUE)
                          if(is.null(E)) { t <- derCOP2(cop=COP, runif(nsim), v, para=para)
                                     E <- mean(t*(1-t)*Lrv(t, r=2))
                               names(E) <- "sim"
                          }
                              return(E) })
  lam3atv <- sapply(v, function(v) { E <- NULL
                                 try(E <- integrate(function(u, para=para) {
                t <- derCOP2(cop=COP, u, v, para=para); return(t*(1-t)*Lrv(t, r=3)) }, 0, +1,
         rel.tol=rel.tol, abs.tol=abs.tol, para=para, subdivisions=200)$value, silent=TRUE)
                          if(is.null(E)) { t <- derCOP2(cop=COP, runif(nsim), v, para=para)
                                     E <- mean(t*(1-t)*Lrv(t, r=3))
                               names(E) <- "sim"
                          }
                              return(E) })
  lam4atv <- sapply(v, function(v) { E <- NULL
                                 try(E <- integrate(function(u, para=para) {
                 t <- derCOP2(cop=COP, u, v, para=para); return(t*(1-t)*Lrv(t, r=4)) }, 0, +1,
          rel.tol=rel.tol, abs.tol=abs.tol, para=para, subdivisions=200)$value, silent=TRUE)
                          if(is.null(E)) { t <- derCOP2(cop=COP, runif(nsim), v, para=para)
                                     E <- mean(t*(1-t)*Lrv(t, r=4))
                               names(E) <- "sim"
                          }
                              return(E) })
  zzz <- list(median=medvatv, L1=lam1atv, L2=lam2atv, T3=lam3atv/lam2atv, T4=lam4atv/lam2atv)
  return(zzz)
}

"mleCOP" <-
function(u, v=NULL, cop=NULL, parafn=function(k) return(k),
          interval=NULL, init.para=NULL, verbose=FALSE, control=list(),
          the.zero=.Machine$double.eps^0.25, s=0, ...) {
   verbose <- as.integer(verbose)
   if(is.null(v)) {
      if(length(names(u)) != 2) {
         warning("a data.frame having only two columns is required")
         return(NULL)
      }
      v <- u[, 2]; u <- u[, 1] # v must come before u
   }
   if(length(u) != length(v)) {
      warning("argument(s) or implied arguments u and v are unequal in ",
              "length, returning NULL")
      return(NULL)
    }
    objfunc <- function(thetas, ...) {
       para <- parafn(thetas)
       if(verbose >= 2) {
         message("mleCOP---parameters (transformed to as copulas receive):")
         if(is.list(para)) {
           tmpp <- para
           tmpp[grep("cop", tmpp)] <- NULL
           for(p in seq_len(length(tmpp))) tmpp[[p]] <- round(tmpp[[p]], digits=8)
         } else {
           tmpp <- para
         }
         paratxt <- paste(names(tmpp), tmpp, sep="=", collapse=", ")
         message("         ", paratxt)
         if(length(s) >= 2) {
           t <- simCOPv(s, cop=cop, para=para)
           plot(s,t, xlab="U, NONEXCEEDANCE PROBABILITY",
                     ylab="V, NONEXCEEDANCE PROBABILITY", lwd=0.62)
           mtext(paratxt, font=2, line=0.5)
         }
       }
       copdf <- densityCOP(u,v, cop=cop, para=para, sumlogs=TRUE,
                                the.zero=the.zero, ...)
       if(verbose >= 1) message("  densityCOP---sumlogs = ", round(copdf, digits=8))
       return(copdf)
    }
    SMALL <- 0.01 # a one percent closeness
    rt <- NULL
    if(! is.null(interval)) {
       # need optimise import!!! stats::optimise
       try(rt <- optimise(f=objfunc, interval=interval, maximum=TRUE, ...))
       if(! is.null(rt)) {
         the.rt <- rt$maximum
         if(abs(1 - the.rt/min(interval)) < SMALL) {
            warning("lower bounds might be too close to solution")
         }
         if(abs(1 - the.rt/max(interval)) < SMALL) {
            warning("upper bounds might be too close to solution")
         }
         rt$packagetext <- "Solution by package copBasic + stats::optimise()"
         rt$para <- parafn(the.rt)
         rt$loglik <- rt$objective
         rt$AIC  <- 2*length(rt$para)                - 2*rt$loglik
         rt$BIC  <-   length(rt$para)*log(length(u)) - 2*rt$loglik
         return(rt)
       } else {
         return(NULL)
       }
    } else if(! is.null(init.para)) {
       control$fnscale <- -1
       if(length(init.para) == 1) {
          warning("init.para is length one, need to use 1D 'Brent' method\n",
                  " but then the interval **will require** specification,\n",
                  " proceeding anyway with init.para and optim() call,\n",
                  " other warning message will be triggered by optim(),\n",
                  " suggest calling > mleCOP(..., interval=c(N,M), ...)")
       }
       try(rt <- optim(init.para, fn=objfunc, control=control, ...))
       if(! is.null(rt)) {
         rt$packagetext <- "Solution by package copBasic + stats::optim()"
         rt$para <- parafn(rt$par)
         rt$loglik <- rt$value
         rt$AIC  <- 2*length(rt$para)                - 2*rt$loglik
         rt$BIC  <-   length(rt$para)*log(length(u)) - 2*rt$loglik
         return(rt)
       } else {
         return(NULL)
       }
    } else {
       warning("ambiguous search criteria for either interval or init.para")
       return(NULL)
    }
}

"N4212cop" <-
function(u, v, para=NULL, infis=100, ...) {
    TT <- para[1]

    if(is.null(para)) {
       warning("Empty para argument, need value on [1,Inf)")
       return()
    }

    if(TT < 1) {
       warning("Theta < 1, invalid parameter")
       return()
    }

    if(TT == 1)    return(PSP(u,v)) # the PSP copula
    if(TT > infis) return(  M(u,v)) # upper copula bounds

    cop <- ( u^-1 - 1 )^TT + (v^-1 - 1)^TT
    cop <- ( 1 + cop^(1/TT) )^-1

    return(cop)
}

"M_N5p12b" <-
function(u,v, para=1, ...) {
  para <- as.integer(para[1])
  if(para < 1) {
    warning("para must be a positive integer")
    return(NA)
  }
  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
    warning("length u = ",length(u), " and length v = ",length(v))
    warning("longer object length is not a multiple of shorter object length, no recycling in M()")
    return(NA)
  }
  # The extra hassle of vectorization made here is to handle situations
  # in which nested integrals are used where uneven vectors can be passed
  if(length(u) == 1) {
     u <- rep(u, length(v))
  } else if(length(v) == 1) {
     v <- rep(v, length(u))
  }
  # Nelsen(2006,p173)
  n <- para
  zz <- sapply(seq_len(length(u)), function(i) {
           for(k in seq_len(n)) {
             x1 <- (  k-1)/n; y1 <- (n-k  )/n
             x2 <-    k   /n; y2 <- (n-k+1)/n
             if(u[i] < x1 | u[i] > x2 | v[i] < y1 | v[i] > y2) next
              return( pmin(u[i]-(k-1)/n, v[i]-(n-k)/n) )
           }; return( pmax(u[i]+v[i]-1,0)) })
  return(zz)
  #zz <- pmax(u+v-1,0) # otherwise
  #for(i in seq_len(m)) {
  #  for(k in seq_len(n)) {
  #    x1 <- (  k-1)/n; y2 <- (n-k+1)/n
  #    x2 <-    k   /n; y1 <- (n-k  )/n
  #    #segments(x1, y1, x1=x2, y1=y2)
  #    if(u[i] < x1 | u[i] > x2 | v[i] < y1 | v[i] > y2) next
  #    zz[i] <- pmin(u[i]-(k-1)/n, v[i]-(n-k)/n)
  #  }
  #}
  # (u,v) \in [(k-1)/n, k/n] x [(n-k+1)/n, (n-k)/n] for k=1,2,..para
  # min(u - (k-1)/n, v - (n-k)/n)
  return(zz)
}

#UV <- simCOP(n=100, cop=Mshuf, para=20)

"vuongCOP" <- function(u, v=NULL, cop1=NULL, cop2=NULL, para1=NULL, para2=NULL,
                                  alpha=0.05, method=c("D12", "AIC", "BIC"),
                                  the.zero=.Machine$double.eps^0.25, ...) {
   method <- match.arg(method)
   if(is.null(v)) {
      if(length(names(u)) != 2) {
         warning("a data.frame having only two columns is required")
         return(NULL)
      }
      v <- u[,2]; u <- u[,1] # v must come before u
   }
   if(length(u) != length(v)) {
      warning("argument(s) or implied arguments u and v are unequal in length, returning NULL")
      return(NULL)
   }

   n <- length(u)
   the.qt <- qt(1 - alpha/2, df=(n-2))

   "Di" <- function(x,y) {
       f1 <- densityCOP(x,y, cop=cop1, para=para1, the.zero=the.zero, ...)
       f2 <- densityCOP(x,y, cop=cop2, para=para2, the.zero=the.zero, ...)
       tmp <- log(f2/f1)
       return(tmp)
   }
   ns <- 1:n
   hatD12 <- sum(sapply(ns, function(i) {  Di(u[i],v[i])             })) / n
   varD12 <- sum(sapply(ns, function(i) { (Di(u[i],v[i]) - hatD12)^2 })) / (n-1)
   sigmaD12 <- sqrt(varD12)

   parameters <- c(alpha, as.integer(n), the.qt, sigmaD12)
   names(parameters) <- c("alpha", "sample.size", "qt(1-alpha/2, df)", "sigmaD12")

      lo  <-  hi  <- the.qt*sigmaD12/sqrt(n)
   D12lo  <- hatD12 - lo; D12hi <- hatD12 + hi
   D12    <- c(VuongD.lower=D12lo, VuongD=hatD12, VuongD.upper=D12hi,
               conf.interval=(100*(1-alpha)))
   names(D12) <- c("VuongD.lower", "VuongD", "VuongD.upper", "conf.interval")
   D12pvalue <- pt(hatD12*sqrt(n)/sigmaD12, df=(n-2))
   if(hatD12 == 0) {
     D12pvalue <- 1 # median
   } else if(hatD12 < 0) {
     D12pvalue <-    D12pvalue  * 2
   } else {
     D12pvalue <- (1-D12pvalue) * 2
   }

   dim1 <- dim(para1)
   dim1 <- ifelse(is.null(dim1), length(para1), sum(dim1))
   dim2 <- dim(para2)
   dim2 <- ifelse(is.null(dim2), length(para2), sum(dim2))
   # AIC and BIC will match if the dimension of the parameter vector/matrix
   # for the two copulas equal each other and thus the difference is zero.
   AICm <- hatD12 -              (dim2 - dim1) / n
   AIClo <- AICm - lo; AIChi <- AICm + hi
   AIC <- c(AIC.lower=AIClo, AIC=AICm, AIC.upper=AIChi,
                     conf.interval=(100*(1-alpha)))
   names(AIC) <- c("AIC.lower", "AIC", "AIC.upper", "conf.interval")
   AICpvalue <- pt(AICm*sqrt(n)/sigmaD12, df=(n-2))
   if(AICm == 0) {
     AICpvalue <- 1 # median
   } else if(hatD12 < 0) {
     AICpvalue <-    AICpvalue  * 2
   } else {
     AICpvalue <- (1-AICpvalue) * 2
   }

   BICm <- hatD12 - (1/2)*log(n)*(dim2 - dim1) / n
   BIClo <- BICm - lo; BIChi <- BICm + hi
   BIC <- c(BIC.lower=BIClo, BIC=BICm, BIC.upper=BIChi,
                     conf.interval=(100*(1-alpha)))
   names(BIC) <- c("BIC.lower", "BIC", "BIC.upper", "conf.interval")
   BICpvalue <- pt(BICm*sqrt(n)/sigmaD12, df=(n-2))
   if(BICm == 0) {
     BICpvalue <- 1 # median
   } else if(hatD12 < 0) {
     BICpvalue <-    BICpvalue  * 2
   } else {
     BICpvalue <- (1-BICpvalue) * 2
   }

   if(method == "D12") {
      CKlo <- D12lo; CKhi <- D12hi
      txt <- "The D12 method used for relative fit judgement (not fit adequacy)"
   } else if(method == "AIC") {
      CKlo <- AIClo; CKhi <- AIChi
      txt <- "The Akaike (AIC) method used for relative fit judgement (not fit adequacy)"
   } else {
      CKlo <- BIClo; CKhi <- BIChi
      txt <- "The Schwarz (BIC) method used for relative fit judgement (not fit adequacy)"
   }
   if(CKlo < 0 & CKhi > 0) { # The interval contains zero
      message <- "Copulas 1 and 2 are not significantly different at 100 x (1-alpha)"
      result  <- 0
   } else if(CKhi < 0) {
      message <- "Copula 1 has better fit than Copula 2 at 100 x (1-alpha) level"
      result  <- 1
   } else if(CKlo > 0) {
      message <- "Copula 2 has better fit than Copula 1 at 100 x (1-alpha) level"
      result  <- 2
   } else {
      message <- "Error: Evidently Copula 1 is the same as Copula 2"
      result  <- NA
   }

   pvalues <- c(D12pvalue, AICpvalue, BICpvalue)
   names(pvalues) <- c("VuongD p-value (two side)",
                          "AIC p-value (two side)",
                          "BIC p-value (two side)")

   zz <- list(title="Vuong's Procedure for Parametric Copula Comparison",
              method=txt, message=message, result=result, p.value=pvalues,
              D12=D12, AIC=AIC, BIC=BIC, parameters=parameters)
   return(zz)
}

"PSP" <-
function(u,v,...) {
  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
    warning("length u = ",length(u), " and length v = ",length(v))
    warning("longer object length is not a multiple of shorter object length, ",
            "no recycling in PSP()")
    return(NA)
  }
  # The extra hassle of vectorization made here is to handle situations
  # in which nested integrals are used where uneven vectors can be passed
  # Situation came to life years after initial release in testing
  # rhoCOP(W) and rhoCOP(M) for which Nelsen (2006) provides analytical solutions
  if(length(u) == 1) {
     u <- rep(u, length(v))
  } else if(length(v) == 1) {
     v <- rep(v, length(u))
  }
  return( exp(log(u) + log(v) - log(u + v - (u*v))) )
  #return(sapply(1:length(u), function(i) {
  #                  exp(log(u[i]) + log(v[i]) - log(u[i] + v[i] - (u[i]*v[i]))) }))
}

"EMPIRcopdf" <-
function(para=NULL, ...) {
  if(length(names(para)) != 2) {
    warning("a data.frame having only two columns is required")
    return(NULL)
  }

  uobs <- para[,1];
  vobs <- para[,2];
  n <- length(uobs);
  
  m <- length(uobs);
  if(m != length(vobs)) {
    warning("lengths of u and v are not identical")
    return(NULL) 
  }
  empcop <- EMPIRcop(uobs,vobs, para=para, ...)
  return(data.frame(u=uobs, v=vobs, empcop=empcop))
}

blomCOPss <-
function(cop=NULL, para=NULL, uu=rep(0.5, 2), vv=rep(0.5, 2), trap.nan=TRUE,
          as.sample=FALSE, ctype=c("weibull", "hazen", "1/n",
                                   "bernstein", "checkerboard"), ...) {
   # Schmid and Schmidt doi:10.1007/s00184-006-0114-3 --- Abbreviated SS here
   if(as.sample) {
      ctype <- match.arg(ctype)
      if(is.null(para)) {
         warning("Sample Blomqvist's Beta desired but para is NULL, returning NULL")
         return(NULL)
      }
      if(length(names(para)) != 2) {
        warning("para argument must be data.frame having only two columns, returning NULL")
        return(NULL)
      }
      return(blomCOPss(cop=EMPIRcop, uu=uu, vv=vv, para=para, ctype=ctype, ...))
   } else {
      if(is.null(cop)) {
         warning("must have copula argument specified, returning NULL")
         return(NULL)
      }
      # This condition seems clear in SS, but such a conditions keeps the
      # computations out of two quadrants. Use of the two other quadrants means
      # that this SS extension to blomCOP() can be used in the matrix products of
      # blomatrixCOP(). So, let us keep this condition commented out:
      #if(any(uu > vv)) { # SS pp.4 uu<=vv |given uu > 0 | vv < 1
      #   warning("at least one in vector uu is greater than one in vv")
      #   #return(NULL)
      #}

      n  <- length(uu)
      gd <- cumprod(uu)[n] + cumprod(1-vv)[n]
      hd <- 1 / ( min(uu) + min(1-vv) - gd)

      C    <- cop(uu[1], uu[2], para=para, ...)
      Cbar <- surfuncCOP(vv[1], vv[2], cop=cop, para=para, ...)
      if(trap.nan) { # Asquith handling as explained. We know the copBasic::PSP needs
        if(is.nan(C))    C    <- 0 # this as PSP does not trap its own NaN
        if(is.nan(Cbar)) Cbar <- 0 # interred with a PSP reflected
      }
      return(hd * (C + Cbar - gd))
   }
}

"EMPIRgridderinv" <-
function(empgrid=NULL, kumaraswamy=FALSE, dergrid=NULL, ...) {

  if(is.null(empgrid)) {
    warning("The gridded empirical copula (say from EMPIRgrid) is NULL")
    return(NULL)
  }
  if(! is.list(empgrid)) {
    warning("The gridded empirical copula is expected as a list from EMPIRgrid")
    return(NULL)
  }

  if(is.null(dergrid)) {
    the.deriv <- EMPIRgridder(empgrid=empgrid,...)
  } else {
    the.deriv <- dergrid
  }

  FF <- empgrid$v
  n <- length(FF)

  # use a mid-point integration method so 1/2s are needed
  # on the tails.
  delF <- diff(FF)
  delF <- c(0.5*delF[1], delF)
  delF[n] <- 0.5*delF[n]

  the.inverse <- matrix(nrow=n, ncol=n)
  Alphas <- Betas <- vector(mode="numeric", length=n)
  Alphas[1] <- NA; Betas[1] <- NA
  for(i in 2:n) {
    x <- the.deriv[i,]

    if(length(x[! is.finite(x)]) > 0) {
      warning("found nonfinite values on row=",i," in grid inversion")
      next
    }

    # invert the CDF by linear approximation
    # we want the QDF with FF (horizontal axis values on same spacing)
    # we know that the x are given in ordered seqeuence to so avoid
    # the warning
    # In regularize.values(x, y, ties, missing(ties)) :
    # collapsing to unique 'x' values
    inv <- approx(x, y=FF, xout=FF, rule=2, ties="ordered")$y

    if(kumaraswamy) {
      beta0 <- sum(inv*delF) # first PWM (mean)
      beta1 <- sum(inv*FF*delF) # second PWM (no name)

      lmr <- lmomco::vec2lmom(c(beta0, 2*beta1 - beta0)) # L-moments
      par.of.kur <- lmomco::parkur(lmr)
      X <- lmomco::quakur(FF, par.of.kur) # Kumuraswamy distribution
      if(is.null(X)) {
        warning("failed Kumaraswamy fit at i=",i,", using approx inv")
        the.inverse[i,] <- inv
        Alphas[i] <- NA
        Betas[i]  <- NA
        next
      }
      the.inverse[i,] <- X
      Alphas[i] <- par.of.kur$para[1]
      Betas[i]  <- par.of.kur$para[2]
    } else {
       the.inverse[i,] <- inv
       Alphas[i] <- NA
       Betas[i]  <- NA
    }
  }
  attributes(the.inverse) <- list(dim=dim(empgrid$empcop),
                                  rownames=empgrid$u,
                                  colnames=empgrid$v,
                                  kumaraswamy=list(Alpha=Alphas,
                                                   Beta=Betas),
                                  message="use the rows!, wrt U")

  return(the.inverse)
}

"level.curvesCOP" <-
function(cop=NULL, para=NULL, ploton=TRUE, lines=TRUE,
         plotMW=FALSE, ramp=TRUE, delu=0.001, delt=0.10,
         getlevel=NULL, silent=TRUE, ...) {

  if(! is.null(delt)) {
    if(is.na(delt) | delt == 0) delt <- NULL # silently treat zero as NULL
  }
  if(! is.null(delt)) {
     if(delt <= 0 | delt > 0.5) {
        warning("Invalid 'delt' argument, must be (0,0.5] or NULL, setting to NULL")
        delt <- NULL
     }
  }
  if(! is.null(getlevel)) {
     if(getlevel < 0 | getlevel > 1) {
        warning("Invalid 'getlevel' argument, must be [0,1] or NULL, setting to NULL")
        getlevel <- NULL
     }
  }

  z <- list(level=getlevel, U=NULL, V=NULL)
  if(is.null(delt)) {
     Ts <- sort(getlevel)
  } else {
     tmp <- NULL # see comment below for seq() error trapping, it seems unlikely
     # that the trapping is needed here, but let us mimic the behavior.
     try(tmp <- seq(0+delt, 1-delt, by=delt), silent=silent)
     if(is.null(tmp)) try(tmp <- seq(0+delt, 1-delt, by=-delt), silent=silent)
     Ts <- sort(c(getlevel, tmp))
  }
  if(is.null(Ts)) {
     warning("The level set is NULL, check 'delt' and 'getlevel' arguments, returning NA")
     return(NA)
  }

  if(ploton) {
    plot(c(0,1), c(0,1), type="n",
         xlab="U, NONEXCEEDANCE PROBABILITY", ylab="V, NONEXCEEDANCE PROBABILITY")
  }

  for(t in Ts) {  # for each level t
    # Concerning the tmp handling, there seems to be cases in which empirical copulas
    # are being used with small sample sizes that can trigger a 'wrong sign in 'by' argument
    # error on the following sequence, if so, let us try reversing the sequence and if that
    # fails then bail out entirely
    tmp <- NULL
    try(tmp <- seq(t+delu, 1-delu, by=delu), silent=silent)
    if(is.null(tmp)) {
      warning("trying to compensate for 'by' error by reversing")
      try(tmp <- seq(t+delu, 1-delu, by=-delu), silent=silent)
    }
    u <- c(t, t+delu/5, t+delu/2, tmp, 1-delu/2, 1-delu/5, 1)
    u <- sort(u)
    v <- sapply(1:length(u), function(i) { COPinv(cop=cop, u[i], t, para=para, ...) })
    if(lines) {
      if(ramp) {
        dots <- list(...)
        if(! "lwd" %in% names(dots)) {
          dots$lwd <- (0.5+2*t)
        } else if(is.function(dots$lwd)) {
          dots$lwd <- dots$lwd(t)
        }
        dots$x <- u; dots$y <- v
        do.call("lines", dots)
      } else {
        lines(u,v, ...)
      }
    }
    if(! is.null(getlevel) &
       isTRUE(all.equal(getlevel,t)) )
               z <- list(level=t, U=u, V=v)
  }
  if(plotMW) {
    abline(0,  1, lty=2, cex=0.5)
    abline(1, -1, lty=2, cex=0.5)
  }
  if(! is.null(getlevel)) return(z)
}

library(copBasic)
library(lmomco)
Lru <- function(u, r=2) { # eq. 15b Jones (2004) and Asquith (2011, 6.10 is wrong with 1/(1-r) and needs 1/(r-1))
  sapply(u, function(u) { sum(sapply(0:(r-2), function(j) {
                  (-1)^j * exp(lchoose(r-1,j)+lchoose(r-1,j+1))*u^(r-2-j)*(1-u)^j })) })/(r-1)
}

para <- vec2par(c(-10,0,10), type="tri")
integrate(function(x, para=para) { u <- cdftri(x, para=para)
       return(u*(1-u)*Lru(u, r=2)) }, -100, +100, para=para) # 7/3
integrate(function(x, para=para) { u <- cdftri(x, para=para)
       return(u*(1-u)*Lru(u, r=2)) }, -Inf, +Inf, para=para) # 7/3
lmomtri(para)$lambdas[2] # 7/3
integrate(function(x, para=para) { u <- cdftri(x, para=para)
       return(u*(1-u)*Lru(u, r=4)) }, -100, +100, para=para) # 0.2103149
integrate(function(x, para=para) { u <- cdftri(x, para=para)
       return(u*(1-u)*Lru(u, r=4)) }, -Inf, +Inf, para=para) # 0.2103149
lmomtri(para)$lambdas[4] # 0.2103149


para <- list(cop=breveCOP, para=list(cop=CLcop, para=10, breve=0.2, reflect=2))
uv <- simCOP(500, cop=COP, para=para, graphics=FALSE, set.seed=1)

ustar <- 0.96; set.seed(1)
v_special <- replicate(10000, simCOPv(ustar, cop=COP, para=para))
lmr <- lmomco::lmoms(v_special)

nsim <- 1e+03
u <- seq(0.001,1, by=0.0005)
medvatu <- med.regressCOP(u, cop=COP, para=para)[,2]
lam1atu <- EvuCOP(u, cop=COP, para=para)
lam2atu <- sapply(u, function(u) { E <- NULL
                               try(E <- integrate(function(v, para=para) {
              t <- derCOP(cop=COP, u, v, para=para); return(t*(1-t)*Lru(t, r=2)) }, 0, +1,
                                         para=para, subdivisions=200)$value, silent=TRUE)
                        if(is.null(E)) {
                                        t <- derCOP(cop=COP, u, runif(nsim), para=para)
                                   E <- mean(t*(1-t)*Lru(t, r=2))
                             names(E) <- "sim"
                        }
                            return(E) })
lam3atu <- sapply(u, function(u) { E <- NULL
                               try(E <- integrate(function(v, para=para) {
              t <- derCOP(cop=COP, u, v, para=para); return(t*(1-t)*Lru(t, r=3)) }, 0, +1,
                                         para=para, subdivisions=200)$value, silent=TRUE)
                        if(is.null(E)) {
                                        t <- derCOP(cop=COP, u, runif(nsim), para=para)
                                   E <- mean(t*(1-t)*Lru(t, r=3))
                             names(E) <- "sim"
                        }
                            return(E) })
lam4atu <- sapply(u, function(u) { E <- NULL
                               try(E <- integrate(function(v, para=para) {
               t <- derCOP(cop=COP, u, v, para=para); return(t*(1-t)*Lru(t, r=4)) }, 0, +1,
                                          para=para, subdivisions=200)$value, silent=TRUE)
                        if(is.null(E)) {
                                        t <- derCOP(cop=COP, u, runif(nsim), para=para)
                                   E <- mean(t*(1-t)*Lru(t, r=4))
                             names(E) <- "sim"
                        }
                            return(E) })

#pdf("LMRvuCOP.pdf", height=7, width=7, useDingbats=FALSE)
par(xpd=NA, las=1, lend=1, col.axis="black", cex.axis=0.9,
    mgp=c(2,0.6,0), mai=c(0.65,0.65,0.40,0.65))
plot(uv[,1], uv[,2], type="n", lwd=0.8, col="grey50", bty="n", xlim=c(0,1), ylim=c(0,1),
      xlab="", ylab="", xaxs="i", yaxs="i", xaxt="n", yaxt="n")
lines(rep(ustar, 2), par()$usr[3:4], lty=4, lwd=1.2, col="seagreen4")
par(mgp=c(2,0.3,0), las=0, col.axis="seagreen4", cex.axis=0.8)
axis(1, at=ustar, label=latex2exp::TeX(r'($u^\star$)'), lwd=0, lwd.ticks=1.2, tcl=-0.25, col.ticks="seagreen4")
axis(3, at=ustar, label=latex2exp::TeX(r'($u^\star$)'), lwd=0, lwd.ticks=1.2, tcl=-0.25, col.ticks="seagreen4")
axis(1, at=ustar, label=FALSE,   lwd=0, lwd.ticks=1.2, tcl=+0.25, col.ticks="seagreen4")
axis(3, at=ustar, label=FALSE,   lwd=0, lwd.ticks=1.2, tcl=+0.25, col.ticks="seagreen4")
par(mgp=c(2,0.6,0), las=1, col.axis="black", cex.axis=0.9)
points(uv[,1], uv[,2], lwd=0.9, pch=21, cex=0.8, col="wheat4", bg="wheat1")
axis(1, at=axTicks(1), label=TRUE,  lwd=1, lwd.ticks=1, col.ticks="black")
axis(3, at=axTicks(1), label=FALSE, lwd=1, lwd.ticks=1, col.ticks="black")
tix <- seq(0.1, 0.9, by=0.2)
axis(1, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="black", tcl=-0.5)
axis(2, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="black", tcl=-0.5)
axis(3, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="black", tcl=-0.5)
tix <- seq(0.05, 0.95, by=0.05); tix <- tix[grep("5$", as.character(tix))]
axis(1, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="black", tcl=-0.2)
axis(2, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="black", tcl=-0.2)
axis(3, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="black", tcl=-0.2)
mtext("U, nonexceedance probability", side=1, line=1.6)
par(mgp=c(2,0.8,0))
axis(2, at=axTicks(2), label=TRUE,  lwd=1, lwd.ticks=1, col.ticks="black")
par(las=0)
mtext("V, nonexceedance probability", side=2, line=2.1)
par(las=1)
lines(u, lam1atu, lwd=2,  col="black"  ) # mean probability
lines(u, medvatu, lwd=2,  col="red1"   ) # median probability
lines(u, lam2atu, lwd=2,  col="deepskyblue3") # conditional L-scale
points(ustar, mean(  v_special), cex=1.7, pch=1,  col="black"  )
points(ustar, median(v_special), cex=1.7, pch=1,  col="red1"   )
points(ustar, lmr$lambdas[1],    cex=1.3, pch=16, col="black"  )
points(ustar, lmr$lambdas[2],    cex=1.3, pch=16, col="deepskyblue3")
par(new=TRUE)
plot(u, lam3atu/lam2atu, type="n", bty="n",xlim=c(0,1), ylim=c(-1, 1),
         xlab="", ylab="", xaxs="i", yaxs="i", xaxt="n", yaxt="n")
par(col.axis="orchid3", las=0, lend=1)
mtext("Lskew and Lkurtosis, dimensionless", 4, line=2.15, col="orchid3")
par(las=1)
axis(4, at=axTicks(2), label=TRUE, lwd=2, lwd.ticks=1, col="orchid3", col.ticks="orchid3")
tix <- seq(-0.9, 0.9, by=0.1); tix <- tix[grep("[05]$", as.character(tix), invert=TRUE)]
axis(4, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="orchid3", tcl=-0.5)

lines(u, lam3atu/lam2atu, col="orchid3", lwd=2) # conditional L-skew
lines(u, lam4atu/lam2atu, col="orchid3", lty=2, lwd=2) # conditional L-kurtosis
points(ustar, lmr$ratios[ 3],    cex=1.2,   pch=24, col="orchid3", bg="orchid1")
points(ustar, lmr$ratios[ 4],    cex=1.2,   pch=25, col="orchid3", bg="orchid1")

txt <- c(paste0("Conditional (Cond.) mean at ustar = ", ustar, ", EvuCOP(...)"),
         "Cond. median at ustar, med.regressCOP(...)",
         "Cond. Lscale (variation) at ustar, LMRvuCOP(...)",
         "Cond. Lskew at ustar, LMRvuCOP(...)",
         "Cond. Lkurtosis at ustar, LMRvuCOP(...)",
         "Demonstration of n=500 sim. values for the copula",
         "Cond. simulated (sim.) (n=10,000) sample mean, mean(...)",
         "Cond. sim. sample median, median(...)",
         "Cond. sim. sample mean, lmoms(...)$lambdas[1]",
         "Cond. sim. sample Lscale, lmoms(...)$lambdas[2]",
         "Cond. sim. sample Lskew, lmoms(...)$ratios[3]",
         "Cond. sim. sample Lkurtosis, lmoms(...)$ratios[4]")
legend("topleft", txt, bty="n", cex=0.6, seg.len=4, y.intersp=1.3,
        lwd=c(2,2,2,2,2, NA, rep(NA, 6)), lty=c(1,1,1,1,2,0,rep(0, 6)),
        pch=c(rep(NA, 5), 21, 1, 1, 16, 16, 24, 25),
        pt.lwd=c(rep(NA,5), 0.9, rep(1,6)),
        pt.cex=c(rep(NA,5), 0.8, 1.7, 1.7, 1.3, 1.3, 1.2, 1.2),
        pt.bg=c(rep(NA,5), "wheat1", rep(NA,4), "orchid1", "orchid1"),
        col=c("black", "red1", "deepskyblue3", "orchid3", "orchid3",
              "wheat4", "black", "red1", "black", "deepskyblue3", "orchid3", "orchid3"))
par(xpd=NA, las=1, lend=1, col.axis="black", cex.axis=0.9,
    mgp=c(2,0.6,0), mai=c(0.65,0.65,0.40,0.65))
#dev.off()



para <- list(cop=breveCOP, para=list(cop=CLcop, para=c(10), breve=0.2, reflect=2))
uv <- simCOP(500, cop=COP, para=para, graphics=FALSE, set.seed=1)

ustar <- 0.96; set.seed(1)
v_special <- replicate(10000, simCOPv(ustar, cop=COP, para=para))
lmr <- lmomco::lmoms(v_special)


u <- v <- seq(0,1, by=0.001)
LMRvu <- LMRvuCOP(u, cop=COP, para=para)
LMRuv <- LMRuvCOP(v, cop=COP, para=para)

#uv <- uv[,2:1]
#LMRvu <- LMRuv
#pdf("LMRvuCOP.pdf", height=7, width=7, useDingbats=FALSE)
par(xpd=NA, las=1, lend=1, col.axis="black", cex.axis=0.9,
    mgp=c(2,0.6,0), mai=c(0.65,0.65,0.40,0.65))
plot( uv[,1], uv[,2], type="n", lwd=0.8, col="grey50", bty="n", xlim=c(0,1), ylim=c(0,1),
      xlab="", ylab="", xaxs="i", yaxs="i", xaxt="n", yaxt="n")
lines(rep(ustar, 2), par()$usr[3:4], lty=4, lwd=1.2, col="seagreen4")
par(mgp=c(2,0.3,0), las=0, col.axis="seagreen4", cex.axis=0.8)
axis(1, at=ustar, label=latex2exp::TeX(r'($u^\star$)'), lwd=0, lwd.ticks=1.2, tcl=-0.25, col.ticks="seagreen4")
axis(3, at=ustar, label=latex2exp::TeX(r'($u^\star$)'), lwd=0, lwd.ticks=1.2, tcl=-0.25, col.ticks="seagreen4")
axis(1, at=ustar, label=FALSE,   lwd=0, lwd.ticks=1.2, tcl=+0.25, col.ticks="seagreen4")
axis(3, at=ustar, label=FALSE,   lwd=0, lwd.ticks=1.2, tcl=+0.25, col.ticks="seagreen4")
par(mgp=c(2,0.6,0), las=1, col.axis="black", cex.axis=0.9)
points(uv[,1], uv[,2], lwd=0.9, pch=21, cex=0.8, col="wheat4", bg="wheat1")
axis(1, at=axTicks(1), label=TRUE,  lwd=1, lwd.ticks=1, col.ticks="black")
axis(3, at=axTicks(1), label=FALSE, lwd=1, lwd.ticks=1, col.ticks="black")
tix <- seq(0.1, 0.9, by=0.2)
axis(1, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="black", tcl=-0.5)
axis(2, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="black", tcl=-0.5)
axis(3, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="black", tcl=-0.5)
tix <- seq(0.05, 0.95, by=0.05); tix <- tix[grep("5$", as.character(tix))]
axis(1, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="black", tcl=-0.2)
axis(2, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="black", tcl=-0.2)
axis(3, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="black", tcl=-0.2)
mtext("U, nonexceedance probability", side=1, line=1.6)
par(mgp=c(2,0.8,0))
axis(2, at=axTicks(2), label=TRUE,  lwd=1, lwd.ticks=1, col.ticks="black")
par(las=0)
mtext("V, nonexceedance probability", side=2, line=2.1)
par(las=1)
lines(u, LMRvu$L1,     lwd=2, col="black"  ) # mean probability
lines(u, LMRvu$median, lwd=2, col="red1"   ) # median probability
lines(u, LMRvu$L2,     lwd=2, col="deepskyblue3") # conditional L-scale
points(ustar,   mean(v_special), cex=1.7, pch=1,  col="black"  )
points(ustar, median(v_special), cex=1.7, pch=1,  col="red1"   )
points(ustar, lmr$lambdas[1],    cex=1.3, pch=16, col="black"  )
points(ustar, lmr$lambdas[2],    cex=1.3, pch=16, col="deepskyblue3")
par(new=TRUE)
plot(u, LMRvu$T3, type="n", bty="n",xlim=c(0,1), ylim=c(-1, 1),
         xlab="", ylab="", xaxs="i", yaxs="i", xaxt="n", yaxt="n")
par(col.axis="orchid3", las=0, lend=1)
mtext("Lskew and Lkurtosis, dimensionless", 4, line=2.15, col="orchid3")
par(las=1)
axis(4, at=axTicks(2), label=TRUE, lwd=2, lwd.ticks=1, col="orchid3", col.ticks="orchid3")
tix <- seq(-0.9, 0.9, by=0.1); tix <- tix[grep("[05]$", as.character(tix), invert=TRUE)]
axis(4, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="orchid3", tcl=-0.5)

lines(u, LMRvu$T3, col="orchid3", lwd=2) # conditional L-skew
lines(u, LMRvu$T4, col="orchid3", lty=2, lwd=2) # conditional L-kurtosis
points(ustar, lmr$ratios[ 3],    cex=1.2,   pch=24, col="orchid3", bg="orchid1")
points(ustar, lmr$ratios[ 4],    cex=1.2,   pch=25, col="orchid3", bg="orchid1")

txt <- c(paste0("Conditional (Cond.) mean at ustar = ", ustar, ", EvuCOP(...)"),
         "Cond. median at ustar, med.regressCOP(...)",
         "Cond. Lscale (variation) at ustar, LMRvuCOP(...)",
         "Cond. Lskew at ustar, LMRvuCOP(...)",
         "Cond. Lkurtosis at ustar, LMRvuCOP(...)",
         "Demonstration of n=500 sim. values for the copula",
         "Cond. simulated (sim.) (n=10,000) sample mean, mean(...)",
         "Cond. sim. sample median, median(...)",
         "Cond. sim. sample mean, lmoms(...)$lambdas[1]",
         "Cond. sim. sample Lscale, lmoms(...)$lambdas[2]",
         "Cond. sim. sample Lskew, lmoms(...)$ratios[3]",
         "Cond. sim. sample Lkurtosis, lmoms(...)$ratios[4]")
legend("topleft", txt, bty="n", cex=0.6, seg.len=4, y.intersp=1.3,
        lwd=c(2,2,2,2,2, NA, rep(NA, 6)), lty=c(1,1,1,1,2,0,rep(0, 6)),
        pch=c(rep(NA, 5), 21, 1, 1, 16, 16, 24, 25),
        pt.lwd=c(rep(NA,5), 0.9, rep(1,6)),
        pt.cex=c(rep(NA,5), 0.8, 1.7, 1.7, 1.3, 1.3, 1.2, 1.2),
        pt.bg=c(rep(NA,5), "wheat1", rep(NA,4), "orchid1", "orchid1"),
        col=c("black", "red1", "deepskyblue3", "orchid3", "orchid3",
              "wheat4", "black", "red1", "black", "deepskyblue3", "orchid3", "orchid3"))
par(xpd=NA, las=1, lend=1, col.axis="black", cex.axis=0.9,
    mgp=c(2,0.6,0), mai=c(0.65,0.65,0.40,0.65))
#dev.off()

"derCOP2" <-
function(cop=NULL, u, v,
         delv=.Machine$double.eps^0.50,
         derdir=c("left", "right", "center"), ...) {

    derdir <- match.arg(derdir)
    if(length(v) == 1) {
       if(v - delv < 0) derdir <- "left"
       if(v + delv > 1) derdir <- "right"
       #str(cop)
       if(derdir == "left") {
         return((cop(u,v+delv,...) - cop(u,v, ...))/delv)
       } else if(derdir == "right") {
         return((cop(u,v,...)      - cop(u,v-delv, ...))/delv)
       } else {
         return((cop(u,v+delv,...) - cop(u,v-delv, ...))/(2*delv))
       }
    } else {
       if(length(v) != length(u)) {
          #warning("length of v and u are not equal, so using only first element of u")
          u <- rep(u[1], length(v))
       }
       der <- vector(mode="numeric", length(u))
       for(i in 1:length(v)) {
          tmpdir <- derdir; au <- u[i]; av <- v[i]
          if(av - delv < 0) tmpdir <- "left"
          if(av + delv > 1) tmpdir <- "right"
          if(tmpdir == "left") {
             der[i] <- (cop(au,av+delv,...) - cop(au,av, ...))/delv
          } else if(tmpdir == "right") {
             der[i] <- (cop(au,av,...)      - cop(au,av-delv, ...))/delv
          } else {
             der[i] <- (cop(au,av+delv,...) - cop(au,av-delv, ...))/(2*delv)
          }
       }
       return(der)
    }
}

"wolfCOPsamc" <-
function(u=NULL, v=NULL, cop=EMPIRcop, para=NULL, para_has_paras=FALSE,
         tol=0.005, minit=10, maxit=100, subdivisions=200, large_n=5E3,
         includelmoms=FALSE, trace=FALSE, verbose=FALSE, ...) {

  if(is.null(para)) {
    if(is.null(v)) {
      para <- as.data.frame(u)
    } else {
      para <- data.frame(u=u, v=v)
    }
  }

  if(minit <= 2) minit <- 2
  if(maxit < minit) {
    warning("maxit < minit, swapping them")
    a <- maxit; maxit <- minit; minit <- a; rm(a)
  }

  if(para_has_paras) {
    n <- large_n
  } else {
    n <- nrow(para)
  }

  k <- n^2 # This will fail if nrow(para) makes no sense
  suppressWarnings( if(length(k) == 0) n <- large_n )
  # Error in if (k < subdivisions^2) k <- subdivisions^2 :
  # argument is of length zero
  # So, the conditional failure permits us to guess whether para_has_paras should have been TRUE
  # and the user forgot. The user is using Monte Carlo for Wolf of a copula family and not the sample.

  k <- n^2 # try again, k is going to become the total length of the random number generation where
  # we want only one call to randtoolbox::halton() and then consume those values in the Monte Carlo.
  if(k < subdivisions^2) k <- subdivisions^2 # Override by some testing heuristics.
  if(k > 1E5) k <- 1E5                       # Override by some testing heuristics, do not get too big.
  RT <- randtoolbox::halton(k, dim=2, ...)   # Finally, the matrix for the Monte Carlo, and if this
  # size does not "appear" later as large enough, then runif() calls are made on the fly.

  if(n > large_n) ix <- seq_len(n) # If the sample size is "large", we are going to subset the
  # matrix heading into the empirical copula, we will be drawing randomly these indices for each
  # iteration.

  dtol <- abs(floor(log10(tol)))
  if(dtol < 2) dtol <- 2 # decimal precision to report along the verbose=TRUE messaging system

  MCwolves <- NULL; MCwolfcub <- -9 # Initialization and the -9 is a poison pill for the percent
  # change on the first iteration, which we will hide from the user.
  m <- subdivisions # shorthand
  i <- 0 # initialization
  while(1) { # infinite loop!
    i <- i + 1 # iteration count
    a <- (1+(i-1)*m); b <- m+((i-1)*m) # beginning and ending indices of RT for each Monte Carlo
    #print(c(a, b))
    #if(a > nrow(RT)) break
    if(a > nrow(RT) | b > nrow(RT) | (b - a)+1 < m) { # Silently enter into random number generation
      # print(c("resampling", a, b, m))               # if we have run out of precomputed MC values
      rt <- matrix(runif(m*2), ncol=2)                # in the RT. Defaults settings should be good
    } else {                                          # enough that this seldom occurs in practice.
      #print(c(a,b, nrow(RT)))
      rt <- RT[a:b,] # subset the RT to those indices of the current iteration
    }
    if(n > large_n) { # critical that this is not >= so that we can intercept para_has_paras
      rx <- sample(ix, large_n) # subsampling the indices of the given sample before passing it to copula
      mcwolf <- 12 * sum(sapply(rt[,2], function(v) { # Really here, only the cop=EMPIRcop is viable
               sum(abs(cop(rt[,1],  v, para=para[rx,], ...) - rt[,1] * v )) }) ) / (m^2 - 1)
    } else {
      mcwolf <- 12 * sum(sapply(rt[,2], function(v) { # Here, we can sneak in cop=Copula Family
               sum(abs(cop(rt[,1],  v, para=para,      ...) - rt[,1] * v )) }) ) / (m^2 - 1)
    }
    MCwolves <-    c(MCwolves, mcwolf) # A growing vector of the Schweizer-Wolff sigmas
    MCwolf   <- mean(MCwolves)         # Compute the grand mean of the growing sample size of sigmas
    pctchg   <- abs( 100 * (MCwolf - MCwolfcub) / MCwolfcub ) # Absolute percent change in mean
    # between iterations.

    if(i >= 2) { # because of initialization, only start reporting after the 3rd iteration
      txt <- paste0(sprintf("%2.2i", i), "(", sprintf(paste0("%0.", dtol, "f"),
                                               round(pctchg, digits=dtol)), "%)")
      if(verbose) message(paste0(txt, ":"), appendLF=
                ifelse(length(grep("0$", as.character(i))) != 0, TRUE, FALSE))
    } else {
      txt <- paste0(sprintf("%2.2i", i), "(-.---%)") # hide the opening results, we are always doing 2+
      if(verbose) message(paste0(txt, ":"), appendLF=FALSE)
    }

    if(trace) {
      txt <- paste0(sprintf(   "wolf=%0.3f : ", round(mcwolf, digits=3)), txt,
                    sprintf(" : WOLF=%0.3f",    round(MCwolf, digits=3)))
      if(n > large_n) { # critical that this is not >= so that we can intercept para_has_paras
        plot(para[rx,], pch=1, col="grey80",      lwd=0.8, cex=0.7, las=1,
             xlab="U, NONEXCEEDANCE PROBABILITY", ylab="V, NONEXCEEDANCE PROBABILITY")
      } else {
        plot(para,      pch=1, col="grey80",      lwd=0.8, cex=0.7, las=1,
             xlab="U, NONEXCEEDANCE PROBABILITY", ylab="V, NONEXCEEDANCE PROBABILITY")
      }
      points(rt,       pch=21, col="darkorange4", lwd=0.8, cex=0.6, bg="darkorange1")
      mtext(txt)
    }

    if(i > minit) { if(pctchg < tol | i >= maxit) break } # Only consider breaking after second iteration
    MCwolfcub <- MCwolf
  }
  if(verbose) message("done")

  nmom     <- pmin(5, length(   MCwolves))
  lmr      <- lmomco::lmoms(    MCwolves,               nmom=nmom, no.stop=TRUE)

  #if(abs(MCwolf-lmr$lambdas[1]) > .Machine$double.eps^0.5) {
  #  print(abs(MCwolf-lmr$lambdas[1])); stop("this condition must never happen")
  #}
  yy <- c(sort(c(range(MCwolves), lmr$lambdas[1])))
  names(yy) <- c("min", "mean", "max")
  if(includelmoms) {
    yy <- c(yy, (lmr$lambdas[2]*sqrt(pi))^2, lmr$lambdas[2])
    if(       nmom <  3) {
      yy <- c(yy, NA, NA, NA)
    } else if(nmom == 3) {
      yy <- c(yy, lmr$ratios[3], NA, NA)
    } else if(nmom == 4) {
      yy <- c(yy, lmr$ratios[3:4], NA)
    } else if(nmom >= 5) {
      yy <- c(yy, lmr$ratios[3:5])
    }
    names(yy) <- c("min", "mean", "max", "var", "lscale", "tau3", "tau4", "tau5")
  }

  lmr <- lmomco::lmoms(log(MCwolves/(1-MCwolves)), nmom=nmom, no.stop=TRUE)
  zz <- c(sort(c(range(log(MCwolves/(1-MCwolves))), lmr$lambdas[1])))
  # print(zz)
  names(zz) <- c("min", "mean", "max")
  if(includelmoms) {
    zz <- c(zz, (lmr$lambdas[2]*sqrt(pi))^2, lmr$lambdas[2])
    if(       nmom <  3) {
      zz <- c(zz, NA, NA, NA)
    } else if(nmom == 3) {
      zz <- c(zz, lmr$ratios[3], NA, NA)
    } else if(nmom == 4) {
      zz <- c(zz, lmr$ratios[3:4], NA)
    } else if(nmom >= 5) {
      zz <- c(zz, lmr$ratios[3:5])
    }
    names(zz) <- c("logitmin",    "logitmean", "logitmax",  "logitvar",
                   "logitlscale", "logittau3", "logittau4", "logittau5")
  }
  zz <- list(wolves=MCwolves, estimates=yy, logitestimates=zz, its=i, lastpctchg=pctchg)
  return(zz)
}

#
#uv <- as.data.frame(matrix(runif(20001*2), ncol=2))
#wl <- wolfCOPsamc(para=uv, verbose=TRUE, maxit=500, tol=0.00005, large_n=1E4)

#uv <- simCOP(10000, cop=PSP, graphics=FALSE)
#wc <- wolfCOPsamc(uv, tol=0.00005, verbose=TRUE, usetime=TRUE)
#plot(wc$wolves)
#lines(par()$usr[1:2], rep(wolfCOP(cop=PSP), 2))
#x   <- seq(0,1, by=0.001)
#lmr <- lmomco::vec2lmom(c(wc$logitestimates[c(2,5:8)]))
#gld <- lmomco::parst3(lmr)
#plot(x, dlmomco(log(x/(1-x)), gld), type="l")

"joint.curvesCOP" <-
function(cop=NULL, para=NULL, type=c("and", "or"),
         probs=c(0.5, 0.8, 0.90, 0.96, 0.98, 0.99, 0.995, 0.998),
         zero2small=TRUE, small=1E-6, divisor=100, delu=0.001, ...) {

  type <- match.arg(type)

  "orfunc" <- function(v, u=NULL, LHS=NULL, cop=NULL, para=NULL, ...) {
      RHS <- duCOP(u, v, cop = cop, para = para, ...)
      #message("u=",u,",  v=",v, ",  LHS=",LHS, ",  RHS=",RHS, ", DELTA=", LHS-RHS)
      return(LHS - RHS)
  }

  zz <- new.env()
  for(t in probs) {
    if(type == "and") {
       # Concerning the tmp handling, there seems to be cases in which empirical copulas
       # are being used with small sample sizes that can trigger a 'wrong sign in 'by' argument
       # error on the following sequence, if so, let us try reversing the sequence and if that
       # fails then bail out enterly.
       delta <- diff(range(c(t+delu, 1-delu)))/divisor
       tmp <- NULL
       try(tmp <- seq(t+delu, 1-delu, by=delta), silent=FALSE)
       if(is.null(tmp)) {
          warning("trying to compensate for 'by' error by reversing")
          try(tmp <- seq(t+delu, 1-delu, by=-delta), silent=FALSE)
       }
       u <- c(t, t+delu/100, t+delu/10, t+delu/5, t+delu/2,
               tmp,
              1-delu/2, 1-delu/5, 1-delu/10, 1-delu/100, 1)
       u  <- sort(unique(u))
       v  <- sapply(1:length(u), function(i) { COPinv(cop=cop, u[i], t, para=para) })
       if(zero2small) v[v == 0] <-     small
       if(zero2small) v[v == 1] <- 1 - small
       uv <- data.frame(U=u, V=v)
       assign(as.character(t), uv, envir=zz)
    } else if(type == "or") {
       delta <- diff(range(c(0, t)))/divisor
       u <- c(0, delu/100, delu/10, delu/5, delu/2,
             seq(delu,t, by=delta),
             t-delu/2, t-delu/5, t-delu/10, t-delu/100, t)
       u <- sort(unique(u))
       v <- vector(mode="numeric", length(u))
       for(i in 1:length(u)) {
          lo <- duCOP(0,0, cop=cop, para=para, ...)
          if(is.na(lo) | ! is.finite(lo)) lo <- .Machine$double.eps
          my.rt <- NULL
          try(my.rt <- uniroot(orfunc, interval=c(lo,t), u=u[i], LHS=t,
                               cop=cop, para=para, ...), silent=TRUE)
          if(is.null(my.rt)) {
             v[i] <- NA
          } else if(length(my.rt$root) != 0) {
             v[i] <- my.rt$root
          } else {
             v[i] <- NA
          }
       }
       if(zero2small) v[v == 0] <-     small
       if(zero2small) v[v == 1] <- 1 - small
       uv <- data.frame(U=u, V=v)
       assign(as.character(t), uv, envir=zz)
    } else {
       stop("should not be here in logic")
    }
  }
  zzz <- as.list(zz); rm(zz)
  return(zzz)
}

"gEVcop" <- function(u,v, para=NULL, ...) {
   if(is.null(para)) {
      warning("no parameters, NULL")
   }
   if(length(para) == 1) {
      if(para[1] < 0 | para[1] > +1) {
         warning("Parameter not in 0 <= rho <= +1")
      }
   } else {
      warning("Parameter rho must be given")
      return(NULL)
   }
   if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
      warning("length u = ", length(u), " and length v = ", length(v))
      warning("longer object length is not a multiple of shorter object length, ",
              "no recycling")
      return(NA)
   }
   if(length(u) == 1) {
      u <- rep(u, length(v))
   } else if(length(v) == 1) {
      v <- rep(v, length(u))
   }
   "A.gEV" <- function(x,y, rho) {
     a <- x/(x+y)
     A <- (x+y-2*rho*sqrt(x*y))/(1-rho^2)
     r1 <- rho^2/(1+rho^2); r2 <- 1/(1+rho^2)
     t1 <- (0 <= a & a <= r1)
     t2 <- (r1 < a & a <= r2)
     t3 <- (r2 < a & a <= 1 )
     A[a < 0] <- NA
     A[t1] <- y[t1]
     A[t3] <- x[t3]
     A[a > 1] <- NA
     return(A)
   }
   return(exp(-A.gEV(-log(u), -log(v), para[1])))
}
#https://doi.org/10.1016/j.advwatres.2006.08.001

"TRIcop" <-
function(u, v, para=NULL, rhotau=NULL, ...) {
  if(is.null(para)) {
    if(is.null(rhotau)) {
      warning("returning NULL because para and rhotau are NULL")
      return(NULL)
    }
    para <- (rhotau + 1) / 2
    names(para) <- "theta"
    names(rhotau)  <- "Spearman Rho or Kendall Tau"
    return(list(para=para, rhotau=rhotau))
  }
  p <- para[1]

  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
    warning("length u = ", length(u), " and length v = ", length(v))
    warning("longer object length is not a multiple of shorter object length, ",
    "no recycling")
    return(NA)
  }
  if(       length(u) == 1) {
    u <- rep(u, length(v))
  } else if(length(v) == 1) {
    v <- rep(v, length(u))
  }

  r <- 1 - (1-p)*v
  cop <- u
  wnt <- 0 <= p*v & p*v <  u & u <  r
  cop[wnt] <- p*v[wnt]
  wnt <- p <= r   & r   <= u & u <= 1
  cop[wnt] <-   u[wnt] + v[wnt] - 1
  return(cop)
}

"isCOP.PQD" <- function(cop=NULL, para=NULL, uv=NULL, empirical=FALSE, verbose=TRUE, ...) {
      tau <- rho <- gin <- blm <- NULL
      blm <- blomCOP(cop=cop, para=para, ...)
      if(verbose) message("  Blomqvist's Beta = ",round(blm, digits=4))
      if(empirical) { # sample versions
         U <- para[,1]; V <- para[,2]; n <- length(U)
         gin <- ( sum(abs(rank(U)+rank(V)-n-1)) -
                  sum(abs(rank(U)-rank(V)))       ) / as.integer(n^2/2)
         if(verbose) message("      Gini's Gamma = ",round(gin, digits=4))

         rho <- cor(U, V, method="spearman")
         if(verbose) message("    Spearman's Rho = ",round(rho, digits=4))

         tau <- cor(U, V, method="kendall")
         if(verbose) message("     Kendall's Tau = ",round(tau, digits=4))
      } else {
         try(gin <- giniCOP(cop=cop, para=para, ...), silent=TRUE)
         if(is.null(gin)) gin <- giniCOP(cop=cop, para=para, by.concordance=TRUE, brute=TRUE, ...)
         if(verbose) message("      Gini's Gamma = ",round(gin, digits=4))

         try(rho <- rhoCOP(cop=cop, para=para, ...), silent=TRUE)
         if(is.null(rho)) rho <- rhoCOP(cop=cop, para=para, brute=TRUE, ...)
         if(verbose) message("    Spearman's Rho = ",round(rho, digits=4))

         try(tau <- tauCOP(cop=cop, para=para, ...), silent=TRUE)
         if(is.null(tau)) tau <- tauCOP(cop=cop, para=para, brute=TRUE, ...)
         if(verbose) message("     Kendall's Tau = ",round(tau, digits=4))
     }

    global.PQD <- ifelse(3*tau >= rho & rho >= 0 & gin >= 0 & blm >= 0, TRUE, FALSE)
    if(is.null(uv)) {
       return(global.PQD)
    } else {
       U <- uv[,1]; V <- uv[,2]
       uvs  <- U*V
       cops <- cop(U, V, para=para, ...)
       deltaCops <- (cops - uvs)
       local.PQD <- deltaCops >= 0
       local.NQD <- deltaCops <= 0
       zeroDelta.Copula.to.Pi <- local.PQD == local.NQD
       zz <- list(global.PQD=global.PQD, Delta.Copula.to.Pi=deltaCops,
                  zeroDelta.Copula.to.Pi=zeroDelta.Copula.to.Pi,
                  local.PQD=local.PQD, local.NQD=local.NQD,
                  beta=blm, gamma=gin, rho=rho, tau=tau)
       return(zz)
    }
}

"med.regressCOP" <-
function(u=seq(0.01,0.99, by=0.01), cop=NULL, para=NULL, level=NA, ...) {
   if(is.null(cop)) {
      warning("must have copula argument specified, returning NULL")
      return(NULL)
   }
   UV <- qua.regressCOP(f=0.5, u=u, cop=cop, para=para, ...)
   if(is.na(level)) return(UV)
   if(length(level) > 1) {
      warning("only the first value of 'level' is used")
      level <- level[1]
   }
   lo <- ifelse(level > 0.5, (1-level)/2, level/2)
   tmp <- UV$V; UV$V <- NULL # copy and then erase
   UV$Vlwr <- qua.regressCOP(f=lo, u=u, cop=cop, para=para, ...)$V
   UV$V    <- tmp # this gets it in the middle
   UV$Vupr <- qua.regressCOP(f=1-lo, u=u, cop=cop, para=para, ...)$V
   return(UV)
}

"aicCOP" <-
function(u, v=NULL, cop=NULL, para=NULL, m=NA, ...) {
  if(is.null(v)) {
    v <- u[,2]
    u <- u[,1]
  }
  if(is.null(cop)) {
    warning("must have copula argument specified, returning NULL")
    return(NULL)
  }
  theo.cop <- COP(u, v, cop=cop, para=para, ...)
  emp.cop  <- EMPIRcop(u,v, para=data.frame(u,v), ...)
  n <- length(theo.cop); k <- length(emp.cop)
  if(n != k) return(Inf)
  mse <- mean((theo.cop - emp.cop)^2)
  if(is.na(m)) m <- length(para)
  return(2*m + n*log(mse))
}

"spectralmeas" <-
function(u, v=NULL, w=NULL, f=0.90, snv=FALSE,
                            smooth=FALSE, nu=100, pdf=FALSE, ...) {
   if(is.null(v)) {
      if(length(names(u)) != 2) {
         warning("a data.frame having only two columns is required")
         return(NULL)
      }
      v <- u[,2]; u <- u[,1] # v must come before u
   }
   if(length(u) != length(v)) {
      warning("argument(s) or implied arguments u and v are unequal ",
              "in length, returning NULL")
      return(NULL)
   }
   if(is.null(w)) {
      warning("need the angles 'w'"); return(NULL)
   }
   if(pdf) smooth <- TRUE

   polar <- psepolar(u,v, f=f, ...)
   Sf <- polar$Sf; WS <- polar$table
   n <- length(WS$Shat); ix <- 1:n
   H3 <- vector(mode="numeric")
   for(i in 1:length(w)) {
     the.w <- w[i]
     In <- ix[WS$Shat >= Sf]; Nn <- length(In)
     Wn <- WS$What[In]
     Wbar <- mean(Wn); Sw <- 1/var(Wn)
     A <- (1 - (Wbar - 0.5)*Sw*(Wn - Wbar))
     p3 <- A/Nn
     if(! smooth) {
       H3[i] <- sum(p3*as.numeric(Wn <= the.w))
       next;
     }
     extra.term <- rep(1, length(Wn))
     if(pdf) {
       extra.term <- sapply(Wn, function(w) dbeta(the.w, nu*w, nu*(1-w)))
     } else if(smooth) {
       extra.term <- sapply(Wn, function(w) pbeta(the.w, nu*w, nu*(1-w)))
     }
     H3[i] <- sum(p3*extra.term)
   }
   ifelse(snv, return(qnorm(H3)), return(H3))
}

"footCOP" <-
function(cop=NULL, para=NULL, by.concordance=FALSE, as.sample=FALSE, ...) {
   if(as.sample) {
      if(is.null(para)) {
         warning("Sample Spearman's Footrule desired but para is NULL, returning NULL")
         return(NULL)
      }
      if(length(names(para)) != 2) {
        warning("para argument must be data.frame having only two columns, returning NULL")
        return(NULL)
      }
      samPSI <- 1 - 3*sum(abs(rank(as.numeric(para[,1])) -
                              rank(as.numeric(para[,2]))))   /   (length(para[,1])^2 - 1)
      return(samPSI)
   }

   if(is.null(cop)) {
      warning("must have copula argument specified, returning NULL")
      return(NULL)
   }
   if(by.concordance) {
      Q <- concordCOP(cop=cop, para=para, cop2=M, ...)
      return((3/2)*Q - (1/2))
   }
   "afunc" <- function(u) { cop(u,u, para=para, ...) }
   my.int <- NULL
   try(my.int <- integrate(afunc, 0,1, ...) )
   if(is.null(my.int)) {
      warning("error encountered on numberical integration---try by.concordance instead?")
      return()
   }
   return(6*my.int$value - 2)
}

"tEVcop" <- function(u, v, para=NULL, ...) {
   # Implementation Joe (2014, p. 189)
   if(is.null(para)) {
      warning("no parameters, NULL")
   }
   if(length(para) == 2) {
      if(para[1] < -1 | para[1] > +1) {
         warning("Parameter not in -1 <= rho <= +1")
      }
      if(para[2] < 0) {
         warning("Parameter Nu < 0")
         return(NULL)
      }
   } else {
      warning("Parameters must be a vector of rho and nu")
      return(NULL)
   }
   if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
      warning("length u = ", length(u), " and length v = ", length(v))
      warning("longer object length is not a multiple of shorter object length, ",
              "no recycling")
      return(NA)
   }
   if(length(u) == 1) {
      u <- rep(u, length(v))
   } else if(length(v) == 1) {
      v <- rep(v, length(u))
   }
   "my.BTev" <- function(w, rho, nu) {
     a <- sqrt((nu+1)/(1-rho^2))
     b <- (w/(1-w))^(1/nu) # note inversion in second term!
     return(w*pt(a*(b-rho), nu+1) + (1-w)*pt(a*(1/b-rho), nu+1))
   }
   x <- -log(u); y <- -log(v) # note minus on -log(v), Joe (2014, p.189) does not
   z <- exp(-(x+y)*my.BTev(x/(x+y), para[1], para[2]))
   z[! is.finite(z)] <- 0
   return(z)
}

# Experimental version shown below based on copula package.
#"tEVcop2" <- function(u, v, para=NULL, ...) {
#   if(is.null(para)) {
#      warning("no parameters, NULL")
#   }
#   if(length(para) == 2) {
#      if(para[1] < -1 | para[1] > +1) {
#         warning("Parameter not in -1 <= rho <= +1")
#      }
#      if(para[2] < 0) {
#         warning("Parameter Nu < 0")
#         return(NULL)
#      }
#   } else {
#      warning("Parameters must be a vector of rho and nu")
#      return(NULL)
#   }
#   if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
#      warning("length u = ", length(u), " and length v = ", length(v))
#      warning("longer object length is not a multiple of shorter object length, ",
#              "no recycling")
#      return(NA)
#   }
#   if(length(u) == 1) {
#      u <- rep(u, length(v))
#   } else if(length(v) == 1) {
#      v <- rep(v, length(u))
#   }
#
#  # https://mediatum.ub.tum.de/doc/1145695/1145695.pdf
#  # https://arxiv.org/pdf/0911.1015.pdf
#  # Derived from copula package (12/05/2018 by WHA)
#  ##' A Pickands dependence function of a bivariate tevCopula
#  ##' w points at which A is to be evaluated
#  "my.ATev" <- function(w, rho, nu) {
#     r2 <- sqrt(1 - rho^2); n2 <- sqrt(nu + 1); cw <- 1-w
#     wnu <-   (w / cw)^(1/nu)
#     x <-     (wnu - rho) / r2 * n2
#     y <- (1 / wnu - rho) / r2 * n2
#     A <- w * pt(x, nu + 1) + cw * pt(y, nu + 1)
#     ifelse(w == 0 | w == 1, 1, A)
#  }
#
#  # Derived from copula::ptevCopula (12/05/2018 by WHA)
#  p <- (r <- uu <- u * v) > 0
#  p <- p & (nna <- ! is.na(p)) # p: positive uu
#  logu    <- log(uu[p])
#  r[  p ] <- exp( logu * my.ATev(log(v[p])/logu, para[1], para[2]))
#  r[! p & nna] <- 0 # when one u is zero
#  return(r)
#}

#para <- c(0.5,6)
#print(tEVcop( 0,0.15, para=para))
#print(tEVcop2(0,0.15, para=para))

#afunc <- function(par, rho=NA, nustar=NA) {
#   par[1] <- pnorm(par[1])
#   par[2] <- exp(par[2])
#   my.rho <- rhoCOP(   tEVcop, par)
#   my.nus <- nustarCOP(tEVcop, par)
#   r <- (rho-my.rho)^2
#   s <- (nustar-my.nus)^2
#   return((r+s))
#}

#UV <- simCOP(1000, cop=GLcop, para=3)
#rho <- cor(UV$U, UV$V, method="spearman")
#nus <- nuskewCOP(para=UV, as.sample=TRUE)
#nut <- nustarCOP(para=UV, as.sample=TRUE)

#para <- optim(par=c(.5, log(5)), fn=afunc, rho=rho, nustar=nut)$par
#para[1] <- pnorm(para[1]); para[2] <- exp(para[2])
#densityCOPplot(cop=GLcop, para=3)
#densityCOPplot(cop=tEVcop, para=para, ploton=FALSE, contour.col=2)

"HRcop" <- function(u,v, para=NULL, ...) {
  if(para < 0) {
     warning("parameter must be 0 <= Theta < Infinity")
     return(NA)
  }
  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
    warning("length u = ",length(u), " and length v = ",length(v))
    warning("longer object length is not a multiple of shorter object length, no recycling")
    return(NA)
  }
  # The extra hassle of vectorization made here is to handle situations
  # in which nested integrals are used where uneven vectors can be passed
  if(length(u) == 1) {
     u <- rep(u, length(v))
  } else if(length(v) == 1) {
     v <- rep(v, length(u))
  }

  if(para < .Machine$double.eps) return(P(u,v))
  d2 <- para/2; di <- 1/para
  x <- -log(u); y <- -log(v)
  opts <- options(warn=-1)
  cop <- exp(-x*pnorm(di + d2*log(x/y)) + -y*pnorm(di + d2*log(y/x)))
  cop[! is.finite(x)] <- 0; cop[! is.finite(y)] <- 0
  cop[x == 0] <- v # because v ---> 1
  cop[y == 0] <- u # because u ---> 1
  cop[is.nan(cop)] <- 1
  options(opts)
  return(cop)
}

# --------------------------------------------------------------------------------------------------
"EMPIRgrid_fast" <-
function(uv=NULL, ctype=c("1/n", "bernstein"), para=NULL,
         gridonly=FALSE, verbose=FALSE, ...) {
  ctype <- tolower(   ctype ) # Remember that we much lower case it here and then feed that
  ctype <- match.arg( ctype ) # variable to the match.arg(). The reasoning has to do with the
  # way that R processes the argument itself, otherwise and error is issued.

  if(is.null(uv) & ! is.null(para)) uv <- para
  uv <- as.matrix(uv)
  if(ncol(uv) != 2) {
    warning("ncol(uv) != 2, function is only built for bivariate data handling")
    return(NULL)
  }

  n  <- nrow(uv) # sample size
  if(n == 1) {
    warning("sample size n=", nrow(n), " is degenerate for this function")
    return(NULL)
  }

  delprob <- 1/n # default for the 1/n
  # if(ctype == "weibull") { # Weibull plotting position formula
  #  delprob <- 1 / (n+1) # Adaption that is not part of 10.1016/j.amc.2024.128827.
  # }
  # It is not certain how to implement delta probabilities other than 1/n because as the algorithm
  # works, it will not result in recomputed Frechet-Hoeffding limits at the bottom of the columns
  # if say Weibull were used. Does one nudge the limits around? Does one offset the lower limits
  # and leave the upper ending at 1 or the other way around? Does one only include the Bernstein
  # smooth on 1/n? Answer to the later question is that Hernandez-Maldonado, Diaz-Viera, Erdely
  # (2012, doi:10.1016/j.petrol.2012.04.018) and think as do Hernandez-Maldonado, Erdely,
  # Diaz-Viera, and Rios (2024, doi:10.1016/j.amc.2024.128827).

  # We can control whether a t(empCOP) on return is needed or not based on which variable
  # for the algorithm that we treat as the primary and the secondary. These need not have
  # relation to our handling of u,v orientation elsewhere in the package. This choice here
  # can lead to confusion on u,v in the Bernstein loop. Just ignore that confusion because
  # the choice of svncn and pvcn ordering was made solely to avoid the transposition of the
  # matrix on return() to save some CPU time.
  svcn   <- 2 # 10.1016/j.amc.2024.128827 does not reuse these variables but I do so here
  pvcn   <- 1 # 10.1016/j.amc.2024.128827 does not reuse these variables but I do so here
  ID     <- cbind(uv, matrix(data=0, nrow=n, ncol=2))
  ID     <- ID[order(ID[,pvcn]),]
  ID[,4] <- seq_len(n) # sort on the primary first, so that when we sort on the secondary
  ID     <- ID[order(ID[,svcn]),] # that the ID remains in that order, which gets us a
  ID[,3] <- seq_len(n) # cleaner access to pvcoo
  # This change in ordering sequence between the variables differs, but seems required relative
  # to the steps in 10.1016/j.amc.2024.128827.
  #print(ID)

  empCOP <- matrix(data=0, nrow=n+1, ncol=n+1)
  for(j in 1:n) { # adaption of n from n+1 relative to 10.1016/j.amc.2024.128827
    empCOP[j+1, n+1] <- j/n # true Frechet-Hoeffding limits : Regardless of plotting position for the
    empCOP[n+1, j+1] <- j/n # true Frechet-Hoeffding limits : delprob (delta probability) variable.
  }
  # Note, some of the adaption of the indices herein is because authors of 10.1016/j.amc.2024.128827
  # are (were) thinking in a 0-based index language but R is 1-based.
  #for(svid in 1:(n-1)) { # adaption to n-1 from n 10.1016/j.amc.2024.128827
  if(verbose) message("Empirical copula: ")
  for(svid in 2:n) { # adaption to avoid systematic svid+1 on below index access
    if(verbose) message(n+1 - svid,"-", appendLF=FALSE) # tweak index so that sequence ends at "1"
    # V <- matrix(data=NA, nrow=n+1, ncol=n+1)     # CONSOLE VISUALIZATION OF THE COORDINATES
    pvcoo <- ID[svid-1, 4] # coordinate/rank of the primary variable
    # Warning, playing with row indexing is tricky; make sure that all samples in the ID matrix
    # get traversed except(?) the terminal because we always know(?) where the last sample will land?
    # The svid-1 ensure that we start at the top of the ID matrix.
    # V[pvcoo+1, svid] <- 1                        # CONSOLE VISUALIZATION OF THE COORDINATES
    #print(V)                                     # CONSOLE VISUALIZATION OF THE COORDINATES
    #for(j in pvcoo:n) { # j+1s are for 1-based R relative to 0-based C++ of 10.1016/j.amc.2024.128827
    for(j in (pvcoo:n)+1) { # adaption to simplify indexing to avoid j+1 in indices below
      empCOP[j, svid] <- delprob         + empCOP[j, svid-1]
    }
    #for(j in 1:(pvcoo-1)) { # j+1s are for 1-based R relative to 0-based C++ of 10.1016/j.amc.2024.128827
    for(j in 2:pvcoo) { # adaption to simplify indexing to avoid j+1 in indices below
      empCOP[j, svid] <- empCOP[j, svid] + empCOP[j, svid-1]
    }
    #print(empCOP) # CONSOLE VISUALIZATION OF THE BUILDING empCOP
    #cat("-----------------------\n")
  }
  if(verbose) message("done")
  rcnm <- c(0, seq_len(n)/n)
  rownames(empCOP) <- colnames(empCOP) <- rcnm

  if(ctype != "bernstein") { # Bernstein smooth was not required, so return.
    if(gridonly) {
      return(empCOP)        # The grid of the empirical copula.
    } else {
      zzz        <- list()
      zzz$u      <- rcnm    # Vector of the U probabilities with Frechet-Hoeffding limits
      zzz$v      <- zzz$u   # Replicate the U probabilities as the V probabilities
      zzz$empcop <- empCOP  # The grid of the empirical copula.
      zzz$deluv  <- 1/n     # The base deluv, which is the inversion of sample size.
      zzz$ctype  <- ctype
      class(zzz) <- c("empirical.copula.grid", class(zzz))
      return(zzz)
    }
  }

  berCOP <- matrix(data=NA, nrow=n+1, ncol=n+1) # Initialization of a matrix to hold the
  berCOP[    1,] <- empCOP[    1,] # Bernstein smooth with initiation of the edges to
  berCOP[    ,1] <- empCOP[    ,1] # the Frechet-Hoeffding limits of a copula through
  berCOP[(n+1),] <- empCOP[(n+1),] # inheritance from the empCOP itself.
  berCOP[,(n+1)] <- empCOP[,(n+1)] #
  # I chose to have the NA as the value sets so that I could study the build up of the triple
  # sapply depth of the smooth.

  ix <- 2:n           # Interior cells that we will be infilling with the Bernstein smooth.
  nx <- length(  ix ) # Length or rather width of the interior cells.
  sx <- seq_len( nx ) # A sequence length vector to feed the sapply() that does in infill.
  ns <- seq_len( n  ) # A sequence length of the double summations of eq.7 10.1016/j.amc.2024.128827.
  v <- ix/n # We have ability to a priori compute the u, probabilities of the columns and because of
  # the double summation, we pickup some additional efficiency.
  if(verbose) message("Bernstein smooth: ")
  for(i in ix) { # Foreach of the rows in the matrix.
    if(verbose) message(n+1 - i,"-", appendLF=FALSE) # tweak index so that sequence ends at "1"
    u <- rep(i/n, nx) # Create the a priori analog vector of u probabilities.
    berCOP[i, ix] <- sapply(sx, function(k ) { # Row infill.
                 sum(sapply(ns, function(ii) { # A is initialized so second summation sees it.
                                           A <- choose(n, ii) * u[k]^ii * (1 - u[k])^(n-ii)
                 sum(sapply(ns, function(jj) {
                   return(empCOP[ii, jj] * A *  choose(n, jj) * v[k]^jj * (1 - v[k])^(n-jj))
                }))  }))  })
  }
  if(verbose) message("done")

  rownames(berCOP) <- colnames(berCOP) <- rcnm

  if(gridonly) {
    return(berCOP)        # The grid of the empirical copula after Bernstein smooth.
  } else {
    zzz        <- list()
    zzz$u      <- rcnm    # Vector of the U probabilities with Frechet-Hoeffding limits
    zzz$v      <- zzz$u   # Replicate the U probabilities as the V probabilities
    zzz$empcop <- berCOP  # The grid of the empirical copula after Bernstein smooth.
    zzz$deluv  <- 1/n     # The base deluv, which is the inversion of sample size.
    zzz$ctype  <- ctype
    class(zzz) <- c("empirical.copula.grid", class(zzz))
    return(zzz)
  }
}

"kfuncCOPinv" <-
function(f, cop=NULL, para=NULL, subdivisions=100L,
            rel.tol=.Machine$double.eps^0.25, abs.tol=rel.tol, ...) {
   if(is.null(cop)) {
      warning("must have copula argument specified, returning NULL")
      return(NULL)
   }
   if(is.null(f)) {
      warning("must have at least on nonexceedance probability, returning NULL")
   }
   if(any(f < 0) || any(f > 1)) {
      warning("invalid nonexceedance probability, returning NULL")
      return(NULL)
   }
   ZinI <- c(0,1)
   "afunc" <- function(z, fk, ...) {
                          fk - kfuncCOP(z, subdivisions=subdivisions,
                                           rel.tol=rel.tol, abs.tol=abs.tol, ...) }
   Z <- sapply(1:length(f), function(i) {
           the.z <- uniroot(afunc, ZinI, fk=f[i], cop=cop, para=para, ...)$root
           return(the.z) })
   return(Z)
}

"giniCOP" <-
function(cop=NULL, para=NULL, by.concordance=FALSE, as.sample=FALSE, ...) {
   if(as.sample) {
      if(is.null(para)) {
         warning("Sample Gini's Gamma desired but para is NULL, returning NULL")
         return(NULL)
      }
      if(length(names(para)) != 2) {
        warning("para argument must be data.frame having only two columns, returning NULL")
        return(NULL)
      }
      R <- rank(para[,1]); S <- rank(para[,2]); n <- length(para[,1])
      samGAM <- sum(abs((n + 1 - R) - S) - abs(R - S))
      return( ( 1/floor(n^2/2) ) * samGAM)
   }

   if(is.null(cop)) {
      warning("must have copula argument specified, returning NULL")
      return(NULL)
   }
   if(by.concordance == TRUE) {
     A <- concordCOP(cop=cop, para=para, cop2=M, ...)
     B <- concordCOP(cop=cop, para=para, cop2=W, ...)
     gini <- A + B
     return(A+B)
   }
   myC1 <- function(x, para=NULL) return(cop(x,   x, para=para, ...))
   myC2 <- function(x, para=NULL) return(cop(x, 1-x, para=para, ...))
   C1 <- integrate(myC1, 0, 1, para=para)
   C2 <- integrate(myC2, 0, 1, para=para)
   A <- C1$value; B <- C2$value
   gini <- 4*(A+B) - 2
   return(gini)
}

"convex2COP" <-
function(u,v, para, ...) {
  if(is.null(para$alpha)) {
    warning("no alpha parameter given")
    return(NULL)
  }
  return(   para$alpha  * COP(u, v, cop=para$cop1, para=para$para1) +
         (1-para$alpha) * COP(u, v, cop=para$cop2, para=para$para2))
}

"rhoCOP" <-
function(cop=NULL, para=NULL, method=c("default", "joe21", "joe12"), as.sample=FALSE,
                              brute=FALSE, delta=0.002, ...) {

   if(as.sample) {
      if(is.null(para)) {
         warning("Sample Spearman's Rho desired but para is NULL, ",
                 "returning NULL")
         return(NULL)
      }
      if(length(names(para)) != 2) {
        warning("para argument must be data.frame having only two columns, ",
                "returning NULL")
        return(NULL)
      }
      return(cor(para[,1], para[,2], method="spearman"))
   }


   method <- match.arg(method)
   if(brute) {
      # Following logic would implement the concordance function via tauCOP
      # if the previous if(brute) was not there, but here we only resort to if
      # brute is request.
      Q <- tauCOP(cop=cop, cop2=P, para=para, brute=brute, delta=delta, ...);
      rho <- 3*Q;
      return(rho)
   }
   myint <- NULL
   if(method == "default") {
      try(myint <- integrate(function(u) {
            sapply(u,function(u) { integrate(function(v) {
            COP( u, v, cop=cop, para=para, ...) - u*v}, 0, 1)$value })}, 0, 1) )
      rho1 <- ifelse(is.null(myint), NA, 12*myint$value)
      # This version seems the most stable given M and W copulas
      return(rho1)
   } else if(method == "joe21") {
      try(myint <- integrate(function(u) {
            sapply(u,function(u) { integrate(function(v) {
            u * derCOP( u, v, cop=cop, para=para, ...)},  0, 1)$value })}, 0, 1))
      ifelse(is.null(myint), return(NA), return(3 - 12*myint$value))
   } else if(method == "joe12") {
      try(myint <- integrate(function(u) {
            sapply(u,function(u) { integrate(function(v) {
            v * derCOP2( u, v, cop=cop, para=para, ...)}, 0, 1)$value })}, 0, 1))
      ifelse(is.null(myint), return(NA), return(3 - 12*myint$value))
   } else {
      stop("Should not be here in logic, EVER")
   }
}

"densityCOP" <-
function(u,v, cop=NULL, para=NULL,
              deluv=.Machine$double.eps^0.25,
              truncate.at.zero=TRUE, the.zero=0,
              sumlogs=FALSE, ...) {
   if (length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
      warning("length u = ", length(u), " and length v = ", length(v))
      warning("longer object length is not a multiple of shorter ",
              "object length, no recycling")
      return(NA)
   }
   if(length(u) == 1) {
      u <- rep(u, length(v))
   } else if(length(v) == 1) {
      v <- rep(v, length(u))
   }
   den <- sapply(1:length(u), function(i) {
                     u1 <- u[i];     v1 <- v[i]
                     u2 <- u1+deluv; v2 <- v1+deluv
                     if(u2 > 1) { u1 <- u[i]-deluv; u2 <- u[i] }
                     if(v2 > 1) { v1 <- v[i]-deluv; v2 <- v[i] }
                     (cop(u2,v2, para=para, ...) -
                      cop(u2,v1, para=para, ...) -
                      cop(u1,v2, para=para, ...) +
                      cop(u1,v1, para=para, ...))/deluv^2 })
   if(truncate.at.zero) den[den <= 0] <- the.zero
   if(sumlogs) {
      return(sum(log(den)))
   } else {
      return(den)
   }
}

"blomCOP" <-
function(cop=NULL, para=NULL, as.sample=FALSE,
                  ctype=c("joe", "weibull", "hazen", "1/n",
                                 "bernstein", "checkerboard"), ...) {
   if(as.sample) {
      ctype <- match.arg(ctype)
      if(is.null(para)) {
         warning("Sample Blomqvist's Beta desired but para is NULL, returning NULL")
         return(NULL)
      }
      if(length(names(para)) != 2) {
        warning("para argument must be data.frame having only two columns, returning NULL")
        return(NULL)
      }
      if(ctype == "joe") {
        n <- nrow(para); A <- (1+n)/2
        return((2/ n)*(sum(as.numeric((rank(para[,1]) - A) *
                                      (rank(para[,2]) - A) >= 0))) - 1)
      } else {
        A <- 1/4 # P(1/2, 1/2) = (1/2) * (1/2)
        return(EMPIRcop(0.5, 0.5, para=para, ctype=ctype, ...)/A - 1)
      }
   } else {
      if(is.null(cop)) {
         warning("must have copula argument specified, returning NULL")
         return(NULL)
      }
      blom <- 4*cop(0.5,0.5, para=para, ...) - 1
      return(blom)
   }
}

"psepolar" <- function(u, v=NULL, f=0.90, ...) {
   if(is.null(v)) {
      if(length(names(u)) != 2) {
         warning("a data.frame having only two columns is required")
         return(NULL)
      }
      v <- u[,2]; u <- u[,1] # v must come before u
   }
   if(length(u) != length(v)) {
      warning("argument(s) or implied arguments u and v are unequal ",
              "in length, returning NULL")
      return(NULL)
   }
   n <- length(u)
   Xstar  <- n/(n+1-rank(u)); Ystar <- n/(n+1-rank(v))
   Shat   <- Xstar + Ystar;   What  <- Xstar/Shat
   FXhat1 <- 1 - 1/Xstar;       FYhat1 <- 1 - 1/Ystar
   FXhat3 <- (rank(u) - 1/2)/n; FYhat3 <- (rank(v) - 1/2)/n
   Sf     <- lmomco::dat2bernqua(f, Shat, ...)
   zz <- data.frame(U=u, V=v, Xstar=Xstar, Ystar=Ystar,
                    FXhat1=FXhat1, FYhat1=FYhat1,
                    FXhat3=FXhat3, FYhat3=FYhat3,
                    What=What, Shat=Shat, Shat_ge_Sf=Shat >= Sf)
   return(list(table=zz, Sf=Sf))
}

"M" <-
function(u,v,...) {
  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
    warning("length u = ",length(u), " and length v = ",length(v))
    warning("longer object length is not a multiple of shorter object length, no recycling in M()")
    return(NA)
  }
  # The extra hassle of vectorization made here is to handle situations
  # in which nested integrals are used where uneven vectors can be passed
  if(length(u) == 1) {
     u <- rep(u, length(v))
  } else if(length(v) == 1) {
     v <- rep(v, length(u))
  }
  return(pmin(u,v))
}

"COP" <-
function(u, v, cop=NULL, para=NULL,
               reflect=c("cop", "surv", "acute", "grave",
                           "1",    "2",     "3",     "4"), ...) {

   if(is.list(para) && ! is.null(para$cop) && is.null(cop)) {
      # The third test is related to whether cop is incoming from
      # the top level of the function. This is a major thing to
      # avoid overwriting the desired copula because
      # is.null(para$cop) is fuzzy meaning is.null(para$c) passes too.
      if(! is.null(para$reflect)) reflect <- para$reflect
      if(! is.null(para$cop    )) cop     <- para$cop
      if(! is.null(para$para   )) para    <- para$para
   }

   reflect <- as.character(reflect) # so that numeric integers would
   reflect <-    match.arg(reflect) #           silently be accepted

   if(! is.list(para)) names(para) <- NULL # removes unsightly named labels often on the para in this package
   zz <- switch(reflect,
                 cop   =             cop(u,     v, para=para, ...),
                 surv  = u + v - 1 + cop(1-u, 1-v, para=para, ...),
                 acute =     v     - cop(1-u,   v, para=para, ...),
                 grave = u         - cop(  u, 1-v, para=para, ...),
                 "1"   =             cop(u,     v, para=para, ...),
                 "2"   = u + v - 1 + cop(1-u, 1-v, para=para, ...),
                 "3"   =     v     - cop(1-u,   v, para=para, ...),
                 "4"   = u         - cop(  u, 1-v, para=para, ...)
         )
  zz[zz > 1] <- 1 # assurance in case any numerical issues have caused a nudge outside the
  zz[zz < 0] <- 0 # applicable range of probability on the closed interval [0,1]
  return(zz)
}

"PAcop" <- function(u,v, para=NULL, ...) {
   if(para < 0) {
      warning("parameter must be 0 <= Theta < Infinity [well numerically 100]")
      return(NA)
   }
   if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
      warning("length u = ",length(u), " and length v = ",length(v))
      warning("longer object length is not a multiple of shorter object length, ",
              "no recycling")
      return(NA)
   }
   # The extra hassle of vectorization made here is to handle situations
   # in which nested integrals are used where uneven vectors can be passed
   if(length(u) == 1) {
      u <- rep(u, length(v))
   } else if(length(v) == 1) {
      v <- rep(v, length(u))
   }

   if(para == 0)   return(P(u,v))
   if(para < 1E-8) return(P(u,v)) # from rhoCOP 
   if(para > 100)  return(M(u,v)) # from rhoCOP 

   return( ((1-u)^(-para) + (1-v)^(-para) - 1)^(-1/para) + u + v - 1 )
}


"PARETOcop" <- function(u, v, para=NULL, ...) {
  PAcop(u, v, para=para, ...)
}

"sectionCOP" <-
function(f, cop=NULL,  para=NULL, wrtV=FALSE, dercop=FALSE, delt=0.005,
            ploton=TRUE, lines=TRUE, xlab="NONEXCEEDANCE PROBABILITY", ...) {

  if(wrtV) {
     #message("Triggering Horizontal Section logic: v = constant")
     txt <- "horizontal section"
     xlab <- paste0("U ", xlab) # note this must be U and not V because f is a V
  } else {
     #message("Triggering Vertical Section logic: u = constant")
     txt <- "vertical section"
     xlab <- paste0("V ", xlab) # note this must be V and not U because f is a U
  }

  if(ploton) plot(c(0,1), c(0,1), type="n", xlab=xlab, ...)

  TT <- seq(0+delt, 1-delt, delt)
  C  <- vector(mode="numeric", length=length(TT))
  if(dercop) {
    C <- sapply(TT, function(x) {
                     ifelse(wrtV, return( derCOP2(x,f, cop=cop, para=para) ),
                                  return(  derCOP(f,x, cop=cop, para=para) )) } )
  } else {
    C <- sapply(TT, function(x) {
                      ifelse(wrtV, return( cop(x,f, para=para) ),
                                   return( cop(f,x, para=para) )) } )
  }
  if(lines & ! is.null(dev.list())) lines(TT, C, ...)
  return(list(t=TT, seccop=C, wrt=txt, fvalue=f, isderivative=dercop))
}

"CLcop" <- function(u, v, para=NULL, tau=NULL, ...) {
    if(is.null(para)) {
      if(is.null(tau)) {
        tau <- cor(u,v, method="kendall")
      }
      if(tau > 0.975) {
        warning("tau > 0.975, simCOP might begin to show rare failures this high")
      }
      para <- 2*tau/(1-tau)
      names(para) <- "theta"
      names(tau)  <- "Kendall Tau"
      return(list(para=para, tau=tau))
    }
    if(length(para) == 1) {
       if(para[1] < -1) {
          warning("Parameter Theta < -1")
          return(NULL)
       }
       tau <- para/(para+2)
    } else {
       warning("Parameter Theta can not be a vector")
       return(NULL)
    }
    if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
       warning("length u = ", length(u), " and length v = ", length(v))
       warning("longer object length is not a multiple of shorter object length, ",
               "no recycling")
       return(NA)
    }
    if(length(u) == 1) {
       u <- rep(u, length(v))
    } else if(length(v) == 1) {
       v <- rep(v, length(u))
    }
    para.small <- 1E-4 # simCOP(10000, cop=CLcop, para=-.0001) [no failures]
    if(abs(para) < para.small) return(u*v)
    if(tau > 0.975) return(M(u,v))
    cop <- u^-para + v^-para - 1
    cop[cop < 0] <- 0
    return(cop^(-1/para))
}

"asCOP" <- function(u,v, f=NULL, ...) {
  if(is.null(f)) {
    warning("'f' is a mandatory argument to be a function")
    return(NA)
  } else if(! is.function(f)) {
    warning("'f' must be a function")
    return(NA)
  }
  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
    warning("length u = ",length(u), " and length v = ",length(v))
    warning("longer object length is not a multiple of shorter object length, no recycling")
    return(NA)
  }
  # The extra hassle of vectorization made here is to handle situations
  # in which nested integrals are used where uneven vectors can be passed
  if(length(u) == 1) {
     u <- rep(u, length(v))
  } else if(length(v) == 1) {
     v <- rep(v, length(u))
  }
  return(sapply(1:length(u), function(i) { f(u[i], v[i], ...) }))
}

"LzCOPpermsym" <-
function(cop=NULL, para=NULL, n=5E4,
         type=c("halton", "sobol", "torus", "runif"),
         as.abs=TRUE, as.vec=FALSE, as.mat=FALSE, plot=FALSE, ...) {

  type <- match.arg(type)
  n <- as.integer( n ) # to silence certain upstream warnings()

  if(type != "runif") {
    if(! "randtoolbox" %in% installed.packages()) {
      warning("randtoolbox package not installed, switch type to 'runif'")
      type <- 'runif'
    }
  }

  if(type == "runif") {
    ruv <- matrix(runif(2*n),    ncol=2)
  } else if(type == "halton") {
    ruv <- randtoolbox::halton(n, dim=2)
  } else if(type == "sobol" ) {
    ruv <- randtoolbox::sobol(n,  dim=2)
  } else if(type == "torus" ) {
    ruv <- randtoolbox::torus(n,  dim=2)
  }

  if(plot) {
    plot(ruv, main=type,
         xlab="U, NONEXCEEDANCE PROBABILITY", ylab="V, NONEXCEEDANCE")
  }

  if(as.vec | as.mat) as.abs <- FALSE

  ifelse(as.abs, absf <- abs, absf <- function(x) x)

  sym <- absf( COP(ruv[,1], ruv[,2], cop=cop, para=para, ...) -
               COP(ruv[,2], ruv[,1], cop=cop, para=para, ...) )
  if(as.mat) {
    return( cbind(ruv, 3 *        sym ) )
  } else {
    if(as.vec) {
      return(          3 *        sym   )
    } else {
      if(as.abs) {
        return(        3 * max(   sym ) )
      } else {
        return(        3 * range( sym ) )
      }
    }
  }
}

"ORDSUWcop" <-
function(u,v, para=list(cop=M, para=NA, part=c(0,1)), ...) {

  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
    warning("length u = ",length(u), " and length v = ",length(v))
    warning("longer object length is not a multiple of shorter object length, no recycling in M()")
    return(NA)
  }
  # The extra hassle of vectorization made here is to handle situations
  # in which nested integrals are used where uneven vectors can be passed
  if(length(u) == 1) {
     u <- rep(u, length(v))
  } else if(length(v) == 1) {
     v <- rep(v, length(u))
  }

  if(is.null(para)) {
    para <- list(cop=copBasic::M, para=NULL, part=c(0,1))
  }

  if(! exists("part", para)) {
    warning("must have the part element of the para list populated")
    return(NULL)
  }

  if(! is.list(para$part)) {
    x  <- as.vector(para$part)
    np <- length(x)
    ck <-   diff(x)
    if(np < 2) {
      warning("para$para given as vector of break pionts but length is not >= 2")
      return(NULL)
    }
    if(any(ck <  0)) {
      warning("para$part given as vector of break points but not monotonic increasing")
      return(NULL)
    }
    if(any(ck == 0)) {
      warning("para$part given as vector of break points but a number is repeated")
      return(NULL)
    }
    myJ <- lapply(seq(1, np-1, by=1), function(i) c(x[i], x[i+1]))
  } else {
    myJ <- para$part
  }

  num_partitions <- length(myJ); kx <- seq_len(num_partitions)
  if(exists("para", para) & is.null(para$para)) {
    myQ <- lapply(kx, function(k) NA)
  } else if(exists("para", para) & length(para$para) == 1 && is.na(para$para)) {
    myQ <- lapply(kx, function(k) NA)
  } else if(! exists("para", para)) {
    myQ <- lapply(kx, function(k) NA)
  } else if(is.list(  para$para) & length(para$para) == 1) {
    myQ <- lapply(kx, function(k) para$para[[1]])
  } else if(is.vector(para$para) & length(para$para) == 1) {
    myQ <- lapply(kx, function(k) para$para)
  } else {
    myQ <- para$para
  }

  if(! exists("cop", para)) {
    myC <- lapply(kx, function(k) copBasic::W)
  } else if(length(para$cop) == 1) {
    if(is.list(para$cop)) {
      myC <- lapply(kx, function(k) para$cop[[1]])
    } else {
      myC <- lapply(kx, function(k) para$cop)
    }
  } else {
    myC <- para$cop
  }

  if(length(myJ) != length(myQ) | length(myJ) != length(myC)) {
    warning("malformed para argument (ORDSUWcop): ",
            paste(c("myJ", "myQ", "myC"), c(length(myJ), length(myQ), length(myC)),
            collapse=",", sep="="))
    message("myJ: "); print(str(myJ))
    message("myQ: "); print(str(myQ))
    message("myC: "); print(str(myC))
    return(NULL)
  }

  zz <- sapply(seq_len(length(u)), function(i) {
           for(k in kx) {
             a <- myJ[[k]][1]; b <- myJ[[k]][2]; ba <- b-a
             # if(u[i] < a | u[i] > b | v[i] < a | v[i] > b) next # outside [a,b]^2
             if(u[i] <   a | u[i] >=     b) next
             if(v[i] < 1-b | v[i] >= 1 - a) next
             return(a+ba*myC[[k]]((u[i]-a)/ba, (v[i]-1+b)/ba, para=myQ[[k]], ...))
           }; return(max(u[i]+v[i]-1, 0)) }) # W(u,v) otherwise
  return(zz)
}

"lcomCOP" <-
function(cop=NULL, para=NULL, as.bilmoms=FALSE, orders=2:5, lo=0, hi=1,
         subdivisions=100L, rel.tol=.Machine$double.eps^0.25, abs.tol=rel.tol,
         stop.on.error=TRUE, ...) {
   if(is.null(cop)) {
      warning("must have copula argument specified, returning NULL")
      return(NULL)
   }
   if(any(orders < 2) | any(orders > 5)) {
     warning("orders < 2 are not logicaly or > 5 are not supported")
     return(NULL)
   }

   orders <- orders - 1
   r <- ifelse(as.bilmoms, 0, 1)
   func1 <- function(t) {          2                       }
   func2 <- function(t) { ( 12*t   -   6)                  }
   func3 <- function(t) { ( 60*t^2 -  60*t   +  12)        }
   func4 <- function(t) { (280*t^3 - 420*t^2 + 180*t - 20) }

   deltasX1wrtX2 <- deltasX2wrtX1 <- rep(NA,4+r)
   zz <- list()

   if(any(orders == 1)) { myint <- NULL
     try(myint <- integrate(function(u) {
                   sapply(u,function(u) { integrate(function(v) {
                   func1(v)*COP(u,v,cop=cop, para=para, ...) }, lo, hi,
                                        subdivisions=subdivisions, rel.tol=rel.tol, abs.tol=abs.tol,
                                        stop.on.error=stop.on.error)$value })},
                                                                lo, hi,
                                        subdivisions=subdivisions, rel.tol=rel.tol, abs.tol=abs.tol,
                                        stop.on.error=stop.on.error) )
     deltasX1wrtX2[1+r] <- ifelse(is.null(myint), NA, myint$value - 0.5)
   }
   if(any(orders == 2)) { myint <- NULL
     try(myint <- integrate(function(u) {
                   sapply(u,function(u) { integrate(function(v) {
                   func2(v)*COP(u,v,cop=cop, para=para, ...) }, lo, hi,
                                        subdivisions=subdivisions, rel.tol=rel.tol, abs.tol=abs.tol,
                                        stop.on.error=stop.on.error)$value })},
                                                                lo, hi,
                                        subdivisions=subdivisions, rel.tol=rel.tol, abs.tol=abs.tol,
                                        stop.on.error=stop.on.error) )
     deltasX1wrtX2[2+r]  <- ifelse(is.null(myint), NA, myint$value - 0.5)
   }
   if(any(orders == 3)) { myint <- NULL
     try(myint <- integrate(function(u) {
                   sapply(u,function(u) { integrate(function(v) {
                   func3(v)*COP(u,v,cop=cop, para=para, ...) }, lo, hi,
                                        subdivisions=subdivisions, rel.tol=rel.tol, abs.tol=abs.tol,
                                        stop.on.error=stop.on.error)$value })},
                                                                lo, hi,
                                        subdivisions=subdivisions, rel.tol=rel.tol, abs.tol=abs.tol,
                                        stop.on.error=stop.on.error) )
     deltasX1wrtX2[3+r]  <- ifelse(is.null(myint), NA, myint$value - 0.5)
   }
   if(any(orders == 4)) { myint <- NULL
     try(myint <- integrate(function(u) {
                   sapply(u,function(u) { integrate(function(v) {
                   func4(v)*COP(u,v,cop=cop, para=para, ...) }, lo, hi,
                                        subdivisions=subdivisions, rel.tol=rel.tol, abs.tol=abs.tol,
                                        stop.on.error=stop.on.error)$value })},
                                                                lo, hi,
                                        subdivisions=subdivisions, rel.tol=rel.tol, abs.tol=abs.tol,
                                        stop.on.error=stop.on.error) )
     deltasX1wrtX2[4+r] <- ifelse(is.null(myint), NA, myint$value - 0.5)
   }

   if(any(orders == 1)) { myint <- NULL
     try(myint <- integrate(function(u) {
                   sapply(u,function(u) { integrate(function(v) {
                   func1(u)*COP(u,v,cop=cop, para=para, ...) }, lo, hi,
                                        subdivisions=subdivisions, rel.tol=rel.tol, abs.tol=abs.tol,
                                        stop.on.error=stop.on.error)$value })},
                                                                lo, hi,
                                        subdivisions=subdivisions, rel.tol=rel.tol, abs.tol=abs.tol,
                                        stop.on.error=stop.on.error) )
     deltasX2wrtX1[1+r] <- ifelse(is.null(myint), NA, myint$value - 0.5)
   }
   if(any(orders == 2)) { myint <- NULL
     try(myint <- integrate(function(u) {
                   sapply(u,function(u) { integrate(function(v) {
                   func2(u)*COP(u,v,cop=cop, para=para, ...) }, lo, hi,
                                        subdivisions=subdivisions, rel.tol=rel.tol, abs.tol=abs.tol,
                                        stop.on.error=stop.on.error)$value })},
                                                                lo, hi,
                                        subdivisions=subdivisions, rel.tol=rel.tol, abs.tol=abs.tol,
                                        stop.on.error=stop.on.error) )
     deltasX2wrtX1[2+r] <- ifelse(is.null(myint), NA, myint$value - 0.5)
   }
   if(any(orders == 3)) { myint <- NULL
     try(myint <- integrate(function(u) {
                   sapply(u,function(u) { integrate(function(v) {
                   func3(u)*COP(u,v,cop=cop, para=para, ...) }, lo, hi,
                                        subdivisions=subdivisions, rel.tol=rel.tol, abs.tol=abs.tol,
                                        stop.on.error=stop.on.error)$value })},
                                                                lo, hi,
                                        subdivisions=subdivisions, rel.tol=rel.tol, abs.tol=abs.tol,
                                        stop.on.error=stop.on.error) )
     deltasX2wrtX1[3+r] <- ifelse(is.null(myint), NA, myint$value - 0.5)
   }
   if(any(orders == 4)) { myint <- NULL
     try(myint <- integrate(function(u) {
                   sapply(u,function(u) { integrate(function(v) {
                   func4(u)*COP(u,v,cop=cop, para=para, ...) }, lo, hi,
                                        subdivisions=subdivisions, rel.tol=rel.tol, abs.tol=abs.tol,
                                        stop.on.error=stop.on.error)$value })},
                                                                lo, hi,
                                        subdivisions=subdivisions, rel.tol=rel.tol, abs.tol=abs.tol,
                                        stop.on.error=stop.on.error) )
     deltasX2wrtX1[4+r] <- ifelse(is.null(myint), NA, myint$value - 0.5)
   }

   if(as.bilmoms) {
     names(deltasX1wrtX2) <- c("BiVarLM:del1[12]", "BiVarLM:del2[12]",
                               "BiVarLM:del3[12]", "BiVarLM:del4[12]")
     names(deltasX2wrtX1) <- c("BiVarLM:del1[21]", "BiVarLM:del2[21]",
                               "BiVarLM:del3[21]", "BiVarLM:del4[21]")
     zz$bilmomUV <- deltasX1wrtX2; zz$bilmomVU <- deltasX2wrtX1
   } else {
     names(deltasX1wrtX2) <- c("nothing",
                               "Lcomom:T2[12]", "Lcomom:T3[12]",
                               "Lcomom:T4[12]", "Lcomom:T5[12]")
     names(deltasX2wrtX1) <- c("nothing",
                               "Lcomom:T2[21]", "Lcomom:T3[21]",
                               "Lcomom:T4[21]", "Lcomom:T5[21]")
     zz$lcomUV <- 6*deltasX1wrtX2; zz$lcomVU <- 6*deltasX2wrtX1
   }
   return(zz)
}

"bicoploc" <-
function(xp, yp=NULL, xout=NA, xpara=NULL, ypara=NULL, dtypex="nor", dtypey="nor",
         ctype=c("weibull", "hazen", "bernstein", "checkerboard"), kumaraswamy=TRUE,
         plotuv=TRUE, plotxy=TRUE, adduv=FALSE, addxy=FALSE, snv=FALSE, limout=TRUE,
         autoleg=TRUE, xleg="topleft", yleg=NULL, rugxy=TRUE, ruglwd=0.5,
         xlim=NULL, ylim=NULL, titleuv="", titlexy="", titlecex=1,
         a=0, ff=pnorm(seq(-5, +5, by=0.1)), locdigits=6,
         paracop=TRUE, verbose=TRUE, x=NULL, y=NULL, ...) {

  USE_PARA_COP <- paracop # The capital letters become easier to "find" in the function
  rm(paracop)

  lo <- .Machine$double.eps; hi <- 1 - lo
  # being at and close to the edges of probability helps to ensure that we by default stress the
  # algorithm in how we handle or trap issues on the edges.
  ff <- sort( unique( c(0, lo, lo^0.5, ff, 1-lo^0.5, hi, 1) ) ) # assurance policy
  # also stretching to the edges, ensures too that we get simply stress various subsystems

  if(is.matrix(xp) | is.data.frame(xp)) {
    xp <- xp[,1]; yp <- yp[,2]
  }

  if(any(is.na(xp))) {
    message("some of the xp are NA, not fully configured to handle such circumstances")
    return(NULL)
  }
  if(any(is.na(yp))) {
    message("some of the yp are NA, not fully configured to handle such circumstances")
    return(NULL)
  }
  if(length(xp) != length(yp)) {
    message("length of xp and yp are not identical")
    return(NULL)
  }
  n <- length(xp)

  if(is.null(xout)) xout <- NA

  if(plotuv) adduv <- FALSE
  if(plotxy) addxy <- FALSE

  PPtxt <- "plotting position"
  a  <- a[1] # silent devectorization
  aa <- as.character(round(as.numeric(a), digits=4)) # to fit into built-in lmomco
  aa <- switch(aa, "0"      = "Weibull",
                   "0.3175" = "Median",
                   "0.375"  = "Blom",
                   "0.40"   = "Cunnane",
                   "0.44"   = "Gringorten",
                   "0.5"    = "Hazen")
  if(! is.null(aa)) PPtxt  <- paste0(aa, " ", PPtxt)

  init.kur <- c(1.5, 2) # initial guesses at Kumaraswamy parameters, these were somewhat guessed at
  # but seem have robustness. Maybe better could be found.

  ctype    <- match.arg(      ctype     )
  txt      <- unlist(strsplit(ctype, ""))
  ctyTXT   <- paste0(toupper(txt[1]), paste(txt[2:length(txt)], sep="", collapse="")) # extract first letter and upper case it
  ctyTXTuv <- paste0("Diagonal inversion points in (U,V) domain strictly by ", ctyTXT, " empirical copula")
  ctyTXTxy <- paste0("Diagonal inversion points in (X,Y) domain strictly by ", ctyTXT, " empirical copula")
  PCHcty   <- 4   # a times symbol
  PCHcex   <- 1.2 # size of the letter for plotting
  PCHlwd   <- 1.8
  PCHcol   <- "mediumorchid1"        # symbol plotting color for the empirical copula diagonal
  PCHbg    <- "white"  # background symbol plotting color for the empirical copula diagonal
  COPpch   <- 23
  COPbg    <- "chocolate1"
  COPcol   <- "chocolate1"
  COPptlwd <- 0.8
  COPcex   <- 0.7
  COPlty   <- 1
  COPlwd   <- 6
  DIAeol   <- "darkgreen"  # color for plotting of kumaraswamy line smooth of diagonal
  DIAcol   <- "seagreen"  # color for plotting of kumaraswamy line smooth of diagonal
  DIAbg    <- "lightgreen"
  DIAcex   <- 0.91
  DIApch   <- 21       # background symbol plotting color for the diagonal
  DIAlwd   <- 3
  DIAlty   <- 1
  DIAptlwd <- 1.1
  OUTcex   <- 1.3
  OUTcol   <- "darkred"    #            symbol plotting color for the Xout, if given
  OUTbg    <- "white"  # background symbol plotting color for the Xout, if given
  OUTpch   <- 16       # plotting character for the predictions from the inverse copula diagonal
  OUTptlwd <- 1
  LOCpch   <- 22       # plotting character for line of organic correlation predictions
  DATcol   <- "black"  # symbol plotting color for the Xs
  DATbg    <- "white"  # background symbol plotting color for the Xs
  DATcex   <- 1.1
  DATpch   <- 21
  DATptlwd <- 0.9
  LOClwd   <- 3
  LOClty   <- 2
  LOCcol   <- "blue"
  LOCbg    <- "white"
  LOCcex   <- 1.9
  LOCptlwd <- 1
  LOCUlty  <- 2
  LOCUlwd  <- 1.5
  LOCUcol  <- "deepskyblue2"
  LEGcex   <- 0.75    # legend() text expansion factor
  SEGlen   <- 3.5      # width of line segments in the legend()
  LEGinset <- 0.02
  RUGcol   <- "wheat3" # ticking color for rug() plotting

  FFedge  <- c(0.001, 0.999) # reasonably wide range when plotting marginal distributions, these
  qFFedge <- qnorm(FFedge)   # standard normal variates of reasonably deep in the tails
  qff     <- qnorm(ff)       # standard normal variates of the ff vector
  # remember, the ff vector is the nonexceedance probability of the diagonal of the copula and hence
  # is that joint probability for which we are playing with.

  i <- 0 # a counter only for the verbose messaging

  if(verbose) message("(", i <- i + 1, ") plotting positions for U and V, recall lmomco::pp() ties.method=first")
  u  <- lmomco::pp(xp, a=a, sort=FALSE) # plotting positions
  v  <- lmomco::pp(yp, a=a, sort=FALSE) # plotting positions
  UV <- data.frame(U=u, V=v) # to become the para argument for the empirical copula

  sortX <- sort(xp) # making other diagnostic plots to potentially come easier to assemble

  # parametric marginal distribution of X and Y (if parameter estimation is otherwise required)
  if(is.null(xpara)) {
    if(verbose) message("(", i <- i + 1, ") starting parameter estimation for X parametric ",
                                         "margin by L-moments for dtypex")
    xpara <- NULL
    if(is.null(x)) {
      thex <- xp
    } else {
      if(verbose) message("(", i <- i + 1, ") using the alternative X values in argument x ",
                                           "for parameter estimation")
      thex <- x
    }
    lmr <- lmomco::lmoms(thex[is.finite(thex)], no.stop=TRUE)
    if(is.null(lmr)) {
      message("degenerate sample in variable 'thex', L-moments in X direction seem to have failed")
      return(NULL)
    }
    try(xpara <- lmomco::lmom2par(lmr, type=dtypex), silent=TRUE)
    if(is.null(xpara)) {
      message("parameter estimation for X by method of L-moments has failed, try another dtypex?")
      return(NULL)
    }
    if(is.null(dtypex)) {
      if(exists("type", xpara)) dtypex <- xpara$type
    }
  } else {
    if(is.null(x)) { # secondary look at the x is for support of the rug() call and nothing more
      thex <- xp
    } else {
      thex <- x
    }
    if(is.null(dtypex) || is.na(dtypex)) {
      dtypex <- "--" # reset to empty to ensure that what was given is used from xpara forevermore
    }
  }

  if(is.null(ypara)) {
    if(verbose) message("(", i <- i + 1, ") starting parameter estimation for Y parametric ",
                                         "margin by L-moments for dtypex")
    ypara <- NULL
    if(is.null(y)) {
      they <- yp
    } else {
      if(verbose) message("(", i <- i + 1, ") using the alternative X values in argument x ",
                                           "for parameter estimation")
      they <- y
    }
    lmr <- lmomco::lmoms(they[is.finite(they)], no.stop=TRUE)
    if(is.null(lmr)) {
      message("degenerate sample in variable 'they', L-moments in Y direction seem to have failed")
      return(NULL)
    }
    try(ypara <- lmomco::lmom2par(lmomco::lmoms(they[is.finite(they)]), type=dtypey), silent=TRUE)
    if(is.null(ypara)) {
      message("parameter estimation for Y by method of L-moments has failed, try another dtypey?")
      print(they)
      return(NULL)
    }
    if(is.null(dtypey)) {
      if(exists("type", ypara)) dtypey <- ypara$type
    }
  } else { # this secondary look at the y is for support of the rug() call and nothing more
    if(is.null(y)) {
      they <- yp
    } else {
      they <- y
    }
    if(is.null(dtypey) || is.na(dtypey)) {
      dtypey <- "--" # reset to empty to ensure that what was given is used from ypara forevermore
    }
  }

  n_thex <- length( thex[is.finite(thex)] )
  n_they <- length( they[is.finite(they)] )

  if(verbose) message("(", i <- i + 1, ") determing association sign by Spearman Rho")
  # recall for the usual line of organic correlation that the sign of Rho sets the sign of the
  # slope of the line, it appears that we can do the same as part of the extension of the logic
  # with copulas to support negatively associated data sets
  rhoS    <- cor(xp, yp, method="spearman")
  rhosign <- sign( rhoS ) # kind of like "Wormsign"

  if(verbose) message("(", i <- i + 1, ") computing line of organic correlation by lmomco::lmrloc()")
  XYp <- data.frame(X=xp, Y=yp) # a bet permissive to allow NAs incoming but works with the is.finite
  XYp <- XYp[complete.cases(XYp),] # check above before the L-moment parameter estimation is called
  XYp <- XYp[is.finite(XYp[,1]) & is.finite(XYp[,2]),] # an finite check here means that we have the
  locpair  <- lmomco::lmrloc( XYp )  # locpair existing, -Inf and +Inf incoming likely stem from
  # simulation play in highly skewed distributions and not in the practical application on
  # real-world data compute the line of organic correlation (reduced major axis), using the
  # method of L-moments.
  names(locpair$loc_lmr) <- c("LMR_PAIR_Intercept", "LMR_PAIR_Slope")
  names(locpair$loc_pmr) <- c("PMR_PAIR_Intercept", "PMR_PAIR_Slope")

  xlmr <- lmomco::par2lmom(xpara)$lambdas; # print(xlmr)
  ylmr <- lmomco::par2lmom(ypara)$lambdas; # print(ylmr)
  m_para  <- rhosign * (ylmr[2] /           xlmr[2]) # slope by relative standard deviations
  b_para  <-            ylmr[1] - (m_para * xlmr[1]) # intercept to pass through the means
  locpara <- c(b_para, m_para)
  names(locpara) <- c("LMR_PARA_Intercept", "LMR_PARA_Slope")
  locpara <- list(loc_lmr=locpara)

  if(any(! is.na(xout))) {
    if(verbose) message("(", i <- i + 1, ") estimating xout by line of organic correlation")
    Yloc      <- locpair$loc_lmr[2] * sortX + locpair$loc_lmr[1]
    Yloc_out  <- locpair$loc_lmr[2] * xout  + locpair$loc_lmr[1]
    Ylocu     <- locpara$loc_lmr[2] * sortX + locpara$loc_lmr[1]
    Ylocu_out <- locpara$loc_lmr[2] * xout  + locpara$loc_lmr[1]

    names(Yloc)      <- NULL
    names(Yloc_out)  <- NULL
    names(Ylocu)     <- NULL
    names(Ylocu_out) <- NULL
  }

  # Invert the diagonals of copula. The ff is the joint probability Pr[X <= x & Y <= y] = C(u,v)
  # and we want to solve on the diagonal for U=V=t --> C(t,t) = ff on the supposition that the
  # diagonal of the copula is the mathematical structure useful for the problem at hand
  if(verbose) message("(", i <- i + 1, ") computing vector of primary diagonal of empirical copula")
  ec <- eco <- COP(ff,   ff, cop=EMPIRcop, para=UV, ctype=ctype, ...)
  # Nonexceedances of the U and V and a copy forming the "original empirical", we will optionally
  # smooth the ec (empirical copula) but never the eco (empirical copula original)

  # Here are two other ways to think about this diagonal and ultimately its inverse. Seems just hit
  # it at mass as in above COP() call is sufficient with fine enough resolution for approx()-like
  # calls to come or for setup of the Kumaraswamy [0,1]-bounded quantile function smooth estimation
  # to ensure that we are on the real number line.
  #ed <- diagCOP(cop=EMPIRcop, para=UV, ctype=ctype, ploton=FALSE, lines=FALSE, delt=0.001)
  #ed$t <- c(0, ed$t, 1); ed$diagcop <- c(0, ed$diagcop, 1)
  #ff <- ed$t; ec <- eco <- ed$diagcop
  #ec <- eco <- diagCOPatf(ff, cop=EMPIRcop, para=UV, ctype=ctype)
  if(kumaraswamy) { # https://en.wikipedia.org/wiki/Kumaraswamy_distribution
    if(verbose) message("(", i <- i + 1, ") fitting Kumaraswamy (method of percentiles)")
    kur.init.para   <- lmomco::vec2par(init.kur, type="kur")
        eckurpara   <- NULL
    try(  eckurpara <- lmomco::disfitqua(ec, ff, type="kur", init.para=kur.init.para), silent=TRUE)
    if(is.null(eckurpara)) {
      try(eckurpara <- lmomco::disfitqua(ec, ff, type="kur"),                          silent=TRUE)
    }
    if(is.null(eckurpara)) {
      if(verbose) message("(", i <- i + 1, ") checking Kumaraswamy, disfitqua failed, disabling")
      kumaraswamy <- FALSE # disabling
    } else if( length(unique(lmomco::par2qua(ff, eckurpara, paracheck=FALSE))) == 1 ) {
      if(verbose) message("(", i <- i + 1, ") checking Kumaraswamy, flat lined, disabling")
      kumaraswamy <- FALSE # disabling
    } else {
      ec <- lmomco::par2qua(ff, eckurpara, paracheck=FALSE) # the replacement of the "smoothed
      # diagonal of the empirical copula through the use of the Kumaraswamy distribution fit to
      # RMSE of method of percentiles. The ec are the "u=v=t" in general copula nomenclature.
    }
  }


  infS <- NA # only to have it populated on output
  ### BEGIN PARAMETRIC COPULA BLOCK
  if(USE_PARA_COP) {
    permsynsim <- 5E4
    infS  <- LzCOPpermsym(cop=EMPIRcop, para=UV, n=permsynsim, type="halton", as.vec=FALSE, ctype=ctype)
    infSv <- LzCOPpermsym(cop=EMPIRcop, para=UV, n=permsynsim, type="halton", as.vec=TRUE,  ctype=ctype)
    tparf <- function(par) c(exp(par[1]), pnorm( par[2] ), pnorm( par[3] ))
    rparf <- function(par) c(log(par[1]), qnorm( par[2] ), qnorm( par[3] ))
    ofunc <- function(par) { # objective function
      mypara <- tparf(par)
      mypara <- list(  cop=PLcop,         para=mypara[1], alpha=mypara[2], beta=mypara[3])
      rhoT   <- rhoCOP(cop=composite1COP, para=mypara)    # simulated Spearman Rho
      infTv  <- LzCOPpermsym(cop=composite1COP, para=mypara, n=permsynsim, type="halton", as.vec=TRUE)
      (rhoT - rhoS)^2 + mean( (infTv - infSv)^2 )
    }
    if(verbose) message("(", i <- i + 1, ") calling optim() for multiD optimization of parametric copula")
    init.par <- rparf(c(1, 0.5, 0.5)); rt <- NULL # initial parameter guess
    try( rt <- optim(init.par, fn=ofunc) ) # 3D optimization
    if(is.null(rt)) {
      message("composite1COP parameter estimation returned NULL, disabling parameteric copula")
      USE_PARA_COP <- FALSE
    } else {
      para.cop <- tparf(rt$par)
      para.cop <- list(    cop=PLcop,         para=para.cop[1], alpha=para.cop[2], beta=para.cop[3])
      rhoT <- rhoCOP(      cop=composite1COP, para=para.cop)
      infT <- LzCOPpermsym(cop=composite1COP, para=para.cop, n=permsynsim, type="halton", as.vec=FALSE)

      # JK <- simCOP(1000, cop=composite1COP, para=para.cop) # developer use only for intermediate check
      faqscop <- c(rhoS, infS, rhoT, infT, para.cop$alpha, para.cop$beta, para.cop$para)
      ctxt <- paste0("CopulaParameter", seq_len(length(para.cop$para)))
      faqscop <- round(faqscop, digits=6)
      names(faqscop) <- c("SpearmanRhoSample",       "LzCOPpermsymSample",
                          "SpearmanRhoFittedCopula", "LzCOPpermsymFittedCopula",
                          "Alpha", "Beta", ctxt)
      dtt <- COP(ff, ff, cop=composite1COP, para=para.cop, ...)
    }
  } ### END PARAMETRIC COPULA BLOCK

  if(! USE_PARA_COP) {
    dtt <- dtty <- rep(NA, length(ff)) # assurance of no misuse by NA these vectors should they leak
    faqscop <- c(rhoS, infS)
    faqscop <- round(faqscop, digits=6)
    names(faqscop) <- c("SpearmanRhoSample", "LzCOPpermsymSample")
  }

  if(rhosign < 0) { # negative association
    ecy  <- 1 - ec
    ecoy <- 1 - eco
    dtty <- 1 - dtt
  } else {          # positive association
    ecy  <-     ec
    ecoy <-     eco
    dtty <-     dtt
  }

  # An open question, do we 1-v the plotting of the diagonal on the Y-axis when Rho < 0
  # and also a 1 - lmomco::quakur() also, or does all of this reflect things in the wrong way?
  if(plotuv) {
    if(! adduv) {
      if(snv) {
        if(verbose) message("(", i <- i + 1, ") plotting the (U,V) domain in SNV units")
        xylim <- qFFedge
        plot(qff, qnorm(eco), xlim=xylim, ylim=xylim, type="n", las=1, xaxs="i", yaxs="i",
             xlab="SNV OF COPULA DIAGONAL (Vector of 'f' joint probability)", mgp=c(2.2, 0.75, 0),
             ylab="SNV INVERSE COPULA DIAGONAL (f = C(U,V))\nTHE U = V NONEXCEEDANCE PROBABILITY")
      } else {
        if(verbose) message("(", i <- i + 1, ") plotting the (U,V) domain in probability")
        xylim <- c(0, 1)
        plot(ff,       eco,  xlim=xylim, ylim=xylim, type="n", las=1, xaxs="r", yaxs="r",
             xlab="COPULA DIAGONAL (Vector of 'f' joint probability)", mgp=c(2.2, 0.75, 0),
             ylab="INVERSE COPULA DIAGONAL (f = C(U,V))\nTHE U = V NONEXCEEDANCE PROBABILITY")
      }
      mtext(titleuv, side=3, line=1, font=2, cex=titlecex)
      axis(3, axTicks(1), labels=FALSE, lwd=NA, lwd.ticks=1)
      axis(4, axTicks(2), labels=FALSE, lwd=NA, lwd.ticks=1)
    }
    if(snv) {
      if(USE_PARA_COP) lines(qff, qnorm(dtt), col=COPcol, lwd=COPlwd)
      points(    qff,     qnorm(eco),  pch=PCHcty,   cex=PCHcex, col=PCHcol, lwd=PCHlwd)
      points(qnorm(UV$U), qnorm(UV$V), pch=DATpch,   cex=DATcex, col=DATcol, lwd=DATptlwd, bg=DATbg)
      if(kumaraswamy)  lines(qff, qnorm(lmomco::qlmomco(ff, eckurpara)), col=DIAcol, lwd=DIAlwd, lend=2)
      points(    qff,     qnorm(ecy),  pch=DIApch,   cex=DIAcex, col=DIAeol, lwd=DIAptlwd, bg=DIAbg)
    } else {
      if(USE_PARA_COP) lines( ff, dtt, col=COPcol, lwd=COPlwd)
      points(     ff,           eco,   pch=PCHcty,   cex=PCHcex, col=PCHcol, lwd=PCHlwd)
      points(UV$U,      UV$V,          pch=DATpch,   cex=DATcex, col=DATcol, lwd=DATptlwd, bg=DATbg)
      if(kumaraswamy)  lines( ff, lmomco::qlmomco(ff, eckurpara),col=DIAcol, lwd=DIAlwd, lend=2)
      points(     ff,           ec,    pch=DIApch,   cex=DIAcex, col=DIAeol, lwd=DIAptlwd, bg=DIAbg)
    }
    if(autoleg) {
      lwd    <- c(COPlwd,  DIAlwd,        NA,      NA,        NA,  NA)
      lty    <- c(COPlty,  DIAlty,        NA,      NA,        NA,  NA)
      col    <- c(COPcol,  DIAcol,    DIAcol,  PCHcol,    DATcol,  NA)
      pch    <- c(    NA,      NA,    DIApch,  PCHcty,    DATpch,  NA)
      pt.bg  <- c(    NA,      NA,     DIAbg,  PCHbg,      DATbg,  NA)
      pt.lwd <- c(    NA,      NA,  DIAptlwd,  PCHlwd,  DATptlwd,  NA)
      pt.cex <- c(    NA,      NA,    DIAcex,  PCHcex,    DATcex,  NA)
      txt    <- c("Diagonal inverse of parametric asymmetric Plackett copula",
                  "Kumaraswamy smooth for diagonal inverse of the empirical copula coordinates",
                  "Coordinates of empirical copula diagonal inverse from joint probability vector",
                   ctyTXTuv,
       paste0("Paired data points (n=", prettyNum(n, big.mark=","), ") between U and V by ", PPtxt),
              'Note, interpretation of "best fit" to data points on this plot is not applicable.')
      if(! USE_PARA_COP) {
        nix <- -1
        lwd    <- lwd[   nix]
        lty    <- lty[   nix]
        col    <- col[   nix]
        pch    <- pch[   nix]
        pt.bg  <- pt.bg[ nix]
        pt.lwd <- pt.lwd[nix]
        pt.cex <- pt.cex[nix]
        txt    <- txt[   nix]
      }
      if(! kumaraswamy) {
        nix <- -grep("Kumaraswamy", txt)
        lwd    <- lwd[   nix]
        lty    <- lty[   nix]
        col    <- col[   nix]
        pch    <- pch[   nix]
        pt.bg  <- pt.bg[ nix]
        pt.lwd <- pt.lwd[nix]
        pt.cex <- pt.cex[nix]
        txt    <- txt[   nix]
      }
      opts <- par(no.readonly=TRUE)
      par(lend=2)
      legend(xleg, yleg, txt,  bty="o", cex=LEGcex, inset=LEGinset, seg.len=SEGlen, box.col=NA,
          bg="white", lwd=lwd, lty=lty, col=col, pch=pch, pt.bg=pt.bg, pt.lwd=pt.lwd, pt.cex=pt.cex)
      par(opts)
    }
  }

  if(verbose) message("(", i <- i + 1, ") solving for Y predictions along the diagonal inversion")
  # solve for Y, like missing record estimation using the best available information on the Y
  # distribution's parameters (i.e., the MOMENTS)
  # Back us off from -Inf and +Inf to NaN results when a probability = 0 | 1, to ensure numerical
    tmf <- ecy;  tmf[tmf <= lo] <- lo; tmf[tmf >= hi] <- hi
    Ysec0  <- lmomco::par2qua(tmf,  ypara,  paracheck=FALSE)
    tmf <- ecoy; tmf[tmf <= lo] <- lo; tmf[tmf >= hi] <- hi
    Ysec0o <- lmomco::par2qua(tmf,  ypara,  paracheck=FALSE)
  if(USE_PARA_COP) {
    tmf <- dtty; tmf[tmf <= lo] <- lo; tmf[tmf >= hi] <- hi
    Ysdtt0 <- lmomco::par2qua(tmf,  ypara,  paracheck=FALSE)
  }

  # The suppressWarnings() because of this
  #   # Warning message:  # when ties exist in the incoming to approx().
  # In regularize.values(x, y, ties, missing(ties), na.rm = na.rm):collapsing to unique 'x' values
  #suppressWarnings( Ysdd <- approx(lmomco::par2qua(dd, xpara), Ysdd0, xout=sortX, rule=2)$y )

  if(any(! is.na(xout))) {  # BEGIN BLOCK OF ORGANIC PREDICTIONS BY COPULA
    # Back us off from -Inf and +Inf to NaN results when a probability = 0 | 1, to ensure numerical
    # return on the approx() application.
    if(verbose) message("(", i <- i + 1, ") solving for Y predictions along given Xs and given (if any) Xouts")
      tmf <- ec;  tmf[tmf <= lo]  <- lo; tmf[tmf >= hi] <- hi
      suppressWarnings( Ysec      <- approx(lmomco::par2qua(tmf, xpara, paracheck=FALSE), Ysec0, xout=sortX,  rule=2)$y )
      suppressWarnings( Ysec_out  <- approx(lmomco::par2qua(tmf, xpara, paracheck=FALSE), Ysec0, xout=xout,   rule=2)$y )
      tmf <- eco; tmf[tmf <= lo]  <- lo; tmf[tmf >= hi] <- hi
      suppressWarnings( Yseco     <- approx(lmomco::par2qua(tmf, xpara, paracheck=FALSE), Ysec0o, xout=sortX, rule=2)$y )
      suppressWarnings( Ysec_outo <- approx(lmomco::par2qua(tmf, xpara, paracheck=FALSE), Ysec0o, xout=xout,  rule=2)$y )
    if(USE_PARA_COP) {
      tmf <- dtt; tmf[tmf <= lo]  <- lo; tmf[tmf >= hi] <- hi
      suppressWarnings( Ysdtt     <- approx(lmomco::par2qua(tmf, xpara, paracheck=FALSE), Ysdtt0, xout=sortX, rule=2)$y )
      suppressWarnings( Ysdtt_out <- approx(lmomco::par2qua(tmf, xpara, paracheck=FALSE), Ysdtt0, xout=xout,  rule=2)$y )
      names(Ysdtt)     <- NULL
      names(Ysdtt_out) <- NULL
    }

    names(Ysec)      <- NULL
    names(Yseco)     <- NULL
    names(Ysec_out)  <- NULL
    names(Ysec_outo) <- NULL
  } # END BLOCK OF ORGANIC PREDICTIONS BY COPULA

  if(plotxy) { # real-world unit plotting
        myxlim <- range(c(xp, lmomco::par2qua(FFedge, xpara, paracheck=FALSE)),      na.rm=TRUE)
        myylim <- range(c(yp, lmomco::par2qua(FFedge, ypara, paracheck=FALSE)),      na.rm=TRUE)
    if(limout) {
        myxlim <- range(c(myxlim, xout),                                             na.rm=TRUE)
        myylim <- range(c(myylim, Yloc, Yloc_out, Ysec, Yseco, Ysec_out, Ysec_outo), na.rm=TRUE)
      if(USE_PARA_COP) {
        myylim <- range(c(myylim, Ysdtt, Ysdtt_out), na.rm=TRUE)
      }
    }
    if(! is.null(xlim)) {
      if(length(xlim) == 2 & ! any(is.na(xlim))) {
        myxlim <- xlim
      } else {
        myxlim <- range(c(xlim, myxlim), na.rm=TRUE)
      }
    }
    if(! is.null(ylim)) {
      if(length(ylim) == 2 & ! any(is.na(ylim))) {
        myylim <- ylim
      } else {
        myylim <- range(c(ylim, myylim), na.rm=TRUE)
      }
    }
    if(! addxy) {
      if(verbose) message("(", i <- i + 1, ") plotting the analyses on the (X,Y) domain")
      plot(xp, yp, type="n", xlim=myxlim, ylim=myylim, las=1, mgp=c(2.2, 0.75, 0),
                   xlab="X OF DISTRIBUTION", ylab="Y OF DISTRIBUTION")
      mtext(titlexy, side=3, line=1, font=2, cex=titlecex)
      axis(3, axTicks(1), labels=FALSE, lwd=NA, lwd.ticks=1)
      axis(4, axTicks(2), labels=FALSE, lwd=NA, lwd.ticks=1)
    }
    if(rugxy) {
      suppressWarnings( rug(thex, ticksize=0.02, side=1, col=RUGcol, lwd=ruglwd) )
      suppressWarnings( rug(they, ticksize=0.02, side=2, col=RUGcol, lwd=ruglwd) )
      # Warning message: In rug(thex, ticksize = 0.02, side = 1, col = RUGcol, lwd = ruglwd) :
      #                  some values will be clipped
    }
      points(xp, yp, pch=DATpch, cex=DATcex, col=DATcol, bg=DATbg, lwd=DATptlwd)
      abline(locpara$loc_lmr[1], locpara$loc_lmr[2], lty=LOCUlty,  lwd=LOCUlwd, col=LOCUcol, lend=2)
      abline(locpair$loc_lmr[1], locpair$loc_lmr[2], lty=LOClty,   lwd=LOClwd,  col=LOCcol,  lend=2)
    if(USE_PARA_COP) {
      tmf <- dtt; tmf[tmf <= lo] <- lo; tmf[tmf >= hi] <- hi # assurance policy
      lines(lmomco::par2qua(tmf, xpara, paracheck=FALSE), Ysdtt0, lty=COPlty, lwd=COPlwd, col=COPcol, lend=2)
    }
      tmf <- ec; tmf[tmf <= lo] <- lo; tmf[tmf >= hi] <- hi # assurance policy
      lines(lmomco::par2qua(tmf,  xpara, paracheck=FALSE), Ysec0,     lwd=DIAlwd,  col=DIAcol, lend=2)
      points(sortX,    Yseco,   pch=PCHcty, cex=PCHcex, col=PCHcol, lwd=PCHlwd,   bg=PCHbg)
      points(sortX,    Ysec,    pch=DIApch, cex=DIAcex, col=DIAcol, lwd=DIAptlwd, bg=DIAbg)
    if(! addxy & any(! is.na(xout))) { # to remain consistent with no messaging if plot() not called
      if(verbose) message("(", i <- i + 1, ") drawing the organic predictions in (X,Y) domain")
      points(xout,   Yloc_out,  pch=LOCpch, cex=LOCcex, col=OUTcol, lwd=LOCptlwd, bg=LOCbg)
      points(xout,   Ylocu_out, pch=LOCpch, cex=LOCcex, col=OUTcol, lwd=LOCptlwd, bg=LOCbg)
      points(xout,   Ysec_out,  pch=OUTpch, cex=OUTcex, col=OUTcol, lwd=OUTptlwd, bg=LOCbg)
      if(USE_PARA_COP) {
        points(xout, Ysdtt_out, pch=COPpch, cex=COPcex, col=COPcol, lwd=COPptlwd, bg=COPbg)
      }
    }

    if(autoleg) {
      lwd    <- c(LOClwd, LOCUlwd, COPlwd, DIAlwd,       NA,     NA,       NA,      NA,       NA,        NA)
      lty    <- c(LOClty, LOCUlty, COPlty, DIAlty,       NA,     NA,       NA,      NA,       NA,        NA)
      col    <- c(LOCcol, LOCUcol, COPcol, DIAcol,   DIAcol, PCHcol,   DATcol,   OUTcol,   OUTcol,   COPcol)
      pch    <- c(NA,          NA,     NA,     NA,   DIApch, PCHcty,   DATpch,   LOCpch,   OUTpch,   COPpch)
      pt.bg  <- c(NA,          NA,     NA,     NA,    DIAbg,  PCHbg,    DATbg,    OUTbg,    OUTbg,    COPbg)
      pt.lwd <- c(NA,          NA,     NA,     NA, DIAptlwd, PCHlwd, DATptlwd, LOCptlwd, OUTptlwd, COPptlwd)
      pt.cex <- c(NA,          NA,     NA,     NA,   DIAcex, PCHcex,   DATcex,   LOCcex,   OUTcex,   COPcex)
      txt    <- c("Line of organic correlation (LOC) using paired data only and fit using linear moments",
                  "PARA-LOC fit by the linear moments of the marginal parametric distributions",
                  "Organic correlation by parametric asymmetric Plackett copula",
                  "Organic correlation by copula diagonal inverse (Kumaraswamy smooth)",
                  "Coordinates of organic correlation by empirical copula diagonal inverse",
                   ctyTXTxy,
                  paste0("Paired data points (n=", prettyNum(n, big.mark=","), ") between X and Y"),
                  "Either LOC predicted Y for requested X on LOC or Ultra-LOC as plotting shows",
                  "Organic correlation predicted Y for requested X through empirical copula",
                  "Organic correlation predicted Y for requested X through parametic copula")
      if(is.null(xout) | length(xout[! is.na(xout)]) == 0) {
        nix <- -c(8, 9, 10)
        txt    <- txt[   nix]
        lwd    <- lwd[   nix]
        lty    <- lty[   nix]
        col    <- col[   nix]
        pch    <- pch[   nix]
        pt.bg  <- pt.bg[ nix]
        pt.lwd <- pt.lwd[nix]
        pt.cex <- pt.cex[nix]
      }
      if(! USE_PARA_COP) {
        nix <- -c(3, length(txt))
        txt    <- txt[   nix]
        lwd    <- lwd[   nix]
        lty    <- lty[   nix]
        col    <- col[   nix]
        pch    <- pch[   nix]
        pt.bg  <- pt.bg[ nix]
        pt.lwd <- pt.lwd[nix]
        pt.cex <- pt.cex[nix]
      }
      if(! kumaraswamy) {
        nix <- -grep("Kumaraswamy", txt)
        txt    <- txt[   nix]
        lwd    <- lwd[   nix]
        lty    <- lty[   nix]
        col    <- col[   nix]
        pch    <- pch[   nix]
        pt.bg  <- pt.bg[ nix]
        pt.lwd <- pt.lwd[nix]
        pt.cex <- pt.cex[nix]
      }
      opts <- par(no.readonly=TRUE)
      par(lend=2)
      legend(xleg, yleg, txt, bty="o", cex=LEGcex, inset=LEGinset, seg.len=SEGlen, box.col=NA,
          bg="white", lwd=lwd, lty=lty, col=col, pch=pch, pt.bg=pt.bg, pt.lwd=pt.lwd, pt.cex=pt.cex)
      par(opts)
    }
  }

  df1 <- data.frame( xout=xout, locpair =Yloc_out, locpara     =Ylocu_out,
                                bicoploc=Ysec_out, bicoploc_emp=Ysec_outo )
  df2 <- data.frame( jtprob=ff, uv=ec, uv_emp=eco)
  if(USE_PARA_COP) {
    df1$bicoploc_cop <- Ysdtt_out
    df2$uv_cop       <-   dtt
  }
  df1 <- round(df1, digits=locdigits)
  df2 <- round(df1, digits=8)

        dtypes  <- c( dtypex,   dtypey )
  names(dtypes) <- c("dtypex", "dtypey")
  faqs <- c(n, n_thex, n_they)
  names(faqs) <- c("paired_sample_size", "nonmissing_x_sample_size", "nonmissing_y_sample_size")

  locsols <- list(locpair=locpair, locpara=locpara)

  zz <- list(organic=df1, locsols=locsols, xpara=xpara, ypara=ypara, dtypes=dtypes,
                          faqs=faqs, faqscop=faqscop, diag=df2)
  return(zz)
}

#set.seed(4); nsim <- 50
#X  <- rnorm(nsim, mean=3, sd=0.6)
#Y  <- rnorm(nsim, mean=0, sd=0.2)
#zz <- bicoploc(X,Y, xout=c(2.5, 3.5, 4), dtypex="nor", dtypey="gov")
#
#set.seed(4); nsim <- 50
#X  <- rexp(nsim, rate=3)
#Y  <- 0.7*X + rnorm(nsim, mean=0, sd=0.2)
#zz <- bicoploc(X,Y, xout=c(2.5, 3.5, 4), dtypex="exp", dtypey="gev")

"rhobevCOP" <-
function(cop=NULL, para=NULL, as.sample=FALSE,
                              brute=FALSE, delta=0.002, ...) {

   if(as.sample) {
      if(is.null(para)) {
         warning("Sample Bivariate Rho desired but para is NULL, ",
                 "returning NULL")
         return(NULL)
      }
      if(length(names(para)) != 2) {
        warning("para argument must be data.frame having only two columns, ",
                "returning NULL")
        return(NULL)
      }

      return(mean(-log(para[,1]) * -log(para[,2])) - 1 )
   }

   if(brute) {
      ws <- seq(.Machine$double.eps, 1-.Machine$double.eps, delta)
      sum <- sum(sapply(ws, function(w) {
              (-log(cop(exp(-w),exp(w-1), para=para, ...)))^(-2) }))
      rhoE <- sum*delta - 1
      return(rhoE)
   }

   myint <- NULL
   try(myint <- integrate(function(w) {
                  (-log(cop(exp(-w),exp(w-1), para=para, ...)))^(-2) }, 0, 1) )
   rhoE <- ifelse(is.null(myint), NA, myint$value - 1)
   return(rhoE)
}

"EMPIRsim" <-
function(n=100, empgrid=NULL, kumaraswamy=FALSE, na.rm=TRUE, keept=FALSE,
                graphics=TRUE, ploton=TRUE, points=TRUE, snv=FALSE,
                infsnv.rm=TRUE, trapinfsnv=.Machine$double.eps, ...) {
  if(! graphics) {
     ploton <- FALSE
     points <- FALSE
  }
  empinv <- EMPIRgridderinv(empgrid=empgrid, kumaraswamy=kumaraswamy)
  rows <- as.numeric(attributes(empinv)$rownames)
  ix <- 1:length(rows)
  cols <- attributes(empinv)$colnames
  U <- vector(mode="numeric", length=n); V <- U
  Ts <- runif(n)
  for(i in 1:n) {
    u <- runif(1); t <- Ts[i]
    ix.needed1 <- max(ix[rows <= u])
    ix.needed2 <- min(ix[rows >= u])
    if(ix.needed1 == 1) ix.needed1 <- 2
    if(ix.needed1 == ix.needed2) {
      v.available <- empinv[ix.needed1,]
      v <- approx(cols, y=v.available, xout=t, rule=2)$y
    } else {
      v.available1 <- empinv[ix.needed1,]
      v1 <- approx(cols, y=v.available1, xout=t, rule=2)$y
      v.available2 <- empinv[ix.needed2,]
      v2 <- approx(cols, y=v.available2, xout=t, rule=2)$y
      w1 <- u - rows[ix.needed1]
      w2 <- rows[ix.needed2] - u
      tw <- 1/w1 + 1/w2
      v <- (v1/w1 + v2/w2)/tw
    }
    U[i] <- u; V[i] <- v
  }

  # Because z is a data.frame, it must be assigned within the ifelse()
  ifelse(keept, z <- data.frame(U=U, V=V, T=Ts), z <- data.frame(U=U, V=V))
  if(na.rm) {
     z <- z[complete.cases(z), ]
     m <- length(z[,1])
     if(m != n) {
        warning("user requested n=",n," simulations but only m=",m,
                " could be made without NA from EMPIRgridderinv")
        row.names(z) <- NULL # reset the rows to "1:m"
     }
  }
  if(snv) {
     if(infsnv.rm) {
        z <- z[z$U != 0, ]; z <- z[z$U != 0, ]
        z <- z[z$V != 1, ]; z <- z[z$V != 1, ]
        row.names(z) <- NULL
     } else if(trapinfsnv) {
        z$U[z$U == 0] <-   trapinfsnv; z$V[z$V == 0] <-   trapinfsnv
        z$U[z$U == 1] <- 1-trapinfsnv; z$V[z$V == 1] <- 1-trapinfsnv
     }
     z$U <- qnorm(z$U)
     z$V <- qnorm(z$V)
  }
  if(ploton) {
     if(snv) {
        plot(z$U, z$V, type="n",
             xlab="STANDARD NORMAL VARIATE OF U", ylab="STANDARD NORMAL SCORE VARIATE OF U")
     } else {
        plot(NA, NA, type="n", xlim=c(0,1), ylim=c(0,1),
             xlab="U, NONEXCEEDANCE PROBABILITY", ylab="V, NONEXCEEDANCE PROBABILITY")
     }
  }
  if(points & ! is.null(dev.list())) points(z$U, z$V,...)

  return(z)
}

"GLcop" <- function(u,v, para=NULL, ...) {
  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
    warning("length u = ",length(u), " and length v = ",length(v))
    warning("longer object length is not a multiple of shorter object length, no recycling")
    return(NA)
  }
  # The extra hassle of vectorization made here is to handle situations
  # in which nested integrals are used where uneven vectors can be passed
  if(length(u) == 1) {
     u <- rep(u, length(v))
  } else if(length(v) == 1) {
     v <- rep(v, length(u))
  }

  if(length(para) == 1) { # Joe (2014, p. 174)
    if(para[1] < 0) {
      warning("parameter must be 0 <= theta < infinity [well numerically 100]")
      return(NA)
    }
    if(para[1] < .Machine$double.eps) return(P(u,v))
    #x <- seq(80,110,by=.1)
    #y <- sapply(x, function(t) rhoCOP(cop=GLcop, para=t))
    #plot(x,y); lines(c(94,94),c(0,1))
    if(para[1] > 94) return(M(u,v)) # 94 determined by rhoCOP experiments
    para[1] <- -para[1]
    suppressWarnings(cop <- u * v * exp( ( (-log(u))^para[1] +
                                           (-log(v))^para[1] )^(1/para[1]) ))
    #cop[is.nan(cop)] <- 0
    para[1] <- -para[1]  # Note switch back to original sign convention
    if(para[1] > 50) { #rint(c(u,v,cop))
      wnt <- u > 0.99 & v > 0.99
      if(any(wnt)) cop[wnt] <- M(u[wnt],v[wnt])
    } else if(para[1] > 35) {
      wnt <- u > 0.995 & v > 0.995
      if(any(wnt)) cop[wnt] <- M(u[wnt],v[wnt])
    } else if(para[1] > 25) {
      wnt <- u > 0.999 & v > 0.999
      if(any(wnt)) cop[wnt] <- M(u[wnt],v[wnt])
    } else if(para[1] > 15) {
      wnt <- u > 0.9998 & v > 0.9998
      if(any(wnt)) cop[wnt] <- M(u[wnt],v[wnt])
    }
    return(cop) ################ END GALAMBOS
  } else if(length(para) == 2) { # Joe (2014, p. 198) COP_LEV
    if(para[1] < 0) {
      warning("theta must be 0 < theta < infinity")
      return(NA)
    }
    if(para[2] < 0) {
      warning("delta must be 0 <= delta < infinity")
      return(NA)
    }
    if(para[1] < .Machine$double.eps^0.5) return(P(u,v))
    # Min-Stable Bivariate Exponential Family, a two-parameter Galambos
    if(para[2] > exp(4.75049-0.89850*log(para[1])-0.03948*log(para[1])^2)) {
       return(M(u,v))
    }
    Afunc <- function(x,y,t,d) { # Joe (2014, p. 198)
      x + y - (x^-t + y^-t - (x^(t*d) + y^(t*d))^(-1/d))^(-1/t)
    }
    cop <- exp(-Afunc(-log(u), -log(v), para[1],para[2])) # Joe (2014, p. 198)
    cop[is.nan(cop)] <- 0 # Does this need to be more sophisticated
    # in the case for the Gamma Power Mixture of Galambos?????
    return(cop) ################ END COP_LEV
  } else if(length(para) == 3) { # Joe (2014, p. 197)
    if(para[1] < 0) {
      warning("theta must be 0 < theta < infinity")
      return(NA)
    }
    if(para[2] < 0) {
      warning("delta must be 0 < delta < infinity")
      return(NA)
    }
    # Note para[3] is not used, just a triggering mechanism.
    # Testing indicates that these tests might not be needed.
    if(para[1] <= .Machine$double.eps^0.25) { # test needed though?
       # Galambos copula of para[2] results for small para[1]
       return(GLcop(u,v, para=para[2]))
    }
    if(para[2] <= .Machine$double.eps^0.25) { # test needed though?
       # MTCJ copula of para[1] for small para[2]
       return((u^-para[1] + v^-para[1] -1)^(-1/para[1]))
    }
    # This next test is very complicated. A nested theta=1:100, delta=1:100
    # was done and rhoCOP run on the combinations as the parameters get
    # large, failures occur in that rhoCOP starts trending down from 0.999..
    # into negatives. WHA then found the curve of maximum rhos within the
    # matrix and isolated the maximums and then ran a log-log regression
    # that is shown below. For parameters to the upper left of this regression
    # the M() copula is returned, this avoid numerical problems.
    # This approach is confirmed in part by watching simCOP at large
    # sample sizes and noting the occasional spikes from near M().
    if(para[2] > exp(4.88376-0.69985*log(para[1])-0.05946*log(para[1])^2)) {
       return(M(u,v))
    }
    # To do, a small para[1] and large para[2] can still leak through and
    # the numerical errors occur deep in the upper right corner. M() should be
    # triggered, but how?

    # Continuing ....
    para[1] <- -para[1]; para[2] <- -para[2]
    uo <- u; vo <- v
    u <- u^para[1]; v <- v^para[1]
    cop <- (u + v - 1 - ( (u-1)^para[2] +
                          (v-1)^para[2] )^(1/para[2])
           )^(1/para[1])
    if(any(is.nan(cop))) { # Some degenerate behavior away from
      # u=0 or v=0 occurs in the lower corner, a leg will kick off from M()
      # So here, we are intercepting the kick via leaving the cop as NaN
      # and then resetting to M() (the minimum)
      cop[is.nan(cop)] <- min(c(uo[is.nan(cop)], vo[is.nan(cop)]))
      # This is.nan treatment differs from that for the Galambos in the
      # length(para[1]) [the regular 1-parameter Galambos]
    }
    # The various "large" para[2] checks below were developed after
    # the log-log regression shown above. Still have some numerical
    # issues right up near the upper-left corner. These tests then
    # copied to the Galambos too during testing. n=10,000 sample sizes
    # These are partly protecting from degeneracy at the boundary as
    # mentioned by Joe (2014, p. 198).
    para[2] <- -para[2] # Note switch back to original sign convention
    if(para[2] > 50) {
      wnt <- uo > 0.99 & vo > 0.99
      if(any(wnt)) cop[wnt] <- M(uo[wnt],vo[wnt])
    } else if(para[2] > 35) {
      wnt <- uo > 0.995 & vo > 0.995
      if(any(wnt)) cop[wnt] <- M(uo[wnt],vo[wnt])
    } else if(para[2] > 25) {
      wnt <- uo > 0.999 & vo > 0.999;
      if(any(wnt)) cop[wnt] <- M(uo[wnt],vo[wnt])
    } else if(para[1] > 15) {
      wnt <- u > 0.9998 & v > 0.9998
      if(any(wnt)) cop[wnt] <- M(uo[wnt],vo[wnt])
    }
    return(cop) ################ END GAMMA POWER MIXTURE
  } else {
    stop("only one, two, or two with third trigger parameters in GLcop supported")
  }
}

"JOcopBB4" <- function(u,v, para=NULL, ...) {
  if(length(para) != 2) {
    warning("GLPMcop/JOcopBB4 through GLcop requires two parameters, ",
            "third trigger is automatic")
    return(NULL)
  }
  GLcop(u,v, para=c(para,1), ...)
}

"GLPMcop" <- function(u,v, para=NULL, ...) {
  if(length(para) != 2) {
    warning("GLPMcop/JOcopBB4 through GLcop requires two parameters, ",
            "third trigger is automatic")
    return(NULL)
  }
  GLcop(u,v, para=c(para,1), ...)
}

"GLEVcop" <- function(u,v, para=NULL, ...) {
  if(length(para) != 2) {
    warning("Lower extreme value limit through GLcop requires two parameters")
    return(NULL)
  }
  GLcop(u,v, para=para, ...)
}

############### LOWER EXTREME VALUE LIMIT OF THE GALAMBOS ########
#h <- matrix(nrow =length(1:100), ncol=length(1:100))
#for(i in 1:100) {
# for(j in 1:100) {
#   message(i, " ", j)
#   h[i,j] <- rhoCOP(cop=GLcop, para=c(i,j))
# }
#}
#contour(h, levels=c(0.999))
#image(h)
#k <- 0
#I <- J <- NA
#for(i in 1:100) {
# here <- (1:100)[h[i,] == max(h[i,], na.rm=TRUE)]
# here <- here[! is.na(here)]
# if(length(here) == 0) next
# print(c(i,here, i+here))
# I[k] <- i; J[k] <- here; k <- k + 1
# points(i/100, here/100, col=2, cex=0.5)
#}
#II <- seq(0.1,200, by=.1)
#plot(c(.1,1000), c(0.1,1000), type="n", log="xy")
#points(I,J)
#LM <- lm(log(J)~log(I)+I(log(I)^2))
#points(I, exp(fitted.values(LM)), col=3)
#lines(II, exp(4.75049-0.89850*log(II)-0.03948*log(II)^2), col=4)
##################################################################


############### GAMMA POWER MIXTURE OF THE GALAMBOS ##############
# The code below was used to make the regression equation
# implemented for joint protection on large parameters for the

#h <- matrix(nrow =length(1:100), ncol=length(1:100))
#for(i in 1:100) {
# for(j in 1:100) {
#   message(i, " ", j)
#   h[i,j] <- rhoCOP(cop=GLcop, para=c(i,j,1))
# }
#}
#contour(h, nlevels=20)
#contour(h, levels=c(0.999))
#diag(h) # looking for 0.999
#(1:100)[diag(h) == max(diag(h), na.rm=TRUE)]
#UV <- simCOP(1000, cop=GLcop, para=c(13,13))
#contour(h, levels=c(0.999))
#k <- 0
#I <- J <- NA
#for(i in 1:100) {
# here <- (1:100)[h[i,] == max(h[i,], na.rm=TRUE)]
# here <- here[! is.na(here)]
# if(length(here) == 0) next
# print(c(i,here, i+here))
# I[k] <- i; J[k] <- here; k <- k + 1
# points(i/100, here/100, col=2, cex=0.5)
#}
#for(i in 1:100) {
# here <- (1:100)[h[,i] == max(h[,i], na.rm=TRUE)]
# here <- here[! is.na(here)]
# if(length(here) == 0) next
# print(c(i,here, i+here))
# I[k] <- i; J[k] <- here; k <- k + 1
# points(i/100, here/100, col=3)
#}
#II <- seq(0.1,200, by=.1)
#plot(c(.1,1000), c(0.1,1000), type="n", log="xy")
#points(I,J)
#LM <- lm(log(J)~log(I)+I(log(I)^2))
#points(I, exp(fitted.values(LM)), col=3)
#lines(II, exp(4.88376-0.69985*log(II)-0.05946*log(II)^2), col=4)
##################################################################

glueCOP <- function(u,v, para=NULL, ...) {
  if(length(para$cop1) == 0) {
    warning("require Copula1 cop1 to be declared")
    return(NULL)
  }
  if(length(para$cop2) == 0) {
    warning("require Copula2 cop2 to be declared")
    return(NULL)
  }
  if(length(para$para1) == 0) {
    warning("require Copula1 parameters para1 to be declared")
    return(NULL)
  }
  if(length(para$para2) == 0) {
    warning("require Copula2 parameters para2 to be declared")
    return(NULL)
  }
  if(is.na(para$glue)) {
    warning("glue needed 0 <= glue <= 1")
    return(NULL)
  }

  # Operationally the following tests are needed although, they are
  # again used inside the COP() call. The compositing functions don't
  # use this tests, but they are here because of the why that
  # u, glue, and v can come in combinations of vector, as in rhoCOP().
  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
     warning("length u = ", length(u), " and length v = ", length(v))
     warning("longer object length is not a multiple of shorter object length, no recycling")
     return(NA)
  }
  if(length(u) == 1) {
     u <- rep(u, length(v))
  } else if (length(v) == 1) {
     v <- rep(v, length(u))
  }
  glue <- para$glue
  cop <- sapply(1:length(u), function(i) {
       if(u[i] <= glue) { glue*COP( u[i]/glue, v[i],
                               cop=para$cop1, para=para$para1, ...)
       } else {       (1-glue)*COP((u[i]-glue)/(1-glue), v[i],
                               cop=para$cop2, para=para$para2, ...) +
                               glue*v[i]
       } })
  return(cop)
}

"composite2COP" <-
function(u, v, para, ...) {
  if(is.null(para$alpha)) {
    warning("no alpha parameter given")
    return(NULL)
  }
  if(is.null(para$beta)) {
    warning("no beta parameter given")
    return(NULL)
  }
  alpha <- para$alpha; alpha.p <- 1 - alpha
  beta  <- para$beta;   beta.p <- 1 - beta
  return(COP(cop=para$cop1, u^alpha,   v^beta,   para=para$para1, ...) *
         COP(cop=para$cop2, u^alpha.p, v^beta.p, para=para$para2, ...))
}

"khoudraji2COP" <-
function(u, v, para, ...) {
  if(is.null(para$alpha)) {
    warning("no alpha parameter given")
    return(NULL)
  }
  if(is.null(para$beta)) {
    warning("no beta parameter given")
    return(NULL)
  }
  alpha <- para$alpha; alpha.p <- 1 - alpha
  beta  <- para$beta;   beta.p <- 1 - beta
  return(COP(cop=para$cop1, u^alpha,   v^beta,   para=para$para1, ...) *
         COP(cop=para$cop2, u^alpha.p, v^beta.p, para=para$para2, ...))
}

"diagCOPinv" <-
function(f, cop=NULL, para=NULL, interval=NULL,
                      silent=TRUE, verbose=FALSE, tol=.Machine$double.eps/10, ...) {
   diagCOPatf(f, cop=cop, para=para, silent=silent, verbose=verbose,
                 interval=interval, tol=tol, ...)
}

"diagCOPatf" <-
 function(f, cop=NULL, para=NULL, interval=NULL,
                       silent=TRUE, verbose=FALSE, tol=.Machine$double.eps/10, ...) {
    if(verbose & length(f) > 1) {
       warning("f argument is not a single value, only first value will be used, ",
               "because verbose return needed")
       f <- f[1]
    }
    if(is.null(cop)) {
       warning("cop argument is NULL, returning NULL")
       return(NULL)
    }

    intv <- c(.Machine$double.eps, 1-.Machine$double.eps)
    if(! is.null(interval)) intv <- interval
    if(intv[1] < 0) {
       warning("invalid first value for the interval, returning NULL")
       return(NULL)
    }
    if(intv[2] > 1) {
       warning("invalid second value for the interval, returning NULL")
       return(NULL)
    }

    "afunc" <- function(uv, aF=NA, ...) (aF - cop(uv, uv, para=para, ...))

    DatF <- sapply(f, function(af) {
                   if(af == 0) return(0)
                   if(af == 1) return(1)
                   rt <- NULL
                   try(rt <- uniroot(afunc, interval=intv, tol=tol, aF=af, ...), silent=silent)
                   if(is.null(rt)) {
                      warning("could not root for the diagonal inverse")
                      return(NA)
                   }
                   if(verbose) return(rt)
                   return(rt$root)
            })
    return(DatF)
 }

blomatrixCOPdec <-
function(cop=NULL, para=NULL, as.sample=FALSE, as.blomCOPss=TRUE,
                  ctype=c("weibull", "hazen", "1/n",
                          "bernstein", "checkerboard"), ...) {
  t <- c(0.10, 0.50, 0.90)
  if(as.sample) {
    ctype <- match.arg(ctype)
    if(is.null(para)) {
      warning("Sample Blomqvist's Beta desired but para is NULL, returning NULL")
      return(NULL)
    }
    if(length(names(para)) != 2) {
      warning("para argument must be data.frame having only two columns, returning NULL")
      return(NULL)
    }
    if(as.blomCOPss) {
      blom <- matrix(c(
          blomCOPss(cop=EMPIRcop, para=para, uu=c(t[1], t[1]), vv=c(t[1], t[1]), ...),
          blomCOPss(cop=EMPIRcop, para=para, uu=c(t[2], t[1]), vv=c(t[2], t[1]), ...),
          blomCOPss(cop=EMPIRcop, para=para, uu=c(t[3], t[1]), vv=c(t[3], t[1]), ...),
          blomCOPss(cop=EMPIRcop, para=para, uu=c(t[1], t[2]), vv=c(t[1], t[2]), ...),
          blomCOPss(cop=EMPIRcop, para=para, uu=c(t[2], t[2]), vv=c(t[2], t[2]), ...),
          blomCOPss(cop=EMPIRcop, para=para, uu=c(t[3], t[2]), vv=c(t[3], t[2]), ...),
          blomCOPss(cop=EMPIRcop, para=para, uu=c(t[1], t[1]), vv=c(t[1], t[3]), ...),
          blomCOPss(cop=EMPIRcop, para=para, uu=c(t[2], t[1]), vv=c(t[2], t[3]), ...),
          blomCOPss(cop=EMPIRcop, para=para, uu=c(t[3], t[1]), vv=c(t[3], t[3]), ...)
      ), ncol=3)
      colnames(blom) <- paste0("U|V=", c("0.10", "0.50", "0.90"))
      rownames(blom) <- rev(colnames(blom))
      return(blom)
    } else {
      A <- 1 / P(t, t[3]); B <- 1 / P(t, t[2]); C <- 1 / P(t, t[1])
      blom <- matrix(c(EMPIRcop(t, t[3], para=para, ctype=ctype, ...) * A - 1,
                       EMPIRcop(t, t[2], para=para, ctype=ctype, ...) * B - 1,
                       EMPIRcop(t, t[1], para=para, ctype=ctype, ...) * C - 1), ncol=3)
      colnames(blom) <- paste0("U|V=", c("0.10", "0.50", "0.90"))
      rownames(blom) <- rev(colnames(blom))
      return(blom)
    }
  } else {
    if(is.null(cop)) {
      warning("must have copula argument specified, returning NULL")
      return(NULL)
    }
    if(as.blomCOPss) {
      blom <- matrix(c(
          blomCOPss(cop=cop, para=para, uu=c(t[1], t[1]), vv=c(t[1], t[1]), ...),
          blomCOPss(cop=cop, para=para, uu=c(t[2], t[1]), vv=c(t[2], t[1]), ...),
          blomCOPss(cop=cop, para=para, uu=c(t[3], t[1]), vv=c(t[3], t[1]), ...),
          blomCOPss(cop=cop, para=para, uu=c(t[1], t[2]), vv=c(t[1], t[2]), ...),
          blomCOPss(cop=cop, para=para, uu=c(t[2], t[2]), vv=c(t[2], t[2]), ...),
          blomCOPss(cop=cop, para=para, uu=c(t[3], t[2]), vv=c(t[3], t[2]), ...),
          blomCOPss(cop=cop, para=para, uu=c(t[1], t[1]), vv=c(t[1], t[3]), ...),
          blomCOPss(cop=cop, para=para, uu=c(t[2], t[1]), vv=c(t[2], t[3]), ...),
          blomCOPss(cop=cop, para=para, uu=c(t[3], t[1]), vv=c(t[3], t[3]), ...)
      ), ncol=3)
      colnames(blom) <- paste0("U|V=", c("0.10", "0.50", "0.90"))
      rownames(blom) <- rev(colnames(blom))
      return(blom)
    } else {
      A <- 1 / P(t, t[3]); B <- 1 / P(t, t[2]); C <- 1 / P(t, t[1])
      blom <- matrix(c(cop(t, t[3], para=para, ...) * A - 1,
                       cop(t, t[2], para=para, ...) * B - 1,
                       cop(t, t[1], para=para, ...) * C - 1), ncol=3)
      colnames(blom) <- paste0("U|V=", c("0.10", "0.50", "0.90"))
      rownames(blom) <- rev(colnames(blom))
      return(blom)
    }
  }
}


blomatrixCOPiqr <-
function(cop=NULL, para=NULL, as.sample=FALSE, as.blomCOPss=TRUE,
                  ctype=c("weibull", "hazen", "1/n",
                          "bernstein", "checkerboard"), ...) {
  t <- c(0.25, 0.50, 0.75)
  if(as.sample) {
    ctype <- match.arg(ctype)
    if(is.null(para)) {
      warning("Sample Blomqvist's Beta desired but para is NULL, returning NULL")
      return(NULL)
    }
    if(length(names(para)) != 2) {
      warning("para argument must be data.frame having only two columns, returning NULL")
      return(NULL)
    }
    if(as.blomCOPss) {
           blom <- matrix(c(
          blomCOPss(cop=EMPIRcop, para=para, uu=c(t[1], t[1]), vv=c(t[1], t[1]), ...),
          blomCOPss(cop=EMPIRcop, para=para, uu=c(t[2], t[1]), vv=c(t[2], t[1]), ...),
          blomCOPss(cop=EMPIRcop, para=para, uu=c(t[3], t[1]), vv=c(t[3], t[1]), ...),
          blomCOPss(cop=EMPIRcop, para=para, uu=c(t[1], t[2]), vv=c(t[1], t[2]), ...),
          blomCOPss(cop=EMPIRcop, para=para, uu=c(t[2], t[2]), vv=c(t[2], t[2]), ...),
          blomCOPss(cop=EMPIRcop, para=para, uu=c(t[3], t[2]), vv=c(t[3], t[2]), ...),
          blomCOPss(cop=EMPIRcop, para=para, uu=c(t[1], t[1]), vv=c(t[1], t[3]), ...),
          blomCOPss(cop=EMPIRcop, para=para, uu=c(t[2], t[1]), vv=c(t[2], t[3]), ...),
          blomCOPss(cop=EMPIRcop, para=para, uu=c(t[3], t[1]), vv=c(t[3], t[3]), ...)
      ), ncol=3)
      colnames(blom) <- paste0("U|V=", c("0.25", "0.50", "0.75"))
      rownames(blom) <- rev(colnames(blom))
      return(blom)
    } else {
      #A <- c(16/3, 16/6, 16/9)
      #B <- c(16/2, 16/4, 16/6)
      #C <- c(16/1, 16/2, 16/3)
      A <- 1 / P(t, t[3]); B <- 1 / P(t, t[2]); C <- 1 / P(t, t[1])
      blom <- matrix(c(EMPIRcop(t, t[3], para=para, ctype=ctype, ...) * A - 1,
                       EMPIRcop(t, t[2], para=para, ctype=ctype, ...) * B - 1,
                       EMPIRcop(t, t[1], para=para, ctype=ctype, ...) * C - 1), ncol=3)
      colnames(blom) <- paste0("U|V=", c("0.25", "0.50", "0.75"))
      rownames(blom) <- rev(colnames(blom))
      return(blom)
    }
  } else {
    if(is.null(cop)) {
      warning("must have copula argument specified, returning NULL")
      return(NULL)
    }
    if(as.blomCOPss) {
      blom <- matrix(c(
          blomCOPss(cop=cop, para=para, uu=c(t[1], t[1]), vv=c(t[1], t[1]), ...),
          blomCOPss(cop=cop, para=para, uu=c(t[2], t[1]), vv=c(t[2], t[1]), ...),
          blomCOPss(cop=cop, para=para, uu=c(t[3], t[1]), vv=c(t[3], t[1]), ...),
          blomCOPss(cop=cop, para=para, uu=c(t[1], t[2]), vv=c(t[1], t[2]), ...),
          blomCOPss(cop=cop, para=para, uu=c(t[2], t[2]), vv=c(t[2], t[2]), ...),
          blomCOPss(cop=cop, para=para, uu=c(t[3], t[2]), vv=c(t[3], t[2]), ...),
          blomCOPss(cop=cop, para=para, uu=c(t[1], t[1]), vv=c(t[1], t[3]), ...),
          blomCOPss(cop=cop, para=para, uu=c(t[2], t[1]), vv=c(t[2], t[3]), ...),
          blomCOPss(cop=cop, para=para, uu=c(t[3], t[1]), vv=c(t[3], t[3]), ...)
      ), ncol=3)
      colnames(blom) <- paste0("U|V=", c("0.25", "0.50", "0.75"))
      rownames(blom) <- rev(colnames(blom))
      return(blom)
    } else {
      #A <- c(16/3, 16/6, 16/9)
      #B <- c(16/2, 16/4, 16/6)
      #C <- c(16/1, 16/2, 16/3)
      A <- 1 / P(t, t[3]); B <- 1 / P(t, t[2]); C <- 1 / P(t, t[1])
      blom <- matrix(c(cop(t, t[3], para=para, ...) * A - 1,
                       cop(t, t[2], para=para, ...) * B - 1,
                       cop(t, t[1], para=para, ...) * C - 1), ncol=3)
      colnames(blom) <- paste0("U|V=", c("0.25", "0.50", "0.75"))
      rownames(blom) <- rev(colnames(blom))
      return(blom)
    }
  }
}

"hoefCOP" <-
function(cop=NULL, para=NULL, p=2, as.sample=FALSE, sample.as.prob=TRUE,
                              brute=FALSE, delta=0.002, ...) {

    if(as.sample) {
      if(is.null(para)) {
         warning("Sample Hoeffding's Phi desired but para is NULL, ",
                 "returning NULL")
         return(NULL)
      }
      if(length(names(para)) != 2) {
        warning("para argument must be data.frame having only two columns, ",
                "returning NULL")
        return(NULL)
      }
      if(as.sample == -1) message("Sample Hoeffding's Phi---CPU intensive!")

      # Gaisser et al. (2010, eq. 3): A multivariate extension version of
      #  Hoeffding's Phi-Square: J. Multivariate Analysis: v. 101, pp. 2571--2586
      "invHd" <- function(d) {
         if(d <= 0) stop("dimension can not be 0 or negative!")
         HdA <- 2/((d+1)*(d+2)) + (1/3)^d
         HdB <- (1/2^d) * factorial(d) /
                   cumprod(sapply(0:d, function(i) { i + 1/2 }))[d+1] # d + 1 !!!!
         return(HdA - HdB)
         # The value 1/invHD(2) must be 90.
      }

      # The equation by those authors is based on probabilities for U. There
      # is a chance that users might feed non probability values so in that
      # case, rank and convert back to a simple probability.
      U <- as.matrix(para); n <- length(U[,1]); d <- length(U[1,])
      if(! sample.as.prob) for(i in 1:d) U[,i] <- rank(U[,i])/n
      ns <- 1:n
      A <- sum(sapply(ns, function(j) {
             sum(sapply(ns, function(k) {
             cumprod(sapply(1:d, function(i) { 1 - max(c(U[j,i], U[k,i])) }))[d]
             } ))
           } ))

      B <- sum(sapply(ns, function(j) {
                           cumprod(sapply(1:d, function(i) { 1 - U[j,i]^2 }))[d]
           } ))

      samPHIsq <- ((1/n)^2) * A - ((2/n)*(1/2)^d)*B + (1/3)^d
      hd <- 1/invHd(d)
      # Note that Hoeffding's Phi (not Phi-Square is being returned)
      return(sqrt(hd*samPHIsq))
   }

   if(is.null(cop)) {
      warning("must have copula argument specified, returning NULL")
      return(NULL)
   }

   if(p == 1) {
      kp <- 12
   } else if(p == 2) {
      kp <- 90
   } else if(p == 3) {
      kp <- 560
   } else if(p == 4) {
      kp <- 3150
   } else if(p == 5) {
      kp <- 16600
   } else {
      kp <- exp(lgamma(2*p+3) - 2*lgamma(p + 1))/2 # Nelsen (2006, p. 213)
   }
   if(brute) {
      us <- vs <- seq(.Machine$double.eps, 1-.Machine$double.eps, delta)
      hoef <- sum(sapply(us, function(u) {
                 sum(sapply(vs, function(v) {
                    abs(COP(u,v, cop=cop, para=para, ...) - u*v)^p }))
               }))
      hoef <- (hoef*delta^2*kp)^(1/p)
      return(hoef)
   }

   myint <- NULL
   try(myint <- integrate(function(u) {
            sapply(u,function(u) {
                      integrate(function(v) {
                          abs(COP( u, v, cop=cop, para=para, ...) - u*v)^p},
                      0, 1)$value
            })}, 0, 1) )
   if(is.null(myint)) {
      return(NA)
   } else if(myint$value == 0 & myint$abs.error == 0) {
      warning("integrate() returned zero with zero absolute error, ",
              "p is likely too large or the copula is Independence")
      return(NA)
   } else {
      return((myint$value*kp)^(1/p))
   }
}

"LpCOP" <-
function(cop=NULL, para=NULL, p=2, brute=FALSE, delta=0.002, ...) {
    return(hoefCOP(cop=cop, para=para, p=p, brute=brute, delta=delta, ...))
}


"LpCOPradsym" <-
function(cop=NULL, para=NULL, p=2, brute=FALSE, delta=0.002, ...) {
   if(brute) {
      sum <- 0
      us <- vs <- seq(.Machine$double.eps, 1-.Machine$double.eps, delta)
      hoef <- sum(sapply(us, function(u) {
                 sum(sapply(vs, function(v) {
                    abs(COP(u,v, cop=cop, para=para, ...) -
                    surCOP(u,v, cop=cop, para=para, ...))^p }))
              }))
      kp <- 1
      Lp <- (sum*hoef^2*kp)^(1/p)
      return(Lp)
   }

   myint <- NULL
   try(myint <- integrate(function(u) {
            sapply(u,function(u) {
                      integrate(function(v) {
                          abs(COP( u, v, cop=cop, para=para, ...) -
                           surCOP(u,v, cop=cop, para=para, ...))^p},
                      0, 1)$value
            })}, 0, 1) )
   if(is.null(myint)) {
      return(NA)
   } else {
      kp <- 1
      return((myint$value*kp)^(1/p))
   }
}


"LpCOPpermsym" <-
function(cop=NULL, para=NULL, p=2, brute=FALSE, delta=0.002, ...) {
   if(brute) {
      us <- vs <- seq(.Machine$double.eps, 1-.Machine$double.eps, delta)
      hoef <- sum(sapply(us, function(u) {
                 sum(sapply(vs, function(v) {
                    abs(COP(u,v, cop=cop, para=para, ...) -
                        COP(v,u, cop=cop, para=para, ...))^p }))
              }))
      kp <- 1
      Lp <- (hoef*delta^2*kp)^(1/p)
      return(Lp)
   }

   myint <- NULL
   try(myint <- integrate(function(u) {
            sapply(u,function(u) {
                      integrate(function(v) {
                          abs(COP(u,v, cop=cop, para=para, ...) -
                              COP(v,u, cop=cop, para=para, ...))^p},
                      0, 1)$value
            })}, 0, 1) )
   if(is.null(myint)) {
      return(NA)
   } else {
      kp <- 1
      return((myint$value*kp)^(1/p))
   }
}

"nuskewCOP" <-
function(cop=NULL, para=NULL, as.sample=FALSE, brute=FALSE, delta=0.002, ...) {
   joeskewCOP(cop=cop, para=para, type="nu",
              brute=brute, delta=delta, as.sample=as.sample, ...)
}
"nustarCOP" <-
function(cop=NULL, para=NULL, as.sample=FALSE, brute=FALSE, delta=0.002, ...) {
   joeskewCOP(cop=cop, para=para, type="nustar",
              brute=brute, delta=delta, as.sample=as.sample, ...)
}
"joeskewCOP" <-
function(cop=NULL, para=NULL, type=c("nu", "nustar", "nuskew"), as.sample=FALSE,
                              brute=FALSE, delta=0.002,...) {
   type = match.arg(type)
   if(as.sample) {
      if(is.null(para)) {
         warning("Sample Nu-Skew desired but para is NULL, ",
                 "returning NULL")
         return(NULL)
      }
      if(length(names(para)) != 2) {
        warning("para argument must be data.frame having only two columns, ",
                "returning NULL")
        return(NULL)
      }

      n <- length(para[,1]); nn <- n^2; ns <- 1:n
      R <- rank(para[,1]); S <- rank(para[,2])
      corsgn <- sign(cor(para[,1],para[,2], method="kendall"))
      #npm <- ifelse(corsgn == -1, +1,   +0)
      spm <- ifelse(corsgn == -1, +2.4, +1.1)
      samNU <- NA
      if(type == "nu" | type == "nuskew") {
         if(as.sample == -1) message("Sample Nu-Skew after Joe (2014)",
                                     "---CPU intensive!")
         samNU <- sum(sapply(ns, function(i) {
                     sum(sapply(ns, function(j) {
                        (j - i)*sum(as.numeric(R <= i & S <= j))/(n+1)
                     } ))
                  } ))/(nn*(n)) # Note division by (n) but not in the similar place below
         return(96*samNU)
      } else if(type == "nustar") {
         if(as.sample == -1) message("Sample Nu-Skew-Star",
                                     "---CPU intensive!")
         samNU <- sum(sapply(ns, function(i) {
                  sum(sapply(ns, function(j) {
                     (j + i)*sum(as.numeric(R <= i & S <= j))/(n+spm)
                  } ))
               } ))/(nn*(n+1)) # Note division by (n+1) but not in the similar place above
         return(12*samNU - 4)
      } else {
         stop("Never should be here in logic")
      }
   }
   #NuStar: double sum of j+i is n^2*(n+1), hence the division shown above and note the
   # division by n within the R <= i & S <= j
   #h <- sapply(1:10, function(k)  { sapply(1:k, function(i) sapply(1:k, function(j) { (j+i) })) } )
   #sum(h[[10]])
   # Now compare to the j-i for which the double sum is 0, hence there is just the nn
   # division on the outside of the sum and there is not division by n for the R <= i & S <= j

   if(brute) {
      us <- vs <- seq(.Machine$double.eps, 1-.Machine$double.eps, delta)
      skew <- NA
      if(type == "nu" | type == "nuskew") {
         skew <- sum(sapply(us, function(u) {
                    sum(sapply(vs, function(v) {
                       (v-u)*COP(u,v,cop=cop,para=para, ...)
                    }))
                 }))
          return(96*skew*delta^2)
      } else if(type == "nustar") {
         skew <- sum(sapply(us, function(u) {
                    sum(sapply(vs, function(v) {
                       (v+u)*COP(u,v,cop=cop,para=para, ...)
                    }))
                 }))
          return(12*skew*delta^2 - 4)
      } else {
         stop("Never should be here in logic")
      }
      return(skew)
   }

   myint <- NULL
   if(type == "nu" | type == "nuskew") {
      try(myint <- integrate(function(u) {
               sapply(u,function(u) { integrate(function(v) {
                          (v-u)*COP(u,v,cop=cop, para=para,...)
               }, 0, 1)$value })}, 0, 1) )
      ifelse(is.null(myint), return(NA), return(96*myint$value))
   } else if(type == "nustar") {
      try(myint <- integrate(function(u) {
               sapply(u,function(u) { integrate(function(v) {
                          (v+u)*COP(u,v,cop=cop, para=para,...)
               }, 0, 1)$value })}, 0, 1) )
      ifelse(is.null(myint), return(NA), return(12*myint$value - 4))
   } else {
      stop("Never should be here in logic")
   }
}

"wolfCOPtest" <-
function(x, y, asuv=FALSE, aslist=TRUE, na.rm=TRUE, digits=6,
               probs=c(0.90, 0.95, 0.98, 0.99, 0.995),
               zmat=NULL, statf=mean, rndphi=20, usepade=FALSE,
               ties.method=c("average", "first", "last", "random", "max", "min"), ...) {
  ties.method <- match.arg(ties.method)
  # The probs are quantile levels of the sigma to report, and these are useful to check against the
  # simulations but also to produce these as critical values should the user be interested in
  # these as well as the p-value.
  if(is.null(probs)      ) probs <- 0.95 # 95th percentile or rather the 5-percent critical value (upper tail).
  if( length(probs) == 0 ) probs <- 0.95 # 95th percentile or rather the 5-percent critical value (upper tail).
  if( length(probs) == "") probs <- 0.95 # 95th percentile or rather the 5-percent critical value (upper tail).

  if(! is.null(zmat)) {
    if(ncol(zmat) != 4) {
      warning("zmat when given must be given as four columns, returning NULL")
      return(NULL)
    }
    ties.method <- "random"
    asuv <- FALSE
  }

  lo <- .Machine$double.eps; hi <- 1 - lo
  if(length(x) == 1) { # If x is just one value, then it is treated as the Schweizer-Wolff Sigma and
       rwolf <- x[1]; lwolf <- log( rwolf / (1-rwolf) ); n <- y[1] # the sample size is in y[1]
    if(lwolf == -Inf) lwolf <- log(    lo / (1 - lo)  )
    if(lwolf == +Inf) lwolf <- log(    hi / (1 - hi)  )
    if(n < 3) {
      warning("sample size is <3, returning NULL")
      return(NULL)
    }
    nuuniq <- nvuniq <- rwolves <- "wolf_direct"
  } else {
    # The && is needed to avoid this case
    # Error in if (!is.null(ncol(x)) & ncol(x) == 2) { : argument is of length zero
    if(! is.null(ncol(x)) && ncol(x) == 2) { # This permit the x to be a two column table
      y <- x[,2]; x <- x[,1]                # from which the x,y are formed.
    }
    if(length(x) != length(y)) {
      warning("length x != length y, returning NULL")
      return(NULL)
    }
    uv <- data.frame(u=x, v=y)
    if(! is.null(zmat)) {
      if(nrow(uv) != nrow(zmat)) {
        warning("nrow of uv and zmat are not equal, returning NULL")
        return(NULL)
      }
    }
    if(na.rm) {
      wnt <- stats::complete.cases(uv)
      uv <- uv[wnt,]
      if(! is.null(zmat)) {
        zul <- zmat[wnt,1]; zvl <- zmat[wnt,2]; zur <- zmat[wnt,3]; zvr <- zmat[wnt,4]
      }
    }

    nuuniq <- nvuniq <- rwolves <- NA
    n <- nrow(uv) # sample size
    if(is.null(zmat)) {
      zul <- zvl <- zur <- zvr <- rep(NA, n)
    } else {
      zul <- zmat[,1]; zvl <- zmat[,2]; zvl <- zmat[,3]; zvr <- zmat[,4]
    }

    if(n < 3) { # This handling of the sample size dates from an much earlier version of this
      # function that had a lower limit of 9. With the empirical distributions for sample sizes
      # 3-40 supported, we drop the minimum sample size down to 3 but with the logic here,
      # we effectively permit samples sizes to be incoming down to:
      warning("sample size is <3; returning NULL")
      return(NULL)
    }
    nrndsim <- "zero needed"
    if(any(! is.na(zul)) || any(! is.na(zvl))) {
      if(! is.function(statf)) {
        warning("statf is needed, but it is not a function, returning NULL")
        return(NULL)
      }
      nuuniq <- nvuniq <- "wolves_by_zmatrix"
      nrndsim <- rndphi * (length(zul[! is.na(zul)]) + length(zvl[! is.na(zvl)]))
      ix <- seq_len(n)
      rwolves <- vector(mode="numeric", length=nrndsim)
      for(i in seq_len(nrndsim)) {
        ruv        <- uv; wu <- ! is.na(zul); wv <- ! is.na(zvl)
        ruv[wu, 1] <- sapply(ix[wu], function(k) runif(1, min=zul[k], max=zur[k]))
        ruv[wv, 2] <- sapply(ix[wv], function(k) runif(1, min=zvl[k], max=zvr[k]))
        ruv[   ,1] <- lmomco::pp(ruv[,1], sort=FALSE, ties.method=ties.method, ...)
        ruv[   ,2] <- lmomco::pp(ruv[,2], sort=FALSE, ties.method=ties.method, ...)
        rwolves[i] <- wolfCOP(para=ruv, as.sample=TRUE)
      }
      rwolf <- statf(rwolves)
    } else {
      if(! is.function(statf)) {
        warning("statf is needed, but it is not a function, returning NULL")
        return(NULL)
      }
      nuuniq <- length(unique(uv[,1])); nvuniq <- length(unique(uv[,2]))
      if(! asuv & (nuuniq != n | nvuniq != n) & ties.method == "random") {
        nrndsim <- rndphi * (n - pmin(length(nuuniq), length(nvuniq)))
        rwolves <- vector(mode="numeric", length=nrndsim)
        for(i in seq_len(nrndsim)) {
          ruv        <- uv[sample(seq_len(nrow(uv)), nrow(uv)),]
          ruv[,1]    <- lmomco::pp(ruv[,1], sort=FALSE, ties.method=ties.method, ...)
          ruv[,2]    <- lmomco::pp(ruv[,2], sort=FALSE, ties.method=ties.method, ...)
          rwolves[i] <- wolfCOP(para=ruv, as.sample=TRUE)
        }
        rwolf <- statf(rwolves)
      } else {
       if(! asuv) { # if true, then the user has provided the paired observations of probability
          uv[,1] <- lmomco::pp(uv[,1], sort=FALSE, ties.method=ties.method, ...)
          uv[,2] <- lmomco::pp(uv[,2], sort=FALSE, ties.method=ties.method, ...)
        }
        rwolf <- wolfCOP(para=uv, as.sample=TRUE) # Schweizer-Wolff Sigma : wolf in (0,1)
      }
    }
  }
  lwolf <- log(rwolf / (1 - rwolf)) # logit transform of the Sigma
  if(lwolf == -Inf) lwolf <- log(lo / (1 - lo))
  if(lwolf == +Inf) lwolf <- log(hi / (1 - hi))

  dtype <- ifelse(n <= 40, "gno", "pe3") # We can see via inst/make_wolfCOPtest/chck_wolfCOPtestP.R
  # and the L-moment ratio diagram on the logit transform of the sigma, that there is a heuristic
  # change over from GNO to PE3 at about n=40. HOWEVER ------------------------------------------
  # Study of the terminal output of the make_wolfCOPtestP_smlsam.R script and the upper tail,
  # informs us that the PE3 is reaching into the upper tail already better than the GNO by sample
  # size of 6 to 9 at p-values as fine as 0.001, so we move to the pe3 throughout.
  dtype <- "pe3"

  # Apply the regressions using the hardwired coefficients herein
  # m <- 7000 # sample sizes for which we declare that Tau3 and Tau4 have become constant, which is
  # is technically close to reality but with the curvilinear regression being used, we eschew the
  # prediction not being monotonic decreasing with sample size want it to have an apparent asymptotic.
  m <- ifelse(n > 7000, 7000, n) # This keeps the apparent trajectory of a Tau3 and Tau4 plot
  # having a hook in it as sample sizes increases to infinite.

  if(usepade) { # See copBasic/inst/make_wolfCOPtest/genmod_wolfCOPtestP_B.R
    "myPade" <- function(x, a=0, b=0) { # https://en.wikipedia.org/wiki/Pade_approximant
                   j <- seq_len(length(a))-1; k <- seq_len(length(b))
                   R <- vector(mode="numeric", length(x)) # The response R(x)
                   for(i in seq_len(length(x))) { # for each of the values in x
                     nj <-     sum(sapply(j, function(j) a[j+1]*x[i]^j)) # j=0 to m
                     dk <- 1 + sum(sapply(k, function(k) b[k  ]*x[i]^k)) # k=1 to n
                     R[i] <- nj / dk
                   }
                   return(R) }

    myAlst <- list(
         logitmu   = c(3.22505751817977, -0.915990158877043, 0.967465808418283, -3.40707532868352),
         logitlam2 = c(-4.81815999220734, 10.0896872864149),
         logittau3 = c(1.09499958951764, -0.39965628645059, 0.497606810543813),
         logittau4 = c(0.0963673653663518, -0.180543732512747, 0.14871284556827, 0.234137175245599))
    myBlst <- list(
         logitmu   = c(-0.312728450592508, 3.01615693521397),
         logitlam2 = c(-4.47041708077035, -0.0428413982170646),
         logittau3 = c(-0.36925380858122, 4.22752165636147),
         logittau4 = c(-2.67356792376944, 1.61273757646706, 1.84737967199012))

    #if(n < 6) n <- 6
    mu    <-      myPade(log10(n), a=myAlst$logitmu,   b=myBlst$logitmu  )
    l2    <- exp( myPade(log10(n), a=myAlst$logitlam2, b=myBlst$logitlam2) )
    t3    <-      myPade(log10(m), a=myAlst$logittau3, b=myBlst$logittau3)
    t4    <-      myPade(log10(m), a=myAlst$logittau4, b=myBlst$logittau4)
  } else {
    # Nonlinear regression coefficients computed PRESS minimization of residuals for the
    # exponent on log10(sample size) term. The regressions come from simulation of the Sigma
    # distribution (its logit) assuming the Independence copula.
    mucoe <- c(-0.00568001, -1.09924807, 1.070086,   -1.293310558)
    l2coe <- c( 0.13029126, -0.00099355, 0.09344292, -2.07221688 )
    t3coe <- c( 0.07670933,  0.00566279, 0.16300227, -1.847265638)
    t4coe <- c( 0.12341757, -9.825e-05,  0.04381592, -2.574804698)
    mu    <- mucoe[1] + mucoe[2] * log10(n) + mucoe[3] * log10(n)^mucoe[4] # Mean    (Lambda1)
    l2    <- l2coe[1] + l2coe[2] * log10(n) + l2coe[3] * log10(n)^l2coe[4] # Lambda2 (L-scale)
    t3    <- t3coe[1] + t3coe[2] * log10(m) + t3coe[3] * log10(m)^t3coe[4] # Tau3    (L-skew)
    t4    <- t4coe[1] + t4coe[2] * log10(m) + t4coe[3] * log10(m)^t4coe[4] # Tau4    (L-kurtosis)
  }

  if(t4 < (5 * t3^2 - 1)/4) t4 <- (5 * t3^2 - 1)/4 # theoretical limits of Tau4
  lmrs  <- c(mu, l2, t3, t4) # Tidy list of the Lmoments of the logit(Sigma) distribution
  lmro  <- lmomco::vec2lmom(lmrs, checklmom=FALSE) # Lmoment "o"bject
  if( ! lmomco::are.lmom.valid(lmro) ) {
    warning("L-moments are invalid, sample size beyond empirical logit estimator(s)?\n",
            "   Lambdas ", paste(round(lmro$lambdas, digits=6), collapse=", "), "\n",
            "    Ratios ", paste(round(lmro$ratios,  digits=6), collapse=", "), "\n",
            "Pretrapping of T3-T4 has been made, do you see a negative Lambda2?\n",
            "If so, then that means that the variation is predicted negative!")
    return(NULL)
  }
  if(dtype == "gno") {
    para  <-  lmomco::pargno(lmro, useHosking=FALSE)
  } else if(dtype == "pe3") {
    para  <-  lmomco::parpe3(lmro, useHosking=FALSE)
  } else {
    stop("should not be here in logic")
  }

  # Lmoment ratio diagram shows
  # Really close adherence to a generalized normal distribution (3-parameter log-normal)
  # and hence that distribution is chosen here.
    neps  <-               lmomco::par2cdf(lwolf, para, paracheck=FALSE)   # CDF of logit(SIGMA)
    quans <- 1 / (1 + exp(-lmomco::par2qua(probs, para, paracheck=FALSE))) # QUA of inverse logit --> quans are Sigmas

  quans     <- round(quans,     digits=digits) # Estimated upper-tail quantiles of Sigma distribution
  lmrs      <- round(lmrs,      digits=digits) # Lmoments of the logit(Sigma) distribution
  para$para <- round(para$para, digits=digits) # parameters of the fitted distribution
  rwolf     <- round(rwolf,     digits=digits) # Sigma as the test statistic
  lwolf     <- round(lwolf,     digits=digits) # logit(Sigma) as needed for p-value lookup

  # Because of the potential confusion with the logit() transform involved, let us have long names.
  names(lmrs)      <- c("mu_TEXT_sigmas",    "lscale_TEXT_sigmas",  "tau3_TEXT_sigmas", "tau4_TEXT_sigmas" )
  names(para$para) <- paste0("para", seq_len(length(para$para)), "_TEXT_sigmas")
  names(lmrs)      <- gsub("_TEXT_", "logit", names(lmrs     ))
  names(para$para) <- gsub("_TEXT_", "logit", names(para$para))

  quatxt           <- paste0("fit_f", gsub("\\.", "p", as.character(100 * probs)))
  names(quans)     <- quatxt

  # We appear to use less system time on system.time(), when we find the file and load it instead
  # of using the data() call.
  smlsam <- system.file("data/wolfCOPtest_data_smlsam.RData", package="copBasic")
  max_n_in_smlsam <- 40 # max sample size within wolfCOPtest_data_smlsam$n (yes hard wired)
  n <- as.integer(n)
  if(n <= max_n_in_smlsam & file.exists(smlsam)) {
    wolfCOPtest_data_smlsam <- NULL # initialize whether or no so R CMD check --as-cran will pass by visibility
    load(smlsam)
    #data(wolfCOPtest_data_smlsam) # importFrom("utils", "data") # in NAMESPACE required
    sata <- wolfCOPtest_data_smlsam; suppressWarnings( rm(wolfCOPtest_data_smlsam) )
    sata <- sata[sata$n == n,]       # isolate the table to the sample size of interest
    if(nrow(sata) == 0) {
      warning("sample size n=", n, " does not exist in wolfCOPtest_data_smlsam, so set pval_small=NA")
      pval_small <- NA
      names(pval_small) <- paste0("p.value(sample_le", max_n_in_smlsam, ")")
    } else {
      sata <- sata[order(sata$probs),] # should be sorted already but do so again if needing to inspect
      row.names(sata) <- NULL; # print(sata, 16)
      suppressWarnings( nep_small <- stats::approx(sata$wolfemp, y=sata$probs, xout=rwolf)$y )
      pval_small <- round(1 - nep_small, digits=16)
      names(pval_small) <- paste0("p.value(sample_le", max_n_in_smlsam, ")")
    }
  } else {
    pval_small <- NA; names(pval_small) <- paste0("p.value(sample_le", max_n_in_smlsam, ")")
  }

  pval <- round(1 - neps, digits=16); names(pval) <- paste0("p.value(dist_", dtype, ")")
  pval <- c(pval, pval_small)

  zz <- c(n, rwolf, lwolf, pval, para$para, lmrs, quans, nuuniq, nvuniq, rwolves)
  names(zz) <- c("sample_size", "sigma", "logit_sigma",
                 names(pval), names(para$para), names(lmrs), quatxt,
                 "num_uuniq", "num_vuniq", "rand_sigma")
  names(zz) <- gsub("_TEXT_", "logit", names(zz))

  if(aslist) {
    wz <- c(rwolf, lwolf); names(wz) <- c("sigma", "logit_sigma")
    zz <- list(sample_size=n, estimate=rwolf, statistic=wz, p.value=pval,
               distpara_by_lmoms=para$para)
    zz$lmoms_logit_sigma <- lmrs  # L-moments of the logit(SIGMAS) distribution
    zz$sigma_quantiles   <- quans # Put these last because this length of vector is mutable, and it
    # visually makes these better on the right side of aslist=FALSE (vector return), in particular.
    zz$num_uuniq  <- nuuniq
    zz$num_vuniq  <- nvuniq
    zz$rand_sigma <- rwolves
  }
  return(zz)
}

"EMPIRcop" <-
function(u,v, para=NULL,
              ctype=c("weibull", "hazen", "1/n", "bernstein", "checkerboard"),
              bernprogress=FALSE, checkerboard.offset=0, ...) {
  bernstein <- FALSE
  ctype <- match.arg(ctype)
  uobs <- vobs <- NA

  if(exists("para", para)) {
     uobs <- para$para[,1]
     vobs <- para$para[,2]
     if(exists("ctype",        para)) ctype <- para$ctype
     if(exists("bernprogress", para)) bernprogress <- para$bernprogress
     para <- para$para # now reset the para to ONLY have the uobs and vobs
     # because if bernstein is triggered the a secondary call to this function
     # is made and bernstein must be turned off.
  }
  if(length(names(para)) != 2) {
     warning("a data.frame having only two columns is required in the para ",
             "argument or para$para")
     return(NULL)
  }

  if(ctype == "bernstein") bernstein <- TRUE

  # number of points based on which the empirical copula is computed
  n <- length(para[,1]); ns <- 1:n

  nu <- length(u); nv <- length(v)
  if(nu > 1 & nv > 1 & nu != nv) {
     warning("length u = ",nu, " and length v = ",nv)
     warning("longer object length is not a multiple of shorter object length, ",
             "no recycling in EMPIRcop()")
     return(NA)
  }
  # The extra hassle of vectorization made here is to handle situations
  # in which nested integrals are used where uneven vectors can be passed.
  if(nu == 1) {
     u <- rep(u, nv)
  } else if(nv == 1) {
     v <- rep(v, nu)
  }
  nu <- length(u) # make sure to RESET IT!,  # number of evaluation points

  if(bernstein) { # will potentially burn SERIOUS CPU time
     if(bernprogress) {
         message("Bernstein triggered, index of each {u,v} shown: ",
                 appendLF=FALSE)
     }
     ber <- sapply(1:nu, function(k) {
                   if(bernprogress) message(k,"-", appendLF=FALSE);
                   tmpA <- sapply(ns, function(i) {
                        A <- choose(n, i) * u[k]^i * (1-u[k])^(n-i)
                        tmpB <- sapply(ns, function(j) {
                                  B <- choose(n, j) * v[k]^j *(1-v[k])^(n-j)
                         return(EMPIRcop(i/n, j/n, para=para, ctype="1/n")*A*B) })
                                  return(sum(tmpB)) })
                        return(sum(tmpA)) })
     if(bernprogress) message("\n", appendLF=FALSE)
     return(ber)
  } else if(ctype == "checkerboard") {
     uv <- cbind(u,v)
     R <- t(apply(para, 2, rank, ...)) # (d, n)-matrix of ranks
     return(vapply(seq_len(nu), function(k) # iterate over rows k of uv
            sum(apply(pmin(pmax(n * uv[k,] - R + 1, 0), 1), 2, prod)) / (n + checkerboard.offset),
            NA_real_)) # pmin(...) = (d, n)-matrix
  } else {
     R <- rank(para[,1]); S <- rank(para[,2])
     if(ctype == "hazen") {
        R <- (R - 0.5)/n; S <- (S - 0.5)/n
     } else if(ctype == "weibull") {
        R <- R/(n+1);     S <- S/(n+1)
     } else {
        R <- R/n;         S <- S/n
     }
     sapply(1:nu, function(i) sum(as.numeric(R <= u[i] & S <= v[i]))/n )
  }
}

# Gumbel-Hougaard Copula
"GHcop" <-
function(u, v, para=NULL, tau=NULL, tau.big=0.985, cor=NULL, ...) {
  if(! is.null(cor)) tau <- cor

  # tau.big <- 0.985 # tests show at least a failure rate < 1/10000 in simCOP
  # if tau is kept this low, which is a really large tau.
  para.big <- 1/(1-tau.big)

  if(is.null(para)) {
     if(is.null(tau)) {
        if(    length(u)     ==  length(v) &
              (length(u) > 1  &  length(v) > 1)) {
           tau <- cor(u,v, method="kendall")
        } else {
           warning("Argument para (Alpha) is NULL and u an v have unequal length",
                   "that prevents Alpha estimation by Kendall's Tau, returning NULL")
           return(NULL)
        }
        para <- 1/(1-tau)
        if(para < 1) {
           warning("negative correlation can not be fit with this copula, ",
                   "returning NULL")
           return(NULL)
        }
        names(para) <- "theta"; names(tau) <- "tau"
        return(list(para=para, tau=tau))
     } else {
        if(tau < 0) {
           warning("Kendall's Tau is negative, not compatible with this copula, ",
                   "returning NULL")
           return(NULL)
        }
        para <- 1/(1-tau)
        names(para) <- "theta"; names(tau) <- "tau"
        return(list(para=para, tau=tau))
     }
  }

  if(length(para) == 1) {
     if(para[1] < 1) {
        warning("Parameter Theta < 1")
        return(NULL)
     }
     if(para[1] > para.big ) {
        warning("Parameter Theta (para) is big (numerically too big), returning M(u,v)")
        return(M(u,v))
     }
  } else if(length(para) == 2) {
    go <- TRUE
    if(para[1] < 1) {
       warning("Parameter Beta1 is < 1");  go <- FALSE
    }
    if(para[2] <= 0) {
       warning("Parameter Beta2 is <= 0"); go <- FALSE
    }
    if(! go) return(NULL)
  } else if(length(para) == 3) {
     if(para[1] < 1) {
        warning("Parameter Theta (para[1]) is < 1, returning NULL")
        return(NULL)
     }
     if(para[2] < 0 | para[2] > 1) {
        warning("Parameter Pi1 is not in [0, 1], returning NULL")
        return(NULL)
     }
     if(para[3] < 0 | para[3] > 1) {
        warning("Parameter Pi2 is not in [0, 1], returning NULL")
        return(NULL)
     }
  } else {
     stop("Should not be here in logic---too many parameters")
  }


  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
    warning("length u = ",length(u), " and length v = ",length(v))
    warning("longer object length is not a multiple of shorter object length, ",
            "no recycling")
    return(NA)
  }
  # The extra hassle of vectorization made here is to handle situations
  # in which nested integrals are used where uneven vectors can be passed
  if(length(u) == 1) {
     u <- rep(u, length(v))
  } else if(length(v) == 1) {
     v <- rep(v, length(u))
  }

  if(length(para) == 1) {
     # R's defaults catch Inf correctly so no need to check on the finiteness
     # and make a local copy
     return(exp(-(((-log(u))^para + (-log(v))^para)^(1/para))))
  } else if(length(para) == 2) {
     lo <- .Machine$double.xmin^0.5 # sqrt or just xmin?
     # v <- simCOPmicro(0.01, cop=GHcop2, para=c(2,88))
     # v <- simCOPmicro(0.99, cop=GHcop2, para=c(exp(5),0.1))
     # UV <- simCOP(1000, cop=GHcop2, para=c(exp(2),.0000006)) # still a problem, some V's == 1
     if(para[2] < 1e-6) para[2] <- 1e-6
     b1 <- para[1]; b2 <- -para[2]
     bb1 <- (u^b2 - 1)^b1; bb2 <- (v^b2 - 1)^b1
     cop <- ((bb1 + bb2)^(1/b1) + 1)^(1/b2)
     wnt <- bb1 == Inf | bb2 == Inf # this makes simCOP() work throughout range of parameters
     cop[wnt] <- pmin(u[wnt], v[wnt], na.rm=TRUE) # from extremely large scale lcomCOP() testing.
     # This fixes the "lower" tail relative to reflection. But does not ensure the upper
     # tail goes to M().
     wnt <- bb1 < lo | bb2 < lo # this makes simCOP() work throughout range of parameters
     cop[wnt] <- pmin(u[wnt], v[wnt], na.rm=TRUE)
     return(cop)
  } else if(length(para) == 3) {
     lo <- .Machine$double.eps^0.50; hi <- 1 - lo
     p2 <- para[2]; p3 <- para[3]; d <- para[1]; di <- 1/d
     if(p2 >= hi) p2 <- hi; if(p2 <= lo) p2 <- lo
     if(p3 >= hi) p3 <- hi; if(p3 <= lo) p3 <- lo
     cop <- vector(mode="numeric", length(u))
     domo <- rep(FALSE, length(u))
     for(i in seq_len(length(u))) {
       x   <- -log(u[i]+lo); y <- -log(1)
       cop2u <- exp(-(( (p2*x)^d + (p3*y)^d )^di + (1-p2)*x + (1-p3)*y))
       x   <- -log(u[i])
       cop1u <- exp(-(( (p2*x)^d + (p3*y)^d )^di + (1-p2)*x + (1-p3)*y))
       if(is.nan(cop1u)) cop1u <- 0
       dercopu <- (cop2u - cop1u) / lo
       if(is.nan(dercopu) | dercopu < hi) {
         domo[i] <- TRUE
         #print(c("u", u[i], cop2u, cop1u, dercopu))
         next
       }

       x   <- -log(1); y <- -log(v[i]+lo)
       cop2v <- exp(-(( (p2*x)^d + (p3*y)^d )^di + (1-p2)*x + (1-p3)*y))
                       y <- -log(v[i]   )
       cop1v <- exp(-(( (p2*x)^d + (p3*y)^d )^di + (1-p2)*x + (1-p3)*y))
       if(is.nan(cop1v)) cop1v <- 0
       dercopv <- (cop2v - cop1v) / lo
       # print(c(cop1v, cop2v)
       if(is.nan(dercopv) | dercopv < hi) {
         domo[i] <- TRUE
         #print(c("v", v[i], cop2v, cop1v, dercopv))
         next
       }
       #print( c(cop2u, cop1u, cop2v, cop1v))
       #if(any(c(cop2u, cop1u, cop2v, cop1v) <= 0)) domo[i] == TRUE
     }
     #print(summary(domo))
     #if(d > 80) return(MOcop(u,v, para=c(p2, p3)))
     # UV <- simCOP(1000, cop=GHcop, para=c(800, 0.4166787, 0.7973851))
     # I arrived at the 80 after massive simulations and looking at lowest d such that
     # regardless of the p2, p3, that complete simCOP() appear okay and that the
     # simple numerical derivatives are working. The for() above seeks to check the numerical
     # derivative performance on the margins and if it appear viable that derCOP() would work
     # then the MOcop is not the bail out.

     cop[domo] <- MOcop(u[domo], v[domo], para=c(p2, p3))
     if(sum(domo) == length(u)) return(cop)
     x   <- -log(u[! domo]); y <- -log(v[! domo])
     cr  <- ( (p2*x)^d + (p3*y)^d )^di
     k2  <- (1-p2)*x; k3 <- (1-p3)*y
     cop[! domo] <- exp(-(cr + k2 + k3))
     #print(c(x[! domo], y[! domo], cr[! domo], u[! domo], v[! domo], cop[! domo]))
     #cop[is.nan(cop)] <- 0
     return(cop)
  } else {
     stop("Should not be here in logic")
  }
}

"AMHcop" <- function(u, v, para=NULL, rho=NULL, tau=NULL,
                            fit=c("rho","tau"), ...) {
    if(is.null(para)) {
      fit <- match.arg(fit)
      if(is.null(tau) & is.null(rho)) {
        if(fit == "rho") {
          rho <- cor(u,v, method="spearman")
        } else {
          tau <- cor(u,v, method="kendall")
        }
      }
      rt <- NULL
      if(is.null(rho)) {
        # Nelsen (2006,p.172)
        if(tau < (5 - 8*log(2))/3 | tau > 1/3) { # [-0.1817, 0.3333]
          warning("Kendall tau=", tau, " is outside limits attainable [-0.1817, 0.3333]")
          return(NULL)
        }
        "ktau" <- function(t) {
          stau <- (3*t-2)/(3*t) - (2*(1-t)^2*log(1-t))/(3*t^2) # Nelsen(2006,p172)
          # stau <- 1 - (2*(t+(1-t)^2 * log(1-t)))/(3*t^2) # Salvadorietal(2007p240)
          tau - stau
        }
        try(rt <- uniroot(ktau, interval=c(-1, 1-.Machine$double.eps)))
        if(! is.null(rt)) {
          para <- rt$root
          names(para) <- "theta"
          names(tau)  <- "Kendall Tau"
          return(list(para=para, tau=tau))
        } else {
          warning("could not solve for parameter Theta via Kendall Tau")
          return(NULL)
        }
      } else {
        #dilog <- function(x) { # Nelsen(2006,p172)
        #  z <- NULL
        #  try( z <- integrate(function(k) log(k)/(1-k), lower=1, upper=x)$value,
        #                      silent=TRUE)
        #  ifelse(is.null(z), return(NaN), return(z))
        #}
        # Nelsen(2006,p172)
        if(rho < 33 - 48*log(2) | rho > 4*pi^2 - 39) { # [-0.2711, +0.4784]
          warning("Spearman rho=", rho, " is outside limits attainable [-0.2711, +0.4784]")
          return(NULL)
        }
        "srho" <- function(t) {
          # source for the infinite sum is Machler(2014)
          srho <- sum(sapply(1:100, function(k) 3*t^k/choose(k+2, 2)^2))
          # Nelsen(2006,p172)
          #srho <- 12*(1+t)/t^2 * dilog(1-t) - 24*(1-t)/t^2*log(1-t)-3*(t+12)/t
          rho - srho
        }
        try(rt <- uniroot(srho, interval=c(-1, 1-.Machine$double.eps)))
        if(! is.null(rt)) {
          para <- rt$root
          names(para) <- "theta"
          names(rho)  <- "Spearman Rho"
          return(list(para=para, rho=rho))
        } else {
          warning("could not solve for parameter Theta via Spearman Rho")
          return(NULL)
        }
      }
    }
    rev <- FALSE
    if(length(para) == 2) {
      rev <- para[2]; para <- para[-2]
    }
    para <- as.numeric(para)
    if(length(para) != 1) {
      warning("copula only uses one parameter")
      return(NULL)
    }
    d <- para[1]
    if(d < -1) {
      warning("parameter Theta < -1")
      return(NULL)
    }
    if(d > 1) {
      warning("parameter Theta > +1")
      return(NULL)
    }
    if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
       warning("length u = ", length(u), " and length v = ", length(v))
       warning("longer object length is not a multiple of shorter object length, ",
               "no recycling")
       return(NA)
    }
    if(length(u) == 1) {
       u <- rep(u, length(v))
    } else if(length(v) == 1) {
       v <- rep(v, length(u))
    }

    if(rev) {
      cop <- u + v - 1 + ((1-u)*(1-v))/(1-d*u*v)
      cop[is.nan(cop)] <- 1
    } else {
      cop <- (u*v)/(1-d*(1-u)*(1-v))
      cop[is.nan(cop)] <- 0
    }
    return(cop)
}

"breveCOP" <-
function(u,v, para, breve=NULL, ...) {

  if(is.null(para$para)) para$para <- para$para1
  if(is.null(para$cop )) para$cop  <- para$cop1

  the.breve <- NULL
  if(! is.null(breve)) {
    the.breve <- breve
  } else if(! is.null(para$breve)) {
    the.breve <- para$breve
  } else if(! is.null(para$beta )) {
    the.breve <- para$beta
  } else {
    if(is.null(the.breve)) {
      warning("no beta or breve (each means the same) parameter given")
      return(NULL)
    }
  }

  if(       the.breve > +1) {
    the.breve <- +1
  } else if(the.breve < -1) {
    the.breve <- -1
  }

  if(the.breve < 0) {
    return( v^(-the.breve) * COP(u,               v^(1+the.breve),
           para=para, ...) )
  } else {
    return( u^(+the.breve) * COP(u^(1-the.breve), v,
           para=para, ...) )
  }
}

"CIRCcop" <-
function(u, v, para=NULL, as.circ=FALSE, ...) {
  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
    warning("length u = ",length(u), " and length v = ",length(v))
    warning("longer object length is not a multiple of shorter object length, ",
            "no recycling in CIRC()")
    return(NA)
  }
  if(length(u) == 1) {
     u <- rep(u, length(v))
  } else if(length(v) == 1) {
     v <- rep(v, length(u))
  }
  if(! is.null(para)) {
    if(exists("as.circ", para)) as.circ <- para$as.circ
  }
  if(is.na(as.circ)) as.circ <- FALSE # revert to default
  if(as.circ) {
    u <- 1 - acos(2*u - 1) / pi
    v <- 1 - acos(2*v - 1) / pi
  }

  return(sapply(1:length(u), function(i) {
             if(     abs(u[i] - v[i])     > 1/2) { min(u[i],  v[i])        } # M()
             else if(abs(u[i] + v[i] - 1) > 1/2) { max(u[i] + v[i] - 1, 0) } # W()
             else {     (u[i] + v[i] )/2  - 1/4 } }))                        # otherwise
}

"level.setCOP" <-
function(cop=NULL, para=NULL, getlevel=NULL, delu=0.001, lines=FALSE, ...) {
   zz <- level.curvesCOP(cop=cop, para=para, getlevel=getlevel, delt=NULL,
                         ploton=FALSE, plotMW=FALSE, lines=lines, delu=delu, ramp=FALSE, ...)
   return(zz)
}

"derCOPinv2" <-
function(cop=NULL, v, t, trace=FALSE,
         delv=.Machine$double.eps^0.50, para=NULL, ...) {

    func <- function(x,v,LHS,cop,delv=delv,para=para, ...) {
            LHS - derCOP2(cop=cop, u=x, v=v, delv=delv, para=para, ...)
    }
    f.lower <- func(0,v,t,cop,delv=delv,para=para, ...)
    f.upper <- func(1,v,t,cop,delv=delv,para=para, ...)
    if(sign(f.lower) == sign(f.upper)) {
      if(trace) message("not opposite signs for f.lower=",f.lower,
                        " and f.upper=",f.upper,
                        " at v=",v, " and t=",t,
                        "\nThis might be because of degenerate derivative on the section at v.")
      return(NA)
    }

    my.rt <- NULL
    try(my.rt <- uniroot(func,interval=c(0,1), v=v, LHS=t,
                              cop=cop, delv=delv, para=para, ...))
    if(is.null(my.rt)) return(NA) # Now the returned root is "u"
    ifelse(length(my.rt$root) != 0, return(my.rt$root), return(NA))
}

"LMRvuCOP" <-
function(u=seq(0.01, 0.99, by=0.01), cop=NULL, para=NULL, nsim=1E5,
         subdivisions=200L, rel.tol=.Machine$double.eps^0.25, abs.tol=rel.tol, ...) {
  # Jones (2004, eq. 15b) and Asquith (2011, 6.10 is wrong with 1/(1-r) and needs 1/(r-1))
  #                                  refer to lmomco R package inst/ERRATA_FOR_ISBN9781463508418.txt
  Lru <- function(u, r=2) {
    sapply(u, function(u) { sum(sapply(0:(r-2), function(j) {
                     (-1)^j * exp(lchoose(r-1,j)+lchoose(r-1,j+1))*u^(r-2-j)*(1-u)^j })) })/(r-1)
  }

  medvatu <- med.regressCOP(u, cop=COP, para=para)[,2]
  lam1atu <- EvuCOP(u, cop=COP, para=para, rel.tol=rel.tol, abs.tol=abs.tol,
                                           subdivisions=subdivisions, ...)
  lam2atu <- sapply(u, function(u) { E <- NULL
                                 try(E <- integrate(function(v, para=para) {
                t <- derCOP(cop=COP, u, v, para=para); return(t*(1-t)*Lru(t, r=2)) }, 0, +1,
                                           para=para, subdivisions=200)$value, silent=TRUE)
                          if(is.null(E)) { t <- derCOP(cop=COP, u, runif(nsim), para=para)
                                     E <- mean(t*(1-t)*Lru(t, r=2))
                               names(E) <- "sim"
                          }
                              return(E) })
  lam3atu <- sapply(u, function(u) { E <- NULL
                                 try(E <- integrate(function(v, para=para) {
                t <- derCOP(cop=COP, u, v, para=para); return(t*(1-t)*Lru(t, r=3)) }, 0, +1,
         rel.tol=rel.tol, abs.tol=abs.tol, para=para, subdivisions=200)$value, silent=TRUE)
                          if(is.null(E)) { t <- derCOP(cop=COP, u, runif(nsim), para=para)
                                     E <- mean(t*(1-t)*Lru(t, r=3))
                               names(E) <- "sim"
                          }
                              return(E) })
  lam4atu <- sapply(u, function(u) { E <- NULL
                                 try(E <- integrate(function(v, para=para) {
                 t <- derCOP(cop=COP, u, v, para=para); return(t*(1-t)*Lru(t, r=4)) }, 0, +1,
          rel.tol=rel.tol, abs.tol=abs.tol, para=para, subdivisions=200)$value, silent=TRUE)
                          if(is.null(E)) { t <- derCOP(cop=COP, u, runif(nsim), para=para)
                                     E <- mean(t*(1-t)*Lru(t, r=4))
                               names(E) <- "sim"
                          }
                              return(E) })
  zzz <- list(median=medvatu, L1=lam1atu, L2=lam2atu, T3=lam3atu/lam2atu, T4=lam4atu/lam2atu)
  return(zzz)
}

"isfuncCOP" <-
function(cop=NULL, para=NULL, delta=0.002, ...) {

  # Therefore properties 2.2.2a-c of Nelsen (2006, p. 8) are investigated on a grid
  # where delta denotes the interval length. The code is broken in to five distinct
  # units so as the message() output can be used to inform the user. The last unit
  # is the 2-increasing and is very very likely the slowest "test".

  if(is.null(cop)) {
     warning("must have copula argument specified, returning NULL")
     return(NULL)
  }
  if(delta < 0 | delta > 0.5) {
     warning("invalid delta argument specified, returning NULL")
     return(NULL)
  }
  uvs <- seq(delta, 1-delta, by=delta)

  message("Checking condition C(u,0) = 0 for all u---", appendLF=FALSE)
  cond <- unique(cop(u=uvs, v=0, cop=cop, para=para, ...))
  if(length(cond) != 1 | cond[1] != 0) {
    message("FALSE.")
    return(FALSE)
  } else {
    message("TRUE.")
  }

  message("Checking condition C(0,v) = 0 for all v---", appendLF=FALSE)
  cond <- unique(cop(u=0, v=uvs, cop=cop, para=para, ...))
  if(length(cond) != 1 | cond[1] != 0) {
    message("FALSE.")
    return(FALSE)
  } else {
    message("TRUE.")
  }

  message("Checking condition C(u,1) = u for all u---", appendLF=FALSE)
  cond <- unique(uvs - cop(u=uvs, v=1, cop=cop, para=para, ...))
  if(length(cond) != 1 | cond[1] != 0) {
    message("FALSE.")
    return(FALSE)
  } else {
    message("TRUE.")
  }
  
  message("Checking condition C(1,v) = v for all v---", appendLF=FALSE)
  cond <- unique(uvs - cop(u=1, v=uvs, cop=cop, para=para, ...))
  if(length(cond) != 1 | cond[1] != 0) {
    message("FALSE")
    return(FALSE)
  } else {
    message("TRUE.")
  }

  message("Checking 2-increasing condition for all (u,v)---", appendLF=FALSE)
  tmp <- sapply(uvs, function(u) {
            if(any(densityCOP(u, uvs, cop=cop, para=para,
                              truncate.at.zero=FALSE, ...) < 0)) {
               message("FALSE.")
               return(FALSE)
            }})
  message("TRUE.")
  return(TRUE) # Copula properties hold in investigated points
}

"tailordCOP" <-
function(cop=NULL, para=NULL, tol=1e-6, plot=FALSE, verbose=FALSE,  ...) {
  resolution <- abs(log10(tol)) - 1
  kapu.tmp <- kapu <- NA; u <- 0
  KAPU1 <- KAPU2 <- vector(mode="numeric")
  if(verbose) message("Upper-Tail Order")
  t <- 0.5
  while(t > .Machine$double.eps) {
    opts <- options(warn=-1)
    kapu.tmp <- log(surCOP(t,t, cop=cop, para=para, ...))/log(t)
    options(opts)
    if(is.nan(kapu.tmp) | ! is.finite(kapu.tmp)) break
    u <- u + 1
    if(verbose) message("  iter.",u,"  t=",t,"  kappaU=",kapu)
    KAPU1[u] <- t; KAPU2[u] <- kapu
    if(! is.na(kapu) & abs(kapu - kapu.tmp) < tol) break
    t <- t - t/2
    kapu <- kapu.tmp
  }
  if(u == 0) {
    if(! verbose) warning("  Apparent infinity on Upper-Tail Order, ",
                          "setting t=1/10000 and recomputing")
    t <- 0.0001; u <- 1
    kapu <- log(surCOP(t,t, cop=cop, para=para, ...))/log(t)
    KAPU1[u] <- t; KAPU2[u] <- kapu
    if(verbose) message("  iter.",u,"  t=",t,"  kappaU=",kapu)
  }

  if(verbose) message("\nLower-Tail Order")
  kapl.tmp <- kapl <- NA; l <- 0
  KAPL1 <- KAPL2 <- vector(mode="numeric")
  t <- 0.5
  while(t > .Machine$double.eps) {
    opts <- options(warn=-1)
    kapl.tmp <- log(cop(t,t, para=para, ...))/log(t)
    options(opts)
    if(is.nan(kapl.tmp) | ! is.finite(kapl.tmp)) break
    l <- l + 1
    if(verbose) message("  iter.",l,"  t=",t,"  kappaL=",kapl)
    KAPL1[l] <- t; KAPL2[l] <- kapl
    if(! is.na(kapl) & abs(kapl - kapl.tmp) < tol) break
    t <- t - t/2
    kapl <- kapl.tmp
  }
  if(l == 0) {
    if(! verbose) warning("  Apparent infinity on Lower-Tail Order, ",
                          "setting t=1/10000 and recomputing")
    t <- 0.0001; l <- 1
    kapl <- log(COP(t,t, cop=cop, para=para, ...))/log(t)
    KAPL1[l] <- t; KAPL2[l] <- kapl
    if(verbose) message("  iter.",l,"  t=",t,"  kappaL=",kapl)
  }
  if(verbose) message("---done.")
  if(plot) {
    xmin <- qnorm(min(1-KAPU1,KAPL1, na.rm=TRUE))
    xmax <- qnorm(max(1-KAPU1,KAPL1, na.rm=TRUE))
    ymin <- min(KAPU2,KAPL2, na.rm=TRUE)
    ymax <- max(KAPU2,KAPL2, na.rm=TRUE)
    plot(qnorm(1-KAPU1), KAPU2, type="l", col=4,
           xlim=c(xmin, xmax), ylim=c(ymin, ymax),
           xlab="STANDARD NORMAL VARIATES",
           ylab="UPPER- AND LOWER-TAIL ORDER")
    lines(qnorm(KAPL1), KAPL2, col=2)
    points(qnorm(1-KAPU1[u]), KAPU2[u], cex=0.7, pch=16, col=4)
    points(qnorm(1-KAPU1[u]), KAPU2[u], pch="U", cex=1.3)
    points(qnorm(  KAPL1[l]), KAPL2[l], cex=0.7, pch=16, col=2)
    points(qnorm(  KAPL1[l]), KAPL2[l], pch="L", cex=1.3)
  }
  return(list(kappaL = round(KAPL2[l], digits=resolution),
              kappaU = round(KAPU2[u], digits=resolution),
              source = "tailordCOP"))
}

"PLpar" <-
function(x, y, rho=NULL, byrho=FALSE, cor=NULL, ...) {
   if(! is.null(cor)) rho   <- cor
   if(! is.null(rho)) byrho <- TRUE
   if(byrho) {
     theta <- NA; names(theta) <- "theta"
     if(! is.null(rho)) {
       the.rho <- rho
     } else {
       the.rho <- cor(x,y, method="spearman")
     }
     if(the.rho == -1) {
        theta <- 0
     } else if(the.rho == 1) {
        theta <- Inf
     } else if(the.rho == 0) {
        theta <- 1
     } else {
        "afunc" <- function(x,LHS) {
           if(x == 1) x <- 1.00001
           a <- x + 1; b <- x - 1; c <- b^2
           RHS <- (a/b) - (2*x*log(x))/c
           return(LHS - RHS)
        }
        try(rt <- uniroot(afunc,
                    interval=c(.Machine$double.eps^0.50,
                               .Machine$double.xmax^0.50),
                    LHS=the.rho))
        theta <- rt$root
     }
     names(theta) <- "theta"
     return(theta)
   } else {
     medx <- median(x)
     medy <- median(y)
     k <- length(x[x < medx & y < medy])
     m <- k/length(x)
     theta <- 4*m^2/(1-2*m)^2
     names(theta) <- "theta"
     return(theta)
   }
}

"PLACKETTpar" <- function(x, y, rho=NULL, byrho=FALSE, cor=NULL, ...) {
   PLpar(x=x, y=y, rho=rho, byrho=byrho, cor=cor, ...)
}

"RAYcop" <-
function(u, v, para=NULL, rho=NULL, method=c("default"),
               rel.tol=.Machine$double.eps^0.5, ...) {

  if(! is.null(rho)) {
    Rho2Theta <- function(rho) {
       coes <- c(1.32682824, -0.38876290, 0.09072305, -0.02921836)
       sapply(rho, function(r) {
       coes[1]*r^1 + coes[2]*r^2 + coes[3]*r^4 + coes[4]*r^6 })
    }
    p <- Rho2Theta(rho)
    names(p) <- "Theta"
    return(p)
  }

  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) {
    warning("length u = ", length(u), " and length v = ", length(v))
    warning("longer object length is not a multiple of shorter object length, ", "no recycling")
    return(NA)
  }
  if(length(u) == 1) {
    u <- rep(u, length(v))
  } else if(length(v) == 1) {
    v <- rep(v, length(u))
  }


  p <- NULL
  if(is.list(para)) {
    if(exists("para",   para)) p      <- para$para
    if(exists("method", para)) method <- para$method
  } else {
    p <- para[1]
  }
  if(is.null(p)) {
    warning("parameter from para to become p in the copula is NULL")
    return(NULL)
  }

  LARGE <- 0.9999
  if(para > LARGE) { # by empirical study of the "default" method
    return( M(u,v) )
  }

  method <- match.arg(method)

  # NEED TO STUDY MORE marcumq.chi <- function(a, b, nu=1) pchisq(b^2, df=2 * nu, ncp=a^2, lower.tail=FALSE)
  # NEED TO STUDY MORE marcumq.ser <- function(a, b, nu=1, n=100) {
  # NEED TO STUDY MORE   k <- exp(-(a^2+b^2)/2)
  # NEED TO STUDY MORE   s <- k*sum(sapply(seq(nu, n, by=1), function(t) (b/a)^t * besselI(a*b, nu=t)))
  # NEED TO STUDY MORE   1 - s
  # NEED TO STUDY MORE }
  # NEED TO STUDY MORE marcumq <- marcumq.chi
  # NEED TO STUDY MORE sapply(seq_len(length((u))), function(i) {
  # NEED TO STUDY MORE   a1 <- -log(1-u[i])
  # NEED TO STUDY MORE   if(is.infinite(a1)) return(v[i])
  # NEED TO STUDY MORE   a2 <- -log(1-v[i])
  # NEED TO STUDY MORE   if(is.infinite(a2)) return(u[i])
  # NEED TO STUDY MORE   a1 <- exp(log(a1) - log(1-p))
  # NEED TO STUDY MORE   a2 <- exp(log(a2) - log(1-p))
  # NEED TO STUDY MORE   a3 <- marcumq.chi(sqrt(2*a1), sqrt(2*p*a2))
  # NEED TO STUDY MORE   a4 <- marcumq.chi(sqrt(2*p*a1), sqrt(2*a2))
  # NEED TO STUDY MORE   zz <- 1 + (1-v[i])*a3 - (1-u[i])*(1-a4)
  # NEED TO STUDY MORE   zz[zz < 0] <- 0
  # NEED TO STUDY MORE   zz[zz > 1] <- 1
  # NEED TO STUDY MORE   return(zz)
  # NEED TO STUDY MORE })

  sapply(seq_len(length((u))), function(i) {
    a1 <- -log(1-u[i]) # use of log1p() does not seem to help get deeper into the tail
    if(is.infinite(a1)) return(v[i])
    a2 <- -log(1-v[i]) # use of log1p() does not seem to help get deeper into the tail
    if(is.infinite(a2)) return(u[i])
    a1 <- exp(log(a1) - log(1-p)) # use of log1p() does not seem to help get deeper into the tail
    a2 <- exp(log(a2) - log(1-p)) # use of log1p() does not seem to help get deeper into the tail
    #    print(c(a1, a2))
    #if(is.infinite(a1)) return(v[i])
    #if(is.infinite(a2)) return(u[i])
    #print(c(a1, a2))
    b1 <- exp(p*a2-a2)
    b2 <- exp(    -a1)
    #print(c(b1, b2))
    # The expon.scaled for besselI means that an 1/exp(-x) multiplier is needed. The formulation
    # below for the integral has a maximal amount of logarithmic opertions until exp() on the last call.
    i1 <- i2 <- NA
    if(method == "default") {
      try(i1 <- integrate(function(s) { x <- 2*sqrt(  s*a1)
                    exp( -s + x + log( besselI(x, 0, expon.scaled=TRUE) ) )
                  }, 0, p*a2, rel.tol=rel.tol)$value, silent=TRUE)
      if(is.na(i1)) return(min(u[i], v[i])) # slightly faster
      #if(is.na(i1)) return(M(u[i], v[i])) # perhaps not ideal, could try Monte Carlo integration
      # at this stage, and have in testing, can not seem to go beyond numerical blowup, so slipping
      # in M copula pinches the relation deep to the upper right and as para ---> 1 the M part
      # progresses down towards the lower left corner

      try(i2 <- integrate(function(t) { x <- 2*sqrt(p*t*a1)
                  exp( -t + x + log( besselI(x, 0, expon.scaled=TRUE) ) )
                  }, 0,   a2, rel.tol=rel.tol)$value, silent=TRUE)
      if(is.na(i2)) return(min(u[i], v[i])) # slightly faster
      #if(is.na(i2)) return(M(u[i], v[i]))
      #print(c(i1, i2))
      #t1 <- b1*(b2*i1 - 1)
      #t2 <- b2*i2
      #print(c(t1, t2))
      #zz <- 1 +  t1 - t2
      zz <- 1 + b1*(b2*i1 - 1) - b2*i2
    }
    zz[zz < 0] <- 0 # attempt to catch any truncation errors should they ever exist
    zz[zz > 1] <- 1 # attempt to catch any truncation errors should they ever exist
    return(zz)
  })
}

#RAYcop(0.5, 0.5, para=0.2)
#RAYcop(0.5, 0.5, para=0.2)
#RAYcop(0, 0.5, para=0.2)
#RAYcop(0, 0, para=0.2)

#UV <- simCOP(1000, cop=RAYcop, para=.991)
#ray1 <- lmomco::lmom2par(lmomco::vec2lmom(c(20, 5)), type="ray")
#ray2 <- lmomco::lmom2par(lmomco::vec2lmom(c(359, 17)), type="ray")
#UV   <- simCOP(1000, cop=RAYcop, para=0.7)
#x <- lmomco::quaray(UV[,1], ray1); y <- lmomco::quaray(UV[,2], ray2)
#plot(x, y)
#abline(v=lmomco::quaray(0.99, ray1))
#abline(h=lmomco::quaray(0.99, ray2))

"PLcop" <-
function(u, v, para=NULL, ...) {
    TT <- para[1]
    if(is.null(para)) {
       warning("Empty para argument, need value on [0,Inf]")
       return()
    }
    if(TT < 0) {
       warning("Theta < 0, invalid parameter")
       return()
    }
    if(TT == 1)         return(  u*v)  # the product copula
    if(TT == 0)         return(W(u,v)) # lower copula bounds
    if(! is.finite(TT)) return(M(u,v)) # upper copula bounds
    cop <- 1 + (TT-1)*(u+v)
    suppressWarnings( tmp <- sqrt(cop^2 - 4*u*v*TT*(TT-1)) ) # December 2023
    # In sqrt(cop^2 - 4 * u * v * TT * (TT - 1)) : NaNs produced
    tmp[is.nan(tmp)] <- 0   # December 2023
    cop <- cop - tmp        # December 2023
    cop <- cop / (2*(TT-1))
    cop[cop < 0] <- 0       # December 2023
    cop[cop > 1] <- 1       # December 2023
    return(cop)
}

"PLACKETTcop" <- function(u, v, para=NULL, ...) {
   PLcop(u, v, para=para, ...)
}

"densityCOPplot" <-
 function(cop=NULL, para=NULL, deluv=0.002,
          getmatrix=c("none", "cdenzz", "cden"), n=0,
          ploton=TRUE, snv=TRUE, origins=TRUE,
          contour.col=1, contour.lwd=1.5, ...) {
   getmatrix <- match.arg(getmatrix)

   U <- V <- seq(deluv, 1-deluv, by=+deluv)
   nU <- length(U); nV <- length(V)
   qU <-  qnorm(U); qV <-  qnorm(V)
   cdenzz <- cden <- matrix(ncol=nU, nrow=nV)

   ZV <- dnorm(qnorm(V))
   for(i in 1:nU) {
      uvals      <- rep(U[i], nV)
      dcop       <- densityCOP(uvals, V, cop=cop, para=para, ...)
      cden[,i]   <- dcop
      cdenzz[,i] <- dcop * dnorm(qnorm(uvals)) * ZV
   }
   rownames(cden)   <- V;  colnames(cden)   <- U
   rownames(cdenzz) <- V;  colnames(cdenzz) <- U

   if(ploton) {
      if(snv) {
        xlim <- ylim <- c(-1, 1)*rep(max(abs(range(c(qU, qV))), 2))
        xlab <- "STANDARD NORMAL VARIATE OF U"
        ylab <- "STANDARD NORMAL VARIATE OF V"
      } else {
        xlim <- ylim <- c(0, 1)
        xlab <- "U, NONEXCEEDANCE PROBABILITY"
        ylab <- "V, NONEXCEEDANCE PROBABILITY"
      }
      plot(xlim, ylim, type="n", xlim=xlim, ylim=ylim, xlab=xlab, ylab=ylab)
      if(snv & origins) {
         abline(h=0, lty=2)
         abline(v=0, lty=2)
      }
   }
   if(n > 0) {
      UV <- simCOP(n=n, cop=cop, para=para, ploton=FALSE, points=TRUE, snv=snv,
                   pch=16, col=rgb(0, 0, 1, 0.3), cex=0.5, ...)
   }
   # The transposition t() is CRITICAL!!!!!!!! Let the 2nd example in
   # densityCOPplot(), which is highly asymmetrical be the canonical demonstration.
   if(snv) {
     contour(x=qU, y=qV, z=t(cdenzz), lwd=contour.lwd, cex=2, add=TRUE, col=contour.col, ...)
   } else {
     contour(x=U, y=V,   z=t(cdenzz), lwd=contour.lwd, cex=2, add=TRUE, col=contour.col, ...)
   }

   if(getmatrix == "cdenzz") {
      return(cdenzz)
   } else if(getmatrix == "cden") {
      return(cden)
   }
}

"level.setCOP2" <-
function(cop=NULL, para=NULL, getlevel=NULL, delv=0.001, lines=FALSE, ...) {
   zz <- level.curvesCOP2(cop=cop, para=para, getlevel=getlevel, delt=NULL,
                          ploton=FALSE, plotMW=FALSE, lines=lines, delv=delv, ramp=FALSE, ...)
   return(zz)
}

"isCOP.radsym" <-
function(cop=NULL, para=NULL, delta=0.005, tol=1e-4, ...) {
  TT <- seq(0+delta, 1-delta, by=delta)
  tmp <- sapply(TT, function(u) { sapply(TT, function(v) {
             return(u + v - 1 + COP(1-u,1-v, cop=cop, para=para, ...) -
                                COP(  u,  v, cop=cop, para=para, ...))
                               } ) })
  ifelse(any(abs(tmp) > tol), return(FALSE), return(TRUE))
}

"bilmoms" <-
function(cop=NULL, para=NULL, only.bilmoms=FALSE,
                   n=1E5, halton=TRUE, usetime=TRUE, ...) {

   if(is.null(cop)) {
      warning("must have copula argument specified, returning NULL")
      return(NULL)
   }

   if(halton) {
     #if(! exists(".Random.seed")) tmp <- runif(1) # insures definition of .Random.seed
     #seed <- sample(.Random.seed, 1)
     uv   <- randtoolbox::halton(n=n, dim=2, usetime=usetime)
   } else {
     uv   <- matrix(data=runif(2*n), ncol=2)
   }

   rho  <- rhoCOP(cop=cop, para=para, ...) # d1 = rho/6 (theoretically)

   cuv  <- COP(uv[,1], uv[,2], cop=cop, para=para, ...)

   d1  <- mean(cuv)
   d1  <- 2*d1 - 1/2
   d2  <- mean((                 2*uv[,2]   -  1)*cuv)
   d2  <- 6*d2 - 1/2
   d3  <- mean(( 60*uv[,2]^2 -  60*uv[,2]   + 12)*cuv)
   d3  <-   d3 - 1/2
   d4  <- mean((280*uv[,2]^3 - 420*uv[,2]^2 + 180*uv[,2] - 20)*cuv)
   d4  <-   d4 - 1/2
   err1 <- abs(d1 - rho/6) # The 1/2 is for an unbiased mean
   deltasX1wrtX2 <- c(d1, d2, d3, d4)
   names(deltasX1wrtX2) <- c("BiVarLM:del1[12]", "BiVarLM:del2[12]",
                             "BiVarLM:del3[12]", "BiVarLM:del4[12]")

   d1  <- mean(cuv)
   d1  <- 2*d1 - 1/2
   d2  <- mean((                 2*uv[,1]   -  1)*cuv)
   d2  <- 6*d2 - 1/2
   d3  <- mean(( 60*uv[,1]^2 -  60*uv[,1]   + 12)*cuv)
   d3  <-   d3 - 1/2
   d4  <- mean((280*uv[,1]^3 - 420*uv[,1]^2 + 180*uv[,1] - 20)*cuv)
   d4  <-   d4 - 1/2
   err2 <- abs(d1 - rho/6) # The 1/2 is for an unbiased mean
   deltasX2wrtX1 <- c(d1, d2, d3, d4)
   names(deltasX2wrtX1) <- c("BiVarLM:del1[21]", "BiVarLM:del2[21]",
                             "BiVarLM:del3[21]", "BiVarLM:del4[21]")
   err <- (err1 + err2)/2
   if(only.bilmoms) {
     zz <- list(bilmomUV=deltasX1wrtX2, bilmomVU=deltasX2wrtX1,
                 error.rho=err, source="bilmoms")
     return(zz)
   }

   ulmoms <- lmomco::lmoms(uv[,1], nmom=5)
   vlmoms <- lmomco::lmoms(uv[,2], nmom=5)

   lcX1X2 <- c(NA, deltasX1wrtX2*6) # L-comoments of X1 wrt X2
   lcX2X1 <- c(NA, deltasX2wrtX1*6) # L-comoments of X2 wrt X1

   L1 <- matrix(c(ulmoms$lambdas[1],        NA,        NA, vlmoms$lambdas[1]), ncol=2)
   # The diagonal should be filled with 1/2 if the n is suitable large because 1/2 is
   # the mean of a uniform random variable.
   L2 <- matrix(c(ulmoms$lambdas[2], lcX2X1[2]*ulmoms$lambdas[2],
                                     lcX1X2[2]*vlmoms$lambdas[2],
                  vlmoms$lambdas[2]), ncol=2)
   # The diagonal should be filled with 1/6 if the n is suitable large because 1/6 is
   # the L-scale of a uniform random variable.

   T2 <- matrix(c(                1, lcX2X1[2], lcX1X2[2],                 1), ncol=2)
   T3 <- matrix(c( ulmoms$ratios[3], lcX2X1[3], lcX1X2[3],  vlmoms$ratios[3]), ncol=2)
   T4 <- matrix(c( ulmoms$ratios[4], lcX2X1[4], lcX1X2[4],  vlmoms$ratios[4]), ncol=2)
   T5 <- matrix(c( ulmoms$ratios[5], lcX2X1[5], lcX1X2[5],  vlmoms$ratios[5]), ncol=2)

   zz <- list(bilmomUV=deltasX1wrtX2, bilmomVU=deltasX2wrtX1,
               error.rho=err,
               bilcomoms=list(L1=L1, L2=L2, T2=T2, T3=T3, T4=T4, T5=T5),
               source="bilmoms")
   return(zz)
}

"med.regressCOP2" <-
function(v=seq(0.01,0.99, by=0.01), cop=NULL, para=NULL, level=NA, ...) {
   if(is.null(cop)) {
      warning("must have copula argument specified, returning NULL")
      return(NULL)
   }
   UV <- qua.regressCOP2(f=0.5, v=v, cop=cop, para=para, ...)
   if(is.na(level)) return(UV)
   if(length(level) > 1) {
      warning("only the first value of 'level' is used")
      level <- level[1]
   }
   lo <- ifelse(level > 0.5, (1-level)/2, level/2)
   tmp <- UV$U; UV$U <- NULL # copy and then erase
   UV$Ulwr <- qua.regressCOP2(f=  lo, v=v, cop=cop, para=para, ...)$U
   UV$U    <- tmp # this gets it in the middle
   UV$Uupr <- qua.regressCOP2(f=1-lo, v=v, cop=cop, para=para, ...)$U
   return(UV)
}

"isCOP.RTI" <-
function(cop=NULL, para=NULL, wrtV=FALSE, delta=0.005, ...) {
  TT <- seq(0+delta, 1-delta, by=delta)
  is.RTI <- TRUE
  if(wrtV) {
    for(u in TT) {
      derC  <- sapply(TT, function(v) { return(derCOP2(u,v, cop=cop, para=para, ...))})
      CdivT <- sapply(TT, function(v) { return(u - cop(u,v, para=para, ...)/(1 - v))})
      if(any(derC < CdivT)) { is.RTI <- FALSE; break }
    }
  } else {
    for(v in TT) {
      derC  <- sapply(TT, function(u) { return(derCOP(u,v, cop=cop, para=para, ...))})
      CdivT <- sapply(TT, function(u) { return(v - cop(u,v, para=para, ...)/(1 - u))})
      if(any(derC < CdivT)) { is.RTI <- FALSE; break }
    }
  }
  return(is.RTI)
}

"statTn" <-
 function(u, v=NULL, cop=NULL, para=NULL, p=2, proot=FALSE, ...) {
   if(is.null(v)) {
     v <- u[,2]
     u <- u[,1]
   }
   proot  <- ifelse(proot, 1/p, 1)
   parcop <- COP(u, v, cop=cop, para=para, ...)
   # The empirical is called 2nd because it can be CPU intensive
   # and if the parametric call is going to error, let us have it
   # do it first and not waste users time.
   empcop <- EMPIRcopdf(para=data.frame(u,v), ...)$empcop
   (sum((parcop - empcop)^p))^proot
}

\encoding{utf8}
\name{isCOP.PQD}
\alias{isCOP.PQD}
\title{The Positively Quadrant Dependency State of a Copula}
\description{
Numerically determine the global property of the \emph{positively quadrant dependency} (PQD) characteristic of a copula as described by Nelsen (2006, p. 188). The random variables \eqn{X} and \eqn{Y} are PQD if for all \eqn{(x,y)} in \eqn{\mathcal{R}^2} when
\eqn{H(x,y) \ge F(x)G(x)} for all \eqn{(x,y)} in \eqn{\mathcal{R}^2}
and thus by the copula \eqn{\mathbf{C}(u,v) \ge uv} for all \eqn{(u,v)} in \eqn{\mathcal{I}^2}. Alternatively, this means that \eqn{\mathbf{C}(u,v) \ge \mathbf{\Pi}}, and thus it can be said that it is globally \dQuote{greater} than independence (\eqn{uv = \Pi}; \code{\link{P}}).

Nelsen (2006) shows that a copula is PQD when
\deqn{0 \le \beta_\mathbf{C} \mbox{,\ } 0 \le \gamma_\mathbf{C}\mbox{,\ and\ } 0 \le \rho_\mathbf{C} \le 3\tau_\mathbf{C}\mbox{,}}
where \eqn{\beta_\mathbf{C}}, \eqn{\gamma_\mathbf{C}}, \eqn{\rho_\mathbf{C}}, and \eqn{\tau_\mathbf{C}} are various copula measures of association or concordance that are respectively described in \code{\link{blomCOP}}, \code{\link{giniCOP}}, \code{\link{rhoCOP}}, and \code{\link{tauCOP}}.
The concept of negatively quadrant dependency (NQD) is the reverse: \eqn{\mathbf{C}(u,v) \le \mathbf{\Pi}} for all \eqn{(u,v)} in \eqn{\mathcal{I}^2}; so NQD is globally \dQuote{smaller} than independence.

Conceptually, PQD is related to the probability that two random variables are simultaneously small (or simultaneously large) is at least as great as it would be if they were \emph{independent}. The graph of a PQD copula lies on or above the copulatic surface of the \emph{independence copula} \eqn{\mathbf{\Pi}}, and conversely a NQD copula lies on or below \eqn{\mathbf{\Pi}}.

Albeit a \dQuote{global} property of a copula, there can be \dQuote{local} variations in the PQD/NQD state. Points in \eqn{\mathcal{I}^2} where \eqn{\mathbf{C}(u,v) - \mathbf{\Pi} \ge 0} are locally PQD, whereas points in \eqn{\mathcal{I}^2} where \eqn{\mathbf{C}(u,v) - \mathbf{\Pi} \le 0} and locally NQD. Lastly, readers are directed to the last examples in \code{\link{wolfCOP}} because as those examples involve the copulatic difference from independence \eqn{\mathbf{C}(u,v) - \mathbf{\Pi} = \mathbf{C}(u,v) - \mathbf{\Pi}} with 3-D renderings.
}
\usage{
isCOP.PQD(cop=NULL, para=NULL, uv=NULL, empirical=FALSE, verbose=TRUE, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{uv}{An optional \R \code{data.frame} of \eqn{U} and \eqn{V} nonexceedance probabilities \eqn{u} and \eqn{v} for the random variables \eqn{X} and \eqn{Y}. This argument triggers different value return behavior (see \bold{Value});}
  \item{empirical}{A logical that will use sample versions for \emph{Gini Gamma}, \emph{Spearman Rho}, and \emph{Kendall Tau}. This feature is \emph{only} applicable if the copula is empirical and therefore the \code{para} argument is the \code{data.frame} of \eqn{u} and \eqn{v}, which will be passed along to sample version functions instead of copula (see \bold{Note});}
  \item{verbose}{A logical that will report the four concordance measures; and}
  \item{...}{Additional arguments to pass, which are then passed to subordinate functions.}
}
\value{
   If \code{uv=NULL} then a logical for the global property of PQD is returned but if argument \code{uv} is a \code{data.frame}, then an \R \code{list} is returned, and that list holds the global condition in \code{global.PQD} and  local condition assessments in \code{local.PQD} and \code{local.NQD}.
}
\note{
The function \code{isCOP.PQD} will try \code{brute} force computations if subordinate calls to one or more functions fails. The user can use \code{...} to set the \code{delta} argument for \code{\link{giniCOP}}, \code{\link{rhoCOP}}, and (or) \code{\link{tauCOP}}.

This function is not guaranteed to work using a \emph{bivariate empirical copula} such as the following operation: \code{copPQD(cop=EMPIRcop, para=the.data)}. An evidently open problem for \pkg{copBasic} is how to support PQD assessment (either globally or locally) for empirical copulas. The \eqn{\tau_\mathbf{C}} for the bivariate empirical copula example \code{brute=TRUE|FALSE} to unity and \eqn{\gamma_\mathbf{C}} and \eqn{\rho_\mathbf{C}} reach maximum number of subdivisions on the numerical integration and thus fail. If an empirical bivariate copula is \dQuote{Tau'd} to itself, is \eqn{\tau_\mathbf{C} \equiv 1} guaranteed? The \eqn{\tau_\mathbf{C}} computation relies on numerical partial derivatives of the copula, whereas the \eqn{\gamma_\mathbf{C}} and \eqn{\rho_\mathbf{C}} use the copula itself. It seems in the end that use of sample versions of \eqn{\gamma_\mathbf{C}}, \eqn{\rho_\mathbf{C}}, and \eqn{\tau_\mathbf{C}} would be appropriate and leave the \eqn{\beta_\mathbf{C}} as either copula or direct sample computation (see \bold{Examples}).

\emph{SPECIAL DEMONSTRATION 1}---Given the following,
\preformatted{
  para <- list(cop1=PLACKETTcop, cop2=PLACKETTcop, para1=c(14.5),para2=c(1.45),
               alpha=0.51, beta=0.15, kappa=0.45, gamma=0.78)
  D <- simCOP(n=500, cop=composite3COP, para=para, cex=0.5, col=1, pch=16)
}
the two different call types to \code{isCOP.PQD} for an empirical copula are illustrative:
\preformatted{
  global.only <- isCOP.PQD(cop=EMPIRcop, para=D, empirical=TRUE)
}
and
\preformatted{
  PQD.list <- isCOP.PQD(cop=EMPIRcop, para=D, empirical=TRUE, uv=D)
  points(D, col=PQD.list$local.PQD+2, lwd=2) # red (if present) is local NQD
}
which in the former only returns the global PQD and the later returns an \R \code{list} with global (\code{global.PQD}), local (\code{local.PQD} as well as \code{local.NQD}), and the four statistics (\code{beta} \eqn{\beta_\mathbf{C}}, \code{gamma} \eqn{\gamma_\mathbf{C}}, \code{rho} \eqn{\rho_\mathbf{C}}, \code{tau} \eqn{\tau_\mathbf{C}}) used to determine global PQD.


\emph{SPECIAL DEMONSTRATION 1}---Lastly, the \code{ctype=}\code{"bernstein"} argument to the empirical copula can be used. Repeated iterations of the following will show that local quadrant dependency can appear slightly different when the \code{bernstein} argument is present. The simulation sample size is reduced considerably for this second example because of the CPU effort triggered by the \emph{Bernstein extension} (see \code{\link{EMPIRcop}}) having been turned on.
\preformatted{
  para <- list(cop1=PLACKETTcop,  cop2=PLACKETTcop, para1=14.5, para2=1.45,
               alpha=0.51, beta=0.15, kappa=0.45, gamma=0.78)
  D <- simCOP(n=50, cop=composite3COP, para=para, cex=0.5, col=1, pch=16)
  PQD.A<- isCOP.PQD(cop=EMPIRcop, para=D, empirical=TRUE, uv=D)
  points(D, col=PQD.A$local.PQD+2, lwd=2) # red (if present) is local NQD
  PQD.B<- isCOP.PQD(cop=EMPIRcop,para=D,empirical=TRUE,uv=D,ctype="bernstein")
  points(D, col=PQD.B$local.PQD+2, lwd=1, pch=3, cex=1.5)
}
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{blomCOP}}, \code{\link{giniCOP}}, \code{\link{rhoCOP}}, \code{\link{tauCOP}}, \code{\link{isCOP.LTD}}, \code{\link{isCOP.RTI}}}
\examples{
\dontrun{
isCOP.PQD(cop=PSP) # TRUE}

\dontrun{
# Example concerning Empirical Bivariate Copula and sample versions for comparison.
set.seed(10); n <- 1000
para <- list(cop1=PLACKETTcop, cop2=PLACKETTcop, para1=0.145,  para2=1.45,
             alpha=0.81, beta=0.8)
D <- simCOP(n=n, cop=composite2COP, para=para, cex=0.5, col=rgb(0,0,0,0.2), pch=16)
#tauCOP(cop=EMPIRcop, para=D)   # ??? but == 1
cor(D$U, D$V, method="kendall") # -0.3224705
blomCOP(cop=EMPIRcop, para=D)   # -0.332
giniCOP(cop=EMPIRcop, para=D)   # -0.3692037
GINI <- sum(abs(rank(D$U)+rank(D$V)-n-1)) - sum(abs(rank(D$U)-rank(D$V)))
print(GINI/as.integer(n^2/2))   # -0.369996
rhoCOP(cop=EMPIRcop, para=D)    # ??? but fails
cor(D$U, D$V, method="spearman")      # -0.456694
lmomco::lcomoms2(D)$T2     #  1.0000000 -0.4568357
                           # -0.4567859  1.0000000}
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}

\encoding{utf8}
\name{blomCOP}
\alias{blomCOP}
\title{The Blomqvist Beta of a Copula}
\description{
Compute the \emph{Blomqvist Beta} \eqn{\beta_\mathbf{C}} of a copula (Nelsen, 2006, p. 182), which is defined at the middle or center of \eqn{\mathcal{I}^2} as

\deqn{\beta_\mathbf{C} = 4\times\mathbf{C}\biggl(\frac{1}{2},\frac{1}{2}\biggr) - 1\mbox{,}}

where the \eqn{u = v = 1/2} and thus shows that \eqn{\beta_\mathbf{C}} is based on the \emph{median joint probability}. The Blomqvist Beta is also called the \emph{medial correlation coefficient}. Nelsen also reports that \dQuote{although, the Blomqvist Beta depends only on the copula only through its value at the center of \eqn{\mathcal{I}^2}, but that [\eqn{\beta_\mathbf{C}}] nevertheless often provides an accurate approximation to both \emph{Spearman Rho} \code{\link{rhoCOP}} and \emph{Kendall Tau} \code{\link{tauCOP}}.}  Kendall Tau \eqn{\tau_\mathbf{C}}, \emph{Gini Gamma} \eqn{\gamma_\mathbf{C}}, and Spearman Rho \eqn{\rho_\mathbf{C}} in relation to \eqn{\beta_\mathbf{C}} satisfy the following inequalities (Nelsen, 2006, exer. 5.17, p. 185):
\deqn{\frac{1}{4}(1 + \beta_\mathbf{C})^2  - 1 \le \tau_\mathbf{C} \le 1 - \frac{1}{4}(1 - \beta_\mathbf{C})^2\mbox{,}}
\deqn{\frac{3}{16}(1 + \beta_\mathbf{C})^3  - 1 \le \rho_\mathbf{C} \le 1 - \frac{3}{16}(1 - \beta_\mathbf{C})^3\mbox{, and}}
\deqn{\frac{3}{8}(1 + \beta_\mathbf{C})^2  - 1 \le \tau_\mathbf{C} \le 1 - \frac{3}{8}(1 - \beta_\mathbf{C})^2\mbox{.}}

A curious aside (Joe, 2014, p. 164) about the \emph{Gaussian copula} is that \emph{Blomqvist Beta} is equal to \emph{Kendall Tau} (\code{\link{tauCOP}}): \eqn{\beta_\mathbf{C} = \tau_\mathbf{C}} (see \bold{Note} in \code{\link{med.regressCOP}} for a demonstration). Finally, a version of Blomqvist Beta defined outside the median is provided by \code{\link{blomCOPss}}.
}
\usage{
blomCOP(cop=NULL, para=NULL, as.sample=FALSE,
                  ctype=c("joe", "weibull", "hazen", "1/n",
                                 "bernstein", "checkerboard"), ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{as.sample}{A logical controlling whether an optional \R \code{data.frame} in \code{para} is used to compute the \eqn{\hat\beta_\mathbf{C}} (see \bold{Note});}
  \item{ctype}{Argument of the same as \code{\link{EMPIRcop}} with the exception of the \code{"joe"} specific to the documentation here. The other choices trigger and are given over to the empirical copula; and}
  \item{...}{Additional arguments to pass to the copula or down to \code{\link{EMPIRcop}} if a sample version had been requested.}
}
\value{
  The value for \eqn{\beta_\mathbf{C}} or sample \eqn{\hat\beta_n} is returned.
}
\note{
The sample \eqn{\hat\beta_n} is most efficiently computed (Joe, 2014, p. 57) by
\deqn{\hat\beta_n = \frac{2}{n} \sum_{i=1}^{n} \mathbf{1}\biggl([r_{i1} - (1 + n)/2]\times
                                                          [r_{i2} - (1 + n)/2] \ge 0\biggr) -
                    1\mbox{,}}
where \eqn{r_{i1}, r_{i2}} are the ranks of the data for \eqn{i = 1, \ldots n}, and \eqn{\mathbf{1}(.)} is an \emph{indicator function} scoring 1 if condition is true otherwise zero. However, the Joe sample estimator is not fully consistent (or vice versa) with the various versions of the empirical copula, \eqn{\mathbf{C}_n}, (\code{\link{EMPIRcop}}) (see the last example in \bold{Examples}). Also, the nature of even and odd sample sizes controls how the median is computed and the issue of samples lying on the median lines in \eqn{U} and \eqn{V} (Genest \emph{et al.}, 2013). The argument \code{ctype} supports triggers to the \eqn{\mathbf{C}_n} in lieu of the Joe sample estimator shown in this documentation.
}
\references{
Genest, C., Carabar\enc{í}{i}n-Aguirre, A., and Harvey, F., 2013, Copula parameter estimation using\cr Blomqvist's beta: Journal de la Soci\eqn{é}{e}t\enc{é}{e} Fran\enc{ç}{c}aise de Statistique, v. 154, no. 1, pp. 5--24.

Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{blomCOPss}}, \code{\link{blomatrixCOPdec}}, \code{\link{blomatrixCOPiqr}},
         \code{\link{footCOP}}, \code{\link{giniCOP}}, \code{\link{hoefCOP}},
         \code{\link{rhoCOP}}, \code{\link{tauCOP}}, \code{\link{wolfCOP}},
         \code{\link{joeskewCOP}}, \code{\link{uvlmoms}}
}
\examples{
blomCOP(cop=PSP) # 1/3 precisely

\dontrun{
# Nelsen (2006, exer. 5.17, p. 185) : All if(...) are TRUE
B <- blomCOP(cop=N4212cop, para=2.2); Bp1 <- 1 + B; Bm1 <- 1 - B
G <- giniCOP(cop=N4212cop, para=2.2); a <- 1/4; b <- 3/16; c <- 3/8
R <-  rhoCOP(cop=N4212cop, para=2.2)
K <-  tauCOP(cop=N4212cop, para=2.2, brute=TRUE) # numerical issues without brute
if( a*Bp1^2 - 1 <= K & K <= 1 - a*Bm1^2 ) print("TRUE") #
if( b*Bp1^3 - 1 <= R & R <= 1 - b*Bm1^3 ) print("TRUE") #
if( c*Bp1^2 - 1 <= G & G <= 1 - c*Bm1^2 ) print("TRUE") #}

\dontrun{
# A demonstration of a special feature of blomCOP for sample data.
# Joe (2014, p. 60; table 60) has 0.749 for GHcop(tau=0.5); n*var(hatB) as n-->infinity
set.seed(1)
theta <- GHcop(tau=0.5)$para; B <- blomCOP(cop=GHcop, para=theta); n <- 1000
H <- sapply(1:1000, function(i) { # Let us test that with pretty large sample size:
	                blomCOP(para=rCOP(n=n, cop=GHcop, para=theta), as.sample=TRUE) })
print(n*var(B-H)) # For 1,000 sims of size n : 0.789, nearly matches Joe's result }

\dontrun{
# Joe (2014, p. 57) says that sqrt(n)(B-HatBeta) is Norm(0, 1 - B^2)
set.seed(1)
n <- 10000; B <- blomCOP(cop=PSP) # Beta = 1/3
H <- sapply(1:100, function(i) { message(i,"-", appendLF=FALSE)
	               blomCOP(para=rCOP(n=n, cop=PSP), as.sample=TRUE) })
lmomco::parnor(lmomco::lmoms(sqrt(n)*(H-B))) # mu = -0.038; sigma = 0.970
# Joe (2014) : sqrt(1-B^2) == standard deviation (sigma) : (1-(1/3)^2) approx 0.973 }

\dontrun{
nn <- 200; set.seed(1)
UV <- simCOP(n=nn+1, cop=PSP, graphics=FALSE)
for(n in nn:(nn+1)) {
  if(as.logical(n \%\% 2)) { # in source \ percent \ percent for latex
    message("Blomquist Betas for an odd  sample size n=", n)
    uv <- UV
  } else {
    message("Blomquist Betas for an even sample size n=", n)
    uv <- UV[-(nn+1), ] # remove the last and 'odd' indexed value to make even
  }
  message(c(" Joe2014: ", blomCOP(as.sample=TRUE, para=uv, ctype="joe"         )))
  message(c(" Weibull: ", blomCOP(as.sample=TRUE, para=uv, ctype="weibull"     )))
  message(c(" Hazen:   ", blomCOP(as.sample=TRUE, para=uv, ctype="hazen"       )))
  message(c(" 1/n:     ", blomCOP(as.sample=TRUE, para=uv, ctype="1/n"         )))
  message(c(" Bernstn: ", blomCOP(as.sample=TRUE, para=uv, ctype="bernstein"   )))
  message(c(" ChckBrd: ", blomCOP(as.sample=TRUE, para=uv, ctype="checkerboard")))
}
# Blomquist Betas for an even sample size n=200
#  Joe2014: 0.32
#  Weibull: 0.32
#  Hazen:   0.32
#  1/n:     0.32
#  Bernstn: 0.323671819423416
#  ChckBrd: 0.32
# Blomquist Betas for an odd  sample size n=201
#  Joe2014: 0.323383084577114
#  Weibull: 0.333333333333333
#  Hazen:   0.333333333333333
#  1/n:     0.293532338308458
#  Bernstn: 0.327747577290946
#  ChckBrd: 0.313432835820896 #}
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{medial correlation coefficient}
\concept{medial correlation}
\keyword{Joe (2014) Examples and Exercises}
\keyword{Nelsen (2006) Examples and Exercises}
\keyword{Gaussian copula}
\keyword{Normal copula}
\keyword{Blomqvist Beta}

\encoding{utf8}
\name{GHcop}
\alias{GHcop}
\title{The Gumbel--Hougaard Extreme Value Copula}
\description{
\emph{SYMMETRIC GUMBEL-HOUGAARD}---The \emph{Gumbel--Hougaard copula} (Nelsen, 2006, pp. 118 and 164) is

\deqn{\mathbf{C}_{\Theta}(u,v) = \mathbf{GH}(u,v) = \mathrm{exp}\biggl\{-\bigl[(-\log u)^\Theta+(-\log v)^\Theta\bigr]^{1/\Theta}\biggr\}\mbox{,}}

where \eqn{\Theta \in [1 , \infty)}. The \eqn{\mathbf{GH}} copula is a \emph{bivariate extreme value copula} (\eqn{BEV}). The parameter \eqn{\Theta} is readily estimated using a \emph{Kendall Tau} (say a sample version \eqn{\hat\tau}) where the \eqn{\tau} of the copula (\eqn{\tau_\mathbf{C}}) is defined as

\deqn{\tau_\mathbf{C} = \frac{\Theta - 1}{\Theta} \rightarrow \Theta = \frac{1}{1-\tau}\mbox{.}}
The copula is readily extended into \eqn{d} dimensions by
\deqn{\mathbf{C}_{\Theta}(u,v) = \mathrm{exp}\biggl\{-\bigl[(-\log u_1)^\Theta+\cdots+(-\log u_d)^\Theta\bigr]^{1/\Theta}\biggl\}\mbox{.}}

However, such an implementation is not available in the \pkg{copBasic} package.

Every Gumbel--Hougaard copula is a \emph{multivariate extreme value} (\eqn{MEV}) copula, and hence useful in analysis of extreme value distributions. The Gumbel--Hougaard copula is the \emph{only} Archimedean \eqn{MEV} (Salvadori \emph{et al.}, 2007, p. 192). The Gumbel--Hougaard copula has respective \emph{lower-} and \emph{upper-tail dependency} parameters of \eqn{\lambda^L = 0} and \eqn{\lambda^U = 2 - 2^{1/\Theta}}, respectively. Nelsen (2006, p. 96) shows that \eqn{\mathbf{C}^r_\theta(u^{1/r}, v^{1/r}) = \mathbf{C}_\theta(u,v)} so that every Gumbel--Hougaard copula has a property known as \emph{max-stable}. A \emph{dependence measure} uniquely defined for \eqn{BEV} copulas is shown under \code{\link{rhobevCOP}}.

A comparison through simulation between Gumbel--Hougaard implementations by the \R packages \pkg{acopula}, \pkg{copBasic}, \pkg{copula}, and \pkg{Gumbel} is shown in the \bold{Examples} section. At least three divergent techniques for random variate generation are used amongst those packages. The simulations also use \pkg{copBasic}-style random variate generation (conditional simulation) using an analytical-numerical hybrid solution to conditional inverse described in the \pkg{Note} section.

\emph{TWO-PARAMETER GUMBEL--HOUGAARD}---A \emph{permutation symmetric} (\code{\link{isCOP.permsym}}) but almost certainly \emph{radial asymmetric} (\code{\link{isCOP.radsym}}) version of the copula is readily constructed (Brahimi \emph{et al.}, 2015) into a two-parameter version:

\deqn{\mathbf{C}(u,v; \beta_1, \beta_2) =
\biggl[
\biggl(\bigl(u^{-\beta_2} -1\bigr)^{\beta_1} +
       \bigl(v^{-\beta_2} -1\bigr)^{\beta_1}
\biggr)^{1/\beta_1} + 1
\biggr]^{-1/\beta_2}\mbox{,}}

where \eqn{\beta_1 \ge 1} and \eqn{\beta_2 > 0}. Both parameters controls the general level of association, whereas parameter \eqn{\beta_2} can be thought of as controlling left-tail dependency (\code{\link{taildepCOP}}, \eqn{\lambda^{[U\mid L]}_{(\beta_1, \beta_2)}}; \emph{e.g.}
\eqn{\lambda^U_{(1.5; \beta_2)} = 0.413} for all \eqn{\beta_2} but
\eqn{\lambda^L_{(1.5; 0.2)} = 0.811} and
\eqn{\lambda^L_{(1.5; 2.2)} = 0.099}. Brahimi \emph{et al.} (2015) report a \emph{Spearman Rho} (\code{\link{rhoCOP}}) for a \eqn{\mathbf{GH}_{(1.5, 0.2)}(u,v)} is 0.5, which is readily confirmed in \pkg{copBasic} by the function call \code{rhoCOP(cop=GHcop, para=c(1.5,0.2))}. The two-parameter \eqn{\mathbf{GH}} is triggered if the length of the \code{para} argument is exactly 2.

\emph{ASYMMETRIC GUMBEL--HOUGAARD}---An asymmetric version of the copula is readily constructed (Joe, 2014, p. 185--186) into a three-parameter version with \emph{Marshall--Olkin} copulas (\code{\link{MOcop}}) on the boundaries:
\deqn{\mathbf{C}(u,v; \Theta, \pi_2, \pi_3) = \mathrm{exp}[-\mathcal{A}(-\log u, -\log v; \Theta, \pi_2, \pi_3)]\mbox{,}}
where \eqn{\Theta \ge 1} as before, \eqn{0 \le \pi_2,  \pi_3 \le 1}, and

\deqn{\mathcal{A}(x, y; \Theta, \pi_2, \pi_3) = \bigl[(\pi_2 x)^\Theta + (\pi_3 y)^\Theta\bigr]^{1/\Theta} + (1-\pi_2)x + (1-\pi_3)y\mbox{.}}

The asymmetric \eqn{\mathbf{GH}} is triggered if the length of the \code{para} argument is exactly 3. The \code{GHcop} function provides no mechanism for estimation of the parameters for the asymmetric version. Reviewing simulations, the bounds on the \eqn{\pi} parameters in Joe (2014, p. 185) \dQuote{[\eqn{0 \le \pi_2 < \pi_3 \le 1}]} might be incorrect---by Joe back referencing to Joe (2014, eq. 4.35, p. 183) the \eqn{\pi}-limits as stated for \pkg{copBasic} are shown. An algorithm for parameter estimation for the asymmetric \eqn{\mathbf{GH}} using two different measures of \emph{bivariate skewness} as well as an arbitrary \emph{measure of association} is shown in section \bold{Details} in \code{\link{joeskewCOP}}.
}
\usage{
GHcop(u, v, para=NULL, tau=NULL, tau.big=0.985, cor=NULL, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (single element or triplet) of parameters---the \eqn{\Theta} parameter of the copula;}
  \item{tau}{Kendall Tau \eqn{\tau} from which to estimate the parameter \eqn{\Theta};}
  \item{tau.big}{The largest value for \eqn{\tau_\mathbf{C}} prior to switching to the \eqn{\mathbf{M}} copula applicable to the the symmetric version of this copula;}
  \item{cor}{A \pkg{copBasic} syntax for \dQuote{the correlation coefficient} suitable for the copula---a synonym for \code{tau}; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned using the \eqn{\Theta} as set by argument \code{para}. Alternative returned values are possible: (1) If \code{para=NULL} and \code{tau} is set, then \eqn{\tau_\mathbf{C} \rightarrow \Theta} and an \R \code{list} is returned. (2) If \code{para=NULL} and \code{tau=NULL}, then an attempt to estimate \eqn{\Theta} from the \code{u} and \code{v} is made by \eqn{\mathrm{cor}(u,v)_\tau \rightarrow \tau_\mathbf{C} \rightarrow \Theta} by either trigger using \code{cor(u,v, method="kendall")} in \R, and an \R \code{list} is returned. The possibly returned \code{list} has the following elements:
  \item{para}{The computed \eqn{\Theta} from the given bivariate data in \code{para}; and}
  \item{tau}{The sample estimate of \eqn{\tau}.}
}
\details{
Numerical experiments seem to indicate for \eqn{\tau_\mathbf{C} > 0.985} that failures in the numerical partial derivatives in \code{\link{derCOP}} and \code{\link{derCOP2}} result---a \eqn{\tau_\mathbf{C}} this large is indeed \emph{large}. As \eqn{\Theta \rightarrow \infty} the Gumbel--Hougaard copula becomes the \emph{\enc{Fréchet}{Frechet}--Hoeffding upper-bound copula} \eqn{\mathbf{M}} (see \code{\link{M}}). A \eqn{\tau_\mathbf{C} \approx 0.985} yields \eqn{\Theta \approx 66 + 2/3}, then for \eqn{\Theta > 1/(1-\tau_\mathbf{C})} flips over to the \eqn{\mathbf{M}} copula with a warning issued.
}
\note{
\emph{SYMMETRIC GUMBEL--HOUGAARD} --- A function for the derivative of the copula (Joe, 2014, p. 172) given \eqn{u} is
\preformatted{
  "GHcop.derCOP" <- function(u, v, para=NULL, ...) {
     x <- -log(u); y <- -log(v)
     A <- exp(-(x^para + y^para)^(1/para)) * (1 + (y/x)^para)^(1/para - 1)
     return(A/u)
  }
}
that can be tested by the following
\preformatted{
  Theta <- 1 / (1 - 0.15) # a Kendall Tau of 0.15
  GHcop.derCOP(     0.5, 0.75, para=Theta) # 0.7787597
  derCOP(cop=GHcop, 0.5, 0.75, para=Theta) # 0.7787597
  # The next two nearly return same value but conversion to GRVs
  # (Gumbel Reduced Variates) to magnify the numerical differences.
  # The GHcop.derCOP is expected to be the more accurate of the two.
  lmomco::prob2grv(GHcop.derCOP(     0.5, 0.9999999, para=Theta)) # 18.83349
  lmomco::prob2grv(derCOP(cop=GHcop, 0.5, 0.9999999, para=Theta)) # 18.71497
  lmomco::prob2grv(derCOP(cop=GHcop, 0.5, 0.9999999, para=Theta,
                                   delu=.Machine$double.eps^.25)) # 18.83341
}
where the last numerical approximation shows that tighter tolerance is needed. A function for the inverse of the derivative (Joe, 2014, p. 172) given \eqn{u} by an analytical-numerical hybrid is
\preformatted{
  "GHcop.derCOPinv" <- function(u,t, para=NULL, verbose=FALSE,
                                     tol=.Machine$double.eps, ...) {
    if(length(u) > 1) warning("only the first value of u will be used")
    if(length(t) > 1) warning("only the first value of t will be used")
    if(is.null(para)) { warning("para can not be NULL"); return(NA) }
    u <- u[1]; t <- t[1]; rt <- NULL
    x <- -log(u); A <- (x + (para - 1)*log(x) - log(t))
    hz <- function(z) { z + (para - 1)*log(z) - A }
    zmax <- x; i <- 0; hofz.lo <- hz(zmax)
    if(sign(hofz.lo) != -1) warning("sign for h(z) is not negative!")
    while(1) {
       i <- i + 1
       if(i > 100) {
          warning("maximum iterations looking for zmax reached"); break
       }
       # increment zmax by 1/2 log cycle, sign(hofz.lo) should be negative!
       if(sign(hz(zmax <- zmax + 1/2)) != sign(hofz.lo)) break
    }
    try(rt <- uniroot(hz, c(x, zmax), tol=tol, ...), silent=FALSE)
    if(verbose) print(rt)
    if(is.null(rt)) {
       warning("NULL on the inversion of the GH copula derivative")
       return(NA)
    }
    zo <- rt$root
    y <- (zo^para - x^para)^(1/para)
    names(y) <- NULL
    return(exp(-y))
  }
}
that can be tested by the following, which also shows how to increase the tolerance on the numerical implementation
\preformatted{
  u <- 0.999; p <- 0.999
  GHcop.derCOPinv(     u, p, para=1.56) # 0.999977
  derCOPinv(cop=GHcop, u, p, para=1.56) # 1 (unity), needs tighter tolerance
  derCOPinv(cop=GHcop, u, p, para=1.56, tol=.Machine$double.eps/10) # 0.999977
}

\emph{ASYMMETRIC GUMBEL--HOUGAARD} --- Set \eqn{\tau_\mathbf{C} = 0.35} then for a symmetric and then reflection on the 1:1 line of the asymmetric Gumbel--Hougaard copula and compute the primary parameter \eqn{\Theta}, and lastly, compute three bivariate \eqn{\nu_\mathbf{C}} skewnesses (\code{\link{nuskewCOP}}):
\preformatted{
  Theta1 <- uniroot(function(t) {
                0.35 - tauCOP(cop=GHcop, para=c(t)) },           c(1,10))$root
  Theta2 <- uniroot(function(t) { # asymmetric
                0.35 - tauCOP(cop=GHcop, para=c(t, 0.6, 0.9)) }, c(1,30))$root
  Theta3 <- uniroot(function(t) { # asymmetric reflection on 1:1
                0.35 - tauCOP(cop=GHcop, para=c(t, 0.9, 0.6)) }, c(1,30))$root
  # Theta1 = 1.538462   and   Theta2 = Theta3 = 2.132856
  # Three "skews" based on a combination of U, V, and C(u,v) [nuskew()]
  nuskewCOP(cop=GHcop,   1.538462) # zero bivariate skewness
  nuskewCOP(cop=GHcop, c(2.132856, 0.6, 0.9)) #  0.1319304
  nuskewCOP(cop=GHcop, c(2.132856, 0.9, 0.6)) # -0.1319304
}
So, holding \eqn{\tau_\mathbf{C}} constant, we see that the \eqn{\mathbf{GH}} has a \eqn{\nu_{\mathbf{GH}(1.538)} = 0}, but the asymmetric case \eqn{\nu_{\mathbf{GH}(2.133, 0.6, 0.9)} = 0.1319304} and \eqn{\nu_{\mathbf{GH}(2.133, 0.9, 0.6)} = -0.1319304}, where the change in sign represents reflection about an equal value line. Finally, compute \emph{L-coskew} by large simulation and the adjective \dQuote{bow} representing the direction of curvature (bowing) of the principle copula density.
\preformatted{
  # Simulations for Theta1 would show no curvature about the diagonal.
  # Simulations for Theta2 would show curvature towards the upper left.
  # Simulations for Theta3 would show curvature towards the lower right.
  UV <- simCOP(1000, cop=GHcop, para=c(Theta1          ))
  UV <- simCOP(1000, cop=GHcop, para=c(Theta2, 0.6, 0.9))
  UV <- simCOP(1000, cop=GHcop, para=c(Theta3, 0.9, 0.6))
  # Because the Tau's are all similar, there is nothing to learn from the
  # L-correlation, let us inspect the L-coskew instead:
  lcm.1 <- lcomCOP(cop=GHcop, para=c(Theta1          ))
  lcm.2 <- lcomCOP(cop=GHcop, para=c(Theta2, 0.6, 0.9))
  lcm.3 <- lcomCOP(cop=GHcop, para=c(Theta3, 0.9, 0.6))
  lcm.1 <-  lapply(lcm.1, function(i) round(i, digits=5))
  lcm.2 <-  lapply(lcm.2, function(i) round(i, digits=5))
  lcm.3 <-  lapply(lcm.3, function(i) round(i, digits=5))

  message("# T3[12]: ", lcm.1$lcomUV[3], " (symmetric) ", # L-coskews
                        lcm.2$lcomUV[3], " (asym.--bow UL) ",
                        lcm.3$lcomUV[3], " (asym.--bow UL)", "\n",
          "# T3[21]: ", lcm.1$lcomVU[3], " (symmetric) ",
                        lcm.2$lcomVU[3], " (asym.--bow LR) ",
                        lcm.3$lcomVU[3], " (asym.--bow LR)")
  message("# T4[12]: ", lcm.1$lcomUV[4], " (symmetric) ", # L-cokurtoses
                        lcm.2$lcomUV[4], " (asym.--bow UL) ",
                        lcm.3$lcomUV[4], " (asym.--bow UL)", "\n",
          "# T4[21]: ", lcm.1$lcomVU[4], " (symmetric) ",
                        lcm.2$lcomVU[4], " (asym.--bow LR) ",
                        lcm.3$lcomVU[4], " (asym.--bow LR)")
  # T3[12]: 0.06962 (symmetric) 0.11321 (asym.--bow UL) 0.01426 (asym.--bow UL)
  # T3[21]: 0.06962 (symmetric) 0.01426 (asym.--bow LR) 0.11321 (asym.--bow LR)
  # T4[12]: 0.05100 (symmetric) 0.04266 (asym.--bow UL) 0.02718 (asym.--bow UL)
  # T4[21]: 0.05100 (symmetric) 0.02718 (asym.--bow LR) 0.04266 (asym.--bow LR)
}
Thus, the \emph{L-comoments} (Asquith, 2011) using their sample values measure something fundamental about the bivariate association between the three copulas choosen. The L-coskews for the symmetrical case are equal and are

\deqn{\tau^{\mathbf{GH}(\Theta_1)}_{3[12]} = \tau^{\mathbf{GH}(\Theta_1)}_{3[21]} = 0.06962 \mbox{,}}

whereas the L-coskew for the curvature to the upper left (LU) are
\deqn{\tau^{\mathbf{GH}(\Theta_2, 0.6, 0.9)}_{3[12]} = 0.11321\mbox{\ and\ } \tau^{\mathbf{GH}(\Theta_2, 0.6, 0.9)}_{3[12]} = 0.01426\mbox{,}}
whereas the L-coskew for the curvature to the lower right (LR) are
\deqn{\tau^{\mathbf{GH}(\Theta_3, 0.9, 0.6)}_{3[12]} = 0.01426\mbox{\ and\ } \tau^{\mathbf{GH}(\Theta_3, 0.6, 0.9)}_{3[12]} = 0.11321\mbox{.}}
Thus, the \eqn{\pi_2} and \eqn{\pi_3} parameters and L-coskew to the bivariate distribution and change the values for L-kurtoses.
}
\references{
Asquith, W.H., 2011, Distributional analysis with L-moment statistics using the R environment for statistical computing: Createspace Independent Publishing Platform, ISBN 978--146350841--8.

Brahimi, B., Chebana, F., and Necir, A., 2015, Copula representation of bivariate L-moments---A new estimation method for multiparameter two-dimensional copula models: Statistics, v. 49, no. 3, pp. 497--521.

Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in Nature---An approach using copulas: Springer, 289 p.

Zhang, L., and Singh, V.P., 2007, Gumbel--Hougaard copula for trivariate rainfall frequency analysis: Journal Hydrologic Engineering, v. 12, Special issue---Copulas in Hydrology, pp. 409--419.
}
\author{W.H. Asquith}
\seealso{\code{\link{M}}, \code{\link{GLcop}}, \code{\link{HRcop}}, \code{\link{tEVcop}}, \code{\link{rhobevCOP}}
}
\examples{
Theta    <- 2.2 # Let us see if numerical and analytical tail deps are the same.
del.lamU <- abs( taildepCOP(cop=GHcop, para=Theta)$lambdaU - (2-2^(1/Theta)) )
as.logical(del.lamU < 1E-6) # TRUE

\dontrun{
  # The simulations match Joe (2014, p. 72) for Gumbel-Hougaard
  n <- 600; nsim <- 1000; set.seed(946) # see for reproducibility
  SM <- sapply(1:nsim, function(i) { rs <- semicorCOP(cop=GHcop, para=1.35, n=n)
                                 c(rs$botleft.semicor, rs$topright.semicor) })
  RhoM     <- round(mean(SM[1,]),          digits=3)
  RhoP     <- round(mean(SM[2,]),          digits=3)
  SE.RhoM  <- round(  sd(SM[1,]),          digits=3)
  SE.RhoP  <- round(  sd(SM[2,]),          digits=3)
  SE.RhoMP <- round(  sd(SM[2,] - SM[1,]), digits=3)
  # Semi-correlations (sRho) and standard errors (SEs)
  message("# sRho[-]=", RhoM, " (SE[-]=", SE.RhoM, ") Joe(p.72)=0.132 (SE[-]=0.08)")
  message("# sRho[+]=", RhoP, " (SE[+]=", SE.RhoP, ") Joe(p.72)=0.415 (SE[+]=0.07)")
  message("# SE(sRho[-] - sRho[+])=", SE.RhoMP, " Joe(p.72) SE=0.10")
  # sRho[-]=0.134 (SE[-]=0.076) Joe(p.72)=0.132 (SE[-]=0.08)
  # sRho[+]=0.407 (SE[+]=0.074) Joe(p.72)=0.415 (SE[+]=0.07)
  # SE(sRho[-] - sRho[+])=0.107 Joe(p.72) SE=0.10
  # Joe (2014, p. 72) reports the values 0.132, 0.415, 0.08, 0.07, 0.10, respectively.}

\dontrun{
  # Let us compare the conditional simulation method of copBasic by numerics and by the
  # above analytical solution for the Gumbel-Hougaard copula to two methods implemented
  # by package gumbel, a presumed Archimedean technique by package acopula, and an
  # Archimedean technique by package copula. Setting seeds by each "method" below does
  # not appear diagnostic because of the differences in which the simulations are made.
  nsim <- 10000; kn <- "kendall" #  The theoretical KENDALL TAU is (1.5-1)/1.5 = 1/3
    # Simulate by conditional simulation using numerical derivative and then inversion
  A <- cor(copBasic::simCOP(nsim, cop=GHcop, para=1.5, graphics=FALSE), method=kn)[1,2]
  U <- runif(nsim) # GHcop.derCOPinv() defined earlier in this documentation.
  V <- sapply(1:nsim, function(i) { GHcop.derCOPinv(U[i], runif(1), para=1.5) })
    # Simulate by conditional simulation using exact analytical solution
  B <- cor(U, y=V, method=kn);  rm(U, V)
    # Simulate by the "common frailty" technique
  C <- cor(gumbel::rgumbel(nsim, 1.5, dim=2, method=1), method=kn)[1,2]
    # Simulate by "K function" (Is the K function method, Archimedean?)
  D <- cor(gumbel::rgumbel(nsim, 1.5, dim=2, method=2), method=kn)[1,2]
    # Simulate by an Archimedean implementation (presumably)
  E <- cor(acopula::rCopula(nsim, pars=1.5), method=kn)[1,2]
    # Simulate by an Archimedean implementation
  G <- cor(copula::rCopula(nsim, copula::gumbelCopula(1.5)), method=kn)[1,2]
  K <- round(c(A, B, C, D, E, G), digits=5); rm(A, B, C, D, E, G, kn); tx <- ", "
  message("Kendall Tau: ", K[1], tx, K[2], tx, K[3], tx, K[4], tx, K[5], tx, K[6])
  # Kendall Tau: 0.32909, 0.32474, 0.33060, 0.32805, 0.32874, 0.33986 -- run 1
  # Kendall Tau: 0.33357, 0.32748, 0.33563, 0.32913, 0.32732, 0.32416 -- run 2
  # Kendall Tau: 0.34311, 0.33415, 0.33815, 0.33224, 0.32961, 0.33008 -- run 3
  # Kendall Tau: 0.32830, 0.33573, 0.32756, 0.33401, 0.33567, 0.33182 -- nsim=50000!
  # All solutions are near 1/3 and it is unknown without further study which of the
  # six methods would result in the least bias and (or) sampling variability.}
}
\keyword{Gumbel--Hougaard copula}
\keyword{copula (formulas)}
\keyword{copula}
\keyword{Joe (2014) Examples and Exercises}
\keyword{Asymmetric Gumbel--Hougaard copula}
\concept{asymmetric Gumbel--Hougaard copula}
\concept{asymmetric Gumbel-Hougaard copula}
\concept{symmetric Gumbel--Hougaard copula}
\concept{symmetric Gumbel-Hougaard copula}
\concept{symmetric Gumbel copula}
\concept{asymmetric Gumbel copula}
\concept{asymmetric logistic copula}
\concept{Gumbel-Hougaard extreme value copula}
\concept{Gumbel--Hougaard extreme value copula}
\keyword{literature errors and inconsistencies}
\keyword{API to the copula package}
\keyword{package copula (comparison to)}
\keyword{copula (extreme value)}
\keyword{extreme value copula}
\keyword{package acopula (application of)}
\keyword{package gumbel (application of)}

\encoding{utf8}
\name{convex2COP}
\alias{convex2COP}
\title{Convex Combination of Two Copulas}
\description{
The \emph{convex composition of two copulas} (Joe, 2014, p. 155) provides for some simple complexity extension between copula families. Let \eqn{\mathbf{A}} and \eqn{\mathbf{B}} be copulas with respective vectors of parameters \eqn{\Theta_\mathbf{A}} and \eqn{\Theta_\mathbf{B}}, then the convex combination of these copulas is

\deqn{\mathbf{C}^{\times}_{\alpha}(u,v) = \alpha\cdot\mathbf{A}(u, v; \Theta_\mathbf{A}) - (1-\alpha)\cdot\mathbf{B}(u,v; \Theta_\mathbf{B})\mbox{,}}

where \eqn{0 \le \alpha \le 1}. The generalization of this function for \eqn{N} number of copulas is provided by \code{\link{convexCOP}}.
}
\usage{
convex2COP(u,v, para, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A special parameter \code{list} (see \bold{Note}); and}
  \item{...}{Additional arguments to pass to the copula.}
}
\value{
  Value(s) for the convex combination copula is returned.
}
\note{
The following descriptions list in detail the structure and content of the \code{para} argument:
\describe{
  \item{\code{alpha}}{--- The \eqn{\alpha} compositing parameter;}
  \item{\code{cop1}}{--- Function of the first copula \eqn{\mathbf{A}};}
  \item{\code{cop2}}{--- Function of the second copula  \eqn{\mathbf{B}};}
  \item{\code{para1}}{--- Vector of parameters \eqn{\Theta_\mathbf{A}} for  \eqn{\mathbf{A}}; and}
  \item{\code{para2}}{--- Vector of parameters \eqn{\Theta_\mathbf{B}} for  \eqn{\mathbf{B}}.}
}
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{COP}}, \code{\link{breveCOP}}, \code{\link{convexCOP}}, \code{\link{composite1COP}}, \code{\link{composite2COP}}, \code{\link{composite3COP}}, \code{\link{FRECHETcop}}, \cr \code{\link{glueCOP}}}
\examples{
para <- list(alpha=0.24, cop1=FRECHETcop, para1=c(0.4, 0.56),
                         cop2=PSP,        para2=NA)
convex2COP(0.87,0.35, para=para) # 0.3188711
\dontrun{
# Suppose we have a target Kendall Tau of 1/3 and a Gumbel-Hougaard copula seems
# attractive but the GH has just too much upper tail dependency for comfort. We
# think from data analysis that an upper tail dependency that is weaker and near
# 2/10 is the better. Let us convex mix in a Plackett copula and optimize.
TargetTau <- tauCOP(cop=GHcop, para=1.5) # 1/3 (Kendall Tau)
taildepCOP(   cop=GHcop, para=1.5, plot = TRUE)$lambdaU  # 0.4126
TargetUpperTailDep <- 2/10

# **Serious CPU time pending for this example**
par <- c(-.10, 4.65) # Initial guess but the first parameter is in standard
# normal for optim() to keep us in the [0,1] domain when converted to probability.
# The guesses of -0.10 (standard deviation) for the convex parameter and 4.65 for
# the Plackett are based on a much longer search times as setup for this problem.
# The simplex for optim() is going to be close to the solution on startup.
"afunc" <- function(par) {
   para <- list(alpha=pnorm(par[1]), cop1=GHcop,       para1=1.5,
                                     cop2=PLACKETTcop, para2=par[2])
   tau  <- tauCOP(cop=convex2COP, para=para)
   taildep <- taildepCOP(cop=convex2COP, para=para, plot = FALSE)$lambdaU
   err <- sqrt((TargetTau - tau)^2 + (TargetUpperTailDep - taildep)^2)
   print(c(pnorm(par[1]), par[2], tau, taildep, err))
   return(err)
}
mysolution <- optim(par, afunc, control=list(abstol=1E-4))

para <- list(alpha=.4846902, cop1=GHcop,       para1=1.5,
                             cop2=PLACKETTcop, para2=4.711464)
UV <- simCOP(n=2500, cop=convex2COP, para=para, snv=TRUE) #}
}
\keyword{copula composition}
\keyword{copula composition (convex combination)}

\encoding{utf8}
\name{NORMcop}
\alias{NORMcop}
\title{The Normal (Gaussian) Copula}
\description{
The \emph{Normal copula} (\emph{Gaussian copula}) (Salvadori \emph{et al.}, 2007, pp. 255--256) is

\deqn{\mathbf{C}_{\Theta}(u,v) = \mathbf{NORM}(u,v; \Theta) =
\int_{-\infty}^{\Phi^{(-1)}(u)}\!\!\!\!\int_{-\infty}^{\Phi^{(-1)}(v)}\!\!\!\!\!\!\!\!
\frac{1}{2\pi\sqrt{1-\Theta^2}}
\mathrm{exp}\biggl(-\frac{s^2 - 2\Theta s t + t^2}{ 2(1-\Theta^2) } \biggr)\,\mathrm{d}s\,\mathrm{d}t\mbox{,}}

where \eqn{\Theta \in [-1,1]} and \eqn{\Phi^{(-1)}(x)} is the quantile function of the \emph{univariate normal distribution}. The copula, as \eqn{\Theta \rightarrow -1^{+}}, limits to the \emph{countermonotonicity copula} (\eqn{\mathbf{W}(u,v)}; \code{\link{W}}), as \eqn{\Theta \rightarrow 0} limits, to the \emph{independence coupla} (\eqn{\mathbf{P}(u,v)}; \code{\link{P}}), and as \eqn{\Theta \rightarrow 1^{-}}, limits to the \emph{comonotonicity copula} (\eqn{\mathbf{M}(u,v)};  \code{\link{M}}). The copula has \emph{lower-tail} and \emph{upper-tail dependency parameters} equal to zero, but such is not true for the closely related \emph{t-Student copula} (\code{\link{Tcop}}). The \emph{Spearman Rho} (\code{\link{rhoCOP}}) is \eqn{\rho_\mathbf{C} = (6/\pi)\cdot\mathrm{asin}(\Theta/2)} and \emph{Kendall Tau} (\code{\link{tauCOP}}) is \eqn{\tau_\mathbf{C} = (2/\pi)\cdot\mathrm{asin}(\Theta)}. The parameter \eqn{\Theta} is readily computed by \eqn{\Theta = 2\cdot\mathrm{sin}(\pi\cdot\rho_\mathbf{C}/6)} or by \eqn{\Theta = \mathrm{sin}(\pi\cdot\tau_\mathbf{C}/2)}.
}
\usage{
NORMcop(u, v, para=NULL, rho=NULL, tau=NULL, fit=c("rho", "tau"), ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (single element) of parameters---the \eqn{\Theta} parameter of the copula;}
  \item{rho}{Optional Spearman Rho from which the parameter will be estimated and presence of \code{rho} trumps \code{tau};}
  \item{tau}{Optional Kendall Tau from which the parameter will be estimated;}
  \item{fit}{If \code{para}, \code{rho}, and \code{tau} are all \code{NULL}, then the \code{u} and \code{v} represent the sample. The measure of association by the \code{fit} declaration will be computed and the parameter estimated subsequently. The \code{fit} has no other utility than to trigger which measure of association is computed internally by the \code{cor} function in \R; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned. Otherwise if either \code{rho} or \code{tau} is given, then the \eqn{\Theta} is computed and a \code{list} having
  \item{para}{The parameter \eqn{\Theta};}
  \item{rho}{Spearman Rho if the \code{rho} is given; and}
  \item{tau}{Kendall Tau if the \code{tau} is given but also if both \code{rho} and \code{tau} are \code{NULL} as mentioned next.}
and if \code{para=NULL} and \code{rho} and \code{tau=NULL}, then the values within \code{u} and \code{v} are used to compute Spearman Rho (\code{fit="rho"}) or Kendall Tau (\code{fit="tau"}) and then compute the parameter, and this is returned in the aforementioned list.
}
\note{
A mimic of the wrappers on \code{mvtnorm::pmvnorm()} from the \pkg{copula} package (v1.1-6) are made within \code{NORMcop}. The rest of the implementation is unique to \pkg{copBasic}. An implementation of a function in the \pkg{copBasic} style that would natively interconnect to \pkg{copula} package version of the copula could be as follows:
\preformatted{
  "NORMcop" <-         # pCoupla() from package copula is analogous to COP()
  function(u,v, para=NULL, ...) {
    if(length(u) == 1) u <- rep(u, length(v)) # see asCOP() for reasoning of
    if(length(v) == 1) v <- rep(v, length(u)) # this "vectorization" hack
    para <- copula::normalCopula(                 c(para), dim=2)
    return( copula::pCopula(matrix(c(u,v), ncol=2), para) )
  }
}
}
\references{
Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in Nature---An approach using copulas: Springer, 289 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{Tcop}}
}
\examples{
\dontrun{
  para <- NORMcop(para=NULL, rho=0.9, taildep=0.3) # 0.907981
  UV <- simCOP(1000, cop=NORMcop, para=para$para)  # compare to similar Tcop example }
}
\keyword{Normal copula}
\keyword{Gaussian copula}
\keyword{copula (formulas)}
\keyword{copula}

\encoding{utf8}
\name{CLcop}
\alias{CLcop}
\title{The Clayton Copula}
\description{
The \emph{Clayton copula} (Joe, 2014, p. 168) is
\deqn{\mathbf{C}_{\Theta}(u,v) = \mathbf{CL}(u,v) = \mathrm{max}\bigl[(u^{-\Theta}+v^{-\Theta}-1; 0)\bigr]^{-1/\Theta}\mbox{,}}
where \eqn{\Theta \in [-1,\infty), \Theta \ne 0}. The copula, as \eqn{\Theta \rightarrow -1^{+}} limits, to the \emph{countermonotonicity coupla} (\eqn{\mathbf{W}(u,v)}; \code{\link{W}}), as \eqn{\Theta \rightarrow 0} limits to the \emph{independence copula} (\eqn{\mathbf{\Pi}(u,v)}; \code{\link{P}}), and as \eqn{\Theta \rightarrow \infty}, limits to the \emph{comonotonicity copula} (\eqn{\mathbf{M}(u,v)};  \code{\link{M}}). The parameter \eqn{\Theta} is readily computed from a \emph{Kendall Tau} (\code{\link{tauCOP}}) by \eqn{\tau_\mathbf{C} = \Theta/(\Theta+2)}.
}
\usage{
CLcop(u, v, para=NULL, tau=NULL, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (single element) of parameters---the \eqn{\Theta} parameter of the copula;}
  \item{tau}{Optional Kendall Tau; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned. Otherwise if \code{tau} is given, then the \eqn{\Theta} is computed and a \code{list} having
  \item{para}{The parameter \eqn{\Theta}, and}
  \item{tau}{Kendall Tau.}
and if \code{para=NULL} and \code{tau=NULL}, then the values within \code{u} and \code{v} are used to compute Kendall Tau and then compute the parameter, and these are returned in the aforementioned list.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{M}}, \code{\link{P}}, \code{\link{W}}
}
\examples{
# Lower tail dependency of Theta = pi --> 2^(-1/pi) = 0.8020089 (Joe, 2014, p. 168)
taildepCOP(cop=CLcop, para=pi)$lambdaL # 0.80201
}
\keyword{Clayton copula}
\keyword{copula (formulas)}
\keyword{copula}
\concept{Clayton copula}
\keyword{Joe (2014) Examples and Exercises}

\encoding{utf8}
\name{M}
\alias{M}
\title{The \enc{Fréchet}{Frechet}--Hoeffding Upper-Bound Copula}
\description{
Compute the \emph{\enc{Fréchet}{Frechet}--Hoeffding upper-bound copula} (Nelsen, 2006, p. 11), which is defined as
\deqn{\mathbf{M}(u,v) = \mathrm{min}(u,v)\mbox{.}}
This is the copula of perfect association (\emph{comonotonicity}, \emph{perfectly positive dependence}) between \eqn{U} and \eqn{V} and is sometimes referred to as the \emph{comonotonicity copula}. Its opposite is the \eqn{\mathbf{W}(u,v)} copula (\emph{countermonotonicity copula}; \code{\link{W}}), and statistical \emph{independence} is the \eqn{\mathbf{\Pi}(u,v)} copula (\code{\link{P}}).
}
\usage{
M(u, v, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{W}}, \code{\link{P}}}
\examples{
M(0.4,0.6)
M(0,0)
M(1,1)
}
\keyword{copula (formulas)}
\keyword{copula (limits/bounds)}
\concept{Frechet upper-bound copula}
\concept{Frechet-Hoeffding upper-bound copula}
\concept{Frechet--Hoeffding upper-bound copula}
\concept{Frechet upper bounds}
\concept{Frechet-Hoeffding upper bounds}
\concept{Frechet--Hoeffding upper bounds}
\keyword{copula}

\encoding{utf8}
\name{spectralmeas}
\alias{spectralmeas}
\title{Estimation of the Spectral Measure}
\description{
Kiriliouk \emph{et al.} (2016, pp. 360--364) describe estimation of the \emph{spectral measure} of bivariate data. Standardize the bivariate data as \eqn{X^\star} and \eqn{Y^\star} as in \code{\link{psepolar}} and select a \dQuote{large} value for the \emph{pseudo-polar radius} \eqn{S_f} for nonexceedance probability \eqn{f}. Estimate the spectral measure \eqn{H(w)}, which is the limiting distribution of the \emph{pseudo-polar angle} component \eqn{W} given that the corresponding radial component \eqn{S} is large:
\deqn{\mathrm{Pr}[W \in \cdot | S > S_f] \rightarrow H(w) \mbox{\ as\ } S_f \rightarrow \infty\mbox{.}}
So, \eqn{H(w)} is the \emph{cumulative distribution function} of the spectral measure for angle \eqn{w \in (0,1)}. The \eqn{S_f} can be specified by a nonexceedance probability \eqn{F} for \eqn{S_f(F)}.

The estimation proceeds as follows:

\bold{Step 1:} Convert the bivariate data \eqn{(X_i, Y_i)} into \eqn{(\widehat{S}_i, \widehat{W}_i)} by \code{\link{psepolar}} and set the threshold \eqn{S_f} according to \dQuote{\eqn{n/k}} (this part involving \eqn{k} does not make sense in Kiriliouk \emph{et al.} (2016)) where for present implementation in \pkg{copBasic} the \eqn{S_f} given the \eqn{f} by the user is based on the empirical distribution of the \eqn{\widehat{S}_i}. The empirical distribution is estimated by the \emph{Bernstein empirical distribution} function from the \pkg{lmomco} package.

\bold{Step 2:} Let \eqn{I_n} denote the set of indices that correspond to the observations when \eqn{\widehat{S}_i \ge S_f} and compute \eqn{N_n} as the cardinality of \eqn{N_n = |I_n|}, which simply means the length of the vector \eqn{I_n}.

\bold{Step 3:} Use the \emph{maximum Euclidean likelihood estimator}, which is the third of three methods mentioned by Kiriliouk \emph{et al.} (2016):
\deqn{\widehat{H}_3(w) = \sum_{i \in I_n} \hat{p}_{3,i} \times \mathbf{1}\bigl[\widehat{W}_i \le w \bigr]\mbox{,}}
where \eqn{\mathbf{1}[\cdot]} is an \emph{indicator function} that is only triggered if \code{smooth=FALSE}, and following the notation of Kiriliouk \emph{et al.} (2016), the \dQuote{3} represents maximum \emph{Euclidean likelihood} estimation. The \eqn{\hat{p}_{3,i}} are are the weights
\deqn{\hat{p}_{3,i} = \frac{1}{N_n}\bigl[ 1 - (\overline{W} - 1/2)S^{-2}_W(\widehat{W}_i - \overline{W}) \bigr]\mbox{,}}
where \eqn{\overline{W}} is the \emph{sample mean} and \eqn{S^2_W} is the \emph{sample variance} of \eqn{\widehat{W}_i}
\deqn{\overline{W} = \frac{1}{N_n} \sum_{i \in I_n} \widehat{W}_i\mbox{\quad and\quad } S^2_W = \frac{1}{N_n - 1} \sum_{i \in I_n} (\widehat{W}_i - \overline{W})^2\mbox{,}}
where Kiriliouk \emph{et al.} (2016, p. 363) do not show the \eqn{N_n - 1} in the denominator for the variance but \pkg{copBasic} uses it because those authors state that the \emph{sample variance} is used.

\bold{Step 4:} A smoothed version of \eqn{\hat{H}_3(w)} is optionally available by
\deqn{\tilde{H}_3(w) = \sum_{i \in I_n} \hat{p}_{3,i} \times \mathcal{B}\bigl(w;\, \widehat{W}_i\nu,\, (1-\widehat{W}_i)\nu\bigr)\mbox{,}}
where \eqn{\mathcal{B}(x; p, q)} is the cumulative distribution function of the \emph{Beta distribution} for \eqn{p, q > 0} and where \eqn{\nu > 0} is a smoothing parameter that can be optimized by cross validation.

\bold{Step 5:} The \emph{spectral density} lastly can be computed optionally as
\deqn{\tilde{h}_3(w) = \sum_{i \in I_n} \hat{p}_{3,i} \times \beta\bigl(w;\, \widehat{W}_i\nu,\, (1-\widehat{W}_i)\nu\bigr)}
where \eqn{\beta(x; p, q)} is the probability density function (pdf) of the Beta distribution. Readers are alerted to the absence of the \eqn{\mathbf{1}[\cdot]} indicator function in the definitions of \eqn{\tilde{H}_3(w)} and \eqn{\tilde{h}_3(w)}. This is correct and matches Kiriliouk \emph{et al.} (2016, eqs. 17.21 and 17.22) though this author was confused for a day or so by the indicator function in what is purported to be the core definition of \eqn{\hat{H}_l(w)} where \eqn{l = 3} in Kiriliouk \emph{et al.} (2016, eq. 17.21 and 17.17).
}
\usage{
spectralmeas(u, v=NULL, w=NULL, f=0.90, snv=FALSE,
                                smooth=FALSE, nu=100, pdf=FALSE, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction (actually the ranks are used so this can be a real-value argument as well);}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction (actually the ranks are used so this can be a real-value argument as well) and if \code{NULL} then \code{u} is treated as a two column \R \code{data.frame};}
  \item{w}{A vector of polar angle values \eqn{W \in [0,1]} on which to compute the \eqn{H(w)};}
  \item{f}{The nonexceedance probability \eqn{F(S_f)} of the \emph{pseudo-polar radius} in \code{\link{psepolar}};}
  \item{snv}{Return the standard normal variate of the \eqn{H} by the well-known transform through the quantile function of the standard normal, \code{qnorm()};}
  \item{smooth}{A logical to return \eqn{\tilde{H}_3(w)} instead of \eqn{H_3(w)};}
  \item{nu}{The \eqn{\nu > 0} smoothing parameter;}
    \item{pdf}{A logical to return the smoothed probability density \eqn{\tilde{h}_3(w)}. If \code{pdf=TRUE}, then internally \code{smooth=TRUE} will be set; and}
  \item{...}{Additional arguments to pass to the \code{\link{psepolar}} function.}
}
\value{
  An \R \code{vector} of \eqn{H_3(w)}, \eqn{\tilde{H}_3(w)}, or \eqn{\tilde{h}_3(w)} is returned.
}
\note{
The purpose of this section is to describe a CPU-intensive study of goodness-of-fit between a \emph{Gumbel--Hougaard copula} (\code{\link{GHcop}}, \eqn{\mathbf{GH}(u,v; \Theta_1)}) parent and a fitted \emph{\enc{Hüsler}{Husler}--Reiss copula} (\code{\link{HRcop}}, \eqn{\mathbf{HR}(u,v; \Theta_2)}). Both of these copulas are extreme values and are somewhat similar to each other, so sample sizes necessary for detection of differences should be large. A two-sided \emph{Kolmogorov--Smirnov} tests (KS test, \code{ks.test()}) is used to measure significance in the differences between the estimated spectral measure distributions at \eqn{f=0.90} (the 90th percentile, \eqn{F(S_f)}) into the right tail.

The true copula will be the \eqn{\mathbf{GH}(\Theta_1)} having parameter \eqn{\Theta_1 = 3.3}. The number of simulations per sample size \code{n} \eqn{\in} \code{seq(50,1000, by=25)} is \code{nsim = 500}. For each sample size, a sample from the true parent is drawn, and a \eqn{\mathbf{HR}(\Theta_2)} fit by \emph{maximum likelihood} (\code{\link{mleCOP}}). The two spectral measure distributions (\eqn{\widehat{H}_{\mathbf{GH}}(w)}, \code{Htru} and \eqn{\widehat{H}_{\mathbf{HR}}(w)}, \code{Hfit}) are estimated for a uniform variate of the angle \code{W} having length equal to the applicable sample size. The KS test is made between \code{Htru} and \code{Hfit}, and number of p-values less than the \eqn{\beta = 0.05} (Type II error because alternative hypothesis is rigged as true) and simulation count are returned and written in file \code{Results.txt}. The sample sizes initially are small and traps of \code{NaN} (abandonment of a simulation run) are made. These traps are needed when the empirical distribution of \code{Htru} or \code{Hfit} degenerates.
\preformatted{
  Results <- NULL
  true.par <- 3.3; true.cop <- GHcop; fit.cop <- HRcop; search <- c(0,100)
  nsim <- 20000; first_time <- TRUE; f <- 0.90; beta <- 0.05
  ns <- c(seq(100, 1000, by=50), 1250, 1500, 1750, 2000)
  for(n in ns) {
    W <- sort(runif(n)); PV <- vector(mode="numeric")
    for(i in 1:(nsim/(n/2))) {
      UV      <- simCOP(n=n, cop=true.cop, para=true.par, graphics=FALSE)
      fit.par <- mleCOP(UV,  cop= fit.cop, interval=search)$para
      UVfit   <- simCOP(n=n, cop= fit.cop, para=fit.par,  graphics=FALSE)
      Htru    <- spectralmeas(UV,    w=W, bound.type="Carv", f=f)
      Hfit    <- spectralmeas(UVfit, w=W, bound.type="Carv", f=f)
      if(length(Htru[! is.nan(Htru)]) != length(Hfit[! is.nan(Hfit)]) |
         length(Htru[! is.nan(Htru)]) == 0 |
         length(Hfit[! is.nan(Hfit)]) == 0) {
         PV[i] <- NA; next
      }   # suppressWarnings() to silence ties warnings from ks.test()
      KS <- suppressWarnings( stats::ks.test(Htru, Hfit)$p.value )
      #plot(FF, H, type="l"); lines(FF, Hfit, col=2); mtext(KS)
      message("-", i, appendLF=FALSE)
      if(length(grep("[02468]0$", as.character(i))) == 1) message()
      PV[i] <- KS
    }
    message(":",n)
    zz <- data.frame(SampleSize=n, NumPVle0p05=sum(PV[! is.na(PV)] <= beta),
                            SimulationCount=length(PV[! is.na(PV)]))
    if(first_time) { Results <- zz; first_time <- FALSE; next }
    Results <- rbind(Results, zz)
  }

  plot(Results$SampleSize, 100 * Results$NumPVle0p05 / Results$SimulationCount,
       type="b", cex=1.1, xlab="Sample size",
       ylab="Percent simulations with p-value < 0.05")
}

The \code{Results} show a general increase in the counts of p-value \eqn{\le} 0.05 as sample size increases. There is variation of course and increasing \code{nsim} would smooth that considerably. The results show for \eqn{n \approx 1{,}000} that the detection of statistically significant differences for extremal \eqn{F(S_f) = 0.90} dependency between the \eqn{\mathbf{GH}(\Theta_1{=}3.3)} and \eqn{\mathbf{HR}(\Theta_2)} are detected at the error rate implied by the specified \eqn{\beta = 0.05}.

This range in sample size can be compared to the Kullback--Leibler sample size (\eqn{n_{fg}}):
\preformatted{
  UV      <- simCOP(n=10000, cop=true.cop, para=true.par, graphics=FALSE)
  fit.par <- mleCOP(UV,      cop= fit.cop, interval=search)$para
  kullCOP(cop1=true.cop, para1=true.par,
          cop2=fit.cop,  para2=fit.par)$KL.sample.size
  # The Kullback-Leibler (integer) sample size for detection of differences at
  # alpha=0.05 are n_fg = (742, 809, 815, 826, 915, 973, 1203) for seven runs
  # Do more to see variation.
}
where the \emph{Kullback--Leilber Divergence} approach is to measure density departures across the whole \eqn{\mathcal{I}^2} domain as opposed to extremal dependency in the right tail as does the spectral measure.

Different runs of the above code will result in different \eqn{n_{fg}} in part because of simulation differences internal to \code{\link{kullCOP}} but also because the \eqn{\Theta_2} has its own slight variation in its fit to the large sample simulation (\eqn{n=10{,}000}) of the parent. However, it seems that \eqn{n_{fg} \approx 900} will be on the order of the \eqn{n} for which the KS test on the spectral measure determines statistical significance with similar error rate.

Now, if the aforementioned simulation run is repeated for \eqn{F(S_f) = 0.95} or \code{f=0.95}, the \eqn{n_{fg}} obviously remains unchanged at about \eqn{900} but the \eqn{n} for which the error rate is about \eqn{\beta = 0.05} is \eqn{n \approx 600}. This sample size is clearly smaller than before and smaller than \eqn{n_{fg}}, therefore, the analysis of the empirical spectral measure deeper into the tail \eqn{F(S_f) = 0.95} requires a smaller sample size to distinguish between the two copula---though the analysis does not address the question as to whether one or both copula are adequate for the problem at hand. For a final comparison, if the aforementioned simulation run is repeated for \eqn{F(S_f) = 0.80} or \code{f=0.80}, then the \eqn{n} for which the error rate is about \eqn{\beta = 0.05} is \eqn{n \approx 1{,}700}. Thus as analysis is made further away from the tail into the center of the distribution, the sample size to distinguish between these two similar copula increases substantially.
}
\references{
Kiriliouk, Anna, Segers, Johan, \enc{Warchoł}{Warchol}, \enc{Michał}{Michal}, 2016, Nonparameteric estimation of extremal dependence: \emph{in} Extreme Value Modeling and Risk Analysis, D.K. Dey and Jun Yan \emph{eds.}, Boca Raton, FL, CRC Press, ISBN 978--1--4987--0129--7.
}
\author{William Asquith \email{william.asquith@ttu.edu}}
\seealso{\code{\link{psepolar}}, \code{\link{stabtaildepf}}
}
\examples{
\dontrun{
  UV <- simCOP(n=500, cop=HRcop, para=1.3, graphics=FALSE)
  W <- seq(0, 1, by=0.005)
  Hu <- spectralmeas(UV, w=W)
  Hs <- spectralmeas(UV, w=W, smooth=TRUE, nu=100)
  plot( W, Hu, type="l", ylab="Spectral Measure H", xlab="Angle")
  lines(W, Hs, col=2) #}

\dontrun{
  "GAUScop" <- function(u,v, para=NULL, ...) {
    if(length(u) == 1) u <- rep(u, length(v))
    if(length(v) == 1) v <- rep(v, length(u))
    return(copula::pCopula(matrix(c(u,v), ncol=2), para))
  }
  GAUSparfn <- function(rho) return(copula::normalCopula(rho, dim=2))
  n <- 2000 # The PSP parent has no upper tail dependency
  uv    <- simCOP(n=n, cop=PSP,      para=NULL, graphics=FALSE)
  PLpar <- mleCOP(uv,  cop=PLcop,    interval=c(0,100))$para
  PLuv  <- simCOP(n=n, cop=PLcop,    para=PLpar, graphics=FALSE)
  GApar <- mleCOP(uv,  cop=GAUScop,  parafn=GAUSparfn, interval=c(-1,1))$para
  GAuv  <- simCOP(n=n, cop=GAUScop,  para=GApar, graphics=FALSE)
  GLpar <- mleCOP(uv,  cop=GLcop,    interval=c(0,100))$para
  GLuv  <- simCOP(n=n, cop=GLcop,    para=GLpar, graphics=FALSE)
  FF <- c(0.001, seq(0.005,0.995, by=0.005), 0.999); qFF <- qnorm(FF)
  f <- 0.90 # Seeking beyond the 90th percentile pseudo-polar radius
  PSPh <- spectralmeas(  uv, w=FF, f=f, smooth=TRUE, snv=TRUE)
  PLh  <- spectralmeas(PLuv, w=FF, f=f, smooth=TRUE, snv=TRUE)
  GAh  <- spectralmeas(GAuv, w=FF, f=f, smooth=TRUE, snv=TRUE)
  GLh  <- spectralmeas(GLuv, w=FF, f=f, smooth=TRUE, snv=TRUE)
  plot(qFF, PSPh, type="l", lwd=2, xlim=c(-3, 3), ylim=c(-3, 3),
       xlab="STANDARD NORMAL VARIATE OF PSEUDO-POLAR ANGLE",
       ylab="STANDARD NORMAL VARIATE OF SPECTRAL MEASURE PROBABILITY")
  lines(qFF, PLh, col="red"      ) # Plackett copula
  lines(qFF, GAh, col="seagreen3") # Gaussian copula
  lines(qFF, GLh, col="blue"     ) # Galambos copula
  # Notice the flat spot and less steep nature of the PSP (black line), which is
  # indicative of no to even spreading tail dependency. The Plackett and Gaussian
  # copulas show no specific steepening near the middle, which remains indicative
  # of no tail dependency with the Plackett being less steep because it has a more
  # dispersed copula density at the right tail is approached than the Gaussian.
  # The Galambos copula has upper tail dependency, which is seen by
  # the mass concentration and steepening of the curve on the plot.}
}
\keyword{copula (goodness-of-fit)}
\keyword{copula (inference)}
\keyword{inference}
\keyword{spectral measure}
\keyword{inference}
\keyword{goodness-of-fit}
\concept{polar}
\keyword{API to the copula package}
\keyword{package copula (comparison to)}

\encoding{utf8}
\name{RAYcop}
\alias{RAYcop}
\title{The Rayleigh Copula}
\description{
The \emph{Rayleigh copula} (Bo\enc{š}{s}koskia and others, 2018) is
\deqn{\mathbf{C}_{\Theta}(u,v) = \mathbf{RAY}(u,v; \Theta) = 1 + A - B\mbox{,}}
\deqn{A = e^{\Theta a_2 - a_2}\biggl(e^{-a_1}\int_0^{\Theta a_2} e^{-s}I_0\bigl(2\sqrt{a_1 s}\bigr)\,\mathrm{d}s - 1\biggr)\,\mathrm{d}s\mbox{,}}
\deqn{B = e^{-a_1}\int_0^{a_2}e^{-s}I_0\bigl(2\sqrt{\Theta a_1 s}\bigr)\,\mathrm{d}s\mbox{,}}
where \eqn{a_1 = -\log(1-u)/(1-\Theta)}, \eqn{a_2 = -\log(1-v)/(1-\Theta)}, \eqn{I_\nu(x)} is the modified Bessel function of the first kind of order \eqn{\nu} (see \code{base::besselI()}), and \eqn{\Theta \in (0,1]}. The copula, as \eqn{\Theta \rightarrow 0^{+}}, limits to the \emph{independence coupla} (\eqn{\mathbf{\Pi}(u,v)}; \code{\link{P}}) and as \eqn{\Theta \rightarrow 1^{-}} limits to the \emph{comonotonicity copula} (\eqn{\mathbf{M}(u,v)}; \code{\link{M}}). Finally, there are formulations of the Rayleigh copula using the \emph{Marcum-Q function}, but the \pkg{copBasic} developer has not been able to make such work. If the Marcum-Q function could be used, then only one integration and not the two involving the modified Bessel function are possible. Infinite integrations begin occurring in the upper right corner for about \eqn{\Theta > 0.995} at which point the \eqn{\mathbf{M}(u,v)} copula is called in the source code.
}
\usage{
RAYcop(u, v, para=NULL, rho=NULL, method=c("default"),
             rel.tol=.Machine$double.eps^0.5, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (single element) of parameters---the \eqn{\Theta} parameter of the copula;}
  \item{rho}{Value for Spearman Rho from which parameter \eqn{\Theta} is computed by polynomial approximation and returned. The estimation appears sufficient for most pratical applications (see \bold{Examples});}
  \item{method}{The computational method of integrals associated with the definition of the copula; this is designed for the ability to switch eventually in sources to Marcum-Q function implementation. The definition in January 2023 and default is to call the two Bessel function integrals shown for the definition in this documentation;}
  \item{rel.tol}{Argument of the same name for \code{integrate()} call; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\note{
Documentation in Zeng and others [Part II] appear to have corrected the Marcum-Q function solution to the copula. The essence of that solution is with a Chi-distribution computation the Marcum-Q. Testing indicates that they have the correct solution, but the derivative for the conditional simulation as built into the design of \pkg{copBasic} has difficulties. Perhaps this is related to numerical precision of the Marcum-Q?
\preformatted{
  sapply(seq_len(length((u))), function(i) {
    a1 <- -log(1-u[i])
    if(is.infinite(a1)) return(v[i])
    a2 <- -log(1-v[i])
    if(is.infinite(a2)) return(u[i])
    a1 <- exp(log(a1) - log(1-p))
    a2 <- exp(log(a2) - log(1-p))
    a3 <- marcumq.chi(sqrt(2*  a1), sqrt(2*p*a2)) # Zeng and others (Part II)
    a4 <- marcumq.chi(sqrt(2*p*a1), sqrt(2*  a2)) # Zeng and others (Part II)
    zz <- 1 + (1-v[i])*a3 - (1-u[i])*(1-a4)       # Zeng and others (Part II)
    zz[zz < 0] <- 0
    zz[zz > 1] <- 1
    return(zz)
  })
}
}
\references{
Bo\enc{š}{s}koskia, P., Debenjaka, A., Boshkoskab, B.M., 2018, Rayleigh copula for describing impedance data with application to condition monitoring of proton exchange membrane fuel cells: European Journal of Operational Research, v. 266, pp. 269--277, \doi{10.1016/j.ejor.2017.08.058}.

Zeng, X., Ren, J., Wang, Z., Marshall, S., and Durrani, T., 2014, Copulas for statistical signal processing (Part I)---Extensions and generalization: Signal Processing v. 14, pp. 691--702, \doi{10.1016/j.sigpro.2013.07.009}.

Zeng, X., Ren, J., Sun, M., Marshall, S., and Durrani, T., 2014, Copulas for statistical signal processing (Part II)---Simulation, optimal selection and practical applications: Signal Processing,\cr v. 94, pp. 681--690, \doi{10.1016/j.sigpro.2013.07.006}.
}
\author{W.H. Asquith}
\seealso{\code{\link{M}}, \code{\link{P}}
}
\examples{
RAYcop(0.2, 0.8, para=0.8) # [1] 0.1994658  (by the dual Bessel functions)

RAYcop(0.8, 0.2, para=RAYcop(rho=rhoCOP(cop=RAYcop, para=0.8)))
# [1] 0.1994651 from polynomial conversion of Rho to Theta

\dontrun{
  # Recipe for assembling the Spearman Rho to Theta polynomial in sources.
  Thetas <- seq(0, 0.999, by=0.001); RHOs <- NULL
  for(p in Thetas)  RHOs <- c(RHOs, rhoCOP(cop=RAYcop, para=p))
  LM <- lm(Thetas ~ RHOs  + I(RHOs^2) + I(RHOs^4) + I(RHOs^6) - 1 )
  Rho2Theta <- function(rho) {
    coes <- c(1.32682824, -0.38876290, 0.09072305, -0.02921836)
    sapply(rho, function(r) coes[1]*r^1 + coes[2]*r^2 + coes[3]*r^4 + coes[4]*r^6)
  }
  plot(RHOs, Thetas, type="l", col=grey(0.8), lwd=12, lend=1,
       xlab="Spearman Rho", ylab="Rayleigh Copula Parameter Theta")
  lines(RHOs, Rho2Theta(RHOs), col="red", lwd=2) # }
}
\keyword{Rayleigh copula}
\keyword{copula (formulas)}
\keyword{copula}
\concept{Rayleigh copula}

\encoding{utf8}
\name{level.setCOP}
\alias{level.setCOP}
\title{Compute a Level Set of a Copula V with respect to U}
\description{
Compute a \emph{level curve} or \emph{level set} of a copula for \eqn{V} with respect to \eqn{U} (Nelsen, 2006, pp. 12--13). The level curve at level \eqn{t} is defined for \eqn{U \mapsto [0+\Delta u, 1-\Delta u, \Delta u]} by
\deqn{t \mapsto \mathbf{C}(u{=}U, v)\mbox{,}}
and solving for \eqn{v}. The function is largely a dispatcher to features implemented in \code{\link{level.curvesCOP}}.
}
\usage{
level.setCOP(cop=NULL, para=NULL, getlevel=NULL, delu=0.001, lines=FALSE, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{getlevel}{The level set for \eqn{t};}
  \item{delu}{The increment for \eqn{\Delta u}. The default is 1 part in 1,000, which should often in practice provide enough smoothness for many copulas;}
  \item{lines}{A logical that matches the argument of the same name in \code{\link{level.curvesCOP}}; and}
  \item{...}{Additional arguments to pass to the \code{lines()} function in \R.}
}
\value{
  The level set for \eqn{t} \eqn{=} \code{getlevel} is returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{level.setCOP2}}, \code{\link{level.curvesCOP}}
}
\examples{
\dontrun{
set <- level.setCOP(cop=PSP, getlevel=0.23, delu=0.005)
level.curvesCOP(cop=PSP)
lines(set$U, set$V, col=2, lwd=2) # manually draw the 23rd percentile
set <- level.setCOP(cop=PSP, para=3.1, getlevel=0.67, lines=TRUE, col=4, lwd=4)
# Notice the change in the lines argument and using levelsetCOP2 to draw.
mtext("Level Curves and Special Level Sets for PSP copula") #}
}
\keyword{visualization}
\keyword{diagnostics}
\concept{level curves}
\concept{level curve}
\concept{level sets}
\concept{level set}
\concept{level contours}
\concept{level contour}

\encoding{utf8}
\name{simcomposite3COP}
\alias{simcomposite3COP}
\title{Compute the L-comoments of a Four-Value Composited Copula by Simulation}
\description{
Simulate copula parameters and compute \emph{L-comoments} and provision for plotting features for a composited copula using four compositing parameters (see \code{\link{composite3COP}}). The compositing parameters are each independent and uniformly distributed:
\deqn{\alpha \sim \mathrm{U}[0,1];\ \beta \sim \mathrm{U}[0,1];\ \kappa \sim \mathrm{U}[0,1];\ \gamma \sim \mathrm{U}[0,1]\mbox{.}}
L-comoment estimation is provided by the \code{\link{lcomCOP}}.
}
\usage{
simcomposite3COP(nsim=100, compositor=composite3COP,
                 parents=NULL, ploton=FALSE, points=FALSE,
                 showpar=FALSE, showresults=FALSE, digits=6, ...)
}
\arguments{
  \item{nsim}{Number of simulations to perform;}
  \item{compositor}{The compositing function that could be either \code{\link{composite1COP}}, \code{\link{composite2COP}}, and \code{\link{composite3COP}};}
  \item{parents}{A special parameter \code{list} (see \bold{Note});}
  \item{ploton}{A logical to toggle on intermediate plotting;}
  \item{points}{A logical to actually draw the simulations on the \code{ploton} by the \code{points()} function in \R;}
  \item{showpar}{Print the simulated parameter set with each iteration;}
  \item{showresults}{Print the results (useful if harvest results from a batch operation in \R);}
  \item{digits}{The number digits to pass to \code{round} if \code{showresults} is true; and}
  \item{...}{Additional arguments to pass.}
}
\value{
An \R matrix of results is returned. Each row represents a single simulation run. The first four columns are the \eqn{\alpha}, \eqn{\beta}, \eqn{\kappa}, and \eqn{\gamma} \emph{compositing parameters} and are labeled as such. The next two columns are the opposing diagonals, by first row and then second, of the \emph{L-comoment correlation}. The following two columns are the opposing diagonals, by row and then second, of the \emph{L-coskew}. The following two columns are the opposing diagonals, by row and then second, of the \emph{L-cokurtosis}. The L-comoment columns are labeled to reflect the L-comoment matrix: \code{T2.21} means the L-comoment correlation row 2 column 1 and \code{T3.12} mean the L-coskew row 1 column 2. The remaining columns represent the \eqn{\Theta_n} parameters for copula 1, the \eqn{\Theta_m} parameters for copula 2. The columns are labeled \code{Cop1Thetas} or \code{Cop2Thetas}.
}
\references{
Asquith, W.H., 2011, Distributional analysis with L-moment statistics using the R environment for statistical computing: Createspace Independent Publishing Platform, ISBN 978--146350841--8.
}
\note{
The following descriptions list in detail the \code{parents} argument structure and content of the \code{para} argument:
\describe{
  \item{\code{cop1}}{--- Function of the first copula;}
  \item{\code{cop2}}{--- Function of the second copula;}
  \item{\code{para1gen}}{--- Function to generate random parameters for the first copula; and}
  \item{\code{para2gen}}{--- Function to generate random parameters for the second copula.}
}
The \code{para} argument of this function are passed to the function contained in \code{compositor} and are therefore subject to further constraints in items should such constraints exist.
}
\author{W.H. Asquith}
\seealso{\code{\link{lcomCOP}}, \code{\link{simcompositeCOP}}}
\examples{
\dontrun{
# EXAMPLE 1: Make a single simulation result.
mainpara <- list(cop1=PLACKETTcop, cop2=PLACKETTcop,
                 para1gen=function() { return(c(10^runif(1, min=-5, max=0))) },
                 para2gen=function() { return(c(10^runif(1, min= 0, max=5))) })
v <- simcompositeCOP(nsim=1, parent=mainpara, showresults=TRUE)
print(v)

# EXAMPLE 2: Make 1000 "results" and plot two columns.
mainpara <- list(cop1=PLACKETTcop, cop2=N4212cop,
                 para1gen=function() { return(c(10^runif(1, min=-5, max=5))) },
                 para2gen=function() { return(c(10^runif(1, min= 0, max=2))) })
v <- simcomposite3COP(nsim=100, parent=mainpara); labs <- colnames(v)
plot(v[,5], v[,7],           # open circles are 1 with respect to 2
     xlab=paste(c(labs[5], "and", labs[6]), collapse=" "),
     ylab=paste(c(labs[6], "and", labs[8]), collapse=" "))
points(v[,6], v[,8], pch=16) # black dots are 2 with respect to 1}
}
\keyword{copula composition}
\keyword{copula (simulation)}
\keyword{visualization}
\keyword{L-comoments}

\encoding{utf8}
\name{giniCOP}
\alias{giniCOP}
\title{The Gini Gamma of a Copula}
\description{
Compute the measure of association known as the \emph{Gini Gamma} \eqn{\gamma_\mathbf{C}} (Nelsen, 2006, pp. 180--182), which is defined as
\deqn{\gamma_\mathbf{C} = \mathcal{Q}(\mathbf{C},\mathbf{M}) + \mathcal{Q}(\mathbf{C},\mathbf{W})\mbox{,}}
where \eqn{\mathbf{C}(u,v)} is the copula, \eqn{\mathbf{M}(u,v)} is the \code{\link{M}} function, and \eqn{\mathbf{W}(u,v)} is the \code{\link{W}} function. The function \eqn{\mathcal{Q}(a,b)} (\link{concordCOP}) is a \emph{concordance function} (Nelsen, 2006, p. 158). Nelsen also reports that \dQuote{Gini Gamma measures a concordance relation of \dQuote{distance} between \eqn{\mathbf{C}(u,v)} and monotone dependence, as represented by the \emph{\enc{Fréchet}{Frechet}--Hoeffding lower bound} and \emph{\enc{Fréchet}{Frechet}--Hoeffding upper bound} copulas [\eqn{\mathbf{M}(u,v)}, \code{\link{M}} and \eqn{\mathbf{W}(u,v)}, \code{\link{W}} respectively]}

A simpler method of computation and the default for \code{giniCOP} is to compute \eqn{\gamma_\mathbf{C}} by

\deqn{\gamma_\mathbf{C}  = 4\biggl[\int_\mathcal{I} \mathbf{C}(u,u)\,\mathrm{d}u +
                             \int_\mathcal{I} \mathbf{C}(u,1-u)\,\mathrm{d}u\biggr] -
                           2\mbox{,}}
or in terms of the \emph{primary diagonal} (alt. \emph{main diagonal}, Genest \emph{et al.}, 2010) \eqn{\delta(t)} and \emph{secondary diagonal} \eqn{\delta^\star(t)} (see \code{\link{diagCOP}}) by
\deqn{\gamma_\mathbf{C}  = 4\biggl[\int_\mathcal{I} \mathbf{\delta}(t)\,\mathrm{d}t +
                             \int_\mathcal{I} \mathbf{\delta^\star
                             }(t)\,\mathrm{d}t\biggr] -
                           2\mbox{.}}


The simpler method is more readily implemented because single integration is fast. Lastly, Nelsen \emph{et al.} (2001, p. 281) show that \eqn{\gamma_\mathbf{C}} also is computable by
\deqn{\gamma_\mathbf{C} = 2\,\mathcal{Q}(\mathbf{C},\mathbf{A})\mbox{,}}
where \eqn{\mathbf{A}} is a \emph{convex combination} (\code{\link{convex2COP}}, using \eqn{\alpha = 1/2}) of the copulas \eqn{\mathbf{M}} and \eqn{\mathbf{W}} or \eqn{\mathbf{A} = (\mathbf{M}+\mathbf{W})/2}. However, integral convergence errors seem to trigger occasionally, and the first definition by summation \eqn{\mathcal{Q}(\mathbf{C},\mathbf{M}) + \mathcal{Q}(\mathbf{C},\mathbf{W})} thus is used. The convex combination is demonstrated in the \bold{Examples} section.
}
\usage{
giniCOP(cop=NULL, para=NULL, by.concordance=FALSE, as.sample=FALSE, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{by.concordance}{Instead of using the single integrals (Nelsen, 2006, pp. 181--182) to compute \eqn{\gamma_\mathbf{C}}, use the concordance function method implemented through \code{\link{concordCOP}};}
  \item{as.sample}{A logical controlling whether an optional \R \code{data.frame} in \code{para} is used to compute the \eqn{\hat\gamma_\mathbf{C}} (see \bold{Note}); and}
  \item{...}{Additional arguments to pass, which are dispatched to the copula function \code{cop} and possibly \code{\link{concordCOP}} if \code{by.concordance=TRUE}, such as \code{delta} used by that function.}
}
\value{
  The value for \eqn{\gamma_\mathbf{C}} is returned.
}
\note{
Conceptually, the sample Gini Gamma (\eqn{\hat\gamma}; Genest \emph{et al.}, 2010) is

\deqn{\hat\gamma = \frac{1}{\lfloor n^2/2 \rfloor}\sum_{i=1}^n \mid (n+1-R_i) - S_i \mid- \mid R_i - S_i\mid\mbox{,}}

where \eqn{{\lfloor m \rfloor}} denotes the integer part of arbitrary \eqn{m > 0}, \eqn{R_i} and \eqn{S_i} are the respective ranks of \eqn{X} and \eqn{Y} and \eqn{n} is sample size. The sampling variance of \eqn{\hat\gamma} under \emph{assumption of independence} between \eqn{X} and \eqn{Y} is

\deqn{\mathrm{var}(\hat\gamma)_{n\ \mathrm{even}} = \frac{2}{3}\frac{(n^2 + 2)}{(n-1)n^2}\mbox{\ and}}

\deqn{\mathrm{var}(\hat\gamma)_{n\ \mathrm{odd}} = \frac{2}{3}\frac{(n^2 + 3)}{(n-1)(n^2-1)}\mbox{.}}

Genest \emph{et al.} (2010) present additional equations for estimation of the distribution \eqn{\hat\gamma} variance for conditions of dependence based on copulas. For a copula having continuous partial derivatives, then as \eqn{n \rightarrow \infty}, the quantity \eqn{(\hat\gamma - \gamma_\mathbf{C})\sqrt{n} \sim \mathrm{Normal}(0, \mathrm{var}(\gamma_\mathbf{C}))}. Genest \emph{et al.} (2010) show variance of \eqn{\hat\gamma} for the \emph{independence copula} (\eqn{\mathbf{C}(u,v) = \mathbf{\Pi}(u,v)}) (\code{\link{P}}) as \eqn{\mathrm{var}(\gamma_\mathbf{C}) = 2/3}. For comparison, the \emph{Spearman Footrule} for independence is smaller at \eqn{\mathrm{var}(\psi_\mathbf{C}) = 2/5} (see \code{\link{footCOP}} \bold{Note}). In Genest \emph{et al.} independence and two examples of dependence (p. 941), \eqn{\mathrm{var}(\hat\gamma) > \mathrm{var}(\hat\psi)}, but those authors do not appear to remark on whether this inequality holds for all copula.
}
\references{
Genest, C., \enc{Nešlehová}{Neslehova}, J., and Ghorbal, N.B., 2010, Spearman's footrule and Gini's gamma---A review with complements: Journal of Nonparametric Statistics, v. 22, no. 8, pp. 937--954, \doi{10.1080/10485250903499667}.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Nelsen, R.B., Quesada-Molina, J.J., \enc{Rodríguez-Lallena}{Rodriguez-Lallena}, J.A., \enc{Úbeda-Flores}{Ubeda-Flores}, M., 2001, Distribution functions of copulas---A class of bivariate probability integral transforms: Statistics and Probability Letters, v. 54, no. 3, pp. 277--282, \doi{10.1016/S0167-7152(01)00060-8}.
}
\author{W.H. Asquith}
\seealso{\code{\link{blomCOP}}, \code{\link{footCOP}}, \code{\link{hoefCOP}},
         \code{\link{rhoCOP}}, \code{\link{tauCOP}}, \code{\link{wolfCOP}},
         \code{\link{joeskewCOP}}, \code{\link{uvlmoms}}
}
\examples{
giniCOP(cop=PSP)                                 #                 = 0.3819757
\dontrun{
giniCOP( cop=PSP, by.concordance=TRUE)           # Q(C,M) + Q(C,W) = 0.3820045
# use convex combination ---triggers integration warning but returns anyway
cxpara <- list(alpha=1/2, cop1=M, cop2=W) # parameters for convex2COP()
2*tauCOP(cop=PSP, cop2=convex2COP, para2=cxpara) #    2*Q(C,A)     = 0.3819807
# where the later issued warnings on the integration}

\dontrun{
n <- 2000; UV <- simCOP(n=n, cop=N4212cop, para=9.3, graphics=FALSE)
giniCOP(para=UV, as.sample=TRUE)                     # 0.9475900 (sample version)
giniCOP(cop=N4212cop, para=9.3)                      # 0.9479528 (copula integration)
giniCOP(cop=N4212cop, para=9.3, by.concordance=TRUE) # 0.9480267 (concordance function)
# where the later issued warnings on the integration}

\dontrun{
# The canoncial example of theoretical and sample estimators of bivariate
# association for the package: Blomqvist Beta, Spearman Footrule, Gini Gamma,
# Hoeffding Phi, Kendall Tau, Spearman Rho, and Schweizer-Wolff Sigma
# and comparison to L-correlation via lmomco::lcomoms2().
n <- 9000; set.seed(56)
para <- list(cop1=PLACKETTcop, cop2=PLACKETTcop, para1=1.45, para2=21.9,
             alpha=0.41, beta=0.08)
D <- simCOP(n=n, cop=composite2COP, para=para, cex=0.5, col=rgb(0,0,0,0.2), pch=16)
blomCOP(cop=composite2COP, para=para)         # 0.4037908 (theoretical)
blomCOP(para=D, as.sample=TRUE)               # 0.4008889 (sample)
footCOP(cop=composite2COP, para=para)         # 0.3721555 (theoretical)
footCOP(para=D, as.sample=TRUE)               # 0.3703623 (sample)
giniCOP(cop=composite2COP, para=para)         # 0.4334687 (theoretical)
giniCOP(para=D, as.sample=TRUE)               # 0.4311698 (sample)
tauCOP(cop=composite2COP,  para=para)         # 0.3806909 (theoretical)
tauCOP(para=D,  as.sample=TRUE)               # 0.3788139 (sample)
rhoCOP(cop=composite2COP,  para=para)         # 0.5257662 (theoretical)
rhoCOP(para=D,  as.sample=TRUE)               # 0.5242380 (sample)
lmomco::lcomoms2(D)$T2      # 1               # 0.5242388 (sample matrix)
                            # 0.5245154 1
hoefCOP(cop=composite2COP, para=para)         # 0.5082776 (theoretical)
subsample <- D[sample(1:n, n/5),] # subsampling for speed
hoefCOP(para=subsample, as.sample=TRUE)       # 0.5033842 (re-sample)
#hoefCOP(para=D, as.sample=TRUE) # major CPU hog, n too big
# because the Ds are already "probabilities" just resample as shown above
wolfCOP(cop=composite2COP, para=para)         # 0.5257662 (theoretical)
#wolfCOP(para=D, as.sample=TRUE) # major CPU hog, n too big
wolfCOP(para=subsample, as.sample=TRUE)       # 0.5338009 (re-sample)}


\dontrun{
set.seed(1); nsim <- 1000
varGIunderIndpendence <- function(n) {
  if(3 \%/\% 2 == 3 / 2) { # even
    (2/3)*( (n^2 + 2) ) / ( (n-1)* n^2    ) # Genest et al. (2010)
  } else {
    (2/3)*( (n^2 + 3) ) / ( (n-1)*(n^2-1) ) # Genest et al. (2010)
  }
}
ns <- c(10, 15, 20, 25, 50, 75, 100)
plot(min(ns):max(ns), varGIunderIndpendence(10:max(ns)), type="l",
     xlab="Sample size", ylab="Variance of Sample Estimator", col="darkgreen")
mtext("Sample Gini Gamma Under Independence", col="darkgreen")
for(n in ns) {
  sGI <- vector(length=nsim)
  for(i in seq_len(nsim)) {
    uv <- simCOP(n=n, cop=P, para=2, graphics=FALSE)
    sGI[i] <- giniCOP(para=uv, as.sample=TRUE)
  }
  varGI <- varGIunderIndpendence(n)
  zz <- round(c(n, mean(sGI), var(sGI), varGI), digits=6)
  names(zz) <- c("n", "mean_sim", "var_sim", "var_Genest")
  print(zz)
  points(n, zz[3], cex=2, pch=21, col="darkgreen", bg="lightgreen")
} # results show proper implementation and Genest et al. (2010, sec. 3) }
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}

\encoding{utf8}
\name{lcomCOPpv}
\alias{lcomCOPpv}
\title{Simulating the Sample Distribution(s) of L-correlation, L-coskew, and L-cokurtosis for a Copula}
\description{
\emph{EXPERIMENTAL:} The function provides two themes of sampling distribution characterization by simulation of the first three L-comoment ratios (L-correlation \eqn{\tau_{2[\ldots]}}, L-coskew \eqn{\tau_{3[\ldots]}} and L-cokurtosis \eqn{\tau_{4[\ldots]}}) of a copula. Subsequently, the sampling distribution can be used for inference.

First, semi-optional Monte Carlo integration estimation of the L-comoments of the parent copula are computed. Second, simulations involving the sample size \eqn{n} presumed the size of the actual sample from which the estimates of the sample L-comoments given as arguments. These simulations result in a report of the L-moments (not L-comoments) of the sampling distribution and these then are used to compute p-values for the L-comoment matrices provided by the user as a function argument.
}
\usage{
lcomCOPpv(n, lcom, cop=NULL, para=NULL, repcoe=5E3, type="gno",
                   mcn=1E4, mcrep=10, usemcmu=FALSE, digits=5, ...)
}
\arguments{
  \item{n}{The sample size \eqn{n}. This argument is semi-optional because \eqn{n = 0} can be given to skip corresponding simulations and the \code{ntable} on return will only contain \code{NA}; this feature permits rapid extraction of the \code{Ntable} and thus the \code{lcom} contents are simply not used;}
  \item{lcom}{The sample L-comoments (see below);}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters, if needed, to pass to the copula;}
  \item{repcoe}{The replication coefficient \eqn{\phi} affecting the number of simulations of size \code{n};}
  \item{type}{The distribution type used for modeling the distribution of the sampling values. The generalized normal (see distribution type \code{"gno"} in package \pkg{lmomco}) accommodates some skewness compared to the symmetry of the normal (\code{"nor"}) just in case situations arise in which non-ignorable skewness in the sample distribution exists. The distribution abbreviations of package \pkg{lmomco} are recognized for the \code{type} argument, but in reality the \code{"nor"} and \code{"gno"} should be more than sufficient;}
  \item{mcn}{The sample size \eqn{N} passed to the \code{\link{bilmoms}} function for the Monte Carlo integration. If \eqn{N = 0} then the Monte Carlo integration is not used, otherwise the minimum sample size is internally reset to \eqn{N=4} so that first four L-moments are computable;}
  \item{mcrep}{The number of replications of the Monte Carlo simulation by \code{\link{bilmoms}};}
  \item{usemcmu}{A logical toggling whether the mean value computed from the replicated Monte Carlo integrations is used instead of the mean values for the small sample simulation for the p-value computations;}
  \item{digits}{The number of digits to round numerical entries in the returned tables and can be \code{NA} for no rounding; and}
  \item{...}{Additional arguments to pass to the \code{\link{bilmoms}} function or to the copula.}
}
\details{
The notation \eqn{r[\ldots]} refers to two specific types of L-comoment definitions and a blend between the two. The notation \eqn{r[12]} means that the \eqn{r}th L-comoment for random variables \eqn{\{X^{(1)}, X^{(2)}\}} where \eqn{X^{(2)}} is the sorted variable and \eqn{X^{(1)}} is shuffled by the sorting index. Conversely, the notation \eqn{r[21]} means that the \eqn{r}th L-comoment for random variables \eqn{\{X^{(1)}, X^{(2)}\}} where \eqn{X^{(1)}} is the sorted variable and \eqn{X^{(2)}} is shuffled by the sorting index. The notation \eqn{r[12:21]} means that the average between the \eqn{r[21]} and \eqn{r[21]} is computed, which might prove useful in circumstances of known or expected symmetry of the L-comoments.

Continuing, \eqn{\hat\tau_{2[12]}} is the sample L-correlation, \eqn{\hat\tau_{3[12]}} is the sample L-coskew, and \eqn{\hat\tau_{4[12]}} is the sample L-cokurtosis all with respect to the sorting of the second variable. The computation of these L-comoment matricies can be made by functions such as function \code{lcomoms2()} in the \pkg{lmomco} package. The number of replications for the simulations involving the \eqn{n} sample size is computed by
\deqn{m = \phi/\sqrt{n}\mbox{,}}
where \eqn{\phi} is the \code{repcoe} replication factor or coefficient. If \code{usemcmu} is \code{TRUE} then \code{mcn} \eqn{> 0} else \code{usemcmu} is reset to \code{FALSE}.
}
\value{
  An \R \code{list} is returned.
  \item{text}{A string functioning as a label for the remaining tables;}
  \item{Ntable}{Another \R \code{list} holding tables of the L-moments of the L-comoments derived from Monte Carlo integration for samples of size \eqn{N =} \code{mcn}. The simulations are replicated \code{mcrep} times; and}
  \item{ntable}{Another \R \code{list} holding tables of the L-moments of the L-comoments derived from the small sample simulations for samples of size \eqn{n =} \code{n} as well as the p-values estimated by a generalized normal distribution (see \pkg{lmomco} package documentation) of the L-moments using either the small sample means or the mean of the replicated Monte Carlo integrations as dictated by \code{usemcmu}. In all circumstances, however, the results for the small sample simulations are tabulated in \code{ntable} only the p-value will be reflective the setting of \code{usemcmu}.}
}
\note{
A significance column for the p-values is added to the right side of the returned \code{ntable} and is used to guide the eye in interpretation of results. The significant codes having the following definitions for a two-tailed form:
\preformatted{
  "_" > 0.1;  ".", 0.1;  "*", 0.05;  "**", 0.01;  "***", 0.001
}
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{lcomCOP}}, \code{\link{COP}}, \code{\link{kullCOP}}, \code{\link{vuongCOP}}}
\examples{
# See Note section of vuongCOP() for an extended discussion of copula inference
\dontrun{
  Tau <- 0.6410811; para <- GHcop(tau=Tau)$para # This Tau is from a situation of
  # two river tributaries. These three L-comoments with univariate L-moments on the
  T2 <- c(1,  0.79908960, 0.79908960, 1) # diagonals are derived from those river
  # tributaries and downstream of the junction data.
  T3 <- c(0, -0.04999318, 0.07689082, 0)
  T4 <- c(0,  0.01773833, 0.04756257, 0) # Is the Ho:GHcop rejectable?
  LCOM <- list(T2=matrix(T2, nrow=2), T3=matrix(T3, nrow=2), T4=matrix(T4, nrow=2))
  set.seed(30312)
  ZZ1 <- lcomCOPpv(75, LCOM, cop=GHcop, para=para, repcoe=2000, usemcmu=FALSE)
  print(ZZ1)
  set.seed(30312)
  ZZ2 <- lcomCOPpv(75, LCOM, cop=GHcop, para=para, repcoe=2000, usemcmu=TRUE)
  print(ZZ2)
  # The results here suggest that the GHcop is not rejectable.}
}
\keyword{inference}
\keyword{hypothesis testing}
\keyword{copula (characteristics)}
\keyword{copula (inference)}
\keyword{L-comoments}

\encoding{utf8}
\name{COPinv}
\alias{COPinv}
\title{The Inverse of a Copula for V with respect to U}
\description{
Compute the \emph{inverse of a copula} for \eqn{V} with respect to \eqn{U} given \eqn{t} or
\deqn{t = \mathbf{C}(u{=}U,v) \rightarrow \mathbf{C}^{(-1)}(u{=}U, t) = v\mbox{,}}
and solving for \eqn{v}. Nelsen (2006, p. 12) does not so name this function as an \dQuote{inverse.} The \code{COPinv} function is internally used by \code{\link{level.curvesCOP}} and \code{\link{level.curvesCOP2}}. A common misapplication that will puzzle the user (including the developer after long breaks from package use) is that the following call and error message are often seen, if \code{silent=FALSE}:
\preformatted{
  COPinv(0.2, 0.25, cop=PSP)
  # Error in uniroot(func, interval = c(lo, 1), u = u, LHS = t, cop = cop,  :
  #  f() values at end points not of opposite sign
  # [1] NA
}
This is a harmless error in the sense that \code{COPinv} is functioning properly. One can not invert a copula for \eqn{u < t} and for \eqn{u = t} the \eqn{v = 1} because of fundamental copula properties.
}
\usage{
COPinv(cop=NULL, u, t, para=NULL, silent=TRUE, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{t}{Nonexceedance probability level \eqn{t};}
  \item{para}{Vector of parameters or other data structures, if needed, to pass to the copula;}
  \item{silent}{The argument of the same name given over to \code{try()} wrapping the \code{uniroot()} operation; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for \eqn{v} are returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{COP}},
         \code{\link{COPinv2}},
         \code{\link{level.curvesCOP}},
         \code{\link{level.curvesCOP2}}
}
\examples{
COPinv(cop=N4212cop, para=2, u=0.5, t=0.2)
}
\keyword{copula (utility)}
\keyword{copula (inverse)}
\keyword{copula operator}

\encoding{utf8}
\name{RFcop}
\alias{RFcop}
\title{The Raftery Copula}
\description{
The \emph{Raftery copula} (Nelsen, 2006, p. 172) is

\deqn{\mathbf{C}_{\Theta}(u,v) = \mathbf{RF}(u,v) = \mathbf{M}(u,v) + \frac{1-\Theta}{1+\Theta}\biggl(uv\biggr)^{1/(1-\Theta)}\biggl[1-\bigl(\mathrm{max}\{u,v\}\bigr)^{-(1+\Theta)/(1-\Theta)}\biggr]\mbox{,}}

where \eqn{\Theta \in (0,1)}. The copula, as \eqn{\Theta \rightarrow 0^{+}} limits, to the \emph{independence coupla} (\eqn{\mathbf{P}(u,v)}; \code{\link{P}}), and as \eqn{\Theta \rightarrow 1^{-}}, limits to the \emph{comonotonicity copula} (\eqn{\mathbf{M}(u,v)};  \code{\link{M}}). The parameter \eqn{\Theta} is readily computed from \emph{Spearman Rho} (\code{\link{rhoCOP}}) by \eqn{\rho_\mathbf{C} = \Theta(4-3\Theta)/(2-\Theta)^2} or from \emph{Kendall Tau} (\code{\link{tauCOP}}) by \eqn{\tau_\mathbf{C} = 2\Theta/(3-\Theta)}. However, this copula like others within the \pkg{copBasic} package can be reflected (rotated) at will with the \code{\link{COP}} abstraction layer to acquire negative or inverse dependency (\emph{countermonotonicity}) (see the \bold{Examples}).
}
\usage{
RFcop(u, v, para=NULL, rho=NULL, tau=NULL, fit=c("rho", "tau"), ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (single element) of parameters---the \eqn{\Theta} parameter of the copula;}
  \item{rho}{Optional Spearman Rho from which the parameter will be estimated and presence of \code{rho} trumps \code{tau};}
  \item{tau}{Optional Kendall Tau from which the parameter will be estimated;}
  \item{fit}{If \code{para}, \code{rho}, and \code{tau} are all \code{NULL}, then the \code{u} and \code{v} represent the sample. The measure of association by the \code{fit} declaration will be computed and the parameter estimated subsequently. The \code{fit} has no other utility than to trigger which measure of association is computed internally by the \code{cor} function in \R; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned. Otherwise if either \code{rho} or \code{tau} is given, then the \eqn{\Theta} is computed and a \code{list} having
  \item{para}{The parameter \eqn{\Theta};}
  \item{rho}{Spearman Rho if the \code{rho} is given; and}
  \item{tau}{Kendall Tau if the \code{tau} is given but also if both \code{rho} and \code{tau} are \code{NULL} as mentioned next.}
and if \code{para=NULL} and \code{rho} and \code{tau=NULL}, then the values within \code{u} and \code{v} are used to compute Spearman Rho (\code{fit="rho"}) or Kendall Tau (\code{fit="tau"}) and then compute the parameter, and this is returned in the aforementioned list.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{M}}, \code{\link{P}}
}
\examples{
# Lower tail dependency of Theta = 0.5 --> 2*(0.5)/(1+0.5) = 2/3 (Nelsen, 2006, p. 214)
taildepCOP(cop=RFcop, para=0.5)$lambdaL # 0.66667

\dontrun{
  # Simulate for a Spearman Rho of 0.7, then extract estimated Theta that internally
  # is based on Kendall Tau of U and V, then convert estimate to equivalent Rho.
  set.seed(1)
  UV <- simCOP(1000, cop=RFcop, RFcop(rho=0.7)$para)
  Theta <- RFcop(UV$U, UV$V, fit="tau")$para # 0.607544
  Rho   <- Theta*(4-3*Theta)/(2-Theta)^2     # 0.682255 (nearly 0.7) #}

\dontrun{
  set.seed(1)
  UV <- simCOP(1000, cop=COP, para=list(cop=RFcop, para=RFcop(rho=0.5)$para, reflect=3))
  cor(UV$U, UV$V, method="spearman") # -0.492677 as expected with reversal of V #}
}
\keyword{Raftery copula}
\keyword{copula (formulas)}
\keyword{copula}
\concept{Raftery copula}
\keyword{Nelsen (2006) Examples and Exercises}
\keyword{Theory Examples (reflection/rotation)}

\encoding{utf8}
\name{EMPIRcopdf}
\alias{EMPIRcopdf}
\title{Data Frame Representation of the Bivariate Empirical Copula}
\description{
Generate an \R \code{data.frame} representation of the \emph{bivariate empirical copula} (Salvadori \emph{et al.}, 2007, p. 140) using the coordinates as preserved in the raw data in the parameter object of the bivariate empirical copula.
}
\usage{
EMPIRcopdf(para=NULL, ...)
}
\arguments{
  \item{para}{A vector (single element) of parameters---the U-statistics of the data (see example) to pass to \code{\link{EMPIRcop}}; and}
  \item{...}{Additional arguments to pass to \code{\link{EMPIRcop}}.}
}
\value{
  An \R \code{data.frame} of \eqn{u}, \eqn{v}, and \eqn{\mathbf{C}_{n}(u, v)} values of the bivariate empirical copula is returned.
}
\references{
Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in Nature---An approach using copulas: Springer, 289 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{EMPIRcop}}}
\examples{
\dontrun{
psp <- simCOP(n=39, cop=PSP, ploton=FALSE, points=FALSE) * 150
# Pretend psp is real data, the * 150 is to clearly get into an arbitrary unit system.

# The sort=FALSE is critical in the following two calls to pp() from lmomco.
fakeU <- lmomco::pp(psp[,1], sort=FALSE) # Weibull plotting position i/(n+1)
fakeV <- lmomco::pp(psp[,2], sort=FALSE) # Weibull plotting position i/(n+1)
uv <- data.frame(U=fakeU, V=fakeV) # our U-statistics

empcop <- EMPIRcopdf(para=uv)
plot(empcop$u, empcop$v, cex=1.75*empcop$empcop, pch=16,
     xlab="U, NONEXCEEDANCE PROBABILITY", ylab="V, NONEXCEEDANCE PROBABILITY")
# Dot size increases with joint probability (height of the copulatic surface).
points(empcop$u, empcop$v, col=2) # red circles}
}
\keyword{empirical copula}
\keyword{empirical copula (utility)}

\encoding{utf8}
\name{derCOPinv2}
\alias{derCOPinv2}
\title{Numerical Derivative Inverse of a Copula for U with respect to V}
\description{
Compute the inverse of a numerical partial derivative for \eqn{U} with respect to \eqn{V} of a copula, which is a \emph{conditional quantile function} for nonexceedance probability \eqn{t}, or
\deqn{t = c_v(u) = \mathbf{C}^{(-1)}_{1 \mid 2}(u \mid v) =  \frac{\delta \mathbf{C}(u,v)}{\delta v}\mbox{,}}
and solving for \eqn{u}. Nelsen (2006, pp. 13, 40--41) shows that this inverse is quite important for random variable generation using the \emph{conditional distribution method}. This function is not vectorized and will not be so.
}
\usage{
derCOPinv2(cop=NULL, v, t, trace=FALSE,
           delv=.Machine$double.eps^0.50, para=NULL, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{v}{A single nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{t}{A single nonexceedance probability level \eqn{t};}
  \item{trace}{A logical controlling a \code{message} on whether the signs on the \code{uniroot} are the same---this is helpful in exploring the numerical derivative limits of a given implementation of a copula.}
  \item{delv}{The \eqn{\Delta v} interval for the derivative;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to \code{cop}; and}
  \item{...}{Additional arguments to pass to the copula.}
}
\value{
  Value(s) for the derivative inverse are returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{derCOP2}}}
\examples{
u <- runif(1); t <- runif(1)
derCOPinv2(u,t, cop=W)   # perfect negative dependence
derCOPinv2(u,t, cop=P)   # independence
derCOPinv2(u,t, cop=M)   # perfect positive dependence
derCOPinv2(u,t, cop=PSP) # a parameterless copula example
\dontrun{
# Simulate 500 values from product (independent) copula
plot(NA,NA, type="n", xlim=c(0,1), ylim=c(0,1), xlab="U", ylab="V")
for(i in 1:500) {
   v <- runif(1); t <- runif(1)
   points(derCOPinv2(cop=P, v, t),v, cex=0.5, pch=16) # black dots
}
# Simulate 500 of a Frechet Family copula and note crossing singularities.
for(i in 1:500) {
   v <- runif(1); t <- runif(1)
   u <- derCOPinv2(v, t, cop=FRECHETcop, para=list(alpha=0.7, beta=0.169))
   points(u,v, cex=2, pch=16, col=2) # red dots
} #}
}
\keyword{copula (utility)}
\keyword{copula (derivative inverse)}
\keyword{conditional quantile function}
\keyword{copula operator}
\keyword{derivative}
\keyword{copula (conditional quantile function)}

\encoding{utf8}
\name{densityCOP}
\alias{densityCOP}
\title{Density of a Copula}
\description{
Numerically estimate the \emph{copula density} for a sequence of \eqn{u} and \eqn{v} probabilities for which each sequence has equal steps that are equal to \eqn{\Delta(uv)}. The density \eqn{c(u,v)} of a copula \eqn{\mathbf{C}(u,v)} is numerically estimated by
\deqn{c(u,v) = \bigl[\mathbf{C}(u_2,v_2) - \mathbf{C}(u_2,v_1) - \mathbf{C}(u_1,v_2) + \mathbf{C}(u_1,v_1)\bigr]\, /\, \bigl[\Delta(uv)\times\Delta(uv)\bigr]\mbox{,}}
where \eqn{c(u,v) \ge 0} (see Nelsen, 2006, p. 10; \code{\link{densityCOPplot}}). The \emph{joint density} can be defined by the coupla density for continuous variables and is the ratio of the joint density function \eqn{f(x,y)} for random variables \eqn{X} and \eqn{Y} to the product of the marginal densities (\eqn{f_x(x)} and \eqn{f_y(y)}):

\deqn{c\bigl(F_x(x), F_y(y)\bigr) = \frac{f(x,y)}{f_x(x)f_y(y)}\mbox{,}}

where \eqn{F_x(x)} and  \eqn{F_y(y)} are the respective cumulative distribution functions of \eqn{X} and \eqn{Y}, and lastly \eqn{u = F_x(x)} and \eqn{v = F_y(y)}.
}
\usage{
densityCOP(u,v, cop=NULL, para=NULL, deluv=.Machine$double.eps^0.25,
                truncate.at.zero=TRUE, the.zero=0, sumlogs=FALSE, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{deluv}{The change in the sequences \eqn{\{u, v\} \mapsto \delta, \ldots, 1-\delta; \delta = \Delta(uv)} probabilities;}
  \item{truncate.at.zero}{A density must be \eqn{c(u,v) \ge 0}, but because this computation is based on a rectangular approximation and not analytical, there exists a possibility that very small rectangles could result in numerical values in \R that are less than zero. This usually can be blamed on rounding. This logical if \code{TRUE} truncates computed densities to zero, and the default assumes that the user is providing a proper copula. A \code{FALSE} value is used by the function \code{\link{isfuncCOP}};}
  \item{the.zero}{The value for \dQuote{the zero} where a small number might be useful for pseudo-maximum likelihood estimation using \code{sumlogs};}
  \item{sumlogs}{Return the \eqn{\sum{\log c(u,v; \Theta)}} where \eqn{\Theta} are the parameters in \code{para} and this feature is provided for \code{\link{mleCOP}}; and}
  \item{...}{Additional arguments to pass to the copula function.}
}
\value{
  Value(s) for \eqn{c(u,v)} are returned.
}
\note{
The \pkg{copBasic} package does not currently have copula densities as analytical solutions implemented. This is because initial design decisions were entirely on cumulative distribution function (CDF) representations of the copula.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{simCOP}}, \code{\link{densityCOPplot}}, \code{\link{kullCOP}}, \code{\link{mleCOP}}}
\examples{
\dontrun{
  # Joe (2014, p. 164) shows the closed form copula density for the Plackett.
  "dPLACKETTcop" <- function(u,v,para) {
     uv <- u*v; upv <- u + v; eta <- para - 1
     A <- para*(1+eta*(upv - 2*uv)); B <- ((1+eta*upv)^2 - 4*para*eta*uv)^(3/2)
     return(A/B)
  }
  dPLACKETTcop(0.32, 0.74,            para=1.3) # 0.9557124
  densityCOP(  0.32, 0.74, cop=PLcop, para=1.3) # 0.9557153 }

\dontrun{
  # Joe (2014, p. 165) shows the corner densities of the Plackett as Theta.
  # copBasic uses numerical density estimation and not analytical formula.
  eps <- .Machine$double.eps
  densityCOP(0,0, cop=PLcop, para=4) # 3.997073  (default eps^0.25)
  densityCOP(1,1, cop=PLcop, para=4) # 3.997073  (default eps^0.25)
  densityCOP(1,1, cop=PLcop, para=4, deluv=eps)     # 0 (silent failure)
  densityCOP(1,1, cop=PLcop, para=4, deluv=eps^0.5) # 4.5
  densityCOP(1,1, cop=PLcop, para=4, deluv=eps^0.4) # 4.002069
  densityCOP(1,1, cop=PLcop, para=4, deluv=eps^0.3) # 3.999513
  # So, we see that the small slicing does have an effect, the default of 0.25 is
  # intented for general application by being away enough from machine limits.}

\dontrun{
  # Joe (2014, p. 170) shows a closed form copula density for "Bivariate Joe/B5" copula
  "dJOEB5cop" <- function(u, v, para) {
     up <- (1-u)^para; vp <- (1-v)^para; eta <- para - 1
     A <- (up + vp - up*vp); B <- (1-u)^eta * (1-v)^eta; C <- (eta + A)
     return(A^(-2 + 1/para) * B * C)
  }
  densityCOP(0.32, 0.74, cop=JOcopB5, para=1.3)  # 0.9410653
  dJOEB5cop( 0.32, 0.74, para=1.3)               # 0.9410973 }
}
\keyword{visualization}
\keyword{copula (density)}
\keyword{Joe (2014) Examples and Exercises}

\encoding{utf8}
\name{MOcop}
\alias{MOcop}
\title{The Marshall--Olkin Copula}
\description{
The \emph{Marshall--Olkin copula} (Dobrowolski and Kumar, 2014, eq. 2.6; Nelsen, 2006, p. 53), which is also known as the \emph{Generalized \enc{Cuadras–Augé}{Cuadras--Auge} copula}, having parameters \eqn{\alpha, \beta \in (0,1)} is given by

\deqn{\mathbf{C}_{(\alpha,\beta)}(u,v) = \mathrm{min}(u^{1-\alpha}v, uv^{1-\beta})\mbox{,\ }}

and efficiently computed by compound equations for \eqn{u^\alpha \ge v^\beta}

\deqn{\mathbf{C}_{(\alpha,\beta)}(u,v) = \mathbf{MO}(u,v) = u^{1-\alpha}v\mbox{,}}

and for \eqn{u^\alpha < v^\beta} is

\deqn{\mathbf{C}_{(\alpha,\beta)}(u,v) = \mathbf{MO}(u,v) = uv^{1-\beta}\mbox{.}}

As \eqn{\alpha, \beta \rightarrow 1}, the copula limits to the \emph{comonotonicity coupla} (\eqn{\mathbf{M}(u,v)}; \code{\link{M}}), and as \eqn{\alpha, \beta \rightarrow 0}, the copula limits to the \emph{independence copula} (\eqn{\mathbf{\Pi}(u,v)}; \code{\link{P}}). The copula can be expected to have a visible singularity as the parameters increase and (or) a simulation size becomes large. The copula with \eqn{\alpha = \beta} is \emph{permutation symmetric} (\code{\link{isCOP.permsym}}, \code{\link{breveCOP}}) and is known as the \emph{\enc{Cuadras–Augé}{Cuadras--Auge}} copula. \emph{Spearman Rho} (\code{\link{rhoCOP}}) of the copula is

\deqn{\rho_\mathbf{C} = 3\alpha\beta / (2\alpha + 2\beta - \alpha\beta)\mbox{,\ }}

and \emph{Kendall tau} (\code{\link{tauCOP}}) is

\deqn{\tau_\mathbf{C} = \alpha\beta / (\alpha + \beta - \alpha\beta)\mbox{.}}

Parameter estimation using \eqn{\rho_\mathbf{C}} and \eqn{\tau_\mathbf{C}} is problematic because the two statistics are so \dQuote{correlated} with each other and that either nonunique solutions or nonconverged solutions manifest with simultaneous numerical optimization. Also, application of \emph{maximum likelihood} (\code{\link{mleCOP}}) can be problematic because of varying contributions of singularities that the copula can produce. The total contribution of singularity is given by \eqn{S(u\rightarrow1,v\rightarrow1) = \alpha\beta/ (\alpha + \beta - \alpha\beta)}, which Nelsen (2006, p. 165) remarks that it is \dQuote{interesting to note that} Kendall tau is the singular component. Lastly, the \emph{L-comoments} of copulas (\code{\link{lcomCOP}}) appear an attractive mechanism for parameter estimation not effected by singularities (refer to \bold{Examples}). The \eqn{\mathbf{MO}} copula has a role in the three-parameter \emph{Gumbel--Hougaard copula} copula (\code{\link{GHcop}}).
}
\usage{
MOcop(u, v, para=NULL, lcomCOP=NULL, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (single element) of parameters---the \eqn{\alpha, \beta} parameters of the copula;}
  \item{lcomCOP}{A vector containing \eqn{\rho_\mathbf{C}} and the two L-coskews (refer to \bold{Examples}) from which the parameters are returned; and}
  \item{...}{Additional arguments to pass.}
}
\value{
Value(s) for the copula are returned. Otherwise if either \code{lcomCOP} is given, then the \eqn{\alpha, \beta} are computed by numerical optimization the the Spearman Rho and the two L-coskews (refer to package \pkg{lmomco} and function \code{lmomco::lcomoms2} for more details) (refer the demonstration of L-comoment parameter estimation in the \bold{Examples} below).
}
\references{
Dobrowolski, E., and Kumar, P., 2014, Some properties of the Marshall--Olkin and generalized Cuadras--\enc{Augé}{Auge} families of copula: Australian Journal of Mathematical Analysis and Applications: v. 11, no. 1, art. 2, pp. 1--13, accessed on August 10, 2025, at \url{https://ajmaa.org/searchroot/files/pdf/v11n1/v11i1p2.pdf}.

Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{P}}, \code{\link{taildepCOP}}
}
\examples{
\dontrun{
  #   **** Demonstration of copula fitting by Method of L-comoments ****
  n    <- 300 # large enough sample size to clearly show L-comoment viability
  UV   <- simCOP(n, cop=MOcop, seed=1, para=c(0.3, 0.7), pch=16, col="seagreen")
  lcm  <- lmomco::lcomoms2(UV) # sample L-comoments (mean off diag. T2 == Spearman Rho)
  sara <- MOcop(lcomCOP=c(mean(diag(lcm$T2[2:1,1:2])), lcm$T3[1,2], lcm$T3[2,1]))
  sara$para <- round(sara$para, digits=8)
  uv   <- simCOP(n, cop=MOcop, seed=1, para=sara$para, col="blue", ploton=FALSE)
  message("MOcop : alpha=", sara$para[1], ", beta=", sara$para[2])
  mara <- mleCOP(UV[,1], v=UV[,2], cop=MOcop, init.para=c(0.2,0.2),
                 parafn=function(x) pnorm(x))
  mara$para <- round(mara$para, digits=8)
  uv   <- simCOP(n, cop=MOcop, seed=1, para=mara$para, col="red", ploton=FALSE)
  message("MOcop : alpha=", mara$para[1], ", beta=", mara$para[2]) #}

\dontrun{
  # This example developed as a demonstrator of the disarray that the
  # Spearman Footrule measures as a nonunique relation to Spearman Rho.
  # The permutation asymmetry causes the relation between the two to
  # "paint" for positive association as shown in the graphic produced.
  cex <- c(1.3, 1.0, 0.7, 0.4)
  col <- c("#7D0112", "#67C7B1", "#1F28A2", "#CCC86C")
  pch <- c(1, 16, 1, 16)
  plot(c(-0.5, 1), c(0, 1), type="n", xlab="Spearman Footrule", ylab="")
  mtext(paste0("Absolute value of Spearman rho\n",
               "Actual sign is the same as for the Footrule"), side=2, line=2)
  mtext(paste0("Attainable Marshall-Olkin copula, MOcop() with permutation\n",
               "inclusion (breveCOP) and reflections (COP)"), line=1)
  for(r in 1:4) { # reflections, COP
    for(k in seq(-1, 1, by=0.1)) { # permutation asymmetry insertion, breveCOP
      for(a in seq(0, 1, by=0.11)) {   # increment slightly different from beta
        for(b in seq(0, 1, by=0.09)) { # increment slightly different from alpha
          para <- list(cop=MOcop, para=c(a,b), breve=k, reflect=r)
          fotT <- footCOP(cop=breveCOP, para=para)
          rhoT <-  rhoCOP(cop=breveCOP, para=para)
          points(fotT, abs(rhoT), lwd=0.8, pch=pch[r], cex=cex[r], col=col[r])
          #UV <- simCOP(100, cop=breveCOP, para=para)
        }
      }
    }
  } #}
}
\keyword{Marshall--Olkin copula}
\keyword{Cuadras--Auge copula}
\keyword{copula (formulas)}
\keyword{copula}
\keyword{L-comoments}
\keyword{Lcomoments}
\concept{Marshall--Olkin copula}
\concept{Marshall-Olkin copula}
\concept{Cuadras-Auge copula}

\encoding{utf8}
\name{HRcop}
\alias{HRcop}
\title{The \enc{Hüsler}{Husler}--Reiss Extreme Value Copula}
\description{
The \emph{\enc{Hüsler}{Husler}--Reiss copula} (Joe, 2014, p. 176) is
\deqn{\mathbf{C}_{\Theta}(u,v) = \mathbf{HR}(u,v) = \mathrm{exp}\bigr[-x \Phi(X) - y\Phi(Y)\bigr]\mbox{,}}
where \eqn{\Theta \ge 0}, \eqn{x = - \log(u)}, \eqn{y = - \log(v)}, \eqn{\Phi(.)} is the cumulative distribution function of the standard normal distribution, \eqn{X} and \eqn{Y} are defined as:
\deqn{X = \frac{1}{\Theta} + \frac{\Theta}{2}  \log\biggl(\frac{x}{y}\biggr)\mbox{\ and\ } Y = \frac{1}{\Theta} + \frac{\Theta}{2} \log\biggl(\frac{y}{x}\biggr)\mbox{.}}
As \eqn{\Theta \rightarrow 0^{+}}, the copula limits to \emph{independence} (\eqn{\mathbf{\Pi}}; \code{\link{P}}). The \eqn{\mathbf{HR}} copula is a \emph{bivariate extreme value copula} (\eqn{BEV}), and the parameter \eqn{\Theta} requires numerical methods. Because there is \dQuote{hardly any practical difference among [these] bivariate exchangeable parametric families} distinguishing between these (\code{\link{GHcop}}, \code{\link{GLcop}}, and \code{\link{HRcop}}) \dQuote{requires extremely large sample sizes} (Hofert \emph{et al.}, 2016, p. 117). This observation is strongly supported by the \emph{L-comoment ratio diagrams} explored in \code{\link{LCOMDIA_ManyCops}}.
}
\usage{
HRcop(u, v, para=NULL, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (single element) of parameters---the \eqn{\Theta} parameter of the copula; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\references{
Hofert, M., Kojadinovic, I., \enc{Mächler}{Machler},  M., and Yan, J., 2018, Elements of copula modeling with R: Dordrecht, Netherlands, Springer.

Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{P}}, \code{\link{GHcop}}, \code{\link{GLcop}}, \code{\link{tEVcop}}
}
\examples{
# Parameter Theta = pi recovery through the Blomqvist Beta (Joe, 2014, p. 176)
qnorm(1 - log( 1 + blomCOP(cop=HRcop, para=pi) ) / ( 2 * log(2) ) )^(-1)
}
\keyword{Husler--Reiss copula}
\keyword{copula (formulas)}
\keyword{copula}
\concept{Husler--Reiss extreme value copula}
\concept{Husler-Reiss extreme value copula}
\concept{Husler-Reiss copula}
\concept{Husler--Reiss copula}
\keyword{Joe (2014) Examples and Exercises}
\keyword{copula (extreme value)}
\keyword{extreme value copula}

\encoding{utf8}
\name{EMPIRsimv}
\alias{EMPIRsimv}
\title{Simulate a Bivariate Empirical Copula For a Fixed Value of U}
\description{
\emph{EXPERIMENTAL}---Perform a simulation on a \emph{bivariate empirical copula} to extract the random variates \eqn{V} from a given and fixed value for \eqn{u=} constant. The purpose of this function is to return a simple vector of the \eqn{V} simulations. This behavior is similar to \code{\link{simCOPmicro}} but differs from the general 2-D simulation implemented in the other functions: \code{\link{EMPIRsim}} and \code{\link{simCOP}}---these two functions generate \R \code{data.frame}s of simulated random variates \eqn{U} and \eqn{V} and optional graphics as well.

For the usual situation in which \eqn{u} is not a value aligned on the grid, then the bounding conditional quantile functions are solved for each of the \eqn{n} simulations and the following interpolation is made by
\deqn{v = \frac{v_1/w_1 + v_2/w_2}{1/w_1 + 1/w_2}\mbox{,}}
which states that that the weighted mean is computed. The values \eqn{v_1} and \eqn{v_2} are ordinates of the conditional quantile function for the respective grid lines to the left and right of the \eqn{u} value. The values \eqn{w_1} \eqn{=} \eqn{u - u^\mathrm{left}_\mathrm{grid}} and \eqn{w_2} \eqn{=} \eqn{u^\mathrm{right}_\mathrm{grid} - u}.
}
\usage{
EMPIRsimv(u, n=1, empgrid=NULL, kumaraswamy=FALSE, ...)
}
\arguments{
  \item{u}{The fixed probability \eqn{u} on which to perform conditional simulation for a sample of size \eqn{n};}
  \item{n}{A sample size, default is 1;}
  \item{empgrid}{Gridded empirical copula from \code{\link{EMPIRgrid}};}
  \item{kumaraswamy}{A logical to trigger Kumaraswamy distribution smoothing of the conditional quantile function that is passed to \code{\link{EMPIRgridderinv}}. The Kumaraswamy distribution is a distribution having support \eqn{[0,1]} with an explicit quantile function and takes the place of a Beta distribution (see \pkg{lmomco} function \code{quakur()} for more details); and}
  \item{...}{Additional arguments to pass.}
}
\value{
  A vector of simulated \eqn{V} values is returned.
}
\author{W.H. Asquith}
\seealso{\code{\link{EMPIRgrid}}, \code{\link{EMPIRsim}}}
\examples{
\dontrun{
nsim <- 3000
para <- list(alpha=0.15,  beta=0.65,
             cop1=PLACKETTcop, cop2=PLACKETTcop, para1=.005, para2=1000)
set.seed(10)
uv <- simCOP(n=nsim, cop=composite2COP, para=para, pch=16, col=rgb(0,0,0,0.2))
uv.grid <- EMPIRgrid(para=uv, deluv=.1)
set.seed(1)
V1 <- EMPIRsimv(u=0.6, n=nsim, empgrid=uv.grid)
set.seed(1)
V2 <- EMPIRsimv(u=0.6, n=nsim, empgrid=uv.grid, kumaraswamy=TRUE)
plot(V1,V2)
abline(0,1)

invgrid1 <- EMPIRgridderinv(empgrid=uv.grid)
invgrid2 <- EMPIRgridderinv(empgrid=uv.grid, kumaraswamy=TRUE)
att <- attributes(invgrid2); kur <- att$kumaraswamy
# Now draw random variates from the Kumaraswamy distribution using
# rlmomco() and vec2par() provided by the lmomco package.
set.seed(1)
kurpar <- lmomco::vec2par(c(kur$Alpha[7], kur$Beta[7]), type="kur")
Vsim <- lmomco::rlmomco(nsim, kurpar)

print(summary(V1))   # Kumaraswamy not core in QDF reconstruction
print(summary(V2))   # Kumaraswamy core in QDF reconstruction
print(summary(Vsim)) # Kumaraswamy use of the kumaraswamy

# Continuing with a conditional quantile 0.74 that will not land along one of the
# grid lines, a weighted interpolation will be done.
set.seed(1) # try not resetting the seed
nsim <- 5000
V <- EMPIRsimv(u=0.74, n=nsim, empgrid=uv.grid)
# It is important that the uv.grid used to make V is the same grid used in inversion
# with kumaraswamy=TRUE to guarantee that the correct Kumaraswamy parameters are
# available if a user is doing cut and paste and exploring these examples.
set.seed(1)
V1 <- lmomco::rlmomco(nsim, lmomco::vec2par(c(kur$Alpha[8], kur$Beta[8]), type="kur"))
set.seed(1)
V2 <- lmomco::rlmomco(nsim, lmomco::vec2par(c(kur$Alpha[9], kur$Beta[9]), type="kur"))

plot( lmomco::pp(V),  sort(V), type="l", lwd=4, col=8) # GREY is empirical from grid
lines(lmomco::pp(V1), sort(V1), col=2, lwd=2) # Kumaraswamy at u=0.7 # RED
lines(lmomco::pp(V2), sort(V2), col=3, lwd=2) # Kumaraswamy at u=0.8 # GREEN

W1 <- 0.74 - 0.7; W2 <- 0.8 - 0.74
Vblend <- (V1/W1 + V2/W2) / sum(1/W1 + 1/W2)
lines(lmomco::pp(Vblend), sort(Vblend), col=4, lwd=2) # BLUE LINE
# Notice how the grey line and the blue diverge for about F < 0.1 and F > 0.9.
# These are the limits of the grid spacing and linear interpolation within the
# grid intervals is being used and not direct simulation from the Kumaraswamy.}
}
\keyword{empirical copula}
\keyword{empirical copula (simulation)}

\encoding{utf8}
\name{JOcopB5}
\alias{JOcopB5}
\title{The Joe/B5 Copula (B5)}
\description{
The \emph{Joe/B5 copula} (Joe, 2014, p. 170) is
\deqn{\mathbf{C}_{\Theta}(u,v) = \mathbf{B5}(u,v) = 1 - \bigl((1-u)^\Theta + (1-v)^\Theta - (1-u)^\Theta (1-v)^\Theta\bigr)^{1/\Theta}\mbox{,}}
where \eqn{\Theta \in [1,\infty)}.
The copula as \eqn{\Theta \rightarrow \infty} limits to the \emph{comonotonicity coupla} (\eqn{\mathbf{M}(u,v)} and \code{\link{M}}), as \eqn{\Theta \rightarrow 1^{+}} limits to \emph{independence copula} (\eqn{\mathbf{\Pi}(u,v)}; \code{\link{P}}). Finally, the parameter \eqn{\Theta} is readily computed from a \emph{Kendall Tau} (\code{\link{tauCOP}}) by
\deqn{\tau_\mathbf{C} = 1 + \frac{2}{2-\Theta}\bigl(\psi(2) - \psi(1 + 2/\Theta)\bigr)\mbox{,}}
where \eqn{\psi} is the \code{digamma()} function and as \eqn{\Theta \rightarrow 2} then \deqn{\tau_\mathbf{C}(\Theta \rightarrow 2) = 1 - \psi'(2)} where \eqn{\psi'} is the \code{trigamma()} function.
}
\usage{
JOcopB5(u, v, para=NULL, tau=NULL, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (single element) of parameters---the \eqn{\Theta} parameter of the copula;}
  \item{tau}{Optional Kendall Tau; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned. Otherwise if \code{tau} is given, then the \eqn{\Theta} is computed and a \code{list} having
  \item{para}{The parameter \eqn{\Theta}, and}
  \item{tau}{Kendall Tau.}
and if \code{para=NULL} and \code{tau=NULL}, then the values within \code{u} and \code{v} are used to compute Kendall Tau and then compute the parameter, and these are returned in the aforementioned list.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{M}}, \code{\link{P}}
}
\examples{
# Upper tail dependency of Theta = pi --> 2 - 2^(1/pi) = 0.753131 (Joe, 2014, p. 171).
taildepCOP(cop=JOcopB5, para=pi)$lambdaU # 0.75313

# Blomqvist Beta of Theta = pi (Joe, 2014, p. 171).
blomCOP(cop=JOcopB5, para=pi)        # 0.5521328
3 - 4*(2*(1/2)^pi - (1/4)^pi)^(1/pi) # 0.5521328

\dontrun{
  # A test near the limiting Theta for trigamma()
  UV   <- simCOP(cop=JOcopB5, para=2, n=10000)
  para <- JOcopB5(UV[,1], UV[,2])$para
  message("Tau difference ", round(2-para, digits=2), " is small.") #}

\dontrun{
  # embedment of parameters for permutation asymmetry and rotation
  para <- list(cop=JOcopB5, para=30, reflect=3, breve=0.5)
  UV <- simCOP(cop=COP, para=list(cop=breveCOP, para=para), n=1000) #}

\dontrun{
  # Performance of the parameter as function of Kendall tau to the sample
  # Kendall tau. There will be some failures in the root solving as Tau
  # goes towards 1.
  ts <- seq(0,1, by=0.001); us <- runif(400)    # Taus and a set pf random Us
  ps <- sapply(ts, function(t) JOcopB5(tau=t)$para)    # parameters from Taus
  ps <- unlist(ps);    names(ps) <- NULL     # handling for solution failures
  ps <- c(ps, rep(NA, length(ts)-length(ps))) # padding the solution failures
  sts <- sapply(ps, function(p) { if(is.na(p)) return(NA) # sample Taus
            t <- cor(us, simCOPv(us, cop=JOcopB5, para=p), method="kendall")
            ifelse(is.null(t), return(NA), return(t)) })
  plot(ts, sts, lwd=0.8, col=grey(0.6), las=1,
       xlab="Kendall tau", ylab="Sample Kendall tau")
  abline(0,1, col="black", lwd=2) # }
}
\keyword{Joe/B5 copula}
\keyword{copula (formulas)}
\keyword{copula}
\concept{Joe/B5 copula}
\keyword{Joe (2014) Examples and Exercises}
\keyword{Theory Examples (reflection/rotation)}

\encoding{utf8}
\name{isfuncCOP}
\alias{isfuncCOP}
\title{Is a General Bivariate Function a Copula by Gridded Search?}
\description{
Is a general bivariate function a copula? Three properties are identified by Nelsen (2006, p. 10) for a bivariate copula \eqn{\mathbf{C}(u,v)}:
\deqn{\mathbf{C}(u,0) = 0 = \mathbf{C}(0,v)\mbox{\quad Nelsen 2.2.2a,}}
\deqn{\mathbf{C}(u,1) = u \mbox{\ and\ } \mathbf{C}(1,v) = v\mbox{\quad Nelsen 2.2.2b, and}}
for every \eqn{u_1, u_2, v_1, v_2} in \eqn{\mathcal{I}^2} such that \eqn{u_1 \le u_2} and \eqn{v_1 \le v_2},
\deqn{\mathbf{C}(u_2, v_2) - \mathbf{C}(u_2, v_1) - \mathbf{C}(u_1, v_2) + \mathbf{C}(u_1, v_1) \ge 0 \mbox{\quad Nelsen 2.2.2c.}}
The last condition is known also as \dQuote{2-increasing.} The \code{isfuncCOP} works along a gridded search in the domain \eqn{\mathcal{I}^2 = [0,1]\times[0,1]} for the 2-increasing check with a resolution \eqn{\Delta u = \Delta v} \eqn{=} \code{delta}. Because there are plenty of true copula functions available in the literature it seems unlikely that this function provides much production utility in investigations. This function is provided because part of the objectives of the \pkg{copBasic} package is for instructional purposes. The computational overhead is far too great for relative benefit to somehow dispatch to this function all the time using the other copula utilities in this package.
}
\usage{
isfuncCOP(cop=NULL, para=NULL, delta=0.002, ...)
}
\arguments{
  \item{cop}{A potential bivariate copula function that accepts two arguments for the \eqn{u} and \eqn{v} and parameters along argument \code{para} with option of additional arguments through the \code{...} argument;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{delta}{The \eqn{\Delta u = \Delta v} of the grid edges; and}
  \item{...}{Additional arguments to pass to the copula function.}
}
\value{
  A logical value of \code{TRUE} or \code{FALSE} is returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{S. Kloibhofer (idea and most code) and W.H. Asquith (documentation)}
\seealso{\code{\link{densityCOP}}
}
\examples{
\dontrun{
"NelsenEx2.11" <- function(u,v, ...) { # Nelsen (2006, exer. 2.11, p. 16)
  if(length(u) > 1 & length(v) > 1 & length(u) != length(v)) return(NA)
  if(length(u) == 1) u <- rep(u, length(v))
  if(length(v) == 1) v <- rep(v, length(u))
  return(sapply(1:length(u), function(i) { upv <- u[i] + v[i]
                 if(2/3 <= upv & upv <= 4/3) return(min(c(u,v,1/3,upv-(2/3))))
                 max(u[i]+v[i]-1, 0) }))
}
isfuncCOP(cop=NelsenEx2.11) # FALSE}
}
\keyword{copula (properties)}
\keyword{Nelsen (2006) Examples and Exercises}

\encoding{utf8}
\name{TRIcop}
\alias{TRIcop}
\title{The Triangular Copula}
\description{
The \emph{Triangular copula} (Nelsen, 2006, exam. 3.3, pp. 59--60; exer. 3.7, pp. 64--65) is
\deqn{\mathbf{C}_{\Theta}(u,v) = \mathbf{TRI}(u,v)} for \eqn{\Theta \in (0,-1)}. The copula, as \eqn{\Theta \rightarrow 0} limits, to the \emph{countermonotonicity coupla} (\eqn{\mathbf{W}(u,v)}; \code{\link{W}}), as \eqn{\Theta \rightarrow 1} limits to the \emph{comonotonicity copula} (\eqn{\mathbf{M}(u,v)};  \code{\link{M}}). The parameter \eqn{\Theta} is readily computed from a \emph{Spearman Rho} (\code{\link{rhoCOP}}) or \emph{Kendall Tau} (\code{\link{tauCOP}}) by \eqn{\rho_\mathbf{C} = \tau_\mathbf{C} = 2\Theta-1}. However in general for finite samples, the sample versions of Spearman Rho and Kendall Tau will not be equal and different \eqn{\Theta} wil result.
}
\usage{
TRIcop(u, v, para=NULL, rhotau=NULL, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (single element) of parameters---the \eqn{\Theta} parameter of the copula;}
  \item{rhotau}{Optional Spearman Rho or Kendall Tau; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned. Otherwise if \code{rhotau} is given, then the \eqn{\Theta} is computed and a \code{list} having
  \item{para}{The parameter \eqn{\Theta}, and}
  \item{rhotau}{Spearman Rho or Kendall Tau.}
and if \code{para=NULL} and \code{rhotau=NULL}, then the values within \code{u} and \code{v} are used to compute Spearman Rho or Kendall Tau and then compute the parameter, and these are returned in the aforementioned list.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{M}}, \code{\link{W}}
}
\examples{
# Concerning Nelsen (2006, exer. 5.6, p. 171)
rhoCOP(cop=TRIcop, para=0.25) # -0.4999916
tauCOP(cop=TRIcop, para=0.25) # -0.5

\dontrun{
  for(i in seq_len(4)) {
    para <- list(cop=TRIcop, para=0.25, reflect=i)
    uv <- simCOP(100, cop=COP, para=para, seed=1, col=i, ploton=! as.logical(i-1))
  } # }
}
\keyword{Triangular copula}
\keyword{copula (formulas)}
\keyword{copula}
\concept{Triangular copula}
\keyword{Nelsen (2006) Examples and Exercises}

\encoding{utf8}
\name{EMPIRcop}
\alias{EMPIRcop}
\title{The Bivariate Empirical Copula}
\description{
The \emph{bivariate empirical copula} (Nelsen, 2006, p. 219) for a bivariate sample of length \eqn{n} is defined for random variables \eqn{X} and \eqn{Y} as

\deqn{\mathbf{C}_n\biggl(\frac{i}{n}, \frac{j}{n}\biggr) = \frac{\mathrm{number\ of\ pairs\ (}x,y\mathrm{)\ with\ }x \le x_{(i)}\mathrm{\ and\ }y \le y_{(j)}}{n}\mbox{,}}

where \eqn{x_{(i)}} and \eqn{y_{(i)}}, \eqn{1 \le i,j \le n} or expressed as
\deqn{\mathbf{C}_n\biggl(\frac{i}{n}, \frac{j}{n}\biggr) =
     \frac{1}{n}\sum_{i=1}^n \mathbf{1}\biggl(\frac{R_i}{n} \le u_i, \frac{S_i}{n} \le v_i \biggr)\mbox{,}}
where \eqn{R_i} and \eqn{S_i} are ranks of the data for \eqn{U} and \eqn{V}, and \eqn{\mathbf{1}(.)} is an \emph{indicator function} that score 1 if condition is true otherwise scoring zero. Using more generic notation, the empirical copula can be defined by
\deqn{\mathbf{C}_{n}(u,v) =
     \frac{1}{n}\sum_{i=1}^n \mathbf{1}\bigl(u^\mathrm{obs}_{i} \le u_i, v^\mathrm{obs}_{i} \le v_i \bigr)\mbox{,}}
where \eqn{u^\mathrm{obs}} and \eqn{v^\mathrm{obs}} are thus some type of nonparametric nonexceedance probabilities based on counts of the underlying data expressed in probabilities.


\emph{Hazen Empirical Copula}---The \dQuote{Hazen form} of the empirical copula is
\deqn{\mathbf{C}^\mathcal{H}_{n}(u,v) =
     \frac{1}{n}\sum_{i=1}^n \mathbf{1}\biggl(\frac{R_i - 0.5}{n} \le u_i, \frac{S_i - 0.5}{n} \le v_i \biggr)\mbox{,}}
which can be triggered by \code{ctype="hazen"}. This form is named for this package because of direct similarity of the \emph{Hazen plotting position} to the above definition. Joe (2014, pp. 247--248) uses the Hazen form. Joe continues by saying \dQuote{[the] adjustment of the uniform score [\eqn{(R - 0.5)/n]}] could be done in an alternative form, but there is [asymptotic] equivalence[, and that] \eqn{\mathbf{C}^\mathcal{H}_{n}} puts mass of \eqn{n^{-1}} at the tuples \eqn{([r_{i1} - 0.5]/n, \ldots, [r_{id} - 0.5]/n)} for \eqn{i = 1, \ldots, n}.} A footnote by Joe (2014) says that \dQuote{the conversion [\eqn{R/(n+1)}] is commonly used for the empirical copula.} This later form is the \dQuote{Weibull form} described next. Joe's preference for the Hazen form is so that the sum of squared normal scores is closer to unity for large \eqn{n} than such a sum would be attained using the Weibull form.


\emph{Weibull Empirical Copula}---The \dQuote{Weibull form} of the empirical copula is
\deqn{\mathbf{C}^\mathcal{W}_{n}(u,v) =
     \frac{1}{n}\sum_{i=1}^n \mathbf{1}\biggl(\frac{R_i}{n+1} \le u_i, \frac{S_i}{n+1} \le v_i \biggr)\mbox{,}}
which can be triggered by \code{ctype="weibull"}. This form is named for this package because of direct similarity of the \emph{Weibull plotting position} to the definition, and this form is the default (see argument description).

\emph{Bernstein Empirical Copula}---The empirical copula can be extended nonparametrically as the \emph{Bernstein empirical copula} (\enc{Hernández-Maldonado}{Hernandez-Maldonado} \emph{et al.}, 2012) and is formulated as

\deqn{\mathbf{C}^\mathcal{B}_n(u,v; \eta) = \sum_{i=1}^n\sum_{j=1}^n \mathbf{C}_{n}\biggl(\frac{i}{n},\frac{j}{n}\biggr) \times \eta(i,j; u,v)\mbox{,}}

where the individual \emph{Bernstein weights} \eqn{\eta(i,j)} for the \eqn{k}th paired value of the \eqn{u} and \eqn{v} vectors are

\deqn{\eta(i,j; u,v) = {n \choose i} u^i (1-u)^{n-i} \times {n \choose j} u^j (1-u)^{n-j}\mbox{.}}

The Bernstein extension, albeit conceptually pure in its shuffling by binomial coefficients and left- and right-tail weightings, is quite CPU intensive as inspection of the equations above indicates a nest of four \code{for()} loops in \R. (The native \R code of \pkg{copBasic} uses the \code{sapply()} function in \R liberally for substantial but not a blazing fast speed increase.) The Bernstein extension results in a smoother surface of the empirical copula and can be triggered by \code{ctype="bernstein"}. Remark, \enc{Hernández-Maldonado}{Hernandez-Maldonado} \emph{et al.} (2024) provide a fast algorithm for the Bernstein smoothing---this implemented in \code{\link{EMPIRgrid_fast}}.

\emph{Checkerboard Empirical Copula}---A simple smoothing to the empirical copula is the \emph{checkerboard empirical copula} (Segers \emph{et al.}, 2017) that has been adapted from the \pkg{copula} package. It is numerically intensive like the Bernstein and possibly of limited usefulness for large sample sizes. The checkerboard extension can be triggered by \code{ctype="checkerboard"} and is formulated as

\deqn{\mathbf{C}^\sharp_{n}(U) = \frac{1}{n+o} \sum_{i=1}^n\prod_{i=1}^d \mathrm{min}[\mathrm{max}[n U_j - R^{(n)}_{i,j} + 1,0],1]\mathrm{,}}

where \eqn{U} is a \eqn{d=2} column matrix of \eqn{u} and \eqn{v}, \eqn{R} is a rank function, and \eqn{o} is an offset term on \eqn{[0,1]}.


The \emph{empirical copula frequency} can be defined (Nelson, 2006, p. 219) as
\deqn{\mathbf{c}_n(u, v) = \mathbf{C}_n\biggl(\frac{i}{n}, \frac{j}{n}\biggr) -
                           \mathbf{C}_n\biggl(\frac{i-1}{n}, \frac{j}{n}\biggr) -
                           \mathbf{C}_n\biggl(\frac{i}{n}, \frac{j-1}{n}\biggr) +
                           \mathbf{C}_n\biggl(\frac{i-i}{n}, \frac{j-1}{n}\biggr)\mbox{.}}

Remembering that the several asymptotically equivalent definitions of the empirical copula do converge as \eqn{n} becomes large, we can show small-sample congruence between the empirical copula from the \pkg{copula} package and \pkg{copBasic} to the Weibull version of \code{EMPIRcop} because of \pkg{copula::pobs()} is itself Weibull:
\preformatted{
  uv <- as.matrix( simCOP(50, cop=P) )
  ctype <- "weibull"; # Try the other ctypes supported by EMPIRcop
  a <- EMPIRcop(uv[,1], uv[,2], para=as.data.frame(uv), ctype=ctype)
  b <- copula::C.n(uv, uv, offset=0) # if ctype="weibull" equal values
  plot(a, b, xlab=paste0('copBasic EMPIRcop(ctype="', ctype, '")'),
             ylab="copula::C.n"); abline(0,1, col="red")
}
}
\usage{
EMPIRcop(u, v, para=NULL,
               ctype=c("weibull", "hazen", "1/n", "bernstein", "checkerboard"),
                          bernprogress=FALSE, checkerboard.offset=0, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (single element) of parameters---the U-statistics of the data (see \bold{Examples}). Alternatively, \code{para} can be a \code{list} holding a \code{para} as would be done if it were a vector, but arguments \code{bernstein} and \code{bernprogress} can be optionally included---this feature is provided so that the Bernstein refinement can be controlled within the context of other functions calling \code{EMPIRcop} such as by \code{\link{level.curvesCOP}};}
  \item{ctype}{An alternative means for trigging the definition of  \eqn{\mathbf{C}_n}, \eqn{\mathbf{C}^\mathcal{W}_n} (default), \eqn{\mathbf{C}^\mathcal{H}_n}, \eqn{\mathbf{C}^\mathcal{B}_n}, or \eqn{\mathbf{C}^\sharp_n}. This argument of the same name is also used by \code{\link{blomCOP}};}
  \item{bernprogress}{The Bernstein copula extension is CPU intensive(!), so a splash counter is pushed to the console via the \code{message()} function in \R so as to not discourage the user;}
  \item{checkerboard.offset}{A scaling of the ratio \code{sum(....)/(n+offset)} for the checkerboard empirical copula; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\note{
Not all theoretical measures of copula dependence (both measures of association and measures of asymmetry), which use numerical integration by the \code{integrate()} function in \R, can be used for all empirical copulas because of \dQuote{divergent} integral errors; however, examples using \emph{Hoeffding Phi} (\eqn{\Phi_\mathbf{C}}; \code{\link{hoefCOP}}) and shown under \bold{Examples}. Other measures of copula dependence include \code{\link{blomCOP}}, \code{\link{footCOP}}, \code{\link{giniCOP}}, \code{\link{rhoCOP}}, \code{\link{tauCOP}}, \code{\link{wolfCOP}}, \code{\link{joeskewCOP}}, and \code{\link{uvlmoms}}. Each of these measures fortunately has a built-in sample estimator.

It is important to distinguish between a sample estimator and the estimation of the measure using the empirical copula itself via the \code{EMPIRcop} function. The sample estimators (triggered by the \code{as.sample} arguments for the measures) are reasonably fast and numerically preferred over using the empirical copula. Further, the generally slow numerical integrations for the theoretical definitions of these copula measures might have difficulties. Limited testing, however, suggests prevalence of numerical integration not erroring using the Bernstein extension of the empirical copula, which must be a by-product of the enhanced and sufficient smoothness for the \R default numerical integration to succeed. Many of the measures have \code{brute} option for a brute-force numerical integration on a regular grid across the empirical copula---these are slow but should not trigger errors. As a general rule, users should still use the sample estimators instead.
}
\references{
\enc{Hernández-Maldonado}{Hernandez-Maldonado}, V.M., \enc{Díaz-Viera}{Diaz-Viera}, M., and Erdely, A., 2012, A joint stochastic simulation method using the Bernstein copula as a flexible tool for modeling nonlinear dependence structures between petrophysical properties: Journal of Petroleum Science and Engineering, v. 90--91, pp. 112--123, \doi{10.1016/j.petrol.2012.04.018}.

\enc{Hernández-Maldonado}{Hernandez-Maldonado}, V.M., Erdely, A., \enc{Díaz-Viera}{Diaz-Viera}, M., and Rios, L., 2024, Fast procedure to compute empirical and Bernstein copulas: Applied Mathematics and Computation, v. 477, article 128827, 14 p., \doi{10.1016/j.amc.2024.128827}.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in Nature---An approach using copulas: Springer, 289 p.

Segers, J., Sibuya, M., and Tsukahara, H., 2017, The empirical beta copula: Journal of Multivariate Analysis, v. 155, pp. 35--51, \doi{10.1016/j.jmva.2016.11.010}.
}
\author{W.H. Asquith}
\seealso{\code{\link{diagCOP}}, \code{\link{level.curvesCOP}}, \code{\link{simCOP}}}
\examples{
\dontrun{
set.seed(62)
EMPIRcop(0.321,0.78, para=simCOP(n=90, cop=N4212cop,
                                 para=2.32, graphics=FALSE)) # [1] 0.3222222
N4212cop(0.321,0.78, para=2.32)                              # [1] 0.3201281}

\dontrun{
set.seed(62) # See note below about another seed to try.
psp <- simCOP(n=34, cop=PSP, ploton=FALSE, points=FALSE) * 150
# Pretend psp is real data, the * 150 is to clearly get into an arbitrary unit system.

# The sort=FALSE is critical in the following two calls. Although the Weibull
# plotting positions are chosen, internally EMPIRcop uses ranks, but the model
# here is to imagine one having a sample in native units of the random variables
# and then casting them into probabilities for other purposes.
fakeU <- lmomco::pp(psp[,1], sort=FALSE) # Weibull plotting position i/(n+1)
fakeV <- lmomco::pp(psp[,2], sort=FALSE) # Weibull plotting position i/(n+1)
uv <- data.frame(U=fakeU, V=fakeV); # our U-statistics

# The next four values should be very close if n above were say 1000, but the
# ctype="bernstein"" should not be used if n >> 34 because of inherently long runtime.
PSP(0.4,0.6)              # 0.3157895 (compare to listed values below)

# Two seeds are shown so that the user can see that depending on the distribution
# of the values given by para that different coincidences of which method is
# equivalent to another exist.
# For set.seed(62) --- "hazen" == "weibull" by coincidence
#    "hazen"     --> 0.3529412
#    "weibull"   --> 0.3529412
#    "1/n"       --> 0.3235294
#    "bernstein" --> 0.3228916
# For set.seed(85) --- "1/n" == "hazen" by coincidence
#    "hazen"     --> 0.3529412
#    "weibull"   --> 0.3823529
#    "1/n"       --> 0.3529412
#    "bernstein" --> 0.3440387

# For set.seed(62) --- not all measures of association can be used for all
# empirical copulas because of 'divergent' integral errors, but this is an example
# for Hoeffding Phi. These computations are CPU intensive, esp. Bernstein.
hoefCOP(as.sample=TRUE, para=uv) #  (sample estimator is fast)  # 0.4987755
hoefCOP(cop=EMPIRcop,   para=uv, ctype="hazen")                 # 0.5035348
hoefCOP(cop=EMPIRcop,   para=uv, ctype="weibull")               # 0.4977145
hoefCOP(cop=EMPIRcop,   para=uv, ctype="1/n")                   # 0.4003646
hoefCOP(cop=EMPIRcop,   para=uv, ctype="bernstein")             # 0.4563724
hoefCOP(cop=EMPIRcop,   para=uv, ctype="checkerboard")          # 0.4952427}

# All other example suites shown below are dependent on the pseudo-data in the
# variable uv. It is suggested to not run with a sample size much larger than the
# above n=34 if the Bernstein comparison is intended (wanted) simply because of
# lengthy(!) run times, but the n=34 does provide a solid demonstration how the
# level curves for Bernstein weights are quite smooth.

\dontrun{
# Now, let us construct as many as three sets of level curves to the sample
# resided in the uv sample from above using the PSP copula.
level.curvesCOP(cop=PSP); # TRUE, parametric, fast, BLACK CURVES

# Empirical copulas can consume lots of CPU.
# RED CURVES, if n is too small, uniroot() errors might be triggered and likely
# will be using the sample size of 34 shown above.
level.curvesCOP(cop=EMPIRcop, para=uv, delu=0.03, col=2, ploton=FALSE)

# GREEN CURVES (large CPU committment)
# Bernstein progress is uninformative because level.curvesCOP() has taken over control.
bpara <- list(para=uv, ctype="bernstein", bernprogress=FALSE)
level.curvesCOP(cop=EMPIRcop, para=bpara, delu=0.03, col=3, ploton=FALSE)
# The delu is increased for faster execution but more important,
# notice the greater smoothness of the Bernstein refinement.}

\dontrun{
# Experimental from R Graphics by Murrell (2005, p.112)
"trans3d" <-                         # blackslashes seem needed for the package
function(x,y,z, pmat) {              # for user manual building but bad syntax
  tmat <- cbind(x,y,z,1) \%*\% pmat    # because remember the percent sign is a
  return(tmat[,1:2] / tmat[,4])      # a comment character in LaTeX.
}

the.grid <- EMPIRgrid(para=uv, ctype="checkerboard")
the.diag <- diagCOP(cop=EMPIRcop, para=uv, ploton=FALSE, lines=FALSE)

the.persp <- persp(the.grid$empcop, theta=-25, phi=20,
                   xlab="U VARIABLE", ylab="V VARIABLE", zlab="COPULA C(u,v)")
the.trace <- trans3d(the.diag$t, the.diag$t, the.diag$diagcop, the.persp)
lines(the.trace, lwd=2, col=2) # The diagonal of the copula

# The following could have been used as an alternative to call persp()
the.persp <- persp(x=the.grid$u, y=the.grid$v, z=the.grid$empcop, theta=-25, phi=20,
                   xlab="U VARIABLE", ylab="V VARIABLE", zlab="COPULA C(u,v)")
lines(the.trace, lwd=2, col=2) # The diagonal of the copula #}
}
\keyword{empirical copula}
\keyword{copula (empirical)}
\keyword{copula}
\concept{Bernstein copula}
\concept{Bernstein empirical copula}
\concept{Hazen copula}
\concept{Hazen empirical copula}
\concept{Weibull copula}
\concept{Weibull empirical copula}
\keyword{package copula (comparison to)}
\keyword{Bernstein polynomial}

\encoding{utf8}
\name{derCOPinv}
\alias{derCOPinv}
\title{Numerical Derivative Inverse  of a Copula for V with respect to U}
\description{
Compute the inverse of a numerical partial derivative for \eqn{V} with respect to \eqn{U} of a copula, which is a \emph{conditional quantile function} for nonexceedance probability \eqn{t}, or
\deqn{t = c_u(v) = \mathbf{C}^{(-1)}_{2 \mid 1}(v \mid u) = \frac{\delta \mathbf{C}(u,v)}{\delta u}\mbox{,}}
and solving for \eqn{v}. Nelsen (2006, pp. 13, 40--41) shows that this inverse is quite important for random variable generation using the \emph{conditional distribution method}. This function is not vectorized and will not be so.
}
\usage{
derCOPinv(cop=NULL, u, t, trace=FALSE,
          delu=.Machine$double.eps^0.50, para=NULL, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{u}{A single nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{t}{A single nonexceedance probability level \eqn{t};}
  \item{trace}{A logical controlling a \code{message} on whether the signs on the \code{uniroot} are the same---this is helpful in exploring the numerical derivative limits of a given implementation of a copula.}
  \item{delu}{The \eqn{\Delta u} interval for the derivative;}
  \item{para}{Vector of parameters or other data structures, if needed, to pass to \code{cop}; and}
  \item{...}{Additional arguments to pass to the copula.}
}
\value{
  Value(s) for the derivative inverse are returned.
}
\note{
\emph{AN EDUCATIONAL OPPORTUNITY}---The Farlie-Gumbel-Morgenstern copula \eqn{\mathbf{FGM}(u,v)} \cr (\code{\link{FGMcop}}) (Joe, 2014, p. 213) is
\deqn{\mathbf{FGM}(u,v; \Theta) = uv[1+\Theta(1-u)(1-v)]\mbox{,}}
where \eqn{-1 \le \Theta \le 1} has analytical solutions to the conditional cumulative distribution function (CDF) \eqn{\mathbf{C}_{2 \mid 1}(v \mid u)} as
\deqn{\mathbf{C}_{2 \mid 1}(v \mid u) = v[1 + \Theta(1-v)(1-2u)]\mbox{,}}
and the inverse of the conditional CDF as
\deqn{\mathbf{C}_{2 \mid 1}(v \mid u) = \frac{[1 + \Theta(1-2u)] - \sqrt{[1+\Theta(1-2u)]^2 - 4t(1-2u)}}{2\Theta(1-2u)}\mbox{.}}
These three functions for the copula can be defined in \R by
\preformatted{
  "FGMcop"       <- function(u,v, para=NULL, ...) u*v*(1 + para*(1-u)*(1-v)  )
  "joeFGMder"    <- function(u,v, para=NULL, ...)   v*(1 + para*(1-v)*(1-2*u))
  "joeFGMderinv" <- function(u,t, para=NULL, ...) {
      K <- (1-2*u)
      ((1 + para*K) - sqrt((1 + para*K)^2 - 4*t*K))/(2*para*K)
  }
}
The \eqn{\mathbf{C}^{(-1)}_{2 \mid 1}(v \mid u)} is critical for simulation by the conditional simulation method. Although exclusively for simulation, \pkg{copBasic} uses inversion of the numerical derivative, the \eqn{\mathbf{FGM}} copula has three representations of supposedly the same analytical algorithm for simulation in the literature (Durante, 2007; Johnson, 1987; Nelsen, 2006). An opportunity for comparison is thus available.

The three analytical algorithms for nonexceedance probability \eqn{t} given \eqn{u} by mathematics and code, following Durante (2007, p. 245), are
\deqn{A = \Theta(1-2u) - 1\mbox{,}}
\deqn{B = \sqrt{A^2 - 4t(A+1)}\mbox{, and}}
\deqn{v = 2t/(B-A)\mbox{,}}
and in \R, this \dQuote{Durante algorithm} is
\preformatted{
  "durFGMderinv" <- function(u,t, para=NULL, ...) { # Durante (2007, p. 245)
      A <- para*(1-2*u) - 1; B <- sqrt(A^2 - 4*t*(A+1)); return(2*t/(B - A))
  }
}
and, letting \eqn{K = (2u - 1)}, following Johnson (1987, p. 185)
\deqn{A = K\Theta - 1}
\deqn{B = \sqrt{1 - 2K\Theta + (K\Theta)^2 + 4tK\Theta}}
\deqn{v = 2t/(B - A)}
and in \R, this \dQuote{Johnson algorithm} is
\preformatted{
  "jonFGMderinv" <- function(u,t, para=NULL, ...) { # Johnson (1987, p. 185)
      K <- (2*u - 1)
      A <- K*para - 1; B <- sqrt(1 - 2*K*para + (K*para)^2 + 4*t*K*para)
      2*t/(B - A)
  }
}
and finally following Nelsen (2006, p. 87)
\deqn{A = 1 + \theta(1 - 2u)\mbox{,}}
\deqn{B = \sqrt{A^2 - 4t(A-1)}\mbox{, and}}
\deqn{v = 2t/(B+A)\mbox{,}}
and in \R, this \dQuote{Nelsen algorithm} is
\preformatted{
  "nelFGMderinv" <- function(u,t, para=NULL, ...) { # Nelsen (2006, p. 87)
      A <- 1 + para*(1-2*u); B <- sqrt(A^2 - 4*t*(A-1)); return(2*t/(B + A))
  }
}
With appropriate code now available, two comparisons can be made in the following sections.


\emph{CONDITIONAL DISTRIBUTION FUNCTION}---A comparison of the analytical \eqn{\mathbf{FGM}(u,v)} derivative shows that Joe's equation is congruent with the numerical derivative of \pkg{copBasic}:
\preformatted{
  joeFGMder(0.8, 0.44, para=0.78)             # 0.3246848      (Joe, 2014)
  derCOP(   0.8, 0.44, para=0.78, cop=FGMcop) # 0.3246848      (copBasic )
}
and the result will be used in the computations that follow.

A comparison for \eqn{t = 0.3246848} of the analytical inverse and the numerical optimization of the numerical derivative of \pkg{copBasic} is
\preformatted{
  joeFGMderinv(0.8, 0.3246848, para=0.78)             # 0.5327603
  derCOPinv(   0.8, 0.3246848, para=0.78, cop=FGMcop) # 0.4399934 --> 0.44
}
where obviously, the two results are not in agreement---so something is amiss. Because many examples in this documentation clearly demonstrate numerical reliability, a tentative conclusion is that Joe's listed equation must be in error. Let us check this hypothesis against the three other sources:
\preformatted{
  durFGMderinv(0.8, 0.3246848, para=0.78) # 0.2074546          (Durante, 2007)
  jonFGMderinv(0.8, 0.3246848, para=0.78) # 0.44               (Johnson, 1987)
  nelFGMderinv(0.8, 0.3246848, para=0.78) # 0.44               (Nelsen,  2006)
}
The result from Durante (2007) is different from both Joe (2014) and from \pkg{copBasic}. However, the Johnson (1987) and Nelsen (2006) versions are equivalent and congruent to \pkg{copBasic} with the \emph{distinctly different} numerical methods of \code{derCOPinv}. These incongruent results demonstrate that care is needed when navigating the copula literature and the usefulness of the \pkg{copBasic}-style implementation of copula theory. In words, these computations show that the \eqn{t \approx 32}nd percentile of the \eqn{\mathbf{FGM}} copula given that the 80th percentile in \eqn{U} is about the 44th percentile of \eqn{V}.
}
\references{
Durante, F., 2007, Families of copulas, Appendix C, \emph{in} Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in Nature---An approach using copulas: Springer, 289 p.

Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

Johnson, M.E., 1987, Multivariate statistical simulation: New York, John Wiley, 230 p.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Zhang, L., and Singh, V.P., 2019, Copulas and their applications in water resources engineering: Cambridge University Press, ISBN 978--1--108--47425--2.
}
\author{W.H. Asquith}
\seealso{\code{\link{derCOP}}}
\examples{
u <- runif(1); t <- runif(1)
derCOPinv(u,t, cop=W)   # perfect negative dependence
derCOPinv(u,t, cop=P)   # independence
derCOPinv(u,t, cop=M)   # perfect positive dependence
derCOPinv(u,t, cop=PSP) # a parameterless copula example
\dontrun{
# Simulate 500 values from product (independent) copula
plot(NA,NA, type="n", xlim=c(0,1), ylim=c(0,1), xlab="U", ylab="V")
for(i in 1:500) {
   u <- runif(1); t <- runif(1)
   points(u, derCOPinv(cop=P, u, t), cex=0.5, pch=16) # black dots
}
# Now simulate 500 from the Nelsen 4.2.12 copula.
for(i in 1:500) {
   u <- runif(1); t <- runif(1)
   points(u,derCOPinv(cop=N4212cop,para=9.3,u,t), cex=2, pch=16, col=2) # red dots
} #}

\dontrun{
# Zhang and Singh (2019) exam. 3.23, p. 105
# show the application of the derivative inversion C2|1
# for u=0.6036 and t=0.6036 ---> v = 0.4719
derCOPinv( cop=CLcop, 0.6036, 0.4028, para=0.5) # 0.4719 for C2|1
derCOPinv2(cop=CLcop, 0.6036, 0.4028, para=0.5) # 0.4719 for C1|2
# and C2|1 and C1|2 are equal because the copula has permutation symmetry
isCOP.permsym(cop=CLcop, para=0.5) # TRUE}
}
\keyword{copula (utility)}
\keyword{copula (derivative inverse)}
\keyword{conditional quantile function}
\keyword{copula operator}
\keyword{derivative}
\keyword{copula (conditional quantile function)}
\keyword{Joe (2014) Examples and Exercises}
\keyword{Nelsen (2006) Examples and Exercises}
\keyword{Salvadori et al. (2007) Examples and Exercises}
\keyword{Zhang and Singh (2019) Examples and Exercises}

\encoding{utf8}
\name{kullCOP}
\alias{kullCOP}
\alias{kullCOPint}
\title{Kullback--Leibler Divergence, Jeffrey Divergence, and Kullback--Leibler Sample Size}
\description{
Compute the \emph{Kullback--Leibler Divergence}, \emph{Jeffrey Divergence}, and \emph{Kullback--Leibler sample size} following Joe (2014, pp. 234--237). Consider two densities \eqn{f = c_1(u,v; \Theta_f)} and \eqn{g = c_2(u,v; \Theta_g)} for two different bivariate copulas \eqn{\mathbf{C}_1(\Theta_1)} and \eqn{\mathbf{C}_2(\Theta_2)} having respective parameters \eqn{\Theta}, then the Kullback--Leibler Divergence of \eqn{f} relative to \eqn{g} is
\deqn{\mathrm{KL}(f {\mid} g) = \int\!\!\int_{\mathcal{I}^2} g\, \log(g/f)\,\mathrm{d}u\mathrm{d}v\mbox{,}}
and Kullback--Leibler Divergence of \eqn{g} relative to \eqn{f} is
\deqn{\mathrm{KL}(g {\mid} f) = \int\!\!\int_{\mathcal{I}^2} f\, \log(f/g)\,\mathrm{d}u\mathrm{d}v\mbox{,}}
where the limits of integration \eqn{\mathcal{I}^2} theoretically are closed on \eqn{[0,1]^2} but an open interval \eqn{(0,1)^2} might be needed for numerical integration. Note, in general \eqn{\mathrm{KL}(f {\mid} g) \ne \mathrm{KL}(g {\mid} f)}. The \eqn{\mathrm{KL}(f {\mid} g)} is the expected log-likelihood ratios of \eqn{g} to \eqn{f} when \eqn{g} is the true density (Joe, 2014, p. 234), whereas \eqn{\mathrm{KL}(g {\mid} f)} is the opposite.

This asymmetry leads to Jeffrey Divergence, which is defined as a symmetrized version of the two Kullback--Leibler Divergences, and is
\deqn{J(f,g) = \mathrm{KL}(f {\mid} g) + \mathrm{KL}(g {\mid} f) = \int\!\!\int_{\mathcal{I}^2} (g-f)\, \log(g/f)\,\mathrm{d}u\mathrm{d}v\mbox{.}}

The variances of the Kullback--Leibler Divergences are defined as
\deqn{\sigma^2_{\mathrm{KL}(f {\mid} g)} = \int\!\!\int_{\mathcal{I}^2} g\,[\log(g/f)]^2\,\mathrm{d}u\mathrm{d}v - [\mathrm{KL}(f|g)]^2\mbox{,}}
and
\deqn{\sigma^2_{\mathrm{KL}(g {\mid} f)} = \int\!\!\int_{\mathcal{I}^2} f\,[\log(f/g)]^2\,\mathrm{d}u\mathrm{d}v - [\mathrm{KL}(g|f)]^2\mbox{.}}

For comparison of copula families \eqn{f} and \eqn{g} and taking an \eqn{\alpha = 0.05}, the Kullback--Leibler sample size is defined as
\deqn{n_{f\!g} = \bigl[\Phi^{(-1)}(1-\alpha) \times \eta_\mathrm{KL}\bigr]^2\mbox{,}}
where \eqn{\Phi^{(-1)}(t)} is the quantile function for the standard normal distribution \eqn{\sim} N(0,1) for nonexceedance probability \eqn{t}, and \eqn{\eta_\mathrm{KL}} is the maximum of
\deqn{\eta_\mathrm{KL} = \mathrm{max}\bigl[\sigma_{\mathrm{KL}(f {\mid} g)}/\mathrm{KL}(f {\mid} g),\, \sigma_{\mathrm{KL}(g {\mid} f)}/\mathrm{KL}(g {\mid} f)\bigr]\mbox{.}}
The \eqn{n_{f\!g}} gives an indication of the sample size needed to distinguish \eqn{f} and \eqn{g} with a probability of at least \eqn{1 - \alpha = 1 - 0.05 = 0.95} or 95 percent.

The \pkg{copBasic} features a \enc{naïve}{naive} \emph{Monte Carlo integration} scheme in the primary interface \code{kullCOP}, although the function \code{kullCOPint} provides for nested numerical integration. This later function is generally fast but suffers too much for general application from integral divergencies issued from the \code{integrate()} function in \R---this must be judged in the light that the \pkg{copBasic} package focuses only on implementation of the function of the copula itself and numerical estimation of copula density (\code{\link{densityCOP}}) and not analytical copula densities or hybrid representations thereof. Sufficient \dQuote{bread crumbs} are left among the code and documentation for users to re-implement if speed is paramount. Numerical comparison to the results of Joe (2014) (see \bold{Examples}) suggests that the default Monte Carlo sample size should be more than sufficient for general inference with the expense of considerable CPU time; however, a couple of repeated calls of \code{kullCOP} would be advised and compute say the mean of the resulting sample sizes.
}
\usage{
kullCOP(cop1=NULL, cop2=NULL, para1=NULL, para2=NULL, alpha=0.05,
           del=0, n=1E5, verbose=TRUE, sobol=FALSE, scrambling=0, ...)

kullCOPint(cop1=NULL, cop2=NULL, para1=NULL, para2=NULL, alpha=0.05,
           del=.Machine$double.eps^0.25, verbose=TRUE, ...)
}
\arguments{
  \item{cop1}{A copula function corresponding to copula \eqn{f} in Joe (2014);}
  \item{para1}{Vector of parameters or other data structure, if needed, to pass to the copula \eqn{f};}
  \item{cop2}{A copula function corresponding to copula \eqn{g} in Joe (2014);}
  \item{para2}{Vector of parameters or other data structure, if needed, to pass to the copula \eqn{g};}
  \item{alpha}{The \eqn{\alpha} in the Kullback--Leibler sample size equation;}
  \item{del}{A small value used to denote the \code{lo} and \code{hi} values of the numerical integration: \code{lo = del} and \code{hi = 1 - del}. If \code{del == 0}, then \code{lo = 0} and \code{hi = 1}, which corresponds to the theoretical limits \eqn{\mathcal{I}^2 = [0,1]^2} and are defaulted here to \eqn{[0,1]^2} because the Monte Carlo algorithm is preferred for general application. The end point control, however, is maintained just in case pathological situations should arise;}
  \item{n}{\code{kullCOP} (Monte Carlo integration) only---the Monte Carlo integration simulation size;}
  \item{verbose}{A logical trigging a couple of status lines of output through the \code{message()} function in \R;}
  \item{sobol}{A logical trigging \emph{Sobol sequences} for the Monte Carlo integration instead of the bivariate uniform distribution. The Sobol sequences are dependent on the \pkg{randtoolbox} package and the \code{sobol()} function of the \pkg{randtoolbox} package, and the Sobol sequences canvas the \eqn{\mathcal{I}^2} domain for smaller \eqn{n} values than required if statistical independence is used for the Monte Carlo integration. Note, the \pkg{randtoolbox} at least at version 2.0.+ has \dQuote{scrambling} of Sobol sequences temporarily disabled, and hence \code{scrambling=0} as default for \code{kullCOP};}
  \item{scrambling}{The argument of the same name for \code{randtoolbox::sobol}; and}
  \item{...}{Additional arguments to pass to the \code{\link{densityCOP}} function.}
}
\value{
  An \R \code{list} is returned having the following components:
  \item{MonteCarlo.sim.size}{\code{kullCOP} (Monte Carlo integration) only---The simulation size for numerical integration;}
  \item{divergences}{A vector of the Kullback--Leibler Divergences and their standard deviations: \eqn{\mathrm{KL}(f {\mid} g)}, \eqn{\sigma_{\mathrm{KL}(f {\mid} g)}}, \eqn{\mathrm{KL}(g {\mid} f)}, and \eqn{\sigma_{\mathrm{KL}(g {\mid} f)}}, respectively;}
  \item{stdev.divergences}{\code{kullCOP} (Monte Carlo integration) only---The standard deviation of the divergences and the variances;}
  \item{Jeffrey.divergence}{Jeffrey Divergence \eqn{J(f,g)};}
  \item{KL.sample.size}{Kullback--Leibler sample size \eqn{n_{f\!g}}; and}
  \item{integrations}{\code{kullCOPint} (numerical integration) only---An \R \code{list} of the outer call of the \code{integrate()} function for the respective numerical integrals shown in this documentation.}
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{densityCOP}}, \code{\link{vuongCOP}}}
\examples{
# See another demonstration under the Note section of statTn().
\dontrun{
# Joe (2014, p. 237, table 5.2)
# Gumbel-Hougaard and Plackett copulas below each have a Kendall Tau of about 0.5, and
# Joe (2014) lists in the table that Jeffrey Divergence is about 0.110 and Kullback-Leibler
# sample size is 133. Joe (2014) does not list the copula parameters just says Tau = 0.5.
# Joe (2014) likely did the numerical integrations using analytical solutions to probability
# densities and not rectangular approximations as in copBasic::densityCOP().
set.seed(1)
KL <- kullCOP(cop1=GHcop,       para1=2,
              cop2=PLACKETTcop, para2=11.40484, sobol=FALSE)
message("Jeffery Divergence is ",          round(KL$Jeffrey.divergence, digits=4),
        " and Kullback-Leibler sample size is ", KL$KL.sample.size, ".")
# Jeffery Divergence is 0.1106 and Kullback-Leibler sample size is 137.
set.seed(1)
KL <- kullCOP(cop1=GHcop,       para1=2,
              cop2=PLACKETTcop, para2=11.40484, sobol=TRUE )
message("Jeffery Divergence is ",          round(KL$Jeffrey.divergence, digits=4),
        " and Kullback-Leibler sample size is ", KL$KL.sample.size, ".")
# Jeffery Divergence is 0.3062 and Kullback-Leibler sample size is 136.


set.seed(1)
S <- replicate(20, kullCOP(cop1=GHcop, para1=2, cop2=PLACKETTcop, sobol=FALSE,
                           para2=11.40484, verbose=FALSE)$KL.sample.size)
print(as.integer(c(mean(S), sd(S)))) # 132 plus/minus 5
S <- replicate(2 , kullCOP(cop1=GHcop, para1=2, cop2=PLACKETTcop,  sobol=TRUE,
                           para2=11.40484, verbose=FALSE)$KL.sample.size)
# The two S in the later replication are both the same (136) for a sobol=TRUE
# does not produce variation and this is thought (June 2023) as a result
# of the disabled scrambling in the randtoolbox::sobol() function. }

\dontrun{
# Joe (2014, p. 237, table 5.3)
# Gumbel-Hougaard and Plackett copulas below each have a Spearman Rho of about 0.5, and
# Joe (2014) lists in the table that Jeffrey Divergence is about 0.063 and Kullback-Leibler
# sample size is 210. Joe (2014) does not list the parameters and just says that Rho = 0.5.
# Joe (2014) likely did the numerical integrations using analytical solutions to probability
# densities and not rectangular approximations as in copBasic::densityCOP().
set.seed(1)
KL <- kullCOP(cop1=GHcop,       para1=1.541071,
              cop2=PLACKETTcop, para2=5.115658, sobol=FALSE)
message("Jeffery Divergence is ",          round(KL$Jeffrey.divergence, digits=4),
        " and Kullback-Leibler sample size is ", KL$KL.sample.size, ".")
# Jeffery Divergence is 0.0642 and Kullback-Leibler sample size is 213.
set.seed(1)
KL <- kullCOP(cop1=GHcop,       para1=1.541071,
              cop2=PLACKETTcop, para2=5.115658, sobol=TRUE )
message("Jeffery Divergence is ",          round(KL$Jeffrey.divergence, digits=4),
        " and Kullback-Leibler sample size is ", KL$KL.sample.size, ".")
# Jeffery Divergence is 0.2001 and Kullback-Leibler sample size is 206.


set.seed(1)
S <- replicate(20, kullCOP(cop1=GHcop, para1=1.541071, cop2=PLACKETTcop,
                           para2=5.115658, verbose=FALSE)$KL.sample.size)
print(as.integer(c(mean(S), sd(S))))  # 220 plus/minus 19 }

\dontrun{
# Compare Jeffery Divergence estimates as functions of sample size when computed
# using Sobol sequences or not for Gumbel-Hougaard and Pareto copulas.
GHpar <- PApar <- 2 # Spearman Rho = 0.6822339
Ns <- as.integer(10^c(seq(2.0, 3.5, by=0.01), seq(3.6, 5, by=0.05)))
JDuni <- sapply(1:length(Ns), function(i) {
                  kullCOP(cop1=GHcop, para1=GHpar, verbose=FALSE,
                          cop2=PAcop, para2=PApar, n=Ns[i],
                          sobol=FALSE)$Jeffrey.divergence })
JDsob <- sapply(1:length(Ns), function(i) {
                  kullCOP(cop1=GHcop, para1=GHpar, verbose=FALSE,
                          cop2=PAcop, para2=PApar, n=Ns[i],
                          sobol=TRUE )$Jeffrey.divergence })
plot(Ns, JDuni, type="l", log="x", # black line, notice likely outliers too
     xlab="Simulation Sample Size", ylab="Jeffery Divergence")
lines(Ns, JDsob, col="red") # red line
legend("topright", c("Monte Carlo", "Sobol sequence"),
                   lwd=c(1,1), col=c("black", "red"), bty="n")
print( c( mean(JDuni), sd(JDuni) ) ) # [1] 0.05915608 0.01284682
print( c( mean(JDsob), sd(JDsob) ) ) # [1] 0.07274190 0.01838939

# The developer notes that plotting KL.sample.size for sobol=TRUE shows
# what appears to be numerical blow up but the Jeffery Divergence does not.
KLuni <- sapply(1:length(Ns), function(i) {
                  kullCOP(cop1=GHcop, para1=GHpar, verbose=FALSE,
                          cop2=PAcop, para2=PApar, n=Ns[i],
                          sobol=FALSE)$KL.sample.size })
KLsob <- sapply(1:length(Ns), function(i) {
                  kullCOP(cop1=GHcop, para1=GHpar, verbose=FALSE,
                          cop2=PAcop, para2=PApar, n=Ns[i],
                          sobol=TRUE )$KL.sample.size })
plot(Ns, KLuni, type="l", log="xy", # black line, notice likely outliers too
     xlab="Simulation Sample Size", ylab="Kullback-Leibler Sample Size")
lines(Ns, KLsob, col="red") # red line
nideal <- kullCOPint(cop1=GHcop, para1=GHpar, cop2=PAcop, para2=PApar)$KL.sample.size
abline(h=nideal, col="green", lwd=3) # nideal sample size is about 210
legend("topright", c("Monte Carlo", "Sobol sequence", "Judged ideal sample size"),
                   lwd=c(1,1,3), col=c("black", "red", "green"), bty="n")

# Let us attempt a visualization to highlight the differences in the two copula by
# simulation. First, using this n = nideal, being the apparent sample size to distinguish
# generally between the two copula having the same Spearman Rho. Do the segments help
# to visually highlight the differences? Next, ask would one judge the parents in the
# simulation being different knowing same Spearman Rho? (Note, the segments are all
# vertical because the U axis is the simulation and the V axis is the conditional
# probability given the U.)
set.seed(1); UVgh <- simCOP(nideal, GHcop, para=GHpar, graphics=FALSE)
set.seed(1); UVpa <- simCOP(nideal, PAcop, para=PApar, graphics=FALSE)
plot(c(0,1), c(0,1), type="n", xlab="U, nonexceedance probability",
                               ylab="V, nonexceedance probability")
segments(UVgh[,1], UVgh[,2], x1=UVpa[,1], y1=UVpa[,2])
points(UVgh, col="lightgreen", pch=16, cex=0.8) # dots
points(UVpa, col="darkgreen",  pch= 1, lwd=0.8) # circles
# Repeat the above n = nideal visualizations but with a change to n = nideal*10, and see
# then that there are visually striking shifts systematically in both both tails but also
# in the U in the interval (0.3, 0.7) belt but to a smaller degree than seen in the tails. }
}
\keyword{inference}
\keyword{copula (inference)}
\keyword{copula (density)}
\keyword{Joe (2014) Examples and Exercises}
\keyword{Sobol sequence (Monte Carlo integration)}
\keyword{Package randtoolbox}
\concept{Kullback--Leibler Divergence}
\concept{Kullback--Leibler sample size}
\concept{Kullback-Leibler Divergence}
\concept{Kullback-Leibler sample size}
\concept{Jeffery Divergence}
\concept{Jeffery's Divergence}
\concept{copula divergence}
\concept{relative entropy}
\concept{I-divergence}
\keyword{package randtoolbox (application of)}

\encoding{utf8}
\name{level.curvesCOP}
\alias{level.curvesCOP}
\title{Compute and Plot Level Curves of a Copula V with respect to U}
\description{
Compute and plot \emph{level curves} or \emph{level sets} of a copula for \eqn{V} with respect to \eqn{U} (Nelsen, 2006, pp. 12--13). The level curves at levels \eqn{t \mapsto [0+\Delta t, 1-\Delta t, \Delta t]} are defined for \eqn{U \mapsto [0+\Delta u, 1-\Delta u, \Delta u]} by
\deqn{t \mapsto \mathbf{C}(u=U, v)\mbox{,}}
and solving for \eqn{v}. Plotting is provided by this function because level curves are such an important visual attribute of a copula and highly useful for pedagogic purposes. The above equation is implemented by the \emph{inverse of a copula} using \code{\link{COPinv}}.
}
\usage{
level.curvesCOP(cop=NULL, para=NULL, ploton=TRUE, lines=TRUE,
                plotMW=FALSE, ramp=TRUE, delu=0.001, delt=0.10,
                getlevel=NULL, silent=TRUE, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{ploton}{A logical to toggle on the plot;}
  \item{lines}{A logical to toggle calls to the \code{lines()} function in \R to draw the lines;}
  \item{plotMW}{A logical to toggle to use the \code{abline()} function in \R to plot cross lines for the \eqn{\mathbf{M}} (\code{\link{M}}) and  \eqn{\mathbf{W}} (\code{\link{W}}) copulas;}
  \item{ramp}{A logical to toggle whether the level curves are ramped in thickness according to the probability of the line. The argument can be used with a \code{lwd} setting through the \code{...} and can either be a constant, which will override the built-in ramp, or \code{lwd} can be defined as a function to replace the built-in ramp in terms of the probability;}
  \item{delu}{The increment for \eqn{\Delta u}. The default is 1 part in 1,000, which should often provide enough smoothness for many copulas in practice;}
  \item{delt}{The increment \eqn{\Delta t} for the level curves to plot, defaults to 10-percent intervals. If \code{delt=0.5}, then only the median plus the consequences of a defined \code{getlevel} is used. If \code{NULL}, then a sequence of \eqn{t} values is not made and only \code{getlevel} is used (if available);}
  \item{getlevel}{If defined, then it is inserted into the sequence of levels \eqn{t} and that level \eqn{t} \eqn{=} \code{getlevel} is returned in an \R \code{list} data structure. If more than one level is desired, then instead of repeated calls to this function, the \code{\link{joint.curvesCOP}} function could be considered;}
  \item{silent}{The argument of the same name given over to \code{try()} wrapping the \code{try()} operation on forming sequences of \eqn{t} for the curves (see sources); and}
  \item{...}{Additional arguments to pass to the \code{lines()} function in \R.}
}
\value{
  Typically no values are returned because this function is used for its side effects, but the arguments can be such that the \eqn{\{u, v\}} for \eqn{\mathbf{C}(u,v) = t} are returned within an \R \code{list}.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{COPinv}}, \code{\link{level.curvesCOP2}}, \code{\link{level.setCOP}}, \code{\link{joint.curvesCOP}}
}
\examples{
\dontrun{
level.curvesCOP(cop=M, para=NULL, delt=0.02) # Upper bounds copula}
\dontrun{
D <- level.curvesCOP(cop=P,   getlevel=0.56)
str(D) # empty
D <- level.curvesCOP(cop=P,   getlevel=0.5)
str(D) # contains stuff
D <- level.curvesCOP(cop=PSP, getlevel=0.8)
str(D) # contains stuff}
}
\keyword{visualization}
\keyword{diagnostics}
\concept{level curves}
\concept{level curve}
\concept{level sets}
\concept{level set}
\concept{level contours}
\concept{level contour}

\encoding{utf8}
\name{PLACKETTsim}
\alias{PLACKETTsim}
\title{Direct Simulation of a Plackett Copula}
\description{
\emph{Plackett copula} simulation (Nelsen, 2006, pp. 89--92) (\code{\link{PLACKETTcop}}) is made by this function using analytical formula (Durante, 2007, p. 247; see source code). The \code{PLACKETTsim} function exists for comparison against the numerical derivative (\emph{conditional distribution method}) methods (\code{\link{simCOP}}, \code{\link{simCOPmicro}}) otherwise used in \pkg{copBasic}.
}
\usage{
PLACKETTsim(n, para=NULL, ...)
}
\arguments{
  \item{n}{Sample size;}
  \item{para}{The \eqn{\Theta} parameter of the Plackett copula; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  An \R \code{data.frame} of the values \eqn{U} and \eqn{V} for the nonexceedance probabilities is returned.
}
\references{
Durante, F., 2007, Families of copulas, Appendix C, \emph{in} Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in Nature---An approach using copulas: Springer, 289 p.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{PLACKETTcop}}, \code{\link{PLACKETTpar}}}
\examples{
PLACKETTsim(10, para= 1  ) # simulate P (independence) copula through a Plackett
PLACKETTsim(10, para=20.3) # simulate strong positive Plackett
}
\keyword{copula (simulation)}
\keyword{Plackett copula}

\encoding{utf8}
\name{LMRuvCOP}
\alias{LMRuvCOP}
\title{Conditional L-moments of the Distribution of U given V}
\description{
Compute the \emph{conditional L-moments} of \eqn{U} given a \eqn{V} (the \eqn{Y} direction) through the \emph{conditional distribution function} \eqn{G(Y)} using the appropriate \emph{partial derivative} of a copula (\eqn{\mathbf{C}(u,v)}) with respect to \eqn{V}. The inversion of the partial derivative is the \emph{conditional quantile function}.
}
\usage{
LMRuvCOP(v=seq(0.01, 0.99, by=0.01), cop=NULL, para=NULL, nsim=1E5,
    subdivisions=200L, rel.tol=.Machine$double.eps^0.25, abs.tol=rel.tol, ...)
}
\arguments{
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{cop}{A copula function with vectorization as in \code{asCOP};}
  \item{para}{Vector of parameters or other data structures, if needed, to pass to the copula;}
  \item{nsim}{Number of simulations for \emph{Monte Carlo integration} if numerical integration were to fail;}
  \item{subdivisions}{Argument of same name passed to \code{integrate()};}
  \item{rel.tol}{Argument of same name passed to \code{integrate()};}
  \item{abs.tol}{Argument of same name passed to \code{integrate()}; and}
  \item{...}{Additional arguments to pass to \code{\link{derCOP2}}.}
}
\value{
  Value(s) in a list for the median and conditional L-moments given \eqn{V=v} are returned including
  \item{median}{The conditional median from \code{\link{med.regressCOP2}};}
  \item{L1}{The conditional mean \eqn{\lambda_1|V=v} from \code{\link{EuvCOP}};}
  \item{L2}{The conditional L-scale \eqn{\lambda_2|V=v};}
  \item{T3}{The conditional L-skew \eqn{\tau_3|V=v};}
  \item{T4}{The conditional L-kurtosis \eqn{\tau_4|V=v};}
}
\author{W.H. Asquith}
\seealso{\code{\link{EuvCOP}}, \code{\link{LMRvuCOP}}, \code{\link{derCOP2}}}
\examples{
# Please refer to Examples under LMRvuCOP().
}
\keyword{copula (utility)}
\keyword{copula operator}
\keyword{copula (conditional distribution)}
\keyword{conditional expectation}
\keyword{L-moments (conditional) of U given V}
\keyword{Conditional L-moments of U given V}

\encoding{utf8}
\name{EMPIRgrid_fast}
\alias{EMPIRgrid_fast}
\title{Fast Grid of the Bivariate Empirical Copula}
\description{
Generate a fast gridded representation of the \emph{bivariate empirical copula} (\enc{Hernández-Maldonado}{Hernandez-Maldonado} \emph{et al.}, 2024). Mathematics for the \code{"1/n"} and \code{"bernstein"} copula are shown under \code{\link{EMPIRcop}}. Herein, the \eqn{\ddot{\mathbf{C}}_n} (double-dots) are used to denote gridding of the copula. This function is a much more recent addition (spring 2026) to the \pkg{copBasic} than its historical ancestor of \code{\link{EMPIRgrid}} because of publication of the algorithm by \enc{Hernández-Maldonado}{Hernandez-Maldonado} \emph{et al.} (2024). The \pkg{copBasic} author personally thanks V.M. \enc{Hernández-Maldonado}{Hernandez-Maldonado} for a May 2026 discussion on the algorithm and some clarifications to the pseudo-code in \enc{Hernández-Maldonado}{Hernandez-Maldonado} \emph{et al.} (2024).
}
\usage{
EMPIRgrid_fast(uv=NULL, ctype=c("1/n", "bernstein"), para=NULL,
               gridonly=FALSE, verbose=FALSE, ...)
}
\arguments{
  \item{uv}{A data frame or matrix of \eqn{u} and \eqn{v} vectors of nonexceedance probabilities (see also \code{para});}
  \item{ctype}{A means to trigger which definition of empirical copula is to be used: \eqn{\mathbf{C}_n(u,v)} (\code{"1/n"}) or \eqn{\mathbf{C}^\mathcal{B}_n(u,v)} (\code{"bernstein"});}
  \item{para}{A matrix of \eqn{u} and \eqn{v} vectors as an alternative to \code{uv} for syntax compatibility for other situations in \pkg{copBasic};}
  \item{verbose}{A logical controlling whether the progress during grid building is to be shown;}
  \item{gridonly}{A logical to return the \code{empcop} component (\eqn{\ddot{\mathbf{C}}_{n}(u,v)} or \eqn{\ddot{\mathbf{C}}^\mathcal{B}_n(u,v)}); and}
  \item{...}{Additional arguments to pass to \code{\link{EMPIRcop}}.}
}
\value{
A \code{matrix} is returned (\eqn{\ddot{\mathbf{C}}_{n}(u,v)} or \eqn{\ddot{\mathbf{C}}^\mathcal{B}_n(u,v)}), if \code{gridonly} is true. Otherwise a \code{list} of the \eqn{u}, \eqn{v}, and \eqn{\ddot{\mathbf{C}}_{n}(u,v)} or \eqn{\ddot{\mathbf{C}}^\mathcal{B}_n(u,v)} is returned. The \code{list} will also contain the \code{deluv} (simply \eqn{1/n} for sample size \eqn{n}) used and the \code{ctype}. The \code{list} is returned as \code{classes} of \code{"empirical.copula.grid"} and \code{"list"}. This is classification might be used in \pkg{copBasic} to check for a grid in other functions. Note, there is no \code{deluv} argument to this function like there is for the historical function \code{\link{EMPIRgrid}}.
}
\references{
\enc{Hernández-Maldonado}{Hernandez-Maldonado}, V.M., Erdely, A., \enc{Díaz-Viera}{Diaz-Viera}, M., and Rios, L., 2024, Fast procedure to compute empirical and Bernstein copulas: Applied Mathematics and Computation, v. 477, article 128827, 14 p., \doi{10.1016/j.amc.2024.128827}.
}
\author{W.H. Asquith}
\seealso{\code{\link{EMPIRgrid}}, \code{\link{EMPIRgrid_lkup}}}
\examples{
  para <- list(cop=GHcop, para=3, breve=0.1)
  uv <- simCOP(10, cop=breveCOP, para=para)
  gf <- EMPIRgrid_fast(para=uv, gridonly=TRUE)
  gg <- EMPIRgrid(para=uv, gridonly=TRUE, deluv=0.1, ctype="1/n")
  print(sum(gf - gg)) # should be zero

  n <- 5; uv <- matrix(runif(n *2), ncol=2)
  empCOP <- EMPIRgrid_fast(uv, ctype="1/n",       gridonly=TRUE)
            image(empCOP, main="1/n"      )
  empCOP <- EMPIRgrid_fast(uv, ctype="bernstein", gridonly=TRUE)
            image(empCOP, main="Bernstein")

\dontrun{
  # This permutation asymmetric copula is shown as a means to cross check
  # the orientation of the matrix computations within the function.
  n <- 500; para <- list(cop=GHcop, para=c(12,15), breve=0.5)
  uv <- copBasic::simCOP(n, cop=breveCOP, para=para, graphics=FALSE)
  empCOP <- EMPIRgrid_fast(uv, gridonly=TRUE)
  image(empCOP); points(uv) #}
}
\keyword{empirical copula}
\keyword{empirical copula (utility)}
\keyword{copulatic surface}
\concept{Bernstein copula}
\concept{Bernstein empirical copula}
\keyword{Bernstein polynomial}
\keyword{gridded empirical copula}

\encoding{utf8}
\name{surCOP}
\alias{surCOP}
\title{The Survival Copula}
\description{
Compute the \emph{survival copula} from a copula (Nelsen, 2006, pp. 32--34), which is defined as

\deqn{\hat{\mathbf{C}}(1-u,1-v) = \hat{\mathbf{C}}(u',v') = \mathrm{Pr}[U > u, V > v] = u' + v' - 1 + \mathbf{C}(1-u', 1-v')\mbox{,}}
where \eqn{u'} and \eqn{v'} are exceedance probabilities and \eqn{\mathbf{C}(u,v)} is the copula (\code{\link{COP}}). The \emph{survival copula} is a reflection of both \eqn{U} and \eqn{V}.

The \emph{survival copula} is an expression of the joint probability that both \eqn{U > v} and \eqn{U > v} when the arguments \eqn{a} and \eqn{b} to \eqn{\hat{\mathbf{C}}(a,b)} are exceedance probabilities as shown. This is unlike a copula that has \eqn{U \le u} and \eqn{V \le v} for nonexceedance probabilities \eqn{u} and \eqn{v}. Alternatively, the joint probability that both \eqn{U > u} and \eqn{V > v} can be solved using just the copula \eqn{1 - u - v + \mathbf{C}(u,v)}, as shown below where the arguments to \eqn{\mathbf{C}(u,v)} are nonexceedance probabilities. The later formula is the \emph{joint survival function} \eqn{\overline{\mathbf{C}}(u,v)} (\code{\link{surfuncCOP}}) defined for a copula (Nelsen, 2006, p. 33) as
\deqn{\overline{\mathbf{C}}(u,v) = \mathrm{Pr}[U > u, V > v] = 1 - u - v + \mathbf{C}(u,v)\mbox{.}}
Users are directed to the collective documentation in \code{\link{COP}} and \code{\link{simCOPmicro}} for more details on copula reflection.
}
\usage{
surCOP(u, v, cop=NULL, para=NULL, exceedance=TRUE, ...)
}
\arguments{
  \item{u}{Exceedance probability \eqn{u' = 1 - u} (\eqn{u} nonexceedance based on \code{exceedance}) in the \eqn{X} direction;}
  \item{v}{Exceedance probability \eqn{v' = 1 - v} (\eqn{v} nonexceedance based on \code{exceedance}) in the \eqn{Y} direction;}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{exceedance}{A logical affirming whether \code{u} and \code{v} are really in exceedance probability or not? If \code{FALSE}, then the complements of the two are made internally and the nonexceedances can thus be passed; and}
  \item{...}{Additional arguments to pass (such as parameters, if needed, for the copula in the form of an \R \code{list}).}
}
\value{
  Value(s) for the survival copula are returned.
}
\note{
The author (Asquith) finds the use of exceedance probabilities delicate in regards to Nelsen's notation. This function and \code{\link{coCOP}} have the \code{exceedance} argument to serve as a reminder that the survival copula as usually defined uses exceedance probabilities as its arguments.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{COP}}, \code{\link{coCOP}}, \code{\link{duCOP}}, \code{\link{surfuncCOP}}, \code{\link{simCOPmicro}}}
\examples{
u  <-  0.26; v  <- 0.55   # nonexceedance probabilities
up <- 1 - u; vp <- 1 - v  #    exceedance probabilities
surCOP(up, vp,   cop=PSP, exceedance=TRUE)  # 0.4043928
surCOP(u, v,     cop=PSP, exceedance=FALSE) # 0.4043928 (same because of symmetry)
surfuncCOP(u, v, cop=PSP)                   # 0.4043928
# All three examples show joint prob. that U > u and V > v.

\dontrun{
# A survival copula is a copula so it increases to the upper right with increasing
# exceedance probabilities. Let us show that by hacking the surCOP function into
# a copula for feeding back into the algorithmic framework of copBasic.
UsersCop <- function(u,v, para=NULL) {
     afunc <- function(u,v, theta=para) { surCOP(u, v, cop=N4212cop, para=theta)}
     return(asCOP(u,v, f=afunc)) }
image(gridCOP(cop=UsersCop, para=1.15), col=terrain.colors(20),
      xlab="U, EXCEEDANCE PROBABILITY", ylab="V, EXCEEDANCE PROBABILITY") #}
}
\keyword{copula operator}
\keyword{joint probability}
\keyword{copula (reflection)}

\encoding{utf8}
\name{hoefCOP}
\alias{hoefCOP}
\alias{LpCOP}
\alias{LpCOPradsym}
\alias{LpCOPpermsym}
\title{The Hoeffding Phi of a Copula or Lp Distances (Independence, Radial Asymmetry, or Reflection Symmetry Forms)}
\description{
Compute the measure of association known as the \emph{Hoeffding Phi} \eqn{\Phi_\mathbf{C}} of a copula from \emph{independence} (\eqn{uv = \mathbf{\Pi}}; \code{\link{P}}) according to Cherunbini \emph{et al.} (2004, p. 164) by
\deqn{\Phi_\mathbf{C} = 3 \sqrt{10\int\!\!\int_{\mathcal{I}^2} \bigl(\mathbf{C}(u,v) - uv\bigr)^2\,\mathrm{d}u\mathrm{d}v}\mbox{,}}

and Nelsen (2006, p. 210) shows this as (and absolute value notation by Nelsen helps in generalization)
\deqn{\Phi_\mathbf{C} = \biggl(90\int\!\!\int_{\mathcal{I}^2} |\mathbf{C}(u,v) - uv|^2\,\mathrm{d}u\mathrm{d}v\biggr)^{1/2}\mbox{,}}

for which \eqn{\Phi^2_\mathbf{C}} (the square of the quantity) is known as the \emph{dependence index}. \enc{Gaißer}{Gaisser} \emph{et al.} (2010, eq. 1) have \eqn{\Phi^2_\mathbf{C}} as the \emph{Hoeffding Phi-Square}, and their definition, when square-rooted, matches Nelsen's listing.

A generalization (Nelsen, 2006) to \eqn{L_p} distances from independence (\eqn{uv = \mathbf{\Pi}}; \code{\link{P}}) through the \code{LpCOP} function is
\deqn{L_p \equiv \Phi_\mathbf{C}(p) = \biggl(k(p)\int\!\!\int_{\mathcal{I}^2} |\mathbf{C}(u,v) - uv|^p\,\mathrm{d}u\mathrm{d}v\biggr)^{1/p}\mbox{,}}

for a \eqn{p: 1 \le p \le \infty} and where \eqn{k(p)} is a normalization constant such that \eqn{\Phi_\mathbf{C}(p) = 1} when the copula \eqn{\mathbf{C}} is \eqn{\mathbf{M}} (see \code{\link{M}}) or \eqn{\mathbf{W}} (see \code{\link{W}}). The \eqn{k(p)} (bivariate definition only) for other powers is given (Nelsen, 2006, exer. 5.44, p. 213) in terms of the \emph{complete gamma function} \eqn{\Gamma(t)} by
\deqn{k(p) = \frac{\Gamma(2p+3)}{2[\Gamma(p + 1)]^2}\mbox{,}}

which is implemented by the \code{hoefCOP} function. It is important to realize that the \eqn{L_p} distances are all \emph{symmetric nonparametric measures of dependence} (Nelsen, 2006, p. 210). These are symmetric because distance from independence is used as evident by \dQuote{\eqn{uv}} in the above definitions.

\emph{Reflection/Radial and Permutation Asymmetry}---Asymmetric forms similar to the above distances exist. Joe (2014, p. 65) shows two measures of bivariate \emph{reflection asymmetry} or \emph{radial asymmetry} (term favored in \pkg{copBasic}) as the distance between \eqn{\mathbf{C}(u,v)} and the survival copula \eqn{\hat{\mathbf{C}}(u,v)} (\code{\link{surCOP}}) measured by
\deqn{L_\infty^{\mathrm{radsym}} = \mathrm{sup}_{0\le u,v\le1}|\mathbf{C}(u,v) - \hat{\mathbf{C}}(u,v)|\mbox{,}}

or its \eqn{L_p^{\mathrm{radsym}}} counterpart
\deqn{L_p^{\mathrm{radsym}} = \biggl[\int\!\!\int_{\mathcal{I}^2} |\mathbf{C}(u,v) - \hat{\mathbf{C}}(u,v)|^p\,\mathrm{d}u\mathrm{d}v\biggr]^{1/p}\,\mathrm{with}\, p \ge 1\mbox{,}}

where \eqn{\hat{\mathbf{C}}(u,v) = u + v - 1 + \mathbf{C}(1-u, 1-v)} and again \eqn{p: 1 \le p \le \infty}. Joe (2014) does not appear to discuss and normalization constants for these two radial asymmetry distances.

Joe (2014, p. 66) offers analogous measures of bivariate \emph{permutation asymmetry} (\code{\link{isCOP.permsym}}) (\eqn{\mathbf{C}(u,v) \not= \mathbf{C}(v,u)}) defined as
\deqn{L_\infty^{\mathrm{permsym}} = \mathrm{sup}_{0\le u,v\le1}|\mathbf{C}(u,v) - \hat{\mathbf{C}}(v,u)|\mbox{,}}

or its \eqn{L_p^{\mathrm{permsym}}} counterpart
\deqn{L_p^{\mathrm{permsym}} = \biggl[\int\!\!\int_{\mathcal{I}^2} |\mathbf{C}(u,v) - \hat{\mathbf{C}}(v,u)|^p\,\mathrm{d}u\mathrm{d}v\biggr]^{1/p}\,\mathrm{with}\, p \ge 1\mbox{,}}

where \eqn{p: 1 \le p \le \infty}. Again, Joe (2014) does not appear to discuss and normalization constants for these two permutation symmetry distances. Joe (2014, p. 65) states that the \dQuote{simplest one-parameter bivariate copula families [and] most of the commonly used two-parameter bivariate copula families are permutation symmetric.} The \eqn{L_\infty^{\mathrm{permsym}}} (or rather a similar form) is implemented by \code{\link{LzCOPpermsym}} and demonstration made in that documentation.

The asymmetrical \eqn{L_\infty} and \eqn{L_p} measures identified by Joe (2014, p. 66) are nonnegative with an upper bounds that depends on \eqn{p}. The bound dependence on \eqn{p} is caused by the lack of normalization constant \eqn{k(p)}. In an earlier paragraph, Joe (2014) indicates an upper bounds of 1/3 for both (likely?) concerning \eqn{L_\infty^{\mathrm{radsym}}} and \eqn{L_\infty^{\mathrm{permsym}}}. Discussion of this 1/3 or rather the integer 3 is made within \code{\link{LzCOPpermsym}}.

The numerical integrations for \eqn{L_p^{\mathrm{radsym}}} and \eqn{L_p^{\mathrm{permsym}}} can readily return zeros. Often inspection of the formula for the \eqn{\mathbf{C}(u,v)} itself would be sufficient to judge whether symmetry exists and hence the distances are uniquely zero.

Joe (2014, p. 66) completes the asymmetry discussion with three definitions of skewness of combinations of random variables \eqn{U} and \eqn{V}: Two definitions are in \code{\link{uvlmoms}} (for \eqn{U + V - 1} and \eqn{U - V}) and two are for \eqn{V-U} (\code{\link{nuskewCOP}}) and \eqn{U+V-1} (\code{\link{nustarCOP}}).
}
\usage{
hoefCOP(     cop=NULL, para=NULL, p=2, as.sample=FALSE,
                                       sample.as.prob=TRUE,
                                       brute=FALSE, delta=0.002, ...)

LpCOP(       cop=NULL, para=NULL, p=2, brute=FALSE, delta=0.002, ...)
LpCOPradsym( cop=NULL, para=NULL, p=2, brute=FALSE, delta=0.002, ...)
LpCOPpermsym(cop=NULL, para=NULL, p=2, brute=FALSE, delta=0.002, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{p}{The value for \eqn{p} as described above with a default to 2 to match the discussion of Nelsen (2006) and the \emph{Hoeffding Phi} of Cherubini \emph{et al.} (2004). Do not confuse \eqn{p} with \eqn{d} described in \bold{Note};}
  \item{as.sample}{A logical controlling whether an optional \R \code{data.frame} in \code{para} is used to compute the \eqn{\hat{\Phi}_\mathbf{C}} (see \bold{Note}). If set to \code{-1}, then the message concerning CPU effort will be suppressed;}
  \item{sample.as.prob}{When \code{as.sample} triggered, what are the units incoming in \code{para}? If they are probabilities, the default is applicable. If they are not, then the columns are re-ranked and divided simply by \eqn{1/n}---more sophisticated \emph{empirical copula} probabilities are not used (\code{\link{EMPIRcop}});}
  \item{brute}{Should brute force be used instead of two nested \code{integrate()} functions in \R to perform the double integration;}
  \item{delta}{The \eqn{\mathrm{d}u} and \eqn{\mathrm{d}v} for the brute force (\code{brute=TRUE}) integration; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  The value for \eqn{\Phi_\mathbf{C}(p)} is returned.
}
\note{
Concerning the distance from independence, when \eqn{p = 1}, then the \emph{Spearman Rho} (\code{\link{rhoCOP}}) of a copula is computed where is it seen in that documentation that the \eqn{k_p(1) = 12}. The respective values of \eqn{k(p)} for select integers \eqn{p} are
\deqn{p \mapsto [1, 2, 3, 4, 5] \equiv k(p) \mapsto \{12, 90, 560, 3150, 16600\}\mbox{,}}
and these values are hardwired into \code{hoefCOP} and \code{LpCOP}. The integers for \eqn{k_p} ensures that the equality in the second line of the examples is \code{TRUE}, but the \eqn{p} can be a noninteger as well. Nelsen (2006, p. 211) reports that when \eqn{p = \infty} that \eqn{L_\infty} is
\deqn{L_\infty \equiv \Phi_\mathbf{C}(\infty) = \Lambda_\mathbf{C} = 4\;\mathrm{sup}_{u,v \in \mathcal{I}}|\mathbf{C}(u,v) - uv|\mbox{.}}

A sample \eqn{\hat{\Phi}_\mathbf{C}} (square root of the \emph{Hoeffding Phi-Square}) based on nonparametric estimation generalized for \eqn{d} dimensions (\eqn{d = 2} for bivariate) is presented by \enc{Gaißer}{Gaisser} \emph{et al.} (2010, eq. 10) for estimated probabilities \eqn{\hat{U}_{ij}} for the \eqn{i}th dimension and \eqn{j}th row (observation) for sample of size \eqn{n}. Those authors suggest that the \eqn{\hat{U}_{ij}} be estimated from the empirical copula:
\deqn{\hat\Phi_\mathbf{C} = \sqrt{h(d)[A + B]}\mbox{,}}
where
\deqn{A = \biggl(\frac{1}{n}\biggr)^2\sum_{j=1}^n\sum_{k=1}^n\prod_{i=1}^d
                    \bigl[1 - \mathrm{max}\bigl(\hat{U}_{ij}, \hat{U}_{ik}\bigr)\bigr]\mbox{,}}
\deqn{B = \biggl(\frac{1}{3}\biggr)^d - \biggl(\frac{2}{n}\biggr)\biggl(\frac{1}{2}\biggr)^d
               \sum_{j=1}^n\prod_{i=1}^d  \bigl[1 - \hat{U}_{ij}^2\bigr]\mbox{.}}
The normalization constant is a function of dimension and is
\deqn{h(d)^{-1} = \frac{2}{(d+1)(d+2)} - \biggl(\frac{1}{2}\biggr)^d\frac{d\,!}{\prod_{i=0}^d\bigl(i+(1/2)\bigr)}+\biggl(\frac{1}{3}\biggr)^d\mbox{.}}
\preformatted{
  set.seed(1); UV <- simCOP(n=1000, cop=PSP)
  hoefCOP(cop=PSP)                                       # 0.4547656 (theo.)
  hoefCOP(para=UV, as.sample=TRUE)                       # 0.4892757
  set.seed(1); UV <- simCOP(n=1000, cop=PSP, snv=TRUE) # std normal variates
  hoefCOP(para=UV, as.sample=TRUE, sample.as.prob=FALSE) # 0.4270324
}
}
\references{
Cherubini, U., Luciano, E., and Vecchiato, W., 2004, Copula methods in finance: Hoboken, NJ, Wiley, 293 p.

\enc{Gaißer}{Gaisser}, S., Ruppert, M., and Schmid, F., 2010, A multivariate version of Hoeffding's Phi-Square: Journal of Multivariate Analysis, v. 101, no. 10, pp. 2571--2586.

Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{blomCOP}}, \code{\link{footCOP}}, \code{\link{giniCOP}},
         \code{\link{rhoCOP}}, \code{\link{tauCOP}}, \code{\link{wolfCOP}},
         \code{\link{joeskewCOP}}, \code{\link{uvlmoms}},
         \code{\link{LzCOPpermsym}}
}
\examples{
\dontrun{
# Example (ii) Gaisser et al. (2010, p. 2574)
Theta <- 0.66 # Phi^2 = Theta^2 ---> Phi == Theta as shown
hoefCOP(cop=convex2COP, para=c(alpha=Theta, cop1=M, cop2=P)) # 0.6599886

rhoCOP(cop=PSP) == hoefCOP(cop=PSP, p=1) # TRUE
LpCOP(cop=PLACKETTcop, para=1.6, p=2.6)  # 0.1445137 (Fractional p)}

\dontrun{
set.seed(938) # Phi(1.6; Plackett) = 0.1184489; L_1 = 0.1168737
UV <- simCOP(cop=PLACKETTcop, para=1.6, n=2000, ploton=FALSE, points=FALSE)
hoefCOP(cop=PLACKETTcop, para=1.6, p=200)  # Large p near internal limits
L_1 <- 4*max(abs(PLACKETTcop(UV$U, UV$V, para=1.6) - UV$U*UV$V)) # p is infty
# and finite n and arguably a sample-like statistic here, now on intuition try
# a more sample-like means
U <- runif(10000); V <- runif(10000)
L_2 <- 4*max(abs(EMPIRcop(U, V, para=UV) - U*V)) # 0.1410254 (not close enough)}

\dontrun{
para <- list(alpha=0.15, beta=0.90, kappa=0.06, gamma=0.96,
             cop1=GHcop, cop2=PLACKETTcop, para1=5.5, para2=0.07)
LpCOPradsym( cop=composite2COP, para=para) # 0.02071164
LpCOPpermsym(cop=composite2COP, para=para) # 0.01540297}

\dontrun{
"MOcop.formula" <- function(u,v, para=para, ...) {
   alpha <- para[1]; beta <- para[2]; return(min(v*u^(1-alpha), u*v^(1-beta)))
}
"MOcop" <- function(u,v, ...) { asCOP(u,v, f=MOcop.formula, ...) }
   LpCOPradsym( cop=MOcop, para=c(0.8, 0.5)) # 0.0261843
   LpCOPpermsym(cop=MOcop, para=c(0.8, 0.5)) # 0.0243912 }
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{dependence index}
\keyword{Lp distance}
\keyword{Nelsen (2006) Examples and Exercises}
\concept{Hoeffding Phi}
\concept{Hoeffding Phi-Square}

\encoding{utf8}
\name{blomatrixCOP}
\alias{blomatrixCOP}
\alias{blomatrixCOPdec}
\alias{blomatrixCOPiqr}
\title{A Matrix of Blomqvist-like Betas of a Copula}
\description{
Compute the \emph{Blomqvist-like Betas matrix} \eqn{\beta^\circ_\mathbf{C}}-matrix of a copula, which is defined at presumably strategic points within \eqn{\mathcal{I}^2}, as (for \code{as.blomCOPss=FALSE} argument)

\deqn{\beta^\circ_\mathbf{C} = \frac{\mathbf{C}(u^\circ,v^\circ)}{\mathbf{\Pi}(1/2, 1/2)} - 1\mbox{,}}

where the \eqn{u^\circ} and \eqn{v^\circ} are of two types of gridded locations in \eqn{\mathcal{I}^2} space and if \eqn{u^\circ = 1/2} and \eqn{v^\circ = 1/2}, then central location of the matrix is \emph{Blomqvist Beta} (\code{\link{blomCOP}}). The definition of \eqn{\beta^\circ_\mathbf{C}} is such that the matrix is entirely zero for the \emph{independence copula} (\eqn{\mathbf{\Pi}(u,v)}) (\code{\link{P}}) when \eqn{\mathbf{C}(u^\circ,v^\circ) = \mathbf{\Pi}(u,v)} at the medial location \eqn{u,v=1/2}. Also, the definition here might be unique to the \pkg{copBasic} package. The decile version (\code{blomatrixCOPdec}) of this function uses \eqn{u^\circ \in (1, 5, 9) / 10} and \eqn{v^\circ \in (1, 5, 9) / 10}. Whereas, the quartile version (\code{blomatrixCOPiqr}) of this function uses  \eqn{u^\circ \in (25, 50, 75) / 100} and \eqn{v^\circ \in (25, 50, 75)/100}. If \code{as.blomCOPss=TRUE} argument is set (\bold{default operation}), then the coordinate locations in the matrix become the \eqn{\beta^\diamond_\mathbf{C}} of \code{\link{blomCOPss}}. As a rule \eqn{\beta^\circ_\mathbf{C} \ne \beta^\diamond_\mathbf{C}}.
}
\usage{
blomatrixCOPdec(cop=NULL, para=NULL, as.sample=FALSE, as.blomCOPss=TRUE,
                  ctype=c("weibull", "hazen", "1/n",
                          "bernstein", "checkerboard"), ...)
blomatrixCOPiqr(cop=NULL, para=NULL, as.sample=FALSE, as.blomCOPss=TRUE,
                  ctype=c("weibull", "hazen", "1/n",
                          "bernstein", "checkerboard"), ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{as.sample}{A logical controlling whether an optional \R \code{data.frame} in \code{para} is used to compute the  \eqn{\hat\beta^\circ_\mathbf{C}}-matrix at which point the \code{ctype} argument will be passed to multiple calls of \code{\link{EMPIRcop}};}
  \item{as.blomCOPss}{A logical to trigger \code{\link{blomCOPss}} for each of the \eqn{(u,v)} locations where for the \code{\link{blomCOPss}} calls (\eqn{\beta^\diamond_\mathbf{C}(\mathbf{u}, \mathbf{v})}): \eqn{u \mapsto (u,u) \mapsto \mathbf{u}} and  \eqn{v  \mapsto (v,v) \mapsto \mathbf{v}};}
  \item{ctype}{Argument of the same as \code{\link{EMPIRcop}}; and}
  \item{...}{Additional arguments to pass to the copula.}
}
\value{
  The matrix for \eqn{\beta^\circ_\mathbf{C}} is returned depending on whether the decile or quartile version has been called.
}
\author{W.H. Asquith}
\seealso{\code{\link{blomCOP}}, \code{\link{blomCOPss}}
}
\examples{
round(blomatrixCOPdec(cop=P), digits=8);     round(blomatrixCOPiqr(cop=P), digits=8)
#          U|V=0.10 U|V=0.50 U|V=0.90        #          U|V=0.25 U|V=0.50 U|V=0.75
# U|V=0.90        0        0        0        # U|V=0.75        0        0        0
# U|V=0.50        0        0        0        # U|V=0.50        0        0        0
# U|V=0.10        0        0        0        # U|V=0.25        0        0        0

round(blomatrixCOPdec(cop=PSP, as.blomCOPss=TRUE), digits=8)
#           U|V=0.10   U|V=0.50   U|V=0.90
# U|V=0.90 0.4736842  0.8181818  0.5153268
# U|V=0.50 0.8181818  0.3333333  0.6459330
# U|V=0.10 0.8901099  0.4736842  0.1708292

round(blomatrixCOPdec(cop=PSP, as.blomCOPss=FALSE), digits=8)
#            U|V=0.10   U|V=0.50  U|V=0.90
# U|V=0.90 0.09890110 0.81818182 4.2631579
# U|V=0.50 0.05263158 0.33333333 0.8181818
# U|V=0.10 0.01010101 0.05263158 0.0989011

\dontrun{
set.seed(1)
td <- c(0.10, 0.50, 0.90)
UVn <- simCOP(n=5000, cop=glueCOP, col=8,
          para=list(glue=0.4, cop1 =PLcop,           cop2=PLcop,
                             para1=PLpar(rho=-0.5), para2=PLpar(rho=+0.5)))
points(td, rep(td[3], 3), cex=2, lwd=2, pch=3, col="red")
points(td, rep(td[2], 3), cex=2, lwd=2, pch=3, col="red")
points(td, rep(td[1], 3), cex=2, lwd=2, pch=3, col="red")

print(blomatrixCOPdec(as.sample=TRUE, para=UVn, ctype="weibull"))
#             U|V=0.10 U|V=0.50 U|V=0.90
# U|V=0.90 -0.08222222   -0.580   -0.190
# U|V=0.50  0.30800000    0.112    0.264
# U|V=0.10  0.84000000    0.620    0.262

BMdn <- blomatrixCOPdec(cop=glueCOP,
          para=list(glue=0.4,  cop1=PLcop,            cop2=PLcop,
                              para1=PLpar(rho=-0.5), para2=PLpar(rho=+0.5)))
print(round(BMdn, digits=8))
#             U|V=0.10   U|V=0.50   U|V=0.90
# U|V=0.90 -0.08110464 -0.5569028 -0.2053772
# U|V=0.50  0.33449815  0.1202744  0.2424668
# U|V=0.10  0.75217766  0.6013719  0.2424668 }


\dontrun{
set.seed(1); nsim <- 2000
para.pop <- list( cop1=GHcop,             cop2=PLcop,  alpha=0.359,
                 para1=c(4.003, 1.099),  para2=0.882,   beta=0.292)
UVs <- simCOP(nsim, cop=composite2COP, para=para.pop)
mtext("GIVEN THIS SAMPLE")
Rho <- rhoCOP(as.sample=TRUE, para=UVs) # Spearman Rho
BMn <- blomatrixCOPdec(as.sample=TRUE, para=UVs, ctype="weibull")

parafn <- function(k) {
  c(exp(k[1])+1, exp(k[2]), exp(k[3]), pnorm(k[4]), pnorm(k[5]))
}
parafn_list <- function(k) {
  k <- parafn(k)
  list(cop1=GHcop, para1=c(k[1], k[2]), alpha=k[4],
       cop2=PLcop, para2=k[4],          beta=k[5])
}
BLOM_ofun <- function(para, statmat=NULL, parafn=NULL, rho=NA) {
   para <- parafn(para)
   new.para <- list(cop1=GHcop, para1=para[1:2], alpha=para[4],
                    cop2=PLcop, para2=para[3],    beta=para[5])
   bm <- blomatrixCOPdec(cop=composite2COP, para=new.para)
   err <- sum((statmat - bm)^2) + (rhoCOP(cop=composite2COP, para=new.para) - rho)^2
   #print(c(para, err))
   return(err)
}

run1 <- function(graphics=TRUE, nsim=0) {
  par.init <- c(log(1), log(1), log(1), qnorm(0.5), qnorm(0.5))
  rt       <- optim(par.init, BLOM_ofun, statmat=BMn, parafn=parafn, rho=Rho)
  para     <- parafn(rt$par)
  para.fit <- list(  cop1=GHcop,      cop2=PLcop,   alpha=para[4],
                    para1=para[1:2], para2=para[3],  beta=para[5])
  uv <- simCOP(nsim, cop=composite2COP, para=para.fit, graphics=graphics, col=2)
  rmse <- round(rmseCOP(uv[,1], uv[,2], ctype="weibull",
                        cop=composite2COP, para=para.fit), digits=8)
  if(graphics) mtext(paste0("RMSE(run1)=", rmse))
  return(list(rmse=rmse, para=para.fit))
}
system.time(RUN1 <- run1(nsim=nsim))
par.init <- c(log(RUN1$para$para1[1]), log(RUN1$para$para1[2]),
              log(RUN1$para$para2), qnorm(RUN1$para$alpha), qnorm(RUN1$para$beta))

RMSE_ofun <- function(para, parafn=NULL) {
   para <- parafn(para)
   new.para <- list(cop1=GHcop, para1=para[1:2], alpha=para[4],
                    cop2=PLcop, para2=para[3],    beta=para[5])
   new.rmse <- rmseCOP(UVs[,1], UVs[,2], cop=composite2COP, para=new.para)
   #print(c(para, new.rmse))
   return(new.rmse)
}
run2 <- function(graphics=TRUE, nsim=0, par.init=NULL) {
  if(is.null(par.init)) {
    par.init <- c(log(1), log(1), log(1), qnorm(0.5), qnorm(0.5))
  }
  rt       <- optim(par.init, RMSE_ofun, parafn=parafn)
  para     <- parafn(rt$par)
  para.fit <- list(  cop1=GHcop,      cop2=PLcop,   alpha=para[4],
                    para1=para[1:2], para2=para[3],  beta=para[5])
  uv <- simCOP(nsim, cop=composite2COP, para=para.fit, graphics=graphics, col=4)
  rmse <- round(rmseCOP(uv[,1], uv[,2], ctype="weibull",
                        cop=composite2COP, para=para.fit), digits=8)
  if(graphics) mtext(paste0("RMSE(run2)=", rmse))
  return(list(rmse=rmse, para=para.fit))
}
system.time(RUN2.1 <- run2(nsim=nsim, par.init=par.init))
system.time(RUN2.2 <- run2(nsim=nsim, par.init=NULL    ))

GIVN <- c(para.pop$alpha, para.pop$para1, para.pop$beta, para.pop$para2)
FIT1   <- c(RUN1$para$alpha, RUN1$para$para1, RUN1$para$beta, RUN1$para$para2)
FIT1   <- round(FIT1, digits=3)
FIT2.1 <- c(RUN2.1$para$alpha, RUN2.1$para$para1, RUN2.1$para$beta, RUN2.1$para$para2)
FIT2.1 <- round(FIT2.1, digits=3)
FIT2.2 <- c(RUN2.2$para$alpha, RUN2.2$para$para1, RUN2.2$para$beta, RUN2.2$para$para2)
FIT2.2 <- round(FIT2.2, digits=3)
nms    <- c("what", "alpha", "para1_1", "para1_2", "beta", "para2")
GIVN   <- c("given",   GIVN);  FIT2.1 <- c("by_Blom1", FIT2.1)
FIT1   <- c("by_RMSE", FIT1);  FIT2.2 <- c("by_Blom2", FIT2.2)
names(GIVN)   <- nms; names(FIT1)   <- nms
names(FIT2.1) <- nms; names(FIT2.2) <- nms
RESL <- cbind(data.frame(GIVN), data.frame(FIT1), data.frame(FIT2.1), data.frame(FIT2.2))
print(RESL) #}
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{Blomqvist Beta}

\encoding{utf8}
\name{diagCOPatf}
\alias{diagCOPatf}
\alias{diagCOPinv}
\title{Numerical Rooting the Diagonal of a Copula}
\description{
Compute a numerical root along the \emph{primary diagonal} (Nelsen, 2006, pp. 12 and 16) of copula \eqn{\mathbf{C}(u,v) = F = \mathbf{C}(t,t)} having joint probability \eqn{F}. The diagonals treat the nonexceedance probabilities \eqn{u} and \eqn{v} as equals (\eqn{u=v=t}). The primary diagonal is defined for a joint nonexceedance probability \eqn{t} as

\deqn{F = \mathbf{C}(t,t) \rightarrow t = \delta_{\mathbf{C}}^{(-1)}(f)\mbox{,}}

where the function solves for \eqn{t}. Examples using the concept behind \code{diagCOPatf} are available under \code{\link{duCOP}} and \code{\link{jointCOP}}, thus the \code{diagCOPatf} function can be also called by either \code{\link{jointCOP}} and \code{\link{joint.curvesCOP}}. Internally, the function uses limits of the root finder that are not equal to the anticipated interval \eqn{[0,1]}, but equal to \dQuote{small} (see description for argument \code{interval}). The function does trap for \code{f = 0} by returning zero and \code{f = 1} by returning unity.
}
\usage{
diagCOPatf(f, cop=NULL, para=NULL, interval=NULL, silent=TRUE, verbose=FALSE,
                                   tol=.Machine$double.eps/10, ...)
diagCOPinv(f, cop=NULL, para=NULL, interval=NULL, silent=TRUE, verbose=FALSE,
                                   tol=.Machine$double.eps/10, ...)
}
\arguments{
  \item{f}{Joint probability values as a nonexceedance probability \eqn{F} for which to compute the root \eqn{t};}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters, if needed, to pass to the copula;}
  \item{interval}{An optional interval for the root search. The default is \code{interval=}\code{c(lo, 1-lo)} for \code{lo=.Machine$double.eps} because of difficulties for an interval on \eqn{[0,1]};}
  \item{silent}{The argument of the same name given over to \code{try()} wrapping the \code{uniroot()} operation;}
  \item{verbose}{If \code{TRUE} then the whole output of the numerical root is returned using only the first value provided by argument \code{f};}
  \item{tol}{The \code{tol}erance to pass to \code{uniroot}. The default here is much smaller than the default of the \code{uniroot()} function in \R because of possibility that \code{diagCOPatf} would be used at extremely large nonexceedance probabilities; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  An \R \code{list} of the root by the \code{uniroot()} function in \R is returned if \code{verbose} is \code{TRUE}, otherwise the roots (diagonal inverses) for \eqn{t} are returned, and if an individual inverse operation fails, then a \code{NA} is returned instead.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{diagCOP}}, \code{\link{jointCOP}}, \code{\link{joint.curvesCOP}}}
\examples{
diagCOPatf(0.67, cop=PSP) # 0.8023879
diagCOPatf(0.99, cop=M)   # 0.99 (now see the example below)

\dontrun{
# Several functions from the lmomco package are needed.
# Suppose we have two phenomena with these log10 L-moments:
lmrA <- lmomco::vec2lmom(c(3.97, 0.485, -0.1178, 0.06857))
lmrB <- lmomco::vec2lmom(c(3.77, 0.475, -0.1377, 0.08280))
# Suppose we think that the Gumbel-Hougaard copula is appropriate with a Tau=0.45
Tau <- 0.45 #  Kendall Tau between A and B.
# Suppose that the F=0.99 for either A and B provides a common risk level when they
# are considered in isolation. But what if A and B are rivers that join and joint
# FF=0.99 at their union is of interest?
FF <- 0.99
parA   <- lmomco::lmom2par(lmrA, type="kap")
parB   <- lmomco::lmom2par(lmrB, type="kap")
EventA <- lmomco::qlmomco(FF, parA)
EventB <- lmomco::qlmomco(FF, parB)
ApB <- 10^(EventA) + 10^(EventB) # Purely an additive conceptualization
# The FF=0.99 event is assumed to occur simultaneously on both streams, which is
# equivalent to saying that the correlation between the two is absolute 1-to-1.

# Now consider including the association as measured by Kendall Tau:
Fjoint  <- diagCOPatf(FF, cop=GHcop, para=GHcop(tau=Tau)$para)
EventAj <- lmomco::qlmomco(Fjoint, parA)
EventBj <- lmomco::qlmomco(Fjoint, parB)
AcB <- 10^(EventAj) + 10^(EventBj) # Joint probability 0.99 at the union

# Now consider the association if the rivers are INDEPENDENT:
Fjoint  <- diagCOPatf(FF, cop=GHcop, para=GHcop(tau=0)$para)
EventAj <- lmomco::qlmomco(Fjoint, parA)
EventBj <- lmomco::qlmomco(Fjoint, parB)
AiB <- 10^(EventAj) + 10^(EventBj) # Joint probability 0.99 at the union

# ApB = 312,000 # The perfectly simultaneous addition makes too little.
# AcB = 323,000 # The copula preserves at least the known association.
# AiB = 330,000 # The independence conceptualization makes too much.}
}
\keyword{visualization}
\keyword{diagnostics}
\keyword{copula (characteristics)}
\concept{diagonal inversion}
\keyword{copula (diagonal)}
\keyword{copula (diagonal inversion)}

\encoding{utf8}
\name{ReineckeWell266}
\docType{data}
\alias{ReineckeWell266}
\title{Porosity and Permeability Data for Well-266 of the Reinecke Oil Field, Horseshoe Atoll, Texas}
\description{
These data represent porosity and permeability data from laboratory analysis for Well-266 Reinecke Oil Field, Horseshoe Atoll, Texas as used for the outstanding article by Saller and Dickson (2011). Dr. A.H. Saller shared a CSV file with the author of the \pkg{copBasic} package sometime in 2011. These data are included in this package because of the instruction potential of the bivariate relation between the geologic properties of permeability and porosity.
}
\usage{
data(ReineckeWell266)
}
\format{
An \R \code{data.frame} with
\describe{
  \item{WELLNO}{The number of the well, no. 266;}
  \item{DEPTH}{The depth in feet to the center of the incremental spacings of the data;}
  \item{FRACDOLOMITE}{The fraction of the core sample that is dolomite, 0 is 100 percent limestone;}
  \item{Kmax}{The maximum permeability without respect to orientation in millidarcies;}
  \item{POROSITY}{The porosity of the core sample; and}
  \item{DOLOMITE}{Is the interval treated as dolomite (1) or limestone (0).}
}
}
\references{
Saller, A.H., Dickson, J.A., 2011, Partial dolomitization of a Pennsylvanian limestone buildup by hydrothermal fluids and its effect on reservoir quality and performance: AAPG Bulletin, v. 95, no. 10, pp. 1745--1762.
}
\examples{
\dontrun{
data(ReineckeWell266)
summary(ReineckeWell266) # show summary statistics}
}
\keyword{datasets}

\encoding{utf8}
\name{jointCOP}
\alias{jointCOP}
\title{Compute Equal Marginal Probabilities Given a Single Joint AND or OR Probability for a Copula}
\description{
Given a single \emph{joint probability} denoted as \eqn{t} for a copula \eqn{\mathbf{C}(u,v)} numerically solve for bivariate marginal probabilities \eqn{U} and \eqn{V} such that they are also equal to each other (\eqn{u = v = w}). For the case of a \bold{joint and} probability, the primary diagonal of the copula (Nelsen, 2006, pp. 12 and 16) is solved for by a simple dispatch to the \code{\link{diagCOPatf}} function instead. Symbolically the solution is
\deqn{\mathrm{Pr}[U \le v,\ V \le v] = t = \mathbf{C}(w,w)\mbox{.}}


For the case of a \bold{joint or} probability, the \emph{dual of a copula (function)} or \eqn{\tilde{\mathbf{C}}(u,v)} from a copula (Nelsen, 2006, pp. 33--34; \code{\link{duCOP}}) is used where symbolicaly the solution is
\deqn{\mathrm{Pr}[U \le v \mathrm{\ or\ } V \le v] = t = \tilde{\mathbf{C}}(u,v) = u + v - \mathbf{C}(u,v)\mbox{,}}
or
\deqn{\mathrm{Pr}[U \le v \mathrm{\ or\ } V \le v] = t = 2w - \mathbf{C}(w,w)\mbox{.}}

The function for \code{type="or"} tests \eqn{\tilde{\mathbf{C}}(0,0)} and if it returns \code{NA} or \code{NaN} then the lower limit for the rooting is treated as \code{.Machine$double.eps} instead of 0 (zero).
}
\usage{
jointCOP(t, cop=NULL, para=NULL, type=c("and", "or"), ...)
}
\arguments{
  \item{t}{The joint probability level \eqn{t};}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{type}{The type of joint probability is to be computed; and}
  \item{...}{Additional arguments to pass to the \code{\link{duCOP}} function of \pkg{copBasic} or \code{uniroot()} function.}
}
\value{
  A vector of the equal \eqn{u} and \eqn{v} probabilties for the given \code{type} at the joint probability level of \eqn{t}. The vector includes the \eqn{t} as the third element.
}
\note{
\emph{ENSEMBLE 1---Counting and Copula Probabilities from a Massive Sample Size:} Simulations can be used to check and verify select copula concepts. Begin with a \emph{Gumbel--Hougaard} copula \eqn{\mathbf{GH}(u,v) = \mathbf{C}_{\Theta}(u,v)} having parameter \eqn{\Theta = 1.5}, which corresponds to a \emph{Kendall Tau} \eqn{\tau_\mathbf{C} = 1/3} (\code{\link{GHcop}}). Next, simulate and count the number of either \eqn{U} or \eqn{V} exceeding the 99th percentile \dQuote{event.} The event can occur either from the random variable \eqn{U} or from \eqn{V} with equal \dQuote{loss.} If the event occurs in both \eqn{U} and \eqn{V}, the loss is just the same as if \eqn{U} or \eqn{V} occurred. So, if a design were at the 100-year level and thus a 0.01 chance of loss each year, then 500 losses in 50,000 years would be expected.
\preformatted{
  set.seed(89); n <- 50000
  UV <- simCOP(n, cop=GHcop, para=1.5, graphics=FALSE)
  length(UV$U[UV$U > 0.99    | UV$V > 0.99])     # 799 times (losses)
  length(UV$U[UV$U > 0.99356 | UV$V > 0.99356])  # 500 times (losses)
}
Letting \code{JP} equal 0.99356, which forces the required acceptance of 500 losses for the design, has conditions of \code{UV$U > JP} \bold{or} \code{UV$V > JP} as well as condition of \code{UV$U > JP} \bold{and} \code{UV$V > JP}. These three conditions are captured using the structure of the \R code listed. Manual searching resulted in a value for \code{JP} equaling 0.99356, which produces the 500 count losses (acceptable losses). Thus, \code{JP} is a marginal bivariate probability (in this case equality between \eqn{U_{\mathrm{crit.}} = V_{\mathrm{crit.}}} declared) necessary to attain a 99th percentile joint protection from loss. The magnitude for either \eqn{U} or \eqn{V} thus must exceed the 99th percentile, and this is what the code shows with 799 losses.

It is important to consider that unless \eqn{U} and \eqn{V} are in perfect positive correlation (\emph{e.g.} \eqn{\mathbf{M}(u,v)}, \code{\link{M}}, \emph{\enc{Fréchet}{Frechet}--Hoeffding upper-bound copula}), that protection from loss needs to be higher than 0.99 if the marginal risk is set at that level. Continuing, if the 99th percentile is the 100-year event, then design criteria should be about 155 years instead [\code{lmomco::prob2T(0.99356)}]. The user can readily see this with the switch to perfect independence with the \eqn{\mathbf{GH}(u,v)} copula with \eqn{\Theta = 1} and produce quite different results or extreme correlation with say \eqn{\Theta > 20}.

To provide the protection for 500 exceedances in \eqn{n =} 50,000 trials and for purposes of demonstration, balance the protection between \eqn{U} and \eqn{V} by setting their probabilities equal, then the theoretical joint probability is
\preformatted{
  diagCOPatf(0.99, 0.99, cop=GHcop, para=1.5)             # 0.9936887
  jointCOP(  0.99,       cop=GHcop, para=1.5, type="and") # 0.9936887
}
and these two probabilities, which in reality are actually based on same computation (\code{\link{diagCOPatf}}), and nearly are the same as \code{JP} \eqn{=} 0.99356 that was determined by the manual searching on the simulated data.

A \bold{mutually inclusive and} condition can be arranged as follows, and it is implicit in the definition that both loss events occur at the same time:
\preformatted{
  length(UV$U[UV$U > 0.99 & UV$V > 0.99])            # 208 losses ( simulated )
  surCOP(  1-0.99, 1-0.99, cop=GHcop, para=1.5) * n  # 209 losses (theoretical)
  surfuncCOP(0.99,   0.99, cop=GHcop, para=1.5) * n  # 209 losses (theoretical)
}

What are the expected number of exceedances if designs for \eqn{U} and \eqn{V} are built at \eqn{U =} \eqn{V = 0.99} marginal probabilities for protection?
\preformatted{
   coCOP(1-0.99, 1-0.99, cop=GHcop, para=1.5)  * n  # 791 losses (theoretical)
  # by the co-copula which from the copula as nonexceedances is
  (1-COP(  0.99,   0.99, cop=GHcop, para=1.5)) * n  # 791 losses (theoretical)
}
Note, the 791 losses is nearly equal to 799, but obviously not 500 as one might incorrectly have imagined strictly in a univariate world.

Now, a couple of questions can be asked about the \bold{joint and} and \bold{joint or} probabilities using the definition of a copula \code{\link{COP}} and the \emph{dual of a copula (function)} (\eqn{\tilde{\mathbf{C}}(u,v)}, \code{\link{duCOP}}), respectively:
\preformatted{
  # The AND nonexceedances:
  length(UV$U[UV$U <= 0.99 & UV$V <= 0.99]) / n    # 0.98402   ( simulated )
    COP(0.99, 0.99, cop=GHcop, para=1.5)           # 0.9841727 (theoretical)

  # The OR nonexceedances:
  length(UV$U[UV$U <= 0.99 | UV$V <= 0.99]) / n    # 0.99584   ( simulated )
  duCOP(0.99, 0.99, cop=GHcop, para=1.5)           # 0.9958273 (theoretical)
}

How about inversion of \eqn{\tilde{\mathbf{C}}(u,v)} by \code{jointCOP} and check against the simulation?
\preformatted{
  jointCOP(0.99, cop=GHcop, para=1.5, type="or")[1]      # 0.9763951 ( theor. )
  length(UV$U[UV$U <= 0.9763951 | UV$V <= 0.9763951])/n  # 0.98982 ( simulated)
}
The second probability is a value quite near to 0.99. So, if one wants mutual loss protection, compute the inversion of the dual of a copula using \code{jointCOP(..., type="or")}. Let us say that 0.80 mutual loss protection is wanted
\preformatted{
  jointCOP(0.80, cop=GHcop, para=1.5, type="or")[1]      # 0.6561286
  n - length(UV$U[UV$U <= 0.6561286 | UV$V <= 0.6561286])# 10049 losses ( sim.)
  n - (1-0.8)*n                                    # 10000 losses (theoretical)
}

The example here shows numerical congruence of 10,049 \eqn{\approx} 10,000. An opposing question is useful. How about a \bold{mutually exclusive or} condition as nonexceedances?
\preformatted{
  # The mutually exclusive OR as nonexceedances:
  length((UV$U[  (UV$U <= 0.99 | UV$V <= 0.99) &
               ! (UV$U <= 0.99 & UV$V <= 0.99)]))        # 591 losses ( simulated )
  # The mutually exclusive OR as exceedances:
  length(UV$U[   (UV$U >  0.99 | UV$V >  0.99) &
               ! (UV$U >  0.99 & UV$V >  0.99)])         # 591 losses ( simulated )
}
It is clear that 208 \eqn{+} 591 \eqn{=} 799 as shown earlier; notice how there are two ways to get at the 591 count. There are 208 mutual loss events and 591 occassions where either \eqn{U} or \eqn{V} is the causation of loss. For comparison, how many observed events by random variable were observed?
\preformatted{
  length(UV$U[  (UV$U > 0.99)]) # 519 ( simulated )
  length(UV$U[  (UV$V > 0.99)]) # 491 ( simulated )
}
If the variables were perfectly uncorrelated (\eqn{\mathbf{P}(u,v)}, \code{\link{P}}, \emph{independence copula}) would be 519 \eqn{+} 491 \eqn{=} 1,007 losses. But for the simulations here, 799 losses occurred, so 1,007 \eqn{-} 799 \eqn{=} 208 losses were at the same time caused by \eqn{U} and \eqn{V}.

Both of the following lengths \code{A} and \code{B} are 799 and both represent a \bold{joint or} condition---the operations do not represent a \bold{mutually exclusive or} condition.
\preformatted{
  A <- length(UV$U[UV$U > 0.99]) + length(UV$U[UV$V > 0.99]) -
       length(UV$U[UV$U > 0.99 & UV$V > 0.99])
  B <- length(UV$U[UV$U > 0.99 | UV$V > 0.99]) # A == B == 799
}

\emph{ENSEMBLE 2---Simulation Study using a Real-World Sample Size:} Now with identities of sorts shown and described above, let us test a theoretically consistent version of a sample of size 150 repeated 1,000 times at the 98th percentile against loss by one or the other random variables or both for slightly correlated \eqn{U} or \eqn{V} again following the Gumbel--Hougaard copula. The losses incurred by mutual event occurrence is the same as if one or the other variables produced an event causing loss.
\preformatted{
  n <- 250; nsim <- 1000; EitherEvent <- 0.98; MutualEvent <- 0.99; Theta<- 1.5
  PT <- jointCOP(EitherEvent, cop=GHcop, para=Theta, type="and")[1] # 0.9873537
  DU <- jointCOP(MutualEvent, cop=GHcop, para=Theta, type="or" )[1] # 0.9763951
}

This next code listing is a redundant example to the one that follows, but it is shown because a slight possibility of confusion in the vectorized conditional evaluations in \R. This first example concretely changes the loss events into binary states and adds them up prior to the condition.
\preformatted{
  set.seed(894234)
  EX1a <- sapply(1:nsim, function(i) {
                 uv <- simCOP(n, cop=GHcop, para=Theta, graphics=FALSE)
          length(uv$U[ as.numeric(uv$U > PT) + as.numeric(uv$V > PT) >= 1 ]) })
  t.test(EX1a, mu=(1-EitherEvent) * n)
}
The expected count \eqn{E[}\code{EX1a}\eqn{] =} 5 and the simulation run yielded 5.058. The \code{t.test()} function in \R results in a statistically insignificant difference. The following two example use a similar there. For sake of both code brevity and clarity, the examples here all restart the simulations at the expense of computation time.
\preformatted{
  set.seed(894234)
  EX1b <- sapply(1:nsim, function(i) {
                uv  <- simCOP(n, cop=GHcop, para=Theta, graphics=FALSE)
                length(uv$U[uv$U > PT | uv$V > PT])  })
  t.test(EX1b, mu=(1-EitherEvent) * n)
}
The expected count \eqn{E[}\code{EX1b}\eqn{] =} 5---the same results are shown as in the first example listing.

Next, let us demonstrate the dual of a copula for mutually occurring events.
\preformatted{
  set.seed(894234)
  EX2 <- sapply(1:nsim, function(i) {
                uv  <- simCOP(n, cop=GHcop, para=Theta, graphics=FALSE)
                length(uv$U[uv$U > DU & uv$V > DU]) })
  t.test(EX2, mu=(1-MutualEvent) * n)
}
The expected count \eqn{E[}\code{EX2}\eqn{] =} 2.5 and the simulation run yielded 2.49. The \code{t.test()} again results in a statistically insignificant difference.


The \eqn{U = V = 0.9873537} for the \dQuote{and} and \eqn{U = V = 0.9763951} for the \dQuote{or} are not equal marginal probabilities. Taking the larger of the two marginal probabilities, the actual joint protection from mutual event occurrence can be computed:
\preformatted{
  duCOP(0.9873537, 0.9873537, cop=GHcop, para=Theta) # 0.9947075
  (1-0.9947075)*n  # which is about 1.32 events per 250 trials.
}
So, the larger protection in terms of joint probabilities provided by \code{EitherEvent} at 0.98 instead of \code{MutualEvent} at 0.99 with respective protection at the 0.9873537 provides a \code{MutualEvent} protection of 0.9947075.
\preformatted{
  set.seed(894234)
  EX3 <- sapply(1:nsim, function(i) {
                uv  <- simCOP(n, cop=GHcop, para=Theta, graphics=FALSE)
                length(uv$U[uv$U > 0.9873537 & uv$V > 0.9873537]) })
  t.test(EX3, mu=(1-0.9947075) * n)
}
The expected count \eqn{E[}\code{EX3}\eqn{] =} 1.32 and the simulation run yielded 1.31. The \code{t.test()} again results in a statistically insignificant difference, and thus the result indistinguishable from the expectation.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{diagCOPatf}}, \code{\link{duCOP}}, \code{\link{joint.curvesCOP}}, \code{\link{level.curvesCOP}}
}
\examples{
jointCOP(0.50, cop=GHcop, para=1.5, type="and") # 0.6461941  0.6461941  0.5000000
jointCOP(2/3,  cop=GHcop, para=1.5, type="or" ) # 0.4994036  0.4994036  0.6666667

# See extended code listings and discussion in the Note section
}
\keyword{joint probability}

\encoding{utf8}
\name{lcomCOP}
\alias{lcomCOP}
\title{L-comoments and Bivariate L-moments of a Copula}
\description{
Compute the \emph{L-comoments} (Serfling and Xiao, 2007; Asquith, 2011) through the \emph{bivariate L-moments (ratios)} (\eqn{\delta^{[\ldots]}_{k;\mathbf{C}}}) of a copula \eqn{\mathbf{C}(u,v; \Theta)}.  The L-comoments include \emph{L-correlation} (\emph{Spearman Rho}), \emph{L-coskew}, and \emph{L-cokurtosis}.  As described by Brahimi \emph{et al.} (2015), the first four bivariate L-moments \eqn{\delta^{[12]}_k} for random variable \eqn{X^{(1)}} or \eqn{U} with respect to (\emph{wrt}) random variable \eqn{X^{(2)}} or \eqn{V} are defined as
\deqn{\delta^{[12]}_{1;\mathbf{C}} = 2\int\!\!\int_{\mathcal{I}^2}
\mathbf{C}(u,v)\,\mathrm{d}u\mathrm{d}v - \frac{1}{2}\mbox{,}}
\deqn{\delta^{[12]}_{2;\mathbf{C}} = \int\!\!\int_{\mathcal{I}^2}
(12v - 6)
\mathbf{C}(u,v)\,\mathrm{d}u\mathrm{d}v - \frac{1}{2}\mbox{,}}
\deqn{\delta^{[12]}_{3;\mathbf{C}} = \int\!\!\int_{\mathcal{I}^2}
(60v^2 - 60v + 12)
\mathbf{C}(u,v)\,\mathrm{d}u\mathrm{d}v - \frac{1}{2}\mbox{, and}}
\deqn{\delta^{[12]}_{4;\mathbf{C}} = \int\!\!\int_{\mathcal{I}^2}
(280v^3 - 420v^2 + 180v - 20)
\mathbf{C}(u,v)\,\mathrm{d}u\mathrm{d}v - \frac{1}{2}\mbox{,}}
where the bivariate L-moments are related to the L-comoment ratios by
\deqn{6\delta^{[12]}_k = \tau^{[12]}_{k+1}\mbox{\quad and \quad}6\delta^{[21]}_k = \tau^{[21]}_{k+1}\mbox{,}}
where in otherwords, \dQuote{the third bivariate L-moment \eqn{\delta^{[12]}_3} is one sixth the L-cokurtosis \eqn{\tau^{[12]}_4}.} The first four bivariate L-moments yield the first five L-comoments. The terms and nomenclature are not easy and also the English grammar adjective \dQuote{ratios} is not always consistent in the literature. The \eqn{\delta^{[\ldots]}_{k;\mathbf{C}}} are \bold{ratios}. The sample L-comoments are supported by the \pkg{lmomco} package, and in particular for the bivariate case, they are supported by the \code{lcomoms2()} function of that package.

Similarly, the \eqn{\delta^{[21]}_k} are computed by switching \eqn{u \rightarrow v} in the polynomials within the above integrals multiplied to the copula in the system of equations with \eqn{u}. In general, \eqn{\delta^{[12]}_k \not= \delta^{[21]}_k} for \eqn{k > 1} unless in the case of \emph{permutation symmetric} (\code{\link{isCOP.permsym}}) copulas. By theory, \eqn{\delta^{[12]}_1 = \delta^{[21]}_1 = \rho_\mathbf{C}/6} where \eqn{\rho_\mathbf{C}} is the \emph{Spearman Rho} \code{\link{rhoCOP}}.

The integral for \eqn{\delta^{[12]}_{4;\mathbf{C}}} does not appear in Brahimi \emph{et al.} (2015) but this and the other forms are verified in the \bold{Examples} and discussion in \bold{Note}. The four \eqn{k \in (1,2,3,4)} for \eqn{U} \emph{wrt} \eqn{V} and \eqn{V} \emph{wrt} \eqn{U} comprise a full spectrum of system of seven (not eight) equations. One equation is lost because \eqn{\delta^{[12]}_1 = \delta^{[21]}_1}.

Chine and Benatia (2017) describe \emph{trimmed L-comoments} as the multivariate extensions of the univariate \emph{trimmed L-moments} (Elamir and Seheult, 2003) that are implemented in \pkg{lmomco}. These are not yet implemented in \pkg{copBasic}.
}
\usage{
lcomCOP(cop=NULL, para=NULL, as.bilmoms=FALSE, orders=2:5, lo=0, hi=1,
        subdivisions=100L, rel.tol=.Machine$double.eps^0.25, abs.tol=rel.tol,
        stop.on.error=TRUE, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{as.bilmoms}{A logical to trigger return of the \eqn{\delta_k} and the return vectors will be named differently;}
  \item{orders}{The orders of the L-comoments to return, which is internally adjusted if the argument \code{as.bilmoms} is set. There is no first order L-comoment and the first index on returned values is set to \code{NA} to remain index consistent with the \pkg{lmomco} package. An order greater than 5 is not supported;}
  \item{lo}{Lower limit of integration (must absurdly leave as zero);}
  \item{hi}{Upper integrand (must assuredly leave as unity);}
  \item{subdivisions}{Maximum number of subintervals for \code{integrate()};}
  \item{rel.tol}{Relative accuracy requested for \code{integrate()};}
  \item{abs.tol}{Absolute accuracy requested for \code{integrate()};}
  \item{stop.on.error}{Logical of same name for \code{integrate()} to silence for some cases of singularity or other more obscure problems the error \dQuote{Error in integrate(function(v) : the integral is probably divergent;} and}
  \item{...}{Additional arguments to pass to the \code{\link{densityCOP}} function and to \code{integrate()}.}
}
\value{
  An \R \code{list} of the L-comoments or bivariate L-moments is returned depending on \code{as.bilmoms} setting.
  \item{bilmomUV}{The bivariate L-moments \eqn{\delta^{[12]}_k} of \eqn{U} with respect to \eqn{V} for \eqn{k \in [1,2,3,4]} if \code{orders} is \code{2:5} and there is no \code{NA} index as for the L-comoments;}
  \item{bilmomVU}{The bivariate L-moments \eqn{\delta^{[21]}_k} of \eqn{V} with respect to \eqn{U} for \eqn{k \in [1,2,3,4]} if \code{orders} is \code{2:5} and there is no \code{NA} index as for the L-comoments;}
  \item{lcomUV}{The L-comoments \eqn{\tau^{[12]}_k} of \eqn{V} with respect to \eqn{U} for \eqn{k \in [2,3,4,5]} if \code{orders} is \code{2:5} and index 1 is \code{NA}; and}
  \item{lcomVU}{The L-comoments \eqn{\tau^{[21]}_k} of \eqn{V} with respect to \eqn{U} for \eqn{k \in [2,3,4,5]} if \code{orders} is \code{2:5} and index 1 is \code{NA}.}
}
\note{
The documentation here is highly parallel to \code{\link{bilmoms}} for which that function was developed some years before \code{lmomCOP} was developed in January 2019. Also, \code{\link{bilmoms}} is based on gridded or Monte Carlo integration, and \code{\link{bilmoms}} is to be considered \bold{deprecated}. However, it is deliberate that related background and various algorithm testing are still made available in \code{\link{bilmoms}}. \emph{L-comoment ratio diagrams} are demonstrated by extensive example for many copula in \code{\link{LCOMDIA_ManyCops}}, \code{\link{LCOMDIA_GH2cop}}, \code{\link{LCOMDIA_GH3cop}}, and \code{\link{ORDSUMcop}}.
}
\references{
Asquith, W.H., 2011, Distributional analysis with L-moment statistics using the R environment for statistical computing: Createspace Independent Publishing Platform, ISBN 978--146350841--8.

Brahimi, B., Chebana, F., and Necir, A., 2015, Copula representation of bivariate L-moments---A new estimation method for multiparameter two-dimensional copula models: Statistics, v. 49, no. 3, pp. 497--521.

Chine, Amel, and Benatia, Fatah, 2017, Bivariate copulas parameters estimation using the trimmed L-moments methods: Afrika Statistika, v. 12, no. 1, pp. 1185--1197.

Elamir, E.A.H, and Seheult, A.H., 2003, Trimmed L-moments: Computational Statistics and Data Analysis, v. 43, p. 299--314.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Serfling, R., and Xiao, P., 2007, A contribution to multivariate L-moments---L-comoment matrices: Journal of Multivariate Analysis, v. 98, pp. 1765--1781.
}
\author{W.H. Asquith}
\seealso{\code{\link{bilmoms}}, \code{\link{lcomCOPpv}}, \code{\link{uvlmoms}}
}
\examples{
\dontrun{
  para <- list(alpha=0.5, beta=0.93, para1=4.5, cop1=GLcop, cop2=PSP)
  lcomCOP(cop=composite2COP, para=para)$lcomUV[3]
  # Lcomom:T3[12]=  +0.156
  lcomCOP(cop=composite2COP, para=para)$lcomVU[3]
  # Lcomom:T3[21]=  -0.0668
  bilmoms(cop=composite2COP, n=10000, para=para, sobol=TRUE)$bilcomoms$T3
  # Tau3[12]=+0.1566, Tau3[21]=-0.0655
  # The numerical default Monte Carlo integration of bilmoms()
  # matches the numerical integration of lcomCOP albeit with a
  # substantially slower and less elegant means in bilmoms().}

\dontrun{
  # The following Spearman Rho and L-coskew values are predicted for a monitoring
  # location of the relation between peak streamflow (V) and time into the
  # water year (U) where U and V are "U-statistics."
  site_srho  <-  0.15536430
  site_T3_12 <-  0.03866056
  site_T3_21 <- -0.03090144

  # Create an objective function for 3D optimization with some explicit intention that
  # transforms are used to keep the parameters in acceptable parameter space for the
  # alpha [0,1] and beta [0,1] and theta for the Plackett copula and the composite1COP
  # used to add two more parameters to the Plackett.
  ofunc <- function(par, srho=NA, T3_12=NA, T3_21=NA) {
    alpha <- pnorm(par[1]) # takes -Inf to +Inf ---> 0 to 1 # compositing domain
    beta  <- pnorm(par[2]) # takes -Inf to +Inf ---> 0 to 1 # compositing domain
    theta <-   exp(par[3]) # takes -Inf to +Inf ---> 0 to +Inf # Plackett domain
    lmr <- lcomCOP(cop=composite1COP,
                   para=list(alpha=alpha, beta=beta, para1=theta, cop1=PLcop))
    return((lmr$lcomUV[2] - srho )^2 + # look carefully, the 2, 3, 3 index
           (lmr$lcomUV[3] - T3_12)^2 + # use on the lmr list are correct, so do not
           (lmr$lcomVU[3] - T3_21)^2)  # expect to see 1, 2, 3 or 2, 3, 4.
  }
  # initial parameter guess ('middle' [0.5] compositing and independence [1]) and
  # showing the transformations involved.
  para_init <- c(qnorm(0.5), qnorm(0.5), log(1))
  rt <- optim(par=para_init, ofunc,
              srho=site_srho, T3_12=site_T3_12, T3_21=site_T3_21)
  lcom_para <- list(alpha=pnorm(rt$par[1]), beta=pnorm(rt$par[2]),
                    para1=exp(rt$par[3]), cop1=PLcop)
  sUV <- simCOP(10000, cop=composite1COP, para=lcom_para, col=grey(0, 0.2), pch=16)
  # Now as an exercise, consider increasing site_srho or negating it. Consider
  # switching the signs on the L-coskews or increasing their magnitudes and study
  # the resulting simulation to develop a personal feeling for L-coskew meaning. #}
}
\keyword{bivariate asymmetry (measures)}
\keyword{bivariate skewness}
\keyword{bivariate L-moments}
\concept{bivariate L-correlation}
\concept{bivariate L-skew}
\concept{bivariate L-kurtosis}
\concept{copula L-comoments}
\concept{copula L-correlation}
\concept{copula L-coskew}
\concept{copula L-cokurtosis}
\keyword{L-comoments}
\concept{Lcomoments}
\keyword{Spearman Rho}
\keyword{L-comoments (ratio diagram)}
\keyword{L-comoment ratio diagram}

\encoding{utf8}
\name{diagCOP}
\alias{diagCOP}
\title{The Diagonals of a Copula}
\description{
Compute the \emph{primary diagonal} or alternatively the \emph{secondary diagonal}  (Nelsen, 2006, pp. 12 and 16) of copula \eqn{\mathbf{C}(u,v)}.
The primary diagonal is defined as
\deqn{\mathbf{\delta}_\mathbf{C}(t) = \mathbf{C}(t,t)\mbox{,}}
and the secondary diagonal is defined as
\deqn{\mathbf{\delta}^{\star}_\mathbf{C}(t) = \mathbf{C}(t,1-t)\mbox{.}}
Plotting is provided by this function because the diagonals are such important visual attributes of a copula. This function computes whole diagonals. If individual values are desired, then users are asked to use function calls along the diagonal such as \code{COP(0.25,0.25, cop=P)} for the primary diagonal and \code{COP(0.25,1-0.25, cop=P)} for the secondary diagonal, where for both examples the \emph{independence copula} (\eqn{uv = \mathbf{\Pi}}; \code{\link{P}}) was chosen for purposes of clarification.

The \eqn{\mathbf{\delta}_\mathbf{C}(t)} is related to order statistics of the multivariate sample (here bivariate) (Durante and Sempi, 2015, p. 68). The probability for the maxima is
\eqn{\mathrm{Pr}[\mathrm{max}(u, v) \le t] = \mathbf{C}(t,t) = \mathbf{\delta}_\mathbf{C}(t) \mbox{\ and }}
the probability for the minima is
\eqn{\mathrm{Pr}[\mathrm{min}(u, v) \le t] = 2t - \mathbf{\delta}_\mathbf{C}(t)\mbox{.}}
}
\usage{
diagCOP(cop=NULL, para=NULL, secondary=FALSE,
        ploton=TRUE, lines=TRUE, delt=0.005, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters, if needed, to pass to the copula;}
  \item{secondary}{A logical to toggle the secondary diagonal;}
  \item{ploton}{A logical to toggle on the plot;}
  \item{lines}{Draw the lines of diagonal to the current device;}
  \item{delt}{The increment of the diagonal curve to plot, defaults to 0.5-percent intervals, which should be small enough to resolve fine curvature for many copulas in practice; and}
  \item{...}{Additional arguments to pass to the \code{plot()} and \code{lines()} functions in \R.}
}
\value{
  An \R \code{list} of the \eqn{t} values, \eqn{\mathbf{\delta}_\mathbf{C}(t,t)} (primary) or \eqn{\mathbf{\delta}^{\star}_\mathbf{C}(t,1-t)} (secondary diagonal), along with a tag as to which diagonal is returned.
}
\references{
Durante, F., and Sempi, C., 2015, Principles of copula theory: Boca Raton, CRC Press, 315 p.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{diagCOPatf}}, \code{\link{COP}}, \code{\link{sectionCOP}}}
\examples{
\dontrun{
# The primary diagonal of the W, P, M, and PSP copulas on the same plot
D <- diagCOP(cop=W,   lwd=2)
D <- diagCOP(cop=P,   lty=2, ploton=FALSE)
D <- diagCOP(cop=M,   col=2, ploton=FALSE)
D <- diagCOP(cop=PSP, col=3, ploton=FALSE)
mtext("PRIMARY DIAGONAL OF SIMPLE COPULAS") # four primary diagonals }

\dontrun{
# The secondary diagonal of the W, P, M, and PSP copulas on the same plot
D <- diagCOP(cop=W,   lwd=2, secondary=TRUE)
D <- diagCOP(cop=P,   lty=2, secondary=TRUE, ploton=FALSE)
D <- diagCOP(cop=M,   col=2, secondary=TRUE, ploton=FALSE)
D <- diagCOP(cop=PSP, col=3, secondary=TRUE, ploton=FALSE)
mtext("SECONDARY DIAGONAL OF SIMPLE COPULAS") # four secondary diagonals }
}
\keyword{visualization}
\keyword{diagnostics}
\keyword{copula (characteristics)}
\keyword{copula (diagonal)}

\encoding{utf8}
\name{wolfCOPtest}
\alias{wolfCOPtest}
\title{Empirically Created Schweizer and Wolff Sigma Test Against an Independence Copula}
\description{
Perform a \emph{Schweizer--Wolff Sigma} test for \eqn{\sigma_\mathbf{C}} stemming from \eqn{\mathrm{NULL} : \mathbf{C} = \mathbf{\Pi}} (\emph{Independence copula}, \code{\link{P}}), where  \eqn{\sigma_\mathbf{C}} is defined under \code{\link{wolfCOP}}. There are no other known hypothesis tests for \eqn{\sigma_\mathbf{C}} in the literature.

This \emph{Schweizer--Wolff Sigma hypothesis test} has been empirically tuned from small to large samples by extensive simulation, and, as implemented, the test is closely tied to the version of sample computation by \code{\link{wolfCOP}} using a particular form of the empirical copula (refer to that function for details). The \code{wolfCOPtest} function internally loads the \code{\link{wolfCOPtest_data_smlsam}} tables, and most users generally will not need to access that dataset themselves unless perhaps for checking veracity of the simulations.
}
\usage{
wolfCOPtest(x, y, asuv=FALSE, aslist=TRUE, na.rm=TRUE, digits=6,
                  probs=c(0.90, 0.95, 0.98, 0.99, 0.995),
                  zmat=NULL, statf=mean, rndphi=20, usepade=FALSE,
            ties.method=c("average", "first", "last", "random", "max", "min"), ...)
}
\arguments{
  \item{x}{Nonexceedance probabilities \eqn{u} in the \eqn{X} direction if \code{asuv=TRUE}, else data in the \eqn{X} direction for which nonexceedance probabilities will be computed by plotting-position formula. If \code{x} is a single value, it is assumed to be a previously computed \eqn{\sigma_\mathbf{C}} and \code{y} will be considered as the sample size. Note, statistical estimation is based on samples of \eqn{n \ge 3}, if the sample size is less than this, then internally \eqn{n = 3} is set for the computations. The \code{x} can also be a two column data structure and the \code{y} will be taken from the second column;}
  \item{y}{Nonexceedance probabilities \eqn{v} in the \eqn{Y} direction if \code{asuv=TRUE}, else data in the \eqn{Y} direction for which nonexceedance probabilities will be computed by plotting-position formula (refer to \code{lmomco::pp()} function, but simply the\cr\code{rank(...)} function is used);}
  \item{asuv}{Logical identifying whether the \code{x} and \code{y} are already in nonexceedance probabilities or rather such need to be computed internally by plotting-position formula (refer to \code{lmomco::pp()} function, but simply the \code{rank(...)} function is used);}
  \item{aslist}{Logical to return a list of the test results rather than a vector;}
  \item{na.rm}{A logical to trigger a \code{complete.cases} operation on internal matrix of the data as probabilities;}
  \item{digits}{Digits for rounding before return;}
  \item{probs}{Quantiles by nonexceedance probability to extract from the fitted distribution---these are common upper-tail critical values. If \code{probs} is empty, then 0.95 nonexceedance probability is silently used;}
  \item{zmat}{An optional matrix of four columns and rows equally those of the data values with columns 1 and 2 containing, respectively, the lower and upper limits of the \eqn{u} and columns 3 and 4 containing, respectively, the lower and upper limits of \eqn{v}. This \code{zmat} feature provides for support if data censoring is involved and simulation between the limits made (refer to \bold{Examples}). Note, if \code{zmat} is given, then \code{ties.method} is set \code{"random"} and \code{asuv} is set false;}
  \item{statf}{Statistical function for reducing simulated \eqn{\sigma} values (mean or median suggested), but a user could \code{plot(density(}\code{result$rand_sigma))} and reach their own decisions on centrality measure;}
  \item{rndphi}{A multiplier on the number of ties or censored values to drive simulation of samples as part of populating the \code{rand_sigma} element, when applicable, on the function return;}
  \item{usepade}{A logical to switch, on estimation of the empirical L-moments of the logit transformed simulations, between (1) power-exponent optimized regression equations, or (2) optimized \emph{\enc{Padé}{Pade} Approximant} forms;}
  \item{ties.method}{Argument of the same name for \code{rank}, but if there are ties, \code{asuv} is set false, and \code{ties.method} is set to random, then simulation is made internally on the ranking and \code{statf} used to compute the \eqn{\sigma} estimate;}
  \item{...}{Additional arguments to pass to the \code{lmomco::pp()} function.}
}
\details{
The implementation here is based on \emph{Monte Carlo simulations} of \eqn{\mathbf{\Pi}} and computation of the L-moments of a \emph{logit transform} of the computed \eqn{\sigma_\mathbf{\Pi}} for which nonlinear regression of the L-moments to the sample size have been created. Development scripts for the simulations are located in the directory \code{inst/make_}\code{wolfCOPtest/} of \pkg{copBasic} sources.

The \emph{logit transformation} is defined as

\deqn{\mathrm{logit}\bigl(\sigma_\mathbf{C}\bigr) = \mathrm{log}\frac{\sigma_\mathbf{C}}{1-\sigma_\mathbf{C}} = \eta\mbox{,}
}
and its inverse is defined as
\deqn{
\sigma_\mathbf{C} = \mathrm{logit}^{(-1)}(\eta) = \frac{1}{1 + \mathrm{exp}(-\eta)}\mbox{.}
}

Sample-sample empirical distributions (\eqn{n \in 3, \cdots, 40}) will have p-values reported on output and missing for \eqn{n \ge 41} (refer to \code{\link{wolfCOPtest_data_smlsam}} for more details).

The predicted L-moments are used to drive from the \pkg{lmomco} package the Pearson type III (PE3) (\code{parpe3}, \code{cdfpe3}, \code{quape3}) of the \emph{logit transform} of \eqn{\sigma_\mathbf{\Pi}} to compute the p-value and supporting information.

Evaluation of an \emph{L-moment ratio diagram} (refer to \pkg{lmomco} functions \code{plotlmrdia()} and \code{lmrdia()}) shows a change from a generalized normal distribution (GNO) to PE3 at about \eqn{n = 40}, but inspection of upper-tail empirical distributions (and hence p-values in the practical range for \eqn{\mathrm{NULL}} rejection) suggest the PE3 tail is better choice for overall application. The \code{wolfCOPtest} function, therefore, exclusively uses the PE3.


To conclude, the figure below shows the \emph{L-moment ratio diagram} of the \emph{L-skew} and \emph{L-kurtosis} diagram for the logit transform of the \emph{L-moments} of the \eqn{\hat\sigma_\mathbf{C}} based on \code{\link{EMPIRgrid_fast}} in the aforementioned build directory of the \pkg{copBasic} package. The figure is based on nearly \eqn{900} million simulations summed across a representative range of sample sizes \eqn{n \in 8, \cdots 15001}, and provides support for the decision by the author to use the PE3 as the distribution model for the hypothesis test.

\if{html}{\figure{lmrdiaT34wolfP.jpg}{options: width=500}}
\if{latex}{\figure{lmrdiaT34wolfP.pdf}{options: width=5in, trim=0in 0in 0in 0in, clip}}

}
\value{
Extensive list or named vector output the test is returned. For the list, the \code{estimate} contains duplicate information as \code{statistic[1]}; this is so that there is an \code{estimate} entry to parallel the return of \code{cor.test(...)} to help users write nomenclature similar calls between the functions.
}
\author{W.H. Asquith}
\seealso{\code{\link{wolfCOP}}, \code{\link{wolfCOPsamc}}, \code{\link{wolfCOPtest_data_smlsam}}
}
\examples{
uv <- simCOP(n=62, cop=PSP, graphics=FALSE, seed=8)
# Approx p-value 0.001 PSP is not independence.
# There is no p.value(sample_le40) entry on the return because too large sample size.
wolfCOPtest(uv[,1], uv[,2], asuv=TRUE, aslist=FALSE, usepade=FALSE)
wolfCOPtest(uv[,1], uv[,2], asuv=TRUE, aslist=FALSE, usepade=TRUE )
# See how the p-values between the empirical approximations are about the same.

uv <- simCOP(n=62, cop=P,   graphics=FALSE, seed=8)
wolfCOPtest(uv[,1], uv[,2], asuv=TRUE, aslist=FALSE) # approx p-value 0.200***
# Note, the small sample p-values are NA for two previous examples because the sample
# size has become "large", but in the next example, we can inspect two p-values
# (1) by the Pearson type III distribution to the predicted L-moments for the sample
# size and (2) by the small-sample empirical tables in object wolfCOPtest_data_smlsam.
uv <- simCOP(n=13, cop=P,   graphics=FALSE, seed=81)
wolfCOPtest(uv[,1], uv[,2], asuv=TRUE, aslist=FALSE)
#   p.value(dist_pe3) p.value(sample_le40)  # four signif. figures kept here as
#          0.2550***            0.25144**   # simulations might tune up internals

\dontrun{
  # This example needs larger nsim but will show that the Sigma is performing close to
  # Spearman Rho, which helps establish the veracity of the algorithms. If the user were
  # to switch out the copula P and replace with CIRCcop (circular copula), then one will
  # see Sigma performing similarly but Rho failing because the CIRCcop is an example of
  # perfect dependence in a setting of independence. Use of much larger nsim for this
  # example will hone the apparent rejection to about 0.05 for P, but clearly we see
  # divergence in performance for CIRCcop.
  alpha <- 0.05; n <- 42; nsim <- 20000; cop <- P; p <- NULL
  set.seed(1)
  wolf   <- replicate(nsim, { wolfCOP(para=rCOP(n=n, cop, para=p),
                              as.sample=TRUE, nlarge=0, usefastgrid=TRUE) })
  wolfpv <- sapply(wolf, function(w) {
                   wolfCOPtest(w, n, asuv=TRUE, aslist=FALSE)[4]})
  message("  Schweizer-Wolff Sigma(n=", n, ") NULL=P rejection rate = ",
           sum(wolfpv < alpha) / nsim)
  set.seed(1)
  rhopv <- replicate(nsim, { uv <- rCOP(n=n, cop=cop, para=p);
                    cor.test(uv[,1], uv[,2], method="spearman")$p.value })
  message("           Spearman Rho(n=", n, ") NULL=P rejection rate = ",
           sum(rhopv  < alpha) / nsim)
  # For nsim <- 20000
  #       P : Schweizer-Wolff Sigma(n=42) NULL=P rejection rate = 0.04935
  #       P :          Spearman Rho(n=42) NULL=P rejection rate = 0.04805
  # CIRCcop : Schweizer-Wolff Sigma(n=42) NULL=P rejection rate = 0.28125
  # CIRCcop :          Spearman Rho(n=42) NULL=P rejection rate = 0.00100
  # Such example computations show that Rho does not "sense" the perfect dependence
  # of CIRCcop because its "correlation" is zero but has obvious dependence when
  # plotted. Therefore, Sigma has a much higher rejection rate that will go to unity
  # as the sample size increase.}

\dontrun{
  lmomco::plotlmrdia(lmomco::lmrdia(usrtrim=TRUE), nopoints=TRUE, xleg="topleft",
                    xlim=c(0.11,0.32), ylim=c(0.12,0.20), autolegend=TRUE,
                    expand.names=TRUE, autoaxes=TRUE, xaxs="i", yaxs="i", lwd.cex=1.3)
  xy <- st <- NULL
  ns <- sort(unique(c(10^seq(0,1,0.01), 10^seq(0, 4, by=0.02))))
  for(n in ns) {
    lmr <- wolfCOPtest(0, n, usepade=FALSE)$lmoms_logit_sigma; if(is.null(lmr)) next
    xy <- rbind(xy, data.frame(n=n, mu=lmr[1], lam2=lmr[2], tau3=lmr[3], tau4=lmr[4]))
  }
  xy <- xy[par()$usr[1] <= xy$tau3 & xy$tau3 <= par()$usr[2],]
  xy <- xy[par()$usr[3] <= xy$tau4 & xy$tau4 <= par()$usr[4],]
  lines(xy$tau3, xy$tau4, col="grey60", lwd=7)
  for(n in ns) {
    lmr <- wolfCOPtest(0, n, usepade=TRUE)$lmoms_logit_sigma; if(is.null(lmr)) next
    st <- rbind(st, data.frame(n=n, mu=lmr[1], lam2=lmr[2], tau3=lmr[3], tau4=lmr[4]))
  }
  st <- st[par()$usr[1] <= st$tau3 & st$tau3 <= par()$usr[2],]
  st <- st[par()$usr[3] <= st$tau4 & st$tau4 <= par()$usr[4],]
  lines(st$tau3, st$tau4, col="red1", lwd=2)
  txt <- c("Logit transform sampling Sigma distribution, wolfCOPtest(usepade=FALSE)",
           "Logit transform sampling Sigma distribution, wolfCOPtest(usepade=TRUE )")
  legend("bottomright", txt, lwd=c(7,2), col=c("grey50", "red1"), bty="n", cex=0.9) #}

\dontrun{
  # Here is a cross check in properties of the mathematics computing W.
  # The examples here also show the pass of 2-columns in x; so, a local copy
  # of a simulations is not needed before passing to wolfCOPtest().
  alpha <- 0.05; n <- 42; nsim <- 2000; cop <- P; p <- NULL
  set.seed(1)
  wolf1   <- replicate(nsim, { wolfCOP(para=rCOP(n=n, cop, para=p),
                              as.sample=TRUE, nlarge=0, usefastgrid=TRUE) })
  wolf1pv <- sapply(wolf1, function(w) { # here, sigma precomputed
                   wolfCOPtest(w, n, asuv=TRUE, aslist=FALSE)[4]})
  set.seed(1)
  wolf2pv <- replicate(nsim, { # here the "true" probabilities are known
                   wolfCOPtest(rCOP(n=n, cop, para=p), asuv=TRUE )$p.value[1]})
  set.seed(1)
  wolf3pv <- replicate(nsim, { # here the probabilities computed internally
                   wolfCOPtest(rCOP(n=n, cop, para=p), asuv=FALSE)$p.value[1]})
  message("  Schweizer-Wolff Sigma(n=", n, ") NULL: P rejection rate = ",
           sum(wolf1pv < alpha) / nsim)
  message("  Schweizer-Wolff Sigma(n=", n, ") NULL: P rejection rate = ",
           sum(wolf2pv < alpha) / nsim)
  message("  Schweizer-Wolff Sigma(n=", n, ") NULL: P rejection rate = ",
           sum(wolf3pv < alpha) / nsim) # }

\dontrun{
  # Here is a cross check of sorts between wolfCOPtest and the Independence test
  # available in the copula package. We use a Clayton copula with small bivariate
  # association and will see for this circumstance that wolfCOPtest out performs
  # copula::indepTest by just a small amount by rejecting the NULL more often.
  sample_size <- 100; ALPHA <- 0.05
  THECOP  <- CLcop; THEPARA <- 0.20; rhoCOP(cop=THECOP, para=0.20) # 0.1358466

  nsim <- 2000; digits <- as.integer( floor( log10(nsim) ) )
  J <- copula::indepTestSim(sample_size, 2, N=100000)

  W <- I <- NULL
  for(i in seq_len(nsim)) {
    if(is.null(W)) {
      uv <- copBasic::simCOP( sample_size, cop=THECOP, para=THEPARA )
      mtext("Copula simulation reference figure", line=0.5, font=2 )
    } else {
      uv <- copBasic::rCOP(   sample_size, cop=THECOP, para=THEPARA)
    }
    I <- c(I,   copula::indepTest(  uv,         J             )$pvalues[1])
    W <- c(W, copBasic::wolfCOPtest(uv[,1], uv[,2], asuV=TRUE )$p.value[1])
  }

  par(lend=2, las=1); hyphen <- "\u00ad"
  plot( sort( rank(I) ) / ( length(I) + 1 ), sort(I), type="l", xaxs="i", yaxs="i",
       xlab="Nonexceedance probability", lwd=3, col="grey50",
       ylab=paste0("P", hyphen, "value of respective test of independence"))
  lines(sort( rank(W) ) / ( length(W) + 1 ), sort(W), col="darkorange1")
  txt <- c(paste0("Independence test (copula::indepTest) has rejection rate = ",
                  round( sum(I < ALPHA) / nsim, digits=digits), ""),
           paste0("Schweizer-Wolff test (copBasic::wolfCOPtest) has rejection rate = ",
                  round( sum(W < ALPHA) / nsim, digits=digits), ""))
  legend("topleft", txt, bty="o", box.lty=0, bg=NA, inset=0.02, cex=0.9,
         lwd=c(3,1), col=c("grey50", "darkorange1"))
  mtext(paste0("Empirical distribution of p", hyphen, "values"), line=0.5, font=2) #}

\dontrun{
  # DATA CENSORING EXAMPLE ------------------------------------------------------------
  set.seed(10) # Example using the zmat feature for support of censoring by simulation.
  muslp <- 0.005; sdslp <- 0.019 # drift with exp used to keep parameter positive
  yy <- 1980:2021; n <- length(yy) # Simulated log-Pearson III annual peak streamflows
  qq <- 10^sapply(yy, function(y) { # put a small drift in the log10 mean and variation
              para <- lmomco::vec2par(c(3.5+(y-min(yy))*muslp,
                               exp(log(0.5)-(y-min(yy))*sdslp), 0.3), type="pe3")
                      lmomco::rlmomco(1, para) }) # flow
  zul <- zur <- rep(NA, n) # initialize the lower and upper limits for years
  zvl <- zvr <- rep(NA, n) # initialize the lower and upper limits for flows
  # NA years are not simulated and the data are used as is
  # Make some highly contrived censoring for the 3, 6, 9, 13, and 35+th observations.
  zul[ 3] <- yy[ 3] - 1.1; zur[ 3] <- yy[ 3] + 1.1
  zul[13] <- yy[13] - 1.5; zur[13] <- yy[13] + 2.4
  zvl[ 3] <- 10; zvr[3] <- qq[3]
  zvl[ 6] <- qq[6]; zvr[6] <- max(qq) + sqrt(.Machine$double.eps) # nudge above max
  zvl[ 9] <- 10^(log10(qq[9]) - 0.3); zvr[9] <- 10^(log10(qq[9]) + 0.2) # log-cycles
   qq[ 9] <- sqrt( zvl[9] * zvr[9] ) # geometric mean
  zvl[(n-6):n] <- 10; zvr[(n-6):n] <- qq[(n-6):n]

  xy <- cbind(yy, qq); zmat <- cbind(zul, zur, zvl, zvr)
  swolf <- wolfCOPtest(xy[,1], xy[,2], zmat=zmat, asuv=FALSE, statf=median, rndphi=500)
  ylim <- log10( range( qq ) ); ylim <- 10^c( floor( ylim[1] ), ceiling( ylim[2] ) )
  plot(  yy, qq, type="n", log="y", xlab="Water year", ylab="Streamflow", ylim=ylim)
  segments(x=zul, x1=zur, y=qq,  y1=qq,  col="darkgoldenrod3")
  segments(x=yy,  x1=yy,  y=zvl, y1=zvr, col="darkgoldenrod3")
  points(yy, qq,  bg="darkgoldenrod1",   col="darkgoldenrod4",  pch=21)
  naive <- wolfCOPtest(xy[,1], xy[,2], asuv=FALSE, ties.method="random")
  txt <- paste0("Sigma simulation (nsim=",
                prettyNum(length(swolf$rand_sigma), big.mark=","),   ") ",
                "p.value=",       round(swolf$p.value[1], digits=3), "; ",
                "Naive p.value=", round(naive$p.value[1], digits=3))
  mtext(txt, line=1) # Visually the point values tell us to expect independence to be
  # rejected, but when the data censoring is taken into account and simulation used,
  # we do not reject independence that is heavily influenced by the last seven data
  # values on the right being censored down through the bottom axis.
  swolf$rand_sigma <- sort(swolf$rand_sigma)
  p <- sapply(swolf$rand_sigma, function(k) { # distribution of p-values
                        n <- ifelse(swolf$sample_size <= 40, 2, 1)
                     wolfCOPtest(k, swolf$sample_size)$p.value[n] })
  swolf$rand_p.values <- p
  # plot(swolf$rand_sigma, swolf$rand_p.values, pch=16, col=grey(0, 0.05)) # }
}
\keyword{copula (hypothesis test)}
\keyword{Schweizer and Wolff Sigma}
\keyword{Schweizer and Wolff Sigma (hypothesis test)}
\keyword{generalized normal distribution}
\keyword{Pearson type III distribution}
\keyword{GNO (see generalized normal distribution)}
\keyword{PE3 (see Pearson type III distribution)}
\keyword{logit transform}
\concept{Hypothesis test for Schweizer and Wolff Sigma}
\keyword{censored data}

\encoding{utf8}
\name{LzCOPpermsym}
\alias{LzCOPpermsym}
\title{Maximum Asymmetry Measure (or Vector) of a Copula by Exchangeability (Permutation Symmetry)}
\description{
Compute a measure of maximum exchangable asymmetry of a copula \eqn{\mathbf{C}_\Theta} using \emph{Exchangeability} (\emph{permutation symmetry}) according to De Baets and De Meyer (2017) by
\deqn{\mu_{\infty\mathbf{C}}^{\mathrm{permsym}} = \mu_\infty^{\mathrm{permsym}} = 3 \times \mathrm{max}\bigl(\,|\,\mathbf{C}_\Theta(u,v) -
                       \mathbf{C}_\Theta(v,u)\,|\,\bigr)}

for \eqn{(u,v) \in \mathcal{I}^2}. De Baets and De Meyer (2017) comment that among many asymmetric metrics with copulas that \eqn{\mu_\infty^{\mathrm{permsym}}} is \dQuote{by far the most interesting} (De Baets and De Meyer, 2017, p. 36). The \eqn{3} multiplier in the definition ensures that \eqn{\mu_\infty^{\mathrm{permsym}} \in [0, 1]}. De Baets and De Meyer also conclude that permutation symmetry of random variables, in general, is not a desired property in statistical models, and those authors use the \eqn{\mu_\infty} notation in lieu of \eqn{L_\infty^{\mathrm{permsym}}} (see documentation related to \code{\link{LpCOPpermsym}} within \code{\link{hoefCOP}} of \emph{Hoeffding Phi}). The term \dQuote{Permutation-Mu} is used for this measure in \code{\link{copBasic-package}} and in similar contexts.
}
\usage{
LzCOPpermsym(cop=NULL, para=NULL, n=5E4,
             type=c("halton", "sobol", "torus", "runif"),
             as.abs=TRUE, as.vec=FALSE, as.mat=FALSE, plot=FALSE, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters, if and as needed, to pass to the copula;}
  \item{n}{The simulation size. The default seems sufficient for many practical applications but is suboptimal because the maximum operator in the definition is expected to potentially underestimate the true maximum. When a vector is returned, the default simulation size appears sufficient for many parameter estimation schemes;}
  \item{type}{The type of random number generator on \eqn{\mathcal{I}^2} for computing the maximum (apparent) (see argument \code{n}) or a vector of signed differences (see \bold{Details});}
  \item{as.abs}{A logical controlling whether the absolute value operation in the \eqn{\mu_\infty^{\mathrm{permsym}}} definition is used. This feature permits flexibility retaining the sign of asymmetry;}
  \item{as.vec}{A logical to disable the maximum operation but instead return the a vector of signed differences in the exchanged variables. If this argument is set true, then \code{as.abs} will be set false. The return of a vector of signed differences (still multiplied by \eqn{3}) could be useful in parameter estimation schemes with a similar vector from an \emph{empirical copula} (\code{\link{EMPIRcop}}) (see \bold{Details});}
  \item{as.mat}{A logical to disable the maximum operation (like \code{as.vec}), but instead return a matrix of the \eqn{\mathcal{I}^2} values with third column as the vector of signed differences. If this argument is set true, then \code{as.abs} will be set false;}
  \item{plot}{A logical to create a plot of the \eqn{\mathcal{I}^2} domain used in the simulation with a plot title showing the \code{type} argument setting;}
  \item{...}{Additional arguments to pass to support flexible implementation.}
}
\details{
\emph{EFFECT OF RANDOM NUMBER GENERATION}---Package \pkg{randtoolbox} provides for random number generation on forms different than simply simulating \emph{uniform independent random variables} for \eqn{\mathcal{I}^2}. The \emph{Halton}, \emph{Sobol}, and \emph{Torus} types are implemented. The \code{plot} argument is useful for the user to see the differences in how these generators canvas the \eqn{\mathcal{I}^2} domain.

The default is Halton, which visually appears to better canvas \eqn{\mathcal{I}^2} without the clumping that simple uniform random variables does and without the larger gaps of Sobol or Torus. Testing indicates that Halton might generally require the smallest simulation size of the others with simple uniform random variables potentially being the worst and hence such is not the default. Exceptions surely exist depending on the style of the asymmetry. Nevertheless, Halton, Sobol, and Torus produce more consistent estimation behavior with each having a monotone approach towards the true maximum than simple uniform random variables.

The following example is a useful illustration of an asymmetrical \emph{Clayton copula} (\eqn{\mathbf{CL}(u,v; \Theta)}, \code{\link{CLcop}}) by \emph{composition of a single copula} (\code{\link{composite1COP}}) with the theorical \eqn{\mu_\infty^{\mathrm{permsym}}} maxima computed by large sample simulation. A user might explore the effect of the random number generation by changing the \code{type} variable.
\preformatted{
  type <- "halton"
  para <- list(cop=CLcop, para=20, alpha=0.3, beta=0.1) # asymmetrical Clayton
  ti <- LzCOPpermsym(cop=composite1COP, para=para, n=2E6, type=type) # large
  ns <- as.integer( 10^seq(1, 4, by=0.05) ) # sequence of simulation sizes
  mi <- sapply(ns, function(n) { # produce vector of maxima for simulation size
               LzCOPpermsym(cop=composite1COP, para=para, n=n, type=type) })
  ylim <- range(c(0.06, mi, ti)) # vertical limits to ensure visibility
  plot(ns, mi, log="x", pch=21, bg=grey(0.9), ylim=ylim, main=type,
       xlab="Simulation size", ylab="Maximum asymmetry measure")
  abline(h=ti, lwd=3, col="seagreen") # large sample size estimate in green
}

\emph{RELATION TO ANOTHER DISTANCES}---The documentation for \code{\link{LpCOPpermsym}} lists a supremum definition \eqn{L_\infty^{\mathrm{permsym}}}, which is like \eqn{\mu_\infty^{\mathrm{permsym}}} but lacks the multiplier of 3. However, that documentation mentions a ratio of 1/3 being as upper bounds and hence the De Baets and De Meyer (2017) reasoning for the 3 multiplier to rescale \eqn{\mu_\infty^{\mathrm{permsym}} \in (0,1)}. The simple interrelations between the two functions are explored in the following example:

\preformatted{
  para <- c(30, 0.2, 0.95); n <- 5E4
  p <- 1
  mean(abs(LzCOPpermsym(cop=GHcop, para=para, n=n,
                          as.vec=TRUE)/3)^p)^(1/p)   # 0.01867929
           LpCOPpermsym(cop=GHcop, para=para, p=p)   # 0.01867940
  p <- 3
  mean(abs(LzCOPpermsym(cop=GHcop, para=para, n=n,
                          as.vec=TRUE)/3)^p)^(1/p)   # 0.02649376
           LpCOPpermsym(cop=GHcop, para=para, p=p)   # 0.02649317
}

The critical note is that \code{\link{LpCOPpermsym}} is the integral of the absolute differences in permuted differences across \eqn{\mathcal{I}^2}. Hence, it is an expectation. The \code{LzCOPpermsym} is difference because of the maximum of the differences. The computations in the example above show how the same information can be extracted from the two functions. De Baets and De Meyer (2017) do not make reference to raising and then rooting by the power \eqn{p} as shown. The examples here provide credence to the default setting of \code{n} (simulation size) for several significant figures of similarity.

\emph{COPULA PARAMETER ESTIMATION}---Parameter estimation using \emph{signed permutation asymmetry vector} can readily be accomplished, which means that we are interested in near-preservation of what permutation asymmetry might be present within the sample. In the self-contained example below, we will assume a parent of \emph{Gumbel--Hougaard} (\eqn{\mathbf{GH}(u,v; \Theta)}, \code{\link{GHcop}}) extended to asymmetry by using three parameters (\eqn{\Theta = (10, 0.8, 0.6)}. Imagine that we unfortunately have a very small sample size (\eqn{n = 100}) as \dQuote{hundred years of data.} The small sample size facilitates the use of the \emph{checkboard empirical copula} (\code{\link{EMPIRcop}}); the sample size is small enough that the checkerboard helps smooth through ties. The simulation size for \code{LzCOPpermsym} is set \dQuote{large} as presumed by the existing default. The premise here is that we desire to have near-precise matching to the sample Spearman Rho in conjunction with the permutation asymmetry.

\preformatted{
  # will need packages ** ABCoptim **, ** metaheuristicOpt **, ** pso **
  para <- c(10, 0.8, 0.6) # parameters of the parent
  nsam <- 100; seed <- 2  # sample size to fit and seed for the RNG
  nsim <- 2E4 # note a change from default n argument in LzCOPpermsym()
  as.vec <- TRUE          # set to FALSE to use just Permutation-Mu
  rhoP <- rhoCOP(cop=GHcop, para=para)          # parent Spearman Rho
  UVsS <- simCOP(cop=GHcop, para=para, n=nsam, seed=seed) # simulate a sample
  rhoS <- rhoCOP(as.sample=TRUE,     para=UVsS) # sample Spearman Rho
  infS <- LzCOPpermsym(cop=EMPIRcop, para=UVsS, n=nsim, type="halton",
                       as.vec=as.vec, ctype="checkerboard")
  # empirical copula used and returning signed asymmetry vector
  # transformation and re-transformation, GHcop paras >1; [0,1]; and [0,1]
  tparf <- function(par) c(exp(par[1]) + 1,  pnorm( par[2] ), pnorm( par[3] ))
  rparf <- function(par) c(log(par[1]  - 1), qnorm( par[2] ), qnorm( par[3] ))

  ofunc <- function(par, norho=FALSE, usetrans=FALSE) { # objective function
    if(usetrans) { mypara <- tparf(par) } else { mypara <- par }
    rhoT   <- rhoCOP(cop=GHcop, para=mypara)    # simulated Spearman Rho
    infT   <- LzCOPpermsym(cop=GHcop, para=mypara, n=nsim, type="halton",
                           as.vec=as.vec)
    err    <-    mean( (infT - infS)^2 )        # mean squared errors
    ifelse(norho, err, (rhoT - rhoS)^2 + err)   # with Spearman Rho or not
  } # norho and usetrans are unaccessible for metaOpt() call to come later
  # ofunc defaults are chosen to keep us from writing two objective functions

  init.par <- c(2, 0.5, 0.5) # initial parameters in real space
  tnit.par <- rparf(init.par); rt <- NULL # initial parameter guess
  try(  rt <- optim(tnit.par, ofunc, norho=FALSE, usetrans=TRUE) )
  # 3D optimization with unbounded parameters using transformation.
  if(is.null(rt)) stop("fatal, optim() returned NULL")
  # construct GHcop parameters from optimization with re-transformation
  sara <- tparf(rt$par)
  rhoT <- rhoCOP(cop=GHcop, para=sara)          # theoretical Spearman Rho
  UVsT <- simCOP(cop=GHcop, para=sara, n=nsam, seed=seed,  # same seed sim by
                 cex=0.3, pch=16, col="red", ploton=FALSE) # est. parameters
  # maximum likelihood
  mara <- mleCOP(UVsS, cop=GHcop, init.para=tnit.par, parafn=tparf)$para

  # metaheuristic methods follow and need lower and upper parameter bounds
  lb <- c(1, 0, 0); ub <- c(100, 1, 1)  # lower and upper parameter bounds
  # Artificial Bee Colony optimization
  # https://en.wikipedia.org/wiki/Artificial_bee_colony_algorithm
  be <- ABCoptim::abc_optim(init.par, ofunc, norho=FALSE, lb=lb, ub=ub)
  sara_bee <- be$par
  # Particle Swarm optimization
  # https://en.wikipedia.org/wiki/Particle_swarm_optimization
  ps <- pso::psoptim(init.par, ofunc, norho=FALSE,  lower=lb, upper=ub)
  sara_pso <- ps$par

  # Genetic Algorithm optimization
  # https://en.wikipedia.org/wiki/Genetic_algorithm
  rngv <- matrix(c(lb, ub), nrow=2, byrow=TRUE)
  ctrl <- list(numPopulation=30) # fairly slow running, so 30 for speed
  mo   <- metaheuristicOpt::metaOpt(ofunc, algorithm="GA",
                             numVar=3, rangeVar=rngv, control=ctrl)
  sara_alt <- mo$result

  level.curvesCOP(cop=GHcop, para=para)
  level.curvesCOP(cop=GHcop, para=mara,     ploton=FALSE, col="blue"    )
  level.curvesCOP(cop=GHcop, para=sara,     ploton=FALSE, col="red"     )
  level.curvesCOP(cop=GHcop, para=sara_bee, ploton=FALSE, col="orchid3" )
  level.curvesCOP(cop=GHcop, para=sara_pso, ploton=FALSE, col="seagreen")
  level.curvesCOP(cop=GHcop, para=sara_alt, ploton=FALSE, col="wheat"   )
}

Comparison of level curves between the known parent, the parameter estimation using function \code{LzCOPpermsym}, and the \emph{maximum likelihood} by \code{\link{mleCOP}} shows that signed asymmetry differences can be used for parameter estimation. One could use the maximum as in the definition, but for purposes of high-dimensional optimization, using the vector might be better to prevent local minima (less optimal solutions) being found if the \eqn{\mu_\infty^{\mathrm{permsym}}} was used. Because vectors of differences between empirical copula and the fitted copula are involved, measures of fit using such differences are expected to be more favorable to optimization than using \code{LzCOPpermsym} than perhaps maximum likelihood. Some measures of fit like RMSE (\code{\link{rmseCOP}}), for example, are often, smaller for the \code{sara} fitted parameters than for the \code{mara} fitted (maximum likelihood). The maximum likelihood approach should favor towards smaller AIC (\code{\link{aicCOP}}) and BIC (\code{\link{bicCOP}}). Finally, setting \code{as.vec <- FALSE}, re-running, and thus using \eqn{\mu_\infty^{\mathrm{permsym}}}, will likely show parameter estimates, visible through the level curves, that are much less favorable.

\preformatted{
  n <- 1E6; nm <- c("Parent", "MLE", "SARA", "ABC", "PSO", "GA")
  AIC <- c(aicCOP(UVsS, cop=GHcop, para=para    ),
           aicCOP(UVsS, cop=GHcop, para=mara    ),
           aicCOP(UVsS, cop=GHcop, para=sara    ),
           aicCOP(UVsS, cop=GHcop, para=sara_bee),
           aicCOP(UVsS, cop=GHcop, para=sara_pso),
           aicCOP(UVsS, cop=GHcop, para=sara_alt))
  RHO <- c(rhoCOP(      cop=GHcop, para=para    ),
           rhoCOP(      cop=GHcop, para=mara    ),
           rhoCOP(      cop=GHcop, para=sara    ),
           rhoCOP(      cop=GHcop, para=sara_bee),
           rhoCOP(      cop=GHcop, para=sara_pso),
           rhoCOP(      cop=GHcop, para=sara_alt)); names(RHO) <- nm
  LZP <- c(LzCOPpermsym(cop=GHcop, para=para,     n=n, type="halton"),
           LzCOPpermsym(cop=GHcop, para=mara,     n=n, type="halton"),
           LzCOPpermsym(cop=GHcop, para=sara,     n=n, type="halton"),
           LzCOPpermsym(cop=GHcop, para=sara_bee, n=n, type="halton"),
           LzCOPpermsym(cop=GHcop, para=sara_pso, n=n, type="halton"),
           LzCOPpermsym(cop=GHcop, para=sara_alt, n=n, type="halton"))
  names(AIC) <- nm; names(RHO) <- nm; names(LZP) <- nm
  print(AIC)
  print(RHO)
  print(LZP) # approximate results are shown in the table below
  #         Parent       MLE      SARA       ABC       PSO        GA
  # AIC  -928.5670 -916.2741 -939.8767 -939.8861 -939.8860 -937.8460
  # RHO  0.6132539 0.5433072 0.6058879 0.6058639 0.6058639 0.6058761
  # LZP  0.1076471 0.1000950 0.1186881 0.1187269 0.1187269 0.1211495
}
}
\value{
  A scalar value for the measure is returned or other as dictated by arguments.
}
\references{
De Baets, B., and De Meyer, H., 2017, Chapter 3---A look at copulas in a curved mirror: New York, Springer, ISBN 978--3--319--64221--5, pp. 33--47.
}
\author{W.H. Asquith}
\seealso{\code{\link{LpCOPpermsym}}, \code{\link{isCOP.permsym}}}
\examples{
LzCOPpermsym(cop=PSP)                            # 0, permutation symmetric
LzCOPpermsym(cop=GHcop, para=c(10, 0.9, 0.3))    # 0.17722861

# See also results of
# isCOP.permsym(cop=PSP)                         # TRUE
# isCOP.permsym(cop=GHcop, para=c(10, 0.9, 0.3)) # FALSE

\dontrun{
  sapply(1:4, function(r) { # Four rotations of a Galambos copula
    Lz <- LzCOPpermsym(cop=COP, para=list(cop=GLcop, para=2, reflect=r))
    UV <- simCOP(1000, cop=COP, para=list(cop=GLcop, para=2, reflect=r))
    mtext(paste0("Reflection ", r, " : Permutation-Mu =", Lz)); Lz })
  # [1] 0.00000000 0.00000000 0.07430326 0.07430326 }

\dontrun{
  # Example purpose = exploring sample sizes permutation symmetry metrics to
  # stablize by some examples of difference metrics for empirical copula that
  # are computed in about the simplist and most efficient manner at the expense
  # of the copying of UV inside the two EMPIRcop() calls, which could become
  # substantial if n ---> large, but we do not require additional book keeping
  # for u <- c(UV[,1], UV[,2]); v <- c(UV[,2], UV[,1]);
  # h <- EMPIRcop(u,v, para=UV) and then h[first half] - h[second half].
  DF <- NULL; para <- list(breve=0.2, para=10, cop=GHcop)
  for(n in as.integer(10^seq(log10(100), log10(50000), by=0.04))) {
    UV  <- simCOP(n, cop=breveCOP, para=para, graphics=FALSE)
    hh  <- EMPIRcop(UV[,1], UV[,2], para=UV) - EMPIRcop(UV[,2], UV[,1], para=UV)
    mad <- mean(abs(hh)); rmse <- sqrt(mean(hh^2)); max <- 3*max(hh)
    DF  <- rbind(DF, data.frame(n, mad, rmse, max))
  }
  ylim <- c(0, max(sapply(2:ncol(DF), function(i) max(DF[,i]))))
  plot( DF$n, DF$mad,  col="red", type="l", log="x", ylim=ylim)
  lines(DF$n, DF$rmse, col="seagreen")
  lines(DF$n, DF$max,  col="blue") # }
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{copula (symmetry)}
\keyword{copula (fitting)}
\keyword{Theory Examples (reflection/rotation)}
\keyword{copula (fitting) Metaheuristic Optimization}
\keyword{particle swarm optimization}
\keyword{artificial bee optimization}
\keyword{package ABCoptim (application of)}
\keyword{package metaheuristicOpt (application of)}
\keyword{package pso (application of)}
\keyword{package randtoolbox (application of)}

\encoding{utf8}
\name{simCOPmicro}
\alias{simCOPmicro}
\alias{simCOPv}
\title{Simulate V from U through a Copula by Numerical Derivative Method}
\description{
Perform bivariate simulation of random but coupled variables \eqn{V} from \eqn{U} through a copula (Nelsen, 2006, p. 32) by inversion of the numerical derivatives of the copula (\code{\link{derCOPinv}}, \code{\link{derCOPinv2}}). The method is more broadly known as \emph{conditional simulation method}. An elaborate implementation is available in \code{\link{simCOP}}, which unlike \code{simCOPmicro}, has provisions (default) for graphical support. The \code{simCOPmicro} function is intended to be a minimalist version for copula simulation, and such a version is useful for pedagogic purposes including \emph{conditional distributions}, \emph{conditional quantile functions}, and \emph{copula reflection} (see \bold{Note} and \code{\link{COP}}). An extended educational discussion of simulation using the conditional method is available in the \bold{Note} section of \code{\link{derCOPinv}}.

Some definitions are needed. The copula of \eqn{(1-U, 1-V)} is the \emph{survival copula} (\code{\link{surCOP}}) and is defined as
\deqn{\hat{\mathbf{C}}(u,v) = u + v - 1 + \mathbf{C}(1-u, 1-v)\mbox{,}}
whereas, following the notation of Joe (2014, pp. 271--272), the copula of \eqn{(1-U, V)} is defined as
\deqn{\acute{\mathbf{C}}(u,v) = v - \mathbf{C}(1-u, v)\mbox{, and}}
 the copula of \eqn{(U, 1-V)} is defined as
\deqn{\grave{\mathbf{C}}(u,v) = u - \mathbf{C}(u, 1-v)\mbox{.}}

Careful consideration of the nomenclature is necessary as confusion with the occurrences of \eqn{1-u} and \eqn{1-v} easily conflate meaning. The nomenclature for the \emph{survival copula} is more elaborately shown under \code{\link{surCOP}}. The difficulty is that the bivariate arguments to the \emph{survival copula} are \emph{exceedance probabilities}.

For simulation, again following the nomenclature of Joe (2014, p. 272), the conditional distribution functions (numerical derivatives; \code{\link{derCOP}} \eqn{\equiv} \eqn{\mathbf{C}_{2 \mid 1}(v \mid u)} and \code{\link{derCOP2}} \eqn{\equiv} \eqn{\mathbf{C}_{1 \mid 2}(u \mid v)}) can be written in terms of \eqn{\mathbf{C}(u \mid v) = \mathbf{C}_{2 \mid 1}(v \mid u)} as
\deqn{  \hat{\mathbf{C}}_{2 \mid 1}(v \mid u) = 1 - \mathbf{C}_{2 \mid 1}(1-v \mid 1-u)\mbox{,}}
\deqn{\acute{\mathbf{C}}_{2 \mid 1}(v \mid u) =     \mathbf{C}_{2 \mid 1}(  v \mid 1-u)\mbox{, and}}
\deqn{\grave{\mathbf{C}}_{2 \mid 1}(v \mid u) = 1 - \mathbf{C}_{2 \mid 1}(1-v \mid   u)\mbox{,}}
where the respective \code{"surv"}, \code{"acute"}, and \code{"grave"} are inverses (conditional quantile functions; inverses of numerical derivatives; \code{\link{derCOPinv}} \eqn{\equiv} \eqn{\mathbf{C}^{(-1)}_{2 \mid 1}(v \mid u)} and \code{\link{derCOPinv2}} \eqn{\equiv} \eqn{\mathbf{C}^{(-1)}_{1 \mid 2}(u \mid v)}) are
\deqn{  \hat{\mathbf{C}}^{(-1)}_{2 \mid 1}(t \mid u) = 1 - \mathbf{C}^{(-1)}_{2 \mid 1}(1-t \mid 1-u)\:\rightarrow\mbox{\ \code{"sur"},}}
\deqn{\acute{\mathbf{C}}^{(-1)}_{2 \mid 1}(t \mid u) =     \mathbf{C}^{(-1)}_{2 \mid 1}(  t \mid 1-u)\:\rightarrow\mbox{\ \code{"acute"}, and}}
\deqn{\grave{\mathbf{C}}^{(-1)}_{2 \mid 1}(t \mid u) = 1 - \mathbf{C}^{(-1)}_{2 \mid 1}(1-t \mid   u)\:\rightarrow\mbox{\ \code{"grave"},}}
where \eqn{t} is a uniformly distributed variable.

To clarify the seemingly clunky nomenclature---Joe (2014) does not provide \dQuote{names} for \eqn{\acute{\mathbf{C}}(u,v)} or \eqn{\grave{\mathbf{C}}(u,v)}---the following guidance is informative:\cr
\eqn{\mbox{}\quad\mbox{}}(1) \code{"surv"} or \eqn{\hat{\mathbf{C}}(u,v)} is a reflection of \eqn{U} and \eqn{V} on the horizontal \emph{and} vertical axes, respectively\cr
\eqn{\mbox{}\quad\mbox{}}(2) \code{"acute"} or \eqn{\acute{\mathbf{C}}(u,v)} is a reflection of \eqn{U} on the horizontal axis, and\cr
\eqn{\mbox{}\quad\mbox{}}(3) \code{"grave"} or \eqn{\grave{\mathbf{C}}(u,v)} is a reflection of \eqn{V} on the verical axis.\cr
The names \code{"acute"} and \code{"grave"} match those used in the \bold{Rd}-format math typesetting instructions.
}
\usage{
simCOPmicro(u, cop=NULL, para=NULL, seed=NULL,
               reflect=c("cop", "surv", "acute", "grave",
                           "1",    "2",     "3",     "4"), ...)
simCOPv(u, cop=NULL, para=NULL,
               reflect=c("cop", "surv", "acute", "grave",
                           "1",    "2",     "3",     "4"), ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction. The \code{runif()} function in \R can be used to drive conditional simulation using the \code{simCOPmicro} function (see \bold{Examples});}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters, if needed, to pass to the copula;}
  \item{seed}{The integer seed to pass immediately to \code{set.seed()} and setting it for the function \code{simCOPv} version will dispatch through the triple dots down to \code{simCOPmicro};}
  \item{reflect}{The reflection of the copula (see above) and the default \code{"cop"} or \code{"1"} is the usual copula definition.  The numbered values correspond, respectively, to the named values; and}
  \item{...}{Additional arguments to pass should they be needed.}
}
\value{
  Simulated value(s) of nonexceedance probability \eqn{v} are returned based on the nonexceedance probabilities \eqn{u} in argument \code{u}.
}
\note{
The advanced features of \code{simCOPmicro} permit simulation of the three permutations of \emph{variable reflection}. The first code simply produce four different \dQuote{copulas} based on the \emph{Gumbel--Hougaard} copula (\eqn{\mathbf{GH}(u,v; \Theta)}; \code{\link{GHcop}}), which has substantial upper tail dependency but no lower tail dependency for \eqn{\Theta = 2.512} as quantified by the \code{\link{taildepCOP}} function call.
\preformatted{
  U <- runif(1500); G <- 2.512; u <- 0.1; up <- 1-u; v <- 0.2; vp <- 1-v
  UV   <- data.frame(U, simCOPmicro(U, cop=GHcop, para=G                 ))
  sUsV <- data.frame(U, simCOPmicro(U, cop=GHcop, para=G, reflect="surv" ))
  sUV  <- data.frame(U, simCOPmicro(U, cop=GHcop, para=G, reflect="acute"))
  UsV  <- data.frame(U, simCOPmicro(U, cop=GHcop, para=G, reflect="grave"))
  taildepCOP(cop=GHcop, para=G) # lambdaL = 2e-05; lambdaU = 0.68224
}
The following code example will verify that the simulations produce values of \eqn{U} and \eqn{V} that are consistent with the \emph{empirical copula} (\code{\link{EMPIRcop}}) results as well as consistent with the variable reflections provided through the \code{\link{COP}} interface. Notice the combinations of nonexceedance and exceedance probabilities blended so that the two returned values for the four different copulas are numerically congruent.
\preformatted{
  c(EMPIRcop(u, v,  para=UV  ), COP(u, v,  cop=GHcop, para=G, reflect="cop"  ))
  c(EMPIRcop(up,vp, para=sUsV), COP(up,vp, cop=GHcop, para=G, reflect="surv" ))
  c(EMPIRcop(up,v,  para=sUV ), COP(up,v,  cop=GHcop, para=G, reflect="acute"))
  c(EMPIRcop(u, vp, para=UsV ), COP(u, vp, cop=GHcop, para=G, reflect="grave"))
}
The user can verify the reflections graphically using code such as this
\preformatted{
  xlab <- "PROBABILITY IN U"; ylab <- "PROBABILITY IN V"
  layout(matrix(c(1,2,3,4), 2, 2, byrow = TRUE));
  plot(UV,   xlab=xlab, ylab=ylab, pch=3, lwd=0.5, col=1)
    mtext("no reflection")
  plot(sUV,  xlab=xlab, ylab=ylab, pch=3, lwd=0.5, col=3)
    mtext("horizontal reflection")
  plot(UsV,  xlab=xlab, ylab=ylab, pch=3, lwd=0.5, col=4)
    mtext("vertical reflection")
  plot(sUsV, xlab=xlab, ylab=ylab, pch=3, lwd=0.5, col=2)
    mtext("double reflection")
}
in which inspection of tails exhibiting the dependency is readily seen on the four plots: upper right tail dependency (no reflection), upper left tail dependency (horizontal reflection), lower right tail dependency (vertical reflection), and lower left tail dependency (double reflection). It is important to stress that these descriptions and graphical depictions of single tail dependency are specific to the \eqn{\mathbf{GH}(u,v; \Theta)} copula chosen for the demonstration.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{simCOP}}}
\examples{
simCOPmicro(runif(1), cop=W  ) # Frechet lower-bound copula
simCOPmicro(runif(1), cop=P  ) # Independence copula
simCOPmicro(runif(1), cop=M  ) # Frechet upper-bound copula
simCOPmicro(runif(1), cop=PSP) # The PSP copula

\dontrun{
# Now let us get more complicated and mix two Plackett copulas together using the
# composite2COP as a "compositor." The parameter argument becomes more complex, but is
# passed as shown into composite2COP.
para <- list(cop1=PLACKETTcop,cop2=PLACKETTcop, alpha=0.3,beta=0.5, para1=0.1,para2=50)
simCOPmicro(runif(5), cop=composite2COP, para=para) #}

\dontrun{
# Now let us implement "our" own version of features of simCOP() but using
# the micro version to manually create just the simulation vector of V.
U <- runif(1500)
UV <- data.frame(U, simCOPmicro(U, cop=N4212cop, para=4))
plot(UV, xlab="PROBABILITY IN U", ylab="PROBABILITY IN V", pch=3, col=2) #}
}
\keyword{copula (simulation)}
\keyword{copula (conditional distribution)}
\keyword{copula (conditional quantile function)}
\keyword{Theory Examples (reflection/rotation)}

\encoding{utf8}
\name{M_N5p12b}
\alias{M_N5p12b}
\title{Shuffles of Upper-Bound Copula, Example 5.12b of Nelsen's Book}
\description{
Compute shuffles of \emph{\enc{Fréchet}{Frechet}--Hoeffding upper-bound copula} (Nelsen, 2006, p. 173), which is defined by partitioning \eqn{\mathbf{M}} within \eqn{\mathcal{I}^2} into \eqn{n} subintervals:
\deqn{\mathbf{M}_n(u,v) = \mathrm{min}\biggl(u-\frac{k-1}{n}, v-\frac{n-k}{n} \biggr)}
for points within the partitions
\deqn{(u,v) \in \biggl[\frac{k-1}{n}, \frac{k}{n}\biggr]\times \biggl[ \frac{n-k}{n}, \frac{n-k+1}{n}\biggr]\mbox{,\ }k = 1,2,\cdots,n}
and for points otherwise out side the partitions
\deqn{\mathbf{M}_n(u,v) = \mathrm{max}(u+v-1,0)\mbox{.}}
The support of \eqn{\mathbf{M}_n} consists of \eqn{n} line segments connecting coordinate pairs \eqn{\{(k-1)/n,\, (n-k)/n\}} and \eqn{\{k/n,\, (n-k+1)/n\}} as stated by Nelsen (2006). It is useful that Nelsen stated such as this helps to identify that Nelsen's typesetting of the terms in the second square brackets---the \eqn{V} direction---is reversed from that shown in this documentation. The \emph{Spearman Rho} (\code{\link{rhoCOP}}) is defined by \eqn{\rho_\mathbf{C} = (2/n^2) - 1}, and the \emph{Kendall Tau} (\code{\link{tauCOP}}) by \eqn{\tau_\mathbf{C} = (2/n) - 1}.
}
\usage{
M_N5p12b(u, v, para=1, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A positive integer \eqn{n \in 1, 2, \cdots}; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{M}}, \code{\link{ORDSUMcop}}, \code{\link{W_N5p12a}}}
\examples{
M_N5p12b(0.4, 0.6, para=3)

\dontrun{
  # Nelsen (2006, exer. 5.12b, p. 173, fig. 5.3b)
  UV <- simCOP(1000, cop=M_N5p12b, para=4) #}
}
\keyword{copula (formulas)}
\keyword{copula (limits/bounds)}
\concept{Shuffle of Frechet upper-bound copula}
\concept{Shuffle of Frechet-Hoeffding upper-bound copula}
\concept{Shuffle of Frechet--Hoeffding upper-bound copula}
\concept{Shuffle of Frechet upper bounds}
\concept{Shuffle of Frechet-Hoeffding upper bounds}
\concept{Shuffle of Frechet--Hoeffding upper bounds}
\keyword{copula}
\keyword{shuffle}
\keyword{Nelsen (2006) Examples and Exercises}

\encoding{utf8}
\name{copBasic.fitpara}
\alias{copBasic.fitpara.beta}
\title{A Single or Multi-Parameter Optimization Engine (Beta Version)}
\description{
An example of a general implementation of a parameter optimization scheme using core features of the \pkg{copBasic} package. Because of the general complexity of the objectives for this function, the interface shown here is considered an \dQuote{beta version} and nomenclature is subject to possibly sweeping changes in the future.
}
\usage{
copBasic.fitpara.beta(uv=NULL, popstat=NULL, statf=NULL, cop=NULL,
                      paradim=1, interval=NULL, par.init=NULL, ...)
}
\arguments{
  \item{uv}{An \R two column \code{matrix} or \code{data.frame} of a sample of nonexceedance probabilities \eqn{u} and \eqn{v};}
  \item{popstat}{The population statistic(s) that will be used if \code{uv} is \code{NULL};}
  \item{statf}{A function responsible at the minimum for computation of the theoretical values of the population statistic(s) that the optimization will revolve around; This function, if supporting an \code{as.sample} interface (\emph{e.g.} \code{\link{hoefCOP}}) and if \code{uv} has been provided, will be dispatched to compute the population statistic(s);}
  \item{cop}{A copula function that is passed along to \code{statf} though of course the \code{statf} function can decide whether to use this argument or not;}
  \item{paradim}{The dimension of the parameters. In reality, the default triggers uni-dimensional root solving using the \code{uniroot()} function in \R or otherwise the \code{optim()} function in \R is used for multi-dimensional minimization with subtle changes in setup (see source code). Alternative logic could be have been used but it is felt that this is the most logical for future adaptations;}
  \item{interval}{The \code{interval} argument by the same name for the \code{uniroot()} function;}
  \item{par.init}{The initial parameter vector for the \code{optim()} function; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  A vector of the values for the parameter variable is returned
}
\note{
\emph{One-Parameter Optimization}---A demonstration of one-dimensional parameter optimization using the \emph{Gini Gamma} (\code{\link{giniCOP}}), which is a measure of bivariate association. There is no native support for Gini Gamma (and most of the other measures of association) in regards to being a parameter estimator at the copula function interface level in \pkg{copBasic}. (A converse example is one provided internally by the \code{\link{GHcop}} copula.)
\preformatted{
  set.seed(24); n <- 230 # sample size to draw from Galambos copula but a
  # different copula was chosen for the fitting.
  sampleUV <- simCOP(n=n, cop=GLcop, para=1.5) # a random sample
  para <- copBasic.fitpara.beta(uv=sampleUV, statf=giniCOP,
                                interval=c(.1,200), cop=HRcop) # 1.959521
}

\emph{Three-Parameter Optimization}---A demonstration of multi-dimensional parameter optimization using the Gini Gamma (\code{\link{giniCOP}}), \emph{Nu-Skew} (\code{\link{nuskewCOP}}), and \emph{Nu-Star} (\code{\link{nustarCOP}}). This is substantially more complicated to implement. The \emph{\enc{Hüsler}{Husler}--Reiss copula} (\code{\link{HRcop}}) is chosen both as part of the sample simulation for the study as well as the copula as part of the modeling.  Using composition by the \code{\link{composite1COP}}, first establish a parent three parameter copula and simulate from it to create a bivariate sample in \code{sampleUV} that will be used for demonstration. A standard normal variate graphic of the simulation is generated by \code{\link{simCOP}} as well---later, additional results will be superimposed.
\preformatted{
  n <- 610; set.seed(1999) # Seed value will be used again (see below)
  pop.para <- list(cop1=HRcop, para1=4, alpha=0.14, beta=0.35)
  sampleUV <- simCOP(n=n, cop=composite1COP, para=pop.para, col=3, snv=TRUE)
}

A custom objective function \code{objfunc} to serve as the \code{statf} for the \code{copBasic.fitpara.beta} call. The objective function has the \code{as.sample} interface (\emph{e.g.} \code{\link{giniCOP}}) implemented for sample estimation. The most subtle feature of function presumably is the use of the \code{pnorm()} function in \R for the \eqn{\alpha} and \eqn{\beta} parameters to keep each parameter in the range \eqn{\alpha, \beta \in (0,1)}. Another subtly, which affects the choice of other copulas from \code{\link{HRcop}}, is how the parameter range of \eqn{\Theta} (the \code{para[1]} variable) is controlled---here the parameter remains untransformed but the lower domain edge is truncated by the return of infinity (\code{return(Inf)}). The \code{getstat} argument is only for short circuiting the objective so that \code{objfunc} can be used to compute the three statistics after the optimal parameters are found.
\preformatted{
  "objfunc" <- function(para, stat=NA, as.sample=FALSE, getstat=FALSE, ...) {
      if(as.sample) {
         return(c(  giniCOP(para=para, as.sample=TRUE),
                  nuskewCOP(para=para, as.sample=TRUE),
                  nustarCOP(para=para, as.sample=TRUE)))
      }
      para[1]   <- ifelse(para[1] < 0, return(Inf), para[1]) # edge check
      para[2:3] <-  pnorm(para[2:3]) # detransform
      para <- list(cop1=HRcop, para1=para[1], alpha=para[2], beta=para[3])
      hp <- c(  giniCOP(composite1COP, para),
              nuskewCOP(composite1COP, para),
              nustarCOP(composite1COP, para))
      if(getstat) return(hp) # short circuit to get the statistics out
      dv <- stat; dv[dv == 0] <- 1 # changing dv "adapts" the error to
      return(sqrt(sum(((stat-hp)/dv)^2))) # trap division by zero
  }
}

The parameter estimation proceeds in the following code. The sample statistics (or \code{target.stats}) are computed and subsequently passed to the optimization using the \code{popstat} argument. Notice also the use of the \code{qnorm()} function in \R to transform the initial guess \eqn{\alpha = \beta = 1/2} into a domain more easily handled by the optimizer (\code{optim()} function in \R). The transformed optimal parameters are computed, and then the values of the three statistics for the fit are computed. Lastly, a \pkg{copBasic} parameter object \code{fit.para} is created, which can be used for later comparisons.
\preformatted{
  txt <- c("GiniGamma", "NuSkew", "NuStar")
  target.stats <- objfunc(sampleUV, as.sample=TRUE); names(target.stats) <- txt
  raw.fit.para <- copBasic.fitpara.beta(popstat=target.stats, statf=objfunc,
         par.init=c(1, qnorm(0.5), qnorm(0.5)), cop=composite1COP, paradim=3)
  fit.stats <- objfunc(raw.fit.para, getstat=TRUE); names(fit.stats) <- txt
  fit.para <- list(cop1=HRcop, para1=raw.fit.para[1],
                   alpha=pnorm(raw.fit.para[2]), beta=pnorm(raw.fit.para[3]))
}

The optimization is completed. It is informative to see what the simulation of the fitted copula looks like. Note: this particular example suffers from identifiability problems between the parameters---meaning that local minima exist or that satisfactory solutions using different parameters than used to generate the random sample can occur. The same seed is used so that one-to-one comparison of points can visually be made with the \code{\link{simCOP}} function call.
\preformatted{
  set.seed(1999) # This value will be used again (see below)
  sampleUV <- simCOP(n=n, cop=composite1COP, para=fit.para,
                ploton=FALSE, pch=16, cex=0.5, col=2, snv=TRUE) # red dots
}

The visual comparison is qualitative. The tabular comparison of the sample statistics to those of the fitted model shows that perhaps an acceptable local minima has been attained in terms of \dQuote{fit} but the subsequent comparison of the parameters of the population used to generate the sample and those of the fitted model seemingly depart substantially in the \eqn{\alpha \rightarrow 0} parameter of the copula composition. The tail dependency parameters are similar, but further goodness-of-fit check is not made.
\preformatted{
                       #   GiniGamma        NuSkew       NuStar
  print(target.stats)  #   0.5219027    -0.1940361    0.6108319
  print(fit.stats)     #   0.5182858    -0.1938848    0.6159566

                          # Parameter  Alpha       Beta
  print(ls.str(pop.para)) #      4.00   0.14      0.350  # given
  print(ls.str(fit.para)) #     11.2    0.187     0.427  # one solution

                                               # Tail Dependency Parameters
  taildepCOP(cop=composite1COP, para=pop.para) # lower=0 : upper=0.5838
  taildepCOP(cop=composite1COP, para=fit.para) # lower=0 : upper=0.5714(est.)
}

The demonstration ends with the comparison of the two asymmetrical density contours.
\preformatted{
  densityCOPplot(cop=composite1COP, para=pop.para, contour.col=3)
  densityCOPplot(cop=composite1COP, para=fit.para, contour.col=2,
                                    ploton=FALSE)
}
}
\author{W.H. Asquith}
\examples{
# See the Note section
}
\keyword{copula (fitting)}
\keyword{copula (goodness-of-fit)}
\keyword{copula (inference)}
\keyword{inference}
\keyword{goodness-of-fit}

\encoding{utf8}
\name{EMPIRgridder2}
\alias{EMPIRgridder2}
\title{Derivatives of the Grid of the Bivariate Empirical Copula for U with respect to V}
\description{
Generate derivatives of \eqn{U} with respect to \eqn{V} of a gridded representation of the \emph{bivariate empirical copula} (see \code{\link{EMPIRcop}}).  This function is the empirical analog to \code{\link{derCOP2}}.
}
\usage{
EMPIRgridder2(empgrid=NULL, ...)
}
\arguments{
  \item{empgrid}{The grid from \code{\link{EMPIRgrid}}; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  The gridded values of the derivatives of the bivariate empirical copula.
}
\author{W.H. Asquith}
\seealso{\code{\link{EMPIRcop}}, \code{\link{EMPIRcopdf}}, \code{\link{EMPIRgrid}}, \code{\link{EMPIRgridder}}}
\examples{
# See examples under EMPIRgridder
}
\keyword{empirical copula}
\keyword{empirical copula (derivatives)}
\keyword{derivative}
\keyword{copulatic surface}
\keyword{gridded empirical copula}

\encoding{utf8}
\name{coCOP}
\alias{coCOP}
\title{The Co-Copula Function}
\description{
Compute the \emph{co-copula (function)} from a copula (Nelsen, 2006, pp. 33--34), which is defined as

\deqn{\mathrm{Pr}[U > u \mathrm{\ or\ } V > v] = \mathbf{C}^{\star}(u',v') = 1 - \mathbf{C}(u',v')\mbox{,}}

where \eqn{\mathbf{C}^{\star}(u',v')} is the co-copula and \eqn{u'} and \eqn{v'} are exceedance probabilities and are equivalent to \eqn{1-u} and \eqn{1-v} respectively. The co-copula is the expression for the probability that either \eqn{U > u} \bold{or} \eqn{V > v} when the arguments to \eqn{\mathbf{C}^{\star}(u',v')} are exceedance probabilities, which is unlike the \emph{dual of a copula (function)} (see \code{\link{duCOP}}) that provides \eqn{\mathrm{Pr}[U \le u \mathrm{\ or\ } V \le v]}.

The co-copula is a function and not in itself a copula. Some rules of copulas mean that \eqn{\mathbf{C}(u,v) + \mathbf{C}^{\star}(u',v') \equiv 1} or in \pkg{copBasic} syntax that the functions \code{COP(u,v)} + \code{coCOP(u,v)} equal unity if the \code{exceedance} argument to \code{coCOP} is set to \code{FALSE}.

The function \code{coCOP} gives \dQuote{risk} against failure if failure is defined as either hazard source \eqn{U} or \eqn{V} occuring by themselves or if both occurred at the same time. Expressing this in terms of an annual probability of occurrence (\eqn{q}), one has
\deqn{q = 1 - \mathrm{Pr}[U > u \mathrm{\ or\ } V > v] =  \mathbf{C}^{\star}(u',v') \mbox{\ or}}
in \R code \code{q <- coCOP(u,v, exceedance=FALSE, ...)}. So, in yet other words and as a mnemonic: \emph{A co-copula is the probabililty of exceedance if the hazard sources \bold{collaborate} or \bold{cooperate} to cause failure.} Also, \eqn{q} can be computed by \code{q <- }\code{coCOP(1 - u, 1 - v,} \code{exceedance=TRUE, ...)}.
}
\usage{
coCOP(u, v, cop=NULL, para=NULL, exceedance=TRUE, ...)
}
\arguments{
  \item{u}{Exceedance probability (\eqn{u' = 1-u}) in the \eqn{X} direction;}
  \item{v}{Exceedance probability (\eqn{v' = 1-v}) in the \eqn{Y} direction;}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{exceedance}{A logical controlling the probability direction. Are \code{u} and \code{v} values given really \eqn{u'} and \eqn{v'}, respectively? If \code{FALSE}, then the complements of the two are made internally and the nonexceedances can thus be passed; and}
  \item{...}{Additional arguments to pass to the copula.}
}
\value{
  The value(s) for the co-copula are returned.
}
\note{
The author (Asquith) finds the use of exceedance probabilities delicate in regards to Nelsen's notation. The \code{coCOP} function and \code{\link{surCOP}} have the \code{exceedance} argument to serve as a reminder that the co-copula as defined in the literature uses \emph{exceedance probabilities} as its arguments, although the arguments as code \code{u} and \code{v} do not mimic the overline nomenclature (\eqn{\,\overline{\cdots}\,}) of the exceedance (survival) probabilities.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{COP}}, \code{\link{surCOP}}, \code{\link{duCOP}}}
\examples{
u <- 1 - runif(1); v <- 1 - runif(1) # as exceedance, in order to reinforce the
# change to exceedance instead of nonexceedance that otherwise dominates this package
message("Exceedance probabilities u' and v' are ", u, " and ", v)
coCOP(u,v,cop=PLACKETTcop, para=10) # Positive association Plackett

# computation using  manual  manipulation to nonexceedance probability
1 - COP(cop=PSP,(1-u),(1-v))
# computation using internal manipulation to nonexceedance probability
  coCOP(cop=PSP,   u,    v)

# Next demonstrate COP + coCOP = unity.
"MOcop.formula" <- function(u,v, para=para, ...) { # Marshall-Olkin copula
   alpha <- para[1]; beta <- para[2]; return(min(v*u^(1-alpha), u*v^(1-beta)))
}
"MOcop" <- function(u,v, ...) { asCOP( u,  v, f=MOcop.formula, ...) }
u <- 0.2; v <- 0.75; ab <- c(1.5, 0.3)
COP(u,v, cop=MOcop, para=ab) + coCOP(1-u,1-v, cop=MOcop, para=ab) # UNITY
}
\keyword{copula operator}
\keyword{joint probability}

\encoding{utf8}
\name{aicCOP}
\alias{aicCOP}
\title{Akaike Information Criterion between a Fitted Coupla and an Empirical Copula}
\description{
Compute the \emph{Akaike information criterion} (AIC) \eqn{\mathrm{AIC}_\mathbf{C}} (Chen and Guo, 2019, p. 29), which is computed using \emph{mean square error} \eqn{\mathrm{MSE}_\mathbf{C}} as

\deqn{\mathrm{MSE}_\mathbf{C} = \frac{1}{n}\sum_{i=1}^n \bigl(\mathbf{C}_n(u_i,v_i) - \mathbf{C}_{\Theta_m}(u_i, v_i)\bigr)^2\mbox{ and}}
\deqn{\mathrm{AIC}_\mathbf{C} = 2m + n\log(\mathrm{MSE}_\mathbf{C})\mbox{,}}

where \eqn{\mathbf{C}_n(u_i,v_i)} is the \emph{empirical copula} (empirical joint probability) for the \eqn{i}th observation, \eqn{\mathbf{C}_{\Theta_m}(u_i, v_i)} is the fitted copula having \eqn{m} parameters in \eqn{\Theta}. The \eqn{\mathbf{C}_n(u_i,v_i)} comes from \code{\link{EMPIRcop}}. The \eqn{\mathrm{AIC}_\mathbf{C}} is in effect saying that the best copula will have its joint probabilities plotting on a 1:1 line with the empirical joint probabilities, which is an \eqn{\mathrm{AIC}_\mathbf{C} = -\infty}. From the \eqn{\mathrm{MSE}_\mathbf{C}} shown above, the root mean square error \code{\link{rmseCOP}} and Bayesian information criterion (BIC) \code{\link{bicCOP}} can be computed. These goodness-of-fits can assist in deciding one copula favorability over another, and another goodness-of-fit using the absolute differences between \eqn{\mathbf{C}_n(u,v)} and \eqn{\mathbf{C}_{\Theta_m}(u, v)} is found under \code{\link{statTn}}.
}
\usage{
aicCOP(u, v=NULL, cop=NULL, para=NULL, m=NA, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction; If not given, then a second column from argument \code{u} is attempted;}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
    \item{m}{The number of parameters in the copula, which is usually determined by length of \code{para} if \code{m=NA}, but some complex compositions of copulas are difficult to authoritatively probe for total parameter lengths and mixing coefficients; and}
  \item{...}{Additional arguments to pass to either copula (likely most commonly to the empirical copula).}
}
\value{
  The value for \eqn{\mathrm{AIC}_\mathbf{C}} is returned.
}
\references{
Chen, Lu, and Guo, Shenglian, 2019, Copulas and its application in hydrology and water resources: Springer Nature, Singapore, ISBN 978--981--13--0574--0.
}
\author{W.H. Asquith}
\seealso{\code{\link{EMPIRcop}}, \code{\link{bicCOP}}, \code{\link{rmseCOP}}
}
\examples{
\dontrun{
  S <- simCOP(80, cop=GHcop, para=5) # Simulate some probabilities, but we
  # must then treat these as data and recompute empirical probabilities.
  U <- lmomco::pp(S$U, sort=FALSE); V <- lmomco::pp(S$V, sort=FALSE)
  # The parent distribution is Gumbel-Hougaard extreme value copula, but in practical
  # applications, we do not know that. Say we speculate that perhaps the Galambos extreme
  # value might be the parent; maximum likelihood is used to fit the single parameter.
  pGL  <- mleCOP(  U,V, cop=GLcop, interval=c(0, 20))$par
  aics <- c(aicCOP(U,V, cop=GLcop, para=pGL), aicCOP(U,V, cop=P), aicCOP(U,V, cop=PSP))
  names(aics) <- c("GLcop", "P", "PSP")
  print(aics) # We will see that the first AIC is the smallest because the
  # Galambos has the nearest overall behavior than the P and PSP copulas.}
}
\keyword{copula (goodness-of-fit)}
\keyword{copula (inference)}
\keyword{inference}
\keyword{goodness-of-fit}
\keyword{Akaike Information Criterion}

\encoding{utf8}
\name{P}
\alias{P}
\title{The Product (Independence) Copula}
\description{
Compute the \emph{product copula} (Nelsen, 2006, p. 12), which is defined as
\deqn{\mathbf{\Pi}(u,v) = uv\mbox{.}}
This is the copula of statistical independence between \eqn{U} and \eqn{V} and is sometimes referred to as the \emph{independence copula}.  The two extreme antithesis copulas are the \emph{\enc{Fréchet}{Frechet}--Hoeffding upper-bound} (\code{\link{M}}) and \emph{\enc{Fréchet}{Frechet}--Hoeffding lower-bound} (\code{\link{W}}) copulas.
}
\usage{
P(u, v, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{M}}, \code{\link{W}}, \code{\link{rhoCOP}}, \code{\link{lcomCOP}}, \code{\link{PLcop}} }
\examples{
P(c(0.4, 0, 1), c(0, 0.6, 1))

\dontrun{
  # PERFECT INDEPENDENCE
  n <- 100000    # large sample size, L-comoments are zero
  lcomCOP(cop=P) # zeros (numerical integration)
  UV <- simCOP(n=n, cop=P, graphics=FALSE)
  lmomco::lcomoms2(UV, nmom=4)
  # The following are Taus_r^{12} and Taus_r^{21}
  # L-corr:        0.00265 and  0.00264 ---> ZERO
  # L-coskew:     -0.00121 and  0.00359 ---> ZERO
  # L-cokurtosis:  0.00123 and  0.00262 ---> ZERO

  # CONTRAST TO MODEST POSITIVE CORRELATION
  rho   <- 0.6; # Spearman Rho
  theta <- PLACKETTpar(rho=rho)  # Theta = 5.115658
  lcomCOP(cop=PLcop, para=theta) #   Rho = 0.6 and rest zeros
  UV <- simCOP(n=n, cop=PLcop, para=theta, graphics=FALSE)
  lmomco::lcomoms2(UV, nmom=4)
  # The following are Taus_r^{12} and Taus_r^{21}
  # L-corr       0.60303    and  0.60303    ---> Spearman Rho
  # L-coskews    4.5339e-05 and -0.00124011 ---> ZERO
  # L-cokurtosis 0.00088549 and  0.00129812 ---> ZERO
  # Pearson R == Spearman Rho for UV L-comoments }
}
\keyword{copula (formulas)}
\concept{independence copula}
\concept{product copula}
\keyword{copula}
\keyword{Pearson correlation coefficient}
\keyword{L-comoments}
\keyword{Lcomoments}
\keyword{package lmomco (application of)}

\encoding{utf8}
\name{asCOP}
\alias{asCOP}
\title{Wrapper on a User-Level Formula to Become a Copula Function}
\description{
This function is intended to document and then to extend a simple API to end users to aid in implementation of other copulas for use within the \pkg{copBasic} package. There is no need or requirement to use \code{asCOP} for almost all users. However, for the mathematical definition of some copulas, the \code{asCOP} function might help considerably. This is because there is a need for special treatment of \eqn{u} and \eqn{v} vectors of probability as each interacts with the vectorization implicit in \R. The special treatment is needed because many copulas are based on the operators such as \code{min()} and \code{max()}. When numerical integration used by the \code{integrate()} function in \R in some copula operators, such as \code{\link{tauCOP}} for the \emph{Kendall Tau} of a copula, special accommodation is needed related to the inherent vectorization in \R and how \code{integrate()} works.

Basically, the problem is that one can not strictly rely in all circumstances on what \R does in terms of value recycling when \eqn{u} and \eqn{v} are of unequal lengths. The source code is straightforward. Simply put, if lengths of \eqn{u} and \eqn{v} are unity, then there is no concern, and even if the length of \eqn{u} (say) is unity and \eqn{v} is 21, then recycling of \eqn{u} would often be okay. The real danger is when \eqn{u} and \eqn{v} have unequal lengths and those lengths are each greater than unity---the \R treatment can not be universally relied upon with the various numerics herein involving optimization and nested numerical integration.

The example shows how a formula definition of a copula that is not a copula already implemented by \pkg{copBasic} is set into a function \code{deltacop} and then used inside another function \code{UsersCop} that will be the official copula that is compatible with a host of functions in \pkg{copBasic}. The use of \code{asCOP} provides the length check necessary on \eqn{u} and \eqn{v}, and the argument \code{...} provides optional parameter support should the user's formula require more settings.
}
\usage{
asCOP(u, v, f=NULL, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{f}{A function for which the user desires to make as a copula; and}
  \item{...}{Additional arguments to pass to the function \code{f} (such as parameters, if needed, for the copula in the form of a list).}
}
\value{
  The value(s) for the copula are returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{COP}}}
\examples{
\dontrun{
  # Concerning Nelsen (2006, exer. 3.7, pp. 64--65)
  "trianglecop" <- function(u,v, para=NULL, ...) {
     # If para is set, then the triangle is rotated 90d clockwise.
     if(! is.null(para) && para == 1) { t <- u; u <- v; v <- t }
     if(length(u) > 1 | length(v) > 1) stop("only scalars for this function")
     v2<-v/2; if(0   <= u    &  u   <= v2 & v2 <= 1/2) { return(u    )
     } else   if(0   <= v2   & v2   <  u  &  u < 1-v2) { return(v2   )
     } else   if(1/2 <= 1-v2 & 1-v2 <= u  &  u <= 1  ) { return(u+v-1)
     } else { stop("should not be here in logic") }
  }
  "UsersCop" <- function(u,v, ...) { asCOP(u,v, f=trianglecop, ...) }
  n=20000; UV <- simCOP(n=n, cop=UsersCop)
  # The a-d elements of the problem now follow:
  # (a) Pr[V = 1 - |2*U -1|] = 1 and Cov(U,V) = 0; so that two random variables
  # can be uncorrelated but each is perfectly predictable from the other
  mean(UV$V - (1 - abs(2*UV$U -1))) # near zero; Nelsen says == 0
  cov(UV$U, UV$V)                   # near zero; Nelsen says == 0

  # (b) Cop(m,n) = Cop(n,m); so that two random variables can be identically
  # distributed, uncorrelated, and not exchangeable
  EMPIRcop(0.95,0.17, para=UV) # = A
  EMPIRcop(0.17,0.95, para=UV) # = B; then A != B

  # (c) Pr[V - U > 0] = 2/3; so that two random variables can be identically
  # distributed, but their difference need not be symmetric about zero
  tmp <- (UV$V - UV$U) > 0
  length(tmp[tmp == TRUE])/n # about 2/3; Nelsen says == 2/3
  # the prior two lines yield about 1/2 for independence copula P()

  # (d) Pr[X + Y > 0] = 2/3; so that uniform random variables on (-1,1) can each
  # be symmetric about zero, but their sum need not be.
  tmp <- ((2*UV$V - 1) + (2*UV$U - 1)) > 0
  length(tmp[tmp == TRUE])/n # about 2/3; Nelsen says == 2/3 }

\dontrun{
  # Concerning Nelsen (2006, exam. 3.10, p. 73)
  "shufflecop" <- # assume scalar arguments for u and v
  function(u,v, para, ...) {
     m <- para$mixer; subcop <- para$subcop
     if(is.na(m) | m <= 0 | m >= 1) stop("m ! in [0,1]")
     if(u <= m) { return(    subcop(1-m+u, v, para=para$para) -
                             subcop(1-m,   v, para=para$para))
     } else {     return(v - subcop(1-m,   v, para=para$para) +
                             subcop(u-m,   v, para=para$para))
     }
  }
  "UsersCop" <- function(u,v, para=NULL) {
     asCOP(u,v, f=shufflecop, para=para)
  }
  n <- 1000; u <- runif(n)
  para <- list(mixer=runif(1), subcop=W, para=20)
  v <- sapply(1:n, function(i) {
      simCOPmicro(u[i], cop=UsersCop, para=para) } )
  plot(data.frame(U=u, V=v), pch=17, col=rgb(1,0,1,1),
     xlab="U, NONEXCEEDANCE PROBABILTY", ylab="V, NONEXCEEDANCE PROBABILITY")
  mtext("Shuffle Copula Nelsen (2006, exam. 3.10, p. 73)")

  # Concerning Nelsen (2006, exam. 5.14, p. 195)
  "deltacop" <- function(u,v, ...) { min(c(u,v,(u^2+v^2)/2))     }
  "UsersCop" <- function(u,v, ...) { asCOP(u,v, f=deltacop, ...) }
  isCOP.PQD(cop=UsersCop) # TRUE + Rho=0.288 and Tau=0.333 as Nelsen says
  isCOP.LTD(cop=UsersCop, wrtV=TRUE) # FALSE as Nelsen says
  isCOP.RTI(cop=UsersCop, wrtV=TRUE) # FALSE as Nelsen says }
}
\keyword{copula (utilities)}
\keyword{copula (formulas)}
\keyword{Nelsen (2006) Examples and Exercises}
\keyword{shuffle (in Examples)}
\keyword{triangle (in Examples)}
\keyword{shuffle}

\encoding{utf8}
\name{qua.regressCOP2}
\alias{qua.regressCOP2}
\title{Perform Quantile Regression using a Copula by Numerical Derivative Method for U with respect to V}
\description{
Perform \emph{quantile regression} (Nelsen, 2006, pp. 217--218) using a copula by numerical derivatives of the copula (\code{\link{derCOPinv2}}). If \eqn{X} and \eqn{Y} are random variables having quantile functions \eqn{x(F)} and \eqn{y(G)} and letting \eqn{x=\tilde{x}(y)} denote a solution to \eqn{\mathrm{Pr}[X \le x\mid Y = y] = F}, where \eqn{F} is a nonexceedance probability. Then the curve \eqn{x=\tilde{x}(y)} is the quantile regression curve of \eqn{U} or \eqn{X} with respect to  \eqn{V} or \eqn{Y}, respectively. If \eqn{F=1/2}, then \emph{median regression} is performed (\code{\link{med.regressCOP2}}). Using copulas, the quantile regression is expressed as
\deqn{\mathrm{Pr}[X \le x\mid Y = y] = \mathrm{Pr}[U \le F(x) \mid V = F] = \mathrm{Pr}[U \le u\mid V = F] = \frac{\delta \mathbf{C}(u,v)}{\delta v}\mbox{,}}
where \eqn{v = G(y)} and \eqn{u = F(x)}. The general algorithm is
\enumerate{
\item Set \eqn{\delta \mathbf{C}(u,v)/\delta v = F},

\item Solve the regression curve \eqn{u = \tilde{u}(v)} (provided by \code{\link{derCOPinv2}}), and

\item Replace \eqn{u} by \eqn{x(u)} and \eqn{v} by \eqn{y(v)}.
}

The last step is optional as step two produces the regression in probability space, which might be desired, and step 3 actually transforms the probability regressions into the quantiles of the respective random variables.
}
\usage{
qua.regressCOP2(f=0.5, v=seq(0.01,0.99, by=0.01), cop=NULL, para=NULL, ...)
}
\arguments{
  \item{f}{A single value of nonexceedance probability \eqn{F} to perform regression at and defaults to median regression \eqn{F=1/2};}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  An \R \code{data.frame} of the regressed probabilities of \eqn{U} and \eqn{V=v} is returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{med.regressCOP2}}, \code{\link{derCOPinv2}}}
\examples{
\dontrun{
# Use a positively associated Plackett copula and perform quantile regression
theta <- 0.10
R <- qua.regressCOP2(cop=PLACKETTcop, para=theta) # 50th percentile regression
plot(R$U,R$V, type="l", lwd=6, xlim=c(0,1), ylim=c(0,1), col=8)
lines((1+(theta-1)*R$V)/(theta+1),R$V, col=4, lwd=1) # theoretical for Plackett,
# compare the theoretical form to that in qua.regressCOP---just switch terms around
# because of symmetry
R <- qua.regressCOP2(f=0.90, cop=PLACKETTcop, para=theta) # 90th-percentile regression
lines(R$U,R$V, col=2, lwd=2)
R <- qua.regressCOP2(f=0.10, cop=PLACKETTcop, para=theta) # 10th-percentile regression
lines(R$U,R$V, col=2, lty=2)
mtext("Quantile Regression U wrt V for Plackett copula")#}
}
\keyword{quantile regression}

\encoding{utf8}
\name{mleCOP}
\alias{mleCOP}
\title{Maximum Pseudo-Log-Likelihood Estimation for Copula Parameter Estimation}
\description{
Perform maximum pseudo-log-likelihood estimation (pMLE) for copula parameters by maximizing the function:
\deqn{\mathcal{L}(\Theta_p) = \sum_{i=1}^n \log\bigl[ c(F_x(x_i), F_y(y_i); \Theta_p)\bigr]\mbox{,}}
where \eqn{\mathcal{L}(\Theta_p)} is the log-likelihood for parameter vector \eqn{\Theta_p} of dimension \eqn{p}, and \eqn{c(u,v; \Theta_p)} is the bivariate copula density. The \eqn{u} and \eqn{v} are estimated by the respective empirical cumulative distribution functions \eqn{u = F_x(\cdots)} and \eqn{v = F_y(\cdots)} for each of the joint realizations of a sample of size \eqn{n}. The \eqn{c(u,v)} is numerically estimated by the copula using the \code{\link{densityCOP}} function.
}
\usage{
mleCOP(u, v=NULL, cop=NULL, parafn=function(k) return(k),
          interval=NULL, init.para=NULL, verbose=FALSE, control=list(),
          the.zero=.Machine$double.eps^0.25, s=0, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction and if \code{NULL} then \code{u} is treated as a two column \R \code{data.frame};}
  \item{cop}{A copula function;}
  \item{parafn}{A function responsible for generating the parameters. This is often just a simple return of a parameter vector as \pkg{copBasic} uses this style of parameterization, but this function can take over parameter remapping to handle boundary conditions to benefit the search or provide an interface into other copula packages in \R (see \bold{Examples});}
  \item{interval}{The search interval for root finding, by \code{stats::optimise()}, if the parameter dimension of the copula is \eqn{p = 1}. The interval is not used for \eqn{p \ge 2};}
  \item{init.para}{The initial guesses for the parameters for the \eqn{p}-dimensional optimization for \eqn{p \ge 2}. The initial guess is used, by \code{stats::optim()}, if the parameter dimension of the copula is \eqn{p = 1} and \code{interval} is \code{NULL} (see \bold{Examples});}
  \item{verbose}{A logical that internally is converted to integer to trigger 1 (sum of logs of \code{\link{densityCOP}} shown), 2 (add reporting of the copula parameter on each iteration), or more levels of verbose reporting scheme within the objective function. This is independent from the \code{control$trace} of function \code{optim()};}
  \item{control}{This argument is the argument of the same name for \code{optim()};}
  \item{the.zero}{The value for \dQuote{the zero} of the copula density function. This argument is the argument of the same name for \code{\link{densityCOP}}. The default here is intended to suggest that a tiny nonzero value for density will trap the numerical zero densities;}
  \item{s}{A vector of at least two presumably uniformly distributed or regular sequence of nonexceedance probabilities in \eqn{U} for simulation of \eqn{V} by \code{\link{simCOPv}} and plotting of these \eqn{U} and \eqn{V}. This plotting is only made if the length of \eqn{s} is nonzero and \code{verbose} is greater than or equal to 2. This plotting feature for the \code{s} is pedagogical  and intended for demonstration or teaching opportunities. This feature has no utility for the optimization itself; and}
  \item{...}{Additional arguments to pass, see source code for the internally used functions that can pick these additional arguments up.}
}
\value{
  The value(s) for the estimated parameters are returned within an \R \code{list} where the elements listed below are populated unique to this package. The other elements of the returned list are generated from either the \code{optimise()} (1D, \eqn{p = 1}) or \code{optim()} (pD, \eqn{p \ge 2}) functions of \R.
  \item{para}{The parameter(s) in a canonical element after the one-dimensional root finding (\eqn{p = 1}) or multi-dimensional optimization (\eqn{p \ge 2}) solutions are passed through \code{parafn} so that these are in the parameter units of the copula and not necessarily those transformed for the optimization;}
  \item{packagetext}{A helpful message unique to the \pkg{copBasic} package;}
  \item{loglik}{The maximum of the log-likelihood matching the name for the same quantity by the function \code{fitCopula} in package \pkg{copula} though a separate implementation is used in \pkg{copBasic};}
  \item{AIC}{Akaike information criterion (AIC) (see also \code{\link{aicCOP}}): \eqn{\mathrm{AIC} = 2p - 2\mathcal{L}(\Theta_p)}; and}
  \item{BIC}{Bayesian information criterion (BIC) (see also \code{\link{bicCOP}}): \eqn{\mathrm{BIC} = p\log(n) - 2\mathcal{L}(\Theta_p)}.}
}
\note{
This section provides for a more thorough assessment of pMLE than shown in the \bold{Examples}.

\emph{INTERFACE TO THE \pkg{COPULA} PACKAGE}---A not uncommon question to the author is how can \pkg{copBasic} support copulas from other packages?  A \pkg{copBasic} pMLE implementation to the \emph{Gaussian copula} from the \pkg{copula} package is thus useful for instruction.

Two interface functions are required for the pMLE situation. First, interface the \pkg{copula} package in a generic form for the \pkg{copBasic} package:
\preformatted{
  "cB2copula" <-  # pCoupla() from package copula is analogous to COP()
  function(u,v, para=NULL, ...) {
    if(length(u) == 1) u <- rep(u, length(v)) # see asCOP() for reasoning of
    if(length(v) == 1) v <- rep(v, length(u)) # this "vectorization" hack
    return(copula::pCopula(matrix(c(u,v), ncol=2), para))
  }
}
where the \code{para} argument above must be built by the features of the \pkg{copula} package. The following function then provides for parameter setup specific to the \emph{Gaussian copula} having parameter \eqn{\rho}:
\preformatted{
  copula2cBpara <- function(rho) return(copula::normalCopula(rho, dim = 2))
}
Now, let us perform a parameter estimate for a sample of size \eqn{n=900}:
\preformatted{
  set.seed(162); UV <- simCOP(n=900, cop=cB2copula, para=copula2cBpara(0.45))
  mleCOP(UV, cop=cB2copula, parafn=copula2cBpara, interval=c(-1,1))$para
  #   rho.1  =  0.4248822
}
The search interval for the \emph{Gaussian copula} is \eqn{\rho \in [-1, 1]}, and the final result is \eqn{\rho = 0.4458822}.

\emph{MULTI-DIMENSIONAL EXAMPLE OF pMLE}---Consider a 2-parameter \emph{Gumbel--Hougaard copula} (\eqn{\mathbf{GH}(\Theta_1, \Theta_2)}) but now use the \code{parafn} argument to provide boundary condition assistance through function \code{GH2pfunc} to the \code{optim()} function that performs the maximization.
\preformatted{
  set.seed(162); UV <- simCOP(n=890, cop=GHcop, para=c(2.4, .06))
  GH2pfunc <- function(p) { return(c(exp(p[1])+1, exp(p[2]))) }
  ML <- mleCOP(UV$U, UV$V, cop=GHcop, init.para=c(1,1), parafn=GH2pfunc)
  print(ML$para) # [1] 2.2755018 0.1194788
}
and the result is \eqn{\Theta_{1,2} = (2.2755018, 0.1194788)}. Next, consider now a 3-parameter \eqn{\mathbf{GH}(\Theta, \pi_1, \pi_2)} copula and again use the \code{parafn} argument through function \code{GH3pfunc}  but notice that the 2nd and 3rd parameters are now mapped into \eqn{0 \le \pi_1, \pi_2 \le 1} domain using the \code{pnorm()} function.
\preformatted{
  set.seed(162); UV <- simCOP(n=500, cop=GHcop, para=c(5.5, .6, .9))
  GH3pfunc <- function(p) { return(c(exp(p[1])+1, pnorm(p[2]), pnorm(p[3]))) }
  ML <- mleCOP(UV$U, UV$V, cop=GHcop, init.para=c(1, .5, .5), parafn=GH3pfunc)
  print(ML$para) # [1] 5.3742229 0.6141652 0.9382638
}
and the result is \eqn{\Theta = 5.3742229} and \eqn{\pi_{1,2} = (0.6141652, 0.9382638)}.

\emph{ANOTHER MULTI-DIMENSIONAL EXAMPLE OF pMLE}---Finally, an experiment can be made fitting a 3-parameter \eqn{\mathbf{GH}(\Theta, \pi_1, \pi_2)} to a simulation from a 2-parameter \eqn{\mathbf{GH}(\beta_1, \beta_2)}, where the seed is just arbitrary and the \emph{Vuong Procedure} (\code{\link{vuongCOP}}) is used to compare fits and make inference. The parameter functions \code{GH2pfunc} and \code{GH3pfunc} are as before.
\preformatted{
  set.seed(10); UV <- simCOP(n=500, cop=GHcop, para=c(1.7, 1.86))
  GH2pfunc <- function(p) { return(c(exp(p[1])+1,   exp(p[2])              )) }
  GH3pfunc <- function(p) { return(c(exp(p[1])+1, pnorm(p[2]), pnorm(p[3]) )) }
  para1 <- mleCOP(UV, cop=GHcop, init.para=c(1,1),     parafn=GH2pfunc)$para
  para2 <- mleCOP(UV, cop=GHcop, init.para=c(1,.5,.5), parafn=GH3pfunc)$para
  vuongCOP(UV, cop1=GHcop, para1=para1, cop2=GHcop, para2=para2)$message
  #[1] "Copula 1 has better fit than Copula 2 at 100 x (1-alpha) level"
}
The results show the 2-p \eqn{\mathbf{GH}} is a better fit to the simulated data than the 3-p \eqn{\mathbf{GH}}, which seems a bit self evident?  Plot some same-seeded simulations just to confirm.
\preformatted{
  set.seed(67) # First the estimated parameters but with the correct model.
  UV <- simCOP(n=200, GHcop, para=para1, snv=TRUE, pch=16, col=2)
  set.seed(67) # Second, the estimated incorrect model.
  UV <- simCOP(n=200, GHcop, para=para2, snv=TRUE, ploton=FALSE)
}
Yes, differences in form are manifest in the produced graphic. Now, let us try another set of parameters and again an arbitrarily-chosen seed.
\preformatted{
  set.seed(10); UV <- simCOP(n=500, cop=GHcop, para=c(1.91, 0.16))
  para1 <- mleCOP(UV, cop=GHcop, init.para=c(1,1),     parafn=GH2pfunc)$para
  para2 <- mleCOP(UV, cop=GHcop, init.para=c(1,.5,.5), parafn=GH3pfunc)$para
  vuongCOP(UV, cop1=GHcop, para1=para1, cop2=GHcop, para2=para2)$message
  #[1] "Copulas 1 and 2 are not significantly different at 100 x (1-alpha)"
}
The results show equivalence, let us now check a graphic.
\preformatted{
  set.seed(67); z <- simCOP(n=200, GHcop, para=para1, snv=TRUE, pch=16, col=2)
  set.seed(67); z <- simCOP(n=200, GHcop, para=para2, snv=TRUE, ploton=FALSE)
}
The differences are small but the differences might be inflating into the lower left corner. What sample size could conceivably begin to distinguish between the copula?
\preformatted{
  kullCOP(cop1=GHcop, cop2=GHcop, para1=para1, para2=para2) # 625 on this run

  nsim <- 20; set.seed(67)
  Results <- sapply(1:nsim, function(i) {
    UV <- simCOP(n=625, cop=GHcop, para=c(1.91, .16), graphics=FALSE)
    p1 <- mleCOP(UV, cop=GHcop, init.para=c(1,1),     parafn=GH2pfunc)$para
    p2 <- mleCOP(UV, cop=GHcop, init.para=c(1,.5,.5), parafn=GH3pfunc)$para
    vuongCOP(UV, cop1=GHcop, para1=p1, cop2=GHcop, para2=p2)$result })
  sum(Results)
}
The summation yields 6 of 20 for which copula 1 has the better fit, but with \eqn{n=1{,}000} instead of \eqn{n=625}, the sum of the \code{Results} is 13 of 20 (so better than half the time). This seems to be in conflict with what the \eqn{n_{fg}} sample size from \code{\link{kullCOP}} should be telling. The author thinks it should be 18 to 19 of 20 (95th percentile) based on what the \code{\link{kullCOP}} is reported to do (NEED TO LOOK INTO THIS).
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{densityCOP}}}
\examples{

# See also extended code listings and discussion in the Note section

\dontrun{
  # Here, we study the trajectory of the objective function in a simple
  # 1-dimensional optimization. See how we must provide the interval.
  set.seed(162); UV <- simCOP(n=188, cop=PLcop, para=5.6)
  ML <- mleCOP(UV$U, UV$V, cop=PLcop, interval=c(0.1, 40)) # 5.225459 estimated

  Thetas <- 10^(seq(log10(0.001), log10(100), by=0.005))
  MLs <- sapply(Thetas, function(k)
                densityCOP(UV$U, UV$V, cop=PLcop, para=k, sumlogs=TRUE))
  plot(Thetas, MLs, log="x", type="l", # draw the pMLE solution process
       xlab="Plackett Theta", ylab="sum of log densities")
  lines(rep(ML$para, 2), c(ML$objective, par()$usr[3]), col="red")
  points(ML$para, ML$objective, pch=16, col="red") #}

\dontrun{
  # Here, we study again 1-dimensional optimization but use the
  # multidimensional version with an alert issued.
  set.seed(149); UV <- simCOP(1000, cop=CLcop, para=pi)
  # Warning messages about using optim() for 1D solution
  mleCOP(UV, cop=CLcop, init.para=2)$para          # 3.082031
  # No warning message, optimise() called instead.
  mleCOP(UV, cop=CLcop, interval=c(0,1E2))$para    # 3.081699 }

\dontrun{
  # Here, we evaluate a 2-dimensional problem using a Plackett again but with
  # the addition of asymmetry towards high V outliers from the Plackett cloud.
  # This example also adds the internal verbose and graphic diagnostics for
  # the iterations of the optimizer. Here, we learn that we need on a time have
  # some idea where the solution might lay so that we can provide a suitable
  # set of initial parameters for the algorithm.
  para <- list(beta=-0.1, cop=PLcop, para1=1000)
  UV <- simCOP(2000, cop=breveCOP, para=para); abline(0, 1, col="red", lwd=3)
  PL2pfunc <- function(p) { # see here example of parameter transform
    list(beta=2*pnorm(p[1])-1, para=exp(p[2]), cop=PLcop) # [-1,+1], >=0
  }
  init.para <- c(0.2535, log(0.02)) # These will not find a solution with this
  # guess of negative association, but the next works by using an external
  # estimate of the Plackett parameters and here we test with a positive
  # skewness (beta for breveCOP > 0) although we know the parent had negative.
  init.para <- c(0.2535, log(PLACKETTpar(UV$U, UV$V, byrho=TRUE))) # beta=0.200
  rt <- mleCOP(u=UV$U, v=UV$V, init.para=init.para, cop=breveCOP,
               parafn=PL2pfunc, verbose=2, s=seq(0,1, by=0.005)) #}
}
\keyword{copula (fitting)}
\keyword{maximum likelihood}
\concept{Vuong Procedure}
\concept{Akaike}
\concept{AIC}
\concept{BIC}
\concept{Vuong}
\concept{Kullback--Leibler}
\concept{Kullback-Leibler}
\keyword{API to the copula package}
\keyword{package copula (comparison to)}

\encoding{utf8}
\name{PARETOcop}
\alias{PARETOcop}
\alias{PAcop}
\title{The Pareto Copula}
\description{
The \emph{Pareto copula} (Nelsen, 2006, pp. 33) is
\deqn{\mathbf{C}_{\Theta}(u,v) = \mathbf{PA}(u,v) = \bigl[(1-u)^{-\Theta}+(1-v)^{-\Theta}\bigr]^{-1/\Theta}\mbox{,}}
where \eqn{\Theta \in [0, \infty)}. As \eqn{\Theta \rightarrow 0^{+}}, the copula limits to the \eqn{\mathbf{\Pi}} copula (\code{\link{P}}) and the \eqn{\mathbf{M}} copula (\code{\link{M}}). The parameterization here has association increasing with increasing \eqn{\Theta}, which differs from Nelsen (2006), and also the Pareto copula is formed with right-tail increasing reflection of the Nelsen (2006) presentation because it is anticipated that \pkg{copBasic} users are more likely to have right-tail dependency situations (say large maxima [right tail] coupling in earth-system data but not small maxima [left tail] coupling).
}
\usage{
PARETOcop(u, v, para=NULL, ...)
    PAcop(u, v, para=NULL, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (single element) of parameters---the \eqn{\Theta} parameter of the copula; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\note{
The Pareto copula is used in a demonstration of \emph{Kendall Function L-moment ratio diagram} construction (see \code{\link{kfuncCOPlmoms}}).
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{M}}, \code{\link{P}}
}
\examples{
\dontrun{
z <- seq(0.01,0.99, by=0.01) # Both copulas have Kendall Tau = 1/3
plot( z, kfuncCOP(z, cop=PAcop, para=1), lwd=2, col="black",
                                xlab="z <= Z", ylab="F_K(z)", type="l")
lines(z, kfuncCOP(z, cop=GHcop, para=1.5), lwd=2, col="red") # red line
# All extreme value copulas have the same Kendall Function [F_K(z)], the
# Gumbel-Hougaard is such a copula and the F_K(z) for the Pareto does not
# plot on top and thus is not an extreme value but shares a "closeness."}
}
\keyword{Pareto copula}
\keyword{copula (formulas)}
\keyword{copula}

\encoding{utf8}
\name{gEVcop}
\alias{gEVcop}
\title{The Gaussian-based (Extreme Value) Copula}
\description{
The \emph{g-EV copula} (Joe, 2014, p. 105) is a limiting form of the \emph{Normal (Gaussian) copula} (\code{\link{NORMcop}}):
\deqn{
\mathbf{C}_{\rho}(u,v) = \mathbf{gEV}(u,v; \rho) =
\mathrm{exp}\bigl(-A(x,y; \rho)\bigr)\mbox{,}
}
where \eqn{x = -\log(u)}, \eqn{y = -\log(v)}, and
\deqn{
A(x,y; \rho) = y\mbox{,}
}
for \eqn{0 \le x/(x+y) \le \rho^2/(1+\rho^2)},
\deqn{
A(x,y; \rho) = (x+y - 2\rho\sqrt{xy})/(1-\rho^2)\mbox{,}
}
for \eqn{\rho^2/(1+\rho^2) \le x/(x+y) \le 1/(1+\rho^2)},
\deqn{
A(x,y; \rho) = x\mbox{,}
}
for \eqn{1/(1+\rho^2) \le x/(x+y) \le 1} and where \eqn{\rho \in [0,1]}. A somewhat curious observation is that this copula has relative hard boundaries into the upper-left and lower-right corners when compared to the other copulas supported by the \pkg{copBasic} package. In other words, the hull defined by the copula has a near hard (not fuzzy) curvilinear boundaries that adjust with the parameter \eqn{\rho}. The lower-case \code{g} in the copula name is deliberately to avoid the sequence \code{GEV} as to never be confused with the \emph{generalized extreme value distribution}.
}
\usage{
gEVcop(u, v, para=NULL, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{The parameter \eqn{\rho}; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{tEVcop}}, \code{\link{NORMcop}}
}
\examples{
\dontrun{
  UV <- simCOP(200, cop=gEVcop, para=0.8) #}

\dontrun{
  # Joe (2014, p. 105) has brief detail indicating rho = [0,1] and though it seems
  # Rho would be a Pearson correlation, this does not seem to be the case. The Rho
  # seems to start with that of the Gaussian and then through the extreme-value
  # transform, it just assumes the role of a parameter called Rho.
  rho <- 0.8
  UV <- simCOP(2000, cop=gEVcop, para=rho)
  P <- cor(UV[,1], UV[,2], method="pearson")
  if(abs(P - rho) < 0.001) {
    print("Yes same")
  } else { print("nope not") } # }

\dontrun{
  gEVparameter <- seq(0.02, 1, by=0.02)
  SpearmanRho <- sapply(gEVparameter, function(k) rhoCOP(cop=gEVcop, para=k))
  plot(gEVparameter, SpearmanRho)

  for(rho in gEVparameter) {
    UV <- simCOP(n=1000, cop=gEVcop, para=rho)
    mtext(paste0("Parameter = ", rho))
  } # }
}
\keyword{g-EV copula}
\keyword{copula (formulas)}
\keyword{copula}
\keyword{package copula (comparison to)}
\concept{g-EV copula}
\keyword{copula (extreme value)}
\keyword{extreme value copula}
\keyword{Spearman Rho}
\keyword{Pearson correlation coefficient}

\encoding{utf8}
\name{ORDSUWcop}
\alias{ORDSUWcop}
\title{Ordinal Sums of W-Copula}
\description{
Compute \emph{W-ordinal sum of the summands} (Klement  \emph{et al.}, 2017) within \eqn{\mathcal{I}^2} into \eqn{n} partitions (possibly infinite) within \eqn{\mathcal{I}^2}. Letting \eqn{\mathcal{J}} denote a \emph{partition} of \eqn{\mathcal{I}^2} and \eqn{\mathcal{J}_i = [a_i,\, b_i]} be the \eqn{i}th partition that does not overlap with others and letting also \eqn{\mathbf{C}_i} be a copula for the \eqn{i}th partition, then the \emph{ordinal sum} of these \eqn{\mathbf{C}_i} with parameters \eqn{\Theta_i} \emph{with respect to} \eqn{\mathcal{J}_i} is the copula \eqn{\mathbf{C}} given by

\deqn{\mathbf{C}\bigl(u,v; \mathcal{J}_i, \mathbf{C}_i, \Theta_i, i \in 1,2,\cdots,n\bigr) = a_i + (b_i-a_i)\mathbf{C}_i\biggl(\frac{u-a_i}{b_i-a_i},\, \frac{v-1+b_i}{b_i-a_i};  \Theta_i\biggr)\  \mbox{for}\ (u,v) \in \mathcal{J}^2\mbox{,}}

for points within the partitions, and for points otherwise outside the partitions the coupla is given by

\deqn{\mathbf{C}\bigl(u,v; \mathcal{J}_i, \mathbf{C}_i, i \in 1,2,\cdots,n\bigr) = \mathbf{W}(u,v)\  \mathrm{for}\ (u,v) \ni \mathcal{J}^2\mbox{, and}}

let \eqn{\mathbf{C}_\mathcal{J}(u,v)} be a convenient abbreviation for the copula. (See \code{\link{ORDSUMcop}}, \emph{M-ordinal sum of the summands}.)
}
\usage{
ORDSUWcop(u,v, para=list(cop=M, para=NA, part=c(0,1)), ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A list of sublists for the coupla, parameters, and partitions (see \bold{Examples}) and some attempt for intelligent in-fill of \code{para} is made within the sources (the default \code{para} is an example for which \code{cop} and \code{para} elements are converted to lists). The user is responsible that \code{part} element properly canvases by end-point alignment all of \eqn{\mathcal{I}^2}; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\references{
Klement, E.P., \enc{Kolesárová}{Kolesarova}, A., Mesiar, R., Saminger-Platz, S., 2017, Copula constructions using ultramodularity (chap. 9) \emph{in} Copulas and dependence models with applications---Contributions in honor of Roger B. Nelsen, \emph{eds.} Flores, U.M., Amo Artero, E., Durante, F., \enc{Sánchez}{Sanchez}, J.F.: Springer, Cham, Switzerland, ISBN 978--3--319--64220--9, \doi{10.1007/978-3-319-64221-5}.
}
\author{W.H. Asquith}
\seealso{\code{\link{copBasic-package}}, \code{\link{W_N5p12a}}, \code{\link{ORDSUMcop}}}
\examples{
para <- list(cop=c(CLcop, GHcop), para=list(5, 2), part=c(0,0.25,1)) # break points
UV <- simCOP(n=100, cop=ORDSUMcop, seed=1, para=para, ploton=TRUE, pch=16)
UV <- simCOP(n=100, cop=ORDSUWcop, seed=1, para=para, ploton=FALSE)

\dontrun{
  para <- list(cop=c(CLcop, M, PLcop, GHcop), para=list(4, NA, 0.1, c(3, 4)),
               part=list(c(0, 0.25), c(0.25, 0.35), c(0.35, 0.85), c(0.85, 1)))
  UV <- simCOP(n=100, cop=ORDSUWcop, para=para, ploton=FALSE)
  plot(c(0,1), c(0,1), xlab="U, NONEXCEEDANCE PROBABILITY", type="n", las=1,
                       ylab="V, NONEXCEEDANCE PROBABILITY")
  for(k in seq_len(length(para$part))) {         #  to draw the partitions
    a <- para$part[[k]][1]; b <- para$part[[k]][2]
    polygon(c(a, b, b, a, a), c(1-a,1-a,1-b,1-b,1-a), lty=2, lwd=0.8, col="lightgreen")
    text( (a+b) / 2,  (1-a+1-b) / 2, k, cex=3, col="blue") # numbered by partition
  }
  points(UV, pch=21, cex=0.8, col=grey(0.1), bg="white") #}

\dontrun{
  para = list(cop=c(GHcop), para=list(c(2, 3)),  # internally replicated
              part=list(c(0, 0.2), c(0.2, 0.3), c(0.3, 0.5), c(0.5, 0.7), c(0.7, 1)))
  UV <- simCOP(n=100, cop=ORDSUWcop, para=para, ploton=FALSE)
  plot(c(0,1), c(0,1), xlab="U, NONEXCEEDANCE PROBABILITY", type="n", las=1,
                       ylab="V, NONEXCEEDANCE PROBABILITY")
  for(k in seq_len(length(para$part))) {         #  to draw the partitions
    a <- para$part[[k]][1]; b <- para$part[[k]][2]
    polygon(c(a, b, b, a, a), c(1-a,1-a,1-b,1-b,1-a), lty=2, lwd=0.8, col="lightgreen")
    text( (a+b) / 2, (1-a+1-b) / 2, k, cex=3, col="blue") # numbered by partition
  }
  points(UV, pch=21, cex=0.8, col=grey(0.1), bg="white") #}
}
\keyword{copula (formulas)}
\concept{Ordinal Sums}
\keyword{copula}
\keyword{ordinal sums}
\keyword{W-ordinal sum of the summands}
\concept{summands}

\encoding{utf8}
\name{composite2COP}
\alias{composite2COP}
\alias{khoudraji2COP}
\title{Composition of Two Copulas with Two Compositing Parameters (Khoudraji Device)}
\description{
The \emph{composition of two copulas} (Salvadori \emph{et al.}, 2007, p. 266, prop. C.3) provides for more sophisticated structures of dependence between variables than many single parameter copula can provide. Further, \emph{asymmetrical copulas} are readily obtained from \emph{symmetrical copulas}. Let \eqn{\mathbf{A}} and \eqn{\mathbf{B}} be copulas with respective parameters \eqn{\Theta_\mathbf{A}} and \eqn{\Theta_\mathbf{B}}, then

\deqn{\mathbf{C}_{\alpha,\beta}(u,v) = \mathbf{A}\bigl(u^\alpha,    v^\beta\bigr) \cdot
                              \mathbf{B}\bigl(u^{1-\alpha},v^{1-\beta}\bigr)\mbox{,}}

defines a family of copulas \eqn{\mathbf{C}_{\alpha,\beta; \Theta_\mathbf{A}, \Theta_\mathbf{B}}} with \bold{two} \emph{compositing parameters} \eqn{\alpha,\beta \in \mathcal{I}:[0,1]}. In particular if \eqn{\alpha = \beta = 1}, then \eqn{\mathbf{C}_{1,1} = \mathbf{A}}, and if \eqn{\alpha = \beta = 0}, then \eqn{\mathbf{C}_{0,0} = \mathbf{B}}. For \eqn{\alpha \ne \beta}, the \eqn{\mathbf{C}_{\alpha,\beta}} is in general asymmetric that is \eqn{\mathbf{C}(u,v) \ne \mathbf{C}(v,u)} for some \eqn{(u,v) \in \mathcal{I}^2}. This construction technique is named the \emph{Khoudraji device} within the \pkg{copula} package (see \code{khoudrajiCopula} therein) (Hofert \emph{et al.}, 2018, p. 120).

It is important to stress that copulas \eqn{\mathbf{A}_{\Theta_A}} and \eqn{\mathbf{B}_{\Theta_B}} can be of different families and each copula parameterized accordingly by the vector of parameters \eqn{\Theta_A} and \eqn{\Theta_B}. This is an interesting feature in the context of building complex structures when pursuing asymmetric measures of dependency such as the \emph{L-comoments}. Symmetry of the copula \eqn{\mathbf{C}} is required for the situation that follows, however.

It is possible to simplify the construction of an asymmetric copula from a symmetric copula by the following. Let \eqn{\mathbf{C}(u,v)} be a symmetric copula, \eqn{\mathbf{C} \ne \mathbf{\Pi}} (for \eqn{\mathbf{\Pi}} see \code{\link{P}}). A family of asymmetric copulas \eqn{\mathbf{C}_{\alpha,\beta}} with \bold{two} \emph{composition parameters} \eqn{0 < \alpha,\beta < 1, \mbox{and\ } \alpha \ne \beta} that also includes \eqn{\mathbf{C}(u,v)} as a limiting case and is given by

\deqn{\mathbf{C}_{\alpha,\beta}(u,v) = u^\alpha v^\beta \cdot \mathbf{C}\bigl(u^{1-\alpha},v^{1-\beta}\bigr)\mbox{.}}

Hofert \emph{et al.} (2018, p. 121) comment that \dQuote{from a practical perspective, a useful subset of families constructed from [the] Khoudraji device is obtained} by choosing \emph{independence copula} (\eqn{\mathbf{\Pi}}, \code{\link{P}}) for one of the copula and that choosing [\eqn{\alpha} or \eqn{\beta}] relative close to \eqn{1} produces nonexchangable [see \code{\link{isCOP.permsym}}] versions of \eqn{\mathbf{C}(u,v)} (meaning \eqn{\mathbf{C}(u,v) \ne \mathbf{C}(v,u)}). For the \pkg{copBasic} package, the \code{khoudraji1COP()} and \code{khoudrajiPCOP()} (the \code{P} meaning \code{\link{P}}) aliases are added for programming clarity for developers desiring to have contrasting copula calls to the three \emph{compositing} copula: \code{\link{composite1COP}}, \code{composite2COP()}, and \code{\link{composite3COP}}.
}
\usage{
composite2COP(u, v, para, ...)
khoudraji2COP(u, v, para, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A special parameter \code{list} (see \bold{Note}); and}
  \item{...}{Additional arguments to pass to the copulas.}
}
\value{
  Value(s) for the composited copula is returned.
}
\note{
The following descriptions list in detail the structure and content of the \code{para} argument:
\describe{
  \item{\code{alpha}}{--- The \eqn{\alpha} compositing parameter;}
  \item{\code{beta}}{--- The \eqn{\beta} compositing parameter;}
  \item{\code{cop1}}{--- Function of the first copula \eqn{\mathbf{A}};}
  \item{\code{cop2}}{--- Function of the second copula \eqn{\mathbf{B}};}
  \item{\code{para1}}{--- Vector of parameters \eqn{\Theta_\mathbf{A}} for \eqn{\mathbf{A}}; and}
  \item{\code{para2}}{--- Vector of parameters \eqn{\Theta_\mathbf{B}} for \eqn{\mathbf{B}}.}
}

The \code{para} argument of this function also can be passed to \code{\link{composite1COP}}; albeit, the second copula and its parameters would not be used. A more complex (extended) composition in \code{\link{composite3COP}} extends this basic parameter structure.
}
\references{
Hofert, M., Kojadinovic, I., \enc{Mächler}{Machler},  M., and Yan, J., 2018, Elements of copula modeling with R: Dordrecht, Netherlands, Springer.

Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in Nature---An approach using copulas: Springer, 289 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{COP}}, \code{\link{breveCOP}}, \code{\link{composite1COP}}, \code{\link{composite3COP}},
\code{\link{convexCOP}}, \code{\link{glueCOP}}}
\examples{
alpha <- 0.24; beta <- 0.23; Theta1 <- NA; Theta2 <- NA
# The W() and PSP() copulas do not take parameters, but example shows how the
# parameters would be set should either or both of the copulas require parameters.
para <- list(alpha=alpha, beta=beta, cop1=W, cop2=PSP, para1=Theta1, para2=Theta2)
print(composite2COP(0.4, 0.6, para)) # 0.2779868

# In this example, the N4212cop uses "3" as its parameter value.
para <- list(alpha=alpha, beta=beta, cop1=W, cop2=N4212cop, para1=Theta1, para2=3)
print(composite2COP(0.4, 0.6, para)) # 0.3387506

\dontrun{
  # This example does a great job of showing a composited copula with a near singularity,
  # but with leakage of chance to the upper left. The example is also critical because
  # it shows that gridCOP is returning a matrix in the proper orientation relative to
  # the level.curvesCOP and simCOP functions. Example is cross-ref'ed from gridCOP() docs.
  layout(matrix(1:2,byrow=TRUE))
  para <- list(alpha=0.5, beta=0.90, cop1=M, cop2=N4212cop, para1=NA, para2=1.4)
  image(gridCOP(cop=composite2COP, para=para, delta=0.01), col=terrain.colors(30),
        xlab="U, NONEXCEEDANCE PROBABILITY", ylab="V, NONEXCEEDANCE PROBABILITY")
  D <- simCOP(n=2000, cop=composite2COP, para=para, ploton=FALSE, pch=4, col=4, cex=0.75)
  level.curvesCOP(cop=composite2COP, para=para, ploton=FALSE, delt=0.05)
  mtext("Theoretical composited copula, level curves, and simulation")

  emp <- EMPIRgrid(para=D, deluv=0.05)     # CPU heavy
  image(emp$empcop, col=terrain.colors(30) ) # image orientation is correct!
  # Depending on balance between sample size, deluv, delu, and delt, one or more:
  # Error in uniroot(func, interval = c(0, 1), u = u, LHS = t, cop = cop,  :
  #   f() values at end points not of opposite sign
  # warnings might be triggered. This is particularly true because of the flat derivative
  # above the near singularity in this example composited copula.
  points(D$U, D$V, pch=4, col="blue", cex=0.75)
  level.curvesCOP(cop=EMPIRcop, para=D, ploton=FALSE, delu=0.02, delt=0.05)
  mtext("Empirical copula from n=2000 simulation") #}

\dontrun{
  # Demonstration in how to add reflection (rotation by 180) degrees of the GHcop
  # and use the two-parameter version of that coupla to accent upper tail coupling
  # with a much weaker lower tail, but we want open up or spread the middle part of
  # the distribution. Lastly, because alpha != beta, permutation asymmetry is added.
  para <- list(cop1=W, para1=NA, cop2=GHcop, para2=c(1,10), alpha=0.01, beta=0.06)
  UV <- simCOP(1000, cop=COP, para=list(cop=composite2COP, reflect=2, para=para)) #}
}
\keyword{copula composition}
\keyword{copula composition (two compositing parameters)}
\keyword{Khoudraji device}

\encoding{utf8}
\name{EMPIRqua.regress}
\alias{EMPIRqua.regress}
\title{Quantile Regression of the Grid of the Bivariate Empirical Copula for V with respect to U}
\description{
Perform \emph{quantile regression} from the gridded inversion of the \emph{bivariate empirical copula} of \eqn{V} with respect to \eqn{U}.
}
\usage{
EMPIRqua.regress(f=0.5, u=seq(0.01,0.99, by=0.01), empinv=NULL,
                 lowess=FALSE, f.lowess=1/5, ...)
}
\arguments{
  \item{f}{The nonexceedance probability \eqn{F} to perform regression at and defaults to median regression \eqn{F=1/2};}
  \item{u}{A vector of \eqn{u} nonexceedance probabilities;}
  \item{empinv}{The grid from \code{\link{EMPIRgridderinv}};}
  \item{lowess}{Perform \code{lowess} smooth on the quantile regression using the smooth factor of \code{f.lowess};}
  \item{f.lowess}{Smooth factor of almost the same argument name fed to the \code{lowess()} function in \R; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  The gridded values of the quantile regression of \eqn{V} with respect to \eqn{U}.
}
\author{W.H. Asquith}
\seealso{\code{\link{EMPIRgridderinv}}, \code{\link{EMPIRqua.regress2}}, \code{\link{EMPIRmed.regress}}, \code{\link{EMPIRmed.regress2}}
}
\examples{
\dontrun{ # EXAMPLE 1
theta <- 25
uv <- simCOP(n=1000, cop=PLACKETTcop, para=theta, ploton=FALSE, points=FALSE)
fakeU <- lmomco::pp(uv[,1], sort=FALSE)
fakeV <- lmomco::pp(uv[,2], sort=FALSE)
uv <- data.frame(U=fakeU, V=fakeV)

uv.grid <- EMPIRgrid(para=uv, deluv=0.05) # CPU hungry
uv.inv1 <- EMPIRgridderinv(empgrid=uv.grid)
plot(uv, pch=16, col=rgb(0,0,0,.1), xlim=c(0,1), ylim=c(0,1),
     xlab="U, NONEXCEEDANCE PROBABILITY", ylab="V, NONEXCEEDANCE PROBABILITY")
lines(qua.regressCOP(f=0.5, cop=PLACKETTcop, para=theta), lwd=2)
lines(qua.regressCOP(f=0.2, cop=PLACKETTcop, para=theta), lwd=2)
lines(qua.regressCOP(f=0.7, cop=PLACKETTcop, para=theta), lwd=2)
lines(qua.regressCOP(f=0.1, cop=PLACKETTcop, para=theta), lwd=2)
lines(qua.regressCOP(f=0.9, cop=PLACKETTcop, para=theta), lwd=2)

med.wrtu <- EMPIRqua.regress(f=0.5, empinv=uv.inv1, lowess=TRUE, f.lowess=0.1)
lines(med.wrtu, col=2, lwd=4)
qua.wrtu <- EMPIRqua.regress(f=0.2, empinv=uv.inv1, lowess=TRUE, f.lowess=0.1)
lines(qua.wrtu, col=2, lwd=2, lty=2)
qua.wrtu <- EMPIRqua.regress(f=0.7, empinv=uv.inv1, lowess=TRUE, f.lowess=0.1)
lines(qua.wrtu, col=2, lwd=2, lty=2)
qua.wrtu <- EMPIRqua.regress(f=0.1, empinv=uv.inv1, lowess=TRUE, f.lowess=0.1)
lines(qua.wrtu, col=2, lwd=2, lty=4)
qua.wrtu <- EMPIRqua.regress(f=0.9, empinv=uv.inv1, lowess=TRUE, f.lowess=0.1)
lines(qua.wrtu, col=2, lwd=2, lty=4)

library(quantreg) # Quantile Regression by quantreg
U <- seq(0.01, 0.99, by=0.01)
rqlm <- rq(V~U, data=uv, tau=0.1)
rq.1 <- rqlm$coefficients[1] + rqlm$coefficients[2]*U
rqlm <- rq(V~U, data=uv, tau=0.2)
rq.2 <- rqlm$coefficients[1] + rqlm$coefficients[2]*U
rqlm <- rq(V~U, data=uv, tau=0.5)
rq.5 <- rqlm$coefficients[1] + rqlm$coefficients[2]*U
rqlm <- rq(V~U, data=uv, tau=0.7)
rq.7 <- rqlm$coefficients[1] + rqlm$coefficients[2]*U
rqlm <- rq(V~U, data=uv, tau=0.9)
rq.9 <- rqlm$coefficients[1] + rqlm$coefficients[2]*U

lines(U, rq.1, col=4, lwd=2, lty=4)
lines(U, rq.2, col=4, lwd=2, lty=2)
lines(U, rq.5, col=4, lwd=4)
lines(U, rq.7, col=4, lwd=2, lty=2)
lines(U, rq.9, col=4, lwd=2, lty=4)#}

\dontrun{ # EXAMPLE 2
# Start again with the PSP copula
uv <- simCOP(n=10000, cop=PSP, ploton=FALSE, points=FALSE)
fakeU <- lmomco::pp(uv[,1], sort=FALSE)
fakeV <- lmomco::pp(uv[,2], sort=FALSE)
uv <- data.frame(U=fakeU, V=fakeV)

uv.grid <- EMPIRgrid(para=uv, deluv=0.05) # CPU hungry
uv.inv1 <- EMPIRgridderinv(empgrid=uv.grid)
plot(uv, pch=16, col=rgb(0,0,0,0.1), xlim=c(0,1), ylim=c(0,1),
     xlab="U, NONEXCEEDANCE PROBABILITY", ylab="V, NONEXCEEDANCE PROBABILITY")
lines(qua.regressCOP(f=0.5, cop=PSP), lwd=2)
lines(qua.regressCOP(f=0.2, cop=PSP), lwd=2)
lines(qua.regressCOP(f=0.7, cop=PSP), lwd=2)
lines(qua.regressCOP(f=0.1, cop=PSP), lwd=2)
lines(qua.regressCOP(f=0.9, cop=PSP), lwd=2)

med.wrtu <- EMPIRqua.regress(f=0.5, empinv=uv.inv1, lowess=TRUE, f.lowess=0.1)
lines(med.wrtu, col=2, lwd=4)
qua.wrtu <- EMPIRqua.regress(f=0.2, empinv=uv.inv1, lowess=TRUE, f.lowess=0.1)
lines(qua.wrtu, col=2, lwd=2, lty=2)
qua.wrtu <- EMPIRqua.regress(f=0.7, empinv=uv.inv1, lowess=TRUE, f.lowess=0.1)
lines(qua.wrtu, col=2, lwd=2, lty=2)
qua.wrtu <- EMPIRqua.regress(f=0.1, empinv=uv.inv1, lowess=TRUE, f.lowess=0.1)
lines(qua.wrtu, col=2, lwd=2, lty=4)
qua.wrtu <- EMPIRqua.regress(f=0.9, empinv=uv.inv1, lowess=TRUE, f.lowess=0.1)
lines(qua.wrtu, col=2, lwd=2, lty=4)

library(quantreg) # Quantile Regression by quantreg
U <- seq(0.01, 0.99, by=0.01)
rqlm <- rq(V~U, data=uv, tau=0.1)
rq.1 <- rqlm$coefficients[1] + rqlm$coefficients[2]*U
rqlm <- rq(V~U, data=uv, tau=0.2)
rq.2 <- rqlm$coefficients[1] + rqlm$coefficients[2]*U
rqlm <- rq(V~U, data=uv, tau=0.5)
rq.5 <- rqlm$coefficients[1] + rqlm$coefficients[2]*U
rqlm <- rq(V~U, data=uv, tau=0.7)
rq.7 <- rqlm$coefficients[1] + rqlm$coefficients[2]*U
rqlm <- rq(V~U, data=uv, tau=0.9)
rq.9 <- rqlm$coefficients[1] + rqlm$coefficients[2]*U

lines(U, rq.1, col=4, lwd=2, lty=4)
lines(U, rq.2, col=4, lwd=2, lty=2)
lines(U, rq.5, col=4, lwd=4)
lines(U, rq.7, col=4, lwd=2, lty=2)
lines(U, rq.9, col=4, lwd=2, lty=4)#}

\dontrun{ # EXAMPLE 3
uv <- simCOP(n=10000, cop=PSP, ploton=FALSE, points=FALSE)
fakeU <- lmomco::pp(uv[,1], sort=FALSE)
fakeV <- lmomco::pp(uv[,2], sort=FALSE)
uv <- data.frame(U=fakeU, V=fakeV)

uv.grid <- EMPIRgrid(para=uv, deluv=0.1) # CPU hungry
uv.inv1 <- EMPIRgridderinv(empgrid=uv.grid)
uv.inv2 <- EMPIRgridderinv2(empgrid=uv.grid)
plot(uv, pch=16, col=rgb(0,0,0,.1), xlim=c(0,1), ylim=c(0,1),
     xlab="U, NONEXCEEDANCE PROBABILITY", ylab="V, NONEXCEEDANCE PROBABILITY")
lines(qua.regressCOP(f=0.5, cop=PSP), col=2)
lines(qua.regressCOP(f=0.2, cop=PSP), col=2)
lines(qua.regressCOP(f=0.7, cop=PSP), col=2)
lines(qua.regressCOP(f=0.1, cop=PSP), col=2)
lines(qua.regressCOP(f=0.9, cop=PSP), col=2)

med.wrtu <- EMPIRqua.regress(f=0.5, empinv=uv.inv1)
lines(med.wrtu, col=2, lwd=4)
qua.wrtu <- EMPIRqua.regress(f=0.2, empinv=uv.inv1)
lines(qua.wrtu, col=2, lwd=2, lty=2)
qua.wrtu <- EMPIRqua.regress(f=0.7, empinv=uv.inv1)
lines(qua.wrtu, col=2, lwd=2, lty=2)
qua.wrtu <- EMPIRqua.regress(f=0.1, empinv=uv.inv1)
lines(qua.wrtu, col=2, lwd=2, lty=4)
qua.wrtu <- EMPIRqua.regress(f=0.9, empinv=uv.inv1)
lines(qua.wrtu, col=2, lwd=2, lty=4)

lines(qua.regressCOP2(f=0.5, cop=PSP), col=4)
lines(qua.regressCOP2(f=0.2, cop=PSP), col=4)
lines(qua.regressCOP2(f=0.7, cop=PSP), col=4)
lines(qua.regressCOP2(f=0.1, cop=PSP), col=4)
lines(qua.regressCOP2(f=0.9, cop=PSP), col=4)

med.wrtv <- EMPIRqua.regress2(f=0.5, empinv=uv.inv2)
lines(med.wrtv, col=4, lwd=4)
qua.wrtv <- EMPIRqua.regress2(f=0.2, empinv=uv.inv2)
lines(qua.wrtv, col=4, lwd=2, lty=2)
qua.wrtv <- EMPIRqua.regress2(f=0.7, empinv=uv.inv2)
lines(qua.wrtv, col=4, lwd=2, lty=2)
qua.wrtv <- EMPIRqua.regress2(f=0.1, empinv=uv.inv2)
lines(qua.wrtv, col=4, lwd=2, lty=4)
qua.wrtv <- EMPIRqua.regress2(f=0.9, empinv=uv.inv2)
lines(qua.wrtv, col=4, lwd=2, lty=4)#}

\dontrun{ # EXAMPLE 4
# Now try a much more complex shape
# lowess smoothing on quantile regression is possible,
# see next example
para   <- list(alpha=0.15,  beta=0.65,
               cop1=PLACKETTcop, cop2=PLACKETTcop, para1=0.005, para2=1000)
uv <- simCOP(n=20000, cop=composite2COP, para=para)
fakeU <- lmomco::pp(uv[,1], sort=FALSE)
fakeV <- lmomco::pp(uv[,2], sort=FALSE)
uv <- data.frame(U=fakeU, V=fakeV)

uv.grid <- EMPIRgrid(para=uv, deluv=0.025) # CPU hungry
uv.inv1 <- EMPIRgridderinv(empgrid=uv.grid)
uv.inv2 <- EMPIRgridderinv2(empgrid=uv.grid)
plot(uv, pch=16, col=rgb(0,0,0,0.1), xlim=c(0,1), ylim=c(0,1),
     xlab="U, NONEXCEEDANCE PROBABILITY", ylab="V, NONEXCEEDANCE PROBABILTIY")
lines(qua.regressCOP(f=0.5, cop=composite2COP, para=para), col=2)
lines(qua.regressCOP(f=0.2, cop=composite2COP, para=para), col=2)
lines(qua.regressCOP(f=0.7, cop=composite2COP, para=para), col=2)
lines(qua.regressCOP(f=0.1, cop=composite2COP, para=para), col=2)
lines(qua.regressCOP(f=0.9, cop=composite2COP, para=para), col=2)

med.wrtu <- EMPIRqua.regress(f=0.5, empinv=uv.inv1)
lines(med.wrtu, col=2, lwd=4)
qua.wrtu <- EMPIRqua.regress(f=0.2, empinv=uv.inv1)
lines(qua.wrtu, col=2, lwd=2, lty=2)
qua.wrtu <- EMPIRqua.regress(f=0.7, empinv=uv.inv1)
lines(qua.wrtu, col=2, lwd=2, lty=2)
qua.wrtu <- EMPIRqua.regress(f=0.1, empinv=uv.inv1)
lines(qua.wrtu, col=2, lwd=2, lty=4)
qua.wrtu <- EMPIRqua.regress(f=0.9, empinv=uv.inv1)
lines(qua.wrtu, col=2, lwd=2, lty=4)

lines(qua.regressCOP2(f=0.5, cop=composite2COP, para=para), col=4)
lines(qua.regressCOP2(f=0.2, cop=composite2COP, para=para), col=4)
lines(qua.regressCOP2(f=0.7, cop=composite2COP, para=para), col=4)
lines(qua.regressCOP2(f=0.1, cop=composite2COP, para=para), col=4)
lines(qua.regressCOP2(f=0.9, cop=composite2COP, para=para), col=4)

med.wrtv <- EMPIRqua.regress2(f=0.5, empinv=uv.inv2)
lines(med.wrtv, col=4, lwd=4)
qua.wrtv <- EMPIRqua.regress2(f=0.2, empinv=uv.inv2)
lines(qua.wrtv, col=4, lwd=2, lty=2)
qua.wrtv <- EMPIRqua.regress2(f=0.7, empinv=uv.inv2)
lines(qua.wrtv, col=4, lwd=2, lty=2)
qua.wrtv <- EMPIRqua.regress2(f=0.1, empinv=uv.inv2)
lines(qua.wrtv, col=4, lwd=2, lty=4)
qua.wrtv <- EMPIRqua.regress2(f=0.9, empinv=uv.inv2)
lines(qua.wrtv, col=4, lwd=2, lty=4)#}

\dontrun{ # EXAMPLE 5
plot(uv, pch=16, col=rgb(0,0,0,.1), xlim=c(0,1), ylim=c(0,1),
     xlab="U, NONEXCEEDANCE PROBABILITY", ylab="V, NONEXCEEDANCE PROBABILITY")
lines(qua.regressCOP(f=0.5, cop=composite2COP, para=para), col=2)
lines(qua.regressCOP(f=0.2, cop=composite2COP, para=para), col=2)
lines(qua.regressCOP(f=0.7, cop=composite2COP, para=para), col=2)
lines(qua.regressCOP(f=0.1, cop=composite2COP, para=para), col=2)
lines(qua.regressCOP(f=0.9, cop=composite2COP, para=para), col=2)

med.wrtu <- EMPIRqua.regress(f=0.5, empinv=uv.inv1, lowess=TRUE)
lines(med.wrtu, col=2, lwd=4)
qua.wrtu <- EMPIRqua.regress(f=0.2, empinv=uv.inv1, lowess=TRUE)
lines(qua.wrtu, col=2, lwd=2, lty=2)
qua.wrtu <- EMPIRqua.regress(f=0.7, empinv=uv.inv1, lowess=TRUE)
lines(qua.wrtu, col=2, lwd=2, lty=2)
qua.wrtu <- EMPIRqua.regress(f=0.1, empinv=uv.inv1, lowess=TRUE)
lines(qua.wrtu, col=2, lwd=2, lty=4)
qua.wrtu <- EMPIRqua.regress(f=0.9, empinv=uv.inv1, lowess=TRUE)
lines(qua.wrtu, col=2, lwd=2, lty=4)

lines(qua.regressCOP2(f=0.5, cop=composite2COP, para=para), col=4)
lines(qua.regressCOP2(f=0.2, cop=composite2COP, para=para), col=4)
lines(qua.regressCOP2(f=0.7, cop=composite2COP, para=para), col=4)
lines(qua.regressCOP2(f=0.1, cop=composite2COP, para=para), col=4)
lines(qua.regressCOP2(f=0.9, cop=composite2COP, para=para), col=4)

med.wrtv <- EMPIRqua.regress2(f=0.5, empinv=uv.inv2, lowess=TRUE)
lines(med.wrtv, col=4, lwd=4)
qua.wrtv <- EMPIRqua.regress2(f=0.2, empinv=uv.inv2, lowess=TRUE)
lines(qua.wrtv, col=4, lwd=2, lty=2)
qua.wrtv <- EMPIRqua.regress2(f=0.7, empinv=uv.inv2, lowess=TRUE)
lines(qua.wrtv, col=4, lwd=2, lty=2)
qua.wrtv <- EMPIRqua.regress2(f=0.1, empinv=uv.inv2, lowess=TRUE)
lines(qua.wrtv, col=4, lwd=2, lty=4)
qua.wrtv <- EMPIRqua.regress2(f=0.9, empinv=uv.inv2, lowess=TRUE)
lines(qua.wrtv, col=4, lwd=2, lty=4)#}

\dontrun{ # EXAMPLE 6 (changing the smoothing on the lowess)
plot(uv, pch=16, col=rgb(0,0,0,0.1), xlim=c(0,1), ylim=c(0,1),
     xlab="U, NONEXCEEDANCE PROBABILITY", ylab="V, NONEXCEEDANCE PROBABILTIY")
lines(qua.regressCOP(f=0.5, cop=composite2COP, para=para), col=2)
lines(qua.regressCOP(f=0.2, cop=composite2COP, para=para), col=2)
lines(qua.regressCOP(f=0.7, cop=composite2COP, para=para), col=2)
lines(qua.regressCOP(f=0.1, cop=composite2COP, para=para), col=2)
lines(qua.regressCOP(f=0.9, cop=composite2COP, para=para), col=2)

med.wrtu <- EMPIRqua.regress(f=0.5, empinv=uv.inv1, lowess=TRUE, f.lowess=0.1)
lines(med.wrtu, col=2, lwd=4)
qua.wrtu <- EMPIRqua.regress(f=0.2, empinv=uv.inv1, lowess=TRUE, f.lowess=0.1)
lines(qua.wrtu, col=2, lwd=2, lty=2)
qua.wrtu <- EMPIRqua.regress(f=0.7, empinv=uv.inv1, lowess=TRUE, f.lowess=0.1)
lines(qua.wrtu, col=2, lwd=2, lty=2)
qua.wrtu <- EMPIRqua.regress(f=0.1, empinv=uv.inv1, lowess=TRUE, f.lowess=0.1)
lines(qua.wrtu, col=2, lwd=2, lty=4)
qua.wrtu <- EMPIRqua.regress(f=0.9, empinv=uv.inv1, lowess=TRUE, f.lowess=0.1)
lines(qua.wrtu, col=2, lwd=2, lty=4)

lines(qua.regressCOP2(f=0.5, cop=composite2COP, para=para), col=4)
lines(qua.regressCOP2(f=0.2, cop=composite2COP, para=para), col=4)
lines(qua.regressCOP2(f=0.7, cop=composite2COP, para=para), col=4)
lines(qua.regressCOP2(f=0.1, cop=composite2COP, para=para), col=4)
lines(qua.regressCOP2(f=0.9, cop=composite2COP, para=para), col=4)

med.wrtv <- EMPIRqua.regress2(f=0.5, empinv=uv.inv2, lowess=TRUE, f.lowess=0.1)
lines(med.wrtv, col=4, lwd=4)
qua.wrtv <- EMPIRqua.regress2(f=0.2, empinv=uv.inv2, lowess=TRUE, f.lowess=0.1)
lines(qua.wrtv, col=4, lwd=2, lty=2)
qua.wrtv <- EMPIRqua.regress2(f=0.7, empinv=uv.inv2, lowess=TRUE, f.lowess=0.1)
lines(qua.wrtv, col=4, lwd=2, lty=2)
qua.wrtv <- EMPIRqua.regress2(f=0.1, empinv=uv.inv2, lowess=TRUE, f.lowess=0.1)
lines(qua.wrtv, col=4, lwd=2, lty=4)
qua.wrtv <- EMPIRqua.regress2(f=0.9, empinv=uv.inv2, lowess=TRUE, f.lowess=0.1)
lines(qua.wrtv, col=4, lwd=2, lty=4)#}
}
\keyword{empirical copula}
\keyword{empirical copula (quantile regression)}

\encoding{utf8}
\name{joint.curvesCOP}
\alias{joint.curvesCOP}
\title{Compute Coordinates of the Marginal Probabilities given joint AND or OR Probabilities}
\description{
Compute the coordinates of the bivariate marginal probabilities for variables \eqn{U} and \eqn{V} given selected probabilities levels \eqn{t} for a copula \eqn{\mathbf{C}(u,v)} for \eqn{v} with respect to \eqn{u}. For the case of a \bold{joint and} probability, symbolically the solution is
\deqn{\mathrm{Pr}[U \le v,\ V \le v] = t = \mathbf{C}(u,v)\mbox{,}}
where \eqn{U \mapsto [t_i, u_{j}, u_{j+1}, \cdots, 1; \Delta t]} (an irregular sequence of \eqn{u} values from the \eqn{i}th value of \eqn{t_i} provided through to unity) and thus
\deqn{t_i \mapsto \mathbf{C}(u=U, v)\mbox{,}}
and solving for the sequence of \eqn{v}. The index \eqn{j} is to indicate that a separate loop is involved and is distinct from \eqn{i}. The pairings \eqn{\{u(t_i), v(t_i)\}} for each \eqn{t} are packaged as an \R \code{data.frame}. This operation is very similiar to the plotting capabilities in \code{\link{level.curvesCOP}} for \emph{level curves} (Nelsen, 2006, pp. 12--13) but implemented in the function \code{joint.curvesCOP} for alternative utility.

For the case of a \bold{joint or} probability, the \emph{dual of a copula (function)} or \eqn{\tilde{\mathbf{C}}(u,v)} from a copula (Nelsen, 2006, pp. 33--34) is used and symbolically the solution is:
\deqn{\mathrm{Pr}[U \le v \mathrm{\ or\ } V \le v] = t = \tilde{\mathbf{C}}(u,v) = u + v - \mathbf{C}(u,v)\mbox{,}}
where \eqn{U \mapsto [0, u_j, u_{j+1}, \cdots, t_i; \Delta t]} (an irregular sequence of \eqn{u} values from zero through to the \eqn{i}th value of \eqn{t}) and thus
\deqn{t_i \mapsto \tilde{\mathbf{C}}(u=U, v)\mbox{,}}
and solving for the sequence of \eqn{v}.  The index \eqn{j} is to indicate that a separate loop is involved and is distinct from \eqn{i}. The pairings \eqn{\{u(t_i), v(t_i)\}} for each \eqn{t} are packaged as an \R \code{data.frame}.
}
\usage{
joint.curvesCOP(cop=NULL, para=NULL, type=c("and", "or"),
                probs=c(0.5, 0.8, 0.90, 0.96, 0.98, 0.99, 0.995, 0.998),
                zero2small=TRUE, small=1E-6, divisor=100, delu=0.001, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{type}{What type of joint probability is to be computed;}
  \item{probs}{The joint probabilities \eqn{t_i} from which to compute the coordinates. The default values represent especially useful annual return period equivalents that are useful in hydrologic risk analyses;}
  \item{zero2small}{A logical controlling whether exactly zero value for probability are converted to a \code{small} value and exactly unity values for probability are converted to the value \code{1 - small}; this logical is useful if transformation from probability space into standard normal variates or \emph{Gumbel reduced variates} (see function \code{prob2grv()} in package \pkg{lmomco}) is later desired by the user for attendant graphics (see \bold{Examples} section);}
  \item{small}{The value for \emph{small} described for \code{zero2small};}
  \item{divisor}{A divisor on a computation of a \eqn{\Delta t} for incrementing through the irregularly-spaced \eqn{u} domain as part of the coordinate computation (see source code);}
  \item{delu}{A \eqn{\Delta u} for setup of the incrementing through the irregularly-space \eqn{u} domain as part of the coordinate computation (see source code); and}
  \item{...}{Additional arguments to pass to the \code{\link{duCOP}} function of \pkg{copBasic} or \code{uniroot()} function in \R.}
}
\value{
  An \R \code{list} is returned with elements each of the given \code{probs}.
}
\note{
The arguments \code{divisor} and \code{delu} provide flexibility to obtain sufficient smoothness in the coordinate curvatures for a given \eqn{t}. The pairings \eqn{\{u(t_i), v(t_i)\}} for each \eqn{t} packaged as \code{data.fame}s within the returned \code{list} each have their own unique length, and this is the reason that a single \dQuote{master} \code{data.frame} is not returned by this function.

\emph{Extended Example}---The code below shows both types of joint probability being computed using the default \code{probs}. The plotting is made in \emph{Gumbel reduced variates} (GRV; see \code{prob2grv} in package \pkg{lmomco}). This transformation is somewhat suitable for the magnitude variation in and at tail depth of the \code{probs}. Also with transformation is being used, the \code{zero2small} logical is kept at \code{TRUE}, which is appropriate. The \code{zero2small} being set is also useful if standard normal variate transformation (by the \code{qnorm()} function in \R) were used instead.

The \emph{Gumbel--Hougaard} copula (\code{\link{GHcop}}) and a reversed Gumbel--Hougaard copula \code{rGH} are composited together by \code{\link{composite2COP}}. These were chosen so that some asymmetry in the solutions by \code{joint.curvesCOP} could be seen. We begin by specifying symmetrical plotting limits for later use and then creating a function for the reversed Gumbel--Hougaard and setting the parameters and composition weights:
\preformatted{
  grvlim <- lmomco::prob2grv(c(0.25,0.999)) # out to 1,000 years
  "rGHcop" <- function(u,v, ...) { u + v - 1 + GHcop(1-u, 1-v, ...) }
  para <- list(alpha=0.16, beta=0.67, cop1 =GHcop, cop2 =rGHcop,
                                      para1=4.5,   para2=2.2)
  tau    <- tauCOP(    cop=composite2COP, para=para) # Tau    = 0.351219
  nuskew <- nuskewCOP(cop=composite2COP, para=para)  # Nuskew = 0.084262
  UV <- simCOP(n=1000,  cop=composite2COP, para=para, snv=TRUE)
}
The code also computed the \emph{Kendall Tau} (\code{\link{tauCOP}}) and \emph{Nu-Skew} (\code{\link{nuskewCOP}}) and the results shown. The code finishes with a simulation by \code{\link{simCOP}} of the copula composition just for reference.

Next, we compute and plot the joint probability curves. The \code{tol}erance for the \code{uniroot} calls is reset from the \R defaults because slight wooble in the numerical solutions exists otherwise.  The \code{AND} and \code{OR} \code{lists} each provide \code{data.frame}s from which successive graphic calls plot the lines. The second \code{plot()} call is commented out so that both sets of joint probability curves are drawn on the same plot.
\preformatted{
  AND <- joint.curvesCOP(cop=composite2COP, para=para, type="and",
                         divisor=1000, tol=.Machine$double.eps)
  plot(grvlim, grvlim, type="n",
       xlab="GUMBEL REDUCED VARIATE IN U", ylab="GUMBEL REDUCED VARIATE IN V")
  for(t in sort(as.numeric(names(AND)))) {
      UV <- get(as.character(t), AND)
      lines(lmomco::prob2grv(UV$U),         lmomco::prob2grv(       UV$V))
      text( lmomco::prob2grv(median(UV$U)), lmomco::prob2grv(median(UV$V)),
           as.character(round(1/(1-t)), digits=0))
  }

  OR <- joint.curvesCOP(cop=composite2COP, para=para, type="or",
                        divisor=1000, tol=.Machine$double.eps)
  for(t in sort(as.numeric(names(OR)))) {
     UV <- get(as.character(t), OR)
     lines(lmomco::prob2grv(UV$U), lmomco::prob2grv(UV$V), col=2)
     text( lmomco::prob2grv(median(UV$U)), lmomco::prob2grv(median(UV$V)),
          as.character(round(1/(1-t)), digits=0), col=2)
  }
  mtext("Return Periods: black=cooperative risk, red=dual risk")
  abline(0,1, lty=2) # dash line is simply and equal value line
}

\emph{Black Curves}---The black curves represent the nonexceedance \bold{joint and} condition. The black curves are a form of \emph{level curve} (see \code{\link{level.curvesCOP}}), but it seems appropriate to not name them as such because Nelsen's examples and others usually have the level curves on an even step interval of probability such as 10-percent level curves. The complement of nonexceedance \bold{joint and} is the probability level that either or both random variables (say \dQuote{hazards}) \eqn{U} or \eqn{V} causes \dQuote{failure} at the respective return period.

\emph{Red Curves}---The red curves represent the nonexceedance \bold{joint or} (inclusive) condition. The complement of nonexceedance \bold{joint or} (inclusive) is the probability level that both random variables (say \dQuote{hazards}) \eqn{U} or \eqn{V} must simultaneously (or dually) occur for \dQuote{failure} at the respective return period. Note, there is obviously asymmetry in the \bold{joint or} curves.

\emph{Interpretation}---Because the two hazards can \dQuote{cooperate} to cause failure (see \code{\link{coCOP}}) for an equal level of protection (say 500-year event) relative to the complement of nonexceedance \bold{joint or} (inclusive) condition (see \code{surCOP}), the marginal probabilities must be considerably higher. Users are encouraged to review \code{\link{copBasic-package}} and the figure therein.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{diagCOPatf}}, \code{\link{duCOP}}, \code{\link{jointCOP}}, \code{\link{joint.curvesCOP2}}, \code{\link{level.curvesCOP}}
}
\examples{
# See Note section
}
\keyword{joint probability}

\encoding{utf8}
\name{statTn}
\alias{statTn}
\title{The Tn Statistic of a Fitted Copula to an Empirical Copula}
\description{
Compute the \eqn{T_n(p)} statistic of Genest \emph{et al.} (2011) that is defined as
\deqn{T_n(p) = \sum_{i=1}^n \big|\mathbf{C}_n(u_i, v_i) - \mathbf{C}_{\Theta_n}(u_i, v_i)\big|^p\mbox{,}}
where \eqn{\mathbf{C}_n(u,v)} is the \emph{empirical copula}, \eqn{\mathbf{C}_{\Theta_n}(u,v)} is the \emph{fitted copula} with estimated parameters \eqn{\Theta_n} from the sample of size \eqn{n}. The \eqn{T_n} for \eqn{p = 2} is reported by those authors to be of general purpose and overall performance in large scale simulation studies. The extension here for arbitrary exponent \eqn{p} is made for flexibility. Alternatively the definition could be associated with the statistic \eqn{T_n(p)^{1/p}} in terms of a root \eqn{1/p} of the summation as shown above.

The \eqn{T_n} statistic is obviously a form of deviation between the empirical (nonparametric) and parametric fitted copula. The distribution of this statistic through Monte Carlo simulation could be used for inference. The inference is based on that a chosen parametric model is suitably close to the empirical copula. The \eqn{T_n(p)} statistic has an advantage of being relatively straightforward to understand and explain to stakeholders and decision makers, is attractive for being suitable in a wide variety of circumstances, but intuitively might have limited statistical power in some situations for it looks at whole copula structure and not say at tail dependency. Finally, other goodness-of-fits using the squared differences between \eqn{\mathbf{C}_n(u,v)} and \eqn{\mathbf{C}_{\Theta_m}(u, v)} are \code{\link{aicCOP}}, \code{\link{bicCOP}}, and \code{\link{rmseCOP}}.
}
\usage{
statTn(u, v=NULL, cop=NULL, para=NULL, p=2, proot=FALSE, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction. If not given, then a second column from argument \code{u} is attempted;}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{p}{The value for \eqn{p}, and the default follows that of Genest \emph{et al.} (2011);}
  \item{proot}{A logical controling whether the \eqn{T_n} returned be rooted by \eqn{1/p}, and the default follows that of Genest \emph{et al.} (2011); and}
  \item{...}{Additional arguments to pass to the copula function and (or) the empirical copula.}
}
\value{
  The value for \eqn{T_n} is returned dependent on the specification of \eqn{p} and whether rooting of the result is desired.
}
\note{
The \bold{Examples} section shows a simple computation of the \eqn{\hat T_n} statistic for a sample and a fitted copula to that sample. Ideally \code{statTn} would be wrapped in a Monte Carlo process of fitting the apparent \dQuote{parent} distribution from the sample data, then for some large replication count, generate \eqn{N} samples of size \eqn{n} from the parent and from these samples compute the empirical copula and also fit parameter(s) of the chosen copula and repeatedly solve for \eqn{T_n}. Given a total of \eqn{N} values of \eqn{T_n}, then the sample \eqn{T_n} or \eqn{\hat{T}_n} can be compared to the distribution, and if \eqn{\hat{T}_n} is greater than say the 95th percentile, then the assumed form of the copula could be rejected.

The \code{distTn} defined below and is dependent on the \code{\link{copBasic.fitpara.beta}} function can be used to demonstrate concepts. (The process is complex enough that user-level implementation of \code{distTn} in \pkg{copBasic} is not presently (2019) thought appropriate.)
\preformatted{
  "distTn" <- function(n, N=1000, statf=NULL,
                       cop=NULL, para=para, interval=NULL, ...) {
      opts <- options(warn=-1)
      message("Estimating Tn distribution: ", appendLF=FALSE)
      Tn <- vector(mode="numeric", N)
      for(i in 1:N) {
         showi <- as.logical(length(grep("0+$", i, perl=TRUE)))
         if(showi) message(i, "-", appendLF=FALSE)
         ruv <- simCOP(n=n, cop=cop, para=para, graphics=FALSE, ...)
         rpara <- copBasic.fitpara.beta(ruv, statf=statf,
                             interval=interval, cop=cop)
         Tn[i] <- ifelse(is.na(rpara), NA, statTn(ruv, cop=cop, para=rpara))
      }
      numNA <- length(Tn[is.na(Tn)])
      message("done: Number of failed parameter estimates=", numNA)
      options(opts)
      return(Tn[! is.na(Tn)])
  }
}

Let us imagine an \eqn{n=400} sample size of a \emph{Galambos copula} (\eqn{\mathbf{GL}(u,v)}; \code{\link{GLcop}}) and then treat the \emph{Plackett copula} (\eqn{\mathbf{PL}(u,v)}; \code{\link{PLACKETTcop}}) as the proper (chosen) model. The estimated parameter by the sample \emph{Blomqvist Beta} of \eqn{\hat\beta_\mathbf{C} = 0.64} using the \code{\link{blomCOP}} function called from within \code{copBasic.fitpara.beta} is then placed in variable \code{para}. The \eqn{\hat\beta_\mathbf{C}} is not the most efficient estimator but for purposes here, but it is fast.  The parameter for the given seed is estimated as about \eqn{\mathbf{PL}(\hat\Theta{=}20.75)}.
\preformatted{
  n <- 400 # sample size
  correctCopula <- GLcop; set.seed(1596)
  sampleUV <- simCOP(n=n, cop=correctCopula, para=1.9) # a random sample
  para.correctCopula <- copBasic.fitpara.beta(uv=sampleUV, statf=blomCOP,
                                interval=c(1,5),      cop=correctCopula)
  chosenCopula <- PLACKETTcop
  para <- copBasic.fitpara.beta(uv=sampleUV, statf=blomCOP,
                                interval=c(.001,200), cop=chosenCopula )
}

Next, compute the sample \eqn{\hat T_n = 0.063} from \code{sampleUV}. The distribution of the \eqn{T_n} is estimated using the \code{distTn} function, and an estimate of the \eqn{\hat T_n} p-value is in turn estimated. A large simulation run \eqn{N = 1{,}000} for a sample of size of \eqn{n = 400} is selected. The \code{distTn} function internally will simulated for \code{N}-replicates from the assumed parent and estimate the parameter. A computation run yields a p-value of approximately 0.01 (depending upon the seed) and is statistically significant at an alpha of 0.05, and therefore, the \eqn{\mathbf{PL}(\Theta{=}20.75)} should be rejected for fitting to these data.
\preformatted{
  sampleTn   <- statTn(sampleUV, cop=chosenCopula, para=para)
  Tns        <- distTn(n=n,      cop=chosenCopula, para=para,
                       interval=c(0.001, 100), statf=blomCOP)
  Tns_pvalue <- 1 - sum(Tns <= sampleTn) / length(Tns) # estimate p-value
}

The demonstration is furthered with a check on the \emph{Kullback--Leibler sample size} \eqn{n_{f\!g}} at the 5-percent significance level (alpha = 0.05) by the \code{\link{kullCOP}} function, which yields \eqn{100}. Given the parent copula as \eqn{\mathbf{GL}(\Theta{=}1.9)}, therefore, it would take approximately 100 samples to distinguish between that copula and a \eqn{\mathbf{PL}(\Theta{=}20.75)} where in this case the fit was through the \eqn{\hat\beta_\mathbf{C} = 0.64}.
\preformatted{
  kullCOP(cop1=correctCopula, para1=1.9,
          cop2=chosenCopula,  para2=para)$KL.sample.size # KL sample size = 100
  vuongCOP(sampleUV, cop1=correctCopula, para1=para.correctCopula,
                     cop2=chosenCopula,  para2=para)$message
  # [1] "Copula 1 has better fit than Copula 2 at 100x(1-alpha) level"
}
The available sample size \eqn{n = 400} is then about four times larger than \eqn{n_{f\!g}} so the sample size \eqn{n} should be sufficient to judge goodness-of-fit. This is a large value but with the sample variability of \eqn{\hat\beta_\mathbf{C}}, it seems that other measures of association such as \emph{Spearman Rho} (\code{\link{rhoCOP}}) or \emph{Kendall Tau} (\code{\link{tauCOP}}) and others cross-referenced therein might be preferable.

The prior conclusion is supported by the p-value of the \eqn{\hat T_n} being about 0.01, which suggests that the \eqn{\mathbf{PL}(u,v)} is not a good model of the available sample data in \code{sampleUV}. Lastly, these judgments are consistent with the \emph{Vuoug Procedure} performed by the \code{\link{vuongCOP}} function, which reports at the 5-percent significance level that \dQuote{copula number 1}---in this case, the \eqn{\mathbf{GL}(u,v)}---has the better fit, and this is obviously consistent with the problem setup because the random sample for investigation was drawn from the Galambos coupla (the parent form).
}
\references{
Genest, C., Kojadinovic, I., \enc{Nešlehová}{Neslehova}, J., and Yan, J., 2011, A goodness-of-fit test for bivariate extreme-value copulas: Bernoulli, v. 17, no. 1, pp. 253--275.
}
\author{W.H. Asquith}
\seealso{\code{\link{aicCOP}}, \code{\link{bicCOP}}, \code{\link{rmseCOP}}, \code{\link{vuongCOP}}, \code{\link{kullCOP}}
}
\examples{
\dontrun{
# Example here is just for Tn. For the example below, the PSP copula is quite different
# from the Gumbel-Hougaard copula and thus, the hatTn would be expected to be different
# from those of the Gumbel-Hougaard and certainly not too near to zero.
samUV  <- simCOP(n=60, cop=PSP, graphics=FALSE, seed=1)   # random sample
hatTau <- cor(samUV$U, samUV$V, method="kendall")         # Kendall Tau
hatTn  <- statTn(samUV, cop=GHcop, para=GHcop(tau=hatTau)$para,
                 ctype="bernstein", bernprogress=TRUE)    # 0.03328789
# hatTn in this case is by itself is somewhat uninformative and requires
# Monte Carlo to put an individual value into context.}
}
\keyword{copula (goodness-of-fit)}
\keyword{copula (inference)}
\keyword{inference}
\keyword{goodness-of-fit}

\encoding{utf8}
\name{N4212cop}
\alias{N4212cop}
\title{The Copula of Equation 4.2.12 of Nelsen's Book}
\description{
The \emph{N4212 copula} (Nelsen, 2006, p. 91; eq. 4.2.12) is named by the author (Asquith) for the \pkg{copBasic} package and is defined as
\deqn{\mathbf{C}_{\mathrm{N4212}}(u,v; \Theta) = \biggl(1 + \bigl[(u^{-1} -1)^\Theta + (v^{-1} -1)^\Theta\bigr]^{1/\Theta}\biggr)^{-1}\mbox{.}}

The \eqn{\mathbf{N4212}(u,v)} copula is \emph{not comprehensive} because for \eqn{\Theta = 1} the copula becomes the so-called \eqn{\mathbf{PSP}(u,v)} copula (see \code{\link{PSP}}) and as \eqn{\Theta \rightarrow \infty} the copula becomes \eqn{\mathbf{M}(u,v)} (see \code{\link{M}}). The copula is undefined for \eqn{\Theta < 1}. The N4212 copula has respective \emph{lower-} and \emph{upper-tail dependency} (see \code{\link{taildepCOP}}).

Although \pkg{copBasic} is intended to not implement or \dQuote{store house} the enormous suite of copula functions available in the literature, the N4212 copula is included to give the package another copula to test or test in numerical examples.
}
\usage{
N4212cop(u, v, para=NULL, infis=100, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (single element) of parameters---the \eqn{\Theta} parameter of the copula;}
  \item{infis}{What is infinity? Testing shows that \code{infis =} \eqn{\Theta > 100} is about right to consider the copula as becoming \eqn{\mathbf{M}(u,v)} (see \code{\link{M}}); and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\examples{
N4212cop(0.4,0.6, para=1) == PSP(0.4,0.6) # TRUE
N4212cop(0.4,0.6, para=10) # 0.3999928
taildepCOP(cop=N4212cop, para=10) # LamL = 0.93303; LamU = 0.92823
\dontrun{
D <- simCOP(n=400, cop=N4212cop, para=2)
D <- simCOP(n=400, cop=N4212cop, para=10,  ploton=FALSE, col=2)
D <- simCOP(n=400, cop=N4212cop, para=100, ploton=FALSE, col=3)#}
}
\keyword{copula (formulas)}
\keyword{copula}

\encoding{utf8}
\name{joeskewCOP}
\alias{joeskewCOP}
\alias{nuskewCOP}
\alias{nustarCOP}
\title{Joe's Nu-Skew and the copBasic Nu-Star of a Copula}
\description{
Compute the measure of \emph{permutation asymmetry}, which can be thought of as \emph{bivariate skewness}, named for the \pkg{copBasic} package as \emph{Nu-Skew} \eqn{\nu_\mathbf{C}} of a copula according to Joe (2014, p. 66) by
\deqn{\nu_\mathbf{C} = 3\mathrm{E}[UV^2 - U^2V] = 6\int\!\!\int_{\mathcal{I}^2} (v-u)\mathbf{C}(u,v)\, \mathrm{d}u\mathrm{d}v\mbox{.}}
This definition is effectively the \code{type="nu"} for the function for which the multiplier \eqn{6} has been converted to \eqn{96} as explained in the \bold{Note}.

Numerical results indicate \eqn{\nu_\mathbf{W} \approx 0} (\code{\link{W}}), \eqn{\nu_\mathbf{\Pi} = 0} (\code{\link{P}}), \eqn{\nu_\mathbf{M} \approx 0} (\code{\link{M}}), \eqn{\nu_\mathbf{PL} \approx 0} for all \eqn{\Theta} (\code{\link{PLcop}}), and the \eqn{\nu^\star_\mathbf{GH} = 0} (\code{\link{GHcop}}); copulas with mirror symmetry across the equal value line have \eqn{\nu_\mathbf{C} = 0}.

Asymmetric copulas do exist. For example, consider an asymmetric Gumbel--Hougaard \eqn{\mathbf{GH}} copula with \eqn{\Theta_p = (5,0.8,p)}:
\preformatted{
  optimize(function(p) { nuskewCOP(cop=GHcop, para=c(5,0.8, p)) },
           c(0,0.99) )$minimum
  UV <- simCOP(n=10000, cop=GHcop, c(5,0.8, 0.2836485)) # inspect the graphics
  48*mean(UV$U*$V^2 - UV$U^2*UV$V) # -0.2847953 (not the 3rd parameter)
}
The minimization yields \eqn{\nu_{\mathbf{GH}(5, 0.8, 0.2836485)} = -0.2796104}, which is close the expectation computed where \eqn{48 = 96/2}.

A complementary definition is supported, triggered by \code{type="nustar"}, and is computed by
\deqn{\nu^\star_\mathbf{C} = 12\int\!\!\int_{\mathcal{I}^2} (v+u)\mathbf{C}(u,v)\, \mathrm{d}u\mathrm{d}v - 4\mbox{,}}
which has been for the \pkg{copBasic} package, \eqn{\nu^\star_\mathbf{C}} is named as \emph{Nu-Star}, which the \eqn{12} and the \eqn{-4} have been chosen so that numerical results indicate \eqn{\nu^\star_\mathbf{W} = -1} (\code{\link{W}}), \eqn{\nu^\star_\mathbf{\Pi} = 0} (\code{\link{P}}), and \eqn{\nu^\star_\mathbf{M} = +1} (\code{\link{M}}).

Lastly, the \code{\link{uvlmoms}} function provides for a quantile-based measure of bivariate skewness based on the difference \eqn{U - V} that also is discussed by Joe (2014, p. 66).
}
\usage{
joeskewCOP(cop=NULL, para=NULL, type=c("nu", "nustar", "nuskew"),
                               as.sample=FALSE, brute=FALSE, delta=0.002, ...)

nuskewCOP(cop=NULL, para=NULL, as.sample=FALSE, brute=FALSE, delta=0.002, ...)
nustarCOP(cop=NULL, para=NULL, as.sample=FALSE, brute=FALSE, delta=0.002, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{type}{The type of metric to compute (\code{nu} and \code{nuskew} are synonymous for \eqn{\nu_\mathbf{C}} and \code{nustar} is for \eqn{\nu^\star_\mathbf{C}});}
  \item{brute}{Should brute force be used instead of two nested \code{integrate()} functions to perform the double integration;}
  \item{delta}{The \eqn{\mathrm{d}u} and \eqn{\mathrm{d}v} for the brute force integration using \code{brute};}
  \item{as.sample}{A logical controlling whether an optional \R \code{data.frame} in \code{para} is used to compute the sample \eqn{\hat\nu} or \eqn{\hat\nu^\star} (see \bold{Note}). If set to \code{-1}, then the message concerning CPU effort will be surpressed; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  The value for \eqn{\nu_\mathbf{C}} or \eqn{\nu^\star_\mathbf{C}} is returned.
}
\details{
The implementation of \code{joeskewCOP} for \pkg{copBasic} provides the second metric of asymmetry, but why?  Consider the results that follow:
\preformatted{
  joeskewCOP(cop=GHcop, para=c(5, 0.8,    0.2836485), type="nu")
     # -0.2796104
  joeskewCOP(cop=GHcop, para=c(5, 0.2836485,    0.8), type="nu")
     # +0.2796103
  joeskewCOP(cop=GHcop, para=c(5, 0.8,    0.2836485), type="nu")
     #  0.3571276
  joeskewCOP(cop=GHcop, para=c(5, 0.2836485,    0.8), type="nu")
     #  0.3571279
  tauCOP(    cop=GHcop, para=c(5, 0.2836485,    0.8))
     #  0.2443377
}
The demonstration shows---at least for the symmetry (switchability) of the 2nd and 3rd parameters (\eqn{\pi_2} and \eqn{\pi_3}) of the asymmetric \eqn{\mathbf{GH}} copula---that the first definition \eqn{\nu} is magnitude symmetric but carries a sign change. The demonstration shows magnitude and sign stability for \eqn{\nu^\star}, and ends with \emph{Kendall Tau} (\code{\link{tauCOP}}). Collectively, Kendall Tau (or the other \emph{symmetric measures of association}, \emph{e.g.} \code{\link{blomCOP}}, \code{\link{footCOP}}, \code{\link{giniCOP}}, \code{\link{hoefCOP}}, \code{\link{rhoCOP}}, \code{\link{wolfCOP}}) when combined with \eqn{\nu} and \eqn{\nu^\star} might provide a framework for parameter optimization of the asymmetric \eqn{\mathbf{GH}} copula (see below).

The asymmetric \eqn{\mathbf{GH}_{(5, 0.2836485, 0.8)}} is not radial (\code{\link{isCOP.radsym}}) or permutation (\code{\link{isCOP.permsym}}), but if \eqn{\pi_2 = \pi_3} then the resulting \eqn{\mathbf{GH}} copula is not radially symmetric but is permutation symmetric:
\preformatted{
  isCOP.radsym( cop=GHcop, para=c(5, 0.2836485, 0.8)) # FALSE
  isCOP.permsym(cop=GHcop, para=c(5, 0.2836485, 0.8)) # FALSE
  isCOP.radsym( cop=GHcop, para=c(5, 0.8,       0.8)) # FALSE
  isCOP.permsym(cop=GHcop, para=c(5, 0.8,       0.8)) # TRUE
}

The use of \eqn{\nu_\mathbf{C}} and \eqn{\nu^\star_\mathbf{C}} with a \emph{measure of association} is just suggested above for parameter optimization. Suppose we have \eqn{\mathbf{GH}_{(5,0.5,0.7)}} with \emph{Spearman Rho} \eqn{\rho = 0.4888}, \eqn{\nu = 0.001475}, and \eqn{\nu^\star = 0.04223}, and the asymmetric \eqn{\mathbf{GH}} coupla is to be fit. Parameter estimation for the asymmetric \eqn{\mathbf{GH}} is accomplished by
\preformatted{
  "fitGHcop" <- function(hats, assocfunc=rhoCOP, init=NA, eps=1E-4, ...) {
     H <- GHcop # shorthand for the copula
     "objfunc" <- function(par) {
        par[1]   <- ifelse(par[1] < 1, return(Inf), exp(par[1])) # edge check
        par[2:3] <-  pnorm(par[2:3]) # detransform
        hp <- c(assocfunc(H, par), nuskewCOP(H, par), nustarCOP(H, par))
        return(sum((hats-hp)^2))
     }
     # Theta=1 and Pi2 = Pi3 = 1/2 # as default initial estimates
     if(is.na(init)) init <- c(1, rep(1/2, times=2))
     opt  <- optim(init, objfunc, ...); par <- opt$par
     para <- c( exp(par[1]), pnorm(par[2:3]) )
     names(para) <- c("Theta", "Pi2", "Pi3")
     fit <- c(assocfunc(H, para), nuskewCOP(H, para), nustarCOP(H, para))
     txt <- c("AssocMeasure", "NuSkew", "NuStar")
     names(fit) <- txt; names(hats) <- txt
     if(opt$value > eps) warning("inspect the fit")
     return(list(para=para, fit=fit, given=hats, optim=opt))
  }
  father <- c(5,.5,.7)
  densityCOPplot(cop=GHcop, para=father, contour.col=8)
  fRho  <- rhoCOP(   cop=GHcop, father)
  fNu   <- nuskewCOP(cop=GHcop, father)
  fStar <- nustarCOP(cop=GHcop, father)

  child <- fitGHcop(c(fRho, fNu, fStar))$para
  densityCOPplot(cop=GHcop, para=child, ploton=FALSE)

  cRho  <- rhoCOP(   cop=GHcop, child)
  cNu   <- nuskewCOP(cop=GHcop, child)
  cStar <- nustarCOP(cop=GHcop, child)
  message("Father stats: ", paste(fRho, fNu, fStar, sep=", "))
  message("Child  stats: ", paste(cRho, cNu, cStar, sep=", "))
  message("Father para: ",  paste(father,      collapse=", "))
  message("Child  para: ",  paste(child,       collapse=", "))
}

The initial parameter estimate has the value \eqn{\Theta = 1}, which is \emph{independence} for the one parameter \eqn{\mathbf{GH}}. The two other parameters are set as \eqn{\pi_2 = \pi_3 = 1/2} to be in the mid-point of their domain. The transformations using the \code{log()} \eqn{\leftrightarrow} \code{exp()} and \code{qnorm()} \eqn{\leftrightarrow} \code{pnorm()} functions in \R are used to keep the optimization in the viable parameter domain. The results produce a fitted copula of \eqn{\mathbf{GH}_{(4.907, 0.5006, 0.7014)}}. This fit aligns well with the parent, and the three statistics are essentially matched during the fitting.

The \eqn{\nu^\star_\mathbf{C}} can be similar to \code{\link{rhoCOP}}, but differences do exist. In the presence of radial symmetry, (\eqn{\nu_\mathbf{C} == 0}), the \eqn{\nu^\star_\mathbf{C}} is nearly equal to \emph{Spearman Rho} for some copulas. Let us test further:
\preformatted{
  p <- 10^seq(0,2,by=.01)
  s <- sapply(p, function(t) nustarCOP(cop=GHcop, para=c(t)))
  r <- sapply(p, function(t)    rhoCOP(cop=GHcop, para=c(t)))
  plot(p,s, log="x", type="l", col=3, lwd=3); lines(p,r)
}
Now let us add some asymmetry
\preformatted{
  s <- sapply(p, function(t) nustarCOP(cop=GHcop, para=c(t, 0.25, 0.75)))
  r <- sapply(p, function(t)    rhoCOP(cop=GHcop, para=c(t, 0.25, 0.75)))
  plot(p,s, log="x", type="l", col=3, lwd=3); lines(p,r)
}
Now let us choose a different (the \emph{Clayton}) copula
\preformatted{
  s <- sapply(p, function(t) nustarCOP(cop=CLcop, para=c(t)))
  r <- sapply(p, function(t)    rhoCOP(cop=CLcop, para=c(t)))
  plot(p,s, log="x", type="l", col=3, lwd=3); lines(p,r)
}
}
\note{
The \eqn{\nu_\mathbf{C}} definition is given with a multiplier of \eqn{6} on the integrals in order to agree with Joe (2014) relation that is also shown. However, in mutual parameter estimation experiments using a simple sum-of-square errors as shown in the \bold{Details}, it is preferred to have \eqn{\nu_\mathbf{C}} measured on a larger scale. Where does the \eqn{96} then come from? It is heuristically made so that the upright and rotated \code{cophalf} (see \bold{Examples} under \code{\link{asCOP}} and \code{\link{bilmoms}} for this copula) acquires \eqn{\nu_\mathbf{C}} values of \eqn{+1} and \eqn{-1}, respectively. As a result to make back comparisons to Joe results, the ratios of \eqn{96} are made in this documentation.

The source code shows slightly different styles of division by the sample size as part of the sample estimation of the statistics. The \eqn{\hat\nu} using just division by the sample size as testing indicates that this statistic is reasonably unbiased for simple copula. The \eqn{\hat\nu^\star} with similar division is a biased statistic and the bias is not symmetrical in magnitude or sign it seems whether the \eqn{\hat\nu^\star} is positive or negative. The salient code is \code{spm <- ifelse(corsgn == -1, +2.4, +1.1)} within the sources for which the corrections were determined heuristically through simulation, and \code{corsgn} is the sign of the sample Spearman Rho through the \code{cor()} function of \R.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{uvskew}}, \code{\link{blomCOP}}, \code{\link{footCOP}}, \code{\link{giniCOP}},
\code{\link{hoefCOP}}, \code{\link{rhoCOP}}, \code{\link{tauCOP}}, \code{\link{wolfCOP}}
}
\examples{
nuskewCOP(cop=GHcop,para=c(1.43,1/2,1))*(6/96) # 0.005886 (Joe, 2014, p. 184; 0.0059)

\dontrun{
joeskewCOP(            cop=GHcop, para=c(8, .7, .5)) # -0.1523491
joeskewCOP(            cop=GHcop, para=c(8, .5, .7)) # +0.1523472
# UV <- simCOP(n=1000, cop=GHcop, para=c(8, .7, .5)) # see the switch in
# UV <- simCOP(n=1000, cop=GHcop, para=c(8, .5, .7)) # curvature}

\dontrun{
para=c(19,0.3,0.8); set.seed(341)
nuskew <-  nuskewCOP( cop=GHcop, para=para) # 0.3057744
UV <- simCOP(n=10000, cop=GHcop, para=para) #   a large simulation
mean((UV$U - UV$V)^3)/(6/96)                # 0.3127398

# Two other definitions of skewness follow and are not numerically the same.
uvskew(u=UV$U, v=UV$V, umv=TRUE)  # 0.3738987  (see documentation uvskew)
uvskew(u=UV$U, v=UV$V, umv=FALSE) # 0.3592739  ( or documentation uvlmoms)
# Yet another definition of skew, which requires large sample approximation
# using the L-comoments (3rd L-comoment is L-coskew).
lmomco::lcomoms2(UV)$T3 # L-coskew of the simulated values [1,2] and [2,1]
#             [,1]        [,2]
#[1,]  0.007398438  0.17076600
#[2,] -0.061060260 -0.00006613
# See the asymmetry in the two L-coskew values and consider this in light of
# the graphic produced by the simCOP() called for n=10,000. The T3[1,1] is
# the sampled L-skew (univariate) of the U margin and T3[2,2] is the same
# but for the V margin. Because the margins are uniform (ideally) then these
# for suitable large sample must be zero because the L-skew of the uniform
# distribution is by definition zero.
#
# Now let us check the sample estimator for sample of size n=300, and the
# t-test will (should) result in acceptance of the NULL hypothesis.
S <- replicate(60, nuskewCOP(para=simCOP(n=300, cop=GHcop, para=para,
                                         graphics=FALSE), as.sample=TRUE))
t.test(S, mu=nuskew)
# t = 0.004633, df = 59, p-value = 0.9963
# alternative hypothesis: true mean is not equal to 0.3057744
# 95 percent confidence interval:
#  0.2854282 0.3262150
# sample estimates:
# mean of x
# 0.3058216 }

\dontrun{
# Let us run a large ensemble of copula properties that use the whole copula
# (not tail properties). We composite a Plackett with a Gumbel-Hougaard for
# which the over all association (correlation) sign is negative, but amongst
# these statistics with nuskew and nustar at the bottom, there are various
# quantities that can be extracted. These could be used for fitting.
set.seed(873)
para <- list(cop1=PLcop, cop2=GHcop, alpha=0.6, beta=0.9,
             para1=.005, para2=c(8.3,0.25,0.7))
UV <- simCOP(1000, cop=composite2COP, para=para) # just to show
  blomCOP(composite2COP, para)            # -0.4078657
  footCOP(composite2COP, para)            # -0.2854227
  hoefCOP(composite2COP, para)            # +0.5713775
  lcomCOP(composite2COP, para)$lcomUV[3]  # +0.1816084
  lcomCOP(composite2COP, para)$lcomVU[3]  # +0.1279844
   rhoCOP(composite2COP, para)            # -0.5688417
rhobevCOP(composite2COP, para)            # -0.2005210
   tauCOP(composite2COP, para)            # -0.4514693
  wolfCOP(composite2COP, para)            # +0.5691933
nustarCOP(composite2COP, para)            # -0.5172434
nuskewCOP(composite2COP, para)            # +0.0714987 }
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{permutation asymmetry}
\keyword{radial asymmetry}
\concept{copula skew}
\keyword{Moments of combined U and V}
\keyword{bivariate skewness}
\keyword{Joe (2014) Examples and Exercises}
\keyword{Nu-Skew and Nu-Star (skewnesses)}

\encoding{utf8}
\name{wolfCOP}
\alias{wolfCOP}
\title{Schweizer and Wolff Sigma of a Copula}
\description{
Compute the measure of association known as \emph{Schweizer--Wolff Sigma} \eqn{\sigma_\mathbf{C}} of a copula according to Nelsen (2006, p. 209) by

\deqn{\sigma_\mathbf{C} = 12\int\!\!\int_{\mathcal{I}^2} \bigl|\mathbf{C}(u,v) - uv\bigr|\,\mathrm{d}u\mathrm{d}v\mbox{,}}

which is \eqn{0 \le \sigma_\mathbf{C} \le 1}. It is obvious that this measure of association, without the positive sign restriction, is similar to the following form of \emph{Spearman Rho} (\code{\link{rhoCOP}}) of a copula:

\deqn{\rho_\mathbf{C} = 12\int\!\!\int_{\mathcal{I}^2} \bigl[\mathbf{C}(u,v) - uv\bigr]\,\mathrm{d}u\mathrm{d}v\mbox{.}}

If a copula is \emph{positively quadrant dependent} (PQD, see \code{\link{isCOP.PQD}}), then \eqn{\sigma_\mathbf{C} = \rho_\mathbf{C}}; conversely if a copula is \emph{negatively quadrant dependent} (NQD), then \eqn{\sigma_\mathbf{C} = -\rho_\mathbf{C}}. However, a feature making \eqn{\sigma_\mathbf{C}} especially attractive is that for random variables \eqn{X} and \eqn{Y}, which are not PQD or NQD---copulas that are neither larger nor smaller than \eqn{\mathbf{\Pi}}---is that \dQuote{\eqn{\sigma_\mathbf{C}} is often a better measure of [dependency] than \eqn{\rho_\mathbf{C}}} (Nelsen, 2006, p. 209).

\enc{Úbeda-Flores}{Ubeda-Flores} (2022) study the maximum difference between \eqn{\sigma_\mathbf{C} - |\rho_\mathbf{C}| \approx 0.60496} for a special class of singular copula (see \bold{Examples}), but states that it \dQuote{remains an open problem to check if, for any copula \eqn{\mathbf{C}_\Theta},} that the maximum listed here is the maximum difference possible.

Li \emph{et al.} (2019) compare \eqn{\sigma_\mathbf{C}} (they label it \eqn{\delta^\mathrm{E}_i}, \dQuote{the extended delta index}) to alternative measures of dependency with emphasis on information content. Those authors also show that dependence can also be measured by

\deqn{\delta_i = \frac{1}{2}\int\!\!\int_{\mathcal{I}^2} \bigl|c(u,v) - 1\bigr|\,\mathrm{d}u\mathrm{d}v\mbox{,}}

where \eqn{c(u,v)} is a \emph{copula density} (\code{\link{densityCOP}}). Li \emph{et al.} (2019, p. 45) \dQuote{measure how much the model output \eqn{Y} and the input \eqn{X_i} are dependent,} and \dQuote{importance of the inputs globally and
quantitatively with model free and moment independent properties.}

By definition for two random variables \eqn{U} and \eqn{V} with cumulative distribution functions of \eqn{F(u)} and \eqn{F(v)}, the variables are \emph{independent}, if and only if, for all subsets (conditions) selecting values \eqn{u} and \eqn{y} of \eqn{U \le u} and \eqn{V \le v}, when for all \eqn{u} and \eqn{v}, the combined cumulative distribution function is

\deqn{H(x, y) = F(x)G(x) = \mathbf{C}(u,v) = uv\mbox{.}}

Note, structural similarity between \eqn{\sigma_\mathbf{C}} and the definition for \code{\link{LzCOPpermsym}}. It seems to follow, by intuition, if therein, the copula on the left were to be replaced with the \emph{permutation symmetric} (\emph{exchangeable symmetric}) copula \code{\link{isCOP.permsym}}, such as \eqn{\mathbf{\Pi} = uv = vu}, then the copula on the right, now being, without loss of generality, \eqn{\mathbf{C}(u,v)} or \eqn{\mathbf{C}(v,u)}, then \eqn{\sigma_\mathbf{C}} must respond partly to dependence existing in the presence of \emph{permutation asymmetry}.
}
\usage{
wolfCOP(cop=NULL, para=NULL, as.sample=FALSE, brute=FALSE, delta=0.002,
        nlarge=500, usefastgrid=TRUE, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{as.sample}{A logical controlling whether an optional \R \code{data.frame} in \code{para} is used to compute the \eqn{\hat{\sigma}_\mathbf{C}} (see \bold{Details}). If set to \code{-1}, then the message concerning CPU effort will be suppressed. This feature of controlling the message is not applicable if the fast computation by \code{usefastgrid} is set and sample size is larger than \code{nlarge} as the double summation in the \enc{Pózos}{Poczos} \emph{et al.} (2015) is bypassed in favor of \code{\link{EMPIRgrid_fast}};}
  \item{brute}{Should brute force be used instead of two nested \code{integrate()} functions in \R to perform the double integration;}
  \item{delta}{The \eqn{\mathrm{d}u} and \eqn{\mathrm{d}v} for the brute force (\code{brute=TRUE}) integration;}
  \item{nlarge}{A large sample size for which to consider the fast-grid computation if \code{usefastgrid} is true;}
  \item{usefastgrid}{A logical controlling whether the fast-grid computation is acceptable to the user; and}
  \item{...}{Additional arguments to pass to the copula itself, to \code{\link{EMPIRgrid_fast}}, or the \code{integrate()} function.}
}
\value{
The value for \eqn{\sigma_\mathbf{C}} is returned, and the sample \eqn{\hat\sigma_\mathbf{C}} computation versions have \code{names()} attached to verify to the user which subsystem was used given the \code{nlarge} and \code{usefastgrid} arguments.
}
\details{
A natural estimator for \eqn{\sigma_\mathbf{C}} is through the \emph{empirical copula} (\enc{Póczos}{Poczos} \emph{et al.}, 2015) and can be computed as
\deqn{\hat{\sigma}_\mathbf{C} = \frac{12}{n^2 - 1}
        \sum_{i=1}^n\sum_{j=1}^n \bigg|\hat{\mathbf{C}}_n\biggl(\frac{i}{n}, \frac{j}{n}\biggr) -
                 \frac{i}{n}\times\frac{j}{n}\bigg|\mbox{,}}
where \eqn{\hat{\mathbf{C}}_n} is the simplest empirical copula of
\deqn{\hat{\mathbf{C}}_n\biggl(\frac{i}{n}, \frac{j}{n}\biggr) =
	\frac{1}{n}\{\# \mathrm{\ of\ } (U_k \le U_i, V_k \le V_j)\}}

The sample version of \eqn{\hat\sigma_\mathbf{C}} is triggered by \code{wolfCOP(para=D, as.sample=TRUE)} for the double summation if \code{usefastgrid} is false and or \code{usefastgrid} is true and the same size is smaller than \code{nlarge}. A Monte Carlo integration version potentially suitable to very large sample sizes or difficult to integrate copula families is located at \code{\link{wolfCOPsamc}}, but the function itself is likely of limited general use because of the implementation of \code{\link{EMPIRgrid_fast}} inside \code{wolfCOP}. It is upon the fast estimator listed above that the extensive simulations leading to \code{\link{wolfCOPtest}} are based. In general, there are several asymptotically equivalent definitions of the empirical copula and some of which are implemented in \code{\link{EMPIRcop}}.

Consideration of speed performance between the \code{as.sample=TRUE} for the sample estimator argued by \enc{Póczos}{Poczos} \emph{et al.}, 2015) (refer to section \bold{Note}), the \code{\link{EMPIRgrid_fast}} is measurably faster by about \eqn{n \ge 500}. The following code shows numerical equivalence between these estimators, but use of other \emph{empirical copula} will have divergence unless samples are large:
\preformatted{
  "MYwolfCOPgrid" <- function(uv, ctype="1/n", ...) {
    ef <- EMPIRgrid_fast(para=uv, gridonly=FALSE, ctype=ctype)
    pf <- ef$empcop; n <- nrow(uv)
    gu <- as.numeric( rownames(ef$empcop) ) # gv <- gu (can compute w/o gv)
    for(i in seq_len(nrow(pf))) pf[i,] <- gu * gu[i]
    return(12 * sum(abs(ef$empcop - pf)) / (n^2 - 1))
  }

  ns <- as.integer( 10^seq(1, 3.3, by=0.1) ) # some sample sizes
  WG <- WS <- vector(mode="numeric", length(ns))
  for(i in seq_len(length(ns))) {
    uv    <- simCOP(n=ns[i], cop=P, graphics=FALSE)
    WG[i] <- wolfCOP( para=uv, as.sample=TRUE, usefastgrid=FALSE)
    WS[i] <- MYwolfCOPgrid(uv)
  }

  opts <- par(xpd=NA, no.readonly=FALSE)
  df <- data.frame(n=ns, wolf_grid=WG, wolf_sample=WS)
  plot(df$n, df$wolf_sample, type="n", xlim=c(10, 10000), ylim=c(0, 0.60),
       xaxs="i", yaxs="i", log="x", las=1,
       xlab="Sample size of Independence copula",
       ylab="Schweizer-Wolff Sigma")
  points(df$n, df$wolf_sample, pch=21, bg="white")
  points(df$n, df$wolf_grid, pch=16, cex=0.6)
  par(opts) # }
}
\note{
An extended example is informative. First, declare a composite of two different Plackett copulas (\code{\link{PLcop}}) and simulate a few hundred values. The parameter settings are such that the data will attain a look of independence.:
\preformatted{
   para <- list(cop1=PLcop, para1=0.145, alpha=0.81,
                cop2=PLcop, para2=21.90,  beta=0.22)
   D <- simCOP(n=300, cop=composite2COP, para=para,
               cex=0.5, col=grey(0, 0.5), pch=16, seed=15)
}

Second, show that this copula is globally PQD (\code{\link{isCOP.PQD}}), but there is a substantial local NQD part of \eqn{\mathcal{I}^2} space that clearly is NQD.
\preformatted{
  cols <- c("red3", "green4")
  PQD <- isCOP.PQD(cop=composite2COP, para=para, uv=D)
  message(PQD$global.PQD) # TRUE
  points(D, col=cols[1+as.numeric(PQD$local.PQD)], lwd=2) # TRUE are green
}

This composited copula intersects, that is, passes through, the \code{\link{P}} copula. By the logic of Nelsen (2006), then the \eqn{\sigma_\mathbf{C}} should be larger than \eqn{\rho_\mathbf{C}} as shown below
\preformatted{
  wolfCOP(cop=composite2COP, para=para) # 0.08373378 (theoretical)
   rhoCOP(cop=composite2COP, para=para) # 0.02845131 (theoretical)
  hoefCOP(cop=composite2COP, para=para) # 0.08563823 (theoretical)
}

In fact, output above also shows Schweizer--Wolff Sigma to be larger than \emph{Blomqvist Beta} (\code{\link{blomCOP}}), \emph{Gini Gamma} (\code{\link{giniCOP}}), and \emph{Kendall Tau} (\code{\link{tauCOP}}). Schweizer--Wolff Sigma captures the fact that although the symbols seems to plot randomly on the figure, symbol coloring for PQD and NQD clearly shows local dependency differences.
}
\references{
Li, L. Liu, Y., and Lu, Z., 2019, A new dependence measure for importance analysis---Application to an environmental model: Applied Mathematical Modelling, v. 74, pp. 43--61, \doi{10.1016/j.apm.2019.04.046}.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

\enc{Póczos}{Poczos}, B., Krishner, S., \enc{Pál}{Pal}, D., \enc{Szepesvári}{Szepesvari}, C., and Schneider, J., 2015, Robust nonparametric copula based dependence estimators, accessed on August 11, 2015, at \url{https://www.cs.cmu.edu/~bapoczos/articles/poczos11nipscopula.pdf}.

Schweizer, B., and Wolff, E.F., 1981, On nonparametric measures of dependence for random variables: The Annuals of Statistics, v. 9, no. 4, pp. 879--855, \url{https://www.jstor.org/stable/2240856}.

\enc{Úbeda-Flores}{Ubeda-Flores}, M., 2022, A note on the maximum difference between Schweizer and Wolff's Sigma and the absolute value of Spearman's Rho: Transactions on Fuzzy Sets and Systems, v. 1, no. 2, pp. 1--5, \doi{10.30495/tfss.2022.1956899.1024}.
}
\author{W.H. Asquith}
\seealso{\code{\link{wolfCOPsamc}}, \code{\link{wolfCOPtest}}\cr
         \code{\link{blomCOP}}, \code{\link{footCOP}}, \code{\link{giniCOP}},
         \code{\link{hoefCOP}}, \code{\link{rhoCOP}}, \code{\link{tauCOP}},
         \code{\link{joeskewCOP}}, \code{\link{uvlmoms}}, \code{\link{ORDSUMcop}}
}
\examples{
wolfCOP(cop=PSP) # 0.4784176

\dontrun{
  # Maximum Difference between Sigma and Absolute Spearman Rho
  k <- 1/2 + sqrt(3)/6 # Ubeda-Flores (2022) or 1/2 - sqrt(3)/6
  para <- list(cop=c(W, W), para=NULL, part=c(0, k, 1))
  rho <- rhoCOP( ORDSUMcop, para=para)
  wol <- wolfCOP(ORDSUMcop, para=para)
  message("Wolf - |Rho| = ", wol - abs(rho))
  # 0.604967382.... that is approx. 0.60496 given by Ubeda-Flores (2022).

  ks <- seq(0.01, 0.99, by=0.002); rhos <- wols <- NULL
  for(k in ks) {
    message(k, "-", appendLF=FALSE)
    para <- list(cop=c(W, W), para=NULL, part=c(0, k, 1))
    rhos <- c(rhos, abs( rhoCOP( cop=ORDSUMcop, para=para, subdivisions=200) ))
    wols <- c(wols,      wolfCOP(cop=ORDSUMcop, para=para, subdivisions=200)  )
  }
  message("done")
  plot( ks, rhos, type="l", lty=2, main="Ubeda-Flores (2022) Example 2.3",
       xlab="Theta of W,W shuffle", ylab="Value")
  lines(ks, wols, lty=1); lines(ks, wols - rhos, lwd=3)
  legend(0.3, 0.9, c("|Spearman Rho|", "Schweizer and Wolff Sigma",
                     "Schweizer and Wolff Sigma minus |Spearman Rho|"),
         bty="n", lty=c(2,1,1), lwd=c(1,1,2), cex=0.8)
  abline(v=1/2 + c(-1, +1) * sqrt(3)/6, lty=4) # Ubeda-Flores (2022) }

\dontrun{
  n <- 1000; UV <- simCOP(n=n, cop=N4212cop, para=7.53, graphics=FALSE, seed=1)
  wolfCOP(cop=N4212cop, para=7.53)                    # 0.9884666 (theoretical)
  wolfCOP(para=UV, as.sample=TRUE, usefastgrid=TRUE ) # 0.9873274 (  sample   )
  wolfCOP(para=UV, as.sample=TRUE, usefastgrid=FALSE) # 0.9873274 (  sample   )
  wolfCOPsamc(para=UV)$estimates[2]       # 0.9861096 (Monte Carlo integration)
  # Much faster as the sample size gets large to use the usefastgrid as shown
  # in lieu of using the Monte Carlo. The wolfCOPsamc() was developed before
  # the algorithm used for EMPIRgrid_fast and used by wolfCOP() was found. }

\dontrun{
  # Redo D from Note section above
  para <- list(cop1 =PLcop,  cop2=PLcop,
               para1=0.145, para2=21.9,  alpha=0.81, beta=0.22)
  D <- simCOP(n=300, cop=composite2COP, para=para,
              cex=0.5, col=rgb(0, 0, 0, 0.2), pch=16)
  PQD <- isCOP.PQD(cop=composite2COP, para=para, uv=D)
  the.grid  <- EMPIRgrid(para=D)
  the.persp <- persp(the.grid$empcop, theta=-25, phi=20, shade=TRUE,
                     xlab="U VARIABLE", ylab="V VARIABLE", zlab="COPULA C(u,v)")
  empcop <- EMPIRcopdf(para=D) # data.frame of all points
  points(trans3d(empcop$u, empcop$v, empcop$empcop, the.persp),  cex=0.7,
         col=rgb(0, 1-sqrt(empcop$empcop), 1, sqrt(empcop$empcop)), pch=16)
  points(trans3d(empcop$u, empcop$v, empcop$empcop, the.persp),
         col=PQD$local.PQD+1, pch=1)

  layout(matrix(c(1,2,3,4), 2, 2, byrow = TRUE), respect = TRUE)
  PQD.NQD.cop <- gridCOP(cop=composite2COP, para=para)
  Pi <- gridCOP(cop=P)
  RHO <- PQD.NQD.cop - Pi; SIG <- abs(RHO)
  the.persp <- persp(PQD.NQD.cop, theta=-25, phi=20, shade=TRUE, cex=0.5,
                 xlab="U VARIABLE", ylab="V VARIABLE", zlab="COPULA C(u,v)")
  mtext("The Copula that has local PQD and NQD", cex=0.5)
  the.persp <- persp(Pi, theta=-25, phi=20, shade=TRUE, cex=0.5,
                 xlab="U VARIABLE", ylab="V VARIABLE", zlab="COPULA C(u,v)")
  mtext("Independence (Pi)", cex=0.5)
  the.persp <- persp(RHO, theta=-25, phi=20, shade=TRUE, cex=0.5,
                 xlab="U VARIABLE", ylab="V VARIABLE", zlab="COPULA C(u,v)")
  mtext("Copula delta: Integrand of Spearman Rho", cex=0.5)
  the.persp <- persp(SIG, theta=-25, phi=20, shade=TRUE, cex=0.5,
                 xlab="U VARIABLE", ylab="V VARIABLE", zlab="COPULA C(u,v)")
  mtext("abs(Copula delta): Integrand of Schweizer-Wolff Sigma", cex=0.5) #}
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{Schweizer and Wolff Sigma}
\keyword{extended delta index}

\encoding{utf8}
\name{prod2COP}
\alias{prod2COP}
\title{The Product of Two Copulas}
\description{
Perform copula multiplication (so-called \dQuote{\eqn{\ast}-product} or \emph{Markov Product}) (Darsow and others, 1992) is a continuous analog of matrix multiplication and yields another copula:
\deqn{\bigl(\mathbf{C}_1 \ast \mathbf{C}_2 \bigr)(u,v) = \mathbf{C}_3(u,v) = \int_\mathcal{I} \frac{\delta \mathbf{C}_1(u, t)}{\delta v} \frac{\delta \mathbf{C}_2(t, v)}{\delta u}\,\mathrm{d}t\mbox{,}}
for copulas \eqn{\mathbf{C}_1(u, v)} and \eqn{\mathbf{C}_2(u, v)} are copulas whose \eqn{\ast}-product yields copula \eqn{\mathbf{C}_3(u, v)} in terms of partial derivatives (\code{\link{derCOP}} and \code{\link{derCOP2}}) of the other two. Nelsen (2006, p. 245) lists several identities of the \eqn{\ast}-product involving the product (\eqn{\mathbf{\Pi}}; \code{\link{P}}), lower bound (\eqn{\mathbf{W}}; \code{\link{W}}), and upper bound (\eqn{\mathbf{M}}; \code{\link{M}}) copulas:
\deqn{\mathbf{\Pi} \ast \mathbf{C} = \mathbf{C} \ast \mathbf{\Pi} = \mathbf{\Pi}\mbox{,}}
\deqn{\mathbf{M} \ast \mathbf{C} = \mathbf{C} \ast \mathbf{M} = \mathbf{M}\mbox{,}}
\deqn{\bigl(\mathbf{W} \ast \mathbf{C}\bigr)(u,v) = v - \mathbf{C}(1-u, v)\mbox{\ and\ } \bigl(\mathbf{C} \ast \mathbf{W}\bigr)(u,v) = u - \mathbf{C}(u, 1-v)\mbox{, and}}
\deqn{\mathbf{W} \ast \mathbf{W} = \mathbf{M}\mbox{ and } \mathbf{W} \ast \mathbf{C} \ast \mathbf{W} = \hat{\mathbf{C}}\mbox{,}}
where \eqn{\hat{\mathbf{C}}} is the \emph{survival copula} (\code{\link{surCOP}}). The \eqn{\ast}-product is associative:
\deqn{\mathbf{A} \ast (\mathbf{B} \ast \mathbf{C}) = (\mathbf{A} \ast \mathbf{B}) \ast \mathbf{C}\mbox{,}}
but \eqn{\ast}-product is not commutative (order independent). Nelsen (2006, p. 245) reports that \dQuote{if we view \eqn{\ast} as a binary operation on the set of copulas, then \eqn{\mathbf{\Pi}} is the null element, and \eqn{\mathbf{M}} is the identity.} Copula multiplication is closely linked to \emph{Markov Processes} (Nelsen, 2006, pp. 244--248).

For other descriptions and computations of copula combination are possible using the \pkg{copBasic} package, see \code{\link{convexCOP}}, \code{\link{convex2COP}}, \code{\link{composite1COP}}, \code{\link{composite2COP}}, \code{\link{composite3COP}},\cr \code{\link{glueCOP}}, and \code{\link{convexCOP}}.
}
\usage{
prod2COP(u,v, cop1=NULL, para1=NULL, cop2=NULL, para2=NULL, para=NULL,
              pinterval=NULL, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{cop1}{The \eqn{\mathbf{C}_1(u,v; \Theta_1)} copula function with vectorization as in \code{asCOP};}
  \item{para1}{Vector of parameters or other data structures for \eqn{\Theta_1}, if needed, to pass to copula \eqn{\mathbf{C}_1(u,v; \Theta_1)};}
  \item{cop2}{The \eqn{\mathbf{C}_2(u,v; \Theta_2)} copula function with vectorization as in \code{asCOP};}
  \item{para2}{Vector of parameters or other data structures for \eqn{\Theta_2}, if needed, to pass to copula \eqn{\mathbf{C}_2(u,v; \Theta_2)};}
  \item{para}{An \R \code{list} that can take the place of the \code{cop1}, \code{para1}, \code{cop2}, and \code{para2} arguments. These four will be populated from same named elements of the \code{list}, and if the other four arguments were specified through the function interface, these are silently ignored;}
  \item{pinterval}{An optional interval for the above integral. The default is \eqn{\mathcal{I} = [0,1]} but the option of the user to replace exact end points with \dQuote{small} numbers is possible (\emph{e.g.} \code{interval=}\code{c(lo, 1-lo)} for say \code{lo=.Machine$double.eps}). This interval is uniquely picked up for the interval in the above definition of \code{prod2COP}. The \code{pinterval} can also be set within the \code{para} and the function will pick it up from there; and}
  \item{...}{Additional arguments to pass to the copulas.}
}
\value{
  Value(s) for the copula are returned.
}
\note{
The \emph{Farlie--Gumbel--Morgenstern copula} (\eqn{\mathbf{FGM}(u,v; \Theta)}; \code{\link{FGMcop}}) is
\deqn{\mathbf{FGM}(u,v; \Theta) = uv[1+\Theta(1-u)(1-v)]\mbox{,}}
where \eqn{-1 \le \Theta \le 1}. Nelsen (2006, exer. 6.12, p. 249) asserts that for \eqn{\mathbf{FGM}_{(\Theta = \alpha)}} and \eqn{\mathbf{FGM}_{(\Theta = \beta)}} with \eqn{\ast}-product as \eqn{\mathbf{FGM}_\alpha \ast \mathbf{FGM}_\beta} that a closed-form solution exists and is
\deqn{\mathbf{FGM}_\alpha \ast \mathbf{FGM}_\beta = \mathbf{FGM}_{(\alpha\beta) / 3}\mbox{.}}
This assertion is numerically true as readily verified using the \code{prod2COP} function:
\preformatted{
  u <- c(0.41, 0.87); v <- c(0.13,0.35); A <- -0.532; B <- 0.235
  FGMcop(  u,v, para= A*B / 3)
  # 0.0521598638574___   0.3034277347150___
  prod2COP(u,v, cop1=FGMcop, para1=A, cop2=FGMcop, para2=B)
  # 0.0521598638312605   0.3034277344807909
}
}
\references{
Darsow, W.F., Nguyen, B., and Olsen, E.T., 1992, Copulas and Markov processes: Illinois Journal of Mathematics, v. 26, pp. 600--624, \doi{10.1215/IJM/1255987328}.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{COP}}, \code{\link{composite1COP}}, \code{\link{composite2COP}}, \code{\link{composite3COP}},
\code{\link{convexCOP}}, \code{\link{convex2COP}}, \code{\link{glueCOP}}}
\examples{
\dontrun{
# Product P * N4212 ---> P (by identity)
u <- c(0.41, 0.87); v <- c(0.13, 0.35)
prod2COP(u,v, cop1=P, cop2=N4212cop, para1=NA, para2=2.12) # 0.0533 and 0.3045
COP(u,v, cop=P)                                            # 0.0533 and 0.3045}

\dontrun{
para <- list(cop1=PLcop, para1=0.19, cop2=PLcop, para2=34.5)
UV <- simCOP(n=1000, cop=prod2COP, para=para, resamv01=FALSE, showresamv01=FALSE)
# This is large simulation run (with a lot of numerical operations) is expected
# at least for the Placketts and chosen parameters to trigger one or more NAs
# from derCOPinv(). The simCOP() function simply continues on with ignoring the
# solution or lack thereof for certain combinations, and simCOP() will report how
# many of the simulated values for sample of size n were computed. For example,
# for one n=1000, some 965 simulated values were returned. The defaults require
# that NAs, empty simulations, remain intact. We can try resampling:
UV <- simCOP(n=1000, cop=prod2COP, para=para, resamv01=TRUE, showresamv01=TRUE)
rhoCOP(cop=prod2COP, para=para) # -0.4271195 (theoretical)
rhoCOP(para=UV, as.sample=TRUE) # -0.4274703 #}

\dontrun{
para <- list(cop1=PLcop, para1=0.19, cop2=PLcop, para2=34.5)
# The prod2COP() might be one of the more sensitive to NAs in simulation because
# of the two partial numerical derivatives involved.
para$pinterval <- c(0.4, 0.6) # totally inappropriate interval for the integral
# for the prod2COP() definition. Because the ... are used so extensively, we have
# the "pinterval" for this function so that interval itself can be passed also.
UV <- simCOP(n=1000, cop=prod2COP, para=para, resamv01=TRUE, showresamv01=TRUE,
                     pinterval=c(0,   1  ))
UV <- simCOP(n=1000, cop=prod2COP, para=para, resamv01=TRUE, showresamv01=TRUE,
                     pinterval=c(0.4, 0.6)) #}
}
\keyword{copula operator}
\concept{copula multiplication}
\concept{copula product}
\concept{Markov Product}
\concept{Markov product}
\concept{Markov Process}
\concept{Markov process}
\keyword{Nelsen (2006) Examples and Exercises}

\encoding{utf8}
\name{EMPIRgridderinv2}
\alias{EMPIRgridderinv2}
\title{Derivative Inverses of the Grid of the Bivariate Empirical Copula for U with respect to V}
\description{
Generate a gridded representation of the inverse of the derivatives of the \emph{bivariate empirical copula} of \eqn{U} with respect to \eqn{V}.  This function is the empirical analog to \code{\link{derCOPinv2}}.
}
\usage{
EMPIRgridderinv2(empgrid=NULL, kumaraswamy=FALSE, dergrid=NULL, ...)
}
\arguments{
  \item{empgrid}{The grid from \code{\link{EMPIRgrid}};}
  \item{kumaraswamy}{A logical to trigger Kumaraswamy smoothing of the conditional quantile function;}
  \item{dergrid}{The results of \code{\link{EMPIRgridder2}} and if left \code{NULL} then that function is called internally. There is some fragility at times in the quality of the numerical derivative and the author has provided this argument so that the derivative can be computed externally and then fed to this inversion function; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  The gridded values of the inverse of the derivative of \eqn{U} with respect to \eqn{V}.
}
\author{W.H. Asquith}
\seealso{\code{\link{EMPIRcop}}, \code{\link{EMPIRcopdf}}, \code{\link{EMPIRgrid}}, \code{\link{EMPIRgridder2}}, \code{\link{EMPIRgridderinv}}, \code{\link{EMPIRgridderinv2}}}
\examples{
# See examples under EMPIRgridderinv
}
\keyword{empirical copula}
\keyword{empirical copula (derivative inverses)}
\keyword{derivative}
\keyword{copulatic surface}
\keyword{gridded empirical copula}

\encoding{utf8}
\name{kfuncCOP_Pd}
\alias{kfuncCOP_Pd}
\title{The Kendall (Distribution) Function of d-Dimensional Independence Copula}
\description{
Compute the \emph{Kendall Distribution Function} of a \eqn{d}-Dimensional \emph{independence copula} (\code{\link{P}}) (Joe, 2014, p. 420):

\deqn{F_K(z; \mathbf{P}_d) = z + z\sum_k^{d-1} \bigr(-\mathrm{log}(k)\bigr)^k/k!\mbox{,}}

where \eqn{F_K} is the nonexceedance probability of joint probability \eqn{z} stemming from \eqn{\mathbf{P}_d(u_{1, 2, \cdots d})}.
}
\usage{
kfuncCOP_Pd(z, d=2)
}
\arguments{
  \item{z}{The values for \eqn{z}; and}
  \item{d}{Number of \eqn{d} dimensions.}
}
\value{
  The value(s) for \eqn{F_K(z; \mathbf{P}_d)} is returned.
}
\note{
Though the \pkg{copBasic} is designed for bivariate relations, \emph{vining} workflows could be used to produce \eqn{d}-dimensional relations from which Kendall functions could be empirically computed from \eqn{d}-variate simulations and a user might want an easy mechanism to look at \emph{independence} on Kendall distribution function plots.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{kfuncCOP}}
}
\examples{
# The 0.50 joint probability has a 0.8465736 nonexceedance probability.
kfuncCOP_Pd(0.50, d=3) # 0.8465736

\dontrun{
  f <- seq(0,1, by=0.01)
  plot(f, kfuncCOP(f, cop=P), type="l", lwd=7, col=grey(0.8),
          xlab="Kendall distribution function (joint probability)",
          ylab="Nonexceedance probability")
  lines(f, kfuncCOP_Pd(f, d=2), lwd=1.5, col="black")
  lines(f, kfuncCOP_Pd(f, d=3), col="seagreen3")
  lines(f, kfuncCOP_Pd(f, d=4), col="seagreen4") # }
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{Kendall Function}

\encoding{utf8}
\name{derCOP}
\alias{derCOP}
\title{Numerical Derivative of a Copula for V with respect to U}
\description{
Compute the numerical partial derivative of a copula, which is a \emph{conditional distribution function}, according to Nelsen (2006, pp. 13, 40--41) with respect to \eqn{u}:

\deqn{0 \le \frac{\delta}{\delta u} \mathbf{C}(u,v) \le 1\mbox{,}}

or

\deqn{\mathrm{Pr}[V \le v\mid U=u] = \mathbf{C}_{2 \mid 1}(v \mid u) = \lim_{\Delta u \rightarrow 0}\frac{\mathbf{C}(u+\Delta u, v) - \mathbf{C}(u,v)}{\Delta u}\mbox{,}}

which is to read as the probability that \eqn{V \le v} given that \eqn{U = u} and corresponds to the \cr \code{derdir="left"} mode of the function. For \code{derdir="right"}, we have
\deqn{\mathrm{Pr}[V \le v\mid U=u] = \lim_{\Delta u \rightarrow 0}\frac{\mathbf{C}(u,v) - \mathbf{C}(u-\Delta u, v)}{\Delta u}\mbox{,}}
and for \code{derdir="center"} (the usual method of computing a derivative), the following results
\deqn{\mathrm{Pr}[V \le v\mid U=u] = \lim_{\Delta u \rightarrow 0}\frac{\mathbf{C}(u+\Delta u,v) - \mathbf{C}(u-\Delta u, v)}{2 \Delta u}\mbox{.}}
The \dQuote{\emph{with respect to \eqn{V}}} versions are available under \code{\link{derCOP2}}.

Copula derivatives (\eqn{\delta \mathbf{C}/\delta u} or say  \eqn{\delta \mathbf{C}/\delta v} \code{\link{derCOP2}}) are non-decreasing functions meaning that if \eqn{v_1 \le v_2}, then \eqn{\mathbf{C}(u, v_2) - \mathbf{C}(u,v_1)} is a non-decreasing function in \eqn{u}, thus
\deqn{\frac{\delta\bigl(\mathbf{C}(u, v_2) - \mathbf{C}(u,v_1)\bigr)}{\delta u}}
is non-negative, which means
\deqn{\frac{\delta\mathbf{C}(u, v_2)}{\delta u} \ge \frac{\delta\mathbf{C}(u, v_1)}{\delta u}\mbox{\ for\ } v_2 \ge v_1\mbox{.}}
}
\usage{
derCOP(cop=NULL, u, v, delu=.Machine$double.eps^0.50,
       derdir=c("left", "right", "center"), ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction. If the length of \code{u} is unity, then the length of \code{v} can be arbitrarily long. If the length of \code{u} is not unity, then the length of \code{v} should be the same, and if not, then  only the first value in \code{v} is silently used;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction (see previous comment on \code{u});}
  \item{delu}{The \eqn{\Delta u} interval for the derivative;}
  \item{derdir}{The direction of the derivative as described above. Default is \code{left} but internally any setting can be temporarily suspended to avoid improper computations (see source code); and}
  \item{...}{Additional arguments to pass such as the parameters often described in \code{para} arguments of other copula functions. (The lack of \code{para=NULL} for \code{derCOP} and \code{\link{derCOP2}} was either design oversight or design foresight but regardless it is too late to enforce package consistency in this matter.)}
}
\value{
  Value(s) for the partial derivative are returned.
}
\note{
A known caveat of the current implementation of the copula derivative is that there is a chance that the \eqn{\Delta u} will span a singularity or discontinuous (or nearly so) portion of a copula should it have a property of singularity (or nearly so). The \code{delu} is chosen small so the chance is mitigated to be a small change and certainly appear to work throughout the examples herein. It is not decided whether a derivative from the positive side (\code{dir="left"}), when failing should switch over to a computation from the negative side (\code{dir="right"}). The distinction is important for the computation of the inverse of the derivative \code{\link{derCOPinv}} because the solution finder needs a sign reversal to work.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{derCOPinv}}, \code{\link{derCOP2}}}
\examples{
derCOP(cop=W, 0.4, 0.6); derCOP(cop=P, 0.4, 0.6); derCOP(cop=M, 0.4, 0.6)

lft <- derCOP(cop=PSP,   0.4, 0.6, derdir="left"  )
rgt <- derCOP(cop=PSP,   0.4, 0.6, derdir="right" )
cnt <- derCOP(cop=PSP,   0.4, 0.6, derdir="center")
cat(c(lft,rgt,cnt,"\n"))
#stopifnot(all.equal(lft,rgt), all.equal(lft,cnt))

# Let us contrive a singularity through this NOT A COPULA in the function "afunc".
"afunc" <- function(u,v, ...) return(ifelse(u <= 0.5, sqrt(u^2+v^2), P(u,v,...)))
lft <- derCOP(cop=afunc, 0.5, 0.67, derdir="left"  )
rgt <- derCOP(cop=afunc, 0.5, 0.67, derdir="right" )
cnt <- derCOP(cop=afunc, 0.5, 0.67, derdir="center")
cat(c(lft,rgt,cnt,"\n")) # The "right" version is correct.
}
\keyword{copula (utility)}
\keyword{copula (derivative)}
\keyword{conditional cumulative distribution function}
\keyword{copula operator}
\keyword{derivative}
\keyword{copula (conditional distribution)}

\encoding{utf8}
\name{FRECHETcop}
\alias{FRECHETcop}
\title{The \enc{Fréchet}{Frechet} Family Copula}
\description{
The \emph{\enc{Fréchet}{Frechet} Family copula} (Durante, 2007, pp. 256--259) is
\deqn{\mathbf{C}_{\alpha, \beta}(u,v) = \mathbf{FF}(u,v) = \alpha\mathbf{M}(u,v) + (1-\alpha-\beta)\mathbf{\Pi}(u,v)+\beta\mathbf{W}(u,v)\mbox{,}}
where \eqn{\alpha, \beta \ge 0} and \eqn{\alpha + \beta \le 1}. The \enc{Fréchet}{Frechet} Family copulas are \emph{convex combinations} of the fundamental copulas \eqn{\mathbf{W}} (\emph{\enc{Fréchet}{Frechet}--Hoeffding lower-bound copula}; \code{\link{W}}), \eqn{\mathbf{\Pi}} (independence; \code{\link{P}}), and \eqn{\mathbf{M}} (\emph{\enc{Fréchet}{Frechet}--Hoeffding upper-bound copula}; \code{\link{M}}). The copula is \emph{comprehensive} because both \eqn{\mathbf{W}} and \eqn{\mathbf{M}} can be obtained. The parameters are readily estimated using \emph{Spearman Rho} (\eqn{\rho_\mathbf{C}}; \code{\link{rhoCOP}}) and \emph{Kendall Tau} (\eqn{\tau_\mathbf{C}}; \code{\link{tauCOP}}) by

\deqn{\tau_\mathbf{C} = \frac{(\alpha - \beta)(\alpha + \beta + 2)}{3}\mbox{\ and\ } \rho_\mathbf{C} = \alpha - \beta\mbox{.}}

The \enc{Fréchet}{Frechet} Family copula virtually always has a visible \emph{singular component} unless \eqn{\alpha, \beta = 0}. The copula has respective \emph{lower-} and \emph{upper-tail dependency parameters} of \eqn{\lambda^L = \alpha} and \eqn{\lambda^U = \alpha} (\code{\link{taildepCOP}}). Durante (2007, p. 257) reports that the \enc{Fréchet}{Frechet} Family copula can approximate any bivariate copula in a \dQuote{unique way} and the error bound can be estimated.
}
\usage{
FRECHETcop(u,v, para=NULL, rho=NULL, tau=NULL, par2rhotau=FALSE, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (two element) of parameters \eqn{\alpha} and \eqn{\beta};}
  \item{rho}{Spearman Rho from which to estimate the parameters;}
  \item{tau}{Kendall Tau from which to estimate the parameters;}
  \item{par2rhotau}{A logical that if \code{TRUE} will return an \R \code{list} of the \eqn{\rho_\mathbf{C}} and \eqn{\tau_\mathbf{C}} for the parameters; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned using the \eqn{\alpha} and \eqn{\beta} as set by argument \code{para}; however, if \code{para=NULL} and \code{rho} and \code{tau} are set and compatible with the copula, then \eqn{\{\rho_\mathbf{C}, \tau_\mathbf{C}\} \rightarrow \{\alpha, \beta\}}, parameter estimation made, and an \R \code{list} is returned.
}
\details{
The function will check the consistency of the parameters whether given by argument or computed from \eqn{\rho_\mathbf{C}} and \eqn{\tau_\mathbf{C}}. The term \dQuote{Family} is used with this particular copula in \pkg{copBasic} so as to draw distinction to the \enc{Fréchet}{Frechet} lower- and upper-bound copulas as the two limiting copulas are called.

For no other reason than that it can be easily done and makes a nice picture, loop through a nest of \eqn{\rho} and \eqn{\tau} for the \enc{Fréchet}{Frechet} Family copula and plot the domain of the resulting parameters:
\preformatted{
  ops <- options(warn=-1) # warning supression because "loops" are dumb
  taus <- rhos <- seq(-1,1, by=0.01)
  plot(NA, NA, type="n", xlim=c(0,1), ylim=c(0,1),
       xlab="Frechet Copula Parameter Alpha",
       ylab="Frechet Copula Parameter Beta")
  for(tau in taus) {
    for(rho in rhos) {
      fcop <- FRECHETcop(rho=rho, tau=tau)
      if(! is.na(fcop$para[1])) points(fcop$para[1], fcop$para[2])
    }
  }
  options(ops)
}
}
\note{
A convex combination (\code{\link{convex2COP}}) of \eqn{\mathbf{\Pi}} and \eqn{\mathbf{M}}, which is a modification of the \enc{Fréchet}{Frechet} Family, is the \emph{Linear Spearman} copula:
\deqn{\mathbf{C}_\alpha(u,v) = (1-\alpha)\mathbf{\Pi}(u,v) + \alpha\mathbf{M}(u,v)\mbox{,}}
for \eqn{0 \le \alpha \le 1}, and the parameter is equal to \eqn{\rho_\mathbf{C}}. When the convex combination is used for construction, the complement of the parameter is equal to \eqn{\rho_\mathbf{C}} (\emph{e.g.} \eqn{1-\alpha = \rho_\mathbf{C}}; \code{\link{rhoCOP}}), which can be validated by
\preformatted{
  rhoCOP(cop=convex2COP, para=list(alpha=1-0.48, cop1=P, cop2=M)) # 0.4799948
}
}
\references{
Durante, F., 2007, Families of copulas, Appendix C, \emph{in} Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in Nature---An approach using copulas: Springer, 289 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{M}}, \code{\link{P}}, \code{\link{W}}
}
\examples{
\dontrun{
ppara <- c(0.25, 0.50)
fcop <- FRECHETcop(para=ppara, par2rhotau=TRUE)
RHO <- fcop$rho; TAU <- fcop$tau

level.curvesCOP(cop=FRECHETcop, para=ppara) # Durante (2007, Fig. C.27(b))
mtext("Frechet Family copula")
 UV <- simCOP(n=50, cop=FRECHETcop, para=ppara, ploton=FALSE, points=FALSE)
tau <- cor(UV$U, UV$V, method="kendall" ) # sample Kendall Tau
rho <- cor(UV$U, UV$V, method="spearman") # sample Spearman Rho
spara <- FRECHETcop(rho=rho, tau=tau) # a fitted Frechet Family copula
spara <- spara$para
if(is.na(spara[1])) { # now a fittable combination is not guaranteed
   warning("sample rho and tau do not provide valid parameters, ",
           "try another simulation")
} else { # now if fit, draw some red-colored level curves for comparison
   level.curvesCOP(cop=FRECHETcop, para=spara, ploton=FALSE, col=2)
} #}
}
\keyword{Frechet Family copula}
\keyword{copula (formulas)}
\keyword{copula}
\keyword{copula (comprehensive)}
\concept{comprehensive copula}
\concept{Linear Spearman copula}
\concept{Linear Spearman}

\encoding{utf8}
\name{JOMAcop}
\alias{JOMAcop}
\title{The Joe--Ma Copula}
\description{
The \emph{Joe--Ma copula} (Joe, 2014, p. 177), crediting Joe and Ma (2000), is

\deqn{\mathbf{C}_{\Theta}(u,v) = \mathbf{JOMA}(u,v) =
1 - F_{\Gamma}\biggl(\biggl\{
\bigl[F_\Gamma^{(-1)}(1-u; \Theta)\bigr]^\Theta +
\bigl[F_\Gamma^{(-1)}(1-v; \Theta)\bigr]^\Theta
              \biggr\}^{(1/\Theta)}; \Theta\biggr)\mbox{,}
}

where \eqn{F_\Gamma(x;\Theta)} is the \emph{cumulative distribution function} of the \emph{Gamma} distribution, \eqn{F_\Gamma^{(-1)}(f;\Theta)} is the \emph{quantile function} of the Gamma distribution, and \eqn{\Theta \in (-1, +1)}, which is a different parameter range than shown in Joe (2014, p. 177) because of numerical difficulties with the Gamma distribution and functional performance in context of \pkg{copBasic} design. The copula limits, as \eqn{\Theta \rightarrow -1}, to the \emph{countermonotonicity coupla} (\eqn{\mathbf{W}(u,v)}; \code{\link{W}}), as \eqn{\Theta \rightarrow 1}, to the \emph{comonotonicity copula} (\eqn{\mathbf{M}(u,v)}; \code{\link{M}}), and as \eqn{\Theta \rightarrow 0^{\pm}}, to the \emph{independence copula} (\eqn{\mathbf{\Pi}(u,v)}; \code{\link{P}}), and as \eqn{\Theta \rightarrow +\infty}. The parameter \eqn{\Theta} is readily computed from \emph{Kendall Tau} (\code{\link{tauCOP}}) or \emph{Spearman Rho} (\code{\link{rhoCOP}}) by root-solving methods. Because the formulation here uses a differing parameter range than shown in Joe (2014), integral formulations for Kendall Tau are not shown in this documentation.
}
\usage{
JOMAcop(u, v, para=NULL, rhotau=NULL,
              cortype=c("kendall", "spearman", "tau", "rho"), ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (single element) of parameters---the \eqn{\Theta} parameter of the copula;}
  \item{rhotau}{Optional Kendall Tau or Spearman Rho and parameter \code{para} is returned depending on the setting of \code{cortype}. The \code{u} and \code{v} can be used for estimation of the parameter as computed through the setting of \code{cortype};}
  \item{cortype}{A character string controlling, if the parameter is not given, to use a Kendall Tau or Spearman Rho for estimation of the parameter. The name of this argument is reflective of an internal call to \code{stats::cor()} to the correlation (association) setting for Kendall Tau or Spearman Rho; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned. Otherwise if \code{tau} is given, then the \eqn{\Theta} is computed and a \code{list} having
  \item{para}{The parameter \eqn{\Theta}, and}
  \item{tau}{Kendall Tau.}
and if \code{para=NULL} and \code{tau=NULL}, then the values within \code{u} and \code{v} are used to compute Kendall Tau and then compute the parameter, and these are returned in the aforementioned list. Or if \code{rho} is given, then the \eqn{\Theta} is computed and a similar \code{list} is returned having similar structure but with Spearman Rho instead.
}
\note{
The implementation of the copula here makes use of the negative association part of Joe's parameterization and rotates the copula to positive association. Experiments show Gamma distribution limitations that are not symmetrical to the problem at hand as the shape parameter in the distribution gets large. The negative association part of the copula makes use of conversion to Joe (2014, p. 179) formula for the \emph{lower bounds} of the copula. Internally, the parameter range is converted to \eqn{\Theta \in (-1, +1)} and therefore \eqn{\Theta = 0} is in the middle and independence. There are two thresholds for absolute value of the transformed \eqn{\Theta} in the source code and these stem from estimation of where the Gamma distribution simply breaks down. The benefit of the parameterization for this copula is we get near complete comprehensiveness that is symmetrical in Kendall Tau or Spearman Rho, which does seem likely in the \R source code that Joe (2014) has accompanying that book.

\bold{Remark}, because reflection of \eqn{1-v} is used for the construction of the positive association, for \eqn{0 < \Theta \le +1}, the copula is the copula of \eqn{(u, 1-v)}; care will be needed in some situations to work with this fact. Also, though the formula above suggests \emph{permutation symmetry} between \eqn{(u,v)} and \eqn{(v,u)}, simulations shown some tendency for skewing off the diagonal of the association. This is currently thought to be tied to the numerical derivative direction used in \pkg{copBasic} simulation; counter point is the following recipe:

\preformatted{
  UV <- simCOP(2000, cop=JOMAcop, para=-0.9, seed=1) # appears to
  # "cry" !!!! towards the upper-right corner and this is the native
  # formulation because the Theta is less than 0. Let us then have a
  # check of permutation symmetry through all four rotations.
  isCOP.permsym(cop=COP, para=list(cop=JOMAcop, para=-0.9, reflect=1))
  # [1] TRUE
  isCOP.permsym(cop=COP, para=list(cop=JOMAcop, para=-0.9, reflect=2))
  # [1] TRUE
  isCOP.permsym(cop=COP, para=list(cop=JOMAcop, para=-0.9, reflect=3))
  # [1] FALSE
  isCOP.permsym(cop=COP, para=list(cop=JOMAcop, para=-0.9, reflect=4))
  # [1] FALSE
  isCOP.permsym(cop=COP, para=list(cop=JOMAcop, para=+0.9, reflect=1))
  # [1] FALSE
  isCOP.permsym(cop=COP, para=list(cop=JOMAcop, para=+0.9, reflect=2))
  # [1] FALSE
  isCOP.permsym(cop=COP, para=list(cop=JOMAcop, para=+0.9, reflect=3))
  # [1] TRUE
  isCOP.permsym(cop=COP, para=list(cop=JOMAcop, para=+0.9, reflect=4))
  # [1] TRUE
}

All this tells us is that when the principle direction of the points are along the primary diagonal from lower left to upper right that variables are not exchangeable and hence there appears some directionality at the Gamma distribution itself within the what seems to be a symmetrical formula undoubtedly for the native form. Is this caused by the skewness itself of the Gamma? Will this be a source of L-coskewness of this copula implementation relative to \emph{Gaussian} (\code{\link{NORMcop}}) or \emph{Frank} (\code{\link{FRcop}}) copulas? Lastly, consult the following
\preformatted{
  lcomCOP(cop=JOMAcop, para=-0.6)
  lcomCOP(cop=FRcop, para=FRcop(rhotau=tauCOP(cop=JOMAcop, para=-0.6))$para)
}
that shows us that the two copula for the same Kendall Tau have differing L-comoments of course but that the Frank as L-coskews of zero and the Joe--Ma does not. One more check is now made to provide more informative about the diagonal and symmetry or not:
\preformatted{
  d <- -0.9; t <- seq(0.001, 0.999, by=0.001)
  UV <- simCOP(1000, cop=JOMAcop, para=d) # "crying" towards the upper right
  abline(1, -1, col="red") # corner, so not symmetrical about the -1 diagonal.
  u <- 0.5; der <- derCOP(cop=JOMAcop, u, t, para=-0.9, derdir="left")
  plot(qnorm(t), qnorm(der), type="l", )
}
that tells us that a vertical second line at \eqn{u = c} has a longer tail above the median and hence confirming that the Gamma distribution involvement must be producing the asymmetry.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

Joe, H., and Ma, C., 2000, Multivariate survival functions with a min-stable property: Journal of Multivariate Analyses, v. 75, no. 1, pp. 13--35, \doi{https://doi.org/10.1006/jmva.1999.1891}.
}
\author{W.H. Asquith}
\seealso{\code{\link{M}}, \code{\link{P}}, \code{\link{W}}
}
\examples{
JOMAcop(rhotau=0.7, method="kendall")$para # 0.8645555

\dontrun{
  n <- 100 # Demonstrate "comprehensiveness" for the copula
  d <- sort( sample(c(-1,1), n, replace=TRUE) * 10^runif(n, min=-0.5, max=0) )
  for(i in 1:n) {
    UV <- simCOP(100, cop=JOMAcop, para=d[i]); mtext(d[i])
  } # }

\dontrun{
  # Close to maximum L-comoments and sample counterparts for general check
  # against the postive and negative aspects and signs. It is interesting
  # that there are some larger than anticipated differences with some of
  # of the sample to the theory. Perhaps numerical improvements are needed?
  UVp <- simCOP(100000, cop=JOMAcop, para=+0.6, graphics=FALSE, seed=1)
  UVn <- simCOP(100000, cop=JOMAcop, para=-0.6, graphics=FALSE, seed=1)
  #                                Lcomom:T3[12,21] Lcomom:T4[12,21]
  lcomCOP(cop=JOMAcop, para=+0.6)  #    0.014081858      0.014592670
  #                                #   -0.014081858      0.014592670
  lcomCOP(cop=JOMAcop, para=-0.6)  #    0.014081858     -0.014592670
  #                                #    0.014081858     -0.014592670
  lmomco::lcomoms2(UVp, nmom=4) #$T3    0.0155001646     0.0161519394
  #                                #   -0.0145945494     0.0148050507
  lmomco::lcomoms2(UVn, nmom=4) #$T4    0.010777365      0.0143115158
  #                                #   -0.0168424348    -0.0131075545 # }

\dontrun{
  # A quick look at L-comoments of the copula with some remarks about limits.
  # The numerical intergrations for the L-comoments "diverage" for
  #   T3.12 NAs begin at -0.988, 0.987
  #   T3.21 NAs begin at -0.988, -----
  #   T4.12 NAs begin at -0.988, -----
  #   T4.12 NAs begin at -0.988, -----
  d <- seq(-1, 1, by=0.001)
  t3.12 <- t4.12 <- rep(NA, length(d))
  t3.21 <- t4.21 <- rep(NA, length(d))
  for(i in seq_len(length(d))) {
    lcm <- lcomCOP(cop=JOMAcop, para=d[i])
    t3.12[i] <- lcm$lcomUV[3]; t4.12[i] <- lcm$lcomUV[4]
    t3.21[i] <- lcm$lcomVU[3]; t4.21[i] <- lcm$lcomVU[4]
  }
  plot(d,  t3.12, col="skyblue4", type="l", lwd=3, ylim=c(-0.02, +0.02),
       xlab="JOMAP parameter", ylab="L-coskew and L-kurtosis")
  lines(d, t4.12, col="skyblue4", lty=2,    lwd=3)
  lines(d, t3.21, col="red")
  lines(d, t4.21, col="red",      lty=2) # }
}
\keyword{Joe-Ma copula}
\keyword{copula (formulas)}
\keyword{copula}
\keyword{copula (comprehensive)}
\concept{comprehensive copula}

\encoding{utf8}
\name{vuongCOP}
\alias{vuongCOP}
\title{The Vuong Procedure for Parametric Copula Comparison}
\description{
Perform the \emph{Vuong Procedure} following Joe (2014, pp. 257--258). Consider two copula densities \eqn{f_1 = c_1(u,v; \Theta_1)} and \eqn{f_2 = c_2(u,v; \Theta_2)} for two different bivariate copulas \eqn{\mathbf{C}_1(\Theta_1)} and \eqn{\mathbf{C}_2(\Theta_2)} having respective parameters \eqn{\Theta_1} and \eqn{\Theta_2} that provide the \dQuote{closest} \emph{Kullback--Leibler Divergence} from the true copula density \eqn{g(u,v)}.

The difference of the Kullback--Leibler Divergence (\code{\link{kullCOP}}) of the two copulas from the true copula density can be measured for a sample of size \eqn{n} and bivariate sample realizations \eqn{\{u_i, v_i\}} by
\deqn{\hat{D}_{12} = n^{-1}\sum_{i=1}^n D_i\mbox{,}}
where \eqn{\hat{D}_{12}} is referred to in the \pkg{copBasic} package as the \dQuote{Vuong \eqn{D}} and \eqn{D_i} is defined as
\deqn{D_i = \log\biggl[\frac{f_1(u_i, v_i; \Theta_2)}{f_2(u_i, v_i; \Theta_1)}\biggr]\mbox{.}}
The variance of \eqn{\hat{D}_{12}} can be estimated by
\deqn{\hat\sigma^2_{12} = (n-1)^{-1}\sum_{i=1}^n (D_i - \hat{D}_{12})^2\mbox{.}}
The sample estimate and variance are readily turned into the \eqn{100{{\times}}(1 - \alpha)} confidence interval by
\deqn{\hat{D}^{(\mathrm{lo})}_{12} < \hat{D}_{12} < \hat{D}^{(\mathrm{hi})}_{12}\mbox{,}}
where, using the quantile (inverse) function of the t-distribution \eqn{\sim} \eqn{\mathcal{T}^{(-1)}(F; \mathrm{df}{=}(n-2))} for nonexceedance probability \eqn{F} and \eqn{n-2} degrees of freedom for \eqn{n} being the sample size, the confidence interval is
\deqn{\hat{D}_{12}-\mathcal{T}^{(-1)}(1-\alpha/2){\times}\hat\sigma_{12}/\sqrt{n} < \hat{D}_{12} < \hat{D}_{12}+\mathcal{T}^{(-1)}(1-\alpha/2){\times}\hat\sigma_{12}/\sqrt{n}\mbox{.}}
Joe (2014, p. 258) reports other interval forms based (1) on the Akaike (AIC) correction and (2) on the Schwarz (BIC) correction, which are defined for AIC as
\deqn{\mathrm{AIC} = \hat{D}_{12} - (2n)^{-1}\log(n)\biggl[\mathrm{dim}(\Theta_2) - \mathrm{dim}(\Theta_1)\biggr]\pm \mathcal{T}^{(-1)}(1-\alpha/2){\times}\hat\sigma_{12}/\sqrt{n}\mbox{,}}
and for BIC as
\deqn{\mathrm{BIC} = \hat{D}_{12} - (2n)^{-1}\log(n)\biggl[\mathrm{dim}(\Theta_2) - \mathrm{dim}(\Theta_1)\biggr]\pm \mathcal{T}^{(-1)}(1-\alpha/2){\times}\hat\sigma_{12}/\sqrt{n}\mbox{.}}
The AIC and BIC corrections incorporate the number of parameters in the copula and for all else being equal the copula with the fewer parameters is preferable. If the two copulas being compared have equal number of parameters than the AIC and BIC equate to \eqn{\hat{D}_{12}} and the same confidence interval because the difference \eqn{[\mathrm{dim}(\Theta_2) - \mathrm{dim}(\Theta_1)]} is zero.

Joe (2014, p. 258) reports that these three intervals can be used for \emph{diagnostic inference} as follows. If an interval contains 0 (zero), then copulas \eqn{\mathbf{C}_1(\Theta_1)} and \eqn{\mathbf{C}_2(\Theta_2)} are not considered significantly different. If the interval does not contain 0, then copula \eqn{\mathbf{C}_1(\Theta_1)} or \eqn{\mathbf{C}_2(\Theta_2)} is the better fit depending on whether the interval is completely below 0 (thus \eqn{\mathbf{C}_1(\Theta_1)} better fit) or above 0 (thus \eqn{\mathbf{C}_2(\Theta_2)} better fit), respectively. Joe (2014) goes on the emphasize that \dQuote{the procedure compares different [copulas] and assesses whether they provide similar fits to the data. [The procedure] does not assess whether [either copula] is a good enough fit.}
}
\usage{
vuongCOP(u, v=NULL, cop1=NULL, cop2=NULL, para1=NULL, para2=NULL,
                    alpha=0.05, method=c("D12", "AIC", "BIC"),
                    the.zero=.Machine$double.eps^0.25, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction and if \code{NULL} then \code{u} is treated as a two column \R \code{data.frame};}
  \item{cop1}{A copula function corresponding to copula \eqn{f_1} in the Vuong Procedure;}
  \item{para1}{Vector of parameters or other data structure, if needed, to pass to the copula \eqn{f_1};}
  \item{cop2}{A copula function corresponding to copula \eqn{f_2} in the the Vuong Procedure;}
  \item{para2}{Vector of parameters or other data structure, if needed, to pass to the copula \eqn{f_2};}
  \item{alpha}{The \eqn{\alpha} in the Vuong Procedure, which results in the \eqn{100{\times}(1 - \alpha)} confidence interval (two sided);}
  \item{method}{The interval to evaluate as to position of the respective statistic form the comparison of the two copulas;}
  \item{the.zero}{The value for \dQuote{the zero} of the copula density function. This argument is the argument of the same name for \code{\link{densityCOP}}. The default here is intended to suggest that a tiny nonzero value for density will trap the numerical zero densities; and}
  \item{...}{Additional arguments to pass to the \code{\link{densityCOP}} function.}
}
\value{
  An \R \code{list} is returned having the following components:
  \item{title}{A descriptive title of the procedure;}
  \item{method}{A textual description of the \code{method} setting;}
  \item{result.text}{A textual description of the result of the Vuong Procedure;}
  \item{result}{A value 1 if \eqn{\mathbf{C}_1(\Theta_1)} is better fit, 2 if copula \eqn{\mathbf{C}_2(\Theta_2)} is better fit, and \code{0} if neither is better (\eqn{\hat{D}_{12} = 0}), and \code{NA} including the likely(?) erroneous situation of \eqn{\mathbf{C}_1(\Theta_1) \equiv \mathbf{C}_2(\Theta_2)};}
  \item{p.value}{The two-sided p-values of the Vuong Procedure inclusive of \eqn{\mathrm{AIC}} and \eqn{\mathrm{BIC}};}
  \item{D12}{A named vector of the lower and upper bounds of Vuong \eqn{D} at the respective confidence interval percentage along with \eqn{\hat{D}_{12}} and \eqn{\sigma^2_{12}};}
  \item{AIC}{A named vector of the lower and upper bounds of Vuong \eqn{\mathrm{AIC}} at the respective confidence interval percentage;}
  \item{BIC}{A named vector of the lower and upper bounds of Vuong \eqn{\mathrm{BIC}} at the respective confidence interval percentage; and}
  \item{parameters}{A named vector of the alpha, sample size, value for the t-distribution quantile \code{qt(1-alpha/2, df=n)}, and \eqn{\hat\sigma_{12}}.}
}
\note{
The \code{vuongCOP} function along with \code{\link{kullCOP}} and features of function \code{\link{densityCOPplot}} represent collective milestones towards \emph{copula inference} and diagnostics post fitting of copulas to the usual measures of association such as the \emph{Kendall Tau} (\eqn{\tau_K}) and \emph{Spearman Rho} (\eqn{\rho_S}) and their copula counterparts \eqn{\tau_\mathbf{C}} (\code{\link{tauCOP}}) and \eqn{\rho_\mathbf{C}} (\code{\link{rhoCOP}}).

For an example application, imagine a problem of say low springflow risk at \dQuote{nearby springs} that jointly should converge in the lower tail because drought usually has a strong regional impact. First, it is necessary to form a reflection of the \emph{Gumbel--Hougaard copula} (\eqn{\mathbf{GH}(u,v; \Theta_{\mathbf{GH}})}; \code{\link{GHcop}}) but parameter estimation using \eqn{\tau_\mathbf{C}} is the same because sample \eqn{\hat\tau_K} is invariant to reflection.
\preformatted{
  "rGHcop" <- function(u,v, ...) { u + v - 1 + GHcop(1-u, 1-v, ...) }
  set.seed(385) # setting so that reported quantities here are reproducible
}
The prior code also sets a seed on the pseudo-random number generator so that reported values here are reproducible. The reflected \eqn{\mathbf{GH}(u,v; \Theta_{\mathbf{GH}})} is denoted \eqn{\mathbf{rGH}(u,v; \Theta_{\mathbf{rGH}})}.

Second, the \eqn{\mathbf{PSP}(u,v)} copula (\code{\link{PSP}}) is chosen as the parent distribution, and this copula has no parameter. The \eqn{\mathbf{PSP}} has lower-tail dependency, which will be important as discussion unfolds. The following two lines of code establish a sample size to be drawn from the \eqn{\mathbf{PSP}} and then simulates a sample from that copula. The color grey is used for the simulated values on the figure produced by \code{\link{simCOP}}, which forms a background example of the joint structure of the \eqn{\mathbf{PSP}} copula.
\preformatted{
  n <- 390
  UV <- simCOP(cop=PSP, n=n, col=8, pch=16) # simulate and form the base image
}
By inspection of the so-produced graphic, it is obvious that there is contraction in the lower-left corner of the plot, which is a geometric representation of tail dependency. The lower-tail dependency thus phenomenologically says that there is joint interconnect during low springflow conditions---both springs are likely to be at low flow simultaneously. The variable \code{UV} contains the bivariate data as uniform variables (nonexceedance probabilities \eqn{u} and \eqn{v}).

The \emph{Plackett copula} (\eqn{\mathbf{PL}(u,v; \Theta_{\mathbf{PL}})}; \code{\link{PLACKETTcop}}) and the \eqn{\mathbf{rGH}(u,v; \Theta_{\mathbf{rGH}})} copula are chosen as candidate models of the \dQuote{unknown} parent. Both \eqn{\mathbf{PL}} and \eqn{\mathbf{rGH}} copulas use different \dQuote{measures of association} for their parameter estimation. Next, sample estimates of the copula parameters using \emph{Schweizer and Wolff Sigma} \eqn{\hat\sigma_\mathbf{C}}. The sample value computations and parameter estimates also are set as shown in the following code:
\preformatted{
  Wolf   <- wolfCOP(para=UV, as.sample=TRUE) # 0.496943
  paraPL <- uniroot(function(p)
                Wolf - wolfCOP(cop=PLACKETTcop, para=p), c(1,30))$root
  paraGH <- uniroot(function(p)
                Wolf - wolfCOP(cop=rGHcop,      para=p), c(1,30))$root
}

\emph{STEP 1---Compute Kullback--Leibler sample size:} The Kullback--Leibler Divergences (\eqn{\mathrm{KL}(f {\mid} g)} and \eqn{\mathrm{KL}(g {\mid} f)}) are computed (\code{\link{kullCOP}}) for the evaluation of the sample size as appropriate for distinguishing between the two candidate copulas 95 percent of the time. The Kullback--Leibler sample size (\eqn{n_{f\!g}}) also is computed as the following code illustrates and provides additional commentary.
\preformatted{
  KL <- replicate(20, kullCOP(cop1=PLcop,  para1=paraPL,       # CPU intensive
                              cop2=rGHcop, para2=paraGH, n=1E5)$KL.sample.size)
  print(round(mean(KL))) #         n_{fg} = 221   sample size
  print(     range(KL))  # 204 <-- n_{fg} --> 252 sample size range
}

Depending on the sample \eqn{\hat\sigma_\mathbf{C}} coming from the simulation of the parent \eqn{\mathbf{PSP}} copula, the call to \code{\link{kullCOP}} will likely report different \eqn{n_{f\!g}} values because \eqn{n_{f\!g}(\mathbf{C}_1(\Theta_1), \mathbf{C}_1(\Theta_1)}. These sample sizes have a range for 20 replications of about \eqn{n_{f\!g}=204{-}252}. The result here is \eqn{n_{f\!g}=221} and thus \bold{the sample size \eqn{n=390} should be more than large enough to generally distinguish between the \eqn{\mathbf{PL}} and \eqn{\mathbf{rGH}} copulas at the respective sample measure of association.}

\emph{STEP 2---Perform the Vuong Procedure:} The Vuong Procedure can now be completed. Now watch the copula and parameter order in the next code for mistakes, the author has purposefully switched order here to draw attention to the need to make sure argument \code{cop1} has the correct parameter(s) for copula 1 (the \eqn{\mathbf{PL}}). The two calls to \code{\link{simCOP}} are made to graphically superimpose these simulations on top of the parent \eqn{\mathbf{PSP}}.
\preformatted{
  VD <- vuongCOP(UV, cop2=rGHcop, para2=paraGH, cop1=PLcop, para1=paraPL)
  print(VD) # "Copula 2 better" or rGHcop (Gumbel-Hougaard is better)
  set.seed(385) # seems harmless enough to reuse the seed to emphasize "fit"
  TMP <-simCOP(cop=PLcop, para=paraPL,n=n,plot=FALSE,col="red",  pch=16,cex=0.5)
  set.seed(385) # seems harmless enough to reuse the seed to emphasize "fit"
  TMP <-simCOP(cop=rGHcop,para=paraGH,n=n,plot=FALSE,col="green",pch=16,cex=0.5)
  rm(TMP) # just cleaning up the workspace.
}

Further discussion of the Vuong Procedure is informative. Simply speaking, the result is that \bold{the \eqn{\mathbf{rGH}} (copula 2) has better fit than \eqn{\mathbf{PL}} (copula 1).} The 95-percent confident limits from the procedure for \eqn{\hat{D}_{12} = 0.049} with p-value \eqn{0.0012}, \eqn{\hat\sigma_{12} = 0.297}, and \eqn{n=390} are \eqn{0.0194 < \hat{D}_{12} < 0.0786}. This interval does not contain zero and is greater than zero and therefore a conclusion may be drawn that copula 2 has the better fit.

\emph{STEP 3---Comparison of lower-tail dependency parameters:} What does the tail dependency do for inference? This can be checked by computing the lower-tail dependency parameters (\eqn{\lambda^L_\mathbf{C}}; \code{\link{taildepCOP}}) in the code that follows for each of the three copulas and the empirical copula with acknowledgment that true sample estimators do not quite exist. Numeric focus need only be on the lower tail, but the four graphics are informative.
\preformatted{
  taildepCOP(cop=PSP,                   plot=TRUE)$lambdaL # = 1/2
  taildepCOP(cop=PLcop,    para=paraPL, plot=TRUE)$lambdaL # = ZERO
  taildepCOP(cop=rGHcop,   para=paraGH, plot=TRUE)$lambdaL # = 0.429
  taildepCOP(cop=EMPIRcop, para=UV,     plot=TRUE)$lambdaL # = 0.328
}

The important aspect of the graphics by \code{\link{taildepCOP}} is that the \eqn{\mathbf{rGH}} has lower-tail dependency whereas the \eqn{\mathbf{PL}} does not. So, based on inspection \eqn{\mathbf{rGH}} is superior given that we known \eqn{\mathbf{PSP}} was the true parent.  The empirical estimate of the \eqn{\hat\lambda^L_\mathbf{C} = 0.328} through the \code{\link{EMPIRcop}} copula indicates that its lower-tail dependency is closer to that of the \eqn{\mathbf{rGH}} relative to \eqn{\mathbf{PL}} and thus \bold{quantitatively by lower-tail dependency the \eqn{\mathbf{rGH}} has a superior fit.}

Therefore the \eqn{\mathbf{rGH}} has a tail dependency more similar to the true model compared to the \eqn{\mathbf{PL}}. Hence for this example, the \eqn{\mathbf{rGH}} is clearly a superior fitting model in terms of the \emph{Vuong Procedure} (fit alone) and the \eqn{\lambda^L_\mathbf{C}} then is used as a follow up to shown that the \eqn{\mathbf{rGH}} might be \dQuote{good enough} an approximation to the \eqn{\mathbf{PSP}}. The efficacy of reflecting the \eqn{\mathbf{GH}} copula into a \dQuote{new} form as \eqn{\mathbf{rGH}} is demonstrated. Users are strongly encouraged to review the so-produced graphic from the \code{\link{simCOP}} call several listings back for \eqn{n=390}, and lastly, this example is one for which absence of the argument \code{snv} (standard normal variate [scores]) by \code{\link{simCOP}} makes the tail dependency issue for the sample size more prominent in the graphic.

\emph{STEP 4---Qualitatively compare using copula density plots:} Graphical depiction of copula density contours by the \code{\link{densityCOPplot}} function supports the conclusion that the \eqn{\mathbf{rGH}} is the superior model relative to the \eqn{\mathbf{PL}}. The so-produced graphic obviously shows that \bold{the \eqn{\mathbf{rGH}} strongly mimics the shape of the parent \eqn{\mathbf{PSP}}.}
\preformatted{
  densityCOPplot(cop=PSP, contour.col=8) # grey is the parent bivariate density
  densityCOPplot(cop=PLcop,  para=paraPL, contour.col="green", ploton=FALSE)
  densityCOPplot(cop=rGHcop, para=paraGH, contour.col="red",   ploton=FALSE)
}

\emph{STEP 5---Compute L-comoments of the data via simulation and estimate the sampling distributions:} An open research problem is the what if any role that \emph{L-comoments} might play in either copula estimation or inference. (There being very little literature on the topic?) Because a measure of association was used for parameter estimation, the L-correlation is uniformative, but a comparison is conceptually useful. The \eqn{\hat\sigma_\mathbf{C} = 0.4969} and \emph{Spearman Rho} of the data \eqn{\hat\rho_S} and the L-correlations \eqn{\hat\rho_S \approx \tau^{[12]}_{2} \approx \tau^{[21]}_{2} \approx 0.497} are all similar as mandated by the mathematics.

Inference using L-coskew and L-cokurtosis seems possible. The following code listing is CPU intensive. First, the L-correlation, L-coskew, and L-cokurtosis values are computed from the simulated sample by the \code{lcomoms2()} function of the \pkg{lmomco} package. Second and third, the respective sampling distributions of these L-comoments (\code{\link{lcomCOPpv}}) for the two copulas are estimated.
\preformatted{
  UVlmr <- lmomco::lcomoms2(UV, nmom=4) # The sample L-comoments
  # This execution will result in nonrejection of rGH copula.
  GHlmr <- lcomCOPpv(n, UVlmr, cop=rGHcop,      para=paraGH) # NONREJECTION
  # LcomType      n     Mean  Lscale    Lskew   Lkurt sample.est p.value signif
  #    Tau3[12] 390 -0.06952 0.01819  0.04505 0.12024   -0.11188 0.08795      .
  #    Tau3[21] 390 -0.06739 0.02084  0.04104 0.12917   -0.10673 0.14162      -
  # Tau3[12:21] 390 -0.06845 0.01713  0.04930 0.11696   -0.10931 0.08161      .
  #    Tau4[12] 390  0.04970 0.01682 -0.01635 0.10150    0.04183 0.38996      -
  #    Tau4[21] 390  0.05129 0.01606 -0.06833 0.13798    0.07804 0.17470      -
  # Tau4[12:21] 390  0.05049 0.01329 -0.02045 0.12001    0.05994 0.35069      -

  # This execution will result in rejection of Plackett copula.
  PLlmr <- lcomCOPpv(n, UVlmr, cop=PLACKETTcop, para=paraPL) # REJECT PLACKETT
  #  LcomType     n     Mean  Lscale    Lskew   Lkurt sample.est p.value signif
  #    Tau3[12] 390 -0.00267 0.02133  0.01556 0.09581   -0.11188 0.00129     **
  #    Tau3[21] 390 -0.00112 0.02022 -0.00663 0.13338   -0.10673 0.00189     **
  # Tau3[12:21] 390 -0.00189 0.01757  0.00906 0.10226   -0.10931 0.00019    ***
  #    Tau4[12] 390  0.00153 0.01652 -0.03320 0.12468    0.04183 0.07924      .
  #    Tau4[21] 390  0.00361 0.01851 -0.01869 0.12052    0.07804 0.00929     **
  # Tau4[12:21] 390  0.00257 0.01362 -0.01194 0.10796    0.05994 0.00744     **
}
Because each copula was fit to a measure of association, the p-values for the L-correlations are all nonsignificant (noninformative because of how the copulas were fit), and therefore p-values quite near to the 50th percentile should be produced. So here, the L-correlation is noninformative on fit even though it might have some role because it is asymmetrical unlike that statistics \eqn{\tau_K} and \eqn{\rho_S}. The results in variable \code{GHlmr} show no statistically significant entries (all p-values \eqn{{>}0.05 = (\alpha=0.1)/2)}) for L-coskew and L-cokurtosis---\bold{the \eqn{\mathbf{rGH}} copula is not rejected.} The results in \code{PLlmr} show many p-values \eqn{{<}0.05 = (\alpha=0.1)/2} for both L-coskew and L-cokurtosis---\bold{the \eqn{\mathbf{PL}} copula is rejected}. The experimental L-comoment inference shown is consistent with results with the Vuong Procedure.

The Vuong Procedure, however, does not address adequacy of fit---it just evaluates which copula fits better. The inspection of the lower tail dependency results previously shown (\eqn{\lambda^L_\mathbf{PSP} = 1/2 \approx \lambda^U_\mathbf{rGH}} = 0.429) along with the L-coskew and L-cokurtosis of the sample being well within the sample distribution suggests that the \eqn{\mathbf{rGH}} is a adequate mimic of the \eqn{\mathbf{PSP}} copula.

Some open research questions concern the numerical performance of the L-comoments as simulation sample size becomes large and whether or not the L-comoments should be computed on the probabilities \eqn{\{u, v\}}. Also should conversion to normal scores be made and if so, should adjustment by the \emph{Hazen plotting positions} (\eqn{u_i = (r_i - 0.5)/n} for rank \eqn{r_i}) be made that Joe (2014) repeatedly advocates when standard normal variates (scores) [\eqn{z_i = \Phi^{(-1)}(u_i)} for quantile function of standard normal distribution \eqn{\Phi(0,1)}]? Collectively, Nelsen (2006) and Salvadori \emph{et al.} (2007) are both silent on the matter of normal score conversion, and in conclusion Nelsen (2006), Salvadori \emph{et al.} (2007), and Joe (2014) also are all silent on L-comoment applications with copulas.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in Nature---An approach using copulas: Springer, 289 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{densityCOP}}, \code{\link{kullCOP}}, \code{\link{simCOP}}, \code{\link{statTn}}, \code{\link{mleCOP}}}
\examples{
# See extended code listings and discussion in the Note section
# See Examples in mleCOP() (Last example therein might suggest a problem in the
# implied 95th percentile associated with n_fg above.
}
\keyword{inference}
\keyword{copula (goodness-of-fit)}
\keyword{copula (inference)}
\keyword{copula (density)}
\keyword{inference}
\keyword{goodness-of-fit}
\concept{Vuong Procedure}
\concept{Akaike}
\concept{AIC}
\concept{BIC}
\concept{Vuong}
\concept{Kullback--Leibler}
\concept{Kullback-Leibler}
\concept{L-comoment copula inference}
\concept{Lcomoment copula inference}

\encoding{utf8}
\name{surfuncCOP}
\alias{surfuncCOP}
\title{The Joint Survival Function}
\description{
Compute the \emph{joint survival function} for a copula (Nelsen, 2006, p. 33), which is defined as
\deqn{\overline{\mathbf{C}}(u,v) = \mathrm{Pr}[U > u, V > v] = 1 - u - v + \mathbf{C}(u,v) = \hat{\mathbf{C}}(1-u, 1-v)\mbox{,}}
where \eqn{\hat{\mathbf{C}}(u',v')} is the \emph{survival copula} (\code{\link{surCOP}}), which is defined by
\deqn{\hat{\mathbf{C}}(u',v') = \mathrm{Pr}[U > u, V > v] =  u' + v' - 1 + \mathbf{C}(1-u',1-v')\mbox{.}}

Although the joint survival function is an expression of the probability that both \eqn{U > v} and \eqn{U > v}, \eqn{\overline{\mathbf{C}}(u,v)} is not a copula.
}
\usage{
surfuncCOP(u, v, cop=NULL, para=NULL, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula; and}
  \item{...}{Additional arguments to pass (such as parameters, if needed, for the copula in the form of a list.}
}
\value{
  Value(s) for the joint survival function are returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{surCOP}}}
\examples{
"MOcop.formula" <- function(u,v, para=para, ...) {
   alpha <- para[1]; beta <- para[2]; return(min(v*u^(1-alpha), u*v^(1-beta)))
}
"MOcop" <- function(u,v, ...) { asCOP(u,v, f=MOcop.formula, ...) }
u <- 0.2; v <- 0.75; ab <- c(1.5, 0.3)
# U **and** V are less than or equal to a threshold +
# U **or**  V are less than or equal to a threshold
surCOP(1-u,1-v, cop=MOcop, para=ab) + duCOP(u,v, cop=MOcop, para=ab) # UNITY
surfuncCOP(u,v, cop=MOcop, para=ab) + duCOP(u,v, cop=MOcop, para=ab) # UNITY

\dontrun{
# The joint survival function is not a copula. So, it does not increases to the upper
# right with increasing exceedance probabilities. Let us show that by hacking the surCOP
# function into a copula for feeding back into the algorithmic framework of copBasic.
UsersCop <- function(u,v, para=NULL) {
     afunc <- function(u,v, theta=para) { surfuncCOP(u, v, cop=N4212cop, para=theta) }
     return(asCOP(u,v, f=afunc)) }
image(gridCOP(cop=UsersCop, para=1.15), col=terrain.colors(20),
      xlab="U, NONEXCEEDANCE PROBABILITY", ylab="V, NONEXCEEDANCE PROBABILITY") #}

\dontrun{
# Conditional return period (Salvadori et al., 2007, p. 159)
UV <- simCOP(n=100000, cop=PLACKETTcop, para=5, graphics=FALSE)
u <- 0.5; v <- 0.99; cd <- UV$V[UV$U > u]
by.counting <- length(cd[cd > v]) / length(cd)                  # 0.0172
by.theo     <- surfuncCOP(u,v, cop=PLACKETTcop, para=5) / (1-u) # 0.0166
by.ec       <- surfuncCOP(u,v, cop=EMPIRcop, para=UV)   / (1-u) # 0.0189
print(1/by.theo) # conditional return period for V > 0.99 given U > 0.5 }
}
\keyword{copula operator}
\keyword{joint probability}
\keyword{conditional return period}
\keyword{Salvadori et al. (2007) Examples and Exercises}

\encoding{utf8}
\name{FGMcop}
\alias{FGMcop}
\alias{FGMicop}
\title{The Generalized Farlie--Gumbel--Morgenstern Copula}
\description{
The \emph{generalized Farlie--Gumbel--Morgenstern copula} (Bekrizade \emph{et al.}, 2012) is
\deqn{\mathbf{C}_{\Theta, \alpha, n}(u,v) = \mathbf{FGM}(u,v) = uv[1 + \Theta(1-u^\alpha)(1-v^\alpha)]^n\mbox{,}}
where \eqn{\Theta \in [-\mathrm{min}\{1, 1/(n\alpha^2)\}, +1/(n\alpha)]}, \eqn{\alpha > 0}, and \eqn{n \in 0,1,2,\cdots}. The copula \eqn{\Theta = 0} or \eqn{\alpha = 0} or \eqn{n = 0} becomes the \emph{independence copula} (\eqn{\mathbf{\Pi}(u,v)}; \code{\link{P}}). When \eqn{\alpha = n = 1}, then the well-known, single-parameter Farlie--Gumbel--Morgenstern copula results, and \emph{Spearman Rho} (\code{\link{rhoCOP}}) is \eqn{\rho_\mathbf{C} = \Theta/3} but in general
\deqn{\rho_\mathbf{C} = 12\sum_{r=1}^n {n \choose r} \Theta^r \biggl[\frac{\phantom{\alpha}\Gamma(r+1)\Gamma(2/\alpha)}{\alpha\Gamma(r+1+2/\alpha)} \biggr]^2
\mbox{.}}
The support of \eqn{\rho_\mathbf{C}(\cdots;\Theta, 1, 1)} is \eqn{[-1/3, +1/3]} but extends via \eqn{\alpha} and \eqn{n} to \eqn{\approx [-0.50, +0.43]}, which shows that the generalization of the copula increases the range of dependency. The generalized version is implemented by \code{FGMcop}.

The \emph{iterated Farlie--Gumbel--Morgenstern copula} (Chine and Benatia, 2017) for the \eqn{r}th iteration is
\deqn{\mathbf{C}_{\beta}(u,v) = \mathbf{FGMi}(u,v) = uv + \sum_{j=1}^{r} \beta_j\cdot(uv)^{[j/2]}\cdot(u'v')^{[(j+1)/2]}\mbox{,}}
where \eqn{u' = 1-u} and \eqn{v' = 1-v} for \eqn{|\beta_j| \le 1} that has \eqn{r} dimensions \eqn{\beta = (\beta_1, \cdots, \beta_j, \cdots, \beta_r)} and \eqn{[t]} is the integer part of \eqn{t}. The copula \eqn{\beta = 0} becomes the \emph{independence copula} (\eqn{\mathbf{\Pi}(u,v)}; \code{\link{P}}). The support of \eqn{\rho_\mathbf{C}(\cdots;\beta)} is approximately \eqn{[-0.43, +0.43]}. The iterated version is implemented by \code{FGMicop}. Internally, the \eqn{r} is determined from the length of the \eqn{\beta} in the \code{para} argument.
}
\usage{
FGMcop( u, v, para=c(NA, 1,1), ...)
FGMicop(u, v, para=NULL,       ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector of parameters. For the generalized version, the \eqn{\Theta}, \eqn{\alpha}, and \eqn{n} of the copula where the default argument shows the need to include the \eqn{\Theta}. However, if a fourth parameter is present, it is treated as a logical to reverse the copula (\eqn{u + v - 1 + \mathbf{FGM}(1-u,1-v; \Theta, \alpha, n)}). Also if a single parameter is given, then the \eqn{\alpha = n = 1} are automatically set to produce the single-parameter Farlie--Gumbel--Morgenstern copula. For the iterated version, the \eqn{\beta} vector of \eqn{r} iterations;}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\references{
Bekrizade, Hakim, Parham, G.A., Zadkarmi, M.R., 2012, The new generalization of Farlie--Gumbel--Morgenstern copulas: Applied Mathematical Sciences, v. 6, no. 71, pp. 3527--3533.

Chine, Amel, and Benatia, Fatah, 2017, Bivariate copulas parameters estimation using the trimmed L-moments methods: Afrika Statistika, v. 12, no. 1, pp. 1185--1197.
}
\author{W.H. Asquith}
\seealso{\code{\link{P}}, \code{\link{mleCOP}}
}
\examples{
\dontrun{
  # Bekrizade et al. (2012, table 1) report for a=2 and n=3 that range in
  # theta = [-0.1667, 0.1667] and range in rho = [-0.1806641, 0.4036458]. However,
  # we see that they have seemingly made an error in listing the lower bounds of theta:
  rhoCOP(FGMcop, para=c(  1/6, 2, 3))  #  0.4036458
  rhoCOP(FGMcop, para=c( -1/6, 2, 3))  # Following error results
  # In cop(u, v, para = para, ...) : parameter Theta < -0.0833333333333333
  rhoCOP(FGMcop, para=c(-1/12, 2, 3))  # -0.1806641 }

\dontrun{
  # Support of FGMrcop(): first for r=1 iterations and then for large r.
  sapply(c(-1, 1), function(t) rhoCOP(cop=FGMicop, para=rep(t, 1)) )
  # [1] -0.3333333  0.3333333
  sapply(c(-1, 1), function(t) rhoCOP(cop=FGMicop, para=rep(t,50)) )
  # [1] -0.4341385  0.4341385}

\dontrun{
  # Maximum likelihood estimation near theta upper bounds for a=3 and n=2.
  set.seed(832)
  UV <- simCOP(3000, cop=FGMcop, para=c(+0.16, 3, 2))
  # Define a transform function for parameter domain, though mleCOP does
  # provide some robustness anyway---not forcing n into the positive
  # domain via as.integer(exp(p[3])) seems to not always be needed.
  FGMpfunc <- function(p) {
    d <- p[1]; a <- exp(p[2]); n <- as.integer(exp(p[3]))
    lwr <- -min(c(1,1/(n*a^2))); upr <- 1/(n*a)
    d <- ifelse(d <= lwr, lwr, ifelse(d >= upr, upr, d))
    return( c(d, a, n) )
  }
  para <- c(0.16, 3, 2); init <- c(0, 1, 1)
  ML <- mleCOP(UV$U, UV$V, cop=FGMcop, init.para=init, parafn=FGMpfunc)
  print(ML$para) # [1] 0.1596361 3.1321228 2.0000000
  # So, we have recovered reasonable estimates of the three parameters
  # given through MLE estimation.
  densityCOPplot(cop=FGMcop, para=   para, contour.col="red")
  densityCOPplot(cop=FGMcop, para=ML$para, ploton=FALSE) #}

\dontrun{
  # L-comoment ratio diagram space of Generalized Farlie-Gumbel-Morgenstern Copula
  FGM <- NULL
  for(n in seq(1, 100, by=1)) {    Fgm <- NULL # to support intermediate plotting
    for(a in 10^seq(-5, 5, by=0.02)) { if(a == 0) next # in case log10 not involved
      rng <- c(-pmin(1, 1/(n*a^2)), 1/(n*a))
      ts  <- runif(10, min=rng[1], max=rng[2])
      for(t in ts) {
        fgm <- lcomCOP(cop=FGMcop, para=c(t, a, n))
        fgm <- data.frame(t, a, n, T2.12=fgm$lcomUV[2], T2.21=fgm$lcomVU[2],
                                   T3.12=fgm$lcomUV[3], T3.21=fgm$lcomVU[3],
                                   T4.12=fgm$lcomUV[4], T4.21=fgm$lcomVU[4])
        row.names(fgm) <- NULL; Fgm <- rbind(Fgm, fgm)
      }
    }
    plot(c(-0.5, +0.5), c(-0.1, 0.20), type="n", las=1, main=n,
         xlab="Spearman Rho", ylab="L-coskew and L-cokurtosis")
    points(Fgm$T2.12, Fgm$T3.12, col="seagreen3")
    points(Fgm$T2.12, Fgm$T4.12, col="orchid3"  )
    FGM <- rbind(FGM, Fgm)
  }
  plot(c(-0.5, +0.5), c(-0.1,0.20), type="n", las=1,
       xlab="Spearman Rho", ylab="L-coskew and L-cokurtosis")
  points(FGM$T2.12, FGM$T3.12, col="seagreen3")
  points(FGM$T2.12, FGM$T4.12, col="orchid3"  )
  save(FGM, file="LCM_FGM.RData") # }
}
\keyword{Farlie--Gumbel--Morgenstern copula}
\keyword{copula (formulas)}
\keyword{copula}
\keyword{Bekrizade et al. (2012) Examples and Exercises}
\concept{Farlie-Gumbel-Morgenstern copula}
\concept{generalized Farlie-Gumbel-Morgenstern copula}
\concept{iterated Farlie-Gumbel-Morgenstern copula}
\keyword{L-comoments (ratio diagram)}
\keyword{L-comoment ratio diagram}

\encoding{utf8}
\name{isCOP.LTD}
\alias{isCOP.LTD}
\title{Is a Copula Left-Tail Decreasing}
\description{
Numerically set a logical whether a copula is \emph{left-tail decreasing} (LTD) as described by Nelsen (2006, pp. 192--193) and Salvadori \emph{et al.} (2007, p. 222).  A copula \eqn{\mathbf{C}(u,v)} is left-tail decreasing for \eqn{\mathrm{LTD}(V{\mid}U)} if and only if for any \eqn{v \in [0,1]} that the following holds
\deqn{\frac{\delta \mathbf{C}(u,v)}{\delta u} \le \frac{\mathbf{C}(u,v)}{u}}
for almost all \eqn{u \in [0,1]}. Similarly, a copula \eqn{\mathbf{C}(u,v)} is left-tail decreasing for \eqn{\mathrm{LTD}(U{\mid}V)} if and only if for any \eqn{u \in [0,1]} that the following holds
\deqn{\frac{\delta \mathbf{C}(u,v)}{\delta v} \le \frac{\mathbf{C}(u,v)}{v}}
for almost all \eqn{v \in [0,1]} where the later definition is controlled by the \code{wrtV=TRUE} argument.

The LTD concept is associated with the concept of \emph{tail monotonicity} (Nelsen, 2006, p. 191). Specifically, but reference to Nelsen (2006) definitions and geometric interpretations is recommended, \eqn{\mathrm{LTD}(V{\mid}U)} (or \eqn{\mathrm{LTD}(V{\mid}U)}) means that the probability \eqn{P[Y \le y \mid X \le x]} (or \eqn{P[X \le x \mid Y \le y]}) is a nonincreasing function of \eqn{x} (or \eqn{y}) for all \eqn{y} (or \eqn{x}).

A positive LTD of either \eqn{\mathrm{LTD}(V{\mid}U)} or \eqn{\mathrm{LTD}(U{\mid}V)} implies positively quadrant dependency (PQD, \code{\link{isCOP.PQD}}) but the condition of PQD does not imply LTD. Finally, the accuracy of the numerical assessment of the returned logical by \code{isCOP.LTD} is dependent on the the \dQuote{smallness} of the \code{delta} argument passed into the function.
}
\usage{
isCOP.LTD(cop=NULL, para=NULL, wrtV=FALSE, delta=0.005, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters, if needed, to pass to the copula;}
  \item{wrtV}{A logical to toggle between with respect to \eqn{v} or \eqn{u} (default);}
  \item{delta}{The increment of \eqn{\{u,v\} \mapsto [0+\Delta\delta, 1-\Delta\delta, \Delta\delta]} set by \code{wrtV}; and}
  \item{...}{Additional arguments to pass to the copula or derivative of a copula function.}
}
\value{
  A logical \code{TRUE} or \code{FALSE} is returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in nature---An approach using copulas: Dordrecht, Netherlands, Springer, Water Science and Technology Library 56, 292 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{isCOP.RTI}}, \code{\link{isCOP.PQD}}}
\examples{
\dontrun{
isCOP.LTD(cop=P, delta=0.01) # independence should be FALSE
# Positive association
isCOP.LTD(cop=PSP)                               # TRUE
# Negative association Plackett
isCOP.LTD(cop=PLACKETTcop, para=0.15)            # FALSE
# Positive association Plackett
isCOP.LTD(cop=PLACKETTcop, para=15)              # TRUE
# Negative association Plackett
isCOP.LTD(cop=PLACKETTcop, wrtv=TRUE, para=0.15) # FALSE
# Positive association Plackett
isCOP.LTD(cop=PLACKETTcop, wrtV=TRUE, para=15)   # TRUE}
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}

\encoding{utf8}
\name{qua.regressCOP.draw}
\alias{qua.regressCOP.draw}
\title{Draw Quantile Regressions using a Copula by Numerical Derivative Method for V with respect to U or U with respect to V}
\description{
Draw a suite of lines for specified nonexceedance probabilities representing the \emph{quantile regression} (Nelsen, 2006, pp. 217--218) of either \eqn{V} with respect to \eqn{U} or \eqn{U} with respect to \eqn{V} depending upon an argument setting.
}
\usage{
qua.regressCOP.draw(f=seq(0.1, 0.9, by=0.1), fs=0.5, cop=NULL, para=NULL,
                    ploton=TRUE, wrtV=FALSE, col=c(4,2), lwd=c(1,2), lty=1,...)
}
\arguments{
  \item{f}{Nonexceedance probability \eqn{F} to perform quantile regression at and defaults to a 10-percent-interval sequence. This vectorization of \code{f} for this function differs from that in \code{\link{qua.regressCOP}} and \code{\link{qua.regressCOP2}};}
  \item{fs}{A special value of nonexceedance probability to draw with second values to arguments \code{col} and \code{lwd} and defaults to the median (\eqn{F = 1/2});}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{ploton}{A logical to toggle on the plot;}
  \item{wrtV}{If \code{wrtV=FALSE} call \code{\link{qua.regressCOP}} and perform quantile regression of \eqn{V} with respect to \eqn{U} and if \code{wrtV=TRUE} call \code{\link{qua.regressCOP2}} and perform regression of \eqn{U} with respect to \eqn{V};}
  \item{col}{A vector of two values for the color of the line to draw, where the first value is used for the \code{f} probabilities and the second value is used for the \code{fs} probability;}
  \item{lwd}{A vector of two values for the line width of the line to draw, where the first value is used for the \code{f} probabilities and the second value is used for the \code{fs} probability;}
  \item{lty}{The line type to draw; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  No values are returned, this function is used for its side effects.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{qua.regressCOP}}, \code{\link{qua.regressCOP2}}}
\examples{
# See example in qua.regressCOP documentation
}
\keyword{quantile regression}
\keyword{visualization}

\encoding{utf8}
\name{composite1COP}
\alias{composite1COP}
\alias{khoudraji1COP}
\alias{khoudrajiPCOP}
\title{Composition of a Single Symmetric Copula with Two Compositing Parameters (Khoudraji Device with Pi Independence)}
\description{
The \emph{composition of a single copula} (Salvadori \emph{et al.}, 2006, p. 266, prop. C.3) is created by the following result related to \dQuote{composition of copulas} in that reference.  This construction technique is named the \emph{Khoudraji device} within the \pkg{copula} package (see \code{khoudrajiCopula} therein) (Hofert \emph{et al.}, 2018, pp. 120--121). Suppose \eqn{\mathbf{C}(u,v)} is a \emph{symmetric copula} (see \code{\link{COP}}) with parameters \eqn{\Theta} and \eqn{\mathbf{C} \ne \mathbf{\Pi}} (for \eqn{\mathbf{\Pi}} see \code{\link{P}}), then a family of generally \emph{asymmetric copulas} \eqn{\mathbf{C}_{\alpha,\beta; \Theta}} with \bold{two} \emph{compositing parameters} \eqn{0 < \alpha,\beta < 1}, and \eqn{\alpha \ne \beta}, which also includes just the copula \eqn{\mathbf{C}(u,v)} as a limiting case for \eqn{\alpha = \beta = 0} and is given by

\deqn{\mathbf{C}_{\alpha,\beta}(u,v) = u^\alpha v^\beta \cdot \mathbf{C}\bigl(u^{1-\alpha},v^{1-\beta}\bigr)\mbox{.}}

The \code{composite1COP} function provides the means for inserting \emph{permutation asymmetry} from a \emph{permutation symmetric} copula as described by Joe (2017, p. 124), but do so in a more general way through the provision of two and not just one parameter. Joe's description is supported herein if one of the \eqn{\alpha} or \eqn{\beta} is held at zero. Very loosely, the \eqn{\alpha > 0} kicks probability density down towards the lower right corner, whereas \eqn{\beta > 0} kicks density up towards the upper left corner. Finally, the \code{\link{composite2COP}} function is based on a slighty more general result that is shown with \code{\link{composite2COP}}, which provides further details of copula composition and more contextualization of Hofert \emph{et al.} (2018) remarks on the \emph{Khoudraji device}.
}
\usage{
composite1COP(u, v, para, ...)
khoudraji1COP(u, v, para, ...)
khoudrajiPCOP(u, v, para, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A special parameter \code{list} (see \bold{Note}); and}
  \item{...}{Additional arguments to pass to the copula.}
}
\value{
  Value(s) for the composited copula are returned.
}
\note{
The following descriptions list in detail the structure and content of the \code{para} argument:
\describe{
  \item{\code{alpha}}{--- The \eqn{\alpha} compositing parameter;}
  \item{\code{beta}}{--- The \eqn{\beta} compositing parameter;}
  \item{\code{cop1}}{--- Function of the copula \eqn{\mathbf{C}(u,v)}; and}
  \item{\code{para1}}{--- Vector of parameters \eqn{\Theta_\mathbf{C}} for \eqn{\mathbf{C}(u,v)}.}
}
For the \code{para} argument, the same nomenclature as used for \code{\link{composite2COP}} is used with obviously \code{cop2} and \code{para2} dropped for \code{composite1COP}. The \code{cop1} and \code{para1} names remain enumerated for \code{composite1COP} so that the \code{para} argument of the more general \code{\link{composite2COP}} function could be used directly in \code{\link{composite1COP}}. Albeit, the second copula and its parameters would not be used. A more complex (extended) composition in \code{\link{composite3COP}} extends this basic parameter structure.
}
\references{
Hofert, M., Kojadinovic, I., \enc{Mächler}{Machler},  M., and Yan, J., 2018, Elements of copula modeling with R: Dordrecht, Netherlands, Springer.

Joe, H., 2017, Parametric copula families for statistical models (chap. 8)  \emph{in} Copulas and dependence models with applications---Contributions in honor of Roger B. Nelsen, \emph{eds.} Flores, U.M., Amo Artero, E., Durante, F., \enc{Sánchez}{Sanchez}, J.F.: Springer, Cham, Switzerland, ISBN 978--3--319--64220--9, \doi{10.1007/978-3-319-64221-5}.

Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in Nature---An approach using copulas: Springer, 289 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{COP}}, \code{\link{breveCOP}}, \code{\link{composite2COP}}, \code{\link{composite3COP}},
\code{\link{convexCOP}}, \code{\link{glueCOP}}}
\examples{
\dontrun{
  alpha <- 0.24; beta <- 0.23; Theta1 <- NA;
  # W() does not use parameters, but show how parameters would be set if needed.
  para  <- list(alpha=alpha, beta=beta, cop1=W, para1=Theta1)
  t <- composite1COP(0.4, 0.6, para)
  if( t != W(0.4, 0.6)) message("Not equal as expected") #}

\dontrun{
  # Hofert et al. (2018, p. 124, eq. 3.15)
  #   No matter what copula is chosen, Kendall tau must be
  #     Tau <= (alpha * beta) / (alpha + beta - alpha * beta)
  #   and those authors report Tau <= 0.5816. We can test this computation by
  para <- list(cop=M, para=NULL, alpha=1-0.6, beta=1-0.95)
  tauCOP(khoudrajiPCOP, para=para) # 0.5816283 }

\dontrun{
  # Next use this as a chance to check logic flow through the various
  # "compositing" operators and their use as needed dispatch to COP().
  my.para <- list(cop1=GHcop, para1=exp(+1.098612) + 1,
                  cop2=PLcop, para2=exp(-1.203973),
                  alpha=0.5,  beta=0.25, kappa=0.1, gamma=0.1,
                  weights=c(0.95, 0.05))
  # uses cop1/2, para1/2, only weights
  nustarCOP(cop=convexCOP,     para=my.para) # 0.8570434

  # uses cop1/2, para1/2, only alpha
  nustarCOP(cop=convex2COP,    para=my.para) # 0.2697063

  # uses cop1,   para1,   only alpha / beta
  nustarCOP(cop=composite1COP, para=my.para) # 0.5103119

  # uses cop1/2, para1/2, only alpha / beta
  nustarCOP(cop=composite2COP, para=my.para) # 0.0714571

  # uses cop1/2, para1/2, only alpha, beta, kappa, gamma
  nustarCOP(cop=composite3COP, para=my.para) # 0.0792634 }

\dontrun{
  # Hofert et al. (2018, p. 121, fig. 3.20, left panel)
  #   The ordering of copula and the "1-" operations on alpha and beta in copBasic
  #   differ from that shown in Hofert et al. (2018), but instead of their
  #   "kho(0.6, 0.95)(CLcop(6), P)" notation for the their left panel, in copBasic
  #   we can reproduce their simulation by the following. So, swapping notation
  #   between the copula package (khoudarjiCopula) and copBasic would be required.
  para <- list(cop=CLcop, para=6, alpha=1-0.95, beta=1-0.6, reflect=1)
  UV <- simCOP(n=5000, cop=khoudrajiPCOP, para=para) # then do it again with
  # reflect = 2, then 3, then 4 to show reflection/rotation dispatch to COP().}

}
\keyword{copula composition}
\keyword{copula composition (two compositing parameters)}
\keyword{copula composition (two compositing parameters with independence)}
\keyword{Khoudraji device}
\keyword{Khoudraji device (with independence)}
\keyword{Hofert et al. (2018) Examples and Exercises}

\encoding{utf8}
\name{psepolar}
\alias{psepolar}
\title{Pseudo-Polar Representation of Bivariate Data}
\description{
Kiriliouk \emph{et al.} (2016, pp. 358--360) describe a \emph{pseudo-polar} representation of bivariate data as a means to explore right-tail extremal dependency between the variables. Let \eqn{(X_i, Y_i)} (real values) or \eqn{(U_i, V_i)} (as probabilities) for \eqn{i = 1, \ldots, n} be a bivariate sample of size \eqn{n}. When such data are transformed into a \dQuote{unit-Pareto} scale by
\deqn{\widehat{X}^\star_i = n/(n+1-R_{X,i}) \mbox{\ and\ } \widehat{Y}^\star_i = n/(n+1-R_{Y,i})\mbox{,}}
where \eqn{R} is \code{rank()}, then letting each \emph{component sum} or \emph{pseudo-polar radius} be defined as
\deqn{\widehat{S}_i = \widehat{X}^\star_i + \widehat{Y}^\star_i\mbox{,}}
and each respective \emph{pseudo-polar angle} be defined as 
\deqn{\widehat{W}_i = \widehat{X}^\star_i / (\widehat{X}^\star_i + \widehat{Y}^\star_i) = \widehat{X}^\star_i / \widehat{S}_i\mbox{,}}
a pseudo-polar representation is available for study.

A scatter plot of \eqn{\widehat{W}_i} (horizontal) versus \eqn{\widehat{S}_i} (vertical) will depict a \emph{pseudo-polar plot} of the data. Kiriliouk \emph{et al.} (2016) approach the pseudo-polar concept as a means to study extremal dependency in the sense of what are the contributions of the \eqn{X} and \eqn{Y} to their sum conditional on the sum being large. The \emph{largeness} of \eqn{\widehat{S}_i} is assessed by its empirical cumulative distribution function and a threshold \eqn{S_f} stemming from \eqn{f} as a nonexceedance probability \eqn{f \in [0,1]}.

A density plot of the \eqn{\widehat{W}_i} is a representation of extremal dependence. If the density plot shows low density for pseudo-polar angles away from 0 and 1 or bimodality on the edges then weak extremal dependency is present. If the density is substantial and uniform away from the the angles 0 and 1 or if the density peaks near \eqn{\widehat{W} \approx 0.5} then extremal dependency is strong.
}
\usage{
psepolar(u, v=NULL, f=0.90, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction (actually the ranks are used so this can be a real-value argument as well);}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction  (actually the ranks are used so this can be a real-value argument as well) and if \code{NULL} then \code{u} is treated as a two column \R \code{data.frame};}
  \item{f}{The nonexceedance probability of the distal \eqn{\widehat{S}} to flag in \code{Shat_ge_Sf} column of the output; and}
  \item{...}{Additional arguments to pass to the \code{dat2bernqua()} function of the \pkg{lmomco} package.}
}
\value{
  An \R \code{data.frame} is returned in the \code{table} element and the \eqn{S_f} is in the \code{Sf} element.
  \item{U}{An echo of the \code{u} input;}
  \item{V}{An echo of the \code{v} input;}
  \item{Xstar}{The \eqn{\widehat{X}^\star_i} (Kiriliouk \emph{et al.}, 2016, eq. 17.8, p. 359);}
  \item{Ystar}{The \eqn{\widehat{Y}^\star_i} (Kiriliouk \emph{et al.}, 2016, eq. 17.8, p. 359);}
  \item{FXhat1}{The \eqn{F_{X,i} = 1 - 1/X^\star_i}, which is the inverse of Kiriliouk \emph{et al.} (2016, eq. 17.1, p. 354);}
  \item{FYhat1}{The \eqn{F_{Y,i} = 1 - 1/Y^\star_i}, which is the inverse of Kiriliouk \emph{et al.} (2016, eq. 17.1, p. 354);}
  \item{FXhat3}{The \eqn{F_{3,X,i} = (R_{X,i} - 0.5)/n} corresponding to the \dQuote{3} alternative identified by Kiriliouk \emph{et al.} (2016, p. 365);}
  \item{FYhat3}{The \eqn{F_{3,Y,i} = (R_{Y,i} - 0.5)/n} corresponding to the \dQuote{3} alternative identified by Kiriliouk \emph{et al.} (2016, p. 365);}
  \item{What}{The \eqn{\widehat{W}_i} (Kiriliouk \emph{et al.}, 2016, eq. 17.9, p. 359);}
  \item{Shat}{The \eqn{\widehat{S}_i} (Kiriliouk \emph{et al.}, 2016, eq. 17.9, p. 359); and}
  \item{Shat_ge_Sf}{A logical on whether the \eqn{\widehat{S}_i} are larger than \eqn{S_f}.}
}
\note{
The default of \code{f=0.90} means that the upper 90th percentile of the component sum will be identified in the output. This percentile is computed by the Bernstein empirical distribution function provided by the \pkg{lmomco} package through the \code{dat2bernqua()} function. Suggested arguments for \code{...} are \code{poly.type="Bernstein"} and \code{bound.type="Carv"} though the former is redundant because it is the default of \code{dat2bernqua()}.
}
\references{
Kiriliouk, Anna, Segers, Johan, \enc{Warchoł}{Warchol}, \enc{Michał}{Michal}, 2016, Nonparameteric estimation of extremal dependence: \emph{in} Extreme Value Modeling and Risk Analysis, D.K. Dey and Jun Yan \emph{eds.}, Boca Raton, FL, CRC Press, ISBN 978--1--4987--0129--7.
}
\author{William Asquith \email{william.asquith@ttu.edu}}
\seealso{\code{\link{spectralmeas}}, \code{\link{stabtaildepf}}
}
\examples{
\dontrun{
pse <- psepolar(simCOP(n=799, cop=PARETOcop, para=4.3,graphics=FALSE),bound.type="Carv")
pse <- pse$table # The Pareto copula has right-tail extreme dependency
plot(1/(1-pse$U), 1/(1-pse$V), col=pse$Shat_ge_Sf+1, lwd=0.8, cex=0.5, log="xy", pch=16)
plot(pse$What, pse$Shat, log="y", col=pse$Shat_ge_Sf+1, lwd=0.8, cex=0.5, pch=16)
plot(density(pse$What[pse$Shat_ge_Sf]), pch=16, xlim=c(0,1)) # then try the
# non-right tail extremal copula PSP as cop=PSP in the above psepolar() call.}
}
\keyword{copula (goodness-of-fit)}
\keyword{copula (inference)}
\keyword{inference}
\keyword{goodness-of-fit}
\keyword{pseudo-polar representation}
\concept{polar}

\encoding{utf8}
\name{EMPIRmed.regress}
\alias{EMPIRmed.regress}
\title{Median Regression of the Grid of the Bivariate Empirical Copula for V with respect to U}
\description{
Perform \emph{median regression} from the gridded inversion of the \emph{bivariate empirical copula} of \eqn{V} with respect to \eqn{U}.
}
\usage{
EMPIRmed.regress(...)
}
\arguments{
  \item{...}{Arguments to pass to \code{\link{EMPIRqua.regress}}.}
}
\value{
  The gridded values of the median regression of \eqn{V} with respect to \eqn{U}.
}
\author{W.H. Asquith}
\seealso{\code{\link{EMPIRgridderinv}}, \code{\link{EMPIRqua.regress}}, \code{\link{EMPIRmed.regress2}}, \code{\link{EMPIRmed.regress2}}
}
\examples{
# See examples under EMPIRqua.regress
}
\keyword{empirical copula}
\keyword{empirical copula (median regression)}
\keyword{empirical copula (quantile regression)}

\encoding{utf8}
\name{isCOP.radsym}
\alias{isCOP.radsym}
\title{Is a Copula Radially Symmetric}
\description{
Numerically set a logical whether a copula is \emph{radially symmetric} (Nelsen, 2006, p. 37) [\emph{reflection symmetric}, Joe (2014, p. 64)] (refer also to \emph{permutation symmetric}, \code{\link{isCOP.permsym}}). A copula \eqn{\mathbf{C}(u,v)} is radially symmetric if and only if for any \eqn{\{u,v\} \in [0,1]} either of the following hold
 \deqn{\mathbf{C}(u,v) = u + v - 1 + \mathbf{C}(1-u, 1-v)}
 or
 \deqn{u + v - 1 + \mathbf{C}(1-u, 1-v) - \mathbf{C}(u,v) \equiv 0\mbox{.}}

Thus, if the equality of the copula \eqn{\mathbf{C}(u,v) = \hat{\mathbf{C}}(u,v)} (the \emph{survival copula}), then radial symmetry exists: \code{\link{COP}} \eqn{=} \code{\link{surCOP}} or \eqn{\mathbf{C}(u,v) = \hat{\mathbf{C}}(1-u,1-v)}. The computation is (can be) CPU intensive.
}
\usage{
isCOP.radsym(cop=NULL, para=NULL, delta=0.005, tol=1e-4, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters, if needed, to pass to the copula;}
  \item{delta}{The increments of  \eqn{\{u,v\} \mapsto [0+\Delta\delta, 1-\Delta\delta, \Delta\delta]};}
  \item{tol}{A tolerance on the check for symmetry, default 1 part in 10,000, which is the test for the \eqn{\equiv 0} (zero equivalence, see source code); and}
  \item{...}{Additional arguments to pass to the copula or derivative of a copula function.}
}
\value{
  A logical \code{TRUE} or \code{FALSE} is returned.
}
\note{
\bold{L-comoments and Radial and Permutation Symmetry} --- A natural question exists: \emph{Is a radially symmetric copula characterized by the L-comoments for orders} \eqn{r{\ge}3} \emph{as having values of zero}? Apparently, yes, and review of copula literature does not seem to directly address this question. Let us consider the two symmetrical copulas: the parameterless \eqn{\mathbf{PSP}(u,v)} (see \code{\link{PSP}}) and the single parameter \eqn{\mathbf{PL}(u,v; \Theta)} (see \code{\link{PLcop}}) with the \eqn{\Theta_\mathbf{PL} = 4.708664} (\code{\link{rhoCOP}}). The two copulas have different radial symmetries as shown below.
\preformatted{
  J <- PLACKETTpar(rho=rhoCOP(cop=PSP)) # Spearman Rho = 0.4784176
  isCOP.radsym(     cop=PSP)            # FALSE
  isCOP.radsym(     cop=PLcop, para=J)  # TRUE
  UV <- simCOP(300, cop=PSP,   para=J, ploton=TRUE,  pch= 1, col="blue", seed=1)
  UV <- simCOP(300, cop=PLcop, para=J, ploton=FALSE, pch=16, col="red" , seed=1)
}
Now, let us compute the L-comoments (\eqn{\tau_k^{[12]|[21]}}) (\code{\link{lcomCOP}}) from each copula in the code that follows. The L-correlations (Spearman Rho) are each about 0.478, which agree with the given \eqn{\rho_\mathbf{C}}. The L-coskews (T3_PSP[12,21]) for the \eqn{\mathbf{PSP}} are about \eqn{-0.129} with those for the \eqn{\mathbf{PL}} copula at zero. The \eqn{\mathbf{PSP}} L-cokurtoses (T4_PSP[12,21]) are about \eqn{0.047}, whereas those for the \eqn{\mathbf{PL}} are zero. The [12], [21] equalities result from \emph{permutation symmetry} (\code{\link{isCOP.permsym}}), but the nonzero L-cokurtoses for the \eqn{\mathbf{PSP}} result from radial asymmetry.
\preformatted{
  lcmPSP <- lcomCOP(cop=PSP)
  lcmPLA <- lcomCOP(cop=PLcop, para=J)
  lcmPSP <-  lapply(lcmPSP, function(i) round(i, digits=5))
  lcmPLA <-  lapply(lcmPLA, function(i) round(i, digits=5))
  message( # L-correlations (T2), L-coskews (T3), L-cokurtoses (T4)
   "T2[12]_PSP == T2_PSP[21] =", lcmPSP$lcomUV[2], "=", lcmPSP$lcomVU[2], "\n",
   "T2[12]_PLA == T2_PLA[21] =", lcmPLA$lcomUV[2], "=", lcmPLA$lcomVU[2], "\n",
   "T3[12]_PSP == T3_PSP[21] =", lcmPSP$lcomUV[3], "=", lcmPSP$lcomVU[3], "\n",
   "T3[12]_PLA == T3_PLA[21] =", lcmPLA$lcomUV[3], "=", lcmPLA$lcomVU[3], "\n",
   "T4[12]_PSP == T4_PSP[21] =", lcmPSP$lcomUV[4], "=", lcmPSP$lcomVU[4], "\n",
   "T4[12]_PLA == T4_PLA[21] =", lcmPLA$lcomUV[4], "=", lcmPLA$lcomVU[4], "\n")
}

Now, \code{isCOP.radsym()} is false for all rotations (reflections) of the \eqn{\mathbf{PSP}}, which requires that in magnitude that the L-coskews are equal but with reflections 3 and 4 rotating the coupla to a \eqn{\rho_\mathbf{C} = \tau_2^{[12]} = \tau_2^{[21]} = -0.478}. The L-cokurtoses are all equal with a change in sign with reflections 3 and 4. The \eqn{\mathbf{PSP}} inherently has a bilateral symmetry about any diagonal, which makes the \eqn{r{\ge}3} L-comoments as mutual equals in magnitude.
\preformatted{
  lcmPSP1 <- lcomCOP(cop=COP, para=list(cop=PSP, reflect=1)) # default
  lcmPSP2 <- lcomCOP(cop=COP, para=list(cop=PSP, reflect=2)) # 180 degrees
  lcmPSP3 <- lcomCOP(cop=COP, para=list(cop=PSP, reflect=3)) # -90 degrees
  lcmPSP4 <- lcomCOP(cop=COP, para=list(cop=PSP, reflect=4)) # +90 degrees
  lcmPSP1 <-  lapply(lcmPSP1, function(i) round(i, digits=5))
  lcmPSP2 <-  lapply(lcmPSP2, function(i) round(i, digits=5))
  lcmPSP3 <-  lapply(lcmPSP3, function(i) round(i, digits=5))
  lcmPSP4 <-  lapply(lcmPSP4, function(i) round(i, digits=5))
  message( #  L-coskews (T3), reflections 1~, 2~, 3~, 4~
   "1~T3[12] == T3[21] = ", lcmPSP1$lcomUV[3], " = ", lcmPSP1$lcomVU[3], "\n",
   "2~T3[12] == T3[21] = ", lcmPSP2$lcomUV[3], " = ", lcmPSP2$lcomVU[3], "\n",
   "3~T3[12] != T3[21] = ", lcmPSP3$lcomUV[3], " = ", lcmPSP3$lcomVU[3], "\n",
   "4~T3[12] != T3[21] = ", lcmPSP4$lcomUV[3], " = ", lcmPSP4$lcomVU[3], "\n")
  # 1*T3[12] == T3[21] = -0.12949 = -0.12949
  # 2*T3[12] == T3[21] =  0.12949 =  0.12949
  # 3*T3[12] != T3[21] =  0.12949 = -0.12949
  # 4*T3[12] != T3[21] = -0.12949 =  0.12949
  message( # L-cokurtoses (T4), reflections 1~, 2~, 3~, 4~
   "1~T4[12] == T3[21] =", lcmPSP1$lcomUV[4], " = ", lcmPSP1$lcomVU[4], "\n",
   "2~T4[12] == T3[21] =", lcmPSP2$lcomUV[4], " = ", lcmPSP2$lcomVU[4], "\n",
   "3~T4[12] == T3[21] =", lcmPSP3$lcomUV[4], " = ", lcmPSP3$lcomVU[4], "\n",
   "4~T4[12] == T3[21] =", lcmPSP4$lcomUV[4], " = ", lcmPSP4$lcomVU[4], "\n")
  # 1~T4[12] == T3[21] =  0.04677 =  0.04677
  # 2~T4[12] == T3[21] =  0.04677 =  0.04677
  # 3~T4[12] == T3[21] = -0.04677 = -0.04677
  # 4~T4[12] == T3[21] = -0.04677 = -0.04677
}
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in Nature---An approach using copulas: Springer, 289 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{isCOP.permsym}}, \code{\link{lcomCOP}}
}
\examples{
# Radially symmetry is computationally intensive and relies on a gridded [0,1]x[0,1]
# space and laborious check on equality. Thus these examples are commented out for the
# R --timings check. Note, proof of radial symmetry absent of algebraic manipulation
# or verification is difficult and subject to the grid fineness to find a nonequality
# from which to immediately conclude FALSE. L-comoments could be used for simultaneous
# radial and permutation symmetry assessment and even beyond those by detection of
# no bilateral symmetry from any perspective (refer to the Note section above).
\dontrun{
  isCOP.radsym(cop=P) # TRUE

  para <- list(cop1=PLcop, cop2=M, para1=c(.03), para2=NA, alpha=0.8, beta=0.5)
  isCOP.radsym(      composite2COP, para=para) # FALSE
  isCOP.permsym(     composite2COP, para=para) # FALSE
  lcm <- lcomCOP(cop=composite2COP, para=para)
  lcm <- lapply(lcm, function(i) round(i, digits=5))
  message("    SpearmanRho, Lcoskew,  Lcokurt\n",
          "Tk[12] ", paste(lcm$lcomUV[2:4], collapse=", "), "\n",
          "Tk[21] ", paste(lcm$lcomVU[2:4], collapse=", "))
  #     SpearmanRho, Lcoskew,  Lcokurt  # Magitudes for L-comoment k>=3
  # Tk[12] -0.25236, 0.22828,  0.01796  # are not equal because there is
  # Tk[21] -0.25236, 0.17612, -0.05165  # no bilateral symmetry.}

\dontrun{
  set.seed(120); theta <- 2
  gh <- simCOP(n=34, cop=GHcop, para=theta, ploton=FALSE, points=FALSE) * 150
  # Pretend gh is real data, the * 150 is to clearly get into an arbitrary unit system.

  # The sort=FALSE is critical in the following two calls
  fakeU <- lmomco::pp(gh[,1], sort=FALSE) # Weibull plotting position i/(n+1)
  fakeV <- lmomco::pp(gh[,2], sort=FALSE) # Weibull plotting position i/(n+1)
  uv <- data.frame(U=fakeU, V=fakeV); # our U-statistics

  isCOP.radsym(cop=EMPIRcop, para=uv) # FALSE
  isCOP.LTD(cop=EMPIRcop,    para=uv) # TRUE
  isCOP.RTI(cop=EMPIRcop,    para=uv) # FALSE
  isCOP.PQD(cop=EMPIRcop,    para=uv, # TRUE
            empirical=TRUE) # need to set this true to avoid integrate()
   # Blomqvist's Beta = 0.2941
   #     Gini's Gamma = 0.5606
   #   Spearman's Rho = 0.6584
   #    Kendall's Tau = 0.5045

  isCOP.radsym(cop=GHcop, para=theta) # FALSE
  isCOP.LTD(   cop=GHcop, para=theta) # TRUE
  isCOP.RTI(   cop=GHcop, para=theta) # TRUE
  isCOP.PQD(   cop=GHcop, para=theta) # TRUE
    # Blomqvist's Beta = 0.5009
    #     Gini's Gamma = 0.5591
    #   Spearman's Rho = 0.6822
    #    Kendall's Tau = 0.5000

  # Notice that isCOP.RTI is not the same for empirical and theoretical.
  # This shows the difficulty in tail dependence parameter estimation for
  # small samples (refer also to Salvadori et al., 2007 p. 175).}
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{copula (symmetry)}
\keyword{L-comoments}
\keyword{Lcomoments}

\encoding{utf8}
\name{kfuncCOP}
\alias{kfuncCOP}
\alias{kmeasCOP}
\title{The Kendall (Distribution) Function of a Copula}
\description{
To begin, there are at least three terms in the literature for what appear as the same function supported by the \code{kfuncCOP} function. The \emph{Kendall Function} also is known as \emph{Kendall Distribution Function} (Nelsen, 2006, p. 163) and \emph{Kendall Measure} (Salvadori \emph{et al.}, 2007, p. 148). Each of these is dealt with in sequel to set the manner of the rather lengthy documentation for this function.

\emph{KENDALL FUNCTION}---The \emph{Kendall Function} (\eqn{F_K}) (Joe, 2014, pp. 419--422) is the cumulative distribution function (CDF) of the vector \eqn{\mathbf{U} = (U_1, U_2, \ldots)} or \eqn{\mathbf{U} = (u,v)} (bivariate) where \eqn{\mathbf{U}} is distributed as the copula: \eqn{\mathbf{U} \sim \mathbf{C}(u,v)}. Letting \eqn{Z} be the random variable for \eqn{\mathbf{C}(u,v): Z = \mathbf{C}(u,v)}, the Kendall Function is defined as

\deqn{F_K(z; \mathbf{C}) = \mathrm{Pr}[Z \le z; \mathbf{U} \sim \mathbf{C}(u,v)]\mbox{,}}

where \eqn{F_K} is the nonexceedance probability of the joint probability \eqn{z} stemming from the \eqn{\mathbf{C}}. Note, unlike its univariate counterpart, \eqn{F_K(z)} is rarely uniformly distributed (Nelsen \emph{et al.}, 2001, p. 278). The inverse \eqn{F_K^{(-1)}(z)} is implemented by the \code{\link{kfuncCOPinv}} function, which could be used for simulation of the correct joint probability using a single uniformly distributed \eqn{\sim} U(0,1) random variable providing the nonexceedance probability of the Kendall function. A reminder is needed that \eqn{Z} is the \bold{joint probability} and \eqn{F_K(z)} is the Kendall Function.

Joe (2014) and others as cited list various special cases of \eqn{F_K(z)}, inequalities, and some useful identities suitable for validation study:\cr
\eqn{\mbox{}\quad\bullet\quad\mbox{}}For \eqn{\mathbf{M}(u,v)} (see \code{\link{M}}): \eqn{F_K(z) = z} for all \eqn{0 < z < 1} for all \eqn{d \ge 2} dimensions;\cr
\eqn{\mbox{}\quad\bullet\quad\mbox{}}For \eqn{\mathbf{W}(u,v)} (see \code{\link{W}}): \eqn{F_K(z) = 1} for all \eqn{0 < z < 1} for \eqn{d = 2} (bivariate only);\cr
\eqn{\mbox{}\quad\bullet\quad\mbox{}}For \eqn{\mathbf{\Pi}(u,v)} (see \code{\link{P}}): \eqn{F_K(z) = z - z \log z} for \eqn{0 < z < 1} for \eqn{d = 2} (bivariate only);\cr
\eqn{\mbox{}\quad\bullet\quad\mbox{}}For any \eqn{\mathbf{C}}: \eqn{z \le F_K(z)} for \eqn{0 < z < 1}; and\cr
\eqn{\mbox{}\quad\bullet\quad\mbox{}}For any \eqn{\mathbf{C}}: \eqn{\mathrm{E}[Z] = 1 - \int_0^1 F_K(t)\,\mathrm{d}t \ge z} (Nelsen, 2001, p. 281) --- Z expectation, not \eqn{F_K}!\cr
\eqn{\mbox{}\quad\bullet\quad\mbox{}}For any \eqn{\mathbf{C}}: \eqn{\tau_\mathbf{C} = 3 - 4\int_0^1 F_K(t)\,\mathrm{d}t} (Nelsen, 2006, p. 163; see \code{\link{tauCOP}} [\bold{Examples}]).\cr
\eqn{\mbox{}\quad\bullet\quad\mbox{}}For any \eqn{\mathbf{C}}: \eqn{F_K(t)} does not uniquely determine the copula.

The last item is from Durante and Sempi (2015, p. 118), and later discussion herein will concern an example of theirs. By coincidence within a few days before receipt of the Durante and Sempi book, experiments using \code{kfuncCOP} suggested that numerically the Galambos (\code{\link{GLcop}}), Gumbel--Hougaard (\code{\link{GHcop}}), and H\enc{ü}{u}sler--Reiss (\code{\link{HRcop}}) extreme value copulas for the same \emph{Kendall Tau} (\eqn{\tau_\mathbf{C}}) all have the same \eqn{F_K(t)}. Therefore, do all \emph{EV}-copulas have the same Kendall Function? Well in fact, they do and Durante and Sempi (2015, p. 207) show that \eqn{F_K(z) = z - (1 - \tau_\mathbf{C})z \log(z)} for an \emph{EV}-copula.

Joe (2014, p. 420) also indicates that strength of \emph{lower-tail dependence} (\code{\link{taildepCOP}}) affects \eqn{F_K(z)} as \eqn{z \rightarrow 0^{+}}, whereas strength of \emph{upper-tail dependence} affects \eqn{F_K(z)} as \eqn{z \rightarrow 1^{-}}. (A demonstration of tail dependence dependence is made in section \bold{Note}.) Also compared to \emph{comonotonicity copula} [\eqn{\mathbf{M}}] there are no \emph{countermonotonicity copula} (\eqn{\mathbf{W}_{d > 2}}) for dimensions greater the bivariate (Joe, 2014, p. 214)

Joe (2014) does not explicitly list an expression of \eqn{F_K(z)} that is computable directly for any \eqn{\mathbf{C}(u,v)}, and Nelsen (2006, p. 163) only lists a form (see later in documentation) for \emph{Archimedean copulas}. Salvadori \emph{et al.} (2007, eq. 3.47, p. 147) also list the Archimedean form; however, Salvadori \emph{et al.} (2007, eq. 3.49, p. 148) \bold{also list a form computable directly for any} \eqn{\mathbf{C}(u,v)}. Considerable numerical experiments and derivations involving the \eqn{\mathbf{\Pi}(u,v)} copula and results for \eqn{K_\mathbf{C}(z)} shown later, indicate that the correct Kendall form for any \eqn{\mathbf{C}(u,v)} is

\deqn{F_K(z) \equiv z + \int_z^1 \frac{\delta\mathbf{C}(u,t)}{\delta u}\,\mathrm{d}u\mbox{,}}

where \eqn{t = \mathbf{C}^{(-1)}(u,z)} for \eqn{0 \le z \le 1}, \eqn{t} can be computed by the \code{\link{COPinv}} function, and the partial derivative \eqn{\delta\mathbf{C}(u,t)/\delta u} can be computed by the \code{\link{derCOP}} function. It is a curiosity that this form is not in Joe (2014), Nelsen \emph{et al.} (2001, 2003), or Nelsen (2006), but just in Salvadori \emph{et al.} (2007) of these books on copulas.\cr

\emph{KENDALL MEASURE}---The actual expression for any \eqn{\mathbf{C}(u,v)} by Salvadori \emph{et al.} (2007, eq. 3.49, p. 148) is for \emph{Kendall Measure} (\eqn{K_\mathbf{C}}) of a copula:
\deqn{K_\mathbf{C}(z) = z - \int_z^1 \frac{\delta\mathbf{C}(u,t)}{\delta u}\,\mathrm{d}u\mbox{,}}
where \eqn{t = \mathbf{C}^{(-1)}(u,z)} for \eqn{0 \le z \le 1}. Those authors report that \eqn{K_\mathbf{C}(z)} is the CDF of a random variable \eqn{Z} whose distribution is \eqn{\mathbf{C}(u,v)}. This is clearly appears to be the same meaning as Joe (2014) and Nelsen (2006). The minus \dQuote{\eqn{-}} in the above equation is very important.

Salvadori \emph{et al.} (2007, p. 148) report that \dQuote{the function \eqn{K_\mathbf{C}(z)} represents a fundamental tool for calculating the return period of extreme events.} The complement of \eqn{K_\mathbf{C}(z)} is \eqn{\overline{K}_\mathbf{C}(z) = 1 - K_\mathbf{C}(z)}, and the \eqn{\overline{K}_\mathbf{C}(z)} inverse
\deqn{\frac{1}{1 - K_\mathbf{C}(z)} = \frac{1}{\overline{K}_\mathbf{C}(z)} = T_{\mathrm{KC}}}
is referred to as a \emph{secondary return period} (Salvadori \emph{et al.}, 2007, pp. 161--170).\cr

\emph{KENDALL DISTRIBUTION FUNCTION}---Nelsen (2006, p. 163) defines the \emph{Kendall Distribution Function} (say \eqn{K^\star_\mathbf{C}(t)}) as

\deqn{K^\star_\mathbf{C}(t) = t - \frac{\phi(t)}{\phi'(t^{+})}\mbox{,}}

where \eqn{\phi(t)} is a \emph{generator function} of an \emph{Archimedean copula} and \eqn{\phi'(t^{+})} is a one-sided derivative (Nelsen, 2006, p. 125), and \eqn{\phi(t)} is \eqn{\phi(\mathbf{C}(u,v)) = \phi(u) + \phi(v)}. This same form also is listed by Salvadori \emph{et al.} (2007, eq. 3.47).

Nelsen (2006) does not seem to list a more general definition for any \eqn{\mathbf{C}(u,v)}. Because there is considerable support for Archimedean copulas in \R, \pkg{copBasic} has deliberately been kept from being yet another Archimedean-based package. This is made for more fundamental theory and pedogogic reasons without the algorithmic efficiency relative to the many convenient properties of Archimedean copulas.

The similarity of \eqn{F_K(z)}, \eqn{K_\mathbf{C}(z)}, and \eqn{K^\star_\mathbf{C}(t)}, however, is obvious---research shows that there are no syntatic differences between \eqn{F_K(z)} and \eqn{K_\mathbf{C}(t)} and \eqn{K^\star_\mathbf{C}(z)}---they all are the CDF of the joint probability \eqn{Z} of the copula. Consider now that Salvadori \emph{et al.} show \eqn{K_\mathbf{C}} having the form \eqn{a - b} and not a form \eqn{a + b} as previously shown for \eqn{F_K(z)}. Which form is thus correct? The greater bulk of this documentation seeks to answer that question, and it must be concluded that Salvadori \emph{et al.} (2007, eq. 3.49) definition for \eqn{K_\mathbf{C}(z)} has a typesetting error.
}
\usage{
kfuncCOP(z, cop=NULL, para=NULL, wrtV=FALSE, as.sample=FALSE,
            verbose=FALSE, subdivisions=100L,
            rel.tol=.Machine$double.eps^0.25, abs.tol=rel.tol, ...)
kmeasCOP(z, cop=NULL, para=NULL, wrtV=FALSE, as.sample=FALSE,
            verbose=FALSE, subdivisions=100L,
            rel.tol=.Machine$double.eps^0.25, abs.tol=rel.tol, ...)
}
\arguments{
  \item{z}{The values for \eqn{z};}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{wrtV}{A logical to toggle between with respect to \eqn{v} or \eqn{u} (default);}
  \item{as.sample}{A control on whether an optional \R \code{data.frame} in \code{para} is used to compute the empirical \eqn{\hat{F}_K(z)}. Let vector length of \code{para} be denoted \eqn{m} and \eqn{i = (1,\ldots,m)}, if \code{as.sample=TRUE}, then for each of the probability pairs in \code{para}, the \emph{empirical copula} (\code{\link{EMPIRcop}}) function with additional arguments \code{...} is used to generate a vector \eqn{(0, F^\sharp_{K,m}, 1)} of length \eqn{m+2}, then a vector of \eqn{(0, z^\sharp_m, 1)} again of length \eqn{m+2} where \eqn{z^\sharp = (i-0.5)/m} is created and linear interpolation by the \code{approx()} function in \R for each of the \eqn{z} values in \code{z} is used to estimate \eqn{\hat{F}_K(z)} (see source code). If \code{as.sample="genest"}, then the \eqn{\hat{F}_K(z)} is estimated without interpolation using a simple empirical copula basis and \dQuote{pseudo-observations} after Genest \emph{et al.} (2006, p. 339). The default \eqn{\hat{F}_K} (\code{as.sample=TRUE}) is the linear interpolation based on the default call of Weibull form of the empirical copula, but that can be confirmed by \code{ctype="weibull"} (see \bold{Note});}
  \item{verbose}{A logical supressing warnings from \code{integrate()} in \R that are usually related to \dQuote{integral divergence} for \eqn{z \rightarrow 0^{+}}. The constructed behavior of \code{kfuncCOP} is to return \eqn{F_K(z \rightarrow 0^{+}) = 0} if numerical integration returns \code{NULL}, and such a construction is made to avoid \dQuote{end points not of opposite sign} during \eqn{F_K(z)} inversion by \code{\link{kfuncCOPinv}};}
  \item{subdivisions}{Argument of same name passed to \code{integrate()},}
  \item{rel.tol}{Argument of same name passed to \code{integrate()},}
  \item{abs.tol}{Argument of same name passed to \code{integrate()}, and}
  \item{...}{Additional arguments to pass.}
}
\value{
  The value(s) for \eqn{F_K(z)} is returned.
}
\note{
\emph{VALIDATION STUDY}---A validation study using the \emph{Independence copula} (\eqn{\mathbf{\Pi =}} \eqn{uv}; \code{\link{P}}) with theoretical results of Joe (2014) and empiricism is readily performed using the expression for \eqn{F_K(z)}:
\preformatted{
  Z <- sort(c(0.01, seq(0,1, by=0.05), 0.99))       # ** Joint probabilities **
  UV <- simCOP(n=4000, cop=P, graphics=FALSE);      kendF <- Z - Z*log(Z)
  emp.kendF  <- kfuncCOP(Z, para=UV, as.sample="genest") # emp. Kendall func
  theo.kendF <- kfuncCOP(Z, cop=P) # theo. Kendall func, numeric integration
  plot(Z, kendF, type="l", col="darkgreen", lwd=4, lty=2, xlim=c(0,1),
       xlab="COPULA(u,v) VALUE [JOINT PROBABILITY]",      ylim=c(0,1),
       ylab="KENDALL FUNCTION, AS NONEXCEEDANCE PROBABILITY") # analytical
  points(Z,      kendF, col="green", lwd=1, lty=2, pch=16) # theoretical values
  points(Z,  emp.kendF, col="blue" , lwd=2, cex=1.5)       #  empirical  values
  lines( Z, theo.kendF, col="red")  # theoretical line by numerical integration
  lines( Z, kfuncCOP(Z, cop=M), lty=4) # dash-dot line (perfect correlation)
  mtext("Kendall Functions: Independence Copula")
}
The figure produced shows that the theoretical relation in Joe (2014) is correct because the empirical values from the simulated sample (\emph{Empirical Kendall Function}; Nelsen \emph{et al.}, 2003) match other curves quite well. Rerunning the above code with either \eqn{\mathbf{M}} (\code{\link{M}}) or \eqn{\mathbf{W}} (\code{\link{W}}) copulas will show that the special cases listed above are consistent with the empirical estimates. The case of \eqn{\mathbf{W}(u,v)} is degenerate at \eqn{z=0}; so, the empirical computation is in error for the smallest \eqn{z} given because of interpolation. The \eqn{\mathbf{M}} copula has \eqn{F_K} along the equal value line (1:1) line.

Now, consider a more comprehensive demonstration using the \code{\link{N4212cop}} copula with some relatively weak dependence in \eqn{\Theta = 1.17}.
\preformatted{
  Theta <- 1.17; print(rhoCOP(cop=N4212cop, para=Theta)) # Spearman Rho = 6/10
  Z <- sort(c(0.01, seq(0,1, by=0.05), 0.99))      # ** Joint probabilities **
  UV <- simCOP(n=16000,      cop=N4212cop,   para=Theta, graphics=TRUE)
  empir.kendF <- kfuncCOP(Z, as.sample=TRUE, para=UV, ctype="weibull" )
  kwrtU       <- kfuncCOP(Z, cop=N4212cop,   para=Theta, wrtV=FALSE   )
  kwrtV       <- kfuncCOP(Z, cop=N4212cop,   para=Theta, wrtV=TRUE    )
  plot(Z, empir.kendF, type="p", col="red", lwd=7, lty=2, xlim=c(0,1),
       xlab="COPULA(u,v) VALUE [JOINT PROBABILITY]",      ylim=c(0,1),
       ylab="KENDALL FUNCTION, AS NONEXCEEDANCE PROBABILITY")
  abline(0,1, lty=2, lwd=0.8); mtext("Kendall Functions: N4212(1.17) Copula")
  lines(Z, kwrtU, col="skyblue4", lwd=4, lty=2)
  lines(Z, kwrtV, col="cyan2",    lwd=2, lty=2)
}
The figure produced again shows congruence between the two theoretical computations and the empirical curve. Now, consider another comprehensive demonstration using the \code{\link{PLACKETTcop}} copula with some strong negative dependence in \eqn{\Theta = 0.04}.
\preformatted{
  Theta <- 0.04
  Z <- sort(c(0.01, seq(0,1, by=0.05), 0.99))      # ** Joint probabilities **
  UV <- simCOP(n=2600,       cop=PLACKETTcop,   para=Theta, graphics=TRUE)
  empir.kendF <- kfuncCOP(Z, as.sample="hazen", para=UV)
  kwrtU       <- kfuncCOP(Z, cop=PLACKETTcop,   para=Theta, wrtV=FALSE)
  kwrtV       <- kfuncCOP(Z, cop=PLACKETTcop,   para=Theta, wrtV=TRUE )
  plot(Z, empir.kendF, type="p", col="red", lwd=7, lty=2, xlim=c(0,1),
       xlab="COPULA(u,v) VALUE [JOINT PROBABILITY]",  ylim=c(0,1),
       ylab="KENDALL FUNCTION, AS NONEXCEEDANCE PROBABILITY")
  abline(0,1, lty=2, lwd=0.8); mtext("Kendall Function: Plackett Copula")
  lines(Z, kwrtU, col="skyblue4", lwd=4, lty=2)
  lines(Z, kwrtV, col="cyan2",    lwd=1, lty=2)
}
The figure so produced again shows congruence between the two theoretical computations and the empirical curve.

Another comparison of \eqn{F_K(z)} is useful and concerns \emph{lower-} and \emph{upper-tail dependency} parameters (\code{\link{taildepCOP}}) with a comparison of three different copula all having the same Kendall Tau. The following code computes and draws the respective \eqn{F_K(z)}:
\preformatted{
  # Given a Kendall Tau of 0.4 and the GHcop, N4212, and Plackett copulas
  # parameters respectively are:
  Phi <- 1.666667; Nu <- 1.111119; Mu <- 6.60344
  Z <- seq(0.005, 0.995, by=0.005) #  ** Joint probabilities **
  GHkenf    <- kfuncCOP(Z, cop=GHcop,       para=Phi, wrtV=FALSE)
  N4212kenf <- kfuncCOP(Z, cop=N4212cop,    para=Nu,  wrtV=FALSE)
  PLkenf    <- kfuncCOP(Z, cop=PLACKETTcop, para=Mu,  wrtV=FALSE)
  plot(qnorm(GHkenf), Z, type="l", col="black", lwd=2,
       xlab="KENDALL FUNCTION, AS STANDARD NORMAL VARIATES",   xlim=c(-3,3),
       ylab="COPULA(u,v) VALUE, AS NONEXCEEDANCE PROBABILITY", ylim=c( 0,1))
  lines(qnorm(N4212kenf), Z, col="red",  lwd=2)
  lines(qnorm(PLkenf),    Z, col="blue", lwd=2)
}
The red curve for the \eqn{\mathbf{N4212}(\Theta{=}1.1)} copula (\code{\link{N4212cop}}) is higher on the left, which shows the impact of its larger lower-tail dependency (\eqn{\lambda_\mathbf{GH}^L{=}0 < \lambda_\mathbf{N4212}^L{=}0.535}), whereas the black curve for the \eqn{\mathbf{GH}(\Theta{=}1.67)} copula (\code{\link{GHcop}}) is similarly (about same magnitude) higher on the right, which shows the impact of its larger upper-tail dependency (\eqn{\lambda_\mathbf{N4212}^L{=}0 < \lambda_\mathbf{GH}^U{=}0.484}). The blue curve for the \eqn{\mathbf{PL}(\Theta = 6.60)} copula (\code{\link{PLACKETTcop}}) nearly overwrites on the left the \eqn{\mathbf{GH}} curve, which is a reflection of both copulae having zero lower-tail dependencies (\eqn{\lambda_\mathbf{GH}^L = \lambda_\mathbf{PL}^L = 0}). Finally, as anticipated by \eqn{\lambda^U}, the curve on the right for the \eqn{\mathbf{N4212}} is just slightly larger for the \eqn{\mathbf{PL}} because the \eqn{\lambda_\mathrm{PL}^U{=}0 < \lambda_\mathbf{N4212}^U{=}0.134} (a small difference however), and again on the right, the \eqn{\mathbf{N4212}} curve is considerably smaller than the \eqn{\mathbf{GH}} because \eqn{\lambda_\mathbf{N4212}^U{=}0 < \lambda_\mathbf{GH}^U{=}0.484}.

Durante and Sempi (2015, p. 118) provide an example of two copula (\eqn{\mathbf{C}_1} and \eqn{\mathbf{C}_2}) having the same \eqn{F_K(z) = \mathrm{min}(2z, 1)}. Let us check that out:
\preformatted{
  "C1" <- function(u,v, ...) {
      if (length(u) == 1) { u <- rep(u, length(v)) } else
      if (length(v) == 1) { v <- rep(v, length(u)) }
      sapply(1:length(u), function(i) {
            min(c(u[i], max(c(v[i]/2, u[i]+v[i]-1)))) })
  }
  "C2" <- function(u,v, ...) {
      if (length(u) == 1) { u <- rep(u, length(v)) } else
      if (length(v) == 1) { v <- rep(v, length(u)) }
      sapply(1:length(u), function(i) { g <- 1/2
       max(c(0, u[i]+v[i]-1, min(c(u[i], v[i]-g)), min(c(u[i]-g, v[i])))) })
  }
  DSkf <- function(t) sapply(t, function(z) min(c(2*z, 1)))
  zs <- seq(0,1, by=.01); plot(zs, DSkf(zs), col="red", cex=3) # theory
  lines(zs, kfuncCOP(zs, cop=C1), lwd=4, col="lightgreen")
  lines(zs, kfuncCOP(zs, cop=C2), lwd=1, col="black"     )
}
The plot produced shows indeed that the numerical operations by \code{kfuncCOP} solidly work on these two strictly singular copulas \eqn{\mathbf{C}_1} and \eqn{\mathbf{C}_2} that are different from the two singular \eqn{\mathbf{M}} and \eqn{\mathbf{W}} copulas. The \eqn{F_K(z)} curves exactly matching the theoretical curve provided by Durante and Sempi are produced.

\emph{CONVERSATIONAL ASIDE}---Interestingly, Durante and Sempi (2015, pp. 115--121), like other authors of major works cited herein, do not list a general expression for the \eqn{F_K(z)} as a function of any \eqn{\mathbf{C}(u,v)}. Those authors well develop the idea of Kendall Function and show results, but for the author (Asquith), the jump based of Theorem 3.9.2 to an expression, such as shown above for \eqn{F_K(z)} based on \eqn{\mathbf{C}(u,t)/\delta u}, as an usable form for any \eqn{\mathbf{C}(u,v)} is difficult.

This is a fascinating topic because, if the Kendall Function is a reasonably important component of copula theory, then why so much difficulty in finding a canonical reference? For this one piece, whereas so much of \pkg{copBasic} features are quite notationally clear in say Nelsen (2006) or Joe (2014) but somehow not for the Kendall Function.

Perhaps to the professional mathematicians, the descriptions (nomenclature) used in all but Salvadori \emph{et al.} (2007) are clear to intended readers. But even Salvadori \emph{et al.} seemingly show theirs in error---perhaps the author (Asquith) has missed something fundamental, but the validations shown in this documentation prove at least that \code{kfuncCOP} does what it is supposed to be doing but perhaps for the wrong reasoning. Lastly, Durante and Sempi (2015) have an error in their expression for Kendall Tau as a function of \eqn{F_K(z)} (see \code{\link{tauCOP}}).


\emph{SECONDARY RETURN PERIOD}---As for \dQuote{Kendall Measure} (after the lead of Salvadori \emph{et al.} [2007, pp. 161--170]), the following code shows for purposes of discussing \emph{secondary return period} that \eqn{F_K(z)} is correct and once and for all as defined in this documentation \eqn{F_K(z) \not\equiv  K_\mathbf{C}(z)}. The secondary return period (\eqn{T_K}) is the expected interarrival time between events exceeding a \eqn{T}-year joint probability either from \eqn{U}, from \eqn{V}, or both. Let us use the 100-year level (primary return period; \eqn{T = 100}), the Gumbel--Hougaard copula (\code{\link{GHcop}}) \eqn{F^{\mathbf{GH}(3.055)}_K(z)} where the choice of \eqn{\Theta = 3.055} is made to match discussion in \code{\link{copBasic-package}} and Salvadori \emph{et al.} (2007, table 3.3, p. 166).
\preformatted{
  set.seed(1)
  # Gumbel-Hougaard [Kendall Tau=0.67267 (Salvadori et al. [2007])]
  Tyear <- 100; ANEP <- 1 - 1/Tyear; nsim <- 30000; ix <- seq_len(nsim)
  1 / (1 - kfuncCOP(ANEP, cop=GHcop, para=3.055) ) # 148.28 years
  BarT <- sapply(1:30, function(i) {
                   UV <- simCOP(n=nsim, cop=GHcop, para=3.055, graphics=FALSE)
                   nsim/sum(GHcop(UV$U, UV$V, para=3.055) > ANEP) })
  message("# BarBarT = ",               round(mean(BarT), digits=2),
          " years and StdDev(BarT) = ", round(  sd(BarT), digits=2)," years")
  # BarBarT = 151.78 years and StdDev(BarT) = 9.74 years
}
The mean of some 30 replications of a large sample simulation run for \eqn{F^\mathbf{GH}(\Theta{=}3.055)_K{(z{=}0.99)}} demonstrates empirical results that closely approximate theory \eqn{151.78 \approx 148.28}, and thus congruence is shown that the definition for \eqn{F_K(z)} must be correct and that for \eqn{K_\mathbf{C}(z)} is incorrect. Table 3.3 in Salvadori \emph{et al.} (2007, table 3.3) lists the secondary return period as 148.3 years, which matches the output of \code{kfuncCOP} and empirical results shown here.

Some additional details on secondary return period from Salvadori \emph{et al.} (2007). Letting \eqn{t_\star} be some critical joint exceedance probability level, \eqn{\overline{F}_K(z)} be the complement of \eqn{F_K(z)}, those authors (p. 166) name \emph{super-critical events} having \eqn{\overline{F}_K(t_\star) < 1 - t_\star}. Thus, the secondary return period  (\eqn{\overline{F}_K(t_\star)^{-1} = T_K}) must be greater than the primary return period (\eqn{t_\star^{-1}}), which is the case here (\eqn{T_K{=}148.3 > T{=}100}) (Salvadori \emph{et al.}, 2007, p. 166).

Salvadori \emph{et al.} (2007) provide considerable discussion of \eqn{T_K} and some highlights are:\cr
\eqn{\mbox{}\quad\bullet\quad\mbox{}}(p. 162) events equally or exceeding probability \eqn{F_K(1 - 1/t_\star)} or having return intervals \eqn{\ge T_K} \dQuote{represent [a] class of potentially dangerous events, the \emph{outliers}, and [\eqn{F_K} can be used to] introduce an \emph{ad hoc} return period for such destructive events.}\cr
\eqn{\mbox{}\quad\bullet\quad\mbox{}}(p. 162) \dQuote{primary return period [\eqn{T}] \ldots only takes into account the fact that a prescribed critical event is expected to appear once in a given time interval [\eqn{T}] \ldots \eqn{\overline{F}_K(t_\star)} provides the \emph{exact probability} that a potentially destructive event will happen at any given realization of \eqn{Z} \ldots and \eqn{T_K} gives the expected time for such an outlier to occur.}\cr
\eqn{\mbox{}\quad\bullet\quad\mbox{}}(p. 164) \dQuote{[\eqn{T}] only predicts that a critical event is \emph{expected} to appear once in a given time interval \ldots would be more important to be able to calculate (1) the probability that a super-critical [sic.] event will occur at any given realization of [\eqn{Z}], and (2) how long it takes, \emph{on average}, for a super-critical event to appear.}\cr
\eqn{\mbox{}\quad\bullet\quad\mbox{}}(p. 164) \dQuote{the function \eqn{\overline{F}_K} turns the difficult analysis of bivariate dynamics of \eqn{X} and \eqn{Y} into a simpler one-dimensional problem.}

}
\source{
The comprehensive demonstrations are shown in the \bold{Note} because of a sign convention and (or) probability convention incompatibility with the equation shown by Salvadori \emph{et al.} (2007, p. 148).  Initial source code development for \pkg{copBasic} was based on an understanding that the terms the \dQuote{Kendall Function} and \dQuote{Kendall Measure} might somehow have separate meanings---that the author must be blamed for misunderstanding the requisite nomenclature---this is evidently not true.

The \eqn{K_\mathbf{C}(z)} as shown herein simply can not reproduce \eqn{F_K(z; \mathbf{\Pi}) = z - z\log z} for the \eqn{\mathbf{\Pi}} copula unless the \dQuote{\eqn{-}} sign in the \eqn{K_\mathbf{C}(z)} equation is changed to a \dQuote{\eqn{+}} to become the \eqn{F_K(z)} definition as shown. The investigative effort to determine a valid function \code{kmeasCOP} was further complicated by fact that neither Durante and Sempi (2015), Joe (2014), Nelsen (2006), and others do not actually present a general equation for \eqn{F_K(z)} computation for any \eqn{\mathbf{C}(u,v)}.

Because of the subtle differences evidently between \dQuote{Kendall functions} (lower case), an explict derivation for \eqn{F_K(z; \mathbf{\Pi})} is informative to confirm what is meant by the \emph{Kendall Function} as defined by \eqn{F_K(z)}. Starting with \eqn{z = \mathbf{\Pi}(u,v) = uv}, then \deqn{v(z) = t = \mathbf{\Pi}^{(-1)}(u, z) = z/u\mbox{,\ and}}
\deqn{\frac{\delta}{\delta u} \mathbf{\Pi}(u,t) = \frac{\delta }{\delta u} u\,v \rightarrow v = \frac{z}{u}\mbox{,}}
substitution can now proceed:
\deqn{F_K(z; \mathbf{\Pi}) = z + \int_z^1 \frac{\delta}{\delta u}\mathbf{\Pi}(u,t)\,\mathrm{d}u\, = z + \int_z^1 \frac{z}{u}\,\mathrm{d}u{,}}
which simplfies to
\deqn{F_K(z; \mathbf{\Pi}) = z + \bigl[z\log(u)\bigr]\bigg|^1_z  = z + z[\log(1) - \log(z)] = z - z\log z\mbox{,}}
which matches the special case shown by Joe (2014) for the independence copula (\eqn{\mathbf{\Pi}}; \code{\link{P}}). It is obvious thus that the \dQuote{\eqn{+}} is needed in the \eqn{F_K(z)} definition in order to stay consistent with the basic form and integration limits shown by Salvadori \emph{et al.} (2007) for \eqn{K_\mathbf{C}(z)}.

Lastly, an example under \code{\link{W}} demonstrates the span of the Kendall function based on \code{\link{M}} and \code{\link{W}} having permutation asymmetry inserted by \code{\link{breveCOP}}. The demonstration shows that the Kendall function is a distribution unto itself with its own nonexceedance probabilities.
}
\references{
Durante, F., and Sempi, C., 2015, Principles of copula theory: Boca Raton, CRC Press, 315 p.

Genest, C., Quessy, J.F., \enc{Rémillard}{Remillard}, B., 2006, Goodness-of-fit procedures for copula models based on the probability integral transformation: Scandinavian Journal of Statistics, v. 33, no. 2, pp. 337--366.

Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Nelsen, R.B., Quesada-Molina, J.J., \enc{Rodríguez-Lallena}{Rodriguez-Lallena}, J.A., \enc{Úbeda-Flores}{Ubeda-Flores}, M., 2001, Distribution functions of copulas---A class of bivariate probability integral transforms: Statistics and Probability Letters, v. 54, no. 3, pp. 277--282.

Nelsen, R.B., Quesada-Molina, J.J., \enc{Rodríguez-Lallena}{Rodriguez-Lallena}, J.A., \enc{Úbeda-Flores}{Ubeda-Flores}, M., 2003, Kendall distribution functions: Statistics and Probability Letters, v. 65, no. 3, pp. 263--268.

Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in nature---An approach using copulas: Dordrecht, Netherlands, Springer, Water Science and Technology Library 56, 292 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{kfuncCOPinv}}, \code{\link{tauCOP}}, \code{\link{derCOP}}, \code{\link{derCOP2}}, \code{\link{derCOPinv}}, \code{\link{derCOPinv2}}}
\examples{
\dontrun{
  # Salvadori et al. (2007, p. 148, fig. 3.5 [right])
  zs <- c(0.0001, seq(0.01, 1, by=0.01), 0.9999)
  plot(zs, kmeasCOP(zs, cop=GHcop, para=5), log="y", type="l", lwd=4,
       xlab="Z <= z", ylab="Kendall Function", xlim=c(0,1), ylim=c(0.001,1)) #}
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{return period (secondary)}
\keyword{Salvadori et al. (2007) Examples and Exercises}
\keyword{Joe (2014) Examples and Exercises}
\keyword{Durante and Sempi (2015) Examples and Exercises}
\keyword{Kendall Measure}
\keyword{Kendall Function}
\concept{Kendall distribution function}
\concept{Kendall function}
\concept{Kendall measure}
\concept{decomposition of Kendall Tau}
\keyword{literature errors and inconsistencies}

\encoding{utf8}
\name{bilmoms}
\alias{bilmoms}
\title{Bivariate L-moments and L-comoments of a Copula}
\description{
\bold{Attention:} This function is deprecated in favor of \code{\link{lcomCOP}}, which uses only direct numerical \code{integrate()} on the integrals shown below. The \code{bilmoms} function is strictly based on Monte Carlo integration.

Compute the \emph{bivariate L-moments (ratios)} (\eqn{\delta^{[\ldots]}_{k;\mathbf{C}}}) of a copula \eqn{\mathbf{C}(u,v; \Theta)} and remap these into the \emph{L-comoment} matrix counterparts (Serfling and Xiao, 2007; Asquith, 2011) including \emph{L-correlation} (\emph{Spearman Rho}), \emph{L-coskew}, and \emph{L-cokurtosis}.  As described by Brahimi \emph{et al.} (2015), the first four bivariate L-moments \eqn{\delta^{[12]}_k} for random variable \eqn{X^{(1)}} or \eqn{U} with respect to (\emph{wrt}) random variable \eqn{X^{(2)}} or \eqn{V} are defined as
\deqn{\delta^{[12]}_{1;\mathbf{C}} = 2\int\!\!\int_{\mathcal{I}^2}
\mathbf{C}(u,v)\,\mathrm{d}u\mathrm{d}v - \frac{1}{2}\mbox{,}}
\deqn{\delta^{[12]}_{2;\mathbf{C}} = \int\!\!\int_{\mathcal{I}^2}
(12v - 6)
\mathbf{C}(u,v)\,\mathrm{d}u\mathrm{d}v - \frac{1}{2}\mbox{,}}
\deqn{\delta^{[12]}_{3;\mathbf{C}} = \int\!\!\int_{\mathcal{I}^2}
(60v^2 - 60v + 12)
\mathbf{C}(u,v)\,\mathrm{d}u\mathrm{d}v - \frac{1}{2}\mbox{, and}}
\deqn{\delta^{[12]}_{4;\mathbf{C}} = \int\!\!\int_{\mathcal{I}^2}
(280v^3 - 420v^2 + 180v - 20)
\mathbf{C}(u,v)\,\mathrm{d}u\mathrm{d}v - \frac{1}{2}\mbox{,}}
where the bivariate L-moments are related to the L-comoment ratios by
\deqn{6\delta^{[12]}_k = \tau^{[12]}_{k+1}\mbox{\quad and \quad}6\delta^{[21]}_k = \tau^{[21]}_{k+1}\mbox{,}}
where in other words, \dQuote{the third bivariate L-moment \eqn{\delta^{[12]}_3} is one sixth the L-cokurtosis \eqn{\tau^{[12]}_4}.} The first four bivariate L-moments yield the first five L-comoments (there is no first order L-comoment ratio). The terms and nomenclature are not easy and also the English grammar adjective \dQuote{ratios} is not always consistent in the literature. The \eqn{\delta^{[\ldots]}_{k;\mathbf{C}}} are \bold{ratios}, and the returned \code{bilcomoms} element by this function holds matrices for the marginal means, marginal L-scales and L-coscales, and then the \bold{ratio} L-comoments.

Similarly, the \eqn{\delta^{[21]}_k} are computed by switching \eqn{u \rightarrow v} in the polynomials within the above integrals multiplied to the copula in the system of equations with \eqn{u}. In general, \eqn{\delta^{[12]}_k \not= \delta^{[21]}_k} for \eqn{k > 1} unless in the case of \emph{permutation symmetric} (\code{\link{isCOP.permsym}}) copulas. By theory, \eqn{\delta^{[12]}_1 = \delta^{[21]}_1 = \rho_\mathbf{C}/6} where \eqn{\rho_\mathbf{C}} is a \emph{Spearman Rho} \code{\link{rhoCOP}}.

The integral for \eqn{\delta^{[12]}_{4;\mathbf{C}}} does not appear in Brahimi \emph{et al.} (2015) but this and the other forms are verified in the \bold{Examples} and discussion in \bold{Note}. The four \eqn{k \in [1,2,3,4]} for \eqn{U} \emph{wrt} \eqn{V} and \eqn{V} \emph{wrt} \eqn{U} comprise a full spectrum of system of seven (not eight) equations. One equation is lost because \eqn{\delta^{[12]}_1 = \delta^{[21]}_1}.
}
\usage{
bilmoms(cop=NULL, para=NULL, only.bilmoms=FALSE,
                  n=1E5, halton=TRUE, usetime=TRUE, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{only.bilmoms}{A logical to trigger return of the \eqn{\delta_k} and skip L-comoment computation;}
  \item{n}{The Monte Carlo integration size. The default seems to be at least an order of magnitude greater than needed for many applied problems;}
  \item{halton}{A logical trigging \emph{Halton sequences} for the Monte Carlo integration instead of the bivariate uniform distribution (independence). The Sobol sequences are dependent on the \code{halton()} function of the \pkg{randtoolbox} package, and the Halton sequences canvas the \eqn{\mathcal{I}^2} domain for smaller \eqn{n} values than required if statistical independence is used for the Monte Carlo integration. Note, \pkg{randtoolbox} at least at version 2.0.+ has \dQuote{scrambling} of Sobol sequences temporarily disabled; therefore, Halton sequences with \code{usetime} set true is used herein (Spring 2026);}
  \item{usetime}{The argument of the same name for \code{randtoolbox::halton()}; and}
  \item{...}{Additional arguments to pass to the \code{\link{COP}} and \code{\link{rhoCOP}} functions.}
}
\value{
  An \R \code{list} of the bivariate L-moments is returned.
  \item{bilmomUV}{The bivariate L-moments \eqn{\delta^{[12]}_k} of \eqn{U} with respect to \eqn{V} for \eqn{k \in [1,2,3,4]};}
  \item{bilmomVU}{The bivariate L-moments \eqn{\delta^{[21]}_k} of \eqn{V} with respect to \eqn{U} for \eqn{k \in [1,2,3,4]};}
  \item{error.rho}{An \dQuote{error} term in units of \eqn{\delta^{[12 \& 21]}_1} used to judge whether the sample size for the Monte Carlo integration is sufficient based on a comparison to the \emph{Spearman Rho} from direct numerical integration (not Monte Carlo based) using \code{\link{rhoCOP}} of the copula. Values for \code{error.rho} \eqn{< 1E{-}5} seem to be sufficient to judge whether \code{n} is large enough;}
  \item{bilcomoms}{If not \code{only.bilmoms}, another \R \code{list} holding the L-comoments (see \bold{Note}) computed by simple remapping of the \eqn{\delta^{[\ldots]}_k} and parallel in structure to the function \code{lcomoms2()} of the \pkg{lmomco} package; and}
  \item{source}{An attribute identifying the computational source of the bivariate L-moments and bivariate L-comoments: \dQuote{bilmoms.}}
}
\note{
The mapping of the bivariate L-moments to their L-comoment matrix counterparts is simple but nuances should be discussed and the meaning of the \code{error.rho} needs further description.  The extra effort to form L-comoment matrices (Serfling and Xiao, 2007; Asquith, 2011) is made so that output matches the structure of the sample L-comoment matrices from the \code{lcomoms2()} function of the \pkg{lmomco} package.

Concerning the triangular or tent-shaped copula of Nelsen (2006, exer. 3.7, pp. 64--65) for demonstration, simulate from the triangular copula a sample of size \eqn{m = 20{,}000} and compute some sample L-comoments using the following CPU intensive code. The function \code{\link{asCOP}} completes the vectorization needed for non-Monte Carlo integration for \code{\link{rhoCOP}}.
\preformatted{
  "trianglecop" <- function(u,v, para=NULL, ...) {
    # If para is set, then the triangle is rotated 90d clockwise.
    if(! is.null(para) && para == 1) { t <- u; u <- v; v <- t }
    if(length(u) > 1 | length(v) > 1) stop("only scalars for this function")
    v2<-v/2; if(0   <= u    & u    <= v2 & v2 <= 1/2) { return(u    )
    } else   if(0   <=   v2 &   v2 <  u  &  u < 1-v2) { return(v2   )
    } else   if(1/2 <= 1-v2 & 1-v2 <= u  &  u <= 1  ) { return(u+v-1)
    } else { stop("should not be here in logic") }
  }
  "TriCop" <- function(u,v, ...) { asCOP(u,v, f=trianglecop, ...) }
  m=20000; SampleUV <- simCOP(n=m, cop=TriCop, graphics=FALSE)
  samLC  <- lmomco::lcomoms2(SampleUV, nmom=5) # sample L-comoments
  theoLC <- bilmoms(cop=TriCop)
}

\emph{The Error in Rho Computation}---The \eqn{\rho_\mathbf{C}} of the copula by numerical integration is computed internally to \code{bilmoms} as
\preformatted{
  rhoC <- rhoCOP(cop=TriCop) # -1.733858e-17
}
and used to compute the \code{error.rho} for \code{bilmoms} (see next code snippet). The  \eqn{\rho_\mathbf{C}} is obviously zero for this copula. Therefore, the bivariate association of \code{TriCop} is zero and thus is an example of a perfectly dependent situation yet of zero correlation. The bivariate L-moments and L-comoments of this copula are computed as
\preformatted{
  mean(replicate(20, bilmoms(cop=TriCop)$error.rho)) # 7.650723e-06
}
where the \code{error.rho} is repeated trials appears firmly \eqn{<}\code{1e-5}, which is near zero (\eqn{\epsilon_\rho \approx 0}). The \code{error.rho} term is defined by taking the first bivariate L-moment and numerically integrated \eqn{\rho_\mathbf{C}} through \code{\link{rhoCOP}} and computing the terms
\deqn{\epsilon^{[12]}_\rho = |\delta^{[12]}_{1;\mathbf{C}} - (\rho_\mathbf{C}/6)|\mbox{,}}
\deqn{\epsilon^{[21]}_\rho = |\delta^{[21]}_{1;\mathbf{C}} - (\rho_\mathbf{C}/6)|\mbox{, and}}
\deqn{\epsilon_\rho = \frac{\epsilon^{[12]}_\rho  + \epsilon^{[21]}_\rho}{2}\mbox{,}}
where the \code{error.rho} \eqn{=} \eqn{\epsilon_\rho}, and values near zero are obviously favorable because this indicates that the Monte Carlo integration sample size \eqn{n} argument is sufficiently large to effectively canvas the \eqn{\mathcal{I}^2} domain. For the situation here, the theoretical \eqn{\rho_\mathbf{C} = 0}, but for \code{n} \eqn{= n = 100}, the \code{error.rho} \eqn{\approx} 0.006 (\emph{e.g.} \code{bilmoms(n=100,} \code{cop=TriCop)$error.rho}) through a 20-unit replication, which is a hint that 100 samples are not large enough and that should be obvious.

The reasoning behind using the \code{error.rho} between conventional numerical integration and the Monte Carlo integration (\code{error.rho}) is that \eqn{\rho_\mathbf{C}} is symmetrical. This choice of \dQuote{convergence} assessment reduces somewhat the sample size needed for Monte Carlo integration into single number representing error.

\emph{Discussion of Theoretical L-comoments}---The theoretical L-comoments in the format structure of the sample L-comoments by the \code{lcomoms2()} function of the \pkg{lmomco} package are formed by the \code{bilmoms} function, and the theoretical values are shown below in sequence with details listed by L-comoment. Now we extract the L-comoment matrices and show the first L-comoment matrix (the matrix of the means):
\preformatted{
  theoLClcm <- theoLC$bilcomoms
  print(theoLClcm$L1)
            [,1]      [,2]
  [1,] 0.4999939        NA
  [2,]        NA 0.5000032
}
where the diagonal should be filled with 1/2, if the \eqn{n} is suitably large, because 1/2 is the mean of the marginal uniform random variables. By definition the secondary diagonal has \code{NA}s. The values shown above are extremely close supporting the idea that default \eqn{n} is large enough. The matrix of means is otherwise uninformative.

The second L-comoment matrix (L-scales and L-coscales) is
\preformatted{
  print(theoLClcm$L2)
                [,1]          [,2]
  [1,]  1.666677e-01 -3.466202e-06
  [2,] -3.466209e-06  1.666674e-01
}
where the diagonal should be filled with 1/6, if the \eqn{n} is suitably large, because 1/6 is the univariate L-scale of the marginal uniform random variables. These values further support that default \eqn{n} is large enough. The diagonal is computed from the univariate L-moments of the margins of the Monte Carlo-generated edges and is otherwise uninformative. The secondary diagonal is a rescaling of the \eqn{\delta^{[\ldots]}_1} by the univariate L-moments of the margins to form L-coscales (nonratios). The copula is perfectly dependent but uncorrelated; so the secondary diagonal has near zeros.

The second L-comoment ratio matrix (coefficient of L-variations and L-correlations) is
\preformatted{
  print(theoLClcm$T2)
                [,1]          [,2]
  [1,]  1.000000e+00 -2.079712e-05
  [2,] -2.079712e-05  1.000000e+00
}
where the diagonal by definition has unities (correlation is unity for a variable on itself) but the secondary diagonal for the L-correlations has near zeros because again the copula is uncorrelated, and the secondary diagonal is computed from the \eqn{\delta^{[\ldots]}_1}. These L-correlations are the \emph{Spearman Rho} values computed external to the algorithms within \code{\link{rhoCOP}}.

The third L-comoment ratio matrix (L-skews and L-coskews) is
\preformatted{
  print(theoLClcm$T3)
                [,1]          [,2]
  [1,]  3.021969e-06 -2.829783e-05
  [2,] -7.501135e-01  4.518901e-06
}
where the diagonal by definition should have nero zeros because the univariate L-skew of a uniform variable is zero. These values further support that default \eqn{n} is large enough.  The secondary diagonal holds L-coskews. The copula has L-coskew of \eqn{U} \emph{wrt} \eqn{V} of numerically near zero (symmetry) but measurable asymmetry of L-coskew of \eqn{V} \emph{wrt} \eqn{U} of \eqn{\tau^{[21]}_3 \approx -0.75}.

The fourth L-comoment ratio matrix (L-kurtosises and L-cokurtosises) is
\preformatted{
  print(theoLClcm$T4)
                [,1]          [,2]
  [1,] -2.623665e-06 -3.325177e-05
  [2,] -2.162954e-04 -2.811630e-06
}
where the diagonal by definition should have nero zeros because the univariate L-kurtosis of a uniform variable is zero---it has no peakedness. These values further support that default \eqn{n} is large enough. The secondary diagonal holds L-cokurtosises and are near zero for this particular copula.

The fifth L-comoment ratio matrix (unnamed) is
\preformatted{
  print(theoLClcm$T5)
               [,1]          [,2]
  [1,] 1.813344e-06 -1.296025e-04
  [2,] 1.246436e-01  1.475012e-06
}
where the diagonal by definition should have nero zeros because the univariate L-kurtosis of a uniform variable is zero---such a random variable has no asymmetry. These values further support that default \eqn{n} is large enough. The secondary diagonal holds the fifth L-comoment ratios.  The copula has \eqn{\tau^{[12]}_5 = 0} of \eqn{U} \emph{wrt} \eqn{V} of numerically near zero (symmetry) but measurable fifth-order L-comoment asymmetry of \eqn{V} \emph{wrt} \eqn{U} of \eqn{\tau^{[21]}_5 \approx 0.125}.

\emph{Comparison of Sample and Theoretical L-comoments}---The previous section shows theoretical values computed as \eqn{\tau^{[21]}_3 \approx -0.75} and \eqn{\tau^{[21]}_5 \approx 0.125} for the two L-comoments substantially away from zero. As sample L-comoments these values are \code{samLC$T3[2,1]} \eqn{=} \eqn{\hat\tau^{[21]}_3 \approx -0.751} and \code{samLC$T5[2,1]} \eqn{=} \eqn{\hat\tau^{[21]}_5 \approx 0.123}. CONCLUSION: \emph{The sample L-comoment algorithms in the \pkg{lmomco} package are validated.}
}
\references{
Asquith, W.H., 2011, Distributional analysis with L-moment statistics using the R environment for statistical computing: Createspace Independent Publishing Platform, ISBN 978--146350841--8.

Brahimi, B., Chebana, F., and Necir, A., 2015, Copula representation of bivariate L-moments---A new estimation method for multiparameter two-dimensional copula models: Statistics, v. 49, no. 3, pp. 497--521.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Serfling, R., and Xiao, P., 2007, A contribution to multivariate L-moments---L-comoment matrices: Journal of Multivariate Analysis, v. 98, pp. 1765--1781.
}
\author{W.H. Asquith}
\seealso{\code{\link{lcomCOP}}, \code{\link{uvlmoms}}
}
\examples{
\dontrun{
  bilmoms(cop=PSP, n=10000, para=NULL, halton=TRUE)$bilcomoms$T3
  # results: Tau3[12]=-0.132, Tau3[21]=-0.132 (Monte Carlo integration)
  lcomCOP(cop=PSP, para=NULL, orders=3)
  # results: Tau3[12]=-0.129, Tau3[21]=-0.129 (direct numerical integration) }

\dontrun{
  para <- list(alpha=0.5, beta=0.93, para1=4.5, cop1=GLcop, cop2=PSP)
  bilmoms(cop=composite2COP, n=10000, para=para, halton=TRUE)$bilcomoms$T3
  # results: Tau3[12]=0.154, Tau3[21]=-0.0691 (Monte Carlo integration)
  lcomCOP(cop=composite2COP, para=para, orders=3)
  # results: Tau3[12]=0.156, Tau3[21]=-0.0668 (direct numerical integration) }

\dontrun{
  para <- list(alpha=0.5, beta=0.93, para1=4.5, cop1=GLcop, cop2=PSP)
  UVsim <- simCOP(n=20000, cop=composite2COP, para=para, graphics=FALSE)
  samLcom <- lmomco::lcomoms2(UVsim, nmom=5) # sample algorithm
  # results: Tau3[12]=0.1489, Tau3[21]=-0.0679 (simulation) }
}
\keyword{bivariate asymmetry (measures)}
\keyword{bivariate skewness}
\keyword{bivariate L-moments}
\keyword{Nelsen (2006) Examples and Exercises}
\keyword{Halton sequence (Monte Carlo integration)}
\keyword{L-comoments}
\keyword{Spearman Rho}
\keyword{Package randtoolbox}
\concept{bivariate L-correlation}
\concept{bivariate L-skew}
\concept{bivariate L-kurtosis}
\concept{copula L-comoments}
\concept{copula L-correlation}
\concept{copula L-coskew}
\concept{copula L-cokurtosis}
\concept{Lcomoments}

\encoding{utf8}
\name{level.curvesCOP2}
\alias{level.curvesCOP2}
\title{Compute and Plot Level Curves of a Copula U with respect to V}
\description{
Compute and plot \emph{level curves} or \emph{level sets} of a copula for \eqn{U} with respect to \eqn{V} (Nelsen, 2006, pp. 12--13). The level curves at a levels \eqn{t \mapsto [0+\Delta t, 1-\Delta t, \Delta t]} are defined for \eqn{V \mapsto [0+\Delta v, 1-\Delta v, \Delta v]} by
\deqn{t = \mathbf{C}(u, v=V)\mbox{,}}
and solving for \eqn{u}. Plotting is provided by this function because level curves are such an important visual attribute of a copula and highly useful for pedagogic  purposes. The above equation is implemented by the \emph{inverse of a copula} using \code{\link{COPinv2}}.
}
\usage{
level.curvesCOP2(cop=NULL, para=NULL, ploton=TRUE, lines=TRUE,
                 plotMW=FALSE, ramp=TRUE, delv=0.001, delt=0.10,
                 getlevel=NULL, silent=TRUE, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{ploton}{A logical to toggle on the plot;}
  \item{lines}{A logical to toggle calls to the \code{lines()} function in \R to draw the lines;}
  \item{plotMW}{A logical to toggle to use \code{abline()} function in \R to plot cross lines for the \eqn{\mathbf{M}} (\code{\link{M}}) and  \eqn{\mathbf{W}} (\code{\link{W}}) copulas;}
   \item{ramp}{A logical to toggle whether the level curves are ramped in thickness according to the probability of the line. The argument can be used with a \code{lwd} setting through the \code{...} and can either be a constant, which will override the built-in ramp, or \code{lwd} can be defined as a function to replace the built-in ramp in terms of the probability;}
  \item{delv}{The increment of \eqn{\Delta v}. The default is 1 part in 1,000, which should often provide enough smoothness for many copulas in practice;}
  \item{delt}{The increment of \eqn{\Delta t} for the level curves to plot, defaults to 10-percent intervals;}
  \item{getlevel}{If defined and level exists upon stepping through using \code{delt}, then the level curve at the \code{getlevel} is returned in an \R \code{list} data structure;}
  \item{silent}{The argument of the same name given over to \code{try()} wrapping the \code{try()} operation on forming sequences of \eqn{t} for the curves (see sources); and}
  \item{...}{Additional arguments to pass to the \code{lines()} function in \R.}
}
\value{
  Typically no values are returned because this function is used for its side effects, but the arguments can be such that the \eqn{\{u, v\}} for \eqn{\mathbf{C}(u,v) = t} are returned within an \R \code{list}.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{COPinv2}}, \code{\link{level.curvesCOP}}, \code{\link{level.setCOP2}}, \code{\link{joint.curvesCOP2}}
}
\examples{
\dontrun{
level.curvesCOP2(cop=M, para=NULL, delt=0.02) # Upper bounds copula}
}
\keyword{visualization}
\keyword{diagnostics}
\concept{level curves}
\concept{level curve}
\concept{level sets}
\concept{level set}
\concept{level contours}
\concept{level contour}

\encoding{utf8}
\name{ReineckeWells}
\docType{data}
\alias{ReineckeWells}
\title{Porosity and Permeability Data for the Reinecke Oil Field, Horseshoe Atoll, Texas}
\description{
These data represent porosity and permeability data from laboratory analysis for the Reinecke Oil Field, Horseshoe Atoll, Texas as used for the outstanding article by Saller and Dickson (2011). Dr. A.H. Art Saller shared a CSV file with the author of the \pkg{copBasic} package sometime in 2011.  These data are included in this package because of the instruction potential of the bivariate relation between the geologic properties of permeability and porosity.
}
\usage{
data(ReineckeWells)
}
\format{
An \R \code{data.frame} with
\describe{
  \item{DOLOMITE}{The fraction of the core sample that is dolomite, 0 is 100 percent limestone;}
  \item{Kmax}{The maximum permeability without respect to orientation in millidarcies;}
  \item{K90}{The horizontal (with respect to 90 degrees of the borehole) permeability in millidarcies;}
  \item{Kvert}{The vertical permeability in millidarcies; and}
  \item{POROSITY}{The porosity of the core sample.}
}
}
\references{
Saller, A.H., Dickson, J.A., 2011, Partial dolomitization of a Pennsylvanian limestone buildup by hydrothermal fluids and its effect on reservoir quality and performance: AAPG Bulletin, v. 95, no. 10, pp. 1745--1762.
}
\examples{
\dontrun{
data(ReineckeWells)
summary(ReineckeWells) # show summary statistics}
}
\keyword{datasets}

\encoding{utf8}
\name{duCOP}
\alias{duCOP}
\title{The Dual of a Copula Function}
\description{
Compute the \emph{dual of a copula (function)} from a copula (Nelsen, 2006, pp. 33--34), which is defined as

\deqn{\mathrm{Pr}[U \le v \mathrm{\ or\ } V \le v] = \tilde{\mathbf{C}}(u,v) = u + v - \mathbf{C}(u,v)\mbox{,}}

where \eqn{\tilde{\mathbf{C}}(u,v)} is the dual of a copula and \eqn{u} and \eqn{v} are nonexceedance probabilities. The dual of a copula is the expression for the probability that either \eqn{U \le u} \bold{or} \eqn{V \le v}, which is unlike the \emph{co-copula (function)} (see \code{\link{coCOP}}) that provides \eqn{\mathrm{Pr}[U > u \mathrm{\ or\ } V > v]}. The dual of a copula is a function and not in itself a copula. The dual of the  \emph{survival copula} (\code{\link{surCOP}}) is the \emph{co-copula (function)} (\code{\link{coCOP}}). Some rules of copulas mean that

\deqn{\hat{\mathbf{C}}(u',v') + \tilde{\mathbf{C}}(u,v) = 1\mbox{,}}

where \eqn{\hat{\mathbf{C}}(u',v')} is the survival copula in terms of exceedance probabilities \eqn{u'} and \eqn{v'} or in \pkg{copBasic} code that the functions \code{\link{surCOP}} + \code{duCOP} equal unity.

The function \code{duCOP} gives \dQuote{protection} against simultaneous (concurrent or dual) risk by failure if and only if failure is caused (defined) by both hazard sources \eqn{U} and \eqn{V} being by themselves responsible for failure. Expressing this in terms of an annual probability of occurrence (\eqn{q}), one has
\deqn{q = 1 - \mathrm{Pr}[U \le v \mathrm{\ or\ } V \le v] = 1 - \tilde{\mathbf{C}}(u,v)\mbox{\ or}}
in \R code \code{q <- 1 - duCOP(u,v)}. So, as a mnemonic: \emph{A dual of a copula is the probability of nonexceedance if the hazard sources must \bold{dual} (concur, link, pair, twin, twain) between each other to cause failure.} An informative graphic is shown within \code{\link{copBasic-package}}.
}
\usage{
duCOP(u, v, cop=NULL, para=NULL, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula; and}
  \item{...}{Additional arguments to pass (such as parameters, if needed, for the copula in the form of a list.}
}
\value{
  Value(s) for the dual of a copula are returned.
}
\note{
There can be confusion in the interpretation and implemenation of the \bold{or} condition of \emph{joint probability} provided by \eqn{\tilde{\mathbf{C}}(u,v)}. Two types of \bold{or}'s seemingly exist depending on one's concept of the meaning of \dQuote{or.}  To start, there is the \dQuote{either or both} conceptualization (\bold{joint or}) that encompasses either \dQuote{event} (say a loss) of importance for random variables \eqn{U} and \eqn{V} \emph{as well as} the \bold{joint and} conditions where both variables simultaneously are generating an event of importance.

Let us continue by performing a massive simulation for the \eqn{\mathbf{PSP}(u,v)} copula (\code{\link{PSP}}) and set an either event standard on the margins as 10 percent for an arbitrary starting point. The \eqn{\mathbf{PSP}} has positive association with lower tail dependency, and the example here considers the left tail as the risk tail.
\preformatted{
  Event <- 0.1; nn <- 100000; set.seed(9238)
  UV <- simCOP(n=nn, cop=PSP, graphics=FALSE) # 1E5 realizations
}
Next, let us step through counting and then make theoretical comparisons using copula theory. The \bold{joint and} condition as nonexceedances is
\preformatted{
  ANDs <- length(UV$U[UV$U <= Event & UV$V <= Event]) / nn
  ANDt <- COP(Event, Event, cop=PSP)
  message(   "Joint AND by simulation = ", round(ANDs, digits=5),
           "\n    Joint AND by theory = ", round(ANDt, digits=5))
  # ANDs = 0.05348 and ANDt = 0.05263 (numerical congruence)
}
where it is obvious that the simulations and theory estimate about the same \bold{joint and} condition. Now, the \bold{joint or} condition as nonexceedances is
\preformatted{
  ORs <- length(UV$U[UV$U <= Event | UV$V <= Event]) / nn
  ORt <- duCOP(Event, Event, cop=PSP)
  message(   "Joint OR by simulation = ", round(ORs, digits=5),
           "\n    Joint OR by theory = ", round(ORt, digits=5))
  # ORs = 0.14779 and ORt = 0.14737 (numerical congruence)
}
where it is obvious that the simulations and theory estimate about the same \bold{joint or} condition. Finally, the joint \bold{mutually exclusive or} condition as nonexceedances is
\preformatted{
  eORs <- length((UV$U[(UV$U <= Event | UV$V <= Event) &
                     ! (UV$U <= Event & UV$V <= Event)])) / nn
  eORt <- ORt - ANDt # theoretical computation
  message(   "Joint exclusive OR by simulation = ", round(eORs, digits=5),
           "\n    Joint exclusive OR by theory = ", round(eORt, digits=5))
  # eORs = 0.09431 and eORt = 0.09474 (numerical congruence)
}
where it is obvious that the simulations and theory estimate about the same joint \bold{mutually exclusive or} condition, and where it is shown that the prior two theoretical joint probabilities can be subtracted from each to yield the \bold{mutually exclusive or} condition.


Let us then play out a scenario in which it is judged that of the events causing damage that the simultaneous occurrence is worse but that engineering against about 5 percent of events not occurring at the same time represents the most funding available. Using numerical methods, it is possible to combine \eqn{\tilde{\mathbf{C}}} and \eqn{\mathbf{C}} and assume equal marginal risk in \eqn{U} and \eqn{V} as the following list shows:
\preformatted{
  "designf" <- function(t) { # a one-off function just for this example
     duCOP(t, t, cop=PSP) - COP(t, t, cop=PSP) - 5/100 # 5 percent
  }
  dThres <- uniroot(designf, c(.Machine$double.eps,0.5))$root
}
where the \code{uniroot} function performs the optimization and the \code{.Machine$double.eps} value is used because the \eqn{\mathbf{PSP}} is \code{NaN} for zero probability. (It is unity for unity marginal probabilities.)

The design threshold on the margins then is \code{dThres} \eqn{\approx} 0.05135. In other words, the \code{designThres} is the marginal probability that results in about 5 percent of events not occurring at the same time. Then considering the simulated sample and counting the nonexceedances by code one achieves:
\preformatted{
  Damage       <- length( UV$U[ UV$U <= dThres | UV$V <= dThres ])
  SimDamage    <- length( UV$U[ UV$U <= dThres & UV$V <= dThres ])
  NonSimDamage <- length((UV$U[(UV$U <= dThres | UV$V <= dThres) &
                             ! (UV$U <= dThres & UV$V <= dThres)]) )
  message(  "                 Damaging Events (sim.) = ", Damage,
          "\n    Simultaneous damaging events (sim.) = ", SimDamage,
          "\n Nonsimultaneous damaging events (sim.) = ", NonSimDamage)
}
but also the theoretical expectations are readily computed using copula theory:
\preformatted{
  tDamage       <- as.integer(duCOP(dThres, dThres, cop=PSP) * nn)
  tSimDamage    <- as.integer(  COP(dThres, dThres, cop=PSP) * nn)
  tNonSimDamage <- tDamage - tSimDamage
  message(  "                 Damaging Events (theory) = ", tDamage,
          "\n    Simultaneous damaging events (theory) = ", tSimDamage,
          "\n Nonsimultaneous damaging events (theory) = ", tNonSimDamage)
}
The counts from the former listing are 7,670; 2,669; and 5,001, whereas the respective counts from the later listing are 7,635; 2,635; and 5,000. Numerical congruency in the counts thus exists.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{COP}}, \code{\link{coCOP}}, \code{\link{surCOP}}, \code{\link{jointCOP}}, \code{\link{joint.curvesCOP}}}
\examples{
u <- runif(1); t <- runif(1)
duCOP(cop=W,u,t)    # joint or probability for perfect negative dependence
duCOP(cop=P,u,t)    # joint or probability for perfect        independence
duCOP(cop=M,u,t)    # joint or probability for perfect positive dependence
duCOP(cop=PSP,u,t)  # joint or probability for some positive    dependence

# Next demonstrate COP + duCOP = unity.
"MOcop.formula" <- function(u,v, para=para, ...) {
   alpha <- para[1]; beta <- para[2]; return(min(v*u^(1-alpha), u*v^(1-beta)))
}
"MOcop" <- function(u,v, ...) { asCOP(u,v, f=MOcop.formula, ...) }

u <- 0.2; v <- 0.75; ab <- c(1.5, 0.3)
surCOP(1-u,1-v, cop=MOcop, para=ab) + duCOP(u,v, cop=MOcop, para=ab) # UNITY

# See extended code listings and discussion in the Note section}
\keyword{copula operator}
\keyword{joint probability}
\keyword{joint probability (exclusive or)}
\concept{exclusive or}
\concept{mutually exclusive or}
\concept{mutually exclusive or condition}

\encoding{utf8}
\name{footCOP}
\alias{footCOP}
\title{The Spearman Footrule of a Copula}
\description{
Compute the measure of association known as the \emph{Spearman Footrule} \eqn{\psi_\mathbf{C}} (Nelsen \emph{et al.}, 2001, p. 281), which is defined as

\deqn{\psi_\mathbf{C} = \frac{3}{2}\mathcal{Q}(\mathbf{C},\mathbf{M}) - \frac{1}{2}\mbox{,}}

where \eqn{\mathbf{C}(u,v)} is the copula, \eqn{\mathbf{M}(u,v)} is the \emph{\enc{Fréchet}{Frechet}--Hoeffding upper bound} (\code{\link{M}}), \eqn{\mathcal{Q}(a,b)} is a \emph{concordance function} (\code{\link{concordCOP}}) (Nelsen, 2006, p. 158), and \eqn{-0.5 \le \psi_\mathbf{C} \le 1}. The \eqn{\psi_\mathbf{C}} in terms of a single integration pass on the copula (Genest \emph{et al.}, 2010, p. 940) is
\deqn{\psi_\mathbf{C} = 1 - \int_{\mathcal{I}^2} |u-v|\,\mathrm{d}\mathbf{C}(u,v) = 6 \int_0^1 \mathbf{C}(u,u)\,\mathrm{d}u - 2\mbox{.}}
% http://www.encyclopediaofmath.org/index.php?title=Spearman_rho_metric&oldid=15466  (accessed Aug 9, 2015)
Note, Nelsen \emph{et al.} (2001) use \eqn{\phi_\mathbf{C}} but that symbol is taken in \pkg{copBasic} for the \emph{Hoeffding Phi} (\code{\link{hoefCOP}}), and Spearman Footrule does not seem to appear in Nelsen (2006). From the definition, Spearman Footrule only depends on the \emph{primary diagnonal} (alt. \emph{main diagonal}, Genest \emph{et al.}, 2010) of the copula, \eqn{\mathbf{C}(t,t)} (\code{\link{diagCOP}}). The function here supports the first \eqn{\psi_\mathbf{C}} computation by \code{as.concordance=TRUE}, but the default is the later definition by the direct integration on the diagonal.
}
\usage{
footCOP(cop=NULL, para=NULL, by.concordance=FALSE, as.sample=FALSE, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{by.concordance}{Instead of using the single integral to compute \eqn{\psi_\mathbf{C}}, use the concordance function method implemented through \code{\link{concordCOP}}; and}
  \item{as.sample}{A logical controlling whether an optional \R \code{data.frame} in \code{para} is used to compute the \eqn{\hat\psi} (see \bold{Note}); and}
  \item{...}{Additional arguments to pass, which are dispatched to the copula function \code{cop} and possibly \code{\link{concordCOP}}, such as \code{brute} or \code{delta} used by that function.}
}
\value{
  The value for \eqn{\psi_\mathbf{C}} is returned.
}
\note{
Conceptually, the sample Spearman Footrule is a standardized sum of the absolute differences in the ranks (Genest \emph{et al.}, 2010, eq. 1). The sample \eqn{\hat\psi} is

\deqn{\hat\psi = 1 - \frac{3}{n^2 - 1}\sum_{i=1}^n |R_i - S_i|\mbox{,}}

where \eqn{R_i} and \eqn{S_i} are the respective ranks of \eqn{X} and \eqn{Y}, \eqn{n} is sample size, and \eqn{{-}0.5 \lessapprox \hat\psi \le 1}. Genest \emph{et al.} (2010, p. 937) also remark that other normalizations have been used in the literature. The approximate \eqn{\hat\psi} lower bounds can be see by the following where \eqn{-0.5} is achieved always for odd sample sizes and rapidly approached as sample size grows:
\preformatted{
  sams    <- seq_len(25) # samples from 1 to 25 in size
  minFoot <- sapply(sams, function(i) 1 - (3/(i^2-1))*sum(abs((1:i)-(i:1))))
  plot(sams, minFoot, type="b", col="brown4")
  lines(par()$usr[1:2], rep(-0.5, 2), lty=2)
}

The sample \eqn{\hat\psi} also can be written (Genest \emph{et al.}, 2010, eq. 3) as

\deqn{\hat\psi = \frac{6}{n-1} \biggl[ \sum_i^n \mathrm{min}\biggl(\frac{R_i}{n+1},\, \frac{S_i}{n+1}\biggr)\biggr] - \frac{2n+1}{n-1}\mbox{,}}

which is not implemented here because the first sample version is modestly faster. Also, the sampling variance of \eqn{\hat\psi} under \emph{assumption of independence} between \eqn{X} and \eqn{Y} (\eqn{\mathrm{E}[\hat\psi] = 0}) is

\deqn{\mathrm{var}(\hat\psi) = \frac{2n^2 + 7}{5(n+1)(n-1)^2}\mbox{.}}


Genest \emph{et al.} (2010, p. 938) say that prior literature shows that in small samples, Spearman Footrule is less variable than the well-known \emph{Spearman Rho} (\code{\link{rhoCOP}}). For a copula having continuous partial derivatives, then as \eqn{n \rightarrow \infty}, the quantity \eqn{(\hat\psi - \psi_\mathbf{C})\sqrt{n} \sim \mathrm{Normal}(0, \mathrm{var}(\gamma_\mathbf{C}))}. Genest \emph{et al.} (2010) show variance of \eqn{\hat\psi} for the \emph{independence copula} (\eqn{\mathbf{C}(u,v) = \mathbf{\Pi}(u,v)}, \code{\link{P}}) as \eqn{\mathrm{var}(\psi_\mathbf{C}) = 2/5}. For comparison, the \emph{Gini Gamma} for independence is larger at \eqn{\mathrm{var}(\gamma_\mathbf{C}) = 2/3} (see \code{\link{giniCOP}} \bold{Note}). Genest \emph{et al.} (2010) also present additional material for estimation of the distribution \eqn{\hat\psi} variance for conditions of dependence based on copulas. In Genest \emph{et al.} (2010, p. 941), independence and two examples of dependence, \eqn{\mathrm{var}(\hat\gamma) > \mathrm{var}(\hat\psi)}, but those authors do not appear to remark on whether this inequality holds for all copula.
}
\references{
Beliakov, G., de Amo, E., Juan \enc{Fernández-Sánchez}{Fernandez-Sanchez}, J., \enc{Úbeda-Flores}{Ubeda-Flores}, M., 2022, Best-possible bounds on the set of copulas with a given value of Spearman's footrule, Fuzzy Sets and Systems, v. 428, pp. 138--152, \doi{10.1016/j.fss.2020.11.011}.

Genest, C., \enc{Nešlehová}{Neslehova}, J., and Ghorbal, N.B., 2010, Spearman's footrule and Gini's gamma---A review with complements: Journal of Nonparametric Statistics, v. 22, no. 8, pp. 937--954, \doi{10.1080/10485250903499667}.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Nelsen, R.B., Quesada-Molina, J.J., \enc{Rodríguez-Lallena}{Rodriguez-Lallena}, J.A., \enc{Úbeda-Flores}{Ubeda-Flores}, M., 2001, Distribution functions of copulas---A class of bivariate probability integral transforms: Statistics and Probability Letters, v. 54, no. 3, pp. 277--282, \doi{10.1016/S0167-7152(01)00060-8}.
}
\author{W.H. Asquith}
\seealso{\code{\link{blomCOP}}, \code{\link{giniCOP}}, \code{\link{hoefCOP}},
         \code{\link{rhoCOP}}, \code{\link{tauCOP}},  \code{\link{wolfCOP}}
}
\examples{
  footCOP(cop=PSP)                      # 0.3177662
# footCOP(cop=PSP, by.concordance=TRUE) # 0.3178025 # slower, likely giving up accuracy

\dontrun{
  set.seed(1); n <- 2000; UV <- simCOP(n=n, cop=GHcop, para=2.3, graphics=FALSE)
  footCOP(para=UV, as.sample=TRUE)                  # 0.5356089 (sample version)
  footCOP(cop=GHcop, para=2.3)                      # 0.5513380 (copula integration)
  footCOP(cop=GHcop, para=2.3, by.concordance=TRUE) # 0.5513562 (concordance function)
  # where the later issued warnings on the integration}

\dontrun{
  # Sweep through the Plackett as means to show -0.5 <= Footrule <= 1, and to show
  for(p in seq(-4,4, by=0.5)) { # that the sample version is in agreement.
    p <- round(10^p, digits=4); UV <- simCOP(10000, cop=PLcop, para=p, lwd=0.5)
    footS <- round(footCOP(as.sample=TRUE, para=UV), digits=4)
    footT <- round(footCOP(cop=PLcop,      para=p ), digits=4)
    mtext(paste0("Plackett(theta=", p, ")"), line=1)
    mtext(paste0("Spearman Footrule : theoretical=", footT, ", sample=", footS))
  } # }

\dontrun{
  set.seed(1); nsim <- 1000
  varFTunderIndepend <- function(n) {
    (2*n^2 + 7) / (5*(n+1)*(n-1)^2) # Genest et al. (2010)
  }
  ns <- c(10, 15, 20, 25, 50, 75, 100)
  plot(min(ns):max(ns), varFTunderIndepend(10:max(ns)), type="l",
       xlab="Sample size", ylab="Variance of Sample Estimator", col="salmon4")
  mtext("Sample Spearman Footrule Under Independence", col="salmon4")
  for(n in ns) {
    sFT <- vector(length=nsim)
    for(i in seq_len(nsim)) {
      uv <- simCOP(n=n, cop=P, para=2, graphics=FALSE)
      sFT[i] <- footCOP(para=uv, as.sample=TRUE)
    }
    varFT <- varFTunderIndepend(n)
    zz <- round(c(n, mean(sFT), var(sFT), varFT), digits=6)
    names(zz) <- c("n", "mean_sim", "var_sim", "var_Genest")
    print(zz)
    points(n, zz[3], cex=2, pch=21, col="salmon4", bg="salmon1")
  } # results show proper implementation and Genest et al. (2010, sec. 3) }
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{Spearman Footrule}
\concept{Footrule}

\encoding{utf8}
\name{CIRCcop}
\alias{CIRCcop}
\title{Copula of Circular Uniform Distribution}
\description{
The \emph{Circular copula} of the coordinates \eqn{(x, y)} of a point chosen at random on the \emph{unit circle} (Nelsen, 2006, p. 56) is given by
\deqn{\mathbf{C}_{\mathrm{CIRC}}(u,v) = \mathbf{M}(u,v) \mathrm{\ for\ }|u-v| > 1/2\mathrm{,}}
\deqn{\mathbf{C}_{\mathrm{CIRC}}(u,v) = \mathbf{W}(u,v) \mathrm{\ for\ }|u+v-1| > 1/2\mathrm{,\ and}}
\deqn{\mathbf{C}_{\mathrm{CIRC}}(u,v) = \frac{u+v}{2} - \frac{1}{4} \mathrm{\ otherwise\ }\mathrm{.}}

The coordinates of the unit circle are given by
\deqn{\mathbf{CIRC}(x,y) = \biggl(\frac{\mathrm{cos}\bigl(\pi(u-1)\bigr)+1}{2}, \frac{\mathrm{cos}\bigl(\pi(v-1)\bigr)+1}{2}\biggr)\mathrm{.}}
}
\usage{
CIRCcop(u, v, para=NULL, as.circ=FALSE, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{Optional parameter list argument that can contain the logical \code{as.circ} instead;}
  \item{as.circ}{A logical, if true, to trigger the transformation \eqn{u = 1 - \mathrm{acos}(2x - 1) / \pi} and \eqn{v = 1 - \mathrm{acos}(2y - 1) / \pi} to convert \eqn{(X,Y)} coordinates of a uniform unit circle to the \eqn{(U,V)} in nonexceedance probability; and}
  \item{...}{Additional arguments to pass, if ever needed.}
}
\value{
  Value(s) for the copula are returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\examples{
CIRCcop(0.5, 0.5) # 0.25 quarterway along the diagonal upward to right
CIRCcop(0.5, 1  ) # 0.50 halfway across in horizontal direction
CIRCcop(1  , 0.5) # 0.50 halfway across in  vertical  direction

\dontrun{
  nsim <- 2000
  opts <- par(xpd=NA, las=1, lend=2, no.readonly=TRUE)
  rtheta <- runif(nsim, min=0, max=2*pi) # polar coordinate simulation
  XY <- data.frame(X=cos(rtheta)/2 + 1/2, Y=sin(rtheta)/2 + 1/2)
  plot(XY, lwd=0.8, col="lightgreen", xaxs="i", yaxs="i",
           xlab="X OF UNIT CIRCLE OR NONEXCEEDANCE PROBABILITY U",
           ylab="Y OF UNIT CIRCLE OR NONEXCEEDANCE PROBABILITY V")
  UV <- simCOP(nsim, cop=CIRCcop, lwd=0.8, col="salmon3", ploton=FALSE)
  theta <- 3/4*pi+0.1 # select a point on the upper left of the circle
  x <- cos(theta)/2 + 1/2; y <- sin(theta)/2 + 1/2 # coordinates
  H <- CIRCcop(x, y, as.circ=TRUE) # 0.218169  # Pr[X <= x & Y <= y]
  points(x, y, pch=16, col="forestgreen", cex=2)
  segments(0, y, x, y, lty=2, lwd=2, col="forestgreen")
  segments(x, 0, x, y, lty=2, lwd=2, col="forestgreen")
  Hemp1 <- sum(XY$X <= x & XY$Y <= y) / nrow(XY) # about 0.22 as expected
  u <- 1-acos(2*x-1)/pi; v <- 1-acos(2*y-1)/pi
  segments(0, v, u, v, lty=2, lwd=2, col="salmon3")
  segments(u, 0, u, v, lty=2, lwd=2, col="salmon3")
  points(u, v, pch=16, cex=2,        col="salmon3")
  arrows(x, y, u, v, code=2, lwd=2, angle=15) # arrow points from (X,Y) coordinate
  # specified by angle theta in radians on the unit circle to the corresponding
  # coordinate in (U,V) domain of uniform circular distribution copula
  Hemp2 <- sum(UV$U <= u & UV$V <= v) / nrow(UV) # about 0.22 as expected
  # Hemp1 and Hemp2 are about equal to each other and converge as nsim
  # gets very large, but the origin of the simulations to get to each
  # are different: (1) one in polar coordinates and (2) by copula.
  # Now, draw the level curve for the empirical Hs and as nsim gets large the two
  # lines will increasingly plot on top of each other.
  lshemp1 <- level.setCOP(cop=CIRCcop, getlevel=Hemp1, lines=TRUE, col="blue", lwd=2)
  lshemp2 <- level.setCOP(cop=CIRCcop, getlevel=Hemp2, lines=TRUE, col="blue", lwd=2)
  txt <- paste0("level curves for Pr[X <= x & Y <= y] and\n",
                "level curves for Pr[U <= u & V <= v],\n",
                "which equal each other as nsim gets large")
  text(0.52, 0.52, txt, srt=-46, col="blue"); par(opts) # }

\dontrun{
  # Nelsen (2007, ex. 3.2, p. 57) # Singular bivariate distribution with
  # standard normal margins that is not bivariate normal.
  U <- runif(500); V <- simCOPmicro(U, cop=CIRCcop)
  X  <- qnorm(U, mean=0, sd=1);    Y <- qnorm(V, mean=0, sd=1)
  plot(X,Y, main="Nelsen (2007, ex. 3.2, p. 57)", xlim=c(-4,4), ylim=c(-4,4),
            lwd=0.8, col="turquoise3")
  rug(X, side=1, col="salmon3", tcl=0.5)
  rug(Y, side=2, col="salmon3", tcl=0.5) #}

\dontrun{
  DX <- c(5, 5, -5, -5); DY <- c(5, 5, -5, -5); D <- 6; R <- D/2
  xylim <- c(-10, 10)
  plot(DX, DY, type="n", xlim=xylim, ylim=xylim, xlab="X", ylab="Y", las=1,
       xaxs="i", yaxs="i")
  for(i in seq_len(length(DX))) lines(rep(DX[i], 2), xylim, lwd=2, col="seagreen")
  for(i in seq_len(length(DY))) lines(xylim, rep(DY[i], 2), lwd=2, col="seagreen")
  for(i in seq_len(length(DX))) {
    for(j in seq_len(length(DY))) {
      UV <- simCOP(n=30, cop=CIRCcop, pch=16, col="darkgreen", cex=0.5, graphics=FALSE)
      points(UV[,1]-0.5, UV[,2]-0.5,  pch=16, col="darkgreen", cex=0.5)
      XY <- data.frame(X=DX[i]+sign(DX[i])*D*(cos(pi*(UV$U-1))+1)/2-sign(DX[i])*R,
                       Y=DY[j]+sign(DY[j])*D*(cos(pi*(UV$V-1))+1)/2-sign(DY[j])*R)
      points(XY, lwd=0.8, col="darkgreen")
    }
    lines(rep(DX[i]+R, 2), xylim, lty=2, col="seagreen")
    lines(rep(DX[i]-R, 2), xylim, lty=2, col="seagreen")
    lines(xylim, rep(DY[i]+R, 2), lty=2, col="seagreen")
    lines(xylim, rep(DY[i]-R, 2), lty=2, col="seagreen")
  } #}

\dontrun{
  para <- list(cop1=CIRCcop, para1=NULL, cop2=W, para2=NULL, alpha=0.8, beta=0.8)
  UV <- simCOP(n=2000, col="salmon3", cop=composite2COP, para=para)
  XY <- data.frame(X=(cos(pi*(UV$U-1))+1)/2, Y=(cos(pi*(UV$V-1))+1)/2)
  plot(XY, type="n", xlab=paste0("X OF CIRCULAR UNIFORM DISTRIBUTION OR\n",
                                 "NONEXCEEDANCE PROBABILITY OF U"),
                     ylab=paste0("Y OF CIRCULAR UNIFORM DISTRIBUTION OR\n",
                                 "NONEXCEEDANCE PROBABILITY OF V"))
  JK <- data.frame(U=1 - acos(2*XY$X - 1)/pi, V=1 - acos(2*XY$Y - 1)/pi)
  segments(x0=UV$U, y0=UV$V, x1=XY$X, y1=XY$Y, col="lightgreen", lwd=0.8)
  points(XY, lwd=0.8, col="darkgreen")
  points(JK, pch=16,  col="salmon3", cex=0.5)

  t <- seq(0.001, 0.999, by=0.001)
  t <- diagCOPatf(t, cop=composite2COP, para=para)
  AB <- data.frame(X=(cos(pi*(t-1))+1)/2, Y=(cos(pi*(t-1))+1)/2)
  lines(AB, lwd=4, col="seagreen") #}
}
\keyword{copula (formulas)}
\keyword{copula}
\keyword{cicular copula}
\keyword{Nelsen (2006) Examples and Exercises}

\encoding{utf8}
\name{qua.regressCOP}
\alias{qua.regressCOP}
\title{Perform Quantile Regression using a Copula by Numerical Derivative Method for V with respect to U}
\description{
Perform \emph{quantile regression} (Nelsen, 2006, pp. 217--218) using a copula by numerical derivatives of the copula (\code{\link{derCOPinv}}). If \eqn{X} and \eqn{Y} are random variables having quantile functions \eqn{x(F)} and \eqn{y(G)} and letting \eqn{y=\tilde{y}(x)} denote a solution to \eqn{\mathrm{Pr}[Y \le y\mid X = x] = F}, where \eqn{F} is a nonexceedance probability. Then the curve \eqn{y=\tilde{y}(x)} is the quantile regression curve of \eqn{V} or \eqn{Y} with respect to \eqn{U} or \eqn{X}, respectively. If \eqn{F=1/2}, then \emph{median regression} is performed (\code{\link{med.regressCOP}}). Using copulas, the quantile regression is expressed as
\deqn{\mathrm{Pr}[Y \le y\mid X = x] = \mathrm{Pr}[V \le G(y) \mid U = F(x)] = \mathrm{Pr}[V \le v\mid U = v] = \frac{\delta \mathbf{C}(u,v)}{\delta u}\mbox{,}}
where \eqn{v = G(y)} and \eqn{u = F(x)}. The general algorithm is
\enumerate{
\item Set \eqn{\delta \mathbf{C}(u,v)/\delta u = F},

\item Solve the regression curve \eqn{v = \tilde{v}(u)} (provided by \code{\link{derCOPinv}}), and

\item Replace \eqn{u} by \eqn{x(u)} and \eqn{v} by \eqn{y(v)}.
}

The last step is optional as step two produces the regression in probability space, which might be desired, and step 3 actually transforms the probability regressions into the quantiles of the respective random variables.
}
\usage{
qua.regressCOP(f=0.5, u=seq(0.01,0.99, by=0.01), cop=NULL, para=NULL, ...)
}
\arguments{
  \item{f}{A single value of nonexceedance probability \eqn{F} to perform regression at and defaults to median regression \eqn{F=1/2};}
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  An \R \code{data.frame} of the regressed probabilities of \eqn{V} and provided \eqn{U=u} values is returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{med.regressCOP}}, \code{\link{derCOPinv}}, \code{\link{qua.regressCOP.draw}}}
\examples{
\dontrun{
# Use a positively associated Plackett copula and perform quantile regression
theta <- 10
R <- qua.regressCOP(cop=PLACKETTcop, para=theta) # 50th percentile regression

plot(R$U,R$V, type="l", lwd=6, xlim=c(0,1), ylim=c(0,1), col=8)
lines(R$U,(1+(theta-1)*R$U)/(theta+1), col=4, lwd=1) # theoretical for Plackett, see
#                                                             (Nelsen, 2006, p. 218)
R <- qua.regressCOP(f=0.90, cop=PLACKETTcop, para=theta) # 90th-percentile regression
lines(R$U,R$V, col=2, lwd=2)
R <- qua.regressCOP(f=0.10, cop=PLACKETTcop, para=theta) # 10th-percentile regression
lines(R$U,R$V, col=3, lty=2)
mtext("Quantile Regression V wrt U for Plackett copula")#}

\dontrun{
# Use a composite copula with two Placketts with compositing parameters alpha and beta.
para <- list(cop1=PLACKETTcop, cop2=PLACKETTcop,
             para1=0.04, para2=5, alpha=0.9, beta=0.6)
plot(c(0,1),c(0,1), type="n", lwd=3,
     xlab="U, NONEXCEEDANCE PROBABILITY", ylab="V, NONEXCEEDANCE PROBABILITY")
# Draw the regression of V on U and then U on V (wrtV=TRUE)
qua.regressCOP.draw(cop=composite2COP, para=para, ploton=FALSE)
qua.regressCOP.draw(cop=composite2COP, para=para, wrtV=TRUE, lty=2, ploton=FALSE)
mtext("Composition of Two Plackett Copulas and Quantile Regression")#}

\dontrun{
# Use a composite copula with two Placketts with compositing parameters alpha and beta.
para <- list(cop1=PLACKETTcop,  cop2=PLACKETTcop,
             para1=0.34, para2=50, alpha=0.63, beta=0.47)
D <- simCOP(n=3000, cop=composite2COP, para=para, cex=0.5)
qua.regressCOP.draw(cop=composite2COP, para=para, ploton=FALSE)
qua.regressCOP.draw(cop=composite2COP, para=para, wrtV=TRUE, lty=2, ploton=FALSE)
level.curvesCOP(cop=composite2COP, para=para, ploton=FALSE)
mtext("Composition of Two Plackett Copulas, Level Curves, Quantile Regression")

para <- list(cop1=PLACKETTcop,  cop2=PLACKETTcop, # Note the singularity
             para1=0, para2=500, alpha=0.63, beta=0.47)
D <- simCOP(n=3000, cop=composite2COP, para=para, cex=0.5)
qua.regressCOP.draw(cop=composite2COP, para=para, ploton=FALSE)
qua.regressCOP.draw(cop=composite2COP, para=para, wrtV=TRUE, lty=2, ploton=FALSE)
level.curvesCOP(cop=composite2COP, para=para, ploton=FALSE)
mtext("Composition of Two Plackett Copulas, Level Curves, Quantile Regression")

pdf("quantile_regression_test.pdf")
for(i in 1:10) {
  para <- list(cop1=PLACKETTcop, cop2=PLACKETTcop, alpha=runif(1), beta=runif(1),
               para1=10^runif(1,min=-4,max=0), para2=10^runif(1,min=0,max=4))
  txts <- c("Alpha=",    round(para$alpha,    digits=4),
            "; Beta=",   round(para$beta,     digits=4),
            "; Theta1=", round(para$para1[1], digits=5),
            "; Theta2=", round(para$para2[1], digits=2))

  D <- simCOP(n=3000, cop=composite2COP, para=para, cex=0.5, col=3)
  mtext(paste(txts, collapse=""))
  qua.regressCOP.draw(f=c(seq(0.05, 0.95, by=0.05)),
                      cop=composite2COP, para=para, ploton=FALSE)
  qua.regressCOP.draw(f=c(seq(0.05, 0.95, by=0.05)),
                      cop=composite2COP, para=para, wrtV=TRUE, ploton=FALSE)
  level.curvesCOP(cop=composite2COP, para=para, ploton=FALSE)
}
dev.off() # done}
}
\keyword{quantile regression}

\encoding{utf8}
\name{GLcop}
\alias{GLcop}
\alias{GLEVcop}
\alias{GLPMcop}
\alias{JOcopBB4}
\title{The Galambos Extreme Value Copula (with Gamma Power Mixture [Joe/BB4] and Lower Extreme Value Limit)}
\description{
The \emph{Galambos copula} (Joe, 2014, p. 174) is
\deqn{\mathbf{C}_{\Theta}(u,v) = \mathbf{GL}(u,v) = uv\,\mathrm{exp}\bigl[\bigl(x^{-\Theta} + y^{-\Theta}\bigr)^{-1/\Theta}\bigr]\mbox{,}}
where \eqn{\Theta \in [0, \infty)}, \eqn{x = -\log u}, and \eqn{y = -\log v}. As \eqn{\Theta \rightarrow 0^{+}}, the copula limits to \emph{independence} (\eqn{\mathbf{\Pi}}; \code{\link{P}}) and as  \eqn{\Theta \rightarrow \infty}, the copula limits to perfect association (\eqn{\mathbf{M}}; \code{\link{M}}). The Galambos is a \emph{bivariate extreme value copula} (\eqn{BEV}), and parameter estimation for \eqn{\Theta} requires numerical methods.

There are two other genetically related forms. Joe (2014, p. 197) describes an extension of the Galambos copula as a \emph{Galambos gamma power mixture} (GLPM), which is Joe's \emph{BB4 copula}, with the following form
\deqn{\mathbf{C}_{\Theta,\delta}(u,v) = \mathbf{GLPM}(u,v) =
\biggl(x + y - 1 - \bigl[(x - 1)^{-\delta} + (y - 1)^{-\delta} \bigr]^{-1/\delta} \biggr)^{-1/\Theta}\mbox{,}}
where \eqn{x = u^{-\Theta}}, \eqn{y = v^{-\Theta}}, and \eqn{\Theta \ge 0, \delta \ge 0}. (Joe shows \eqn{\delta > 0}, but zero itself seems to work without numerical problems in practical application.) As \eqn{\delta \rightarrow 0^{+}}, the \dQuote{MTCJ family} (Mardia--Takahasi--Cook--Johnson) results (implemented internally with \eqn{\Theta} as the incoming parameter). As \eqn{\Theta \rightarrow 0^{+}}, the Galambos above results with \eqn{\delta} as the incoming parameter.

This Galambos extension in turn has a \emph{lower extreme value limit form} that leads to a \emph{min-stable bivariate exponential} having \emph{Pickand dependence function} of
\deqn{A(x,y; \Theta, \delta) = x + y - \bigl[x^{-\Theta} + y^{-\Theta} - (x^{\Theta\delta} + y^{\Theta\delta})^{-1/\delta} \bigr]^{-1/\Theta}\mbox{,}}
where this third copula is
\deqn{\mathbf{C}^{LEV}_{\Theta,\delta}(u,v) = \mathbf{GLEV}(u,v) =
\mathrm{exp}[-A(-\log u, -\log v; \Theta, \delta)]\mbox{,}}
for \eqn{\Theta \ge 0, \delta \ge 0} and is known as the \emph{two-parameter Galambos}. (Joe shows \eqn{\delta > 0}, but \eqn{\delta = 0} itself seems to work without numerical problems in practical application.)
}
\usage{
GLcop(   u, v, para=NULL, ...)
GLEVcop( u, v, para=NULL, ...)
GLPMcop( u, v, para=NULL, ...) # inserts third parameter automatically
JOcopBB4(u, v, para=NULL, ...) # inserts third parameter automatically
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{To trigger \eqn{\mathbf{GL}(u,v)}, a vector (single element) of \eqn{\Theta}, to trigger \eqn{\mathbf{GLEV}(u,v)}, a two element vector of \eqn{\Theta} and \eqn{\delta} and alias is \code{GLEVcop}, and to trigger \eqn{\mathbf{GLPM}(u,v)}, a three element vector of \eqn{\Theta}, \eqn{\delta}, and any number (the presence of the third entry alone is the triggering mechanism) though aliases \code{GLPM} or \code{JOcopBB4} will insert the third parameter automatically for convenience; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\note{
Joe (2014, p. 198) shows \eqn{\mathbf{GLEV}(u,v; \Theta, \delta)} as a two-parameter Galambos, but its use within the text seemingly is not otherwise obvious. However, testing of the implementation here seems to show that this copula is really not broader in form than \eqn{\mathbf{GL}(u,v; \alpha)}. The \eqn{\alpha} can always(?) be chosen to mimic the \eqn{\{\Theta, \delta\}}. This assertion can be tested from a semi-independent direction. First, define an alternative style of one-parameter Galambos:
\preformatted{
  GL1cop <- function(u,v, para=NULL, ...) {
     GL1pA <- function(x,y,t) { # Pickend dependence func form 1p Galambos
        x + y - (x^-t + y^-t)^(-1/t)
     }
     if(length(u) == 1) { u <- rep(u, length(v)) } else
     if(length(v) == 1) { v <- rep(v, length(u)) }
     exp(-GL1pA(-log(u), -log(v), para[1]))
  }
}
Second, redefine the two-parameter Galambos:
\preformatted{
  GL2cop <- function(u,v, para=NULL, ...) {
     GL2pA <- function(x,y,t,d) { # Pickend dependence func form 2p Galambos
        x + y - (x^-t + y^-t - (x^(t*d) + y^(t*d))^(-1/d))^(-1/t)
     }
     if(length(u) == 1) { u <- rep(u, length(v)) } else
     if(length(v) == 1) { v <- rep(v, length(u)) }
     exp(-GL2pA(-log(u), -log(v), para[1], para[2]))
  }
}
Next, we can combine the Pickend dependence functions into an objective function. This objective function will permit the computation of the \eqn{\alpha} given a pair \eqn{\{\Theta, \delta\}}.
\preformatted{
  objfunc <- function(a,t=NA,d=NA, x=0.7, y=0.7) {
     lhs <- (x^-t + y^-t - (x^(t*d) + y^(t*d))^(-1/d))^(-1/t)
     rhs <- (x^-a + y^-a)^(-1/a); return(rhs - lhs) # to be uniroot'ed
  }
}
A demonstration can now be made:
\preformatted{
  t <- 0.6; d <- 4; lohi <- c(0,100)
  set.seed(3); UV <- simCOP(3000, cop=GL2cop, para=c(t,d), pch=16,col=3,cex=0.5)
  a <- uniroot(objfunc, interval=lohi, t=t, d=d)$root
  set.seed(3); UV <- simCOP(3000, cop=GL1cop, para=a, lwd=0.5, ploton=FALSE)
}
The graphic so produced shows almost perfect overlap in the simulated values. To date, the author has not really found that the two parameters can be chosen such that the one-parameter version can not attain. Extensive numerical experiments using simulated parameter combinations through the use of various copula metrics (tail dependencies, L-comoments, etc) have not found material differences. \bold{Has the author of this package missed something?}
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{M}}, \code{\link{P}}, \code{\link{GHcop}}, \code{\link{HRcop}}, \code{\link{tEVcop}}
}
\examples{
# Theta = pi for GLcop and recovery through Blomqvist Beta     (Joe, 2014, p. 175)
log(2)/(log(log(2)/log(1+blomCOP(cop=GLcop, para=pi))))

# Theta = 2 and delta = 3 for the GLPM form and Blomqvist Beta (Joe, 2014, p. 197)
t <- 2; Btheo <- blomCOP(GLPMcop, para=c(t,3))
Bform <- (2^(t+1) - 1 - taildepCOP(GLPMcop, para=c(t,3))$lambdaU*(2^t -1))^(-1/t)
print(c(Btheo, 4*Bform-1)) # [1] 0.8611903 0.8611900

\dontrun{
  # See the Note section but check Blomqvist Beta here:
  blomCOP(cop=GLcop, para=c(6.043619))  # 0.8552863 (2p version)
  blomCOP(cop=GLcop, para=c(5.6, 0.3))  # 0.8552863 (1p version) }
}
\keyword{Galambos copula}
\keyword{Galambos copula (lower extreme value form)}
\keyword{copula (formulas)}
\keyword{copula}
\concept{Galambos extreme value copula}
\keyword{Joe (2014) Examples and Exercises}
\keyword{copula (extreme value)}
\keyword{extreme value copula}
\keyword{Joe/BB4 copula}
\concept{Joe/BB4 copula}

\encoding{utf8}
\name{med.regressCOP}
\alias{med.regressCOP}
\title{Perform Median Regression using a Copula by Numerical Derivative Method for V with respect to U}
\description{
Perform \emph{median regression} (Nelsen, 2006, pp. 217--218) of a copula by inversion of numerical derivatives of the copula (\code{\link{derCOPinv}}). The documentation for \code{\link{qua.regressCOP}} provides mathematical details. The \code{\link{qua.regressCOP.draw}} supports so-called \emph{quantile regression} along various probability levels (see \bold{Examples}).
}
\usage{
med.regressCOP(u=seq(0.01,0.99, by=0.01), cop=NULL, para=NULL, level=NA, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{level}{The level of the prediction interval to compute. For example, \code{level=0.95} will compute the 95-percent prediction interval as will \code{level=0.05} because internally a reflection check is made; and}
  \item{...}{Additional arguments to pass such \code{\link{qua.regressCOP}} and \code{\link{derCOPinv}} that are called in succession.}
}
\value{
  An \R \code{data.frame} of the median regressed probabilities of \eqn{V} and provided \eqn{U} values is returned. Note: if \code{level} is used, then the column ordering of the returned \code{data.frame} changes---please access the columns by the named idiom. The lower- and upper-prediction interval is contained in the columns respectively titled \code{Vlwr} and \code{Vupr} to mimic nomenclature somewhat of function \code{predict.lm()} in \R.
}
\note{
An extended demonstration is needed concerning prediction intervals by median regression and a comparison to well-known linear regression. This also affords and opportunity to have \pkg{copBasic} interact with the \pkg{copula} package to gain access to the \emph{Gaussian copula} and a \emph{maximum pseudo-likelihood} estimation of the parameter.

First, a function \code{NORMcop()} is defined to form the interconnect between the two packages. It is critically important that the user recognize that the so-called \code{copula} object as built by the \pkg{copula} package is treated as the canonical \code{para} argument in \pkg{copBasic} calls herein.
\preformatted{
  "NORMcop" <-  # pCoupla() from package copula is analogous to COP()
  function(u,v, para=NULL, ...) {
    if(length(u) == 1) u <- rep(u, length(v)) # see asCOP() for reasoning of
    if(length(v) == 1) v <- rep(v, length(u)) # this "vectorization" hack
    return(copula::pCopula(matrix(c(u,v), ncol=2), para))
  }
}
The parameter \eqn{\Theta \in [-1, 1]} (\emph{Pearson R}) and \eqn{\rho_\mathbf{C}(\Theta)} (\emph{Spearman Rho}, \code{\link{rhoCOP}}) and \eqn{\tau_\mathbf{C}(\Theta)} (\emph{Kendall Tau}, \code{\link{tauCOP}}) are according to Salvadori \emph{et al.} (2007, p. 255) the values
\deqn{\rho_\mathbf{C}(\Theta) = \frac{2}{\pi}\,\mathrm{arcsin}(\Theta)}
and
\deqn{\tau_\mathbf{C}(\Theta) = \frac{6}{\pi}\,\mathrm{arcsin}(\Theta/2)\mbox{.}}

Second, a bivariate \emph{Gaussian copula} is defined with a parameter \eqn{\Theta = 0.7} (thus \eqn{\rho_\mathbf{C} = 0.6829105}, \code{rhoCOP(NORMcop, norm.cop)}) and then \eqn{n=255} samples simulated from it. These are then cast into standard normal variates to mimic the idea of bivariate data in nonprobability units and facilitation regression comparison.
\preformatted{
  norm.cop <- copula::normalCopula(c(0.7), dim = 2) # define a Gaussian copula
  UVs      <- copula::rCopula(255, norm.cop) # draw 255 samples
  UVs      <- as.data.frame(UVs); X <- qnorm(UVs[,1]); Y <- qnorm(UVs[,2])
}

Third, the \emph{Weibull plotting positions} from the \code{pp()} function of package \pkg{lmomco} are used to estimate the empirical probabilities of the data in \code{UV} that are casted into an \R \code{matrix} because the \pkg{copula} package expects the data as a \code{matrix} for the default parameter estimation. The code is completed by the specification of the fitted \emph{Gaussian copula} in \code{fnorm.cop}.
\preformatted{
  UV <- as.matrix(data.frame(U=lmomco::pp(X, sort=FALSE),
                             V=lmomco::pp(Y, sort=FALSE)))
  para <- copula::fitCopula(copula::normalCopula(dim=2), UV)
  para <- summary(para)$coefficients[1] # maximum pseudo-likelihood est.
  fnorm.cop <- copula::normalCopula(para, dim=2)
}

Fourth, ordinary-least-squares (OLS) linear regression for \eqn{Y\mid X} and \eqn{X\mid Y} is computed, and the results plotted on top of the data points. The 2/3rd-prediction limits are computed by \code{predict.lm()} and also shown.
\preformatted{
  # Classical linear regressions from two perspectives.
  LMyx <- lm(Y~X);       LMxy <- lm(X~Y)
  YonX <- summary(LMyx); XonY <- summary(LMyx)

  QUorV <- seq(-3,3, by=0.05) # vector for graphical operations
  plot(X, Y, col=8, pch=21)
  lines(QUorV, YonX$coefficients[1]+YonX$coefficients[2]*QUorV, col=2, lwd=4)
  tmp <- predict.lm(LMyx, list(X=X), level=2/3, interval="prediction")
  lines(X, tmp[,2], col=2); lines(X, tmp[,3], col="red"  )

  lines(XonY$coefficients[1]+XonY$coefficients[2]*QUorV, QUorV, col=3, lwd=4)
  tmp <- predict.lm(LMxy, list(Y=Y), level=2/3, interval="prediction")
  lines(tmp[,2], Y, col="green"); lines(tmp[,3], Y, col="green")
}

Finally, the demonstration ends with plotting of the median regression for the Gaussian copula and drawing the regression lines. The two median regression lines are nearly coincident with the OLS regression lines as anticipated with a reasonably large sample size albeit maximum pseudo-likelihood was used to estimate the copula parameter. The mean of a uniform distributed variable given say \eqn{U = u} (horizontal axis) is 1/2, which coincides with the median. The median regression lines thus are coincident with the OLS lines even though OLS and real-space (native units of \eqn{X} and \eqn{Y}) were not used for their computation. The 2/3-prediction intervals also are plotted for comparison.
\preformatted{
  UorV    <- c(0.001, seq(.02,0.98, by=.02), 0.999)
  MEDreg  <- med.regressCOP( u=UorV, level=2/3, cop=NORMcop, para=fnorm.cop)
  MEDreg2 <- med.regressCOP2(v=UorV, level=2/3, cop=NORMcop, para=fnorm.cop)
  lines(qnorm(UorV),         qnorm(MEDreg$V),    col="blue",    lty=2)
  lines(qnorm(UorV),         qnorm(MEDreg$Vlwr), col="blue",    lty=2)
  lines(qnorm(UorV),         qnorm(MEDreg$Vupr), col="blue",    lty=2)
  lines(qnorm(MEDreg2$U),    qnorm(UorV),        col="magenta", lty=2)
  lines(qnorm(MEDreg2$Ulwr), qnorm(UorV),        col="magenta", lty=2)
  lines(qnorm(MEDreg2$Uupr), qnorm(UorV),        col="magenta", lty=2)
}

A curious aside (Joe, 2014, p. 164) about the \emph{Gaussian copula} is that \emph{Blomqvist Beta} (\code{\link{blomCOP}}) is equal to \emph{Kendall Tau} (\code{\link{tauCOP}}), which can be checked by
\preformatted{
  blomCOP(cop=NORMcop, para=norm.cop) # 0.4936334
  tauCOP( cop=NORMcop, para=norm.cop) # 0.493633
}
and obviously the \eqn{\beta_\mathbf{C} = \tau_\mathbf{C}} are numerically the same.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in Nature---An approach using copulas: Springer, 289 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{med.regressCOP2}}, \code{\link{qua.regressCOP}}, \code{\link{qua.regressCOP.draw}}
}
\examples{
\dontrun{
# INDEPENDENCE YIELDS STRAIGHT LINES, RED IS THE MEDIAN REGRESSION
FF <- seq(0.1, 0.9, by=0.1)
plot(c(0,1),c(0,1), type="n", lwd=3,
     xlab="U, NONEXCEEDANCE PROBABILITY", ylab="V, NONEXCEEDANCE PROBABILITY")
# Draw the regression of V on U and then U on V (wrtV=TRUE)
qua.regressCOP.draw(f=FF, cop=P, para=NA, ploton=FALSE)
qua.regressCOP.draw(f=FF, cop=P, para=NA, ploton=FALSE, wrtV=TRUE, lty=2)#}

\dontrun{
# NEGATIVE PLACKETT THETA YIELDS CURVES DOWN TO RIGHT, RED IS THE MEDIAN REGRESSION
theta <- 0.5; FF <- seq(0.1, 0.9, by=0.1)
plot(c(0,1),c(0,1), type="n", lwd=3,
     xlab="U, NONEXCEEDANCE PROBABILITY", ylab="V, NONEXCEEDANCE PROBABILITY")
# Draw the regression of V on U and then U on V (wrtV=TRUE)
qua.regressCOP.draw(f=FF, cop=PLACKETTcop, ploton=FALSE, para=theta)
qua.regressCOP.draw(f=FF, cop=PLACKETTcop, ploton=FALSE, para=theta, wrtV=TRUE, lty=2)#}
}
\keyword{median regression}
\keyword{quantile regression}
\concept{quantile regression}
\keyword{Normal copula}
\keyword{Gaussian copula}
\keyword{API to the copula package}
\keyword{package copula (comparison to)}

\encoding{utf8}
\name{blomCOPss}
\alias{blomCOPss}
\title{Blomqvist (Schmid--Schmidt) Betas of a Copula}
\description{
Compute the \emph{Blomqvist (Schmid--Schmidt) Betas} \eqn{\beta^\diamond_\mathbf{C}} (Schmid and Schmidt, 2007) defined for arbitrary dimension \eqn{d} of a copula \eqn{\mathbf{C}_(u_1, \cdots, u_d; \Theta)} (\code{\link{COP}}) for parameters \eqn{\Theta}. The copula survival function is \eqn{\overline{\mathbf{C}}(u_1, \cdots, u_d; \Theta)} (\code{\link{surfuncCOP}}). The Beta, though the \pkg{copBasic} package is built around bivariate copula only, is defined as
\deqn{\beta^\diamond_\mathbf{C} = h_d(\mathbf{u}, \mathbf{v})\bigl[
\bigl(\mathbf{C}(\mathbf{u}) + \overline{\mathbf{C}}(\mathbf{v})\bigr) - g_d(\mathbf{u}, \mathbf{v})
\bigr]\mbox{,}}
where \eqn{h_d} and \eqn{g_d} are norming constants defined below. The superscript \eqn{\diamond} (diamond) is chosen for \pkg{copBasic} because of the alliteration to \dQuote{dimension.} The bold face font for \eqn{\mathbf{u}} and \eqn{\mathbf{v}} shows these arguments as vectors of length \eqn{d} reflecting \dQuote{cutting points} on nonexceedance probabilities in each of the dimensions. The \eqn{\mathbf{u}} functions as the arguments \eqn{(u,v)} pair used in copula of this package and represents the first cutting point for a \eqn{\mathrm{Pr}[U \le u, V \le v] = \mathbf{C}(u,v)}, and \eqn{\mathbf{v}} functions as the arguments \eqn{u,v} pair for this package and represents the second cutting point for a \eqn{\mathrm{Pr}[U > u, V > v] = 1 - u - v + \mathbf{C}(u,v) = \overline{\mathbf{C}}(u,v)}. This notation of vectored (bold face) and nonvectored \dQuote{u} and \dQuote{v} is a little obtuse but as the properties of \eqn{\beta^\diamond_\mathbf{C}} are summarized clarity for the reader is anticipated. In short, the \eqn{\mathbf{u}} will reference the coordinate pairs in the lower right quadrant and the \eqn{\mathbf{v}} will reference the coordinate pairs in the upper right quadrant.

The norming constant \eqn{h_d} is defined as
\deqn{h_d(\mathbf{u}, \mathbf{v}) = \frac{1}{\bigl( \mathrm{min}(u_1,   \cdots,   u_d) + \mathrm{min}(1-v_1, \cdots, 1-v_d) - g_d(\mathbf{u}, \mathbf{v})\bigr)}\mbox{,}}
and \eqn{g_d} is defined as
\deqn{g_d(\mathbf{u}, \mathbf{v}) = \prod^d_{i=1}u_i + \prod^d_{i=1}(1-v_i)\mbox{,}}
where the cutting points \eqn{\mathbf{u}} and \eqn{\mathbf{v}} are in a domain \eqn{D : \{(\mathbf{u}, \mathbf{v})\} \in [0,1]^{2d}} given \eqn{\mathbf{u} \le \mathbf{v}} and \eqn{\mathbf{u} > 0} or \eqn{\mathbf{v} < \mathbf{1}}. The reader must careful remember that these \eqn{\mathbf{u}} and \eqn{\mathbf{v}} are vectors of probabilities.

The norming constants provide for \eqn{-1 \le \beta^\diamond_\mathbf{C} \le +1}. Using the function argument defaults for \eqn{d=2} dimensions \eqn{\mathbf{u} = (1,1)/2} for \code{uu} and \eqn{\mathbf{v} = (1,1)/2} for \code{vv}, results in (1) \eqn{\beta^\diamond_\mathbf{C} = 1} if \eqn{\mathbf{C} = \mathbf{M}} \emph{comonotonicity copula} (\code{\link{M}}) (\code{blomCOPss(cop=M) == 1}), (2) \eqn{\beta^\diamond_\mathbf{C} = 0} if \eqn{\mathbf{C} = \mathbf{P}} \emph{independence copula} (\code{\link{P}}) (\code{blomCOPss(cop=P) == 0}), and (3) if \eqn{\mathbf{C} = \mathbf{W}} \emph{countermonotonicity copula} (\code{\link{W}})\eqn{\beta^\diamond_\mathbf{C} = 1} (\code{blomCOPss(cop=W) == -1}).

Schmid and Schmidt (2007) list three important cases extending the \eqn{\mathbf{M}} and \eqn{\mathbf{P}} examples. First, \eqn{\beta^\diamond_\mathbf{C}(\mathbf{1/2}, \mathbf{1/2}) = \beta_\mathbf{C}(1/2, 1/2)}, which is \emph{Blomqvist Beta} (\eqn{\beta_\mathbf{C}(1/2, 1/2)}) (\code{\link{blomCOP}}) and measures overall dependence.

Second, \eqn{\beta^\diamond_\mathbf{C}(\mathbf{u}, \mathbf{v})} with \eqn{\mathbf{u} < 1/2 < \mathbf{v}}, which measures dependence in the tail regions. (Note, the author of \pkg{copBasic} thinks \dQuote{regions} as a plural is need in the previous sentence; Schmid and Schmidt (2007) use the singular \dQuote{region.} This is potentially important as seemingly simultaneous tail dependency in the lower and upper perspectives would be provided. More discussion is provided in \bold{Examples}.)

Third and presumably very important in practical applications, \eqn{\mathrm{lim}_{p\downarrow 0}\, \beta^\diamond_\mathbf{C}(\mathbf{p}, \mathbf{1}) = \lambda^L_{\beta^\diamond_\mathbf{C}}} for \eqn{\mathbf{p} = \mathbf{u} = (p,\cdots,p)} measures lower-tail dependence. This measure is equal to the \emph{lower-tail dependence parameter} \eqn{\lambda^L_\mathbf{C} = \lambda^L_{\beta^\diamond_\mathbf{C}}} without some of the computational nuances required as  \eqn{\lambda^L_\mathbf{C}} is defined at \code{\link{taildepCOP}}.

Schmid and Schmidt (2007) do not list how the \emph{upper-tail dependence parameter} \eqn{\lambda^U_\mathbf{C}} could be computed in terms of \eqn{\beta^\diamond_\mathbf{C}}. The expression for study of the upper-tail dependency is \eqn{\lambda^U_{\beta^\diamond_\mathbf{C}} = \beta^\diamond_\mathbf{C}(\mathbf{0}, \mathbf{p})} for \eqn{\mathbf{p} = \mathbf{v} = (p,\cdots,p)} as \eqn{p \rightarrow 0^+}, and \eqn{\lambda^U_\mathbf{C} = \lambda^U_{\beta^\diamond_\mathbf{C}}} without some of the computational nuances required as \eqn{\lambda^U_\mathbf{C}} is defined at \code{\link{taildepCOP}}. These tail dependencies are computed and compared in the \bold{Examples} and confirmation of this function being used to estimate both tail-dependency parameters is confirmed.
}
\usage{
blomCOPss(cop=NULL, para=NULL, uu=rep(0.5, 2), vv=rep(0.5, 2), trap.nan=TRUE,
          as.sample=FALSE, ctype=c("weibull", "hazen", "1/n",
                                   "bernstein", "checkerboard"), ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{uu}{The vector for \eqn{\mathbf{u}} and the defaults with \code{vv} as such for same operation as \code{\link{blomCOP}} (\eqn{\beta^\diamond_\mathbf{C}(\mathbf{1/2}, \mathbf{1/2})});}
  \item{vv}{The vector for \eqn{\mathbf{v}} and the defaults with \code{uu} as such for same operation as \code{\link{blomCOP}} (\eqn{\beta^\diamond_\mathbf{C}(\mathbf{1/2}, \mathbf{1/2})});}
  \item{trap.nan}{A logical to trigger 0 if \eqn{(0,0)} is \code{NaN} or if \eqn{(1,1)} is \code{NaN}. This feature is present on a package-specific purpose because the \code{\link{PSP}} copula deliberately retains edge \code{NaN} as a stress case;}
  \item{as.sample}{A logical controlling whether an optional \R \code{data.frame} in \code{para} is used to compute the  \eqn{\hat\beta^\diamond_\mathbf{C}} at which point the \code{ctype} argument will be passed to \code{\link{EMPIRcop}};}
    \item{ctype}{Argument of the same as \code{\link{EMPIRcop}}; and}
  \item{...}{Additional arguments to pass to the copula.}
}
\value{
  The \eqn{\beta^\diamond_\mathbf{C}} is returned.
}
\note{
Sample estimation of the \eqn{\beta^\diamond_\mathbf{C}} is possible. The \code{as.sample} triggers internally a call to the \emph{empirical copula} (\eqn{\mathbf{C}_n}) (\code{\link{EMPIRcop}}) for the \code{ctype} for the copula and its survival function form. Expansive more details are provided by \code{\link{taildepCOP}} (section \bold{Note}::\emph{DEMONSTRATION (Tail Dependence)}). A comparison of the \eqn{\hat\lambda^U_{\beta^\diamond_\mathbf{C}}} and \eqn{\hat\lambda^U_{\beta^\diamond_\mathbf{C}}} is made.
}
\references{
Schmid, Friedrich, and Schmidt, Rafael, 2007, Nonparametric inference on multivariate versions of Blomqvist's beta and related measures of tail dependence: Metrika, v. 66, pp. 323--354, \doi{10.1007/s00184-006-0114-3}.
}
\author{W.H. Asquith}
\seealso{\code{\link{blomCOP}}, \code{\link{blomatrixCOP}}, \code{\link{taildepCOP}}
}
\examples{
blomCOP(  cop=PSP) # [1] 0.3333333
blomCOPss(cop=PSP) # [1] 0.3333333

\dontrun{
# The calls below for blomCOPss() are technically the same for sample versions.
UV <- simCOP(1000, cop=PSP, graphics=FALSE)  # HatBeta(0.1,0.9) = 0.277___
blomCOPss(para=UV, cop=EMPIRcop,   uu=c(0.1,0.1), vv=c(0.90,0.90))
blomCOPss(para=UV, as.sample=TRUE, uu=c(0.1,0.1), vv=c(0.90,0.90)) #}

\dontrun{
set.seed(1)
para <- c(3, 6) # define parameters of two-parameter GHcop
UV <- simCOP(1000, cop=GHcop, para=para) # simulate to show general structure

# compute the tail dependencies from havling into the limits
taildepCOP(cop=GHcop, para=para, plot=TRUE)
# lower tail dependency = 0.96222
# upper tail dependency = 0.74008
# The two parameters influence how strongly the tail dependencies are.

# Schmid and Schmidt (2007, eq. 24) define the lower-tail dependency in terms of
# the Beta and p-->0 Beta(c(p,p), c(1,1)). Lets compute these and produce content
# suitable to show on the tail-dependency plot that the assertion for the lower
# dependency by Beta() is correct, which it is and then extend to the upper-tail
# dependency parameter that the authors seem to not have defined.
usr <- par()$usr[1:2]         # grab horizontal edges of the plot, and set up the
uuLO <- rep(pnorm(usr[1]), 2) # the uu for the lower tail and the vv for the upper
vvUP <- rep(pnorm(usr[2]), 2) # tail and then plot both with overplotting symbols
# lower-tail estimate and see how it plots along the value from taildepCOP()
SchmidsL <- blomCOPss(cop=GHcop, para=para, uu=uuLO, vv=c(1,1))
points(usr[1], SchmidsL, col="darkgreen", cex=2, pch=1, lwd=2)
points(usr[1], SchmidsL, col="darkgreen", cex=2, pch=3, lwd=2)
points(usr[1], SchmidsL, col="darkgreen", cex=2, pch=4, lwd=2)
# upper-tail estimate and see how it plots along the value from taildepCOP()
SchmidsU <- blomCOPss(cop=GHcop, para=para, uu=c(0,0), vv=vvUP)
points(usr[2], SchmidsU, col="darkgreen", cex=2, pch=1, lwd=2)
points(usr[2], SchmidsU, col="darkgreen", cex=2, pch=3, lwd=2)
points(usr[2], SchmidsU, col="darkgreen", cex=2, pch=4, lwd=2)
# SchmidsL lower tail dependency = 0.962224
# SchmidsU upper tail dependency = 0.740079
# The author has an expectation that the SchmidsL and SchmidsU values are
# more reliable than those stemming from taildepCOP() because of the limiting
# behavior (or its implementation therein) compared to direct computation by
# blomCOPss().

# Mow for sake of curiosity, let us see how the trajectory of the Blomqvist
# (Schmid--Schmidt) Betas at arriving at the tail dependencies as p-->0|1.
# It is very informative that the trajectories of blomCOPss() and taildepCOP()
# as each hones towards the two dependency parameters are different and this
# highlights the fact that the computational underpinnings are different.
psl <- pnorm(seq(0, usr[1], by=-diff(range(c(0, usr[1]))) / 1000))
lines(qnorm(psl), sapply(psl, function(p) {
              blomCOPss(cop=GHcop, para=para, uu=rep(p, 2), vv=c(1,1)) }),
      col="darkgreen", lty=2, lwd=2)
psu <- pnorm(seq(0, usr[2], by= diff(range(c(0, usr[2]))) / 1000))
lines(qnorm(psu), sapply(psu, function(p) {
              blomCOPss(cop=GHcop, para=para, uu=c(0,0), vv=rep(p, 2)) }),
      col="darkgreen", lty=2, lwd=2) #}
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{Blomqvist Beta}
\keyword{Blomqvist (Schmid-Schmidt) Beta}
\keyword{copula (tail characteristics)}
\keyword{copula (tail properties)}
\keyword{tail dependence}

\encoding{utf8}
\name{W}
\alias{W}
\title{The \enc{Fréchet}{Frechet}--Hoeffding Lower-Bound Copula}
\description{
Compute the \emph{\enc{Fréchet}{Frechet}--Hoeffding lower-bound copula} (Nelsen, 2006, p. 11), which is defined as
\deqn{\mathbf{W}(u,v) = \mathrm{max}(u+v-1,0)\mbox{.}}
This is the copula of perfect anti-association (\emph{countermonotonicity}, \emph{perfectly negative dependence}) between \eqn{U} and \eqn{V} and is sometimes referred to as the \emph{countermonotonicity copula}. Its opposite is the \eqn{\mathbf{M}(u,v)} copula (\emph{comonotonicity copula}; \code{\link{M}}), and statistical \emph{independence} is the \eqn{\mathbf{\Pi}(u,v)} copula (\code{\link{P}}).
}
\usage{
W(u, v, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{M}},       \code{\link{P}},
        \code{\link{breveCOP}}, \code{\link{kfuncCOP}}}
\examples{
W(0.41, 0.60) # just barely touching the support, so small, 0.01
W(0.25, 0.45) # no contact with the support, so 0
W(1,    1   ) # total consumption of the support, so 1

\dontrun{
  # This example shows the impact of the "breve" permutation asymmetry addition
  # to perfect negative correlation that though the plot of u,v changes with
  # spread direction with sign of the breve, that the distribution function of
  # the joint distribution still ranges uniformly 1 for breve = 0 down towards
  # independence as breve approaches -1 and + 1. Then, repeat similarily for
  # perfect positive correlation.
  ff <- c(0.001, seq(0.01, 0.99, by=0.01), 0.999)
  bs <- seq(-1, +1, by=0.1)
  plot(c(0,1), c(0,1), type="n", xlab="Joint probability (Kendall function)",
       ylab="Nonexceedance probability of Kendall function")
  for(b in bs) {
    lines(ff, kfuncCOP(ff, cop=breveCOP, para=list(cop=W, breve=b)),
          col=ifelse(b < 0, grey(0.8), "blue"), lwd=ifelse(b < 0, 6, 1))
  }
  # The top line will be W (breve = 0) and therefore its Kendall function is
  # uniform distribution, and the lowest line is independence.
  for(b in bs) {
    lines(ff, kfuncCOP(ff, cop=breveCOP, para=list(cop=M, breve=b)),
          col=ifelse(b < 0, grey(0.8), "seagreen"), lwd=ifelse(b < 0, 6, 1))
  }
  # The bottom line will be M (breve = 0) and therefore its Kendall function is
  # perfect correlation. The top line by M and breve == +/-1 is independence
  lines(ff, kfuncCOP(ff, cop=P), col="red", lwd=3) # draw independence in red
  legend("bottomright", c("Breve in (0,+1] on M copula by breveCOP()",
                          "Breve in (0,+1] on W copula by breveCOP()",
                          "Perfect independence by P() copula",
                          "Breves in [-1, 0] in M or W copulas by breveCOP()"),
         lwd=c(1,1,3,6), col=c("seagreen", "blue", "red", grey(0.8))) # }
}
\keyword{copula (formulas)}
\keyword{copula (limits/bounds)}
\concept{Frechet lower-bound copula}
\concept{Frechet-Hoeffding lower-bound copula}
\concept{Frechet--Hoeffding lower-bound copula}
\concept{Frechet lower bound}
\concept{Frechet-Hoeffding lower-bound}
\concept{Frechet--Hoeffding lower-bound}
\keyword{copula}
\keyword{Kendall Function}
\concept{Kendall distribution function}
\concept{Kendall function}
\concept{Kendall measure}

\encoding{utf8}
\name{composite3COP}
\alias{composite3COP}
\title{(Extended) Composition of Two Copulas with Four Compositing Parameters}
\description{
The \emph{(extended) composition of two copulas}  (Salvadori \emph{et al.}, 2006, p. 266, prop. C.4) provides for even more sophisticated structures of dependence between variables than two-copula composition in \code{\link{composite2COP}}. Let \eqn{\mathbf{A}} and \eqn{\mathbf{B}} be copulas with respective parameters \eqn{\Theta_\mathbf{A}} and \eqn{\Theta_\mathbf{B}}, then

\deqn{\mathbf{C}_{\alpha,\beta,\kappa,\gamma}(u,v) = u^\kappa v^\gamma \cdot
                              \mathbf{A}\bigl([u^{1-\kappa}]^\alpha,    [v^{1-\gamma}]^\beta\bigr) \cdot
                              \mathbf{B}\bigl([u^{1-\kappa}]^{1-\alpha},[v^{1-\gamma}]^{1-\beta}\bigr)\mbox{,}}

defines a family of copulas \eqn{\mathbf{C}_{\alpha,\beta,\kappa,\gamma}} with \bold{four} \emph{compositing parameters} \eqn{\alpha,\beta,\kappa,\gamma \in (0,1)}.

It is important to stress that copulas \eqn{\mathbf{A}_{\Theta_A}} and \eqn{\mathbf{B}_{\Theta_B}} can be of different families and each parameterized accordingly by the vectors of parameters \eqn{\Theta_A} and \eqn{\Theta_B}.
}
\usage{
composite3COP(u, v, para, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in \eqn{Y} direction;}
  \item{para}{A special parameter \code{list} (see \bold{Note}); and}
  \item{...}{Additional arguments to pass to \code{\link{composite2COP}}.}
}
\value{
  A value for the composited copula is returned.
}
\note{
The following descriptions list in detail the structure and content of the \code{para} argument:
\describe{
  \item{\code{alpha}}{--- The \eqn{\alpha} compositing parameter;}
  \item{\code{beta}}{--- The \eqn{\beta} compositing parameter;}
  \item{\code{kappa}}{--- The \eqn{\kappa} compositing parameter;}
  \item{\code{gamma}}{--- The \eqn{\gamma} compositing parameter;}
  \item{\code{cop1}}{--- Function of the first copula \eqn{\mathbf{A}};}
  \item{\code{cop2}}{--- Function of the second copula \eqn{\mathbf{B}};}
  \item{\code{para1}}{--- Vector of parameters \eqn{\Theta_\mathbf{A}} for \eqn{\mathbf{A}}; and}
  \item{\code{para2}}{--- Vector of parameters \eqn{\Theta_\mathbf{B}} for \eqn{\mathbf{B}}.}

The first example produces two plots. These are extremely informative for many nuances of copula theory. Whereas it is difficult in prose to describe, users are strongly encouraged that once full understanding of connection of red and green between the easier to understand bivariate plot and the plot showing the sections and derivatives of the sections is achieved that much of copula theory will be mastered---get a copy of Nelsen (2006) and (or) Salvadori \emph{et al.} (2007).
}
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in Nature---An approach using copulas: Springer, 289 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{COP}}, \code{\link{breveCOP}}, \code{\link{simCOP}}, \code{\link{composite1COP}}, \code{\link{composite2COP}}, \code{\link{convexCOP}}, \code{\link{glueCOP}}, \code{\link{simcomposite3COP}}}
\examples{
\dontrun{
para <- list(cop1=PLACKETTcop, cop2=N4212cop,
             para1=10^(runif(1,min=-5,max=5)), para2=runif(1,min=1,max=100),
             alpha=runif(1), beta=runif(1), kappa=runif(1), gamma=runif(1))
txts <- c("Alpha=",    round(para$alpha,    digits=4),
          "; Beta=",   round(para$beta,     digits=4),
          "; Kappa=",  round(para$kappa,    digits=4),
          "; Gamma=",  round(para$gamma,    digits=4),
          "; Theta1=", round(para$para1[1], digits=5),
          "; Theta2=", round(para$para2[1], digits=2))
layout(matrix(1:2, byrow=TRUE))
D <- simCOP(n=300, cop=composite3COP, para=para, cex=0.5, col=rgb(0,0,0,0.2), pch=16)
mtext(paste(txts,collapse=""))

f <- round(runif(1),digits=2)
ftxt <- c("Sectionals (thick) and derivatives (thin) at f=",f," nonexceedance prob.")
segments(f,0,f,1, col=3, lwd=2); segments(0,f,1,f, col=2, lwd=2)
t <- sectionCOP(f,cop=composite3COP,para=para, col=3, lwd=4)
t <- sectionCOP(f,cop=composite3COP,para=para, dercop=TRUE, ploton=FALSE,col=3)
t <- sectionCOP(f,cop=composite3COP,para=para, wrtV=TRUE,   ploton=FALSE,col=2,lwd=4)
t <- sectionCOP(f,cop=composite3COP,para=para, wrtV=TRUE,   ploton=FALSE,col=2,
                  dercop=TRUE)
mtext(paste(ftxt, collapse=""))#}
}
\keyword{copula composition}
\keyword{copula composition (four compositing parameters)}

\encoding{utf8}
\name{EMPIRgrid_lkup}
\alias{EMPIRgrid_lkup}
\title{Empirical Copula by Bilinear Interpolation of Gridded Bivariate Empirical Copula}
\description{
Compute joint probability through a gridded \emph{empirical copula} \eqn{\mathbf{C}_n(u,v)} using bilinear interpolation (vectorized, no \code{for} looping). The function is designed to act like \code{\link{EMPIRcop}}, but in lieu of consuming the bivariate data in \code{para} argument, the \code{EMPIRgrid_lkup} operates off a precomputed gridded list representation of the empirical copula computed either from \code{\link{EMPIRgrid}} (legacy) or \code{\link{EMPIRgrid_fast}}. Given a gridded empirical copula denoted as \eqn{\ddot{\mathbf{C}}_n} and bounding cells of a rectangle \eqn{11} (lower left), \eqn{12} (lower right), \eqn{21} (upper left), and \eqn{22}, the \eqn{\mathbf{C}_n(u,v)} can be estimated by bilinear interpolation in a weighted mean form as:

\deqn{\mathbf{C}_n(u,v) \approx w_{11}\times\ddot{\mathbf{C}}_{11} +
                                w_{12}\times\ddot{\mathbf{C}}_{12} +
                                w_{21}\times\ddot{\mathbf{C}}_{21} +
                                w_{22}\times\ddot{\mathbf{C}}_{22}\text{,}}
for which the weights are
\deqn{w_{11} = \frac{(u_2 - u  )(v_2 - v  )}{(u_2 - u_1)(v_2 - v_1)}\text{,\ }
      w_{12} = \frac{(u_2 - u  )(v   - v_1)}{(u_2 - u_1)(v_2 - v_1)}\text{,}}
\deqn{w_{21} = \frac{(u   - u_1)(v_2 - v  )}{(u_2 - u_1)(v_2 - v_1)}\text{,\ and\ }
      w_{22} = \frac{(u   - u_1)(v   - v_1)}{(u_2 - u_1)(v_2 - v_1)}\text{.}}
}
\usage{
EMPIRgrid_lkup(u, v, para=NULL, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A gridded empirical data structure from \code{\link{EMPIRgrid}} and \code{\link{EMPIRgrid_fast}}; and}
  \item{...}{Additional arguments to pass to \code{\link{EMPIRgrid_fast}} if \code{para} is null.}
}
\value{
  Value(s) for the copula are returned.
}
\author{W.H. Asquith}
\seealso{\code{\link{EMPIRgrid}}, \code{\link{EMPIRgrid_fast}}}
\examples{
\dontrun{
  n <- 2000
  para  <- list(cop=GHcop, para=c(2.3,1.5), breve=0.5)
  uv    <- simCOP(n, cop=breveCOP, para=para, graphics=TRUE)
  system.time(gridn <- EMPIRgrid(     uv, ctype="1/n", deluv=1/2000))
  system.time(gridf <- EMPIRgrid_fast(uv, ctype="1/n"))
  # gridf should be shown to be over 100 times faster.
  copn  <- EMPIRgrid_lkup(uv[,1], uv[,2], para=gridn)
  copf  <- EMPIRgrid_lkup(uv[,1], uv[,2], para=gridf)
  plot(copn, copf); abline(0,1) # equal value line }

\dontrun{
  n <- 21 # Make a small sample demonstration and exhaustive list of
  # comparisons in empirical copula computation styles.
  para  <- list(cop=GHcop, para=c(2.3,1.5), breve=0.5)
  uv    <- simCOP(n, cop=breveCOP, para=para,    graphics=TRUE )
  gridn <- EMPIRgrid_fast(uv, ctype="1/n",       gridonly=FALSE)
  gridw <- EMPIRgrid_fast(uv, ctype="bernstein", gridonly=FALSE)
  copw  <- EMPIRcop(uv[,1], uv[,2], para=uv, ctype="weibull"     )
  copn  <- EMPIRcop(uv[,1], uv[,2], para=uv, ctype="1/n"         )
  coph  <- EMPIRcop(uv[,1], uv[,2], para=uv, ctype="hazen"       )
  copb  <- EMPIRcop(uv[,1], uv[,2], para=uv, ctype="bernstein"   )
  copc  <- EMPIRcop(uv[,1], uv[,2], para=uv, ctype="checkerboard")
  copgn <- EMPIRgrid_lkup(uv[,1], uv[,2], para=gridn)
  copgb <- EMPIRgrid_lkup(uv[,1], uv[,2], para=gridw)

  plot(  copgn, copw, pch=24, col="seagreen4", bg="seagreen2")
  points(copgn, copn, pch=24, col="red4"     , bg="red1"     )
  points(copgn, coph, pch=24, col="blue4"    , bg="blue1"    )
  points(copgn, copb, pch=24, col="orchid4"  , bg="orchid1"  )
  points(copgn, copc, pch=24, col="salmon4"  , bg="salmon1"  )

  points(copgb, copw, pch=25, col="seagreen", bg="white")
  points(copgb, copn, pch=25, col="red"     , bg="white")
  points(copgb, coph, pch=25, col="blue"    , bg="white")
  points(copgb, copb, pch=25, col="orchid1" , bg="white")
  points(copgb, copc, pch=25, col="salmon"  , bg="white")

  abline(0,1, lty=2) # equal value line }
}
\keyword{empirical copula}
\keyword{empirical copula (utility)}
\keyword{copulatic surface}
\keyword{gridded empirical copula}

\encoding{utf8}
\name{bicoploc}
\alias{bicoploc}
\title{Analog to Line of Organic Correlation by Copula Diagonal}
\description{
\emph{HIGHLY EXPERIMENTAL AND SUBJECT TO OVERHAUL OR REMOVAL}---Compute an analog to the \emph{line of organic correlation} (\emph{reduced major axis}) using the \emph{diagonal} of a copula.

\deqn{\mathrm{Pr}[U \le u, V \le v] = \mathbf{C}(u,v) = f\mbox{.}}

The primary diagonal is defined as
\deqn{\mathbf{\delta}_\mathbf{C}(t) = \mathbf{C}(t,t) = f\mbox{.}}


Two diagnostic plots can be plotted by the arguments available for this function. The plot for \eqn{(U,V)} coordinate nonexceedance probability domain along with the analyses involving the copula diagonal inversion comes first, which is followed by that for \eqn{(X,Y)} coordinate domain along with the well-known line of organic correlation by the method of L-moments and such a \dQuote{line} (could be a curve) by copula diagonal inversion.

This much infrastructure written for flexibility in how a copula would interact for the purpose of estimation with moment preservation. The simple \eqn{u = v} might be sufficient but let us have some flexibility.
}
\usage{
bicoploc(xp, yp=NULL, xout=NA, xpara=NULL, ypara=NULL, dtypex="nor", dtypey="nor",
         ctype=c("weibull", "hazen", "bernstein", "checkerboard"), kumaraswamy=TRUE,
         plotuv=TRUE, plotxy=TRUE, adduv=FALSE, addxy=FALSE, snv=FALSE, limout=TRUE,
         autoleg=TRUE, xleg="topleft", yleg=NULL, rugxy=TRUE, ruglwd=0.5,
         xlim=NULL, ylim=NULL, titleuv="", titlexy="", titlecex=1,
         a=0, ff=pnorm(seq(-5, +5, by=0.1)), locdigits=6,
         paracop=TRUE, verbose=TRUE, x=NULL, y=NULL, ...)
}
\arguments{
  \item{xp}{Numeric vector giving paired data points of \eqn{X}. If this is a matrix or data frame, then the first and second columns are extracted for the \code{xp} and \code{yp} internall;}
  \item{yp}{Optional numeric vector giving paired data points of \eqn{Y} depending on the composition of \code{x};}
  \item{xout}{An optional set of numeric values specifying where interpolation through the diagonal inversion is to take place;}
  \item{xpara}{An \pkg{lmomco} package parameter object for the \eqn{X} variable, which if not provided will trigger an method of L-moment parameter estimation for the distribution \code{dtypex};}
  \item{ypara}{An \pkg{lmomco} package parameter object for the \eqn{Y} variable, which if not provided will trigger an method of L-moment parameter estimation for the distribution \code{dtypey};}
  \item{dtypex}{The \pkg{lmomco} package distribution abbreviations (see \code{lmomco::dist.list()}) for the \eqn{X} variable. If this argument is set \code{NULL} and \code{xpara} is given with an element of \code{type}, then that distribution type is assigned internally to \code{dtypex};}
  \item{dtypey}{The \pkg{lmomco} package distribution abbreviations (see \code{lmomco::dist.list()}) for the \eqn{Y} variable; If this argument is set \code{NULL} and \code{xpara} is given with an element of \code{type}, then that distribution type is assigned internally to \code{dtypex};}
  \item{ctype}{Argument of the same name for the empirical copula for dispatch to \code{\link{EMPIRcop}}. The \code{1/n} form is disabled for \code{bicoploc} operations based on limited experiments. The first letter of the argument's value is extracted, converted to upper case, and used as the plotting character in the two diagnostic plots;}
  \item{kumaraswamy}{A logical to trigger Kumaraswamy distribution smoothing of the copula diagonal inversion from the \emph{empirical copula}. The Kumaraswamy distribution is a distribution having support \eqn{[0,1]} with an explicit quantile function and takes the place of a Beta distribution (see \pkg{lmomco} function \code{quakur()} for more details). The smoothing by Kumaraswamy will provide a continuous real number on the interval, which should insure no flat-lining as one rolls on to or off off the discrete interval provided by the empirical copula for expected sample sizes of the operations anticipated for the \code{bicoploc} function;}
  \item{plotuv}{A logical to trigger plotting of the analyses in the \eqn{(U,V)} coordinate nonexceedance probability domain along with the analyses involving the copula diagonal inversion. If set true, then \code{adduv} is set false internally;}
  \item{plotxy}{A logical to trigger plotting of the analyses in the \eqn{(X,Y)} coordinate domain along with the well-known line of organic correlation by the method of L-moments and such a \dQuote{line}  (could be a curve) by copula diagonal inversion. If set true, then \code{addxy} is set false internally;}
  \item{adduv}{A logical when set true will not call the \code{plot()} function for \eqn{(U,V)} coordinate nonexceedance probability domain but the other graphical operations of lines and points will be called;}
  \item{addxy}{A logical when set true will not call the \code{plot()} function for \eqn{(X,Y)} coordinate domain but the other graphical operations of lines and points will be called;}
  \item{snv}{A logical when set true will plot the \eqn{(U,V)} coordinate nonexceedance probability domain in units of standard normal variates;}
  \item{limout}{A logical when set true will plot the \eqn{(X,Y)} coordinate domain with horizontal and vertical axis limits inflated to the \code{xout} and \eqn{Y} predictions;}
  \item{autoleg}{A logical when set will draw a legend for the plots if the plots are requested;}
  \item{xleg}{The value to become the argument \code{x} in the \code{legend()} call. The default setting is based on general assumption that this \code{bicoploc()} function is to be more commonly used in positive association circumstances between \eqn{X} and \eqn{Y} (positive Spearman Rho);}
  \item{yleg}{The value to become the argument \code{y} in the \code{legend()} call;}
  \item{rugxy}{Call \code{rug()} plotting operations on the \eqn{X} and \eqn{Y} values used in the parameter estimation of the parametric marginal distributions using the \code{xp} and \code{yp}, unless either or both have been overridden for the contents in \code{x} and (or) \code{y} arguments;}
  \item{ruglwd}{The line wide passed into \code{rug()}. Because plotting of small and large sample sizes can make it difficult in the smaller samples to see the line, it is judged useful to explicitly have this setting as an declared argument;}
  \item{xlim}{A numeric vector that if precisely of length 2 and no missing values therein, will override the horizontal limits of the \code{plotxy} plot of the \eqn{(X,Y)} domain. Otherwise, the contents of \code{xlim}, if not null, are inserted into a range computation for the limits to apply;}
  \item{ylim}{A numeric vector that if precisely of length 2 and no missing values therein, will override the vertical limits of the \code{plotxy} plot of the \eqn{(X,Y)} domain. Otherwise, the contents of \code{ylim}, if not null, are inserted into a range computation for the limits to apply;}
  \item{titleuv}{An optional title for the \eqn{(U,V)} domain plot;}
  \item{titlexy}{An optional title for the \eqn{(X,Y)} domain plot;}
  \item{titlecex}{The character expansion factor for the titles;}
  \item{a}{Value for the plotting-position formula for \code{lmomco::pp()}, default is \code{a=0}, which specifies \emph{Weibull plotting positions};}
  \item{ff}{The nonexceedance joint probability of of the copula diagonal from which inversion computes the marginal nonexceedance probability values for \eqn{u=v=t} as \eqn{\mathbf{C}(t,t) = f} where \code{ff} is the variable notation for the joint probability \eqn{f};}
  \item{locdigits}{Number of digits for rounding exclusive to the \code{loc} data frame produced in the returned list. The reasoning for this setting is that the expected application in practical circumstances will have discipline knowledge of the rounding depth suitable;}
  \item{paracop}{A logical trigging the use of the parametric asymmetric copula fit by numerical optimization to the \eqn{(U,V)} domain of the data (see \bold{Details});}
  \item{verbose}{Show messages of incremental progress with a incremental counter on the message;}
  \item{x}{Numeric vector of \eqn{X} values to be used in parameter estimation of the marginal parametric distribution if \code{xpara=NULL} and these values are internally replaced with \code{xp} (paired \eqn{X}) if not otherwise specified. This provides the ability to insert an alternative and presumably longer vector of the entire \eqn{X} sample without the restriction of individual values paired to the \eqn{Y}. A feature unique to providing the \code{x} is that missing values can be and are removed on the fly prior to parameter estimation of the marginal distribution. The \code{x} can be specific independent of \code{y} or even at all for either. Absolutely no provision is made that \code{x} can be a matrix or data frame holding the \code{y};}
  \item{y}{Numeric vector of \eqn{Y} values to be used in parameter estimation of the marginal parametric distribution if \code{ypara=NULL} and these values are internally replaced with \code{yp} (paired \eqn{Y}) if not otherwise specified. This provides the ability to insert an alternative and presumably longer vector of the entire \eqn{Y} sample without the restriction of individual values paired to the \eqn{X}. A feature unique to providing the \code{y} is that missing values can be and are removed on the fly prior to parameter estimation of the marginal distribution. The \code{x} can be specific independent of \code{y} or even at all for either; and}
  \item{...}{Additional arguments to pass.}
}
\details{
\emph{ON THE USE OF AN PARAMETRIC ASYMMETRIC COPULA}---
}
\value{
  Lists, vectors, and data frames for the computations and predictions are returned.
  \item{organic}{A list containing a data frame of the predictions for \code{xout} by the conventional LOC (\code{locpair}) (see also \code{locsols$lmrloc}), the estimates by the L-moments of the parameters of the marginal distributions (\code{locpara}), the estimates by copula diagonal with Kumaraswamy smoothing (if requested) (\code{bicoploc}), and the predictions based solely on the empirical copula approximation for the diagonal (\code{bicoploc_emp}). The \code{bicoploc} and \code{bicoploc_emp} are equal to each other if Kumaraswamy was not used. The \code{bicoploc} is intended to be the official output from the \code{bicoploc()} function. The furthest right column is \code{bicoploc_cop} and represents the predicting values using a parametric copula as fit to the \eqn{(U,V)} domain mapped into the \eqn{(X,Y)} domain as explained elsewhere in this documentation or sources;}
  \item{locsols}{A list of solutions to the LOC based (1) (\code{locpair}) on the conventional definition on the paired data (\code{xp} and \code{yp}) with the finiteness check previously described by \code{lmomco::lmrloc()} and (2) (\code{locpara}) the LOC solution \emph{not on the paired data} but extractable from the L-moments of the parameters for the marginal distributions. Certain permutations of available features will either have the two L-moment solutions equal, or just the slopes equal, or differing in both intercept and slope. The \code{lmrloc} list contains both L-moment and product moment estimation of the LOC to adhere precisely to \code{lmomco::lmrloc()} output;}
  \item{xpara}{The parameters of the marginal distribution in \eqn{X} either as given in \code{xpara}, as estimated from \code{xp}, or estimated from \code{x} by the method of L-moments through the \pkg{lmomco} package;}
  \item{ypara}{The parameters of the marginal distribution in \eqn{Y} either as given in \code{ypara}, as estimated from \code{yp}, or estimated from \code{y} by the method of L-moments through the \pkg{lmomco} package;}
  \item{faqs}{A named vector containing some numerical facts about the operations and principally the requisite sample sizes involved are reported here;}
  \item{faqscop}{A named vector containing some numerical facts about the operations involving the fitting of the parametric asymmetric copula (Plackett by default) to the \eqn{(U,V)} domain; and}
  \item{diag}{A data frame containing information on the copula diagonal including the joint probability column \code{jtprob} (the \code{ff} as stand in for \eqn{C(t,t) = f}), the \eqn{u=v=t} in column \code{uv} by the Kumaraswamy smooth (if requested), and the solely empirical copula version in column \code{uv_emp}. If the Kumaraswamy smooth is not used, then \code{uv} and \code{uv_emp} will be equal to each other. The furthest right column is \code{uv_cop} and represents the values using a parametric copula as fit to the \eqn{(U,V)} domain as explained elsewhere in this documentation or sources.}
}
\note{
The use of a copula diagonal inversion for purposes of a line of organic correlation analog within the text-book literature and elsewhere is unknown to the developers (December 2023).

Though \code{bicoploc} has extensive logic for working through the copula diagonal, if we know the parent distribution and we just have the marginal probabilities equal to each other (the diagonal), then we recover the moments of \eqn{Y} simply with \eqn{u = v}:
\preformatted{
  library(lmomco)
  ff    <- c(0.0001, seq(0.001, 0.999, by=0.001), 0.9999)
  xpara <- lmomco::vec2par(c(3, 0.6, -0.4), type="pe3")
  ypara <- lmomco::vec2par(c(3, 0.4, +0.6), type="pe3")
  xx <-   rlmomco(100000,  xpara)
  yy <- approx(qlmomco(ff, xpara), qlmomco(ff, ypara), xout=xx)$y
  lmr2par(yy, type="aep4")$para
  #        mu     sigma     gamma
  # 2.9972742 0.3993914 0.5980866
}
}
\references{
Kruskal, W.H., 1953, On the uniqueness of the line of organic correlation: Biometrics, vol. 9, no. 1, pp. 47--58, \doi{10.2307/3001632}.
}
\author{W.H. Asquith}
\seealso{\code{\link{diagCOPatf}}}
\examples{
# paracop set to FALSE in these examples for speed
set.seed(4); nsim <- 50
X  <- rnorm(nsim, mean=3, sd=0.6)
Y  <- rnorm(nsim, mean=0, sd=0.2)
zz <- bicoploc(X,Y, xout=c(2.5, 3.5, 4), dtypex="nor", dtypey="nor",   paracop=FALSE)
# cor(X,Y, method="spearman") # +0.0785114 POSITIVE

set.seed(1); nsim <- 50
X  <- rnorm(nsim, mean=3, sd=0.6)
Y  <- rnorm(nsim, mean=0, sd=0.2)
zz <- bicoploc(X,Y, xout=c(2.5, 3.5, 4), dtypex="nor", dtypey="nor",   paracop=FALSE)
# cor(X,Y, method="spearman") # -0.1351741 NEGATIVE

set.seed(1); nsim <- 50
X  <-           rnorm(nsim, mean=3, sd=0.6)
Y  <- 0.843*X + rnorm(nsim, mean=0, sd=0.2)
zz <- bicoploc(X,Y, xout=c(2.5, 3.5, 4), dtypex="nor", dtypey="nor",   paracop=FALSE)
# cor(X,Y, method="spearman") # for nsim=1E6 RHO=0.92367

set.seed(1); nsim <- 50
X  <-           lmomco::rlmomco(nsim, lmomco::vec2par(c(3, 0.6, +0.5), type="pe3"))
Y  <- 0.3*X^2 + lmomco::rlmomco(nsim, lmomco::vec2par(c(0, 0.4, 0),    type="pe3"))
zz <- bicoploc(X,Y, xout=c(2.5, 3.5, 4.5), dtypex="nor", dtypey="nor", paracop=FALSE)
# cor(X,Y, method="spearman") # for nsim=1E6 RHO=0.92366

set.seed(1); nsim <- 50
X  <-           lmomco::rlmomco(nsim, lmomco::vec2par(c(3, 0.6, +0.5), type="pe3"))
Y  <- 0.3*X^2 + lmomco::rlmomco(nsim, lmomco::vec2par(c(0, 0.4, 0),    type="pe3"))
zz <- bicoploc(X,Y, xout=c(2.5, 3.5, 4.5), dtypex="gev", dtypey="gev", paracop=FALSE)

\dontrun{
########################################################################################
# Image 800 samples in X and Y and though created as pairs, let us assume only
# 50 are actually paired for purposes of demonstration of specified parameters
# and (or) alternative x and y vectors providing the larger sample.
set.seed(1); nsim <- 800; npair <- 50
X  <-           lmomco::rlmomco(nsim, lmomco::vec2par(c(3, 0.6, +0.5), type="pe3"))
Y  <- 0.3*X^2 + rnorm(nsim, mean=0, sd=0.3)
ix <- sample(seq_len(nsim), npair, replace=FALSE)
Xp <- X[ix]; Yp <- Y[ix]; dtypex <- "gev"; dtypey <- "gev"
xpara <- lmomco::lmr2par(X, type=dtypex); ypara <- lmomco::lmr2par(Y, type=dtypey)

# The next two bicoploc() calls produce identical results except for the density
# of data points along the axes for the rug plots. Ultimately, the same parameter
# estimates for the margins exists for both calls. The plotuv is disabled so that
# the user can tab between the two plotxy plots and see that they are the same.
zz <- bicoploc(Xp,Yp, xout=c(2.5, 3.5, 4.5), ruglwd=0.9, plotuv=FALSE,
               xpara=xpara, ypara=ypara, dtypex=NULL, dtypey=NULL)
mtext("Example of specific xpara and ypara from larger sample", line=0.5)

zz <- bicoploc(Xp,Yp, xout=c(2.5, 3.5, 4.5), ruglwd=0.9, plotuv=FALSE,
               xpara=xpara, ypara=ypara, dtypex=NULL, dtypey=NULL, x=X, y=Y)
mtext("Example of specific xpara and ypara from larger sample", line=0.5) #}

\dontrun{
########################################################################################
set.seed(1); nsim <- 50
UV <- rCOP(nsim, cop=breveCOP, para=list(cop=W, alpha=0, beta=0))
X  <- qnorm(UV[,1], mean=3, sd=0.6)
Y  <- qnorm(UV[,2], mean=2, sd=0.2)
zz <- bicoploc(X,Y, xout=c(1.5, 2.5, 3.5, 4), dtypex="nor", dtypey="nor")

set.seed(1); nsim <- 50
UV <- rCOP(nsim, cop=breveCOP, para=list(cop=W, alpha=0, beta=0.5))
X  <- qnorm(UV[,1], mean=3, sd=0.6)
Y  <- qnorm(UV[,2], mean=2, sd=0.2)
zz <- bicoploc(X,Y, xout=c(1.5, 2.5, 3.5, 4), dtypex="nor", dtypey="nor")

set.seed(1); nsim <- 50
UV <- rCOP(nsim, cop=breveCOP, para=list(cop=M_N5p12b, para=3, alpha=0, beta=0.5))
X  <- qnorm(UV[,1], mean=3, sd=0.6)^2
Y  <- qnorm(UV[,2], mean=2, sd=0.2)
zz <- bicoploc(X,Y, xout=c(1.5, 2.5, 3.5, 4), ylim=c(1,2.5),
                    xleg="bottomleft", dtypex="aep4", dtypey="nor")

# A TRUE COUNTER EXAMPLE? Are the 1-v operations not sufficient depending on
# rotation? Is the secondary diagonal of a copula useful? Is there something wrong
# with the reflection operations as implemented at the end of November 2023?
# Should negatives be turned into positives by reversing Y within the internal logic?
# The solution current at end of November 2023 seems proper.
set.seed(1); nsim <- 500
para <- list(cop1=W, para1=4, cop2=GHcop, para2=c(30,.6, .9), alpha=0, beta=0.1)
UV <- rCOP(nsim, cop=composite2COP, para=para)
X  <- -qexp(UV[,1], rate=10)+0.1 # Or the problem is a reflected exponential but the
Y  <- qnorm(UV[,2], mean=2, sd=0.2)  # exp version in lmomco can not handle
zz <- bicoploc(X,Y, xout=c(-0.05, 0, 0.2), dtypex="exp", dtypey="nor")
# Possibly, try another distribution.
zz <- bicoploc(X,Y, xout=c(-0.3, -0.2, 0), ylim=c(1,4), dtypex="aep4", dtypey="nor") #}

\dontrun{
########################################################################################
set.seed(1); nsim <- 200; npair <- 30
para <- list(cop=PLcop, para=80, alpha=0.3, beta=0.05)
UV <- rCOP(nsim, cop=composite1COP, para=para, resamv01=TRUE)
X  <- lmomco::qlmomco(UV[,1], lmomco::vec2par(c(3, 0.6, +0.4), type="pe3"))
Y  <- lmomco::qlmomco(UV[,2], lmomco::vec2par(c(3, 0.4, +0.0), type="pe3"))
ix <- sample(seq_len(nsim), npair, replace=FALSE)
Xp <- X[ix]; Yp <- Y[ix]; dtypex <- "pe3"; dtypey <- "pe3"
xpara <- lmomco::lmr2par(X, type=dtypex); ypara <- lmomco::lmr2par(Y, type=dtypey)
plot(10^X, 10^Y, log="xy", las=1, pch=21, lwd=0.8, col="black", bg="white",
                 xlab="SOME RISK PHENOMENON IN X-DIRECTION",
                 ylab="SOME RISK PHENOMENON IN Y-DIRECTION")
xout <- c(1.5, 2.5, 3.5, 4)
xlim <- c(1.5, 4.5); ylim <- c(1.8, 5.0)
zz <- bicoploc(Xp,Yp, xout=xout,xpara=xpara,ypara=ypara,  xlim=xlim,ylim=ylim)

zz <- bicoploc(Xp,Yp, xout=xout,xpara=xpara,ypara=ypara,  xlim=xlim,ylim=ylim,x=X,y=Y)
zz <- bicoploc(Xp,Yp, xout=xout,dtypex="pe3",dtypey="pe3",xlim=xlim,ylim=ylim,x=X,y=Y)

zz <- bicoploc(X, Y,  xout=xout,dtypex="pe3",dtypey="pe3",xlim=xlim,ylim=ylim)#}
}
\keyword{bivariate prediction}
\keyword{copula (diagonal)}
\keyword{copula (diagonal inversion)}
\keyword{line of organic correlation}
\concept{organic correlation}
\concept{reduced major axis}

\encoding{utf8}
\name{copBasic-package}
\docType{package}
\alias{copBasic-package}
\title{Basic Theoretical Copula, Empirical Copula, and Various Utility Functions}
\description{
The \pkg{copBasic} package is oriented around \emph{bivariate copula theory} and mathematical operations closely follow the recommended texts of Nelsen (2006) and Joe (2014) as well as select other references. Another recommended text is Salvadori \emph{et al.} (2007) and Hofert \emph{et al.} (2018) and are cited herein, but about half of that excellent book concerns univariate applications. The primal objective of \pkg{copBasic} is to provide a study of numerous results shown by authoritative texts on copulas. It is intended that the package will help other copula students in self study, potential course work, and applied circumstances.\cr

\bold{Notes on copulas that are supported.} The author has focused on pedagogical aspects of copulas, and this package is a \emph{diary} of sorts that began in fall 2008. Originally, the author did not implement many copulas in the \pkg{copBasic} in order to deliberately avoid redundancy to that support such as it exists on the \R CRAN. Though as time has progressed, other copulas have been added based on needs of a user community, needs to show some concepts in the general theory, or test algorithms. For example, the Clayton copula (\code{\link{CLcop}}) was a late arriving addition the package (\emph{c.}2017), which was added to assist a specific user; the Frank copula (\code{\link{FRcop}} was added in December 2024 because of an inquiry on using \pkg{copBasic} for \emph{vine copula} (see vine example under \code{\link{FRcop}}). The Normal (\code{\link{NORMcop}}), Marshall--Olkin (\code{\link{MOcop}}), and t-Student copulas (\code{\link{Tcop}}) were added in summer 2025 because of an inquiry on meanings and interpretive properties of L-comoments.\cr

The language and vocabulary of copulas are formidable even within the realm of \emph{bivariate} or \emph{bicopula} as design basis for the package. Often \dQuote{vocabulary} words are often emphasized in \emph{italics}, which is used extensively and usually near the opening of function-by-function documentation to identify vocabulary words, such as \emph{survival copula} (see \code{\link{surCOP}}). This syntax tries to mimic and accentuate the terminology in Nelsen (2006), Joe (2014), and generally most other texts. Italics are used to draw connections between concepts.\cr

In conjunction with the summary of functions in \bold{copBasic-package}, the extensive cross referencing to functions and expansive keyword indexing should be beneficial. The author had no experience with copulas prior to a chance happening upon Nelsen (2006) in \emph{c.}2008. The \pkg{copBasic} package is a personal \emph{tour de force} in self-guided learning. Over time, this package and user's manual have been helpful to many others.\cr

\bold{Helpful Navigation of Copulas Implemented in the copBasic Package}

Some entry points to the copulas implemented are listed in the \bold{Table of Copulas}:
\tabular{lclr}{
\bold{Name} \tab \bold{Symbol} \tab \bold{Function} \tab \bold{Concept} \cr
Lower-bounds copula \tab \eqn{\mathbf{W}(u,v)} \tab \code{\link{W}} \tab copula\cr
Independence copula \tab \eqn{\mathbf{\Pi}(u,v)} \tab \code{\link{P}} \tab copula\cr
Upper-bounds copula \tab \eqn{\mathbf{M}(u,v)} \tab \code{\link{M}} \tab copula\cr
\enc{Fréchet}{Frechet} Family copula \tab \eqn{\mathbf{FF}(u,v)} \tab \code{\link{FRECHETcop}} \tab copula\cr
Ali\enc{–}{-}Mikhail\enc{–}{-}Haq copula \tab \eqn{\mathbf{AMH}(u,v)} \tab \code{\link{AMHcop}} \tab copula\cr
Clayton copula \tab \eqn{\mathbf{CL}(u,v)} \tab \code{\link{CLcop}} \tab copula\cr
Copula of uniform circle \tab \eqn{\mathbf{CIRC}(u,v)} \tab \code{\link{CIRCcop}} \tab copula\cr
Farlie\enc{–}{-}Gumbel\enc{–}{-}Morgenstern (generalized) \tab \eqn{\mathbf{FGM}(u,v)} \tab \code{\link{FGMcop}} \tab copula\cr
Frank \tab \eqn{\mathbf{FR}(u,v)} \tab \code{\link{FRcop}} \tab copula\cr
Galambos copula \tab \eqn{\mathbf{GL}(u,v)} \tab \code{\link{GLcop}} \tab copula\cr
Gumbel\enc{–}{-}Hougaard copula \tab \eqn{\mathbf{GH}(u,v)} \tab \code{\link{GHcop}} \tab copula\cr
Hairpin copula \tab \eqn{\mathbf{HAIRPIN(u,v)}} \tab \code{\link{HAIRPINcop}} \tab copula\cr
\enc{Hüsler}{Husler}\enc{–}{-}Reiss copula \tab \eqn{\;{HR}(u,v)} \tab \code{\link{HRcop}} \tab copula\cr
Joe B5 (the \dQuote{Joe}) copula \tab \eqn{\mathbf{B5}(u,v)} \tab \code{\link{JOcopB5}} \tab copula\cr
Joe\enc{–}{-}Ma copula \tab \eqn{\mathbf{JOMA}(u,v)} \tab \code{\link{JOMAcop}} \tab copula\cr
Marshall\enc{–}{-}Olkin copula \tab \eqn{\mathbf{MO}(u,v)} \tab \code{\link{MOcop}} \tab copula\cr
Nelsen eq.4-2-12 copula \tab \eqn{\mathbf{N4212cop}(u,v)} \tab \code{\link{N4212cop}} \tab copula\cr
Nelsen eq.4-2-20 copula \tab \eqn{\mathbf{N4220cop}(u,v)} \tab \code{\link{N4220cop}} \tab copula\cr
Nelsen eq.4-2-20 copula (reflected) \tab \eqn{\mathbf{rN4220cop}(u,v)} \tab \code{\link{rN4220cop}} \tab copula\cr
Normal copula (refer also to \code{\link{Tcop}}) \tab \eqn{\mathbf{NORM}(u,v)} \tab \code{\link{NORMcop}} \tab copula\cr
Ordinal Sums by Copula \tab \eqn{\mathbf{C}_\mathcal{J}(u,v)} \tab \code{\link{ORDSUMcop}} \tab copula\cr
Ordinal Sums by W-Copula \tab \eqn{\mathbf{C}_\mathcal{J}(u,v)} \tab \code{\link{ORDSUWcop}} \tab copula\cr
Pareto copula \tab \eqn{\mathbf{PA}(u,v)} \tab \code{\link{PAcop}} \tab copula\cr
Plackett copula \tab \eqn{\mathbf{PL}(u,v)} \tab \code{\link{PLcop}} \tab copula\cr
PSP copula \tab \eqn{\mathbf{PSP}(u,v)} \tab \code{\link{PSP}} \tab copula\cr
Raftery copula \tab \eqn{\mathbf{RF}(u,v)} \tab \code{\link{RFcop}} \tab copula\cr
Rayleigh copula \tab \eqn{\mathbf{RAY}(u,v)} \tab \code{\link{RAYcop}} \tab copula\cr
g-EV copula (Gaussian extreme value) \tab \eqn{\mathbf{gEV}(u,v)} \tab \code{\link{gEVcop}} \tab copula\cr
t-EV copula (t-distribution extreme value) \tab \eqn{\mathbf{tEV}(u,v)} \tab \code{\link{tEVcop}} \tab copula\cr
t-Student copula (refer also to \code{\link{NORMcop}}) \tab \eqn{\mathbf{T}(u,v)} \tab \code{\link{Tcop}} \tab copula\cr
Triangular copula \tab \eqn{\mathbf{TRI}(u,v)} \tab \code{\link{TRIcop}} \tab copula\cr
}


A few comments on notation herein are needed. A bold math typeface is used to represent a copula function or family such as \eqn{\mathbf{\Pi}} (see \code{\link{P}}) for the \emph{independence copula}. The syntax \eqn{\mathcal{R}\times\mathcal{R} \equiv \mathcal{R}^2} denotes the orthogonal domain of two real numbers, and \eqn{[0,1]\times [0,1]} \eqn{\equiv} \eqn{\mathcal{I}\times\mathcal{I} \equiv \mathcal{I}^2} denotes the orthogonal domain on the unit square of probabilities. Limits of integration \eqn{[0,1]} or \eqn{[0,1]^2} involving copulas are thus shown as \eqn{\mathcal{I}} and \eqn{\mathcal{I}^2}, respectively.\cr

\emph{Random variables} \eqn{X} and \eqn{Y} respectively denote the horizontal and vertical directions in \eqn{\mathcal{R}^2}. Their probabilistic counterparts are uniformly distributed random variables on \eqn{[0, 1]}, are respectively denoted as \eqn{U} and \eqn{V}, and necessarily also are the respective directions in \eqn{\mathcal{I}^2} (\eqn{U} denotes the horizontal, \eqn{V} denotes the vertical). Often realizations of these random variables are respectively \eqn{x} and \eqn{y} for \eqn{X} and \eqn{Y} and \eqn{u} and \eqn{v} for \eqn{U} and \eqn{V}. There is an obvious difference between nonexceedance probability \eqn{F} and its complement, which is exceedance probability defined as \eqn{1-F}. Both \eqn{u} and \eqn{v} herein are in nonexceedance probability. Arguments to many functions herein are \code{u} \eqn{= u} and \code{v} \eqn{= v} and are almost \emph{exclusively nonexceedance} but there are instances for which the probability arguments are \code{u} \eqn{= 1 - u = u'} and \code{v} \eqn{= 1 - v = v'}.\cr

\bold{Bivariate Association} --- Several of the functions listed above are measures of \emph{bivariate association.} Two of the measures (\emph{Kendall Tau}, \code{\link{tauCOP}}; \emph{Spearman Rho}, \code{\link{rhoCOP}}) are widely known. \R provides native support for their sample estimation of course (\code{stats::cor()}), but each function can be used to call the \code{cor()} function for parallelism to the other measures of this package. The other measures (\emph{Blomqvist Beta}, \emph{Gini Gamma}, \emph{Hoeffding Phi}, \emph{Schweizer--Wolff Sigma}, \emph{Spearman Footrule}) support sample estimation by specially formed calls to their respective functions: \code{\link{blomCOP}}, \code{\link{giniCOP}}, \code{\link{hoefCOP}}, \code{\link{wolfCOP}}, and \code{\link{footCOP}}. \emph{Gini Gamma} (\code{\link{giniCOP}}) documentation (also \code{\link{joeskewCOP}}) shows extensive use of theoretical and sample computations for these and other functions.\cr


\bold{Helpful Navigation of the copBasic Package}

Some other entry points into the package are listed in the following table:
\tabular{lclr}{
\bold{Name} \tab \bold{Symbol} \tab \bold{Function} \tab \bold{Concept} \cr
Copula  \tab \eqn{\mathbf{C}(u,v)} \tab \code{\link{COP}} \tab copula theory\cr
\eqn{\cdots} \tab \eqn{\mathbf{C}(u,v)} \tab \code{\link{COP}}\code{(..., reflect=*)} \tab reflection\cr
\eqn{\cdots} \tab \eqn{\mathbf{C}(u,v)} \tab \code{\link{COP}}\code{(..., reflect=*)} \tab rotation\cr
Survival copula \tab \eqn{\hat{\mathbf{C}}(u',v')} \tab \code{\link{surCOP}} \tab copula theory\cr
Joint survival function \tab \eqn{\overline{\mathbf{C}}(u,v)} \tab \code{\link{surfuncCOP}} \tab copula theory\cr
Co-copula \tab \eqn{\mathbf{C}^\star(u',v')} \tab \code{\link{coCOP}} \tab copula theory\cr
Dual of a copula \tab \eqn{\tilde{\mathbf{C}}(u,v)} \tab \code{\link{duCOP}} \tab copula theory\cr
Primary copula diagonal  \tab \eqn{\delta(t)} \tab \code{\link{diagCOP}} \tab copula theory\cr
Secondary copula diagonal  \tab \eqn{\delta^\star(t)} \tab \code{\link{diagCOP}} \tab copula theory\cr
Inverse copula diagonal \tab \eqn{\delta^{(-1)}(f)} \tab \code{\link{diagCOPatf}} \tab copula theory\cr
Joint probability \tab \eqn{{-}{-}} \tab \code{\link{jointCOP}} \tab copula theory\cr
Bivariate L-moments \tab \eqn{\delta^{[\ldots]}_{k;\mathbf{C}}} \tab \code{\link{bilmoms}} and \code{\link{lcomCOP}} \tab bivariate moments\cr
Bivariate L-comoments \tab \eqn{\tau^{[\ldots]}_{k;\mathbf{C}}} \tab \code{\link{bilmoms}} and \code{\link{lcomCOP}} \tab bivariate moments\cr
Blomqvist Beta \tab \eqn{\beta_\mathbf{C}} \tab \code{\link{blomCOP}} \tab bivariate association\cr
Gini Gamma \tab \eqn{\gamma_\mathbf{C}} \tab \code{\link{giniCOP}} \tab bivariate association\cr
Hoeffding Phi \tab \eqn{\Phi_\mathbf{C}} \tab \code{\link{hoefCOP}} \tab bivariate association\cr
Nu-Skew \tab \eqn{\nu_\mathbf{C}} \tab \code{\link{nuskewCOP}} \tab bivariate moments\cr
Nu-Star (skew) \tab \eqn{\nu^\star_\mathbf{C}} \tab \code{\link{nustarCOP}} \tab bivariate moments\cr
Lp distance to independence \tab  \eqn{\Phi_\mathbf{C} \rightarrow L_p} \tab \code{\link{LpCOP}} \tab bivariate association\cr
Permutation-Mu \tab  \eqn{\mu_{\infty\mathbf{C}}^\mathrm{permsym}} \tab \code{\link{LzCOPpermsym}} \tab permutation asymmetry\cr
Kendall Tau \tab \eqn{\tau_\mathbf{C}} \tab \code{\link{tauCOP}} \tab bivariate association\cr
Kendall Measure \tab \eqn{K_\mathbf{C}(z)} \tab \code{\link{kmeasCOP}} \tab copula theory\cr
Kendall Function \tab \eqn{F_K(z)} \tab \code{\link{kfuncCOP}} \tab copula theory\cr
Kendall Function (\eqn{d}-\eqn{\mathbf{P}}) \tab \eqn{F_K(z; \mathbf{P}_d)} \tab \code{\link{kfuncCOP_Pd}} \tab copula theory\cr
Inverse Kendall Function \tab \eqn{F_K^{(-1)}(z)} \tab \code{\link{kfuncCOPinv}} \tab copula theory\cr
An L-moment of \eqn{F_K(z)} \tab \eqn{\lambda_r(F_K)} \tab\code{\link{kfuncCOPlmom}} \tab L-moment theory\cr
L-moments of \eqn{F_K(z)} \tab \eqn{\lambda_r(F_K)} \tab \code{\link{kfuncCOPlmoms}} \tab L-moment theory\cr
Semi-correlations (negatives) \tab \eqn{\rho_N^{-}(a)} \tab \code{\link{semicorCOP}} \tab bivariate tail association\cr
Semi-correlations (positives) \tab \eqn{\rho_N^{+}(a)} \tab \code{\link{semicorCOP}} \tab bivariate tail association\cr
Spearman Footrule \tab \eqn{\psi_\mathbf{C}} \tab \code{\link{footCOP}} \tab bivariate association\cr
Spearman Rho \tab \eqn{\rho_\mathbf{C}} \tab \code{\link{rhoCOP}} \tab bivariate association\cr
Schweizer\enc{–}{-}Wolff Sigma \tab \eqn{\sigma_\mathbf{C}} \tab \code{\link{wolfCOP}} \tab bivariate association\cr
\eqn{\cdots} \tab \eqn{\hat\sigma_\mathbf{C}} \tab \code{\link{wolfCOPsamc}} \tab bivariate association\cr
\eqn{\cdots} \tab \eqn{\sigma_\mathbf{C}{=}\mathbf{\Pi}} \tab \code{\link{wolfCOPtest}} \tab bivariate association\cr
Density of a copula\tab \eqn{c(u,v)} \tab \code{\link{densityCOP}} \tab copula density\cr
Density visualization \tab \eqn{{-}{-}} \tab \code{\link{densityCOPplot}} \tab copula density\cr
Parametric copulatic surface \tab \eqn{{-}{-}} \tab \code{\link{gridCOP}} \tab copulatic surface\cr
Parametric simulation \tab \eqn{{-}{-}} \tab \code{\link{simCOP}} or   \code{\link{rCOP}}\tab copula simulation\cr
Parametric simulation \tab \eqn{{-}{-}} \tab \code{\link{simCOPmicro}} \tab copula simulation\cr
Maximum likelihood \tab \eqn{\mathcal{L}(\Theta_d)} \tab \code{\link{mleCOP}} \tab copula fitting\cr
Akaike information criterion \tab \eqn{\mathrm{AIC}_\mathbf{C}} \tab \code{\link{aicCOP}} \tab goodness-of-fit\cr
Bayesian information crit. \tab \eqn{\mathrm{BIC}_\mathbf{C}} \tab \code{\link{bicCOP}} \tab goodness-of-fit\cr
Root mean square error \tab \eqn{\mathrm{RMSE}_\mathbf{C}} \tab \code{\link{rmseCOP}} \tab goodness-of-fit\cr
Another goodness-of-fit \tab \eqn{T_n} \tab \code{\link{statTn}} \tab goodness-of-fit\cr
}

Some entry points into the package for \emph{empirical copula} are listed in the following table:

\tabular{lclr}{
\bold{Name} \tab \bold{Symbol} \tab \bold{Function} \tab \bold{Concept} \cr
Empirical copula \tab \eqn{\mathbf{C}_n(u,v)} \tab \code{\link{EMPIRcop}} \tab copula\cr
Empirical simulation \tab \eqn{{-}{-}} \tab \code{\link{EMPIRsim}} \tab copula simulation\cr
Empirical simulation \tab \eqn{{-}{-}} \tab \code{\link{EMPIRsimv}} \tab copula simulation\cr
Empirical (legacy) surface \tab \eqn{\ddot{\mathbf{C}}_n(u,v)} \tab \code{\link{EMPIRgrid}} \tab copulatic surface\cr
Empirical (fast) surface \tab \eqn{\ddot{\mathbf{C}}_n(u,v)} \tab \code{\link{EMPIRgrid_fast}} \tab copulatic surface\cr
Empirical copula (grid lookup) \tab \eqn{\ddot{\mathbf{C}}_n(u,v)} \tab \code{\link{EMPIRgrid_lkup}} \tab copulatic surface\cr
}

\bold{Goodness-of-fit} --- Concerning \emph{goodness-of-fit} and although not quite the same as copula properties (such as \dQuote{correlation}) per se as the coefficients aforementioned in the prior paragraph, three goodness-of-fit metrics of a copula compared to the empirical copula, which are all based the \emph{mean square error} (MSE), are \code{\link{aicCOP}}, \code{\link{bicCOP}}, and \code{\link{rmseCOP}}. This triad of functions is useful for making decisions on whether a copula is more favorable than another to a given dataset. However, because they are genetically related by using MSE and if these are used for copula fitting by minimization, the fits will be identical. A statement of \dQuote{not quite the same} is made because the previously described copula properties are generally defined as types of deviations from other copulas (such as \code{\link{P}}). Another goodness-of-fit statistic is \code{\link{statTn}}, which is based on magnitude summation of fitted copula difference from the empirical copula. These four (\code{\link{aicCOP}}, \code{\link{bicCOP}}, \code{\link{rmseCOP}}, and \code{\link{statTn}}) collectively are relative simple and readily understood measures. These bulk sample statistics are useful, but generally thought to not capture the nuances of tail behavior (\code{\link{semicorCOP}} and \code{\link{taildepCOP}} might be useful).\cr

\bold{Bivariate Skewness and L-comoments} --- \emph{Bivariate skewness} measures are supported in the functions \code{\link{joeskewCOP}} (\code{\link{nuskewCOP}} and \code{\link{nustarCOP}}) and \code{\link{uvlmoms}} (\code{\link{uvskew}}). Extensive discussion and example computations of bivariate skewness are provided in the \code{\link{joeskewCOP}} documentation.  Lastly, so-called \emph{bivariate L-moments} and \emph{bivariate L-comoments} of a copula are directly computable in \code{\link{bilmoms}} (though that function using Monte Carlo integration is deprecated) and \code{\link{lcomCOP}} (direct numerical integration). Function \code{\link{lcomCOP}} is the theoretical counterpart to the \emph{sample L-comoments} provided in the \pkg{lmomco} package by \code{lmomco::lcomoms2()} function. Parameter estimation by L-comoments is demonstrated for the \code{\link{MOcop}} under its \bold{Examples} and for a potentially complex \code{\link{composite1COP}} within \bold{Examples} of \code{\link{lcomCOP}}. Demonstrations of \emph{L-comoment ratio diagrams} and construction methods for various copula is seen in \bold{Examples} within \code{\link{LCOMDIA_ManyCops}} (a collection of vary many 1-parameter copula driven by Spearman Rho, \code{\link{rhoCOP}}), \code{\link{LCOMDIA_GH2cop}} (2-parameter Gumbel--Hougaard copula, \code{\link{GHcop}}),  \code{\link{LCOMDIA_GH3cop}} (3-parameter Gumbel--Hougaard copula, \code{\link{GHcop}}), and \code{\link{ORDSUMcop}} (ordinal sums of M-copula).

\bold{Conditional Simulation} --- Random simulation methods by several functions are listed in the previous table  (\code{\link{simCOPmicro}}, \code{\link{simCOP}}). The \pkg{copBasic} package explicitly uses only \emph{conditional simulation} also known as the \emph{conditional distribution method} for \emph{random variate} generation following Nelsen (2006, pp. 40--41). The \emph{numerical derivatives} (\code{\link{derCOP}} and \code{\link{derCOP2}}) and their \emph{inversions} (\code{\link{derCOPinv}} and \code{\link{derCOPinv2}}) (next table) represent the foundation of the conditional simulation. There are other methods in the literature and available in other \R packages, and a comparison of some methods is made in the \bold{Examples} section of the Gumbel--Hougaard copula (\code{\link{GHcop}}).\cr

Several functions in \pkg{copBasic} make the distinction between \eqn{V} with respect to (\emph{wrt}) \eqn{U} and \eqn{U} \emph{wrt} \eqn{V}, and a guide for the nomenclature involving \emph{wrt} distinctions is listed in the following table:
\tabular{lclr}{
\bold{Name} \tab \bold{Symbol} \tab \bold{Function} \tab \bold{Concept} \cr
Copula inversion \tab \eqn{V} \emph{wrt} \eqn{U} \tab \code{\link{COPinv}} \tab copula operator\cr
Copula inversion \tab \eqn{U} \emph{wrt} \eqn{V} \tab \code{\link{COPinv2}} \tab copula operator\cr
Copula derivative \tab \eqn{\delta \mathbf{C}/\delta u} \tab \code{\link{derCOP}} \tab copula operator\cr
Copula derivative \tab \eqn{\delta \mathbf{C}/\delta v} \tab \code{\link{derCOP2}} \tab copula operator\cr
Copula derivative inversion \tab \eqn{V} \emph{wrt} \eqn{U} \tab \code{\link{derCOPinv}} \tab copula operator\cr
Copula derivative inversion \tab \eqn{U} \emph{wrt} \eqn{V} \tab \code{\link{derCOPinv2}} \tab copula operator\cr
Joint curves \tab \eqn{t \mapsto \mathbf{C}(u=U, v)} \tab \code{\link{joint.curvesCOP}} \tab copula theory\cr
Joint curves \tab \eqn{t \mapsto \mathbf{C}(u, v=V)} \tab \code{\link{joint.curvesCOP2}} \tab copula theory\cr
Level curves \tab \eqn{t \mapsto \mathbf{C}(u=U, v)} \tab \code{\link{level.curvesCOP}} \tab copula theory\cr
Level curves \tab \eqn{t \mapsto \mathbf{C}(u, v=V)} \tab \code{\link{level.curvesCOP2}} \tab copula theory\cr
Level set \tab \eqn{V} \emph{wrt} \eqn{U} \tab \code{\link{level.setCOP}} \tab copula theory\cr
Level set \tab \eqn{U} \emph{wrt} \eqn{V} \tab \code{\link{level.setCOP2}} \tab copula theory\cr
Median regression \tab \eqn{V} \emph{wrt} \eqn{U} \tab \code{\link{med.regressCOP}} \tab copula theory\cr
Median regression \tab \eqn{U} \emph{wrt} \eqn{V} \tab \code{\link{med.regressCOP2}} \tab copula theory\cr
Quantile regression \tab \eqn{V} \emph{wrt} \eqn{U} \tab \code{\link{qua.regressCOP}} \tab copula theory\cr
Quantile regression \tab \eqn{U} \emph{wrt} \eqn{V} \tab \code{\link{qua.regressCOP2}} \tab copula theory\cr
Copula section \tab \eqn{t \mapsto \mathbf{C}(t,a)} \tab \code{\link{sectionCOP}} \tab copula theory\cr
Copula section \tab \eqn{t \mapsto \mathbf{C}(a,t)} \tab \code{\link{sectionCOP}} \tab copula theory\cr
}

The previous three tables do not list all of the myriad of special functions to support similar operations on \emph{empirical copulas}. All empirical copula operators and utilities are prepended with \code{EMPIR} in the function name. An additional note concerning package nomenclature is that an appended \dQuote{\code{2}} to a function name indicates \eqn{U} \emph{wrt} \eqn{V} (\emph{e.g.} \code{\link{EMPIRgridderinv2}} for an inversion of the partial derivatives \eqn{\delta \mathbf{C}/\delta v} across the grid of the empirical copula).

Some additional functions to compute often salient features or characteristics of copulas or bivariate data, including functions for bivariate inference or goodness-of-fit, are listed in the following table:\cr\cr

\tabular{lclr}{
\bold{Name} \tab \bold{Symbol} \tab \bold{Function} \tab \bold{Concept} \cr
Left-tail decreasing \tab \eqn{V} \emph{wrt} \eqn{U}  \tab \code{\link{isCOP.LTD}} \tab bivariate association\cr
Left-tail decreasing \tab \eqn{U} \emph{wrt} \eqn{V}  \tab \code{\link{isCOP.LTD}} \tab bivariate association\cr
Right-tail increasing \tab \eqn{V} \emph{wrt} \eqn{U}  \tab \code{\link{isCOP.RTI}} \tab bivariate association\cr
Right-tail increasing \tab \eqn{U} \emph{wrt} \eqn{V}  \tab \code{\link{isCOP.RTI}} \tab bivariate association\cr
Pseudo-polar representation \tab \eqn{(\widehat{S},\widehat{W})} \tab \code{\link{psepolar}} \tab extremal dependency\cr
Tail concentration function \tab \eqn{q_\mathbf{C}(t)} \tab \code{\link{tailconCOP}} \tab bivariate tail association\cr
Tail (lower) dependency \tab \eqn{\lambda^L_\mathbf{C}} \tab \code{\link{taildepCOP}} \tab bivariate tail association\cr
Tail (upper) dependency \tab \eqn{\lambda^U_\mathbf{C}} \tab \code{\link{taildepCOP}} \tab bivariate tail association\cr
Tail (lower) order \tab \eqn{\kappa^L_\mathbf{C}} \tab \code{\link{tailordCOP}} \tab bivariate tail association\cr
Tail (upper) order \tab \eqn{\kappa^U_\mathbf{C}} \tab \code{\link{tailordCOP}} \tab bivariate tail association\cr
Neg'ly quadrant dependency \tab NQD \tab \code{\link{isCOP.PQD}} \tab bivariate association\cr
Pos'ly quadrant dependency \tab PQD \tab \code{\link{isCOP.PQD}} \tab bivariate association\cr
Permutation symmetry \tab \eqn{\mathrm{permsym}} \tab \code{\link{isCOP.permsym}} \tab copula symmetry\cr
Radial symmetry \tab \eqn{\mathrm{radsym}} \tab \code{\link{isCOP.radsym}} \tab copula symmetry\cr
Skewness (Joe, 2014) \tab \eqn{\eta(p; \psi)} \tab \code{\link{uvskew}} \tab bivariate skewness\cr
Kullback\enc{–}{-}Leibler Divergence \tab \eqn{\mathrm{KL}(f \mid g)} \tab \code{\link{kullCOP}} \tab bivariate inference\cr
KL sample size \tab \eqn{n_{f\!g}} \tab \code{\link{kullCOP}} \tab bivariate inference\cr
The Vuong Procedure \tab \eqn{{-}{-}} \tab \code{\link{vuongCOP}} \tab bivariate inference\cr
Spectral measure \tab \eqn{H(w)} \tab \code{\link{spectralmeas}} \tab extremal dependency inference\cr
Stable tail dependence \tab \eqn{\widehat{l}(x,y)} \tab \code{\link{stabtaildepf}} \tab extremal dependency inference\cr
L-comoments (samp. distr.) \tab \eqn{{-}{-}} \tab \code{\link{lcomCOPpv}} \tab experimental bivariate inference\cr
}

The \bold{Table of Probabilities} that follows lists important relations between various joint probability concepts, the copula, nonexceedance probabilities \eqn{u} and \eqn{v}, and exceedance probabilities \eqn{u'} and \eqn{v'}. A compact summary of these probability relations has obvious usefulness. The notation \eqn{[\ldots, \ldots]} is to read as \eqn{[\ldots \mathrm{\ and\ } \ldots]}, and the \eqn{[\ldots \mid  \ldots]} is to be read as \eqn{[\ldots \mathrm{\ given\ } \ldots]}.
\tabular{rcl}{
\bold{Probability} \tab \bold{and} \tab \bold{Symbol Convention}\cr
\eqn{\mathrm{Pr}[\,U \le u, V \le v\,]}     \tab \eqn{=} \tab \eqn{\mathbf{C}(u,v)} \enc{—}{-} The copula, \code{\link{COP}}\cr
\eqn{\mathrm{Pr}[\,U > u, V > v\,]}         \tab \eqn{=} \tab \eqn{\hat{\mathbf{C}}(u',v')} \enc{—}{-} The survival copula, \code{\link{surCOP}}\cr
\eqn{\mathrm{Pr}[\,U \le u, V > v\,]}       \tab \eqn{=} \tab \eqn{u - \mathbf{C}(u,v')}\cr
\eqn{\mathrm{Pr}[\,U > u, V \le v\,]}       \tab \eqn{=} \tab \eqn{v - \mathbf{C}(u',v)}\cr
\eqn{\mathrm{Pr}[\,U \le u \mid V \le v\,]} \tab \eqn{=} \tab \eqn{\mathbf{C}(u,v)/v}\cr
\eqn{\mathrm{Pr}[\,V \le v \mid U \le u\,]} \tab \eqn{=} \tab \eqn{\mathbf{C}(u,v)/u}\cr
\eqn{\mathrm{Pr}[\,U \le u \mid V > v\,]}   \tab \eqn{=} \tab \eqn{\bigl(u - \mathbf{C}(u,v)\bigr)/(1 - v)}\cr
\eqn{\mathrm{Pr}[\,V \le v \mid U > u\,]}   \tab \eqn{=} \tab \eqn{\bigl(v - \mathbf{C}(u,v)\bigr)/(1 - u)}\cr
\eqn{\mathrm{Pr}[\,U > u \mid V > v\,]}   \tab \eqn{=} \tab \eqn{\hat{\mathbf{C}}(u',v')/u' = \overline{\mathbf{C}}(u,v)/(1-u)}\cr
\eqn{\mathrm{Pr}[\,V > v \mid U > u\,]}   \tab \eqn{=} \tab \eqn{\hat{\mathbf{C}}(u',v')/v' = \overline{\mathbf{C}}(u,v)/(1-v)}\cr
\eqn{\mathrm{Pr}[\,V \le v \mid U = u\,]}   \tab \eqn{=} \tab \eqn{\delta \mathbf{C}(u,v)/\delta u} \enc{—}{-} Partial derivative, \code{\link{derCOP}}\cr
\eqn{\mathrm{Pr}[\,U \le u \mid V = v\,]}   \tab \eqn{=} \tab \eqn{\delta \mathbf{C}(u,v)/\delta v} \enc{—}{-} Partial derivative, \code{\link{derCOP2}}\cr
\eqn{\mathrm{Pr}[\,U > u \mathrm{\ or\ } V > v\,]}     \tab \eqn{=} \tab \eqn{\mathbf{C}^\star(u',v') = 1 - \mathbf{C}(u',v')} \enc{—}{-} The co-copula, \code{\link{coCOP}}\cr
\eqn{\mathrm{Pr}[\,U \le u \mathrm{\ or\ } V \le v\,]} \tab \eqn{=} \tab \eqn{\tilde{\mathbf{C}}(u,v) = u + v - \mathbf{C}(u,v)} \enc{—}{-} The dual of a copula, \code{\link{duCOP}}\cr
}

\bold{Table of Conditional Expectations} that follows includes \emph{conditional expectation} (\emph{conditional mean}) (\code{\link{EuvCOP}}, \code{\link{EvuCOP}}) and \emph{conditional L-moments}:

\tabular{rcl}{
\bold{Probability} \tab \bold{and} \tab \bold{Symbol Convention}\cr
\eqn{E[\,V \mid U = u\,]} \tab \eqn{=} \tab \eqn{\int_0^1 (1 - \delta \mathbf{C}(u,v)/\delta u)\,\mathrm{d}v} \enc{—}{-} Expectation of V given U, \code{\link{EvuCOP}}\cr
\eqn{E[\,U \mid V = v\,]} \tab \eqn{=} \tab \eqn{\int_0^1 (1 - \delta \mathbf{C}(u,v)/\delta v)\,\mathrm{d}u} \enc{—}{-} Expectation of U given V, \code{\link{EuvCOP}}\cr
\eqn{(\lambda_r, \tau_r)[\,V \mid U = u\,]} \tab \eqn{\mapsto} \tab uses \code{\link{derCOP}}   \enc{—}{-} conditional L-moments of V given U, \code{\link{LMRvuCOP}}\cr
\eqn{(\lambda_r, \tau_r)[\,U \mid V = v\,]} \tab \eqn{\mapsto} \tab uses \code{\link{derCOP2}} \enc{—}{-} conditional L-moments of U given V, \code{\link{LMRuvCOP}}\cr
}

The function \code{\link{jointCOP}} has considerable demonstration in its \bold{Note} section of the \bold{joint and} and \bold{joint or} relations shown through simulation and counting scenarios. Also there is a demonstration in the \bold{Note} section of function \code{\link{duCOP}} on application of the concepts of \bold{joint and} conditions, \bold{joint or} conditions, and importantly joint \bold{mutually exclusive or} conditions.

\bold{Copula Construction Methods}

\emph{Permutation asymmetry} (\emph{nonexchangability}) can be added to (inserted in) a copula by \code{\link{breveCOP}}. One, two, or more copulas can be \dQuote{composited,} \dQuote{combined,} or \dQuote{multiplied} in interesting ways to create highly unique bivariate relations and as a result, complex dependence structures can be formed. Such operations can often insert permutation asymmetry as well. To such an end, the package provides three main functions for copula composition: \code{\link{composite1COP}} composites a single copula with two compositing parameters (\emph{Khoudraji device} with \emph{independence}), \code{\link{composite2COP}} (\emph{Khoudraji device}) composites two copulas with two compositing parameters, and \code{\link{composite3COP}} composites two copulas with four compositing parameters.\cr

Two copulas also can be combined through a weighted convex combination using \code{\link{convex2COP}} with a single weighting parameter, and even \eqn{N} number of copulas can be combined by weights using \code{\link{convexCOP}}. So-called \dQuote{gluing} two copula by a parameter is provided by \code{\link{glueCOP}}. Multiplication of two copulas to form a third is supported by \code{\link{prod2COP}}. All eight functions for compositing, combining, or multiplying copulas are compatible with joint probability simulation (\code{\link{simCOP}}), measures of association (\emph{e.g.} \eqn{\rho_\mathbf{C}}), and presumably all other copula operations using \pkg{copBasic} features. Finally, \emph{ordinal sums} of copula are provided by \code{\link{ORDSUMcop}} and \code{\link{ORDSUWcop}} as particularly interesting methods of combining copulas.

\tabular{cclr}{
\bold{No. of copulas} \tab \bold{Combining Parameters} \tab \bold{Function} \tab \bold{Concept}\cr
1 \tab \eqn{\beta} \tab \code{\link{breveCOP}} \tab adding permutation asymmetry\cr
1 \tab \eqn{\alpha, \beta} \tab \code{\link{composite1COP}} \tab copula combination\cr
1 \tab \eqn{\alpha, \beta} \tab \code{\link{khoudraji1COP}} \tab copula combination\cr
1 \tab \eqn{\alpha, \beta} \tab \code{\link{khoudrajiPCOP}} \tab copula combination\cr
2 \tab \eqn{\alpha, \beta} \tab \code{\link{composite2COP}} \tab copula combination\cr
2 \tab \eqn{\alpha, \beta} \tab \code{\link{khoudraji2COP}} \tab copula combination\cr
2 \tab \eqn{\alpha, \beta, \kappa, \gamma} \tab \code{\link{composite3COP}} \tab copula combination\cr
2 \tab \eqn{\alpha, (1-\alpha)} \tab \code{\link{convex2COP}} \tab weighted copula combination\cr
\eqn{N} \tab \eqn{\omega_{i \in N}} \tab \code{\link{convexCOP}} \tab weighted copula combination\cr
2 \tab \eqn{\gamma} \tab \code{\link{glueCOP}} \tab gluing of coupla\cr
2 \tab \eqn{\bigl(\mathbf{C}_1 \ast \mathbf{C}_2 \bigr)} \tab \code{\link{prod2COP}} \tab copula multiplication\cr
\eqn{N} \tab \eqn{\mathbf{C}_{\mathcal{J}i}} for \eqn{\mathcal{J}_{i \in N}} partitions \tab \code{\link{ORDSUMcop}} \tab \code{\link{M}}-ordinal sums of copulas\cr
\eqn{N} \tab \eqn{\mathbf{C}_{\mathcal{J}i}} for \eqn{\mathcal{J}_{i \in N}} partitions \tab \code{\link{ORDSUWcop}} \tab \code{\link{W}}-ordinal sums of copulas\cr
}

\bold{Useful Copula Relations by Visualization}

There are a myriad of relations amongst variables computable through copulas, and these were listed in the \bold{Table of Probabilities} earlier in this documentation. There is a script located in the \code{inst/doc} directory of the \pkg{copBasic} sources titled \code{CopulaRelations_BaseFigure_inR.txt}. This script demonstrates, using the \code{\link{PSP}} copula, relations between the copula (\code{\link{COP}}), survival copula (\code{\link{surCOP}}), joint survival function of a copula (\code{\link{surfuncCOP}}), co-copula (\code{\link{coCOP}}), and dual of a copula function (\code{\link{duCOP}}). The script performs simulation and manual counts observations meeting various criteria in order to compute their \emph{empirical probabilities}. The script produces a base figure, which after extending in editing software, is suitable for educational description and is provided at the end of this documentation.

\bold{A Review of \dQuote{Return Periods} using Copulas}

Study of natural hazards commonly use as \emph{annual return periods} \eqn{T} in years, which are defined for a nonexceedance probability \eqn{q} as \eqn{T = 1/(1-q)}. In bivariate analysis, there immediately emerge two types of return periods representing \eqn{T_{q;\,\mathrm{coop}}} and \eqn{T_{q;\,\mathrm{dual}}} conditions between nonexceedances of the two hazard sources (random variables) \eqn{U} and \eqn{V}. It is usual in many applications for \eqn{T} to be expressed equivalently as a probability \eqn{q} in common for both variables.

Incidentally, the \eqn{\mathrm{Pr}[\,U > u \mid V > v\,]} and \eqn{\mathrm{Pr}[\,V > v \mid U > u\,]} probabilities also are useful for \emph{conditional return period} computations following Salvadori \emph{et al.} (2007, pp. 159--160) but are not further considered here. Also the \eqn{F_K(w)} (\emph{Kendall Function} or \emph{Kendall Measure} of a copula) is the core tool for \emph{secondary return period} computations (see \code{\link{kfuncCOP}}).


Let the copula \eqn{\mathbf{C}(u,v; \Theta)} for nonexceedances \eqn{u} and \eqn{v} be set for some copula family (formula) by a parameter vector \eqn{\Theta}. The copula family and parameters define the joint coupling (loosely meant the dependency/correlation) between hazards \eqn{U} and \eqn{V}. If \dQuote{failure} occurs if \bold{either} or \bold{both} hazards \eqn{U} and \eqn{V} are at probability \eqn{q} threshold (\eqn{u = v = 1 - 1/T = q}) for \eqn{T}-year return period, then the \bold{real return period} of failure is defined using either the copula \eqn{\mathbf{C}(q,q; \Theta)} or the \emph{co-copula} \eqn{\mathbf{C}^\star(q',q'; \Theta)} for exceedance probability \eqn{q' = 1 - q} is

\deqn{T_{q;\,\mathrm{coop}} = \frac{1}{1 - \mathbf{C}(q, q; \Theta)} = \frac{1}{\mathbf{C}^\star(1-q, 1-q; \Theta)}\mbox{\ and}}
\deqn{T_{q;\,\mathrm{coop}} \equiv \frac{1}{\mathrm{cooperative\ risk}}\mbox{.}}

Or in words, the hazard sources \bold{collaborate} or \bold{cooperate} to cause failure. If failure occurs,  however, if and only if \bold{both} hazards \eqn{U} and \eqn{V} occur simultaneously (the hazards must \dQuote{dually work together} or be \dQuote{conjunctive}), then the \bold{real return period} is defined using either the \emph{dual of a copula (function)} \eqn{\tilde{\mathbf{C}}(q,q; \Theta)}, the \emph{joint survival function} \eqn{\overline{\mathbf{C}}(q,q;\Theta)}, or \emph{survival copula} \eqn{\hat{\mathbf{C}}(q',q'; \Theta)} as

\deqn{T_{q;\,\mathrm{dual}} = \frac{1}{1 - \tilde{\mathbf{C}}(q,q; \Theta)} = \frac{1}{\overline{\mathbf{C}}(q,q;\Theta)} = \frac{1}{\hat{\mathbf{C}}(q',q';\Theta)} \mbox{\ and}}
\deqn{T_{q;\,\mathrm{dual}} \equiv \frac{1}{\mathrm{complement\ of\ dual\ protection}}\mbox{.}}


Numerical demonstration is informative. Salvadori \emph{et al.} (2007, p. 151) show for a \emph{Gumbel--Hougaard copula} (\code{\link{GHcop}}) having \eqn{\Theta =} 3.055 and \eqn{T =} 1,000 years (\eqn{q = 0.999}) that \eqn{T_{q;\,\mathrm{coop}} = 797.1} years and that \eqn{T_{q;\,\mathrm{dual}}} = 1,341.4 years, which means that average return periods between \dQuote{failures} are
\deqn{T_{q;\,\mathrm{coop}} \le T \le T_{q;\,\mathrm{dual}}\mbox{\ and thus}}
\deqn{797.1 \le T \le 1314.4\mbox{\ years.}}

With the following code, these bounding return-period values are readily computed and verified using the \code{prob2T()} function from the \pkg{lmomco} package along with \pkg{copBasic} functions \code{\link{COP}} (generic functional interface to a copula) and \code{\link{duCOP}} (\emph{dual of a copula}):
\preformatted{
  q <- lmomco::T2prob(1000)
  lmomco::prob2T(  COP(q,q, cop=GHcop, para=3.055)) #  797.110
  lmomco::prob2T(duCOP(q,q, cop=GHcop, para=3.055)) # 1341.438
}
An early source (in 2005) by some of those authors cited on p. 151 of Salvadori \emph{et al.} (2007; their citation \dQuote{[67]}) shows \eqn{T_{q;\,\mathrm{dual}} = 798} years---a rounding error seems to have been committed. Finally and just for reference, a Gumbel--Hougaard copula having \eqn{\Theta = 3.055} corresponds to an analytical \emph{Kendall Tau} (see \code{\link{GHcop}}) of \eqn{\tau \approx 0.673}, which can be verified through numerical integration available from \code{\link{tauCOP}} as:
\preformatted{
  tauCOP(cop=GHcop, para=3.055, brute=TRUE) # 0.6726542
}
Thus, a \dQuote{better understanding of the statistical characteristics of [multiple hazard sources] requires the study of their joint distribution} (Salvadori \emph{et al.}, 2007, p. 150).


\bold{Interaction of copBasic to the Copulas and Features of Other Packages}

The \pkg{copBasic} package (\emph{c.}2008--11) was not originally intended to be a port of the numerous bivariate copulas or over reimplementation other bivariate copulas available in \R, but in support of didactic and teaching efforts. Though as the package passed its 10th year in 2018, the original intent changed to a more generalized purpose as more copula were added and other features by request. It is useful to point out a demonstration showing an implementation of the \emph{Gaussian copula} from the \pkg{copula} package, which is shown in the \bold{Note} section of \code{\link{med.regressCOP}} in a circumstance of ordinary least squares linear regression compared to \emph{median regression} of a copula as well as prediction limits of both regressions. Another demonstration in context of \emph{maximum pseudo-log-likelihood estimation} of copula parameters is seen in the \bold{Note} section \code{\link{mleCOP}}, and also see \dQuote{\bold{API to the copula package}} or \dQuote{\bold{package copula (comparison to)}} entries in the Index of this user manual.

\if{html}{\figure{CopulaRelationsFigure4pkg.jpg}{options: width=500}}
\if{latex}{\figure{CopulaRelationsFigure4pkg.pdf}{options: width=4in, trim=1in 2.25in 0.75in 1in, clip}}


}
\references{
Cherubini, U., Luciano, E., and Vecchiato, W., 2004, Copula methods in finance: Hoboken, NJ, Wiley, 293 p.

\enc{Hernández-Maldonado}{Hernandez-Maldonado}, V., \enc{Díaz-Viera}{Diaz-Viera}, M., and Erdely, A., 2012, A joint stochastic simulation method using the Bernstein copula as a flexible tool for modeling nonlinear dependence structures between petrophysical properties: Journal of Petroleum Science and Engineering, v. 90--91, pp. 112--123.

Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

Hofert, M., Kojadinovic, I., \enc{Mächler}{Machler},  M., and Yan, J., 2018, Elements of copula modeling with R: Dordrecht, Netherlands, Springer.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in nature---An approach using copulas: Dordrecht, Netherlands, Springer, Water Science and Technology Library 56, 292 p.
}
\author{William Asquith \email{william.asquith@ttu.edu}}
\examples{
\dontrun{
  # Nelsen (2006, p. 75, exer. 3.15b) provides for a nice test of copBasic features.
  "mcdurv" <- function(u,v, theta) {
     ifelse(u > theta & u < 1-theta & v > theta & v < 1 - theta,
               return(M(u,v) - theta), # Upper bounds copula with a shift
               return(W(u,v)))         # Lower bounds copula
  }
  "MCDURV" <- function(u,v, para=NULL) {
     if(is.null(para))         stop("need theta")
     if(para < 0 | para > 0.5) stop("theta ! in [0, 1/2]")
     return(asCOP(u, v, f=mcdurv, para))
  }
  "afunc" <- function(t) { # a sample size = 1,000 hard wired
     return(cov(simCOP(n=1000, cop=MCDURV, para=t, ploton=FALSE, points=FALSE))[1,2])
  }
  set.seed(6234) # setup covariance based on parameter "t" and the "root" parameter
  print(uniroot(afunc, c(0, 0.5))) # "t" by simulation = 0.1023742
  # Nelsen reports that if theta approx. 0.103 then covariance of U and V is zero.
  # So, one will have mutually completely dependent uncorrelated uniform variables!

  # Let us check some familiar measures of association:
  rhoCOP( cop=MCDURV, para=0.1023742) # Spearman Rho = 0.005854481 (near zero)
  tauCOP( cop=MCDURV, para=0.1023742) # Kendall Tau  = 0.2648521
  wolfCOP(cop=MCDURV, para=0.1023742) # S & W Sigma  = 0.4690174 (less familiar)
  D <- simCOP(n=1000, cop=MCDURV, para=0.1023742) # Plot mimics Nelsen (2006, fig. 3.11)
  # L-comoments (matrices) measure high dimensional variable comovements
  # L-comoments (refer also to the lmomco package)
  lcomCOP(cop=MCDURV, para=0.1023742)
  lmomco::lcomoms2(simCOP(n=1000, cop=MCDURV, para=0.1023742), nmom=5)
  lmomco::lcomoms2(simCOP(n=1000, cop=MCDURV, para=0),   nmom=5) # Perfect neg. corr.
  lmomco::lcomoms2(simCOP(n=1000, cop=MCDURV, para=0.5), nmom=5) # Perfect pos. corr.
  # T2 (L-correlation), T3 (L-coskew), T4 (L-cokurtosis), and T5 matrices result. For
  # Theta = 0 or 0.5 see the matrix symmetry with a sign change for L-coskew and T5 on
  # the off diagonals (offdiags). See unities for T2. See near zero for offdiag terms
  # in T2 near zero. But then see that T4 off diagonals are quite different from those
  # for Theta 0.1024 relative to 0 or 0.5. As a result, T4 has captured a unique
  # property of U vs V.}
}
\keyword{multivariate}
\concept{copula theory}
\keyword{L-comoments}
\keyword{Lcomoments}
\keyword{Nelsen (2006) Examples and Exercises}
\keyword{Salvadori et al. (2007) Examples and Exercises}
\keyword{return period (conditional)}
\keyword{return period (secondary)}
\keyword{Table of Copulas}
\keyword{Table of Probabilities}
\keyword{Table of Expectations}
\keyword{Table of Probabilities and Expectations}

\encoding{utf8}
\name{convexCOP}
\alias{convexCOP}
\title{Convex Combination of an Arbitrary Number of Copulas}
\description{
The \emph{convex composition of \eqn{N} number of copulas} (Salvadori \emph{et al.}, p. 132, 2007) provides for complexity extension between coupla families. Let \eqn{\mathbf{C}_{i}} be a copula with respective vector of parameters \eqn{\Theta_i}, then the convex combination of these copulas is

\deqn{\mathbf{C}^{\times}_{\omega}(u,v) = \sum_{i=1}^N \omega_i \mathbf{C}_{i}(u, v; \Theta_i)\mbox{,}}

where \eqn{\sum_{i=1}^N \omega_i = 1} for \eqn{N} number of copulas. The weights \eqn{\omega} are silently treated as \eqn{1/N} if the \code{weights} element is absent in the \R \code{list} argument \code{para}.
}
\usage{
convexCOP(u,v, para, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A special parameter \code{list} (see \bold{Note}); and}
  \item{...}{Additional arguments to pass to the copula.}
}
\value{
  Value(s) for the convex combination copula is returned.
}
\note{
The following descriptions list in detail the structure and content of the \code{para} argument but please reference the \bold{Examples} to see the \code{i} notation:
\describe{
  \item{\code{copi}}{--- The \eqn{i}th copula;}
  \item{\code{parai}}{--- Vector of parameters \eqn{\Theta_i}; and}
  \item{\code{weights}}{--- Optional vector of weights whose sum will be rescaled to unity; default is \eqn{1/N} for each weight.}
}
}
\references{
Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in Nature---An approach using copulas: Springer, 289 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{COP}}, \code{\link{breveCOP}}, \code{\link{convex2COP}}, \code{\link{composite1COP}}, \code{\link{composite2COP}}, \code{\link{composite3COP}}, \code{\link{glueCOP}}}
\examples{
# The copulas and parameters are named by sequence number appended to cop and para.
para1 <- list(cop1=GHcop, cop2=PLcop, para1=8, para2=.03, weights=c(.8,.2))
para2 <- list(cop1=GHcop, cop2=PLcop, para1=8, para2=.03, alpha=0.8)
H <- convexCOP( 0.6,0.4, para=para1)
G <- convex2COP(0.6,0.4, para=para2)
if( abs(H-G) <= 1e-6 )  message("They are equal.")

\dontrun{
# A convex combination of three copulas. A GHcop with strong positive association and
# a Plackett with strong negative association, and independence. The weights favor the
# GHcop but a little outlier and expansive spread is superimposed on the core trend.
para <- list(cop1=GHcop, cop2=PLcop, cop3=P,
             para1=8, para2=.03, para3=NA, weights=c(40,7,10))
UV <- simCOP(1000, cop=convexCOP, para=para, lwd=0.8) #}
}
\keyword{copula composition}
\keyword{copula composition (convex combination)}

\encoding{utf8}
\name{PLACKETTcop}
\alias{PLACKETTcop}
\alias{PLcop}
\title{The Plackett Copula}
\description{
The \emph{Plackett copula} (Nelsen, 2006, pp. 89--92) is
\deqn{\mathbf{C}_\Theta(u,v) = \mathbf{PL}(u,v) = \frac{[1+(\Theta-1)(u+v)]-\sqrt{[1+(\Theta-1)(u+v)]^2 - 4uv\Theta(\Theta-1)}}{2(\Theta - 1)}\mbox{.}}

The Plackett copula (\eqn{\mathbf{PL}(u,v)}) is \emph{comprehensive} because as \eqn{\Theta \rightarrow 0} the copula becomes \eqn{\mathbf{W}(u,v)} (see \code{\link{W}}, \emph{countermonotonicity}), as \eqn{\Theta \rightarrow \infty} the copula becomes \eqn{\mathbf{M}(u,v)} (see \code{\link{M}},  \emph{comonotonicity}) and for \eqn{\Theta = 1} the copula is \eqn{\mathbf{\Pi}(u,v)} (see \code{\link{P}},  \emph{independence}).

Nelsen (2006, p. 90) shows that
\deqn{\Theta = \frac{H(x,y)[1 - F(x) - G(y) + H(x,y)]}{[F(x) - H(x,y)][G(y) - H(x,y)]}\mbox{,}}
where \eqn{F(x)} and \eqn{G(y)} are cumulative distribution function for random variables \eqn{X} and \eqn{Y}, respectively, and \eqn{H(x,y)} is the joint distribution function. Only Plackett copulas have a constant \eqn{\Theta} for any pair \eqn{\{x,y\}}. Hence, Plackett copulas are also known as \emph{constant global cross ratio} or \emph{contingency-type} distributions. The copula therefore is intimately tied to \emph{contingency tables} and in particular the bivariate Plackett defined herein is tied to a \eqn{2\times2} contingency table. Consider the \eqn{2\times 2} contingency table shown at the end of this section, then \eqn{\Theta} is defined as

\deqn{\Theta = \frac{a/c}{b/d} = \frac{\frac{a}{a+c}/\frac{c}{a+c}}{\frac{b}{b+d}/\frac{d}{b+d}}\mbox{\ and\ }\Theta = \frac{a/b}{c/d} = \frac{\frac{a}{a+b}/\frac{b}{a+b}}{\frac{c}{c+d}/\frac{d}{c+d}}\mbox{,}}

where it is obvious that \eqn{\Theta = ad/bc} and \eqn{a}, \eqn{b}, \eqn{c}, and \eqn{d} can be replaced by proporations for a sample of size \eqn{n} by \eqn{a/n}, \eqn{b/n}, \eqn{c/n}, and \eqn{d/n}, respectively. Finally, this copula has been widely used in modeling and as an alternative to bivariate distributions and has respective \emph{lower-} and \emph{upper-tail dependency parameters} of \eqn{\lambda^L = 0} and \eqn{\lambda^U = 0} (\code{\link{taildepCOP}}).

\tabular{rccc}{
\eqn{{-}{-}}     \tab \bold{Low} \tab \bold{High} \tab \bold{Sums}\cr
\bold{Low}  \tab \eqn{a}    \tab \eqn{b}     \tab \eqn{a+b}\cr
\bold{High} \tab \eqn{c}    \tab \eqn{d}     \tab \eqn{c+d}\cr
\bold{Sums} \tab \eqn{a+c}  \tab \eqn{b+d}   \tab \eqn{{-}{-}}
}
}
\usage{
PLACKETTcop(u, v, para=NULL, ...)
      PLcop(u, v, para=NULL, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (single element) of parameters---the \eqn{\Theta} parameter of the copula; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\note{
The Plackett copula was the first (2008) copula implemented in \pkg{copBasic} as part of initial development of the code base for instructional purposes. Thus, this particular copula has a separate parameter estimation function in \code{\link{PLACKETTpar}} as a historical vestige of a class project.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{PLACKETTpar}}, \code{\link{PLpar}}, \code{\link{PLACKETTsim}}, \code{\link{W}}, \code{\link{M}}, \code{\link{densityCOP}}
}
\examples{
PLACKETTcop(0.4, 0.6, para=1)
P(0.4, 0.6) # independence copula, same two values because Theta == 1
PLcop(0.4, 0.6, para=10.25) # joint probability through positive association

\dontrun{
# Joe (2014, p. 164) shows the closed form copula density of the Plackett.
"dPLACKETTcop" <- function(u,v,para) {
   eta <- para - 1; A <- para*(1 + eta*(u+v-2*u*v))
   B <- ((1 + eta*(u+v))^2 - 4*para*eta*u*v)^(3/2); return(A/B)
}
u <- 0.08; v <- 0.67 # Two probabilities to make numerical evaluations.
del <- 0.0001 # a 'small' differential value of probability
u1 <- u; u2 <- u+del; v1 <- v; v2 <- v+del
# Density following (Nelsen, 2006, p. 10)
dCrect <- (PLcop(u2, v2, para=10.25) - PLcop(u2, v1, para=10.25) -
           PLcop(u1, v2, para=10.25) + PLcop(u1, v1, para=10.25)) / del^2
dCanal <- dPLACKETTcop(u, v, para=10.25)
dCfunc <-   densityCOP(u, v, para=10.25, cop=PLcop, deluv = del)
R <- round(c(dCrect, dCanal, dCfunc), digits=6)
message("Density: ", R[1], "(manual), ", R[2], "(analytical), ", R[3], "(function)");
# Density: 0.255377(manual), 0.255373(analytical), 0.255377(function)

# Comparison of partial derivatives
dUr <- (PLcop(u2, v2, para=10.25) - PLcop(u1, v2, para=10.25)) / del
dVr <- (PLcop(u2, v2, para=10.25) - PLcop(u2, v1, para=10.25)) / del
dU  <- derCOP( u, v, cop=PLcop, para=10.25)
dV  <- derCOP2(u, v, cop=PLcop, para=10.25)
R   <- round(c(dU, dV, dUr, dVr), digits=6)
message("Partial derivatives dU=", R[1], " and dUr=", R[3], "\n",
        "                    dV=", R[2], " and dVr=", R[4]) #}
}
\keyword{Plackett copula}
\keyword{copula (formulas)}
\keyword{copula (density)}
\keyword{copula (comprehensive)}
\concept{comprehensive copula}
\keyword{copula}
\keyword{Joe (2014) Examples and Exercises}

\encoding{utf8}
\name{FRcop}
\alias{FRcop}
\title{The Frank Copula}
\description{
The \emph{Frank copula} (Joe, 2014, p. 165) is

\deqn{\mathbf{C}_{\Theta}(u,v) = \mathbf{FR}(u,v) =
-\frac{1}{\Theta}\mathrm{log}\biggl[\frac{1 - \mathrm{e}^{-\Theta} - \bigl(1 - \mathrm{e}^{-\Theta u}\bigr) \bigl(1 - \mathrm{e}^{-\Theta v}\bigr)}{1 - \mathrm{e}^{-\Theta}}\biggr]\mbox{,}
}

where \eqn{\Theta \in [-\infty, +\infty], \Theta \ne 0}. The copula, as \eqn{\Theta \rightarrow -\infty} limits, to the \emph{countermonotonicity coupla} (\eqn{\mathbf{W}(u,v)}; \code{\link{W}}), as \eqn{\Theta \rightarrow 0^{\pm}} limits to the \emph{independence copula} (\eqn{\mathbf{\Pi}(u,v)}; \code{\link{P}}), and as \eqn{\Theta \rightarrow +\infty}, limits to the \emph{comonotonicity copula} (\eqn{\mathbf{M}(u,v)};  \code{\link{M}}). The parameter \eqn{\Theta} is readily computed from a \emph{Kendall Tau} (\code{\link{tauCOP}}) by numerical methods as  \eqn{\tau_{\mathbf{C}}(\Theta) = 1 + 4\Theta^{-1}[D_1(\Theta) - 1]}
or from a \emph{Spearman Rho} (\code{\link{rhoCOP}}) as
\eqn{\rho_{\mathbf{C}}(\Theta) = 1 + 4\Theta^{-1}[D_2(\Theta) - D_1(\Theta)]} for \emph{Debye function} as
\deqn{
  D_k(x, k) = k x^{-k} \int_0^x t^k \bigl(\mathrm{e}^{t} - 1\bigr)^{-1}\, \mathrm{d}t\mbox{.}
}
}
\usage{
FRcop(u, v, para=NULL, rhotau=NULL, userhotau_chk=TRUE,
            cortype=c("kendall", "spearman", "tau", "rho"), ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (single element) of parameters---the \eqn{\Theta} parameter of the copula;}
  \item{rhotau}{Optional Kendall Tau or Spearman Rho and parameter \code{para} is returned depending on the setting of \code{cortype}. The \code{u} and \code{v} can be used for estimation of the parameter as computed through the setting of \code{cortype};}
  \item{cortype}{A character string controlling, if the parameter is not given, to use a Kendall Tau or Spearman Rho for estimation of the parameter. The name of this argument is reflective of an internal call to \code{stats::cor()} to the correlation (association) setting for Kendall Tau or Spearman Rho;}
  \item{userhotau_chk}{A logical to trigger computation of Kendall Tau for the given parameter and used as a secondary check on numerical limits of the copula implementation for the package; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned. Otherwise if \code{tau} is given, then the \eqn{\Theta} is computed and a \code{list} having
  \item{para}{The parameter \eqn{\Theta}, and}
  \item{tau}{Kendall Tau.}
and if \code{para=NULL} and \code{tau=NULL}, then the values within \code{u} and \code{v} are used to compute Kendall Tau and then compute the parameter, and these are returned in the aforementioned list. Or if \code{rho} is given, then the \eqn{\Theta} is computed and a similar \code{list} is returned having similar structure but with Spearman Rho instead.
}
\note{
Whether because of default directions of derivative in \code{\link{derCOP}} for partial derivative of the copula or other numerical challenges, the implementation uses the negative parameter whether positive or not and rotates the copula as needed for complete operation of \code{\link{simCOP}}. Several default limiting conditions are in the sources before conversion to independence, perfect positive dependence, or perfect negative dependence.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{M}}, \code{\link{P}}, \code{\link{W}}
}
\examples{
UV  <- simCOP(n=1000, cop=FRcop, para=20, seed=1)
print(FRcop(UV[,1], UV[,2], cortype="kendall" )$tau) # 0.8072993
print(FRcop(UV[,1], UV[,2], cortype="spearman")$rho) # 0.9536648

\dontrun{
  # Joe (2014) example follows by extending the functionality of copBasic
  # into a 3-dimensional C-vine copula using Frank and Gumbel copulas and
  # Kendall Taus and Kendall Partial Taus. The example is expanded to show
  # how advanced features of copBasic can be incorporated for asymmetry
  # (if needed) and reflections (rotations) of copula:
  nsim <- 5000 # number of simulations but not too large for long CPU
  d <- 3       # three dimensions in Joe (2014, algorithm 15) implementation
  GHcop_para <- GHcop(tau   =0.5                   )$para # theta = 2
  FRcop_para <- FRcop(rhotau=0.7, cortype="kendall")$para # theta = 11.41155
  # Substantial notation complexity by structuring the C-vine by matrices for
  # copula families and their parameters then dial in asymmetry by "breves"
  # (see copBasic::breveCOP) and then reflection (rotation ability in the
  # copulas themselves). With zero breves, we have no asymmetry (permutation)
  # and we are going with the basic copula formula, so Reflects are all 1.
  Cops     <- matrix(c(NA,    "GHcop",     "FRcop",
                       NA,         NA,         "M",
                       NA,         NA,         NA),  nrow=d, ncol=d, byrow=TRUE)
  Thetas   <- matrix(c(NA, GHcop_para,  FRcop_para,
                       NA,         NA,          NA,
                       NA,         NA,          NA), nrow=d, ncol=d, byrow=TRUE)
  Breves   <- matrix(c(NA,          0,           0,
                       NA,         NA,           0,
                       NA,         NA,          NA), nrow=d, ncol=d, byrow=TRUE)
  Reflects <- matrix(c(NA,          1,           1,
                       NA,         NA,           1,
                       NA,         NA,          NA), nrow=d, ncol=d, byrow=TRUE)
  # see copBasic::breveCOP(), copBasic::GHcop(), copBasic::M()
  # see also copBasic::COP() for how reflection work within the package
  set.seed(1); U <- NULL # seed and vector of the U_{(1,2,3)} probabilities
  for(i in 1:nsim) {
    w <- runif(d); u <- rep(NA, d); u[1] <- w[1]
    # looks messy but just a way dump a host of "parameters" including family
    # itself down into copBasic logic
    para <- list(cop=breveCOP, para=list(cop=eval(parse(text=Cops[1,2])),
            para=Thetas[1,2], breve=Breves[1,2], reflect=Reflects[1,2]))
    u[2] <- derCOPinv(u[1], t=w[2], cop=COP, para=para) # conditional quantiles
    for(j in 3:d) {
      q <- w[j]
      for(l in (j-1):1) { # Joe (2014, algorithm 16)
        para <- list(cop=breveCOP, para=list(cop=eval(parse(text=Cops[l,j])),
                para=Thetas[l,j], breve=Breves[l,j], reflect=Reflects[l,j]))
        q <- derCOPinv(w[l], t=q, cop=COP, para=para)   # conditional quantiles
      }
      u[j] <- q
    }
    U <- rbind(U, matrix(u, ncol=d, byrow=TRUE))
  }
  U <- as.data.frame( U )
  names( U ) <- paste0("U", seq_len(d))

  # Kendall Partial Tau; Joe (2014, p. 8.66, Theorem 8.66)
  "pc3dCOP" <- function(taujk, tauhj, tauhk) { # close attention to h subscript!
    etajk <- sin( pi*taujk / 2) # Expansion to trig by Joe (2014, theorem 8.19),
    etahj <- sin( pi*tauhj / 2) # Joe says this definition "might be"
    etahk <- sin( pi*tauhk / 2) # better than partial tau on scores.
    (2/pi) * asin( (etajk - etahj*etahk) / sqrt( (1-etahj^2) * (1-etahk^2) ) )
  }
  # Joe (2014, pp. 404-405)
  # Kendall partial tau but needs pairwise Kendall taus to compute partial tau.
  "PairwiseTaus" <- function(U) {
    ktau <- matrix(NA, nrow=d, ncol=d)
    for(j in 1:d) { for(l in 1:d) {
        if(j <  l) next
        if(j == l) { ktau[l,j] <- 1; next } # 1s on diagonal as required.
        ktau[l,j] <- ktau[j,l] <- cor(U[,l], U[,j], method="kendall") # symmetrical
    } }
    return(ktau)
  }
  PairwiseTaus  <- PairwiseTaus(U) # [1,2] and [1,3] are 0.5 and 0.7 matching
  # requirement and more importantly for Joe (2014, table 8.3, p. 405). Now, [2,3] is
  # about 0.783 which again matches Joe's results of 0.782.
  Tau23given1pc <- pc3dCOP(PairwiseTaus[2,3], PairwiseTaus[2,1], PairwiseTaus[3,1])
  # Joe (2014, table 8.3, p. 405) reports tau^{pc}_{jk;h} as 0.848 by giant
  # simulation and we get about 0.852199 for some modest nsim and set.seed(1). }
}
\keyword{Frank copula}
\keyword{copula (formulas)}
\keyword{copula}
\concept{Frank copula}
\keyword{Joe (2014) Examples and Exercises}
\keyword{Debye function}
\keyword{C-vine copula}
\keyword{Vine Copula}
\keyword{copula (comprehensive)}
\concept{comprehensive copula}
\concept{C-vine}
\concept{C-vines}

\encoding{utf8}
\name{HAIRPINcop}
\alias{HAIRPINcop}
\title{The Hairpin Copula}
\description{
The \emph{Hairpin copula} (Durante \emph{et al.}, 2014) is

\deqn{\mathbf{C}_{\Theta}(u,v) = \mathbf{HAIRPIN}(u,v) =
\mathrm{min}\bigl[u, v, \mathrm{mean}\bigl(f(u,v) \bigr) \bigr]\mbox{,}}

as vectorized for the pairs \eqn{(u_i, v_i)} for \eqn{i \in 1,2,\cdots}, and \eqn{f(x) = \alpha x^\phi}, \eqn{1 \le \alpha \le \infty}, and \eqn{1 \le \phi \le 2}. The copula, as \eqn{\alpha \rightarrow \infty} or \eqn{\phi \rightarrow 1} limits, to the \emph{comonotonicity coupla} (\eqn{\mathbf{M}(u,v)}; \code{\link{M}}).
}
\usage{
HAIRPINcop(u, v, para=c(1, 2, 1), ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (three element) of two parameters and the reflection: \eqn{\alpha} and \eqn{\phi} of the coupla then the reflection number \eqn{n \in 1, 2, 3, 4} following the reflection pattern of \code{\link{COP}}; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\references{
Durante, F., \enc{Fernández-Sánchez}{Fernandez-Sanchez}, J., and Trutschnig, W., 2014, Multivariate copulas with hairpin support: Journal of Multivariate Analysis, v. 130, pp. 323--334, \doi{10.1016/j.jmva.2014.06.009}.}
\author{W.H. Asquith}
\seealso{\code{\link{M}}
}
\examples{
\dontrun{
  # Produce a Celtic Knot look through simulation -----------------------------
  nsim <- 1.5E4
  plot(c(0, 1), c(0, 1), type="n", bty="n",
       xaxt="n", yaxt="n", xaxs="r", yaxs="r", xlab="", ylab="")
  sapply(1:4, function(n) {
                  UV <- simCOP(nsim, cop=HAIRPINcop, para=c(1.12, 1.8, n),
                 seed=1, ploton=FALSE, pch=16, cex=0.6, col=n); return(n) })
  sapply(1:4, function(n) {
                  UV <- simCOP(nsim, cop=HAIRPINcop, para=c(1.12, 1.3, n),
                 seed=1, ploton=FALSE, pch=16, cex=0.6, col=n); return(n) }) # }
}
\keyword{Hairpin copula}
\keyword{copula (formulas)}
\keyword{copula}

\encoding{utf8}
\name{isCOP.RTI}
\alias{isCOP.RTI}
\title{Is a Copula Right-Tail Increasing}
\description{
Numerically set a logical whether a copula is \emph{right-tail increasing} (RTI) as described by Nelsen (2006, pp. 192--193) and Salvadori \emph{et al.} (2007, p. 222). A copula \eqn{\mathbf{C}(u,v)} is right-tail decreasing for \eqn{\mathrm{RTI}(V{\mid}U)} if and only if for any \eqn{v \in [0,1]},
\deqn{\frac{\delta \mathbf{C}(u,v)}{\delta u} \le \frac{v - \mathbf{C}(u,v)}{1 - u}}
for almost all \eqn{u \in [0,1]}. Similarly, a copula \eqn{\mathbf{C}(u,v)} is right-tail decreasing for \eqn{\mathrm{RTI}(U{\mid}V)} if and only if for any \eqn{u \in [0,1]},
\deqn{\frac{\delta \mathbf{C}(u,v)}{\delta v} \le \frac{u - \mathbf{C}(u,v)}{1 - v}}
for almost all \eqn{v \in [0,1]} where the later definition is controlled by the \code{wrtV=TRUE} argument.

The RTI concept is associated with the concept of \emph{tail monotonicity} (Nelsen, 2006, p. 191). Specifically, but reference to Nelsen (2006) definitions and geometric interpretations is recommended, \eqn{\mathrm{RTI}(V{\mid}U)} (or \eqn{\mathrm{RTI}(V{\mid}U)}) means that the probability \eqn{P[Y > y \mid X > x]} (or \eqn{P[X > x \mid Y > y]}) is a nondecreasing function of \eqn{x} (or \eqn{y}) for all \eqn{y} (or \eqn{x}).

A positive RTI of either \eqn{\mathrm{RTI}(V{\mid}U)} or \eqn{\mathrm{RTI}(U{\mid}V)} implies positively quadrant dependency (PQD, \code{\link{isCOP.PQD}}) but the condition of PQD does not imply RTI. Finally, the accuracy of the numerical assessment of the returned logical by \code{isCOP.RTI} is dependent on the the smallness of the \code{delta} argument passed into the function.
}
\usage{
isCOP.RTI(cop=NULL, para=NULL, wrtV=FALSE, delta=0.005, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters, if needed, to pass to the copula;}
  \item{wrtV}{A logical to toggle between with respect to \eqn{v} or \eqn{u} (default);}
  \item{delta}{The increment of  \eqn{\{u,v\} \mapsto [0+\Delta\delta, 1-\Delta\delta, \Delta\delta]} set by \code{wrtV}; and}
  \item{...}{Additional arguments to pass to the copula or derivative of a copula function.}
}
\value{
  A logical \code{TRUE} or \code{FALSE} is returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in nature---An approach using copulas: Dordrecht, Netherlands, Springer, Water Science and Technology Library 56, 292 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{isCOP.LTD}}, \code{\link{isCOP.PQD}}}
\examples{
\dontrun{
isCOP.RTI(cop=P, delta=0.01) # independence should be FALSE
# but function returns TRUE. Note, same logic for isCOP.LTD returns FALSE.
isCOP.RTI(cop=PSP)                              # TRUE  : positive assoc.
isCOP.RTI(cop=PLACKETTcop, para=.15)            # FALSE : negative assoc. Plackett
isCOP.RTI(cop=PLACKETTcop, para=15)             # TRUE  : positive assoc. Plackett
isCOP.RTI(cop=PLACKETTcop, wrtv=TRUE, para=.15) # FALSE : negative assoc. Plackett
isCOP.RTI(cop=PLACKETTcop, wrtV=TRUE, para=15)  # TRUE  : positive assoc. Plackett}
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}

\encoding{utf8}
\name{wolfCOPtest_data_smlsam}
\docType{data}
\alias{wolfCOPtest_data_smlsam}
\title{Small Sample Components for Schweizer--Wolff Sigma Test}
\description{
Data for the \emph{ test} components for small samples. Empirical sampling distributions of \emph{Schweizer--Wolff Sigma} (\eqn{\hat\sigma_\mathbf{C}}, \code{\link{wolfCOP}}) for \eqn{n \in 3, \cdots, 40} of the \eqn{\mathrm{NULL} : \mathbf{C} = \mathbf{\Pi}} (\emph{Independence copula}, \code{\link{P}}), where  \eqn{\sigma_\mathbf{C}} is defined under \code{\link{wolfCOP}}, using simulation sizes of 900,000 for each \eqn{n}. So, total simulation size was 34,200,000. Continuing, this data set is a concatenated table of empirical distributions formed from the unique number of \eqn{\hat\sigma_\mathbf{C}} (discrete variables in small samples), which large-simulation attainable up to \eqn{\pm4.26} standard deviations, were computed and tabulated in the \code{wolfCOPtest_data_smlsam} table. Algorithms for the small-sample components are located at \code{inst/}\code{make_wolfCOPtest/}\code{smlsam/}\code{make_}\code{wolfCOPtestP_}\code{smlsam.R}, and this script is the origin of the \eqn{n = 40} in \code{\link{wolfCOPtest}} source code.

For the smallest of sample sizes, the \emph{empirical distribution} is a step function, and for each plateau, a nudge using \code{.Machine$double.eps} has been used to make \code{approx()} for sensible in \code{\link{wolfCOPtest}}. In the \bold{Examples} herein, the jaggedness of the empirical distribution can readily be seen and visually smoothing by about \eqn{n > 10}. Finally, the \code{\link{wolfCOPtest}} function internally loads the \code{wolfCOPtest_data_smlsam} tables, and most users will not need to access this dataset themselves unless for curiosity.
}
\usage{
data(wolfCOPtest_data_smlsam)
}
\format{
An \R \code{data.frame} titled \code{wolfCOPtest_data_smlsam} with
\describe{
  \item{n}{Sample size;}
  \item{probs}{Nonexceedance probability of empirical distribution of \eqn{\hat\sigma_\mathbf{C}} (\code{\link{wolfCOP}}). Note, \code{quantile(..., type=4)} was used for computations because of structural similarity to sample \eqn{\hat\sigma_\mathbf{C}} as implemented in \pkg{copBasic};}
  \item{wolfemp}{Value for \eqn{\hat\sigma_\mathbf{C}} (\code{\link{wolfCOP}});}
}
}
\seealso{\code{\link{wolfCOPtest}}}
\examples{
data(wolfCOPtest_data_smlsam)

\dontrun{
  # Demonstrate the empirical distributions of Schweizer-Wolff against a parametric
  # distribution model. We see quite quickly that the distribution model mimics
  # the empirical distribution at quite small sample sizes.
  data("wolfCOPtest_data_smlsam")
  ff <- pnorm(seq(-3, 3, by=0.1))
  plot(c(-3, 3), c(0,1), type="n", xlab="Standard normal variate",
       ylab="Schweizer-Wolff Sigma given C(u,v)=PI")
  ns <- c(3, 5, 10, 20, 40)
  for(n in ns) {
    sata <- wolfCOPtest_data_smlsam[wolfCOPtest_data_smlsam$n == n,]
    lines(qnorm(sata$probs), sata$wolfemp, col="red", lwd=2)
  }
  ns <- c(3, 4, 5, 7, 10, 15, 20, 30, 40, 50, 70, 100, 130, 200, 500, 1000, 5000)
  for(n in ns) {
    wt   <- wolfCOPtest(0, n) # next isolate the distribution type
    # If copy and paste from R help, then one needs to add backslashes to avoid
    # unrecognized escape errors from R in the next line of code.
    type <- gsub("\\)$", "", gsub("p.value\\(dist_", "", names(wt$p.value)[1]))
    para <- lmomco::vec2par(wt$distpara_by_lmoms, type=type) # type == pe3
    qq   <- 1/(1 + exp(-lmomco::qlmomco(ff, para)))
    lines(qnorm(ff), qq, col="blue")
    text(0, 1/(1 + exp(-lmomco::qlmomco(0.5, para))), n, cex=0.8, font=2)
  }
  par(lheight=0.8)
  txt <- c(paste0(lmomco::prettydist(type), " model by estimated L-moments\n",
                  "of Sigma distribution for select sample sizes"),
           paste0("Empirical distribution for select sample sizes\n",
                  "in wolfCOPtest_data_smlsam data object"))
  legend("topleft", txt, box.lty=0, inset=0.01, cex=0.8, adj=c(0, 0.8),
         y.intersp=1.6, lwd=c(1,2), col=c("blue", "red"), bg=NA)
  par(lheight=1) #}

\dontrun{
  W <- seq(0, 1, by=0.01) # Span all Schweizer-Wolff Sigma, when the first and second
  # arguments to wolfCOPtest are scalars, these are treated as sigma and sample size.
  for(i in 3:40) { # samples sizes 3 to 40 in wolfCOPtest_data_smlsam.
    Pd <- sapply(W, function(w) wolfCOPtest(w, i, usepade=FALSE)$p.value[1])
    Pe <- sapply(W, function(w) wolfCOPtest(w, i, usepade=FALSE)$p.value[2])
    Pp <- sapply(W, function(w) wolfCOPtest(w, i, usepade=TRUE )$p.value[1])
    # p-value by the generalized normal (parametric) distribution
    plot( W, Pd, type="l", col="grey70", lty=2, lwd=7, main=paste0("n=", i),
         xlab="Schweizer-Wolff Sigma", ylab="Exceedance probability (p-value)")
    points(W, Pp, col="skyblue2", lwd=1) # large sample p-values
    # p-value derived from wolfCOP_data_smlsam tables:
    lines(W, Pe, col="green4",    lwd=4) # small sample p-values
  } #}
}
\keyword{datasets}
\keyword{Schweizer and Wolff Sigma}
\keyword{Pearson type III distribution}
\keyword{PE3 (see Pearson type III distribution)}
\keyword{logit transform}

\encoding{utf8}
\name{tEVcop}
\alias{tEVcop}
\title{The t-EV (Extreme Value) Copula}
\description{
The \emph{t-EV copula} (Joe, 2014, p. 189) is a limiting form of the \emph{t-Student copula} (multivariate t-distribution) (\code{\link{Tcop}}):
\deqn{
\mathbf{C}_{\rho,\nu}(u,v) = \mathbf{tEV}(u,v; \rho, \nu) =
\mathrm{exp}\bigl(-(x+y) \times B(x/(x+y); \rho, \nu)\bigr)\mbox{,}
}
where \eqn{x = -\log(u)}, \eqn{y = -\log(v)}, and letting \eqn{\eta = \sqrt{(\nu+1)/(1-\rho^2)}} define
\deqn{
B(w; \rho, \nu) = w \times t_{\nu+1}\bigl(\eta[(w/[1-w])^{1/\nu}-\rho]\bigr) + (1-w) \times t_{\nu+1}\bigl(\eta[([1-w]/w)^{1/\nu}-\rho]\bigr)\mbox{,}
}
where \eqn{t_{\nu+1}} is the cumulative distribution function of the \emph{univariate t-distribution} with \eqn{\nu+1} degrees of freedom. As \eqn{\nu \rightarrow \infty}, the copula weakly converges to the \emph{\enc{Hüsler}{Husler}--Reiss copula} (\code{\link{HRcop}}) because the t-distribution converges to the normal (refer to \bold{Examples} for a study of this copula).

The \eqn{\mathbf{tEV}(u,v; \rho, \nu)} copula is a two-parameter option when working with extreme-value copula. There is a caveat though. Demarta and McNeil (2004) conclude that \dQuote{the parameter of the \emph{Gumbel--Hougaard} (\code{\link{GHcop}}) or \emph{Galambos} (\code{\link{GLcop}}) A-functions [the \emph{Pickend dependence function} and B-function by association] can always be chosen so that the curve is extremely close to that of the t-EV A-function for any values of \eqn{\nu} and \eqn{\rho}. The implication is that in all situations where the t-EV copula might be deemed an appropriate model then the practitioner can work instead with the simpler Gumbel or Galambos copulas.}
}
\usage{
tEVcop(u, v, para=NULL, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (two element) of parameters in \eqn{\rho} and \eqn{\nu} order; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\note{
Note, Joe (2014) shows \eqn{x = \log(u)} (note absence of the minus sign)---this is not correct.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

Demarta, S., and McNeil, A.J., 2004, The t copula and related copulas: International Statistical Review, v. 33, no. 1, pp. 111--129, \doi{10.1111/j.1751-5823.2005.tb00254.x}
}
\author{W.H. Asquith}
\seealso{\code{\link{GHcop}}, \code{\link{GLcop}}, \code{\link{HRcop}}, \code{\link{Tcop}}
}
\examples{
\dontrun{
  tau <- 1/3 # Example from copula::evCopula.Rd
  tev.cop <- copula::tevCopula(copula::iTau(copula::tevCopula(), tau))
  copula::pCopula(c(0.1,.5), copula=tev.cop)         # 0.07811367
  tEVcop(0.1, 0.5, para=slot(tev.cop, "parameters")) # 0.07811367 }

\dontrun{
  nsim <- 2000; pargh <- c(5, 0.5, 0.5)
  UV <- simCOP(nsim, cop=GHcop, para=pargh)
  U <- lmomco::pp(UV[,1], sort=FALSE)
  V <- lmomco::pp(UV[,2], sort=FALSE)
  RT <- mleCOP(u=U, v=V, cop=tEVcop, init.para=c(0.5, log(4)),
               parafn=function(k) return( c(k[1], exp(k[2])) ) )
  partev <- RT$para

  FT <- simCOP(nsim, cop=tEVcop, para=RT$para)

  tauCOP(cop=GHcop,  para=pargh )
  tauCOP(cop=tEVcop, para=partev)

  tauCOP(cop=GHcop,  para=pargh ) # [1] 0.3003678
  tauCOP(cop=tEVcop, para=partev) # [1] 0.3178904

  densityCOPplot(cop=GHcop,  para=pargh)
  densityCOPplot(cop=tEVcop, para=partev, ploton=FALSE, contour.col="red") #}

\dontrun{
  # A demonstration Joe (2014, p. 190) for which tEvcop() has
  # upper tail dependence parameter as
  para <- c(0.8, 10)
  lamU <- 2 * pt( -sqrt( (para[2]+1) * (1-para[1]) / (1+para[1]) ), para[2]+1)
  "tEVcop.copula" <- function(u,v, para=NULL, ...) {
        if(length(u) == 1) u <- rep(u,length(v)); if(length(v)==1) v <- rep(v,length(u))
        return(copula::pCopula(matrix(c(u,v), ncol=2),
               copula::tevCopula(param=para[1], df=para[2])))
  }
  lamU.copBasic <- taildepCOP(cop=tEVcop,        para)$lambdaU
  lamU.copula   <- taildepCOP(cop=tEVcop.copula, para)$lambdaU
  print(c(lamU, lamU.copBasic, lamU.copula))
  #[1] 0.2925185 0.2925200 0.2925200 # So, we see that they all match.}

\dontrun{
  # Convergence of tEVcop to HRcop as nu goes to infinity.
  nu <- 10^(seq(-4, 2, by=0.1)) # nu right end point rho dependent
  rho <- 0.7 # otherwise, expect to see 'zeros' errors on the plot()
  # Compute Blomqvist Beta (fast computation is reason for choice)
  btEV <- sapply(nu, function(n) blomCOP(tEVcop, para=c(rho, n)))
  limit.thetas <- sqrt(2 / (nu*(1-rho))) # for nu --> infinity HRcop
  thetas <- sapply(btEV, function(b) {
       uniroot(function(l, blom=NA) { blom - blomCOP(HRcop, para=l) },
       interval=c(0,10), blom=b)$root })
  plot(limit.thetas, thetas, log="xy", type="b",
       xlab="Theta of HRcop via limit nu --> infinity",
       ylab="Theta from Blomqvist Beta equivalent HRcop to tEVcop")
  abline(0,1)
  mtext(paste0("Notice the 'weak' convergence to lower left, and \n",
               "convergence increasing with rho"))
  # Another reference of note
  # https://mediatum.ub.tum.de/doc/1145695/1145695.pdf (p.39) #}
}
\keyword{t-EV copula}
\keyword{copula (formulas)}
\keyword{copula}
\keyword{package copula (comparison to)}
\concept{t-EV copula}
\keyword{copula (extreme value)}
\keyword{extreme value copula}
\keyword{Joe (2014) Examples and Exercises}
\keyword{Student t}

\encoding{utf8}
\name{wolfCOPsamc}
\alias{wolfCOPsamc}
\title{Monte Carlo Integration of Schweizer--Wolff Sigma}
\description{
Perform \emph{Monte Carlo integration} for the measure of dependence known as \emph{Schweizer--Wolff Sigma} \eqn{\sigma_\mathbf{C} \in (0,1)} of an \emph{empirical copula} (\eqn{\mathbf{C}_n(u,v)}). This function can be useful as the \code{as.sample} computation provided by \code{\link{wolfCOP}} scales very poorly as sample size increases. Please refer to \code{\link{wolfCOP}} for more details, including theory, about \eqn{\sigma_\mathbf{C}}. Sample estimation can be made by

\deqn{\hat\sigma_\mathbf{C} = 12\int\!\!\int_{\mathcal{I}^2} \bigl|\mathbf{C}_n(u,v) - uv\bigr|\,\mathrm{d}u\mathrm{d}v\mbox{,}}

for which the integrals in the above formula are replaced with Monte Carlo integration provided by this function. Allowance is made internally to swap out \eqn{\mathbf{C}_n(u,v)} for a copula family \eqn{\mathbf{C}(u,v)}, which means this function can be used for integration for families that might have numerical integration convergence problems in \code{\link{wolfCOP}}.
}
\usage{
wolfCOPsamc(u=NULL, v=NULL, cop=EMPIRcop, para=NULL, para_has_paras=FALSE,
            tol=0.005, minit=10, maxit=100, subdivisions=200, large_n=5E3,
            includelmoms=FALSE, trace=FALSE, verbose=FALSE, ...)
}
\arguments{
  \item{u}{Optional nonexceedance probability \eqn{u} in the \eqn{X} direction. This argument can also by a two-column data frame from which the \code{para} argument to \code{\link{EMPIRcop}} when \code{cop=EMPIRcop}) is internally constructed;}
  \item{v}{Optional nonexceedance probability \eqn{v} in the \eqn{Y} direction from which the \code{para} argument to \code{\link{EMPIRcop}} when \code{cop=EMPIRcop} is internally constructed;}
  \item{cop}{The empirical copula, \code{\link{EMPIRcop}}, or a copula family;}
  \item{para}{Optional parameter content for the \code{para} argument to the specified copula in \code{cop}. For an empirical copula application (default), the \code{para} must be a two-column data frame, but if \code{cop} is set to a copula family, then this function can be used as an alternative \eqn{\sigma_\mathbf{C}} computation using Monte Carlo integration in lieu of numerical integration exclusive to \code{\link{wolfCOP}} (refer to \bold{Examples} how to use the function in this way);}
  \item{para_has_paras}{A logical to inform processing whether the \code{para} argument contains parameters for a copula family in lieu of \code{para} containing the sample for the empirical copula (\code{cop=EMPIRcop}). Internally, \code{para_has_paras} is used to disable random subsetting by setting the sample size variable to \code{large_n}. Technically, it is possible internally to detection whether \code{para_has_paras} needed to be true (even if false) as the number of random variates for the \code{halton()} function of the \pkg{randtoolbox} package is computed, and the code will silently proceed as if \code{para_has_paras=TRUE};}
  \item{tol}{A tolerance in absolute percent change in the global mean as the iterations progress. This tolerance should be good enough for many practical large sample applications, but can be reduced considerable if the function is used as the backdoor to integration of a given copula family instead of a sample estimator;}
  \item{minit}{Minimum number of iterations regardless of \code{tol} setting;}
  \item{maxit}{Maximum number of iterations;}
  \item{subdivisions}{Number of subdivisions and somewhat an analog to the term used in numerical integration. This subdivisions will be the size of the Monte Carlo sample for each iteration;}
  \item{large_n}{For an empirical copula and as the sample size gets large, not only does the \eqn{\sigma_\mathbf{C}} computation slow down because a double summation is involved, but the counting that empirical copula must also perform gets large, therefore, the \code{para} that will be ultimately passed to the empirical copula is randomly subsetted by \code{large_n}. This is done in order to speed up iteration times at expense of increasing time that \code{tol} presumably is reached;}
  \item{includelmoms}{A logical to include upto the first five linear moments for \eqn{\sigma_\mathbf{C}} and logit-transformed \eqn{\mathrm{log}[\sigma_\mathbf{C}/(1-\sigma_\mathbf{C})]} in the returned list.}
  \item{trace}{A logical, if true, that will produce a scatter plot of the sample (or the current iteration subset [refer to \code{large_n}]) along with the random sample points used for the current Monte Carlo iteration;}
  \item{verbose}{A logical providing percent change results by iteration to the console; and}
  \item{...}{Additional arguments to pass to the \code{halton()} function of the \pkg{randtoolbox} and the copula specified by \code{cop} (\code{\link{EMPIRcop}}) or a copula family.}
}
\value{
  A list of the Monte Carlo integration results is returned containing
\item{wolves}{A vector of the \eqn{\sigma_\mathbf{C}} by iteration;}
\item{estimates}{The minimum, mean (the formal estimate \eqn{\hat\sigma_\mathbf{C}}), maximum along with optional variance (computed from second L-moment), L-scale (second L-moment), L-skew, and L-kurtosis of the \code{wolves};}
\item{estimates}{Using logit-transformation of the \code{wolves}, the minimum, mean, maximum along with optional variance (computed from second L-moment), L-scale (second L-moment), L-skew, and L-kurtosis of the \code{wolves};}
\item{its}{Number of iterations before termination by \code{maxits} or upon reaching \code{tol};}
\item{lastpctchg}{The percent change on the last iteration.}
}
\note{
Beyond about 1,000 sample sizes \code{wolfCOP(as.sample=TRUE)} scales in time poorly, so we see in the following recipe the giant increase in speed at a small cost in precision on the Schweizer--Wolff Sigma estimate.
\preformatted{
  UV       <- simCOP(3000, cop=CLcop, para=3, graphics=FALSE, seed=473)
  theowolf <- wolfCOP(     cop=CLcop, para=3) # 0.7864391
  system.time( sfwolf <- wolfCOP(    para=UV, as.sample=TRUE,
                                          nlarge=1, usefastgrid=TRUE ) )
  system.time( smwolf <- wolfCOP(    para=UV, as.sample=TRUE,
                                          nlarge=1, usefastgrid=FALSE) )
  system.time( mcwolf <- wolfCOPsamc(para=UV,   usetime=TRUE) )
  #   user  system elapsed   WOLF(sample fast) = 0.8002883
  #  0.600   0.017   0.616
  #   user  system elapsed   WOLF(sample slow) = 0.8002883
  # 188.072  25.594 213.632
  #   user  system elapsed   WOLF(monte carlo) = 0.799808
  #  12.729   1.449  14.191
}


If one wanted to make some comparisons of computed results between \code{\link{wolfCOP}} and \code{wolfCOPsamc}, the following recipe, with a suitably large sample size, that we see a general spread around the equal value line. The use of Monte Carlo integration approach is not advised when computational speed of \code{\link{wolfCOP}} is sufficient.  This \code{wolfCOPsamc} function is potentially of less usefulness than suggested because of the \code{\link{EMPIRgrid_fast}} within \code{\link{wolfCOP}} for rapid estimation by a gridded empirical copula.
\preformatted{
  nsam <- 1000; nsim <- 20; wolf <- mcwolf <- rep(NA, length=nsim)
  for(i in seq_len(nsim)) {
    uv <- as.data.frame(matrix(runif(nsam*2), ncol=2))
      wolf[i] <- wolfCOP(    para=uv, as.sample=TRUE)
    mcwolf[i] <- wolfCOPsamc(para=uv)$estimates[2]
    print(c(i, wolf[i], mcwolf[i]))
  }
  df <- data.frame(sigma=wolf, sigma_mc=mcwolf)
  df <- df[complete.cases(df),]
  xylim <- range( c(range(df$sigma, na.rm=TRUE), range(df$sigma_mc)) )
  plot(df$sigma, df$sigma_mc, xlim=xylim, ylim=xylim,
       xlab="Schweizer-Wolff sigma by whole-sample counting",
       ylab="Schweizer-Wolff sigma by Monte Carlo integration")
  lines(par()$usr[1:2], par()$usr[3:4]) # equal value line }
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Schweizer, B., and Wolff, E.F., 1981, On nonparametric measures of dependence for random variables: The Annuals of Statistics, v. 9, no. 4, pp. 879--855, \url{https://www.jstor.org/stable/2240856}.
}
\author{W.H. Asquith}
\seealso{\code{\link{wolfCOP}}, \code{\link{EMPIRcop}}
}
\examples{
n <- 40; para=c(1.3, 2.4)              # Small sample, for speed.
UV <- simCOP(40, para=para, cop=GHcop, seed=1) # Simulate a sample
wolfCOP(    para=para, cop=GHcop)      # Theo. value + next 2 lines
wolfCOPsamc(para=para, cop=GHcop)$estimate[2] # Same as next line
wolfCOPsamc(para=para, cop=GHcop, para_has_paras=TRUE)$estimate[2]
wolfCOP(    para=UV, as.sample=TRUE, usefastgrid=TRUE, nlarge=1) # Spring 2026
wolfCOP(    para=UV, as.sample=TRUE, usefastgrid=FALSE) # original pkg implementation
wolfCOPsamc(para=UV)$estimate[2]       # Monte Carlo estimate

\dontrun{
  # Test the trace and verbose arguments
  para <- list(cop=c(P, M), part=c(0,0.5,1))
  UV   <- simCOP(1000, cop=ORDSUWcop, para=para)
  SW   <- wolfCOPsamc(para=UV, tol=0.0005, trace=TRUE, verbose=TRUE) #}

\dontrun{
  # Test the various interfaces for a sample computation along with
  # setting the usetime to the random number subsystem.
  UV <- simCOP(1000, cop=CLcop, para=3, graphics=FALSE)
  mc1 <- wolfCOPsamc(para=UV, usetime=TRUE)$estimates[2]
  mc2 <- wolfCOPsamc(u=UV,    usetime=TRUE)$estimates[2]
  mc3 <- wolfCOPsamc(u=UV[,1], v=UV[,2], usetime=TRUE)$estimates[2] #}

\dontrun{
  # Let us check on the veracity of wolfCOPsamc() by a cluster-based computation
  # of realization of mean values of Lmoments of logit-transformed Sigmas for a
  # given sample size against the projected Lmoments (Lskew and Lkurtosis) in
  # regressions implemented within wolfCOPtest(). We will need the ellipse and
  # parallel packages already installed to the user's machine for this test.
  nsam <- 40 # Sample size (not related to simulation settings in the example.)
  lmomco::plotlmrdia(lmomco::lmrdia(), empty=TRUE, # Lmoment ratio diagram
            xlim=c(0.1, 0.35), ylim=c(0.1, 0.20), las=1, xaxs="i", yaxs="i")
  lmomco::plotlmrdia(lmomco::lmrdia(usrtrim=TRUE), add=TRUE, expand.names=TRUE,
             autolegend=TRUE, xleg="topleft", ncol=1, nolimits=TRUE,
             nogov=TRUE, nogpa=TRUE, noexp=TRUE,  nonor=TRUE, nogum =TRUE,
             noray=TRUE, nouni=TRUE, noaep4=TRUE, noglo=TRUE, nopdq3=TRUE)
  xy <- NULL # coordinates of the Lskew (T3) and Lkurtosis(T4) of logit-Sigma
  for(n in c(3:99, 10^seq(2, 6, by=0.1))) { # a vector of sample sizes
    wt <- wolfCOPtest(0, n)$lmoms_logit_sigma # Lmoments of logit-Sigma
    xy <- rbind(xy, data.frame(tau3=wt[3], tau4=wt[4])) # isolate the T3 and T4
  }
  lines( xy[,1], xy[,2], col="wheat3", lwd=4) # plot the estimation of T3 and T4
  wt <- wolfCOPtest(0, nsam)$lmoms_logit_sigma # isolate the prediction for the
  points(wt[3], wt[4], pch=16) # sample size that we are studying
  cols <- c("salmon4", "salmon1", "seagreen4", "seagreen1")
  # All the code below can be run with on setting of doMC false and then true.
  doMC <- FALSE # set to true for wolfCOPsamc() else wolfCOP(as.sample=TRUE)
  LMR  <- NULL; ncore <- 8; nloop <- 40; nsim <- 500 # last of initializations
  # The wolfCOPsamc() has long running time but by about 500-1000 samples it
  # likely will then begin outpacing the wolfCOP(). We have the nsam default as
  # small for this example and nsim enough for definitive stability in the results
  # and nloop large enough for an authoritative ellipse to be drawn.
  if(nloop < 4) stop("Need at least four for Lmoment computation later")
  for(j in seq_len(nloop)) {
    message(j, "-", appendLF=FALSE)
    if(length(grep("[02468]0$", as.character(j))) > 0) message("", appendLF=TRUE)
    cl <- parallel::makeCluster(getOption("cl.cores", ncore)) # make CPU cluster
    parallel::clusterExport( cl, "nsam") # make "nsam" visible in cluster FUN
    if(doMC) { # set tolerance "up" just for some speed increase for the example
      wolves <- parallel::parSapply(cl=cl, seq_len(nsim), FUN=function(i) {
           wolfCOPsamc(para=as.data.frame(matrix(runif(nsam*2),
                           ncol=2)), tol=0.01, usetime=TRUE)$estimates[2] })
    } else {
      wolves <- parallel::parSapply(cl=cl, seq_len(nsim), FUN=function(i) {
           wolfCOP(    para=as.data.frame(matrix(runif(nsam*2),
                           ncol=2)), as.sample=TRUE)                      })
    }
    parallel::stopCluster(cl) # stop the cluster as it will be restarted as needed
    wolves <- log( wolves / (1 - wolves) ) # make logit-Sigmas
    lmr    <- Lmoments::Lmoments(wolves)   # faster than lmomco::lmoms()
    lmr[3] <- lmr[3] / lmr[2]; lmr[4] <- lmr[4] / lmr[2] # Lskew and Lkurtosis
    LMR <- rbind(LMR, data.frame(L1=lmr[1], L2=lmr[2], T3=lmr[3], T4=lmr[4]))
    if((par()$usr[1] <= lmr[3] & lmr[3] <= par()$usr[2]) &
       (par()$usr[3] <= lmr[4] & lmr[4] <= par()$usr[4])) {
      if(doMC) {
        points(lmr[3], lmr[4], col=cols[1], bg=cols[2], pch=24, lwd=0.9, cex=0.8)
      } else {
        points(lmr[3], lmr[4], col=cols[3], bg=cols[4], pch=25, lwd=0.9, cex=0.8)
      }
    }
  }
  # Lskew and Lkurtosis are approximately multivariate normally distributed.
  Trho <- stats::cor( LMR$T3,  LMR$T4 ) # Pearson correlation coefficient
  Tscl <- c(Lmoments::Lmoments(LMR$T3)[2] * sqrt(pi), # standard deviations in U
            Lmoments::Lmoments(LMR$T4)[2] * sqrt(pi)) # standard deviations in V
  Tctr <- c(mean(LMR$T3), mean(LMR$T4))    # ellipse center
  Telp <- ellipse::ellipse(Trho, scale=Tscl, centre=Tctr, npoints=200, level=0.95)
  if(doMC) {
    lines(Telp, col=cols[1], lty=4, lwd=2) # ellipse itself
    points(mean(LMR$T3), mean(LMR$T4), pch=21, cex=2, col=cols[3], bg=cols[2])
  } else {
    lines(Telp, col=cols[3], lty=4, lwd=2) # ellipse itself
    points(mean(LMR$T3), mean(LMR$T4), pch=21, cex=2, col=cols[3], bg=cols[4])
  }
  points(wt[3], wt[4], pch=16) # sample size that we are studying # }
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{Monte Carlo integration}
\keyword{Schweizer and Wolff Sigma}
\keyword{Package randtoolbox}
\keyword{Package ellipse}
\keyword{Package parallel}
\keyword{Package Lmoments}

\encoding{utf8}
\name{taildepCOP}
\alias{taildepCOP}
\title{The Lower- and Upper-Tail Dependency Parameters of a Copula}
\description{
Compute the \emph{lower-} and \emph{upper-tail dependency parameters} (if they exist), respectively, of a copula according to Nelsen (2006, pp. 214--215). Graphical confirmation of the computations is important, and therefore, the function can also generate a plot. The dependency parameters are expressions of conditional probability that \eqn{Y} is greater than the \eqn{100{\times}}th percentile of its distribution \eqn{G} given that \eqn{X} is greater than the \eqn{100{\times}t}-th percentile of its distribution \eqn{F} as \eqn{t} approaches unity. Specifics in terms of quantile functions \eqn{G^{(-1)}(t) = y(t)} and \eqn{F^{(-1)}(t) = x(t)} follow.

The \emph{lower-tail dependence parameter} \eqn{\lambda^L_\mathbf{C}} is defined as
\deqn{\lambda^L_\mathbf{C} = \lim_{t{\rightarrow 0^{+}}} \mathrm{Pr}[Y \le y(t)\mid X \le x(t)]\mbox{, and}}
the \emph{upper-tail dependence parameter} \eqn{\lambda^U_\mathbf{C}} with reversed inequalities is defined as
\deqn{\lambda^U_\mathbf{C} = \lim_{t{\rightarrow 1^{-}}} \mathrm{Pr}[Y > y(t)\mid X > x(t)]\mbox{.}}


Nelsen (2006, p. 214) also notes that both \eqn{\lambda^L_\mathbf{C}} and \eqn{\lambda^U_\mathbf{C}} are nonparametric and depend only on the copula of \eqn{X} and \eqn{Y}, and Nelsen shows that each can be computed if the above limits exist as follows:
\deqn{\lambda^L_\mathbf{C} = \lim_{t{\rightarrow 0^{+}}} \frac{\mathbf{C}(t,t)}{t} = \delta_\mathbf{C}'(0^{+})\mbox{\ and}}
\deqn{\lambda^U_\mathbf{C} = \lim_{t{\rightarrow 1^{-}}} \frac{1 - 2t - \mathbf{C}(t,t)}{1 - t} = 2 - \lim_{t{\rightarrow 1^{-}}} \frac{1 - \mathbf{C}(t,t)}{1-t} = 2 - \delta_\mathbf{C}'(1^{-})\mbox{,}}
where \eqn{\delta_\mathbf{C}'(t)} is the derivative of the diagonal of the copula. Multiple presentations are shown because algebraic variants are shown across the literature.

If \eqn{\lambda^L_\mathbf{C} \in (0,1]}, then \eqn{\mathbf{C}} has lower-tail dependence but if \eqn{\lambda^L_\mathbf{C} = 0}, then \eqn{\mathbf{C}} has \emph{no} lower-tail dependence. Likewise, if \eqn{\lambda^U_\mathbf{C} \in (0,1]}, then \eqn{\mathbf{C}} has upper-tail dependence but if \eqn{\lambda^U_\mathbf{C} = 0}, then \eqn{\mathbf{C}} has \emph{no} upper-tail dependence.
}
\usage{
taildepCOP(cop=NULL, para=NULL, tol=1e-6, divisor=2, plot=FALSE, ylim=NULL,
                                verbose=FALSE, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{tol}{A tolerance on convergence;}
  \item{divisor}{The divisor on the incremental reductions towards \eqn{0^+} and \eqn{0^-} by the algorithm;}
  \item{plot}{A logical plotting a diagnostic plot of the diagonal derivatives and label the limits;}
  \item{ylim}{Optional vertical limits if the plot is turned on. Although the dependence parameters are bounded as described above, numerical stability can be a problem. Stability is especially a problem if an empirical copula is being used; theefore, the bounds of the plot are left open unless the user locks them down with this argument;}
  \item{verbose}{Show incremental progress; and}
  \item{...}{Additional arguments to pass to the copula function.}
}
\value{
  An \R list is returned.
  \item{lambdaL}{The rounded value of \eqn{\lambda^L_\mathbf{C}};}
  \item{lambdaU}{The rounded value of \eqn{\lambda^U_\mathbf{C}};}
  \item{source}{An attribute identifying the computational source: \dQuote{taildepCOP}.}
}
\note{
\emph{IMPLEMENTED ALGORITHM}---The algorithm implemented for \code{taildepCOP} is based on halves (or alternatives by the setting of \code{divisor} argument) and uses the copula function (not an analytical or even numeric derivative of the diagonal, \eqn{\delta_\mathbf{C}'(t)}). Starting from the median or \eqn{t = 0.5}, each limit is respectively computed by successive halving (or the setting of argument \code{divisor}) of the distance towards \eqn{0^{+}} and \eqn{1^{-}} and checking the change in computed value against the tolerance \code{tol} argument. After the change becomes less than the tolerance, convergence is assumed. Other tests are made for \code{NaN} to aid in breaking the successive halvings. The rounding for the numerical results for \eqn{\lambda^U_\mathbf{C}} and \eqn{\lambda^L_\mathbf{C}} is an order of magnitude larger than the tolerance.

Users are encouraged to plot the results and further verify whether the convergence makes sense. The plot produced when \code{plot=TRUE} shows the probability \eqn{t} transformed into standard normal variates by the \code{qnorm()} function in \R so that the distal reaches of each tail and thus limit are readily seen. The terminal points of each limit computation are shown by a small dot and the letter \dQuote{L} and \dQuote{U} also are plotted at the terminal points.


Joe (2014, p. 63) reports that \dQuote{the empirical measure of tail dependence [\eqn{\hat\lambda_\mathbf{C}^L} or \eqn{\hat\lambda_\mathbf{C}^U}] for data does not really exist because of the limit.}  Joe (2014) suggests that other sources in the literature are the \dQuote{best that can be done} (\enc{Dobrić}{Dobric} and Schmid, 2005; Frahm \emph{et al.}, 2005). Another source of discussion is by Schmidt and \enc{Stadtmüller}{Stadtmuller} (2006). The results therein are not yet followed up for the \pkg{copBasic} package. Picking up the simulation dealt with extensively in the \bold{Note} section of \code{\link{vuongCOP}}, a user might try this:
\preformatted{
  set.seed(385); n <- 390
  UV <- simCOP(cop=PSP, n=n, col=8, pch=16, graphics=FALSE)
  taildepCOP(cop=EMPIRcop, para=UV, plot=TRUE, divisor=8, ylim=c(0,1))
  taildepCOP(cop=PSP) # lower=0.5, upper=0
}
The returned tail dependency parameters are numerically of little importance and in strict terms likely misleading. What should be of interest are the plotted trajectories of the lower and upper lines. Note: the lower tail wobbles but seems to show stability towards near \eqn{\hat\lambda_\mathbf{C}^L = 0.5} and upper-tail line wobbles downward towards \eqn{\hat\lambda_\mathbf{C}^U = 0}. These values are, respectively, the tail dependencies of the \eqn{\mathbf{PSP}} copula (\code{\link{PSP}}). A user might try increasing the sample size by an order of magnitude and rerunning the above code. Lastly, Salvadori \emph{et al.} (2006, pp. 173--175) caution on the difficulties of nonparametric tail dependency estimation. Given objectives of the \pkg{copBasic} package, estimation of \eqn{\hat\lambda_\mathbf{C}^L} and \eqn{\hat\lambda_\mathbf{C}^U}, therefore, is an open development opportunity.

\emph{DEMONSTRATION (Tail Dependence)}---The following example shows a comparison between early code examples by Charpentier (2012) concerning copulas and tail dependence using real-world data. Consider the \code{lossalae} data set and the following code requiring the \pkg{evd} package:
\preformatted{
  library(evd); X <- lossalae # Charpentier (2012)
  library(copBasic)
  fakeU <- lmomco::pp(X[,1],sort=FALSE,a=0) # Weibull plotting position i/(n+1)
  fakeV <- lmomco::pp(X[,2],sort=FALSE,a=0) # Weibull plotting position i/(n+1)
  uv <- data.frame(U=fakeU, V=fakeV)  # parameter "object" for Empirical copula
  plot(uv)
  TD <- taildepCOP(cop=EMPIRcop, para=uv, divisor=25,
                   plot=TRUE, ylim=c(0, 7/10))
  U <- rank(X[,1])/(nrow(X)+1); V <- rank(X[,2])/(nrow(X)+1)# Charpentier(2012)
  Lemp <- function(z) sum((U<=z)   & (V<=z))   / sum(U<=z  )# Charpentier(2012)
  Remp <- function(z) sum((U>=1-z) & (V>=1-z)) / sum(U>=1-z)# Charpentier(2012)
  u <- seq(0.001, 0.5, by=.001)                             # Charpentier(2012)
  L <- Vectorize(Lemp)(u); R <- Vectorize(Remp)(rev(u))     # Charpentier(2012)
  lines(qnorm(c(u, u + 0.5 - u[1])), c(L,R)) # modified after Charpentier(2012)
  legend("bottomright", c("Lower-tail dependency by taildepCOP()",
                          "Upper-tail dependency by taildepCOP()",
                          "Charpentier (2012)"), bty="n", cex=0.9,
                          lwd=1, lty=1, col=c("red", "blue", "black"))
}
The figure that will have been generated shows considerably similarity to that from the algorithms of Charpentier. Now, let us extend the discussion by using the \emph{Blomqvist (Schmid--Schmidt) Betas} (\code{\link{blomCOPss}}) that have a formulation permitting lower- and upper-tail dependency parameters in a different manner than the definitions of this documentation for \code{taildepCOP}.
\preformatted{
  edge <- 30 * 1 / (1+nrow(X)) # as few as 30 samples into the tails
  psl <- pnorm(seq(0, qnorm(  edge), by=-0.005))
  psu <- pnorm(seq(0, qnorm(1-edge), by= 0.005))
  lines(qnorm(psl),
        sapply(psl, function(p) { blomCOPss(as.sample=TRUE, para=uv,
                    ctype="checkerboard", uu=rep(p, 2), vv=c(1,1)) }),
                    col="darkgreen", lty=1, lwd=2)
  lines(qnorm(psu),
        sapply(psu, function(p) { blomCOPss(as.sample=TRUE, para=uv,
                    ctype="checkerboard", uu=c(0,0), vv=rep(p, 2)) }),
                    col="darkgreen", lty=1, lwd=2)
  points(0, blomCOP(as.sample=TRUE, para=uv), pch=16, col="magenta", cex=2)
  legend("topleft", c("Tail dependency by Blomqvist (Schmid-Schmidt) Betas",
                      "Blomqvist Beta C(1/2, 1/2)"),
                    bty="n", cex=0.9, lwd=2, lty=c(1,NA), pch=c(NA,16),
                    pt.cex=c(NA,2), col=c("darkgreen", "magenta"))
}
The thick green lines show that the dependency parameters to the left and right are approached along a different trajectory using the definitions in \code{\link{blomCOPss}}. It seems at some stage in analysis that the practitioner will need to decide how deep into the tail the sample will permit for a reliable estimate of the dependency parameters. \emph{Blomqvist Beta} (\eqn{\hat\beta_\mathbf{C}}) (\code{\link{blomCOP}} is shown as the magenta dot at the median and henceforth from do the trajectories of \eqn{\hat\lambda^L_{\beta^\diamond_\mathbf{C}}} and \eqn{\hat\lambda^U_{\beta^\diamond_\mathbf{C}}} extend as their \eqn{\mathrm{lim} p \rightarrow 0^+}. Ultimately, for the data shown in the figure, perhaps the \eqn{\hat\lambda_\mathbf{C}^L = 0.1} and the \eqn{\hat\lambda_\mathbf{C}^U = 0.3} and a parametric copula fitted in part to such values.
}
\references{
Charpentier, A., 2012, Copulas and tail dependence, part 1: R-bloggers, dated Sept. 17, 2012, accessed on February 2, 2019 at \cr
\url{https://www.r-bloggers.com/2012/09/copulas-and-tail-dependence-part-1/}

\enc{Dobrić}{Dobric}, J. and Schmid, F., 2005, Nonparametric estimation of the lower tail dependence \eqn{\lambda^L} in bivariate copulas: Journal of Applied Statistics, v. 32, no. 4, pp. 387--407,\cr \doi{10.1080/02664760500079217}.

Frahm, G., Junker, M., and Schmidt, R., 2005, Estimating the tail-dependence coefficient---\cr Properties and pitfalls: Insurance---Mathematics and Economics, v. 37, no. 1, pp. 80--100.

Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in Nature---An approach using copulas: Springer, 289 p.

Schmidt, R., and \enc{Stadtmüller}{Stadtmuller}, U., 2006, Nonparametric estimation of tail dependence: The Scandinavian Journal of Statistics, v. 33, pp. 307--335.
}
\author{W.H. Asquith}
\seealso{\code{\link{COP}}, \code{\link{tailconCOP}}, \code{\link{tailordCOP}}, \code{\link{blomCOPss}}}
\examples{
  # Plot the tail dependencies by nonexceedance probability for a
  # for a positive association Plackett copula and see that both are zero.
  taildepCOP(cop=PLACKETTcop, para=3, plot=TRUE)
  # So, Plackett has no tail dependency, as Nelsen (2006, p. 215) shows.

\dontrun{
  # The results that follow match those reported by Nelsen (2006, p. 215).
  taildepCOP(cop=MOcop, para=c(0.4, 0.9)) # LambL = 0, LambU = 0.4 [min(alpha,beta)]}

\dontrun{
  # Analytical solution to Gumbel-Hougaard copula from the copula package:
  copula::lambda(copula::gumbelCopula(3))
  #   lower    upper
  # 0.000000 0.740079
  # Numerical approximation (see copBasic::GHcop for analytical formula):
  as.data.frame(taildepCOP(GHcop, para=3))
  #  lambdaL lambdaU     source
  #1 0.00012 0.74008 taildepCOP}

\dontrun{
  # Plot the tail dependencies by nonexceedance probability
  # for the PSP copula, which has lower but no upper-tail dependence.
  taildepCOP(cop=PSP, para=NULL, plot=TRUE) # LambL=0.5, LambU=0
  # which is readily confirmed by simCOP(1000, cop=PSP)
  # Nelsen (2006, p. 216) reports that this copula has LambL=1/2 and LambU=0,
  # and we get the same results here.

  # How about some composited Plackett-Plackett copulas?
  # Each has upper- and lower-tail dependence parameters equal to zero.
  para <- list( cop1=PLACKETTcop,  cop2=PLACKETTcop, alpha=0.9392,
             para1=0.00395,     para2=4.67,         beta=0.5699)
  taildepCOP(cop=composite2COP, para=para, plot=TRUE, verbose=TRUE) #}

\dontrun{
  # This next Plackett-Plackett is interesting because at its core it looks
  # like it should be both tail dependent like M() but the shapes of the curves
  # are quite different from those of M(). This example shows numerical
  # instability for the upper tail but not the lower tail. So, we extend the
  # example to shown the tail dependency trajectories by blomCOPss(). And again
  # it is seen that the lower tail as a stable solution but the upper tail
  # has instability at 6 standard deviations into the upper tail.
  para <- list(  cop1=PLACKETTcop,  cop2=PLACKETTcop, alpha=0.0063,
               para1=0.101,        para2=4493,         beta=0.0167)
  taildepCOP(cop=composite2COP, para=para, plot=TRUE)
  lsu <- pnorm(seq(-7, 0, by=.01))
  psu <- pnorm(seq( 0, 7, by=.01))
  lines(qnorm(lsu), sapply(lsu, function(p) {
          blomCOPss(cop=composite2COP, para=para, vv=c(1,1), uu=rep(p, 2)) }),
                      col="darkgreen", lty=2, lwd=1)
  lines(qnorm(psu), sapply(psu, function(p) {
          blomCOPss(cop=composite2COP, para=para, uu=c(0,0), vv=rep(p, 2)) }),
                      col="darkgreen", lty=2, lwd=1) #}
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{copula (tail characteristics)}
\keyword{copula (tail properties)}
\keyword{Nelsen (2006) Examples and Exercises}
\keyword{tail dependence}
\keyword{package copula (comparison to)}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{copula (tail characteristics)}
\keyword{copula (tail properties)}
\keyword{tail dependence}
\keyword{Blomqvist (Schmid-Schmidt) Beta}

\encoding{utf8}
\name{glueCOP}
\alias{glueCOP}
\title{Gluing Two Copulas}
\description{
The \emph{gluing copula} technique (Erdely, 2017, p. 71), given two bivariate copulas \eqn{\mathbf{C}_A} and \eqn{\mathbf{C}_B} and a fixed value \eqn{0 \le \gamma \le 1} is
\deqn{\mathbf{C}_{\gamma}(u,v) = \gamma\cdot\mathbf{C}_A\bigl(u/\gamma, v\bigr)}
for \eqn{0 \le u \le \gamma} and
\deqn{\mathbf{C}_{\gamma}(u,v) = (1-\gamma)\cdot\mathbf{C}_B\bigl((u-\gamma)
\,/\,(1-\gamma), v\bigr)}
for \eqn{\gamma \le u \le 1} and \eqn{\gamma} represents the \emph{gluing point} in \eqn{u} (horizontal axis). The logic is simply the rescaling of \eqn{\mathbf{C}_A} to \eqn{[0,\gamma] \times [0,1]} and \eqn{\mathbf{C}_B} to \eqn{[\gamma,1] \times [0,1]}. Copula gluing is potentially useful in circumstances for which regression is non-monotone.
}
\usage{
glueCOP(u, v, para=NULL, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A special parameter \code{list} (see \bold{Note}) with a mandatory element of \code{glue}  parameter \eqn{\gamma}; and}
  \item{...}{Additional arguments to pass to the copulas.}
}
\value{
  Value(s) for the copula are returned.
}
\note{
The following descriptions list in detail the structure and content of the \code{para} argument:
\describe{
  \item{\code{glue}}{--- The \eqn{\gamma} gluing parameter;}
  \item{\code{cop1}}{--- Function of the first copula \eqn{\mathbf{A}};}
  \item{\code{cop2}}{--- Function of the second copula  \eqn{\mathbf{B}};}
  \item{\code{para1}}{--- Vector of parameters \eqn{\Theta_\mathbf{A}} for  \eqn{\mathbf{A}}; and}
  \item{\code{para2}}{--- Vector of parameters \eqn{\Theta_\mathbf{B}} for  \eqn{\mathbf{B}}.}
}
}
\references{
Erdely, A., 2017, Copula-based piecewise regression (chap. 5)  \emph{in} Copulas and dependence models with applications---Contributions in honor of Roger B. Nelsen, \emph{eds.} Flores, U.M., Amo Artero, E., Durante, F., \enc{Sánchez}{Sanchez}, J.F.: Springer, Cham, Switzerland, ISBN 978--3--319--64220--9, \doi{10.1007/978-3-319-64221-5}.
}
\author{W.H. Asquith}
\seealso{\code{\link{COP}}, \code{\link{breveCOP}}, \code{\link{composite1COP}}, \code{\link{composite2COP}}, \code{\link{composite3COP}}, \code{\link{convexCOP}}
}
\examples{
\dontrun{
  para <- list(cop1=PLACKETTcop, para1=.2, cop2=GLcop, para2=1.2, glue=0.6)
  densityCOPplot(cop=glueCOP, para=para) # }

\dontrun{
  # Concerning Nelsen (2006, exam. 3.3, pp. 59-61)
  # Concerning Erdely (2017, exam. 5.1, p. 71)
  # Concerning Erdely (2017, exam. 5.2, p. 75)
  # Nelsen's example is a triangle with vertex at [G, 1].
  # Erdley's example permits the construction using glueCOP from M and W.
  "coptri" <- function(u,v, para=NA, ...) {
     p <- para[1]; r <- 1 - (1-p)*v
     if(length(u) > 1 | length(v) > 1) stop("only scalars for this function")
     if(0 <= u & u <= p*v & p*v <= p) {            return(u)
     } else if(  0 <= p*v & p*v <  u & u <  r) {   return(p*v)
     } else if(  p <= r   & r   <= u & u <= 1 ) {  return(u+v-1)
     } else { stop("should not be here in logic") }
  }
  "UsersCop" <- function(u,v, ...) { asCOP(u,v, f=coptri, ...) }
  # Demonstrate Nelsen's triangular copula    (black dots )
  UV <- simCOP(cop=UsersCop, para=0.35, cex=0.5, pch=16)
  # Add Erdley's gluing of M() and W() copula (red circles)
  para <- list(cop1=M, cop2=W, para1=NA, para2=NA, glue=0.35)
  UV <- simCOP(cop=glueCOP,  para=para, col=2,   ploton=FALSE)
  # We see in the plot that the triangular copulas are the same.

  # For G = 0.5, Erdley shows Spearman Rho = 2*G-1 = 0, but
  #  Schweizer-Wolff = G^2 + (G-1)^2 = 0.5, let us check these:
  para <- list(cop1=M, cop2=W, para1=NA, para2=NA, glue=0.5)
  rhoCOP( cop=glueCOP, para=para) # -2.181726e-17
  wolfCOP(cop=glueCOP, para=para) #  0.4999953
  # So, rhoCOP() indicates independence, but wolfCOP() indicates
  # dependence at the minimum value possible for a triangular copula.}

\dontrun{
  x <- seq(0,1, by=0.05); para <- list(cop1=M, cop2=W, para1=NA, para2=NA)
  LCM <- lapply(x, function(i) { para$glue <- i;
                   lcomCOP(cop=glueCOP, para=para, stop.on.error=FALSE) })
  LCM12 <- lapply(LCM, function(i) i$lcomUV)
  LCM21 <- lapply(LCM, function(i) i$lcomVU)
  LCM12 <- t( sapply(LCM12, function(i) i) )
  LCM21 <- t( sapply(LCM21, function(i) i) )
  plot( x, LCM12[,3], ylim=c(-0.8, +0.8), type="l", lty=2,
        xlab="glueCOP() gluing parameter value",
        ylab="L-coskew or L-cokurtosis")
  lines(x, LCM21[,3], col="black")
  lines(x, LCM12[,4])
  lines(x, LCM21[,4], col="darkorchid3")
  rho <- LCM12[,2]
  plot(rho, LCM12[,3], ylim=c(-0.8, +0.8), type="l", lty=2,
        xlab="Spearman Rho", ylab="L-coskew or L-cokurtosis")
  lines(rho, LCM21[,3], col="black")
  lines(rho, LCM12[,4], lty=2)
  lines(rho, LCM21[,4], col="darkorchid3") # }
}
\keyword{copula (formulas)}
\keyword{copula gluing}
\keyword{copula composition}
\keyword{Erdely (2017) Examples and Exercises}

\encoding{utf8}
\name{ORDSUMcop}
\alias{ORDSUMcop}
\title{Ordinal Sums of M-Copula}
\description{
Compute \emph{ordinal sums of a copula} (Nelsen, 2006, p. 63) or \emph{M-ordinal sum of the summands} (Klement \emph{et al.}, 2017) within \eqn{\mathcal{I}^2} into \eqn{n} partitions (possibly infinite) within \eqn{\mathcal{I}^2}. According to Nelsen, letting \eqn{\mathcal{J}} denote a \emph{partition} of \eqn{\mathcal{I}^2} and \eqn{\mathcal{J}_i = [a_i,\, b_i]} be the \eqn{i}th partition that does not overlap with others and letting also \eqn{\mathbf{C}_i} be a copula for the \eqn{i}th partition, then the \emph{ordinal sum} of these \eqn{\mathbf{C}_i} with parameters \eqn{\Theta_i} \emph{with respect to} \eqn{\mathcal{J}_i} is the copula \eqn{\mathbf{C}} given by

\deqn{\mathbf{C}\bigl(u,v; \mathcal{J}_i, \mathbf{C}_i, \Theta_i, i \in 1,2,\cdots,n\bigr) = a_i + (b_i-a_i)\mathbf{C}_i\biggl(\frac{u-a_i}{b_i-a_i},\, \frac{v-a_i}{b_i-a_i};  \Theta_i\biggr)\  \mbox{for}\ (u,v) \in \mathcal{J}^2\mbox{,}}

for points within the partitions, and for points otherwise outside the partitions the coupla is given by

\deqn{\mathbf{C}\bigl(u,v; \mathcal{J}_i, \mathbf{C}_i, i \in 1,2,\cdots,n\bigr) = \mathbf{M}(u,v)\  \mathrm{for}\ (u,v) \ni \mathcal{J}^2\mbox{, and}}

let \eqn{\mathbf{C}_\mathcal{J}(u,v)} be a convenient abbreviation for the copula. Finally, Nelsen (2006, theorem 3.2.1) states that a copula is an ordinal sum if and only if for a \eqn{t} if \eqn{\mathbf{C}(t,t)=t} for \eqn{t \in (0,1)}. The \emph{diagonal of a coupla} can be useful for quick assessment (see \bold{Examples}) of this theorem. (See \code{\link{ORDSUWcop}}, \emph{W-ordinal sum of the summands}.)
}
\usage{
ORDSUMcop(u,v, para=list(cop=W, para=NA, part=c(0,1)), ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A list of sublists for the coupla, parameters, and partitions (see \bold{Examples}) and some attempt for intelligent in-fill of \code{para} is made within the sources (the default \code{para} is an example for which \code{cop} and \code{para} elements are converted to lists). The user is responsible that \code{part} element properly canvases by end-point alignment all of \eqn{\mathcal{I}^2}; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Klement, E.P., \enc{Kolesárová}{Kolesarova}, A., Mesiar, R., Saminger-Platz, S., 2017, Copula constructions using ultramodularity (chap. 9) \emph{in} Copulas and dependence models with applications---Contributions in honor of Roger B. Nelsen, \emph{eds.} Flores, U.M., Amo Artero, E., Durante, F., \enc{Sánchez}{Sanchez}, J.F.: Springer, Cham, Switzerland, ISBN 978--3--319--64220--9, \doi{10.1007/978-3-319-64221-5}.
}
\author{W.H. Asquith}
\seealso{\code{\link{copBasic-package}}, \code{\link{W_N5p12a}}, \code{\link{ORDSUWcop}}}
\examples{
\dontrun{
  para <- list(cop=c(CLcop, M, PLcop, GHcop), para=list(4, NA, 0.1, c(3, 4)),
               part=list(c(0, 0.25), c(0.25, 0.35), c(0.35, 0.85), c(0.85, 1)))
  UV <- simCOP(n=100, cop=ORDSUMcop, para=para, ploton=FALSE)
  plot(c(0,1), c(0,1), xlab="U, nonexceedance probability", type="n", las=1,
                       ylab="V, nonexceedance probability")
  for(k in seq_len(length(para$part))) {         #  to draw the partitions
    a <- para$part[[k]][1]; b <- para$part[[k]][2]
    polygon(c(a, b, b, a, a), c(a, a, b, b, a), lty=2, lwd=0.8, col="lightgreen")
    text( (a+b) / 2,  (a+b) / 2, k, cex=3, col="blue") # numbered by partition
  }
  points(UV, pch=21, cex=0.8, col=grey(0.1), bg="white") #}

\dontrun{
  para <- list(cop=c(GHcop), para=list(c(2, 3)), # internally replicated
               part=list(c(0, 0.2), c(0.2, 0.3), c(0.3, 0.5), c(0.5, 0.7), c(0.7, 1)))
  UV <- simCOP(n=100, cop=ORDSUMcop, para=para, ploton=FALSE)
  plot(c(0,1), c(0,1), xlab="U, nonexceedance probability", type="n", las=1,
                       ylab="V, nonexceedance probability")
  for(k in seq_len(length(para$part))) {         #  to draw the partitions
    a <- para$part[[k]][1]; b <- para$part[[k]][2]
    polygon(c(a, b, b, a, a), c(a, a, b, b, a), lty=2, lwd=0.8, col="lightgreen")
    text( (a+b) / 2,  (a+b) / 2, k, cex=3, col="blue") # numbered by partition
  }
 points(UV, pch=21, cex=0.8, col=grey(0.1), bg="white") #}

\dontrun{
  # In this example, it is important that the delt is of the resolution
  # matching the edges of the partitions.
  para <- list(cop=P, para=list(NULL),
               part=list(c(0, 0.257), c(0.257, 0.358), c(0.358, 1)))
  DI <- diagCOP(cop=ORDSUMcop, para=para, delt=0.001)
  if(sum(DI$diagcop == DI$t) >= 1) {
    message("The ORDSUMcop() operation is an ordinal sum if there exists\n",
            "a t=(0,1) exists such that C(t,t)=t by Nelsen (2006, theorem 3.2.1).")
  }
  lines(par()$usr[1:2], par()$usr[3:4], col="red") #}

\dontrun{
  # Expansive demonstration of high levels of "shape" of bivariate relations beyond
  # the "simple" measures of association (Rho, Tau, Wolf, ...):
  myCop <- ORDSUMcop # Note, integration problems more likely, if changed to
  # ORDSUWcop at which point |Spearman Rhos| might integrate to beyond unity.
  # Note, hardwired parameters in the demo-definitions of custom subcouplas to
  # avoid confusion to study departure away from true M() and W() copulas, and to
  # have these examples producing permutation asymmetry and L-comoments not exchangable
  # though only the with respect to UV [12] being plotted. We swap to M, W as shown
  # in the example (three lines below) to make the default of example exchangable.
  myM <- function(u,v, para=NULL, ...)         P(u,   v, para=NULL, ...)
  myW <- function(u,v, para=NULL, ...) u - PAcop(u, 1-v, para=7, ...) # rotate 90d
  myM <- M; myW <- W # comment this line out for "custom subcopulas" after which
  # the "myM" means first defined subcopula and "myW" means second defined subcopula.
  # ------------- Demonstrate with mid-partioning -------------
  mara <- list(cop=c(myM, myW),  para=NULL, part=c(0, 0.5, 1))
  UVm  <- simCOP(1E3, cop=myCop, para=mara, seed=1, col="red")
  wara <- list(cop=c(myW, myM),  para=NULL, part=c(0, 0.5, 1))
  UVw  <- simCOP(1E3, cop=myCop, para=wara, seed=1, col="blue", ploton=FALSE)
  cor(UVm[,1], UVm[,2], method="spearman"); cor(UVw[,1], UVw[,2], method="spearman")
  # -----------------------------------------------------------
  opts <- par(las=1, lend=1, ljoin=1) # all lab horizontal, butted, mitered
  del  <- 0.005
  nsim <- 300; ps <- c(0.001, seq(del, 1-del, by=del), 0.999)
  # The ylim might need extension, it seems cases of L-comoments exceeding unity
  # can be found depending on the settings of the myM and myW.
  plot(c(0, 1), c(-1, 1), type="n", ylab="Spearman Rho, L-coskew, and L-cokurtosis",
       xlab="Ordinal sum partioning parameter")
  # -- Stuff below can be run again with changing the three logical
  show_points     <- FALSE # variables as user needs. We like TRUE
  show_guidelines <- TRUE  # FALSE, FALSE for a look by sampling
  dolcomCOP       <- TRUE  # before FALSE, TRUE, TRUE for theory.
  T2m12 <- T3m12 <- T4m12 <- T2w12 <- T3w12 <- T4w12 <- NULL # L-comoments
  T2m21 <- T3m21 <- T4m21 <- T2w21 <- T3w21 <- T4w21 <- NULL # L-comoments
  NuSk_m <- NuSt_m <- NuSk_w <- NuSt_w <- NULL # Joe "skews"
  # lcomCOP()          # theoretical L-comoments by integration
  # lmomco::lcomoms2() # sample L-comoments estimation
  # nuskewCOP() is a permutation asymmetry measure and nustarCOP() is something
  # slightliy more complex in what it measures, but it is important to note that
  # each is in a linear combination to the dual integral of C(u,v), and therefore,
  # we can expect them to be coupled to Spearman Rho itself in a rather simple way.
  for(p in ps) { message(p, "-", appendLF=FALSE)
    mara <- list(cop=c(myM, myW), para=NULL, part=c(0, p, 1))
    wara <- list(cop=c(myW, myM), para=NULL, part=c(0, p, 1))
    if(dolcomCOP) { # theoretical (numerical integration)
      lcm    <- lcomCOP(cop=myCop, para=mara, stop.on.error=FALSE)
      lcw    <- lcomCOP(cop=myCop, para=wara, stop.on.error=FALSE)
      T2m12  <- c(T2m12, lcm$lcomUV[2]); T2w12 <- c(T2w12, lcw$lcomUV[2])
      T2m21  <- c(T2m21, lcm$lcomVU[2]); T2w21 <- c(T2w21, lcw$lcomVU[2])
      T3m12  <- c(T3m12, lcm$lcomUV[3]); T4m12 <- c(T4m12, lcm$lcomUV[4])
      T3w12  <- c(T3w12, lcw$lcomUV[3]); T4w12 <- c(T4w12, lcw$lcomUV[4])
      T3m21  <- c(T3m21, lcm$lcomVU[3]); T4m21 <- c(T4m21, lcm$lcomVU[4])
      T3w21  <- c(T3w21, lcw$lcomVU[3]); T4w21 <- c(T4w21, lcw$lcomVU[4])
      NuSk_m <- c(NuSk_m, nuskewCOP( cop=myCop, para=mara) )
      NuSt_m <- c(NuSt_m, nustarCOP( cop=myCop, para=mara) )
      NuSk_w <- c(NuSk_w, nuskewCOP( cop=myCop, para=wara) )
      NuSt_w <- c(NuSt_w, nustarCOP( cop=myCop, para=wara) )
    } else {        # sample versions
      uvm    <- rCOP(nsim, cop=myCop, para=mara, seed=1)
      uvw    <- rCOP(nsim, cop=myCop, para=wara, seed=1)
      lcm    <- lmomco::lcomoms2(uvm, nmom=4)
      lcm    <- list(lcomUV=c(NA, lcm$T2[1,2], lcm$T3[1,2], lcm$T4[1,2]),
                     lcomVU=c(NA, lcm$T2[2,1], lcm$T3[2,1], lcm$T4[2,1]))
      lcw    <- lmomco::lcomoms2(uvw, nmom=4)
      lcw    <- list(lcomUV=c(NA, lcw$T2[1,2], lcw$T3[1,2], lcw$T4[1,2]),
                     lcomVU=c(NA, lcw$T2[2,1], lcw$T3[2,1], lcw$T4[2,1]))
      T2m12  <- c(T2m12, lcm$lcomUV[2]); T2w12 <- c(T2w12, lcw$lcomUV[2])
      T2m21  <- c(T2m21, lcm$lcomVU[2]); T2w21 <- c(T2w21, lcw$lcomVU[2])
      T3m12  <- c(T3m12, lcm$lcomUV[3]); T4m12 <- c(T4m12, lcm$lcomUV[4])
      T3w12  <- c(T3w12, lcw$lcomUV[3]); T4w12 <- c(T4w12, lcw$lcomUV[4])
      T3m21  <- c(T3m21, lcm$lcomVU[3]); T4m21 <- c(T4m21, lcm$lcomVU[4])
      T3w21  <- c(T3w21, lcw$lcomVU[3]); T4w21 <- c(T4w21, lcw$lcomVU[4])
      NuSk_m <- c(NuSk_m, nuskewCOP( para=uvm, as.sample=TRUE) )
      NuSt_m <- c(NuSt_m, nustarCOP( para=uvm, as.sample=TRUE) )
      NuSk_w <- c(NuSk_w, nuskewCOP( para=uvw, as.sample=TRUE) )
      NuSt_w <- c(NuSt_w, nustarCOP( para=uvw, as.sample=TRUE) )
    }
    # -----------------------------------------------------
    if(show_points) {
      n <- length(T3m12)
      points(p,  T2m12[n], pch="R", col="seagreen4"  ) # Spearman Rho
      points(p,  T2w12[n], pch="R", col="darkorchid4") # Spearman Rho
      points(p,  T3m12[n], pch=24,  col="seagreen4",   bg="seagreen4"  )
      points(p,  T4m12[n], pch=25,  col="seagreen4",   bg="seagreen4"  )
      points(p,  T3w12[n], pch=24,  col="darkorchid4", bg="darkorchid4")
      points(p,  T4w12[n], pch=25,  col="darkorchid4", bg="darkorchid4")
      points(p, NuSk_m[n], pch=2,   col="deepskyblue2") # open upright tri
      points(p, NuSt_m[n], pch=8,   col="seagreen4"   ) # pch == star
      points(p, NuSk_w[n], pch=6,   col="deepskyblue2") # open flipped tri
      points(p, NuSt_w[n], pch=8,   col="darkorchid4" ) # pch == star
    }
  }
  message("done")
  if(show_guidelines) {
    psna <- c(ps, NA, ps) # insert NA to "line break"
    lines(psna, c( T2m12, NA,  T2w12), lwd=3, col="grey80")
    lines(psna, c( T3m12, NA,  T3w12), lty=1)
    lines(psna, c( T4m12, NA,  T4w12), lty=2)
    lines(psna, c(NuSk_m, NA, NuSk_w), lty=1)
    lines(psna, c(NuSt_m, NA, NuSt_w), lty=2)
  }
  par(opts)
  # Note, the nuskews will plot along origin line if the "image" of the two
  # subcouplas are permutation symmetric and result in a hexagram (six-
  # sided star) being "plotted." If subcopulas themselves are not permutation
  # symmetric, then triangles for nuskew will diverge from each other.}

\dontrun{
  # Now, look at the coupling between the Spearman Rho and the L-comoments through
  # formation of L-comoment ratio diagrams of the ordinal sums.
  # Here, we just draw lines from whatever the vectors stemming from the previous
  # example produced by dolcomCOP being set TRUE (theoretical) or FALSE (sample)
  # L-comoments. For the simple, ordinal sum of M,W or W,M, then L-cokurtosis
  # will plot on top of each other; therefore, we use the thick line for T4m12.
  plot(c(-1, 1), c(-1, 1), type="n", las=1,
       xlab="Spearman Rho (driven by ordinal sum parition)"
       ylab="L-coskew (cT3) or L-cokurtosis (cT4)")
  lines(par()$usr[1:2], rep(0, 2))
  lines(T2m12, T3m12, lty=1, col="seagreen4", lwd=3)
  lines(T2m12, T4m12, lty=2, col="seagreen4", lwd=3)
  lines(T2w12, T3w12, lty=1, col="darkorchid4")
  lines(T2w12, T4w12, lty=2, col="darkorchid4") # (myM,myW);(myW,myM) plot on T4m21?
  txt <- c("L-coskew cT3[12], cop(myM, myW)", "L-cokurtosis cT4[12], cop(myM, myW)",
           "L-coskew cT3[12], cop(myW, myM)", "L-cokurtosis cT4[12], cop(myW, myM)")
  legend("topleft", txt, bty="n", seg.len=3, lty=c(1, 2, 1, 2), lwd=c(3, 3, 1, 1),
         col=c("seagreen4", "seagreen4", "darkorchid4", "darkorchid4"))
  # --------------------------
  # If the user is working with sufficiently complex copulas, then differing
  # trajectories through the domain occur.
  plot(c(-1, 1), c(-1, 1), type="n", las=1, ylab="L-coskew or L-cokurtosis",
       xlab="Spearman Rho (driven by ordinal sum partition)")
  lines(par()$usr[1:2], rep(0, 2))
  lines(T2m21, T3m21, lty=1, col="seagreen4", lwd=3)
  lines(T2m21, T4m21, lty=2, col="seagreen4", lwd=3)
  lines(T2w21, T3w21, lty=1, col="darkorchid4")
  lines(T2w21, T4w21, lty=2, col="darkorchid4") # (myM,myW);(myW,myM) plot on T4m21?
  txt <- c("L-coskew cT3[21], cop(myM, myW)", "L-cokurtosis cT4[21], cop(myM, myW)",
           "L-coskew cT3[21], cop(myW, myM)", "L-cokurtosis cT4[21], cop(myW, myM)")
  legend("topleft", txt, bty="n", seg.len=3, lty=c(1, 2, 1, 2), lwd=c(3, 3, 1, 1),
         col=c("seagreen4", "seagreen4", "darkorchid4", "darkorchid4")) # }

\dontrun{
  # --------------------------
  # Next, let us build polynomial approximations for the boundaries and the polynomial
  # degree has been set pretty large in order that the user would have built-in
  # flexibility to construction of these approximations for more complex copula than
  # the stacking of M,W and W,M.
  X    <- c(T2m12, T2m21); Y3mw <- c(T3m12, T3m21); xs <- seq(-1,1, by=0.001)
  T3fT2mw <- lm(Y3mw~I(X^ 1)+I(X^ 2)+I(X^ 3)+I(X^ 4)+I(X^ 5)+I(X^ 6)+
                     I(X^ 7)+I(X^ 8)+I(X^ 9)+I(X^10)+I(X^11)+I(X^12))
  plot(X, Y3mw, xlab="Spearman Rho", ylab="L-coskew (cT3)")
  lines(xs, predict(T3fT2mw, newdata=data.frame(X=xs)), col="red")
  mtext("2-partition ORDSUMcop : M, W")

  X    <- c(T2w12, T2w21); Y3wm <- c(T3w12, T3w21)
  T3fT2wm <- lm(Y3wm~I(X^ 1)+I(X^ 2)+I(X^ 3)+I(X^ 4)+I(X^ 5)+I(X^ 6)+
                     I(X^ 7)+I(X^ 8)+I(X^ 9)+I(X^10)+I(X^11)+I(X^12))
  plot(X, Y3wm, xlab="Spearman Rho", ylab="L-coskew (cT3)")
  lines(xs, predict(T3fT2wm, newdata=data.frame(X=xs)), col="red")
  mtext("2-partition ORDSUMcop : W, M")

  X  <- c(T2m12, T2m21, T2w12, T2w21); Y4 <- c(T4m12, T4m21, T4w12, T4w21)
  T4fT2mwwm <- lm(Y4~I(X^ 1)+I(X^ 2)+I(X^ 3)+I(X^ 4)+I(X^ 5)+I(X^ 6)+
                     I(X^ 7)+I(X^ 8)+I(X^ 9)+I(X^10)+I(X^11)+I(X^12))

  plot(X, Y4, xlab="Spearman Rho", ylab="L-cokurtosis (cT4)")
  lines(xs, predict(T4fT2mwwm, newdata=data.frame(X=xs)), col="red")

  X  <- c(T3m12, T3m21, T3w12, T3w21)
  Y4 <- c(T4m12, T4m21, T4w12, T4w21)
  plot(X, Y4, type="l", xlab="L-coskew", ylab="L-cokurtosis")
  points(T3m12, T4m12, col="seagreen4")
  points(T3w12, T4w12, col="orchid3"  ) # }

\dontrun{
  # --------------------------
  # Next, let us extract the coefficients and build-up stand alone functions that
  # could be used to draw the L-comoments of the ordinal sums and potentially
  # overlaid on L-comoment ratio diagrams such as seen built under the Examples in the
  # LCOMDIA*.Rd documentation files (for instance LCOMDIA_ManyCops.Rd) of the package.
  coesA <- T3fT2mw$coefficients;   names(coesA) <- NULL
  coesB <- T3fT2wm$coefficients;   names(coesB) <- NULL
  coesC <- T4fT2mwwm$coefficients; names(coesC) <- NULL
  print(coesA, 12); print(coesB, 12); print(coesC, 12)
  coesT3 <- sapply(seq_len(length(coesA)), function(i) {
               mean(c(abs(coesA[i]), abs(coesB[i]))) })
  print(coesT3, 12)
  "T34fT2" <- function(rho, type=c("T3mw", "T3wm", "T4mwwm")) {
    type <- match.arg(type)
    cs3 <- c(0.618673707079,
             0.175958850204, 0.512576942435, 0.145431255262, 0.265479078036,
             0.163312255777, 1.016327127995, 0.537153927577, 2.225271502140,
             0.636080402864, 2.191529289514, 0.292183399458, 0.823239656100)
    if(type == "T3mw") {
      cs <- c(-1,-1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1)*cs3
    } else if(type == "T3wm") {
      cs <- c( 1, 1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1)*cs3
    } else if(type == "T4mwwm") {
      cs <- c( 0.236681925424,
              -0.429191591357, -0.573421581827, 0.352083105242, 1.042508365628,
              -0.710792468014, -4.321529301154, 2.229039543342, 9.419379550877,
              -2.652104491697, -9.275892279321, 1.208446517629, 3.472344853036)
      snmw <- rep(1, length(cs))
    } else {
      stop("should not be here in logic")
    }
    z <- sapply(rho, function(r) sum(sapply(seq_len(length(cs)),
                                     function(p) cs[p] * r^(p-1))))
    return(z)
  }

  # Finally, plot again the L-comoment ratio diagram of Spearman Rho and the
  # two higher L-comoments and a L-coskew and L-cokurtosis diagram could also be
  # readily built from a vector of Spearman Rhos.
  ylim <- c(-0.7, 0.7)
  plot( xs, T34fT2(xs, "T3mw"), type="n", ylim=ylim,
       xlab="Spearman Rho", ylab="L-coskew or L-cokurtosis approximations")
  lines(par()$usr[1:2], rep(0, 2))
  lines(xs, T34fT2(xs, "T3mw"),   col="brown3")
  lines(xs, T34fT2(xs, "T3wm"),   col="blue3" )
  lines(xs, T34fT2(xs, "T4mwwm"), col="orange")
  # --------------------------
  ylim <- c(-0.7, 0.7)
  plot(c(T34fT2(xs, "T3mw"  ), T34fT2(xs, "T3wm"  )),
       c(T34fT2(xs, "T4mwwm"), T34fT2(xs, "T4mwwm")), type="n", ylim=ylim,
       xlab="L-coskew approximations", ylab="L-cokurtosis approximations")
  lines(par()$usr[1:2], rep(0, 2)); lines(rep(0, 2), par()$usr[3:4])
  lines(T34fT2(xs, "T3mw"), T34fT2(xs, "T4mwwm"),      col="seagreen4")
  lines(T34fT2(xs, "T3wm"), T34fT2(xs, "T4mwwm"),      col="orchid3"  )
  text(-0.35, 0.07, "Ordinal sum of\nM and W", font=2, col="seagreen4")
  text(+0.35, 0.07, "Ordinal sum of\nW and M", font=2, col="orchid3"  ) # }
}
\keyword{copula (formulas)}
\concept{Ordinal Sums}
\keyword{copula}
\keyword{ordinal sums}
\keyword{M-ordinal sum of the summands}
\concept{summands}
\keyword{L-comoments (ratio diagram)}
\keyword{L-comoment ratio diagram}

\encoding{utf8}
\name{EMPIRgrid}
\alias{EMPIRgrid}
\title{Grid of the Bivariate Empirical Copula}
\description{
Generate a gridded representation of the \emph{bivariate empirical copula} (see \code{\link{EMPIRcop}}, Salvadori \emph{et al.}, 2007, p. 140). Herein, the \eqn{\ddot{\mathbf{C}}_n} (double-dots) are used to denote gridding of the copula. This function has the primary intention of supporting 3-D renderings or 2-D images of the \emph{copulatic surface}, but many empirical copula functions in \pkg{copBasic} rely on the grid of the empirical copula---unlike the functions that support parametric copulas. The algorithm within \code{EMPIRgrid} is slow because of nested loops, but a vastly faster version with some tweaks (not having an explicit \code{deluv}) is available under \code{\link{EMPIRgrid_fast}}.
}
\usage{
EMPIRgrid(para=NULL, deluv=0.05, gridonly=FALSE, verbose=FALSE, ...)
}
\arguments{
  \item{para}{A matrix of \eqn{u} and \eqn{v} vectors as an alternative to \code{uv} for syntax compatibility for other situations in \pkg{copBasic};}
  \item{deluv}{A delta value of the both the \eqn{u} and \eqn{v} axes (grid edges) for empirical copula estimation by the \code{\link{EMPIRcop}} function;}
  \item{gridonly}{A logical to return the \code{empcop} component (\eqn{\ddot{\mathbf{C}}_{n}(u,v)}) passed through \code{...} the \code{ctype} for \code{\link{EMPIRcop}};}
  \item{verbose}{A logical controlling whether the progress during grid building is to be shown; and}
  \item{...}{Additional arguments to pass to \code{\link{EMPIRcop}}.}
}
\value{
A \code{matrix} is returned (\eqn{\ddot{\mathbf{C}}_{n}(u,v)}), if \code{gridonly} is true. Otherwise a \code{list} of the \eqn{u}, \eqn{v}, and \eqn{\ddot{\mathbf{C}}_{n}(u,v)} or other definitions (\code{ctype} for \code{\link{EMPIRcop}}) is returned. The \code{list} will also contain the \code{deluv} used to generate the grid and if the user passed through \code{...} the \code{ctype} for \code{\link{EMPIRcop}}, then that is preserved, otherwise set at \code{"unknown"}. The \code{list} is returned as \code{classes} of \code{"empirical.copula.grid"} and \code{"list"}. This is classification might be used in \pkg{copBasic} to check for a grid in other functions.
}
\note{
The extensive suite of examples is included here because the various ways that algorithms involving empirical copulas can be tested. The figures also provide excellent tools for education on copulas.
}
\references{
Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in Nature---An approach using copulas: Springer, 289 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{EMPIRcop}}, \code{\link{EMPIRgrid_fast}}, \code{\link{EMPIRgrid_lkup}}, \code{\link{EMPIRcopdf}}}
\examples{
\dontrun{
  # EXAMPLE 1:
  psp <- simCOP(n=490, cop=PSP, ploton=FALSE, points=FALSE) * 150
  # Pretend psp is real data, the * 150 is to clearly get into an arbitrary unit system.

  # The sort=FALSE is critical in the following two calls to pp() from lmomco.
  fakeU <- lmomco::pp(psp[,1], sort=FALSE)   # Weibull plotting position i/(n+1)
  fakeV <- lmomco::pp(psp[,2], sort=FALSE)   # Weibull plotting position i/(n+1)
  uv <- data.frame(U=fakeU, V=fakeV) # our U-statistics

  # The follow function is used to make 3-D --> 2-D transformation
  # From R Graphics by Murrell (2005, p.112)
  "trans3d" <-                         # back slashes seem needed for the package
  function(x,y,z, pmat) {              # for user manual building but bad syntax
    tmat <- cbind(x,y,z,1) \%*\% pmat    # because remember the percent sign is a
    return(tmat[,1:2] / tmat[,4])      # a comment character in LaTeX.
  }

  the.grid <- EMPIRgrid(para=uv)
  cop.diag <- diagCOP(cop=EMPIRcop, para=uv, ploton=FALSE, lines=FALSE)
  empcop   <- EMPIRcopdf(para=uv) # data.frame of all points

  # EXAMPLE 1: PLOT NUMBER 1
  the.persp <- persp(the.grid$empcop, theta=-25, phi=20,
                     xlab="U VARIABLE", ylab="V VARIABLE", zlab="COPULA C(u,v)")

  # EXAMPLE 1: PLOT NUMBER 2 (see change in interaction with variable 'the.grid')
  the.persp <- persp(x=the.grid$u, y=the.grid$v, z=the.grid$empcop, theta=-25, phi=20,
                     xlab="U VARIABLE", ylab="V VARIABLE", zlab="COPULA C(u,v)")

  the.diag <- trans3d(cop.diag$t, cop.diag$t, cop.diag$diagcop, the.persp)
  lines(the.diag, lwd=4, col=3, lty=2)

  points(trans3d(empcop$u, empcop$v, empcop$empcop, the.persp),
       col=rgb(0,1-sqrt(empcop$empcop),1,sqrt(empcop$empcop)), pch=16)
  # the sqrt() is needed to darken or enhance the colors

  S <- sectionCOP(cop=PSP, 0.2, ploton=FALSE, lines=FALSE)
  thelines <- trans3d(rep(0.2, length(S$t)), S$t, S$seccop, the.persp)
  lines(thelines, lwd=2, col=6)
  S <- sectionCOP(cop=PSP, 0.2, ploton=FALSE, lines=FALSE, dercop=TRUE)
  thelines <- trans3d(rep(0.2, length(S$t)), S$t, S$seccop, the.persp)
  lines(thelines, lwd=2, col=6, lty=2)

  S <- sectionCOP(cop=PSP, 0.85, ploton=FALSE, lines=FALSE, wrtV=TRUE)
  thelines <- trans3d(S$t, rep(0.85, length(S$t)), S$seccop, the.persp)
  lines(thelines, lwd=2, col=2)
  S <- sectionCOP(cop=PSP, 0.85, ploton=FALSE, lines=FALSE, dercop=TRUE)
  thelines <- trans3d(S$t, rep(0.85, length(S$t)), S$seccop, the.persp)
  lines(thelines, lwd=2, col=2, lty=2)

  empder <- EMPIRgridder(empgrid=the.grid)
  thelines <- trans3d(rep(0.2, length(the.grid$v)), the.grid$v, empder[3,], the.persp)
  lines(thelines, lwd=4, col=6) #}

\dontrun{
  # EXAMPLE 2:
  # An asymmetric example to demonstrate that the grid is populated with the
  # correct orientation---meaning U is the horizontal and V is the vertical.

  # EXAMPLE 2: PLOT NUMBER 1 # See the asymmetry
  uv <- simCOP(1000, cop=MOcop, para=c(0.4,0.9)) # The parameters cause asymmetry.
  mtext("Simulation from a defined Marshall-Olkin Copula")
  the.grid <- EMPIRgrid(para=uv, deluv=0.025)

  # EXAMPLE 2: PLOT NUMBER 2
  # The second plot by image() will show a "hook" of sorts along the singularity.
  image(the.grid$empcop, col=terrain.colors(40)) # Second plot is made
  mtext("Image of gridded Empirical Copula")

  # EXAMPLE 2: PLOT NUMBER 3
  empcop <- EMPIRcopdf(para=uv) # data.frame of all points
  # The third plot is the 3-D version overlain with the data points.
  the.persp <- persp(x=the.grid$u, y=the.grid$v, z=the.grid$empcop, theta=240, phi=40,
                     xlab="U VARIABLE", ylab="V VARIABLE", zlab="COPULA C(u,v)")
  points(trans3d(empcop$u, empcop$v, empcop$empcop, the.persp),
         col=rgb(0,1-sqrt(empcop$empcop),1,sqrt(empcop$empcop)), pch=16)
  mtext("3-D representation of gridded empirical copula with data points")

  # EXAMPLE 2: PLOT NUMBER 4
  # The fourth plot shows a simulation and the quasi-emergence of the singularity
  # that of course the empirical perspective "knows" nothing about. Do not use
  # the Kumaraswamy smoothing because in this case the singularity because
  # too smoothed out relative to the raw empirical, but of course the sample size
  # is large enough to see such things. (Try kumaraswamy=TRUE)
  empsim <- EMPIRsim(n=1000, empgrid = the.grid, kumaraswamy=FALSE)
  mtext("Simulations from the Empirical Copula") #}

\dontrun{
  # EXAMPLE 3:
  psp <- simCOP(n=4900, cop=PSP, ploton=FALSE, points=FALSE) * 150
  # Pretend psp is real data, the * 150 is to clearly get into an arbitrary unit system.

  # The sort=FALSE is critical in the following two calls to pp() from lmomco.
  fakeU <- lmomco::pp(psp[,1], sort=FALSE)   # Weibull plotting position i/(n+1)
  fakeV <- lmomco::pp(psp[,2], sort=FALSE)   # Weibull plotting position i/(n+1)
  uv <- data.frame(U=fakeU, V=fakeV) # our U-statistics

  # EXAMPLE 3: # PLOT NUMBER 1
  deluv <- 0.0125 # going to cause long run time with large n
  # The small deluv is used to explore solution quality at U=0 and 1.
  the.grid <- EMPIRgrid(para=uv, deluv=deluv)
  the.persp <- persp(x=the.grid$u, y=the.grid$v, z=the.grid$empcop, theta=-25, phi=20,
                     xlab="U VARIABLE", ylab="V VARIABLE", zlab="COPULA C(u,v)")

  S <- sectionCOP(cop=PSP, 1, ploton=FALSE, lines=FALSE)
  thelines <- trans3d(rep(1, length(S$t)), S$t, S$seccop, the.persp)
  lines(thelines, lwd=2, col=2)

  S <- sectionCOP(cop=PSP, 0, ploton=FALSE, lines=FALSE)
  thelines <- trans3d(rep(0, length(S$t)), S$t, S$seccop, the.persp)
  lines(thelines, lwd=2, col=2)

  S <- sectionCOP(cop=PSP, 1, ploton=FALSE, lines=FALSE, dercop=TRUE)
  thelines <- trans3d(rep(1, length(S$t)), S$t, S$seccop, the.persp)
  lines(thelines, lwd=2, col=2, lty=2)

  S <- sectionCOP(cop=PSP, 2*deluv, ploton=FALSE, lines=FALSE, dercop=TRUE)
  thelines <- trans3d(rep(2*deluv, length(S$t)), S$t, S$seccop, the.persp)
  lines(thelines, lwd=2, col=2, lty=2)

  empder <- EMPIRgridder(empgrid=the.grid)
  thelines <- trans3d(rep(2*deluv,length(the.grid$v)),the.grid$v,empder[3,],the.persp)
  lines(thelines, lwd=4, col=5, lty=2)

  pdf("conditional_distributions.pdf")
    ix <- 1:length(attributes(empder)$rownames)
    for(i in ix) {
      u <- as.numeric(attributes(empder)$rownames[i])
      S <- sectionCOP(cop=PSP, u, ploton=FALSE, lines=FALSE, dercop=TRUE)
      # The red line is the true.
      plot(S$t, S$seccop, lwd=2, col=2, lty=2, type="l", xlim=c(0,1), ylim=c(0,1),
           xlab="V, NONEXCEEDANCE PROBABILITY", ylab="V, VALUE")
      lines(the.grid$v, empder[i,], lwd=4, col=5, lty=2) # empirical
      mtext(paste("Conditioned on U=",u," nonexceedance probability"))
   }
  dev.off() #}
}
\keyword{empirical copula}
\keyword{empirical copula (utility)}
\keyword{copulatic surface}
\keyword{gridded empirical copula}

\encoding{utf8}
\name{LMRvuCOP}
\alias{LMRvuCOP}
\title{Conditional L-moments of the Distribution of V given U}
\description{
Compute the \emph{conditional L-moments} of \eqn{V} given a \eqn{U} (the \eqn{X} direction) through the \emph{conditional distribution function} \eqn{F(X)} using the appropriate \emph{partial derivative} of a copula (\eqn{\mathbf{C}(u,v)}) with respect to \eqn{U}. The conditional L-moments are computed for \eqn{\lambda_1} (mean) from \code{\link{EvuCOP}} (refer to mathematics therein) and for \eqn{r \ge 2} are computed by

\deqn{\lambda_r = \int_0^1 F(k) \times (1-F(k)) \times L_r(F(k))\,\mathrm{d}k\mbox{,}}

where \eqn{F(k)} becomes replaced in \pkg{copBasic} with the conditional distribution function of a copula by Nelsen (2006, pp. 13, 40--41) with respect to \eqn{u}:

\deqn{0 \le \frac{\delta}{\delta u} \mathbf{C}(u,v) \le 1\mbox{,}}

which is \code{\link{derCOP}} and \eqn{L_r(u)} is defined (Jones, 2004; Asquith, 2011) as

\deqn{L_r(z) = \frac{1}{r-1}\sum_{j=0}^{r-2} (-1)^j {r-1 \choose j}{r-1 \choose j+1}z^{r-2-j}(1-z)^j\mbox{.}}

Asquith (2011) mistakenly lists the leading multiplier \eqn{1/(1-r)} in lieu of \eqn{1/(1-r)} (correct) as shown above; the \pkg{lmomco} package contains a suitable errata as discovered in \pkg{copBasic} development for \code{LMRvuCOP} and \code{\link{LMRuvCOP}}.
}
\usage{
LMRvuCOP(u=seq(0.01, 0.99, by=0.01), cop=NULL, para=NULL, nsim=1E5,
      subdivisions=200L, rel.tol=.Machine$double.eps^0.25, abs.tol=rel.tol, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{cop}{A copula function with vectorization as in \code{asCOP};}
  \item{para}{Vector of parameters or other data structures, if needed, to pass to the copula;}
  \item{nsim}{Number of simulations for \emph{Monte Carlo integration} if numerical integration were to fail;}
  \item{subdivisions}{Argument of same name passed to \code{integrate()};}
  \item{rel.tol}{Argument of same name passed to \code{integrate()};}
  \item{abs.tol}{Argument of same name passed to \code{integrate()}; and}
  \item{...}{Additional arguments to pass to \code{\link{derCOP}}.}
}
\value{
  Value(s) in a list for the median and conditional L-moments given \eqn{U=u} are returned including
  \item{median}{The conditional median from \code{\link{med.regressCOP}};}
  \item{L1}{The conditional mean \eqn{\lambda_1|U=u} from \code{\link{EvuCOP}};}
  \item{L2}{The conditional L-scale \eqn{\lambda_2|U=u};}
  \item{T3}{The conditional L-skew \eqn{\tau_3|U=u};}
  \item{T4}{The conditional L-kurtosis \eqn{\tau_4|U=u};}}
\references{
Asquith, W.H., 2011, Distributional analysis with L-moment statistics using the R environment for statistical computing: Createspace Independent Publishing Platform, ISBN 978--146350841--8.

Jones, M.C., 2004, On some expressions for variance, covariance, skewness and L-moments:
Journal of Statistical Planning and Inference, v.126, pp. 97--106, \doi{10.1016/j.jspi.2003.09.001}.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{EvuCOP}}, \code{\link{LMRuvCOP}}, \code{\link{derCOP}}}
\examples{
\dontrun{
  para <- list(cop=breveCOP, para=list(cop=CLcop, para=c(10), breve=0.2, reflect=2))
  uv <- simCOP(500, cop=COP, para=para, graphics=FALSE, set.seed=1)

  ustar <- 0.96; set.seed(1)
  v_special <- replicate(10000, simCOPv(ustar, cop=COP, para=para))
  lmr <- lmomco::lmoms(v_special)

  u <- v <- seq(0,1, by=0.001)
  LMRvu <- LMRvuCOP(u, cop=COP, para=para)
  # LMRuv <- LMRuvCOP(v, cop=COP, para=para)
  # LMRvu <- LMRuv # swap here to make a quick study of U given V
  # uv <- uv[,2:1] # swap here to make a quick study of U given V
  # pdf("LMRvuCOP.pdf", height=7, width=7, useDingbats=FALSE)
  par(xpd=NA, las=1, lend=1, col.axis="black", cex.axis=0.9,
      mgp=c(2,0.6,0), mai=c(0.65,0.65,0.40,0.65))
  plot( uv[,1], uv[,2], type="n", lwd=0.8, col="grey50", bty="n", xlim=c(0,1),
        ylim=c(0,1), xlab="", ylab="", xaxs="i", yaxs="i", xaxt="n", yaxt="n")
  lines(rep(ustar, 2), par()$usr[3:4], lty=4, lwd=1.2, col="seagreen4")
  par(mgp=c(2,0.3,0), las=0, col.axis="seagreen4", cex.axis=0.8)
  txt <- latex2exp::TeX(r'($u^\star$)')
  axis(1, at=ustar, label=txt,  lwd=0, lwd.ticks=1.2, tcl=-0.25, col.ticks="seagreen4")
  axis(3, at=ustar, label=txt,  lwd=0, lwd.ticks=1.2, tcl=-0.25, col.ticks="seagreen4")
  axis(1, at=ustar, label=FALSE,lwd=0, lwd.ticks=1.2, tcl=+0.25, col.ticks="seagreen4")
  axis(3, at=ustar, label=FALSE,lwd=0, lwd.ticks=1.2, tcl=+0.25, col.ticks="seagreen4")
  par(mgp=c(2,0.6,0), las=1, col.axis="black", cex.axis=0.9)
  points(uv[,1], uv[,2], lwd=0.9, pch=21, cex=0.8, col="wheat4", bg="wheat1")
  axis(1, at=axTicks(1), label=TRUE,  lwd=1, lwd.ticks=1, col.ticks="black")
  axis(3, at=axTicks(1), label=FALSE, lwd=1, lwd.ticks=1, col.ticks="black")
  tix <- seq(0.1, 0.9, by=0.2)
  axis(1, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="black", tcl=-0.5)
  axis(2, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="black", tcl=-0.5)
  axis(3, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="black", tcl=-0.5)
  tix <- seq(0.05, 0.95, by=0.05); tix <- tix[grep("5$", as.character(tix))]
  axis(1, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="black", tcl=-0.2)
  axis(2, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="black", tcl=-0.2)
  axis(3, at=tix, label=FALSE, lwd=0, lwd.ticks=1, col.ticks="black", tcl=-0.2)
  mtext("U, nonexceedance probability", side=1, line=1.6)
  par(mgp=c(2,0.8,0))
  axis(2, at=axTicks(2), label=TRUE,  lwd=1, lwd.ticks=1, col.ticks="black")
  par(las=0)
  mtext("V, nonexceedance probability", side=2, line=2.1)
  par(las=1)
  lines(u, LMRvu$L1,     lwd=2, col="black"  ) # mean probability
  lines(u, LMRvu$median, lwd=2, col="red1"   ) # median probability
  lines(u, LMRvu$L2,     lwd=2, col="deepskyblue3") # conditional L-scale
  points(ustar,   mean(v_special), cex=1.7, pch=1,  col="black"  )
  points(ustar, median(v_special), cex=1.7, pch=1,  col="red1"   )
  points(ustar, lmr$lambdas[1],    cex=1.3, pch=16, col="black"  )
  points(ustar, lmr$lambdas[2],    cex=1.3, pch=16, col="deepskyblue3")
  par(new=TRUE)
  plot(u, LMRvu$T3, type="n", bty="n",xlim=c(0,1), ylim=c(-1, 1),
           xlab="", ylab="", xaxs="i", yaxs="i", xaxt="n", yaxt="n")
  par(col.axis="orchid3", las=0, lend=1); colo <- "orchid3"
  mtext("Lskew and Lkurtosis, dimensionless", 4, line=2.15, col=colo)
  par(las=1)
  axis(4, at=axTicks(2), label=TRUE, lwd=2, lwd.ticks=1, col=colo, col.ticks=colo)
  tix <- seq(-0.9, 0.9, by=0.1)
  tix <- tix[grep("[05]$", as.character(tix), invert=TRUE)]
  axis(4, at=tix, label=FALSE, lwd=0, lwd.ticks=1, tcl=-0.5, col.ticks=colo)
  lines(u, LMRvu$T3, col=colo, lwd=2) # conditional L-skew
  lines(u, LMRvu$T4, col=colo, lty=2, lwd=2) # conditional L-kurtosis
  points(ustar, lmr$ratios[ 3],    cex=1.2,   pch=24, col=colo, bg="orchid1")
  points(ustar, lmr$ratios[ 4],    cex=1.2,   pch=25, col=colo, bg="orchid1")
  txt <- c(paste0("Conditional (Cond.) mean at ustar = ", ustar, ", EvuCOP(...)"),
           "Cond. median at ustar, med.regressCOP(...)",
           "Cond. Lscale (variation) at ustar, LMRvuCOP(...)",
           "Cond. Lskew at ustar, LMRvuCOP(...)",
           "Cond. Lkurtosis at ustar, LMRvuCOP(...)",
           "Demonstration of n=500 sim. values for the copula",
           "Cond. simulated (sim.) (n=10,000) sample mean, mean(...)",
           "Cond. sim. sample median, median(...)",
           "Cond. sim. sample mean, lmoms(...)$lambdas[1]",
           "Cond. sim. sample Lscale, lmoms(...)$lambdas[2]",
           "Cond. sim. sample Lskew, lmoms(...)$ratios[3]",
           "Cond. sim. sample Lkurtosis, lmoms(...)$ratios[4]")
  legend("topleft", txt, bty="n", cex=0.6, seg.len=4, y.intersp=1.3,
         lwd=c(2,2,2,2,2, NA, rep(NA, 6)), lty=c(1,1,1,1,2,0,rep(0, 6)),
         pch=c(rep(NA, 5), 21, 1, 1, 16, 16, 24, 25),
         pt.lwd=c(rep(NA,5), 0.9, rep(1,6)),
         pt.cex=c(rep(NA,5), 0.8, 1.7, 1.7, 1.3, 1.3, 1.2, 1.2),
         pt.bg=c(rep(NA,5), "wheat1", rep(NA,4), "orchid1", "orchid1"),
         col=c("black", "red1", "deepskyblue3", colo, colo,
         "wheat4", "black", "red1", "black", "deepskyblue3", colo, colo))
  par(xpd=NA, las=1, lend=1, col.axis="black", cex.axis=0.9,
      mgp=c(2,0.6,0), mai=c(0.65,0.65,0.40,0.65))
  # dev.off() # }
}
\keyword{copula (utility)}
\keyword{copula operator}
\keyword{copula (conditional distribution)}
\keyword{conditional expectation}
\keyword{L-moments (conditional) of V given U}
\keyword{Conditional L-moments of V given U}

\encoding{utf8}
\name{AMHcop}
\alias{AMHcop}
\title{The Ali--Mikhail--Haq Copula}
\description{
The \emph{Ali--Mikhail--Haq copula} (Nelsen, 2006, pp. 92--93, 172) is
\deqn{\mathbf{C}_{\Theta}(u,v) = \mathbf{AMH}(u,v) = \frac{uv}{1 - \Theta(1-u)(1-v)}\mbox{,}}
where \eqn{\Theta \in [-1,+1)}, where the right boundary,
\eqn{\Theta = 1}, can sometimes be considered valid according to \enc{Mächler}{Machler} (2014). The copula \eqn{\Theta \rightarrow 0} becomes the \emph{independence copula} (\eqn{\mathbf{\Pi}(u,v)}; \code{\link{P}}), and the parameter \eqn{\Theta} is readily computed from a \emph{Kendall Tau} (\code{\link{tauCOP}}) by
\deqn{\tau_\mathbf{C} = \frac{3\Theta - 2}{3\Theta} -
                        \frac{2(1-\Theta)^2\log(1-\Theta)}{3\Theta^2}\mbox{,}}
and by \emph{Spearman Rho} (\code{\link{rhoCOP}}), through \enc{Mächler}{Machler} (2014), by
\deqn{\rho_\mathbf{C} = \sum_{k=1}^\infty \frac{3\Theta^k}{{k + 2 \choose 2}^2}\mbox{.}}

The support of \eqn{\tau_\mathbf{C}} is \eqn{[(5 - 8\log(2))/3, 1/3] } \eqn{\approx} \eqn{[-0.1817258, 0.3333333]} and the \eqn{\rho_\mathbf{C}} is \eqn{[33 - 48\log(2), 4\pi^2 - 39]} \eqn{\approx} \eqn{[-0.2710647, 0.4784176]}, which shows that this copula has a limited range of dependency. The infinite summation is easier to work with than
Nelsen (2006, p. 172) definition of
\deqn{\rho_\mathbf{C} = \frac{12(1+\Theta)}{\Theta^2}\mathrm{dilog}(1-\Theta)-
                        \frac{24(1-\Theta)}{\Theta^2}\log(1-\Theta)-
                        \frac{3(\Theta+12)}{\Theta}\mbox{,}}
where the \eqn{\mathrm{dilog(x)}} is the dilogarithm function defined by
\deqn{\mathrm{dilog}(x) = \int_1^x \frac{\log(t)}{1-t}\,\mathrm{d}t\mbox{.}}
The integral version has more nuances with approaches toward \eqn{\Theta = 0} and \eqn{\Theta = 1} than the infinite sum version.
}
\usage{
AMHcop(u, v, para=NULL, rho=NULL, tau=NULL, fit=c("rho", "tau"), ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (single element) of parameters---the \eqn{\Theta} parameter of the copula. However, if a second parameter is present, it is treated as a logical to reverse the copula (\eqn{u + v - 1 + \mathbf{AMH}(1-u,1-v; \Theta)});}
  \item{rho}{Optional Spearman Rho from which the parameter will be estimated and presence of \code{rho} trumps \code{tau};}
  \item{tau}{Optional Kendall Tau from which the parameter will be estimated;}
  \item{fit}{If \code{para}, \code{rho}, and \code{tau} are all \code{NULL}, the the \code{u} and \code{v} represent the sample. The measure of association by the \code{fit} declaration will be computed and the parameter estimated subsequently. The \code{fit} has not other utility than to trigger which measure of association is computed internally by the \code{cor} function in \R; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned. Otherwise if \code{tau} is given, then the \eqn{\Theta} is computed and a \code{list} having
  \item{para}{The parameter \eqn{\Theta}, and}
  \item{tau}{Kendall Tau.}
and if \code{para=NULL} and \code{tau=NULL}, then the values within \code{u} and \code{v} are used to compute Kendall Tau and then compute the parameter, and these are returned in the aforementioned list.
}

\note{
\enc{Mächler}{Machler} (2014) reports on accurate computation of \eqn{\tau_\mathbf{C}} and \eqn{\rho_\mathbf{C}} for this copula for conditions of \eqn{\Theta \rightarrow 0} and in particular derives the following equation, which does not have \eqn{\Theta} in the denominator:
\deqn{\rho_\mathbf{C} = \sum_{k=1}^{\infty} \frac{3\Theta^k}{{k+2 \choose 2}^2}\mbox{.}}
The \pkg{copula} package provides a Taylor series expansion for \eqn{\tau_\mathbf{C}} for small \eqn{\Theta} in the \code{copula::tauAMH()}. This is demonstrated here between the implementation of \eqn{\tau = 0} for parameter estimation in the \pkg{copBasic} package to that in the more sophisticated implementation in the \pkg{copula} package.
\preformatted{
  copula::tauAMH(AMHcop(tau=0)$para) # theta = -2.313076e-07
}
It is seen that the numerical approaches yield quite similar results for small \eqn{\tau_\mathbf{C}}, and finally, a comparison to the \eqn{\rho_\mathbf{C}} is informative:
\preformatted{
  rhoCOP(AMHcop, para=1E-9)    # 3.333333e-10 (two nested integrations)
  copula:::.rhoAmhCopula(1E-9) # 3.333333e-10 (cutoff based)
  theta <- seq(-1,1, by=.0001)
  RHOa <- sapply(theta, function(t) rhoCOP(AMHcop, para=t))
  RHOb <- sapply(theta, function(t) copula:::.rhoAmhCopula(t))
  plot(10^theta, RHOa-RHOb, type="l", col=2)
}
The plot shows that the apparent differences are less than 1 part in 100 million---The \pkg{copBasic} computation is radically slower though, but \code{\link{rhoCOP}} was designed for generality of copula family.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

\enc{Mächler}{Machler}, Martin, 2014, Spearman's Rho for the AMH copula---A beautiful formula: copula package vignette, accessed on April 7, 2018, at \url{https://CRAN.R-project.org/package=copula} under the vignette \emph{rhoAMH-dilog.pdf}.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Pranesh, Kumar, 2010, Probability distributions and estimation of Ali--Mikhail--Haq copula: Applied Mathematical Sciences, v. 4, no. 14, p. 657--666.
}
\author{W.H. Asquith}
\seealso{\code{\link{P}}
}
\examples{
\dontrun{
  t <- 0.9 # The Theta of the copula and we will compute Spearman Rho.
  di <- integrate(function(t) log(t)/(1-t), lower=1, upper=(1-t))$value
  A <- di*(1+t) - 2*log(1-t) + 2*t*log(1-t) - 3*t # Nelsen (2007, p. 172)
  rho <- 12*A/t^2 - 3    # 0.4070369
  rhoCOP(AMHcop, para=t) # 0.4070369
  sum(sapply(1:100, function(k) 3*t^k/choose(k+2, 2)^2)) # Machler (2014)
  # 0.4070369 (see Note section, very many tens of terms are needed) }

\dontrun{
  layout(matrix(1:2, byrow=TRUE)) # Note: Kendall Tau is same on reversal.
  s <- 2; set.seed(s); nsim <- 10000
  UVn <- simCOP(nsim, cop=AMHcop, para=c(-0.9, "FALSE" ), col=4)
  mtext("Normal definition [default]") # '2nd' parameter could be skipped
  set.seed(s) # seed used to keep Rho/Tau inside attainable limits
  UVr <- simCOP(nsim, cop=AMHcop, para=c(-0.9, "TRUE"),   col=2)
  mtext("Reversed definition")
  AMHcop(UVn[,1], UVn[,2], fit="rho")$rho # -0.2581653
  AMHcop(UVr[,1], UVr[,2], fit="rho")$rho # -0.2570689
  rhoCOP(cop=AMHcop, para=-0.9)           # -0.2483124
  AMHcop(UVn[,1], UVn[,2], fit="tau")$tau # -0.1731904
  AMHcop(UVr[,1], UVr[,2], fit="tau")$tau # -0.1724820
  tauCOP(cop=AMHcop, para=-0.9)           # -0.1663313 }
}
\keyword{Ali--Mikhail--Haq copula}
\keyword{copula (formulas)}
\keyword{copula}
\concept{Ali-Mikhail-Haq copula}
\keyword{package copula (comparison to)}

\encoding{utf8}
\name{EMPIRgridder}
\alias{EMPIRgridder}
\title{Derivatives of the Grid of the Bivariate Empirical Copula for V with respect to U}
\description{
Generate derivatives of \eqn{V} with respect to \eqn{U} of a gridded representation of the \emph{bivariate empirical copula} (see \code{\link{EMPIRcop}}). This function is the empirical analog to \code{\link{derCOP}}.
}
\usage{
EMPIRgridder(empgrid=NULL, ...)
}
\arguments{
  \item{empgrid}{The grid from \code{\link{EMPIRgrid}}; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  The gridded values of the derivatives of the bivariate empirical copula.
}
\author{W.H. Asquith}
\seealso{\code{\link{EMPIRcop}}, \code{\link{EMPIRcopdf}}, \code{\link{EMPIRgrid}}, \code{\link{EMPIRgridder2}}}
\examples{
\dontrun{
para   <- list(alpha=0.15,  beta=0.65, cop1=PLACKETTcop, cop2=PLACKETTcop,
               para1=0.005, para2=1000)
uv <- simCOP(n=1000, cop=composite2COP, para=para)
fakeU <- lmomco::pp(uv[,1], sort=FALSE)
fakeV <- lmomco::pp(uv[,2], sort=FALSE)
uv <- data.frame(U=fakeU, V=fakeV)

"trans3d" <-                         # back slashes seem needed for the package
function(x,y,z, pmat) {              # for user manual building but bad syntax
  tmat <- cbind(x,y,z,1) \%*\% pmat    # because remember the percent sign is a
  return(tmat[,1:2] / tmat[,4])      # a comment character in LaTeX.
}

the.grid <- EMPIRgrid(para=uv, deluv=0.1)
the.diag <- diagCOP(cop=EMPIRcop, para=uv, ploton=FALSE, lines=FALSE)
empcop <- EMPIRcopdf(para=uv) # data.frame of all points

the.persp <- persp(the.grid$empcop, theta=-25, phi=20,
                   xlab="U VARIABLE", ylab="V VARIABLE", zlab="COPULA C(u,v)")
points(trans3d(empcop$u, empcop$v, empcop$empcop, the.persp),
       col=rgb(0,1-sqrt(empcop$empcop),1,sqrt(empcop$empcop)), pch=16, cex=0.75)

# Now extract the copula sections
some.lines <- trans3d(rep(0.2, length(the.grid$v)),
                      the.grid$v, the.grid$empcop[3,], the.persp)
lines(some.lines, lwd=2, col=2)
some.lines <- trans3d(the.grid$u, rep(0.6, length(the.grid$u)),
                      the.grid$empcop[,7], the.persp)
lines(some.lines, lwd=2, col=3)
some.lines <- trans3d(rep(0.7, length(the.grid$v)), the.grid$v,
                      the.grid$empcop[8,], the.persp)
lines(some.lines, lwd=2, col=6)

# Now compute some derivatives or conditional cumulative
# distribution functions
empder <- EMPIRgridder(empgrid=the.grid)
some.lines <- trans3d(rep(0.2, length(the.grid$v)), the.grid$v, empder[3,], the.persp)
lines(some.lines, lwd=4, col=2)

empder <- EMPIRgridder2(empgrid=the.grid)
some.lines <- trans3d(the.grid$u, rep(0.6, length(the.grid$u)), empder[,7], the.persp)
lines(some.lines, lwd=4, col=3)

empder <- EMPIRgridder(empgrid=the.grid)
some.lines <- trans3d(rep(0.7, length(the.grid$v)), the.grid$v, empder[8,], the.persp)
lines(some.lines, lwd=4, col=6)

# Demonstrate conditional quantile function extraction for
# the 70th percentile of U and see how it plots on top of
# the thick purple line
empinv <- EMPIRgridderinv(empgrid=the.grid)
some.lines <- trans3d(rep(0.7, length(the.grid$v)), empinv[8,],
                      attributes(empinv)$colnames, the.persp)
lines(some.lines, lwd=4, col=5, lty=2)#}
}
\keyword{empirical copula}
\keyword{empirical copula (derivatives)}
\keyword{derivative}
\keyword{copulatic surface}
\keyword{gridded empirical copula}

\encoding{utf8}
\name{N4220cop}
\alias{N4220cop}
\title{The Copula of Equation 4.2.20 of Nelsen's Book}
\description{
The \emph{N4220 copula} (Nelsen, 2006, p. 118; eq. 4.2.20) is named by the author (Asquith) for the \pkg{copBasic} package and is defined as

\deqn{\mathbf{C}_{\mathrm{N4220}}(u,v; \Theta) = \biggl[\mathrm{log}\biggl( \mathrm{exp}\bigl(u^{-\Theta}\bigr) +
\mathrm{exp}\bigl(v^{-\Theta}\bigr) -
\mathrm{exp}(1)\biggr)\biggr]^{-1/\Theta}\mbox{.}}

The \eqn{\mathbf{N4220}(u,v)} copula is \emph{not comprehensive} because for \eqn{\Theta = 0^{+}} the copula becomes the \emph{independence} \eqn{\mathbf{P}(u,v)} copula (\code{\link{P}}) and as \eqn{\Theta \rightarrow \infty} the copula becomes \eqn{\mathbf{M}(u,v)} (\code{\link{M}}). The copula is undefined for \eqn{\Theta < 0}. This copula has \emph{lower-tail dependency}, which is the reverse of the \eqn{\mathbf{rN4220}(u,v)} that has \emph{uppr-tail dependency.} Hence, a reflected (refer to \code{\link{COP}}, \code{reflect=2}) version of this copula is natively supported as the \code{\link{rN4220cop}} to put the tail dependency into the upper-right corner and more \dQuote{hydrologic-system like.}
}
\usage{
N4220cop(u, v, para=NULL, insertM=TRUE, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A vector (single element) of parameters---the \eqn{\Theta} parameter of the copula;}
  \item{insertM}{For the tail that pinches out towards \eqn{\mathbf{M}(u,v)} (\code{\link{M}}) use an \emph{ad hoc} algorithm to insert that copula into the tail (refer to source code) and that algorithm is dependent on the \code{\link{simCOP}} and the numerical partial derivative inversion for simulation that the package is based around; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\examples{
sapply(c(0.1, 1, 10), function(p) N4220cop(0.4, 0.6, para=p))
# 0.2611771 0.3714941 0.4000000 # perfect correlation controls for p=10
}
\keyword{copula (formulas)}
\keyword{copula}

\encoding{utf8}
\name{stabtaildepf}
\alias{stabtaildepf}
\title{Estimation of the Stable Tail Dependence Function}
\description{
Kiriliouk \emph{et al.} (2016, pp. 364--366) describe a technique for estimation of a \emph{empirical stable tail dependence function} for a random sample. The function is defined as
\deqn{\widehat{l}(x,y) = \frac{1}{k}\sum_{i=1}^n \mathbf{1}\bigl[ R_{i,x,n} > n + 1 - kx \mbox{\ or\ } R_{i,y,n} > n + 1 - ky \bigr]\mbox{,}}
where \eqn{\mathbf{1}[\cdot]} is an \emph{indicator function}, \eqn{R} denotes the \code{rank()} of the elements and \eqn{k \in [1,\ldots,n]} and \eqn{k} is intended to be \dQuote{large enough} that \eqn{\widehat{l}(x,y)} has converged to a limit.

The \dQuote{\enc{Capéraà}{Caperaa}--\enc{Fougères}{Fougeres} smooth} of the empirical stable tail dependence function is defined for a coordinate pair \eqn{(x,y)} as
\deqn{\widehat{l}_{CF}(x,y) = 2 \sum_{i \in I_n} \widehat{p}_{3,i} \times \mathrm{max}\bigl[\widehat{W}_i x,\, (1-\widehat{W}_i) y \bigr]\mbox{,}}
where \eqn{\widehat{p}_{3,i}} are the weights for the \emph{maximum Euclidean likelihood} estimator (see \code{\link{spectralmeas}}) and \eqn{\widehat{W}_i} are the \emph{pseudo-polar angles} (see \code{\link{spectralmeas}}) for the index set \eqn{I_n} defined by \eqn{I_n = \{i = 1, \ldots, n : \widehat{S}_i > \widehat{S}_{(k{+}1)}\}}, where \eqn{\widehat{S}_{(k+1)}} denotes the \eqn{(k{+}1)}-th largest observation of the pseudo-polar radii \eqn{\widehat{S}_i} where the cardinality of \eqn{I_n} is exactly \eqn{k} elements long. (Tentatively, then this definition of \eqn{I_n} is ever so slightly different than in \code{\link{spectralmeas}}.)  Lastly, see the multiplier of \eqn{2} on the smooth form, and this multiplier is missing in Kiriliouk \emph{et al.} (2016, p. 365) but shown in Kiriliouk \emph{et al.} (2016, eq. 17.14, p. 360). Numerical experiments indicate that the \eqn{2} is needed for \eqn{\widehat{l}_{CF}(x,y)} but evidently not in \eqn{\widehat{l}(x,y)}.

The visualization of \eqn{l(x,y)} commences by setting a constant (\eqn{c > 0}) as \eqn{c_i \in 0.2,0.4,0.6,0.8} (say). The \eqn{y} are solved for \eqn{x \in [0,\ldots,c_i]} through the \eqn{l(x,y)} for each of the \eqn{c_i}. Each solution set constitutes a \emph{level set} for the stable tail dependence function. If the bivariate data have \emph{asymptotic independence} (to the right), then a level set or the level sets for all the \eqn{c} are equal to the lines \eqn{x + y = c}. Conversely, if the bivariate data have \emph{asymptotic dependence} (to the right), then the level sets will make 90-degree bends for \eqn{\mathrm{max}(x,y) = c}.
}
\usage{
stabtaildepf(uv=NULL, xy=NULL, k=function(n) as.integer(0.5*n), levelset=TRUE,
             ploton=TRUE, title=TRUE, delu=0.01, smooth=FALSE, ...)
}
\arguments{
  \item{uv}{An \R \code{data.frame} of \eqn{u} and \eqn{v} nonexceedance probabilities in the respective \eqn{X} (horizontal) and \eqn{Y} (vertical) directions. Note, \code{rank()}s are called on these so strictly speaking this need not be as nonexceedance probabilities. This is not an optional argument;}
  \item{xy}{A vector of the scalar coordinates \eqn{(x,y)}, which are \dQuote{the relative distances to the upper endpoints of [these respective] variables} (Kiriliouk \emph{et al.}, 2016, p. 356). This is a major point of nomenclature confusion. If these are in probability units, they are \emph{exceedance probabilities}. Though tested for \code{NULL} and a warning issued, these can be \code{NULL} only if \code{levelset=TRUE} but can be set to \code{xy=NA} if \code{levelset=FALSE} and \code{smooth=TRUE} (see discussion in \bold{Note});}
  \item{k}{The \eqn{k} for both the \eqn{\widehat{l}(x,y)} and \eqn{\widehat{l}_{CF}}, though the effect of \eqn{k} might not quite be the same for each. The default seems to work fairly well;}
  \item{levelset}{A logical triggering the construction of the level sets for \eqn{c} \eqn{=} \code{seq(0.1,} \code{1,} \code{by=0.1)};}
  \item{ploton}{A logical to call the \code{plot()} function;}
  \item{title}{A logical to trigger a title for the plot if \code{ploton=TRUE};}
  \item{delu}{The \eqn{\Delta x} for a sequence of \eqn{x} \eqn{=} \code{seq(0,c,by=delu)};}
  \item{smooth}{A logical controlling whether \eqn{\widehat{l}(x,y)} or the \enc{Capéraà}{Caperaa}--\enc{Fougères}{Fougeres} smooth function \eqn{\widehat{l}_{CF}(x,y)} is used; and}
  \item{...}{Additional arguments to pass.}
}
\value{
Varies according to argument settings. In particular, the \code{levelset=TRUE} will cause an \R \code{list} to be returned with the elements having the character string of the respective \eqn{c} values and the each holding a \code{data.frame} of the \eqn{(x,y)} coordinates.
}
\note{
This function is also called in a secondary recursion mode. The default \code{levelset=TRUE} makes a secondary call with \code{levelset=FALSE} to compute the \eqn{\widehat{l}(x,y)} for the benefit of the looping on the one-dimensional root to solve for a single \eqn{y} in \eqn{\widehat{l}(x,y) = c} given a single \eqn{x}. If \code{levelset=TRUE} and \code{smooth=TRUE}, then a secondary call with \code{smooth=TRUE} and \code{levelset=FALSE} is made to internally return an \R \code{list} containing scalar \eqn{N_n} and vectors \eqn{\widehat{W}_n} and \eqn{\widehat{p}_{3,i}} for similar looping and one-dimensional rooting for \eqn{\widehat{l}_{CF}(x,y)}.

If \code{levelset=FALSE}, then \code{xy} is required to hold the \eqn{(x,y)} coordinate pair of interest. A demonstration follows and shows the limiting behavior of a random sample from the \code{\link{N4212cop}} copula.
\preformatted{
  n <- 2000 # very CPU intensive this and the next code snippet
  UV <- simCOP(n=n, cop=N4212cop, para=pi); k <- 1:n
  lhat <- sapply(k, function(j)
                 stabtaildepf(xy=c(0.1, 0.1), uv=UV, levelset=FALSE, k=j))
  plot(k, lhat, xlab="k in [1,n]", cex=0.8, lwd=0.8, type="b",
                ylab="Empirical Stable Tail Dependence Function")
  mtext("Empirical function in the 0.10 x 0.10 Pr square (upper left corner)")
}

The \R \code{list} that can be used to compute \eqn{\widehat{l}_{CF}(x,y)} is retrievable by
\preformatted{
  x <- 0.1; y <- 0.1; k <- 1:(n-1)
  lhatCF <- sapply(k, function(j) {
     Hlis <- stabtaildepf(xy=NA, uv=UV, levelset=FALSE, smooth=TRUE, k=j)
     2*sum(Hlis$p3 * sapply(1:Hlis$Nn, function(i) {
                 max(c(Hlis$Wn[i]*x, (1-Hlis$Wn[i])*y)) }))
  })
  lines(k, lhatCF, col="red")
}
The smooth line (red) of \code{lhatCF} is somewhat closer to the limiting behavior of \code{lhat}, but it is problematic to determine computational consistency. Mathematical consistency with Kiriliouk \emph{et al.} (2016) appears to be achieved. The \bold{Examples} section TODO.
}
\references{
Beirlant, J., Escobar-Bach, M., Goegebeur, Y., Guillou, A., 2016, Bias-corrected estimation of stable tail dependence function: Journal Multivariate Analysis, v. 143, pp. 453--466, \doi{10.1016/j.jmva.2015.10.006}.

Kiriliouk, Anna, Segers, Johan, \enc{Warchoł}{Warchol}, \enc{Michał}{Michal}, 2016, Nonparameteric estimation of extremal dependence: \emph{in} Extreme Value Modeling and Risk Analysis, D.K. Dey and Jun Yan \emph{eds.}, Boca Raton, FL, CRC Press, ISBN 978--1--4987--0129--7.
}
\author{William Asquith \email{william.asquith@ttu.edu}}
\seealso{\code{\link{psepolar}}, \code{\link{spectralmeas}}
}
\examples{
\dontrun{
UV <- simCOP(n=1200, cop=GLcop, para=2.1) # Galambos copula
tmp1 <- stabtaildepf(UV) # the lines are curves (strong tail dependence)
tmp2 <- stabtaildepf(UV, smooth=TRUE, ploton=FALSE, col="red") #}
}
\keyword{stable tail dependence function}
\keyword{tail dependence}
\keyword{literature errors and inconsistencies}

\encoding{utf8}
\name{sectionCOP}
\alias{sectionCOP}
\title{The Sections or Derivative of the Sections of a Copula}
\description{
Compute the \emph{copula sections} or the \emph{(partial) derivatives of copula sections} of a copula (Nelsen, 2006, pp. 12--14). The \emph{horizontal section} at \eqn{V=a} (a constant) is
\deqn{t \mapsto \mathbf{C}(t,a)\mbox{, and}}
the \emph{vertical  section} at \eqn{U=a} (a constant, with respect to \eqn{V} or \code{wrtV=TRUE}) is
\deqn{t \mapsto \mathbf{C}(a,t)\mbox{.}}
The partial derivatives of the copula sections are \emph{conditional cumulative distribution functions} (see \code{\link{derCOP}} and \code{\link{derCOP2}}). The derivatives are constrained as
\deqn{0 \le \frac{\delta}{\delta u}\mathbf{C}(u,v) \le 1\mbox{, and}}
\deqn{0 \le \frac{\delta}{\delta v}\mathbf{C}(u,v) \le 1\mbox{.}}
}
\usage{
sectionCOP(f, cop=NULL,  para=NULL, wrtV=FALSE, dercop=FALSE, delt=0.005,
              ploton=TRUE, lines=TRUE, xlab="NONEXCEEDANCE PROBABILITY", ...)
}
\arguments{
  \item{f}{A single value of nonexceedance probability \eqn{u} or \eqn{v} along the horizontal \eqn{U} axis or vertical \eqn{V} axis of the unit square \eqn{\mathcal{I}^2};}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters, if needed, to pass to the copula;}
  \item{wrtV}{A logical to toggle between with respect to \eqn{v} or \eqn{u} (default). The default provides the vertical section whereas the horizontal comes from \code{wrtV = TRUE};}
  \item{dercop}{A logical that triggers the derivative of the section;}
  \item{delt}{The increment of the level curves to plot, defaults to 5-percent intervals;}
  \item{ploton}{A logical to toggle on the plot;}
  \item{lines}{Draw the lines of diagonal to the current device;}
  \item{xlab}{A label for the x-axis title passed to \code{plot()} in \R; and}
  \item{...}{Additional arguments to pass to the \code{plot()} and \code{lines()} functions in \R.}
}
\value{
  An \R \code{list} is returned.
  \item{t}{The nonexceedance probability along the section. The nomenclature \eqn{t} mimics Nelsen (2006) and is \emph{not} the same as the \eqn{u} or \eqn{v};}
  \item{seccop}{The section of the copula or its derivative;}
  \item{wrt}{A text string declaring what the setting for \code{wrtV} was;}
  \item{fvalue}{The provided value of nonexceedance probability; and}
  \item{isderivative}{A logical stating whether the derivative of the section is \code{seccop}.}
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{COP}}, \code{\link{diagCOP}}}
\examples{
\dontrun{
# EXAMPLE 1, plot the v=0.55 section and then u=0.55 section, which will overlay
# the other because the PSP is a symmetrical copula
tmp <- sectionCOP(0.55, cop=PSP, ylab="COPULA SECTIONS",  lwd=5, col=2)
tmp <- sectionCOP(0.55, cop=PSP, wrtV=TRUE, ploton=FALSE, lwd=2, col=3)
# now add the v=0.85 section and the u=0.85, again overlay each other
tmp <- sectionCOP(0.85, cop=PSP, ploton=FALSE,             lwd=5, col=2, lty=2)
tmp <- sectionCOP(0.85, cop=PSP, wrtV=TRUE, ploton=FALSE,  lwd=2, col=3, lty=2)#}

\dontrun{
# EXAMPLE 2, v=0.35 section and derivative (the conditional distribution) function
tmp <- sectionCOP(0.35, cop=PSP, ylab="COPULA SECTIONS OR DERIV.", lwd=5, col=3)
tmp <- sectionCOP(0.35, cop=PSP, dercop=TRUE, ploton=FALSE,               col=3)
# The thin green line represents the cumulative distribution function conditional
# on u = 0.35 from the derCOP function.  Then see Example 3}

\dontrun{
# EXAMPLE 3 (random selection commented out)
#para <- list(cop1=PLACKETTcop,  cop2=PLACKETTcop, alpha=runif(1), beta=runif(1),
#             para1=10^runif(1,min=-4, max=0), para2=10^runif(1,min= 0, max=4))
para <- list(cop1=PLACKETTcop,  cop2=PLACKETTcop, alpha=0.7, beta=0.22,
             para1=0.0155, para2=214.4)
txts <- c("Alpha=",    round(para$alpha,    digits=4),
          "; Beta=",   round(para$beta,     digits=4),
          "; Theta1=", round(para$para1[1], digits=5),
          "; Theta2=", round(para$para2[1], digits=2))
layout(matrix(1:2,byrow=TRUE))
D <- simCOP(n=1000, cop=composite2COP, para=para, cex=0.5, col=rgb(0,0,0,0.2), pch=16)
mtext(paste(txts,collapse=""))
#f <- c(runif(1),runif(1))
f <- c(0.2,0.9) # RED is the horizontal section and BLACK is the vertical section
segments(f[1],0,f[1],1, col=2, lwd=2); segments(0,f[2],1,f[2], lwd=2)
ftxt <- c("Sections (thick) and derivatives (thin) at ", f, " nonexceed. prob.")
tmp <- sectionCOP(f[1],cop=composite2COP,para=para, col=2, lwd=4)
tmp <- sectionCOP(f[1],cop=composite2COP,para=para, dercop=TRUE, ploton=FALSE, col=2)
tmp <- sectionCOP(f[2],cop=composite2COP,para=para,wrtV=TRUE,ploton=FALSE,lwd=4)
tmp <- sectionCOP(f[2],cop=composite2COP,para=para,wrtV=TRUE,ploton=FALSE,dercop=TRUE)
mtext(paste(ftxt, collapse=""))
# The thin lines are the CDFs conditional on the respective values of "f". Carefully
# compare the point densities along and near the sections in the top plot to the
# respective shapes of the CDFs in the bottom plot. If the bottom plot were rotated
# 90 degrees clockwise and then reflected top to bottom, the conditional quantile
# function QDF results. Reflection is needed because, by convention, QDFs are monotonic
# increasing to right---functions derCOPinv() and derCOPinv2() provide the CDF inversion.}
}
\keyword{visualization}
\keyword{diagnostics}
\keyword{copula section}

\encoding{utf8}
\name{simCOP}
\alias{simCOP}
\alias{rCOP}
\title{Simulate a Copula by Numerical Derivative Method}
\description{
Perform a simulation and visualization of a copula using numerical partial derivatives of the copula (Nelsen, 2006, p. 32). The method is more broadly known as \emph{conditional simulation method}. Because a focus of \pkg{copBasic} is on copula theory for pedagogic purposes, the coupling between simulation and subsequent visualization is emphasized by this function by it providing for both simulation and plotting operations by default.

The \code{simCOP} function is based on a uniformly simulating nonexceedance probability \eqn{u} and then conditioning the \eqn{v} from the inverse of the sectional derivative for \eqn{V} with respect to \eqn{U} (see \code{\link{derCOPinv}}). The function for speed will only report a warning if at least one of the requested simulations in \code{n} could not be made because of \code{uniroot}'ing problems in \code{\link{derCOPinv}}. The returned \code{data.frame} will be shortened automatically, but this can be controlled by \code{na.rm}. Failure of a simulation is purely dependent failure of the derivative inversion. In general, inversion should be quite robust for continuous or near continuous copulas and even copulas with singularities should be more or less okay. Lastly, the logical combination \code{na.rm=FALSE} and \code{keept=TRUE} could be used to isolate those combinations giving \code{\link{derCOPinv}} problems. The implemented simulation method in the \pkg{copBasic} package is known as the \emph{conditional distribution method} (Nelsen, 2006; pp. 40--41), \emph{conditional method}, or \emph{Rosenblatt transform} (Joe, 2014, p. 270).
}
\usage{
simCOP(n=100, cop=NULL, para=NULL, na.rm=TRUE, seed=NULL, keept=FALSE,
              graphics=TRUE, ploton=TRUE, points=TRUE, snv=FALSE,
              infsnv.rm=TRUE, trapinfsnv=.Machine$double.eps,
              resamv01=FALSE, showresamv01=FALSE, ...)
rCOP(n, cop=NULL, para=NULL, na.rm=TRUE, seed=NULL,
              resamv01=FALSE, showresamv01=FALSE, ...)
}
\arguments{
  \item{n}{A sample size, default is \eqn{n = 100};}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters, if needed, to pass to the copula;}
  \item{na.rm}{A logical to toggle the removal of \code{NA} entries should they form on the returned \code{data.frame}. A well implemented copula should accommodate and not return \code{NA} but because this package relies on numerical derivation, it was decided to have a mechanism to handle this;}
    \item{seed}{The integer seed to pass immediately to \code{set.seed()};}
  \item{keept}{Keep the \eqn{t} uniform random variable for the simulation as the last column in the returned \code{data.frame};}
  \item{graphics}{A logical that will disable graphics by setting \code{ploton} and \code{points} to \code{FALSE} and overriding whatever their settings were;}
  \item{ploton}{A logical to toggle on the plot (see \bold{Examples} in \code{\link{vuongCOP}});}
  \item{points}{A logical to actually draw the simulations by the \code{points()} function in \R;}
  \item{snv}{A logical to convert the \eqn{\{u,v\}} to standard normal scores (variates) both for the optional graphics and the returned \R \code{data.frame}. Curiously, Joe (2014) advocates extensively for use of normal scores, which is in contrast to Nelsen (2006) who does not;}
   \item{infsnv.rm}{A logical that will quietly strip out any occurrences of \eqn{u = \{0,1\}} or \eqn{v = \{0,1\}} from the simulations because these are infinity in magnitude when converted to standard normal variates has been selected. Thus, this logical only impacts logic flow when \code{snv} is \code{TRUE}. The \code{infsnv.rm} is mutually exclusive from \code{trapinfsnv};}
   \item{trapinfsnv}{If \code{TRUE} and presumably small, the numerical value of this argument (\eqn{\eta}) is used to replace \eqn{u = \{0,1\}} and \eqn{v = \{0,1\}} with \eqn{u(0) =  v(0) = \eta} or \eqn{u(1) = v(1) = 1 - \eta} as appropriate when conversion to standard normal variates has been selected. The setting of \code{trapinfsnv} only is used if \code{snv} is \code{TRUE} and \code{infsnv.rm} is \code{FALSE};}
  \item{resamv01}{A logical triggering resampling for the elements of \eqn{v=0} and \eqn{v=1} (see \bold{Examples}). This is a relatively late addition to \pkg{copBasic} logic and hence is disabled by default. If this is set to true, then the operations related to \code{infsnv.rm} and \code{trapinfsnv} are never to be involved later down in the functions' logic;}
  \item{showresamv01}{A logical providing a trigger to display a \code{message()} within the resampling loops for \eqn{v <= 0, v >= 1}; and}
  \item{...}{Additional arguments to pass to the \code{points()} function in \R.}
}
\value{
  An \R \code{data.frame} of the simulated values is returned.
}
\note{
Function \code{rCOP} is a light-weight implementation for bivariate copula random variates that dispatches to \code{\link{simCOPmicro}} and bypasses the graphical and other features of \code{simCOP}. Finally, an experimental parallel for the empirical copula is \code{\link{EMPIRsim}}. Note, the equivalent of a \code{resamv01} is not implemented at the level of \code{\link{simCOPmicro}} because it is better to do such at the abstraction level of \code{rCOP} and \code{simCOP}.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{derCOPinv}}, \code{\link{simCOPmicro}}}
\examples{
simCOP(n=5, cop=PARETOcop, para=2.4)

# We can find some unusual simulation combinations for which V == 0 or 1 and might have
# downstream operations simply not able to handle infinite quantiles (say) at the edges.
# We can readily offload the issue back to the rCOP() or simCOP() with resamv01 argument.
nsim <- 800; para <- list(cop=PLcop, para=150, alpha=0.8, beta=0.3)
JK <- rCOP(nsim, cop=composite1COP, para=para, seed=1, resamv01=FALSE)
print(JK[JK[,2] == 1,]) # 189 0.9437248 1.   So, row 189 in this example has V=1, and
UV <- rCOP(nsim, cop=composite1COP, para=para, seed=1, resamv01=TRUE ) # changing the
# resamv01 argument to TRUE, we get this message and no V=1 in the returned data frame.
# rCOP() has some v >= 1, resampling those     <---- This is the message output.

\dontrun{
# The simCOP function is oft used in other Examples sections through this package.
simCOP(n=10, cop=W)            # Frechet lower-bound copula
simCOP(n=10, cop=P)            # Independence copula
simCOP(n=10, cop=M, col=2)     # Frechet upper-bound copula
simCOP(n=10, cop=PSP)          # The PSP copula}

\dontrun{
# Now simulate the PSP copula, add the level curves of the copula, and demonstrate
# the uniform distribution of marginals on the correct axes (U [top] and V [left]).
D <- simCOP(n=400, cop=PSP) # store simulated values in D
level.curvesCOP(cop=PSP, ploton=FALSE)
rug(D$U, side=3, col=2); rug(D$V, side=4, col=2)

# Now let us get more complicated and mix two Plackett copulas together using the
# composite2COP as a "compositor." The parameter argument becomes more complex, but
# is passed as shown into composite2COP.
para <- list(cop1=PLACKETTcop,cop2=PLACKETTcop, alpha=0.3,beta=0.5, para1=0.1,para2=50)
D <- simCOP(n=950, cop=composite2COP, para=para, col=grey(0, 0.2), pch=16, snv=TRUE) #}
}
\keyword{copula (simulation)}
\keyword{visualization}
\concept{conditional approach}
\concept{Rosenblatt transform}

\encoding{utf8}
\name{densityCOPplot}
\alias{densityCOPplot}
\title{Contour Density Plot of a Copula}
\description{
Generate a \emph{contour density plot} after the advocation of Joe (2014, pp. 9--15). Such graphics are plots of \emph{scaled copula densities} (\eqn{c^\star(u,v)}, bivariate herein) that are copula densities scaled to the standard normal distribution \eqn{\sim} N(0,1) margins. Joe (2014) repeatedly emphasizes such plots in contrast to the uniform distribution \eqn{\sim} U(0,1) margins. Nelsen (2006) does not discuss such scaling but seemingly Nelsen's objectives for his book were somewhat different.

The density of copula \eqn{\mathbf{C}(u,v)} is numerically estimated by
\deqn{c(u,v) = \bigl[\mathbf{C}(u_2,v_2) - \mathbf{C}(u_2,v_1) - \mathbf{C}(u_1,v_2) + \mathbf{C}(u_1,v_1)\bigr]\, /\, \bigl[\Delta(uv)\times\Delta(uv)\bigr]\mbox{,}}
where \eqn{c(u,v) \ge 0} (see Nelsen, 2006, p. 10; \code{\link{densityCOP}}). Given a numerically estimated quantity \eqn{c^\star(u,v) = c(u,v)\times\phi(\Phi^{(-1)}(u))\times\phi(\Phi^{(-1)}(v))} for copula density \eqn{c(u,v)}, a grid of the \eqn{c^\star(u,v)} values can be contoured by the \code{contour()} function in \R.  The density function of the N(0,1) is \eqn{\phi(z)} for standard normal variate \eqn{z} and the quantile function of the N(0,1) is \eqn{\Phi^{(-1)}(t)} for nonexceedance probability \eqn{t}.

A grid (matrix) of \eqn{c(u,v)} or \eqn{c^\star(u,v)} is defined for sequence of \eqn{u} and \eqn{v} probabilities for which each sequence has equal steps that are equal to \eqn{\Delta(uv)}. This function has as focus on plotting of the contour lines of \eqn{c^\star(u,v)} but the \R \code{matrix} of either \eqn{c(u,v)} or \eqn{c^\star(u,v)} can be requested on return. For either matrix, the \code{colnames()} and \code{rownames()} (the \R functions) are set equal to the sequence of \eqn{u} and \eqn{v}, respectively. Neither the column or row names are set to the standard normal variates for the matrix of \eqn{c^\star(u,v)}, the names remain in terms of nonexceedance probability.

For plotting and other uses of normal scores of data, Joe (2014, p. 245) advocates that one should use the plotting position formula \eqn{u_i = (i-1/2)/n} (\emph{Hazen plotting position}) for normal scores \eqn{z_i = \Phi^{-1}(u_i)} in preference to \eqn{i/(n+1)} (\emph{Weibull plotting position}) because \eqn{n^{-1}\sum_{i=1}^{n} z^2_i} is closer to unity. Other examples of Joe's advocation for the Hazen plotting positions are available (Joe, 2014, pp. 9, 17, 245, 247--248).
}
\usage{
densityCOPplot(cop=NULL, para=NULL, deluv=0.002,
               getmatrix=c("none", "cdenzz", "cden"), n=0,
               ploton=TRUE, snv=TRUE, origins=TRUE,
               contour.col=1, contour.lwd=1.5, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{deluv}{The change in the sequences \eqn{\{u, v\} \mapsto \delta, \ldots, 1-\delta; \delta = \Delta(uv)} probabilities;}
  \item{getmatrix}{A trigger on whether the density matrix is to be returned. The option \code{cdenzz} returns the density scaled by the two standard normal densities (\eqn{c^\star(u,v)}), whereas the option \code{cden} returns simply the copula density (\eqn{c(u,v)});}
  \item{ploton}{A logical to toggle on the plot;}
  \item{snv}{A logical to toggle standard normal variates for the axes;}
  \item{origins}{A logical to plot the origin lines, if and only if \code{snv} is true;}
  \item{contour.col}{The color of the contour lines, which corresponds to the \code{col} argument of the \code{contour} function in \R;}
  \item{contour.lwd}{The width of the contour lines, which corresponds to the \code{lwd} argument of the \code{contour} function in \R;}
  \item{n}{An optional sample size for which simulation of this many values from the copula will be made by \code{\link{simCOP}} and drawn; and}
  \item{...}{Additional arguments to pass to the copula function and to the \code{contour} function in \R (\emph{e.g.} to turn off labeling of contours add \code{drawlabels=FALSE}).}
}
\value{
  This is a high-level function used for its side effects; an \R \code{matrix} can be triggered, however, as a returned value.
}
\note{
Joe (2014, p. 244) says \dQuote{if [density] plots show multimodality, then multivariate techniques of mixture models, cluster analysis[,] and nonparametric functional data analysis might be more appropriate} than relatively straightforward parametric copula models.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{simCOP}}, \code{\link{densityCOP}}}
\examples{
\dontrun{
  # Joe (2014, p. 5) names rMTCJ = reflected Mardia-Takahasi-Cook-Johnson copula
  "rMTCJ" <- function(u, v, para, ...) {
     u + v - 1 + ((1-u)^(-para) + (1-v)^(-para) - 1)^(-1/para)
  } # Survival Copula ("reflected" in Joe's terms)
  densityCOPplot(cop=rMTCJ, para=1.0760, n=9000, snv=TRUE)
  # Density plot matches that shown by Joe (2014, p. 11, fig. 1.2, lower left plot)
  # for a Spearman Rho equaling 0.5. Let us compute then Rho:
  rhoCOP(cop=rMTCJ, para=1.076075) # 0.4999958

  # Now, let us get really wild with a composition with TWO modes!
  # This example also proves that the grid orientation is consistent with respect
  # to the orientation of the simulations.
  para <- list(alpha=0.15, beta=0.90, kappa=0.06, gamma=0.96,
               cop1=GHcop, cop2=PLACKETTcop, para1=5.5, para2=0.07)
  densityCOPplot(cop=composite2COP, para=para, n=9000)

  # Now, let us hack back to a contour density plot with U(0,1) and not N(0,1) margins
  # just so show that capability exists, but emphasis of densityCOPplot is clearly
  # on Joe's advocation, because it does not have a default trigger to use U(0,1) margins.
  set.seed(12)
  H <- densityCOPplot(cop=PLACKETTcop, para=41.25, getmatrix="cdenzz", n=1000, snv=FALSE)
  set.seed(12)
  UV <- simCOP(cop=PLACKETTcop, para=41.25, n=1000, col=8, snv=FALSE)
  U  <- as.numeric(colnames(H)); V <- as.numeric(rownames(H))
  contour(x=U, y=V, z=t(H), lwd=1.5, cex=2, add=TRUE, col=2) #}

\dontrun{
  set.seed(12)
  UV <- rCOP(400,  cop=PSP, pch=16, col=8, n=400)
  CL <- mleCOP(UV, cop=CLcop,   interval=c(1  , 20))
  JO <- mleCOP(UV, cop=JOcopB5, interval=c(0.1, 20))
  PL <- mleCOP(UV, cop=PLcop,   interval=c(0.1, 20))

  AICs <- c(CL$AIC, JO$AIC, PL$AIC)
  names(AICs) <- c("Clayton", "Joe(B5)", "Plackett")
  print(round(AICs, digits=2))
  #  Clayton    Joe(B5)   Plackett
  #  -156.77     -36.91    -118.38
  # So, we conclude Clayton is the best fit of the three.

  plot(qnorm(UV[,1]), qnorm(UV[,2]), pch=16, col=8, cex=0.5,
       xlab="Standard normal variate of U", xlim=c(-3, 3),
       ylab="Standard normal variate of V", ylim=c(-3, 3))
  densityCOPplot(cop=PSP, contour.col=grey(0.5), lty=2,
                 contour.lwd=3.5, ploton=FALSE, snv=TRUE)
  densityCOPplot(cop=CLcop,       para=CL$para,
                 contour.col=2,   ploton=FALSE, snv=TRUE)
  densityCOPplot(cop=JOcopB5,     para=JO$para,
                 contour.col=3,   ploton=FALSE, snv=TRUE)
  densityCOPplot(cop=PLcop,       para=PL$para,
                 contour.col=4,   ploton=FALSE, snv=TRUE) #}
}
\keyword{visualization}
\keyword{copula (density)}
\keyword{Joe (2014) Examples and Exercises}

\encoding{utf8}
\name{EMPIRsim}
\alias{EMPIRsim}
\title{Simulate a Bivariate Empirical Copula}
\description{
\emph{EXPERIMENTAL}---Perform a simulation on a \emph{bivariate empirical copula} to produce the random variates \eqn{U} and \eqn{V} and return an \R \code{data.frame} of them.  The method is more broadly known as \emph{conditional simulation method}. This function is an empirical parallel to \code{\link{simCOP}} that is used for parametric copulas. If circumstances require conditional simulation of \eqn{V{\mid}U}, then function \code{\link{EMPIRsimv}}, which produces a vector of \eqn{V} from a fixed \eqn{u}, should be used.

For the usual situation in which an individual \eqn{u} during the simulation loops is not a value aligned on the grid, then the bounding conditional quantile functions are solved for each of the \eqn{n} simulations and the following interpolation is made by
\deqn{v = \frac{v_1/w_1 + v_2/w_2}{1/w_1 + 1/w_2}\mbox{,}}
which states that that the weighted mean is computed. The values \eqn{v_1} and \eqn{v_2} are ordinates of the conditional quantile function for the respective grid lines to the left and right of the \eqn{u} value. The values \eqn{w_1} \eqn{=} \eqn{u - u^\mathrm{left}_\mathrm{grid}} and \eqn{w_2} \eqn{=} \eqn{u^\mathrm{right}_\mathrm{grid} - u}.
}
\usage{
EMPIRsim(n=100, empgrid=NULL, kumaraswamy=FALSE, na.rm=TRUE, keept=FALSE,
                graphics=TRUE, ploton=TRUE, points=TRUE, snv=FALSE,
                infsnv.rm=TRUE, trapinfsnv=.Machine$double.eps, ...)
}
\arguments{
  \item{n}{A sample size, default is 100;}
  \item{empgrid}{Gridded empirical copula from \code{\link{EMPIRgrid}};}
  \item{kumaraswamy}{A logical to trigger Kumaraswamy distribution smoothing of the conditional quantile function that is passed to \code{\link{EMPIRgridderinv}}. The Kumaraswamy distribution is a distribution having support \eqn{[0,1]} with an explicit quantile function and takes the place of a Beta distribution (see \pkg{lmomco} function \code{quakur()} for more details);}
    \item{na.rm}{A logical to toggle the removal of \code{NA} entries on the returned \code{data.frame};}
  \item{keept}{Keep the \eqn{t} uniform random variable for the simulation as the last column in the returned \code{data.frame};}
  \item{graphics}{A logical that will disable graphics by setting \code{ploton} and \code{points} to \code{FALSE} and overriding whatever their settings were;}
  \item{ploton}{A logical to toggle on the plot;}
  \item{points}{A logical to actually draw the simulations by the \code{points()} function in \R;}
  \item{snv}{A logical to convert the \eqn{\{u,v\}} to standard normal scores (variates) both for the optional graphics and the returned \code{data.frame}. Joe (2014) advocates extensively for use of normal scores, which is in  contrast to Nelsen (2006) who does not;}
   \item{infsnv.rm}{A logical that will quietly strip out any occurrences of \eqn{u = \{0,1\}} or \eqn{v = \{0,1\}} from the simulations because these are infinity in magnitude when converted to standard normal variates is to occur. Thus, this logical only impacts logic flow when \code{snv} is \code{TRUE}. The \code{infsnv.rm} is mutually exclusive from \code{trapinfsnv};}
   \item{trapinfsnv}{If \code{TRUE} and presumably small, the numerical value of this argument (\eqn{\eta}) is used to replace \eqn{u = \{0,1\}} and \eqn{v = \{0,1\}} with \eqn{u(0) =  v(0) = \eta} or \eqn{u(1) = v(1) = 1 - \eta} as appropriate when conversion to standard normal variates is to occur. The setting of \code{trapinfsnv} only is used if \code{snv} is \code{TRUE} and \code{infsnv.rm} is \code{FALSE}; and}
  \item{...}{Additional arguments to pass to the \code{points()} function in \R.}
}
\value{
  An \R \code{data.frame} of the simulated values is returned.
}
\author{W.H. Asquith}
\seealso{\code{\link{EMPIRgrid}}, \code{\link{EMPIRgridderinv}},  \code{\link{EMPIRsimv}}}
\examples{

# See other examples under EMPIRsimv

\dontrun{
pdf("EMPIRsim_experiment.pdf")
  nsim <- 5000
  para <- list(alpha=0.15, beta=0.65,
               cop1=PLACKETTcop, cop2=PLACKETTcop, para1=0.005, para2=1000)
  set.seed(1)
  uv <- simCOP(n=nsim, cop=composite2COP, para=para, snv=TRUE,
               pch=16, col=rgb(0,0,0,.2))
  mtext("A highly complex simulated bivariate relation")
  # set.seed(1) # try not resetting the seed
  uv.grid <- EMPIRgrid(para=uv, deluv=0.025)

  uv2 <- EMPIRsim(n=nsim, empgrid=uv.grid, kumaraswamy=FALSE, snv=TRUE,
                  col=rgb(1,0,0,0.1), pch=16)
  mtext("Resimulation without Kumaraswamy smoothing")

  uv3 <- EMPIRsim(n=nsim, empgrid=uv.grid, kumaraswamy=TRUE, snv=TRUE,
                  col=rgb(1,0,0,0.1),pch=16)
  mtext("Resimulation but using the Kumaraswamy Distribution for smoothing")
dev.off()#}
}
\keyword{empirical copula}
\keyword{empirical copula (simulation)}

\encoding{utf8}
\name{semicorCOP}
\alias{semicorCOP}
\title{Lower and Upper Semi-Correlations of a Copula}
\description{
Compute the \emph{lower semi-correlations} (bottom-left)
\deqn{\rho^{N{-}{-}}_\mathbf{C}(u,v; a) = \rho_N^{{-}{-}}(a) \mbox{ and}}
compute the \emph{upper semi-correlations} (top-right)
\deqn{\rho^{N{+}{+}}_\mathbf{C}(u,v; a) = \rho_N^{{+}{+}}(a)}
of a copula \eqn{\mathbf{C}(u,v)} (Joe, 2014, p. 73) using numerical simulation. The semi-correlations are defined as
\deqn{\rho_N^{{-}{-}}(a) = \mathrm{cor}[Z_1, Z_2 \mid Z_1 < -a, Z_2 < -a]\mbox{,}}
\deqn{\rho_N^{{+}{+}}(a) = \mathrm{cor}[Z_1, Z_2 \mid Z_1 > +a, Z_2 > +a]\mbox{,\ and}}
\deqn{\rho_N(a > -\infty) = \mathrm{cor}[Z_1, Z_2]\mbox{,}}
where \eqn{\mathrm{cor[z_1, z_2]}} is the familiar \emph{Pearson correlation function}, which is in \R the syntax \code{cor(...,} \code{method="pearson")}, parameter \eqn{a \ge 0} is a truncation point that identifies \emph{truncated tail regions} (Joe, 2014, p. 73), and lastly \eqn{(Z_1, Z_2) \sim \mathbf{C}(\Phi, \Phi)} and thus from the standard normal distribution \eqn{(Z_1, Z_2) = (\Phi^{-1}(u), \Phi^{-1}(v))} where the random variables \eqn{(U,V) \sim \mathbf{C}}.

The semi-correlations are extended for the \pkg{copBasic} package into bottom right and top left versions as well by
\deqn{\rho_N^{{+}{-}}(a) = \mathrm{cor}[Z_1, Z_2 \mid Z_1 > +a, Z_2 < -a]\mbox{,\ and}}
\deqn{\rho_N^{{-}{+}}(a) = \mathrm{cor}[Z_1, Z_2 \mid Z_1 < -a, Z_2 > +a]\mbox{.}}
As a result, the notations \eqn{{-}{-}}, \eqn{{+}{+}}, \eqn{{+}{-}}, and \eqn{{-}{+}} can be used to represent each of the respective corners bottom-left, top-right, bottom-right, and top-left of the \eqn{(u,v)} domain with the respective truncation. These words are used in the variable names of the returned list from the function.
}
\usage{
semicorCOP(cop=NULL, para=NULL, truncation=0, n=0, as.sample=FALSE, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{truncation}{The truncation value for \eqn{a}, which is in standard normal variates, and the default of zero is the origin (medians);}
  \item{n}{The sample size \eqn{n} for simulation estimates of the \eqn{\rho_N};}
  \item{as.sample}{A logical controlling whether an optional \code{data.frame} in \code{para} is used to compute the \eqn{\hat\rho_N} (see \bold{Note}); and}
  \item{...}{Additional arguments to pass to the copula.}
}
\value{
  The value(s) for \eqn{\rho_N}, \eqn{\rho_N^{{-}{-}}}, \eqn{\rho_N^{{+}{+}}}, \eqn{\rho_N^{{+}{-}}}, and \eqn{\rho_N^{{-}{+}}} are returned.
}
\note{
The sample semi-correlations can be computed from a two-column table that is passed into the function using the \code{para} argument. Although the truncation point \eqn{a \ge 0}, as \eqn{a} increases and focus is increasingly made into one or the other truncated tail regions, the sample version with data becomes decreasing well estimated because the available sample size diminishes. The \code{para} argument can contain probabilities or raw data because internally the function computes the \emph{Hazen plotting positions} (\emph{e.g.} \eqn{u_i = (i-0.5) / n} for rank \eqn{i} and sample size \eqn{n}) because Joe (2014, pp. 9, 17, 245, 247--248) repeatedly emphasizes this form of plotting position when normal scores are involved.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{giniCOP}}, \code{\link{rhoCOP}}, \code{\link{tauCOP}}, \code{\link{COP}}
}
\examples{
\dontrun{
# Gumbel-Hougaard copula with Pearson rhoN = 0.4 (by definition)
run <- sapply(1:50, function(i) semicorCOP(cop=GHcop, para=1.350, n=600))
mean(unlist(run[1,])) # cor.normal.scores
mean(unlist(run[2,])) #  botleft.semicor
mean(unlist(run[3,])) # topright.semicor
sd(  unlist(run[1,])) # cor.normal.scores (These are our sampling variations
sd(  unlist(run[2,])) #  botleft.semicor   for the n=600 used as a Monte
sd(  unlist(run[3,])) # topright.semicor   Carlo simulation.)
# The function returns:    rhoN = 0.392112, rhoN--= 0.117674, rhoN++= 0.404733
#  standard deviations           (0.038883)        (0.073392)        (0.073942)
# Joe (2014, p. 72) shows: rhoN = 0.4_____, rhoN--= 0.132___, rhoN++= 0.415___
#  standard deviations           (not avail)       (0.08____)        (0.07____)
# We see alignment with the results of Joe with his n=600. #}

\dontrun{
p <- 0.5 # Reasonable strong positive association for the Raftery copula,
# but then we are going to be reflecting and rotating the copula.
# See similar Example under COP() function.
"RFcop1" <- function(u,v, para) COP(u,v, cop=RFcop, para=para, reflect="1")
"RFcop2" <- function(u,v, para) COP(u,v, cop=RFcop, para=para, reflect="2")
"RFcop3" <- function(u,v, para) COP(u,v, cop=RFcop, para=para, reflect="3")
"RFcop4" <- function(u,v, para) COP(u,v, cop=RFcop, para=para, reflect="4")
RF <- NULL; n <- 10000
RF <- rbind(RF, as.data.frame(semicorCOP(cop=RFcop1, para=p, n=n))[,1:5])
RF <- rbind(RF, as.data.frame(semicorCOP(cop=RFcop2, para=p, n=n))[,1:5])
RF <- rbind(RF, as.data.frame(semicorCOP(cop=RFcop3, para=p, n=n))[,1:5])
RF <- rbind(RF, as.data.frame(semicorCOP(cop=RFcop4, para=p, n=n))[,1:5])
print(RF[,1:3]) # total sample and lower and upper semi-correlations
#   cor.normal.scores botleft.semicor topright.semicor
# 1         0.5587837      0.74124686       0.10027641
# 2         0.5567889      0.10302772       0.73729702
# 3        -0.5807201     -0.04683536      -0.01714573 # see near zeros --,++
# 4        -0.5698139      0.03040520       0.05125916 # see near zeros --,++
print(RF[,2:5]) # now look at all four corners
#   botleft.semicor topright.semicor topleft.semicor botright.semicor
# 1      0.74124686       0.10027641      0.01529508       0.02046530
# 2      0.10302772       0.73729702     -0.05195628       0.01747874
# 3     -0.04683536      -0.01714573     -0.11106842      -0.74077321
# 4      0.03040520       0.05125916     -0.74516061      -0.07636233
# Notice how the tight tail of the copula, being reflected and rotated
# into each of the four corners, shows semi-correlation magnitude of 0.74.
# See the copula density plots of COP() Examples section. }
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{copula (tail characteristics)}
\keyword{copula (tail properties)}
\keyword{semi-correlation coefficient}
\concept{semi-correlation}
\keyword{Joe (2014) Examples and Exercises}
\keyword{Theory Examples (reflection/rotation)}

\encoding{utf8}
\name{gridCOP}
\alias{gridCOP}
\title{Compute a Copula on a Grid}
\description{
Compute a grid of copula values. This function has the primary intention of supporting 3D renderings or 2D images of the \emph{copulatic surface}. Users should be aware of the convention of the placement of the plotting origin and the various plotting mechanisms available to them in \R. By convention copulatic surfaces start in lower left corner for \eqn{u = v = 0}, but matrix conventions (or at least how some functions plot matrices) start with the origin in the upper left.
}
\usage{
gridCOP(cop=NULL, para=NULL, delta=0.05, transpose=TRUE, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{delta}{The \eqn{\Delta u = \Delta v} of the grid edges;}
  \item{transpose}{A logical to transpose the returned grid. This is needed if functions such as \code{image()} in \R are to be used for visualization (see last example in \bold{Examples} with \code{\link{composite2COP}}); and}
  \item{...}{Additional arguments to pass.}
}
\value{
  The values for \eqn{\mathbf{C}(u,v)} are returned as a grid as an \R \code{matrix}.
}
\author{W.H. Asquith}
\seealso{\code{\link{EMPIRcopdf}}
}
\examples{
\dontrun{
the.grid <- gridCOP(cop=PSP)
the.grid[1,1] <- 0 # replace the NaN
image(the.grid) # ramps to the upper right }

\dontrun{
# See this composite copula also used in densityCOPplot() documentation.
para <- list(alpha=0.15, beta=0.90, kappa=0.06, gamma=0.96,
             cop1=GHcop, cop2=PLACKETTcop, para1=5.5, para2=0.07)
GR <- gridCOP(cop=composite2COP, para=para, delta=0.005)
image(GR, col=terrain.colors(20)) # asymmetric, high curvature in top half }
}
\keyword{copula utility}
\keyword{copulatic surface}

\encoding{utf8}
\name{EvuCOP}
\alias{EvuCOP}
\title{Expected value of V given U}
\description{
Compute the \emph{expected value} of \eqn{V} given a \eqn{U} (the \eqn{X} direction) through the \emph{conditional distribution function} \eqn{F(X)} using the appropriate \emph{partial derivative} of a copula (\eqn{\mathbf{C}(u,v)}) with respect to \eqn{U}. The inversion of the partial derivative is the \emph{conditional quantile function}. Basic principles provide the expectation for a \eqn{x \ge 0} is
\deqn{E[X] = \int_0^\infty xf(x)\mathrm{d}x = \int_0^\infty \bigl(1-F_x(X)\bigr)\mathrm{d}x\mbox{,}}
which for the setting here becomes
\deqn{E[V \mid U = u] = \int_0^1 \bigl(1 - \frac{\delta}{\delta u} \mathbf{C}(u,v)\bigr)\mathrm{d}v\mbox{.}}
This function solves the integral using the \code{\link{derCOP}} function. Verification study is provided in the \bold{Note} section.
}
\usage{
EvuCOP(u=seq(0.01, 0.99, by=0.01), cop=NULL, para=NULL, asuv=FALSE, nsim=1E5,
   subdivisions=200L, rel.tol=.Machine$double.eps^0.25, abs.tol=rel.tol, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{cop}{A copula function with vectorization as in \code{asCOP};}
  \item{para}{Vector of parameters or other data structures, if needed, to pass to the copula;}
  \item{asuv}{Return a data frame of the \eqn{U} and \eqn{V};}
  \item{nsim}{Number of simulations for \emph{Monte Carlo integration} if numerical integration were to fail (refer to \bold{Note} in \code{\link{EvuCOP}});}
  \item{subdivisions}{Argument of same name passed to \code{integrate()};}
  \item{rel.tol}{Argument of same name passed to \code{integrate()};}
  \item{abs.tol}{Argument of same name passed to \code{integrate()}; and}
  \item{...}{Additional arguments to pass to \code{\link{derCOP}}.}
}
\value{
  Value(s) for the expectation are returned.
}
\note{
For the \code{\link{PSP}} copula with no parameters, compute the the median and mean \eqn{V} given \eqn{U=0.4}, respectively:
\preformatted{
  U <- 0.4; n <- 1E4
  med.regressCOP(u=U, cop=PSP)                            # V = 0.4912752
  set.seed(1)
  median(replicate(n, derCOPinv(cop=PSP, U, runif(1)) ) ) # V = 0.4876440
  mean(  replicate(n, derCOPinv(cop=PSP, U, runif(1)) ) ) # V = 0.5049729
}

It is seen in the above that the median \eqn{V} given \eqn{U} is very close to the mean, but is not equal. Using the derivative inversion within \code{\link{med.regressCOP}} the median is about 0.491 and then using large-sample simulation, about 0.491 too is computed. This confirms the median and long standing proven use of \code{\link{derCOP}} (conditional distribution function) and \code{\link{derCOPinv}} (conditional quantile function) within the package. The expectation (mean) by simulation provides the anchor point to check implementation of \code{EuvCOP()}. The mean for \eqn{V} given \eqn{U} is about 0.505. Continuing, the core logic of \code{EvuCOP()} is to use numerical integration of the conditional distribution function (the partial derivative) and not bother for speed purposes to use the inversion of the partial derivative:
\preformatted{
  integrate(function(v)                      1-derCOP(   cop=PSP, U, v),
            lower=0, upper=1) # 0.5047805 with absolute error < 1.4e-11

  integrate(function(v) sapply(v, function(t)  derCOPinv(cop=PSP, U, t)),
            lower=0, upper=1) # 0.5047862 with absolute error < 7.2e-05
}
The two integrals match, which functions as a confirmation of the \eqn{(1-F)} term in the mathematical definition. Finally, the two integrals match the simulation results. The expectation or mean \eqn{V \mid U=0.4} for the \code{\link{PSP}} copula is about 0.5048.
}
\author{W.H. Asquith}
\seealso{\code{\link{EuvCOP}}, \code{\link{derCOP}}}
\examples{
# Highly asymmetric and reflected Clayton copula for which visualization
para <- list(cop=CLcop, para=30, alpha=0.2, beta=0.6, reflect=3)
# UV <- simCOP(5000, cop=breveCOP, para=para, cex=0.5); abline(v=0.25, col="red")
EvuCOP(0.25, cop=breveCOP, para=para)  # 0.5982261
# confirms that at U=0.25 that an intuitive estimate would be about 0.6.

\dontrun{
  # Secondary validation of the EvuCOP() and EuvCOP() implementation
  UV <- simCOP(200, cop=PSP)
  u <- seq(0.005, 0.995, by=0.005)
  v <- sapply(u, function(t) integrate(function(k)
                         1 - derCOP(cop=PSP, t, k),  lower=0, upper=1)$value)
  lines(u,v, col="red", lwd=7, lty=1)   # red line
  v <- seq(0.005, 0.995, by=0.005)
  u <- sapply(v, function(t) integrate(function(k)
                         1 - derCOP2(cop=PSP, k, t), lower=0, upper=1)$value)
  lines(u,v, col="red", lwd=7, lty=2)   # dashed red line

  uv <- seq(0.005, 0.995, by=0.005)     # solid and dashed white lines
  lines(EvuCOP(uv, cop=PSP, asuv=TRUE), col="white",  lwd=3,   lty=1)
  lines(EuvCOP(uv, cop=PSP, asuv=TRUE), col="white",  lwd=3,   lty=3)

  # median regression lines for comparison, green and green dashed lines
  lines(med.regressCOP( uv, cop=PSP), col="seagreen", lwd=1.5, lty=1)
  lines(med.regressCOP2(uv, cop=PSP), col="seagreen", lwd=1.5, lty=4) #}

\dontrun{
  uv <- seq(0.005, 0.995, by=0.005) # stress testing eample with singularity
  UV <- simCOP(50, cop=M_N5p12b, para=2)
  lines(EvuCOP(uv, cop=M_N5p12b, para=2, asuv=TRUE), col="red",     lwd=5)
  lines(EuvCOP(uv, cop=M_N5p12b, para=2, asuv=TRUE), col="skyblue", lwd=1) #}

\dontrun{
  uv <- seq(0.005, 0.995, by=0.005) # more asymmetry ---- mean regression in UV and VU
  para <- list(cop=GHcop, para=23, alpha=0.1, beta=0.6, reflect=3)
  para <- list(cop=breveCOP, para=para)
  UV <- simCOP(200, cop=COP, para=para)
  lines(EuvCOP(uv,  cop=COP, para=para, asuv=TRUE), col="red",  lwd=2)
  lines(EvuCOP(uv,  cop=COP, para=para, asuv=TRUE), col="blue", lwd=2) #}

\dontrun{
  # Open questions? The derCOP() and derCOPinv() functions of the package have long
  # been known to work "properly." But let us think again on the situation of
  # permutation symmetry about the equal value line. Recalling that this symmetry is
  # orthogonal to the equal value line, it remains open whether there could be
  # asymmetry in the vertical (or horizontal). Let us draw some median regression lines
  # and see that the do not plot perfectly on the equal value line, but could this be
  # down to numerical issues and by association the simulation of the copula itself
  # that is also using derCOPinv() (conditional simulation method). Then, we can plot
  # the expectations and we see that these are not equal to the medians, but again are
  # close. *** Do results here indicate edges of numerical performance? ***
  t <- seq(0.01, 0.99, by=0.01)
  UV <- simCOP(10000,   cop=N4212cop, para=4, pch=21, lwd=0.8, col=8, bg="white")
  lines(med.regressCOP( cop=N4212cop, para=4, asuv=TRUE), col="red")
  lines(med.regressCOP2(cop=N4212cop, para=4, asuv=TRUE), col="red")
  abline(0, 1, col="deepskyblue", lwd=3); abline(v=0.5, col="deepskyblue", lwd=4)
  lines(EvuCOP(t, cop=N4212cop, para=4, asuv=TRUE), pch=16, col="darkgreen")
  lines(EuvCOP(t, cop=N4212cop, para=4, asuv=TRUE), pch=16, col="darkgreen") #}
}
\keyword{copula (utility)}
\keyword{copula operator}
\keyword{copula (conditional distribution)}
\keyword{conditional expectation}
\keyword{mean V given U}
\concept{expectation}
\keyword{Theory Examples (reflection/rotation)}

\encoding{utf8}
\name{rmseCOP}
\alias{rmseCOP}
\title{Root Mean Square Error between a Fitted Copula and an Empirical Copula}
\description{
Compute the \emph{root mean square error} \eqn{\mathrm{RMSE}_\mathbf{C}} (Chen and Guo, 2019, p. 29), which is computed using \emph{mean square error} \eqn{\mathrm{MSE}} as

\deqn{\mathrm{MSE}_\mathbf{C} = \frac{1}{n}\sum_{i=1}^n \bigl(\mathbf{C}_n(u_i,v_i) - \mathbf{C}_{\Theta_m}(u_i, v_i)\bigr)^2\mbox{ and}}
\deqn{\mathrm{RMSE}_\mathbf{C} = \sqrt{\mathrm{MSE}_\mathbf{C}}\mbox{,}}

where \eqn{\mathbf{C}_n(u_i,v_i)} is the \emph{empirical copula} (empirical joint probability) for the \eqn{i}th observation, \eqn{\mathbf{C}_{\Theta_m}(u_i, v_i)} is the fitted copula having \eqn{m} parameters in \eqn{\Theta}. The \eqn{\mathbf{C}_n(u_i,v_i)} comes from \code{\link{EMPIRcop}}. The \eqn{\mathrm{RMSE}_\mathbf{C}} is in effect saying that the best copula will have its joint probabilities plotting on a 1:1 line with the empirical joint probabilities, which is an \eqn{\mathrm{RMSE}_\mathbf{C}=0}. From the \eqn{\mathrm{MSE}_\mathbf{C}} shown above, the \emph{Akaike information criterion} (AIC) \code{\link{aicCOP}} and \emph{Bayesian information criterion} (BIC) \code{\link{bicCOP}} can be computed, which add a penalty for \eqn{m} parameters. These goodness-of-fits can assist in deciding one copula favorability over another, and another goodness-of-fit using the absolute differences between \eqn{\mathbf{C}_n(u,v)} and \eqn{\mathbf{C}_{\Theta_m}(u, v)} is found under \code{\link{statTn}}.
}
\usage{
rmseCOP(u, v=NULL, cop=NULL, para=NULL, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction; If not given, then a second column from argument \code{u} is attempted;}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula; and}
  \item{...}{Additional arguments to pass to either copula (likely most commonly to the empirical copula).}
}
\value{
  The value for \eqn{\mathrm{RMSE}_\mathbf{C}} is returned.
}
\references{
Chen, Lu, and Guo, Shenglian, 2019, Copulas and its application in hydrology and water resources: Springer Nature, Singapore, ISBN 978--981--13--0574--0.
}
\author{W.H. Asquith}
\seealso{\code{\link{EMPIRcop}}, \code{\link{aicCOP}}, \code{\link{bicCOP}}
}
\examples{
\dontrun{
S <- simCOP(80, cop=GHcop, para=5) # Simulate some probabilities, but we
# must then treat these as data and recompute empirical probabilities.
U <- lmomco::pp(S$U, sort=FALSE); V <- lmomco::pp(S$V, sort=FALSE)
# The parent distribution is Gumbel-Hougaard extreme value copula.
# But in practical application we do not know that but say we speculate that
# perhaps the Galambos extreme value might be the parent. Then maximum
# likelihood is used to fit the single parameter.
pGL <- mleCOP(U,V, cop=GLcop, interval=c(0,20))$par

rmses <- c(rmseCOP(U,V, cop=GLcop, para=pGL),
           rmseCOP(U,V, cop=P),
           rmseCOP(U,V, cop=PSP))
names(rmses) <- c("GLcop", "P", "PSP")
print(rmses) # We will see that the first RMSE is the smallest as the
# Galambos has the nearest overall behavior than the P and PSP copulas.}
}
\keyword{copula (goodness-of-fit)}
\keyword{copula (inference)}
\keyword{inference}
\keyword{goodness-of-fit}

\encoding{utf8}
\name{tauCOP}
\alias{tauCOP}
\alias{concordCOP}
\title{The Kendall Tau and Concordance Function of a Copula}
\description{
Compute the measure of association known as the \emph{Kendall Tau} (\eqn{\tau_\mathbf{C}}) of a copula (\eqn{\tau_\mathbf{C}}) according to Nelsen (2006, sec. 5.1.1 and p. 161) by
\deqn{\tau_\mathbf{C} = \mathcal{Q}(\mathbf{C}, \mathbf{C}) = 4\int\!\!\int_{\mathcal{I}^2}
                         \mathbf{C}(u,v)\,\mathrm{d}\mathbf{C}(u,v) - 1\mbox{,}}
where \eqn{\mathcal{Q}(\mathbf{C}, \mathbf{C})} is a \emph{concordance function} (\code{concordCOP}) of a copula with itself. Nelsen (2006, p. 164) reports however that this form is often not amenable to computation when there is a singular component to the copula and that the expression
\deqn{\tau_\mathbf{C} = 1 - 4\int\!\!\int_{\mathcal{I}^2}
                         \frac{\delta\mathbf{C}(u,v)}{\delta u}
                         \frac{\delta\mathbf{C}(u,v)}{\delta v}\,
                         \mathrm{d}u\mathrm{d}v}
is to be preferred. Such an expression hence relies on the partial numerical derivatives of the copula provided by \code{\link{derCOP}} and \code{\link{derCOP2}}. The Nelsen (2006) preferred expression is used by the \code{tauCOP} function. Nelsen (2006, pp. 175--176) reports that the relation between \eqn{\tau_\mathbf{C}} and \eqn{\rho_\mathbf{C}} (\code{\link{rhoCOP}}) is
\eqn{-1 \le 3\tau - 2\rho \le 1} (see \code{\link{rhoCOP}} for more details).

Nelsen (2006, pp. 160--161) lists some special identities involving \eqn{\mathcal{Q}(\mathbf{C}_1,\mathbf{C}_2)}:
\deqn{\mathcal{Q}(\mathbf{M}, \mathbf{M}) = 4\int_0^1 u\,\mathrm{d}u - 1 = 1\mbox{,}}
\deqn{\mathcal{Q}(\mathbf{M}, \mathbf{\Pi}) = 4\int_0^1 u^2\,\mathrm{d}u - 1 = 1/3\mbox{,}}
\deqn{\mathcal{Q}(\mathbf{M}, \mathbf{W}) = 4\int_{1/2}^1 (2u-1)\,\mathrm{d}u - 1 = 0\mbox{,}}
\deqn{\mathcal{Q}(\mathbf{W}, \mathbf{\Pi}) = 4\int_0^1 u(1-u)\,\mathrm{d}u - 1 = -1/3\mbox{,}}
\deqn{\mathcal{Q}(\mathbf{W}, \mathbf{W}) = 4\int_0^1 0\,\mathrm{d}u - 1 = -1\mbox{, and}}
\deqn{\mathcal{Q}(\mathbf{\Pi}, \mathbf{\Pi}) = 4\int\!\!\int_{\mathcal{I}^2} uv\,\mathrm{d}u\mathrm{d}v - 1 = 0\mbox{.}}


Kendall Tau also can be expressed in terms of the \emph{Kendall Function} (\eqn{F_K(z)}; \code{\link{kfuncCOP}}):
\deqn{\tau_\mathbf{C} = 3 - 4\int_0^1 F_K(t)\,\mathrm{d}t\mbox{,}}
which is readily verified by code shown in \bold{Examples}. This definition might be useful if integration errors are encountered for some arbitrary copula and arbitrary parameter set. In fact, should two attempts (see source code) at dual integration of the partial derivatives occur, the implementation switches over to integration of the Kendall Function (\emph{e.g.} \code{tauCOP(cop=N4212cop, para=2)}). Note, Durante and Sempi have erroneously dropped the multiplication by \dQuote{\eqn{4}} as shown above in their definition of \eqn{\tau_\mathbf{C}} as a function of \eqn{F_K(t)} (Durante and Sempi, 2015, eq. 3.9.4, p. 121).
}
\usage{
tauCOP( cop=NULL,  para=NULL,
       cop2=NULL, para2=NULL, as.sample=FALSE, brute=FALSE, delta=0.002, ...)

concordCOP(cop=NULL,  para=NULL, cop2=NULL, para2=NULL, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{cop2}{A second copula function;}
  \item{para2}{Vector of parameters or other data structure, if needed, to pass to the second copula;}
    \item{as.sample}{A logical controlling whether an optional \R \code{data.frame} in \code{para} is used to compute the \eqn{\hat\tau} by dispatch to \code{cor()} function in \R with \code{method = "kendall"};}
  \item{brute}{Should brute force be used instead of two nested \code{integrate()} functions in \R to perform the double integration;}
  \item{delta}{The \eqn{\mathrm{d}u} and \eqn{\mathrm{d}v} for the brute force integration using \code{brute}; and}
  \item{...}{Additional arguments to pass on to \code{\link{derCOP}} and \code{\link{derCOP2}}.}
}
\value{
  The value for \eqn{\tau_\mathbf{C}} is returned.
}
\note{
Although titled for computation of the Kendall Tau, the \code{tauCOP} function also is the implementation of the \emph{concordance function} \eqn{\mathcal{Q}(\mathbf{C}_1, \mathbf{C}_2)} (see Nelsen (2006, pp. 158--159) when given two different copulas and respective parameters as arguments. The function \code{concordCOP} just dispatches to \code{tauCOP}. A useful relation is
\deqn{\int\!\!\int_{\mathcal{I}^2} \mathbf{C}_1(u,v)\,\mathrm{d}\mathbf{C}_2(u,v) =
      \frac{1}{2} - \int\!\!\int_{\mathcal{I}^2} \frac{\delta}{\delta u}\mathbf{C}_1(u,v)\,\frac{\delta}{\delta v}\mathbf{C}_2(u,v)\,\mathrm{d}u\mathrm{d}v\mbox{,}}
where \eqn{\mathbf{C}_1(u,v)} is the first copula and \eqn{\mathbf{C}_2(u,v)} is the second copula.

Nelsen \emph{et al.} (2001, p. 281) lists several measures of association defined by the concordance function:\cr
\eqn{\mbox{}\quad\mathrm{1.}\quad\mbox{}}\eqn{\tau_\mathbf{C} = \quad\ \mathcal{Q}(\mathbf{C}, \mathbf{C})} : (Kendall Tau; \code{tauCOP});\cr
\eqn{\mbox{}\quad\mathrm{2.}\quad\mbox{}}\eqn{\rho_\mathbf{C} = 3\cdot\mathcal{Q}(\mathbf{C}, \mathbf{\Pi})} : (Spearman Rho; \code{\link{rhoCOP}});\cr
\eqn{\mbox{}\quad\mathrm{3.}\quad\mbox{}}\eqn{\gamma_\mathbf{C} = 2\cdot\mathcal{Q}(\mathbf{C}, [\mathbf{M}+\mathbf{W}]/2)} : (Gini Gamma; \code{\link{giniCOP}}); and\cr
\eqn{\mbox{}\quad\mathrm{4.}\quad\mbox{}}\eqn{\psi_\mathbf{C} = \frac{3}{2}\cdot\mathcal{Q}(\mathbf{C}, \mathbf{M}) - \frac{1}{2}} : (Spearman Footrule; \code{\link{footCOP}}).
}
\references{
Durante, F., and Sempi, C., 2015, Principles of copula theory: Boca Raton, CRC Press, 315 p.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Nelsen, R.B., Quesada-Molina, J.J., \enc{Rodríguez-Lallena}{Rodriguez-Lallena}, J.A., and \enc{Úbeda-Flores}{Ubeda-Flores}, M., 2001, Distribution functions of copulas---A class of bivariate probability integral transforms: Statistics and Probability Letters, v. 54, no. 3, pp. 277--282.
}
\author{W.H. Asquith}
\seealso{\code{\link{blomCOP}}, \code{\link{footCOP}}, \code{\link{giniCOP}},
         \code{\link{hoefCOP}}, \code{\link{rhoCOP}}, \code{\link{wolfCOP}},
         \code{\link{joeskewCOP}}, \code{\link{uvlmoms}},
         \code{\link{derCOP}}, \code{\link{derCOP2}}, \code{\link{kfuncCOP}}}
\examples{
\dontrun{
tauCOP(cop=PSP) # 1/3
# Now compute Kendall Tau via integration of the Kendall Function.
# 3 - 4*integrate(function(t) kfuncCOP(t, cop=PSP), 0, 1)$value # 0.3333314}

\dontrun{
tauCOP(cop=PSP, brute=TRUE) # 0.3306625
# CPU heavy example showing that the dual-integration (fast) results in
# a Kendall Tau that matches a sample version
dotau <- function(n) {
   uv <- simCOP(n=n, cop=PSP, ploton=FALSE, points=FALSE)
   return(cor(uv$U, uv$V, method="kendall"))
}
set.seed(817600)
taus <- replicate(100, dotau(100))
tau.sample <- mean(taus); print(tau.sample) # 0.3342034}

\dontrun{
# Nelsen (2006, pp. 160-161, numeric results shown thereine)
# The rational values or integers may be derived analytically.
tauCOP(cop=M, cop2=M) #   1, correct
tauCOP(cop=M, cop2=P) # 1/3, correct
tauCOP(cop=P, cop2=M) # 1/3, correct
tauCOP(cop=M, cop2=W) #   0, correct
tauCOP(cop=W, cop2=M) # throws warning, swaps copulas, == tauCOP(M,W)
tauCOP(cop=W, cop2=P) # throws warning, swaps copulas, approx. -1/3
tauCOP(cop=P, cop2=W) # -1/3, correct
tauCOP(cop=P, cop2=P) #    0, correct
tauCOP(cop=M, cop2=W, brute=TRUE) #    0, correct}

\dontrun{
para <- list(cop1=PLACKETTcop,  cop2=PLACKETTcop,
             para1=0.00395, para2=4.67, alpha=0.9392, beta=0.5699)
tauCOP(cop=composite2COP, para=para) # -0.4671213

para <- list(cop1=PLACKETTcop,  cop2=PLACKETTcop,
             para1=0.14147, para2=20.96, alpha=0.0411, beta=0.6873)
tauCOP(cop=composite2COP, para=para) # +0.1950727

para <- list(cop1=PLACKETTcop,  cop2=PLACKETTcop,
             para1=0.10137, para2=4492.87, alpha=0.0063, beta=0.0167)
# Theoretical attempt fails because para2 is large and thus a singularity
# is emerging and internal copula swapping does not help.
tauCOP(cop=composite2COP, para=para) # fails (0.94+-.01)
tauCOP(cop=composite2COP, para=para, brute=TRUE) # about 0.94+-.01}
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{Nelsen (2006) Examples and Exercises}
\keyword{concordance function}
\keyword{literature errors and inconsistencies}
\keyword{Kendall Tau}

\encoding{utf8}
\name{W_N5p12a}
\alias{W_N5p12a}
\title{Ordinal Sums of Lower-Bound Copula, Example 5.12a of Nelsen's Book}
\description{
Compute shuffles of \emph{\enc{Fréchet}{Frechet}--Hoeffding lower-bound copula} (Nelsen, 2006, p. 173), which is defined by partitioning \eqn{\mathbf{W}} within \eqn{\mathcal{I}^2} into \eqn{n} subintervals:
\deqn{\mathbf{W}_n(u,v) = \mathrm{max}\biggl(\frac{k-1}{n}, u+v-\frac{k}{n} \biggr)}
for points within the partitions
\deqn{(u,v) \in \biggl[\frac{k-1}{n}, \frac{k}{n}\biggr]\times \biggl[ \frac{k-1}{n}, \frac{k}{n}\biggr]\mbox{,\ }k = 1,2,\cdots,n}
and for points otherwise out side the partitions
\deqn{\mathbf{W}_n(u,v) = \mathrm{min}(u,v)\mbox{.}}
The support of \eqn{\mathbf{W}_n} consists of \eqn{n} line segments connecting coordinate pairs \eqn{\{(k-1)/n,\, k/n\}} and \eqn{\{k/n,\, (k-1)/n\}} as stated by Nelsen (2006). The \emph{Spearman Rho} (\code{\link{rhoCOP}}) is defined by \eqn{\rho_\mathbf{C} = 1 - (2/n^2)}, and the \emph{Kendall Tau} (\code{\link{tauCOP}}) by \eqn{\tau_\mathbf{C} = 1 - (2/n)}.
}
\usage{
W_N5p12a(u, v, para=1, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A positive integer \eqn{n \in 1, 2, \cdots}; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  Value(s) for the copula are returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{W}}, \code{\link{ORDSUMcop}}, \code{\link{ORDSUWcop}}, \code{\link{M_N5p12b}}}
\examples{
W_N5p12a(0.4, 0.6, para=5)

\dontrun{
  # Nelsen (2006, exer. 5.12a, p. 172, fig. 5.3a)
  UV <- simCOP(1000, cop=W_N5p12a, para=4) # which is the same as
  para <- list(cop=c(W, W, W, W), para=NULL, part=c(0,0.25,0.50,0.75,1))
  UV <- simCOP(1000, cop=ORDSUMcop, para=para) }
}
\keyword{copula (formulas)}
\keyword{copula (limits/bounds)}
\concept{Shuffle of Frechet lower-bound copula}
\concept{Shuffle of Frechet-Hoeffding lower-bound copula}
\concept{Shuffle of Frechet--Hoeffding lower-bound copula}
\concept{Shuffle of Frechet lower bound}
\concept{Shuffle of Frechet-Hoeffding lower-bound}
\concept{Shuffle of Frechet--Hoeffding lower-bound}
\keyword{copula}
\keyword{shuffle}
\keyword{Nelsen (2006) Examples and Exercises}

\encoding{utf8}
\name{LCOMDIA_GH2cop}
\alias{LCOMDIA_GH2cop}
\title{L-comoment Ratio Diagrams for 2-Parameter Gumbel--Hougaard Extreme Value Copula}
\description{
L-comoment ratio diagrams by computation example are shown for the 2-parameter \emph{Gumbel--Hougaard copula} (\code{\link{GHcop}}). Diagram computations for the 3-parameter version are found under \code{\link{LCOMDIA_GH3cop}}. Diagram computations for the 1-parameter version are found among many copulas in \code{\link{LCOMDIA_ManyCops}}.
}
\author{W.H. Asquith}
\seealso{\code{\link{lcomCOP}}, \code{\link{LCOMDIA_GH3cop}}, \code{\link{LCOMDIA_ManyCops}}
}
\examples{
\dontrun{
  # b1 <- 1 # fixing 2pGHcop gives it L-coskews and L-cokurtoses matching the Clayton.
  GH   <- lcomCOP(cop=GHcop, para=c(1, 1.23107734))
  clfunc  <- function(k, rho=NA) rho - rhoCOP(cop=CLcop, para=exp(k) - 1)
  cl   <- exp( uniroot(clfunc, c(-9, +5), rho=GH$lcomUV[2])$root)- 1
  CL   <- lcomCOP(cop=CLcop, para=cl) # }

\dontrun{
  # ------------------------------------------------------------------------------------
  # L-comoments of the GH2. Alert, there is a commented out write_feather below, so for
  # a "production" run, uncomment that. The reasoning for kicking-out output every nsim
  # so that another R session can already be "harvesting" output should the user seek
  # to study the output earlier than total run will make.
  PLOT <- FALSE
  for(f in 1:20) {
    if(PLOT) {
      plot(c(0, 1), c(-0.2, +0.2), type="n", xaxs="i", yaxs="i", las=1,
           xlab="Spearman Rho", ylab="L-coskew (cT3) or L-cokurtosis (cT4)")
      legend("topright", c("L-coskew of GHcop(para=c(b1,b2))",
                           "L-cokurtosis of GHcop(para=c(b1,b2))"), bty="n",
            pch=c(15, 16), pt.cex=0.5, col=c("seagreen4", "orchid3"))
    }
    fileGH2 <- tempfile(); message("Temp File = ", fileGH2)
    nsim <- 10000
    for(i in seq_len(nsim)) {
      if(length(grep("000$", as.character(i)))) message(i,"-", appendLF=FALSE)
      b1 <- 10^(runif(1, min=-5, max=3))+1
      b2 <- ifelse(length(grep("[13579]$", as.character(i))),   # 1/2 between 0..1
              10^runif(1, min=0, max=3), 10^runif(1, min=-6, max=0)) # and 1/2 > 1
      GH2 <- lcomCOP(cop=GHcop, para=c(b1, b2), stop.on.error=FALSE)
      GH2$lcomUV <- round(GH2$lcomUV, digits=12)
      GH2$lcomVU <- round(GH2$lcomVU, digits=12)
      d2 <- data.frame(b1, b2,
              T2.12=GH2$lcomUV[2], T3.12=GH2$lcomUV[3], T4.12=GH2$lcomUV[4],
              T2.21=GH2$lcomVU[2], T3.21=GH2$lcomVU[3], T4.21=GH2$lcomVU[4])
      row.names(d2) <- NULL; #print(d2)
      if(abs(d2$T2.12) < 1 & d2$T2.12 > 0) {
        if(PLOT) points(d2$T2.12, d2$T3.12, cex=0.5, pch=15, col="seagreen4")
        if(PLOT) points(d2$T2.12, d2$T4.12, cex=0.5, pch=16, col="orchid3"  )
      }
      if(abs(d2$T2.21) < 1 & d2$T2.21 > 0) {
        if(PLOT) points(d2$T2.21, d2$T3.21, cex=0.5, pch=15, col="seagreen4")
        if(PLOT) points(d2$T2.21, d2$T4.21, cex=0.5, pch=16, col="orchid3"  )
      }
      k <- i == 1 # triggering mechanism for write.table()
      write.table(d2, file=fileGH2, sep="\t", append=! k, col.names=k, row.names=FALSE)
    }
    message("done")
    fGH2 <- read.table(fileGH2, sep="\t", header=TRUE)
    file <- paste0("LCM_GH2_", f, ".feather")
    message("writing file '", file, "' to ", getwd())
    # feather::write_feather(fGH2, file)
  }
  # ------------------------------------------------------------------------------------
  files <- list.files(pattern=".feather")
  files <- files[grep("GH2_", files)]
  GH2 <- NULL
  for(file in files) GH2 <- rbind(GH2, as.data.frame(feather::read_feather(file)))

  dr <- 0.001; rhos <- seq(0,1, by=dr); T2 <- c(GH2$T2.12, GH2$T2.21)
  T3 <- c(GH2$T3.12, GH2$T3.21);        T4 <- c(GH2$T4.12, GH2$T4.21)
  T2 <- c(0,0, T2, 1,1); T3 <- c(0,0, T3, 0,0); T4 <- c(0,0, T4, 0,0)
  df <- data.frame(T2, T3, T4)
  df <- df[df$T2 >= 0,] # giant simulations show no situations of negatives
  df$T2[df$T2 > 1] <- 1 # giant simulations show only a few meet this condition and are
  # greater than one at the fifth or beyond decimal, so declare as round-off issues and
  # not failure itself in the 2-parameter GHcop. (Like 3 in 200,000 simulations)
  df <- df[! (df$T2 > 0.997 & df$T3 > -0.468319*df$T2 + 0.468319), ]

  plot(df$T2, df$T3, pch=16, cex=0.2, col=grey(0.8), las=1,
       xlab="Spearman Rho", ylab="L-coskew (cT3)")
  negT3 <- posT3 <- NULL
  for(rho in rhos) {
    negT3 <- c(negT3, min(df$T3[abs(df$T2 - rho) <= dr]))
    posT3 <- c(posT3, max(df$T3[abs(df$T2 - rho) <= dr]))
  }
  X <- rhos[is.finite(negT3)]; Y <- negT3[is.finite(negT3)]
  negT3 <- lm(Y~I(X^ 1)+I(X^ 2)+I(X^ 3)+I(X^ 4)+I(X^ 5)+I(X^ 6)+
                I(X^ 7)+I(X^ 8)+I(X^ 9)+I(X^10)+I(X^11)+I(X^12) )
  X <- rhos[is.finite(posT3)]; Y <- posT3[is.finite(posT3)]
  posT3 <- lm(Y~I(X^ 1)+I(X^ 2)+I(X^ 3)+I(X^ 4)+I(X^ 5)+I(X^ 6)+
                I(X^ 7)+I(X^ 8)+I(X^ 9)+I(X^10)+I(X^11)+I(X^12) )
  lines(rhos, predict(negT3, newdata=data.frame(X=rhos)), lwd=2)
  lines(rhos, predict(posT3, newdata=data.frame(X=rhos)), lwd=2)

  t2t3hull <- grDevices::chull(df$T2, df$T3)
  polygon(df$T2[t2t3hull],  df$T3[t2t3hull], col=NULL, border="seagreen4", lwd=3,lty=2)
  write.table(data.frame(T2=df$T2[t2t3hull], T3=df$T3[t2t3hull]),
              file="lcom_GH2cop_nodes.txt", sep="\t")
  # The user might consider plotting this polygon() underneath L-comoment curves
  # computed and drawn through the recipe in LCOMDIA_ManyCops.Rd.

  X <- df$T2[t2t3hull]; Y <- df$T3[t2t3hull]
  X <- c(X[Y < 0], 0, 1); Y <- c(Y[Y < 0], 0, 0)
  negT3 <- lm(Y~I(X^ 1)+I(X^ 2)+I(X^ 3)+I(X^ 4)+I(X^ 5)+I(X^ 6)+
                I(X^ 7)+I(X^ 8)+I(X^ 9)+I(X^10)+I(X^11)+I(X^12) )
  X <- df$T2[t2t3hull]; Y <- df$T3[t2t3hull]
  X <- c(X[Y > 0], 0, 1); Y <- c(Y[Y > 0], 0, 0)
  posT3 <- lm(Y~I(X^ 1)+I(X^ 2)+I(X^ 3)+I(X^ 4)+I(X^ 5)+I(X^ 6)+
                I(X^ 7)+I(X^ 8)+I(X^ 9)+I(X^10)+I(X^11)+I(X^12) )
  plot(df$T2, df$T3, pch=16, cex=0.2, col="lightgreen", las=1,
       xlab="Spearman Rho", ylab="L-coskew (cT3)")
  lines(rhos, predict(negT3, newdata=data.frame(X=rhos)), lwd=2)
  lines(rhos, predict(posT3, newdata=data.frame(X=rhos)), lwd=2)
  print(negT3, 12); print(posT3, 12)
  "T3fT2" <- function(rho, type=c("neg", "pos")) {
    type <- match.arg(type)
    if(type == "neg") {
      cs <- c(1.77467318015e-04,
          -3.65537522263e-01, 1.32139171758e+00, -2.26891136600e+01, 2.14717293829e+02,
          -1.20819018654e+03, 4.36103722683e+03, -1.04460882313e+04, 1.67876630052e+04,
          -1.78950352179e+04, 1.21363232163e+04, -4.74068383919e+03, 8.11989541291e+02)
    } else if(type == "pos") {
      cs <- c(-8.77010579087e-06,
           2.45271423943e-01, -2.67994483238e-01, 1.28094964505e+00, -1.28531448606e+01,
           7.61335214074e+01, -2.87746627019e+02, 7.17887377219e+02, -1.19681860869e+03,
           1.31930086424e+03, -9.23025213380e+02, 3.71244634445e+02, -6.53809357075e+01)
    } else {
      stop("should not be here in logic")
    }
    z <- sapply(rho, function(r) sum(sapply(seq_len(length(cs)),
                                     function(p) cs[p] * r^(p-1))))
    return(z)
  }
  plot(df$T2, df$T3, pch=16, cex=0.2, col="lightgreen", las=1,
       xlab="Spearman Rho", ylab="L-coskew (cT3)")
  lines(rhos, T3fT2(rhos, type='neg')) # a last check that the polynomial approximations
  lines(rhos, T3fT2(rhos, type='pos')) # have been ported to subroutine acceptably.
  # The user might consider plotting these two polynomials underneath L-comoment curves
  # computed and drawn through the recipe in LCOMDIA_ManyCops.Rd.
  # ------------------------------------------------------------------------------------}
}
\keyword{L-comoments (ratio diagram)}
\keyword{L-comoment ratio diagram}

\encoding{utf8}
\name{tailordCOP}
\alias{tailordCOP}
\title{The Lower- and Upper-Tail Orders of a Copula}
\description{
Compute the \emph{lower-} and \emph{upper-tail orders} (if they exist), respectively, of a copula \eqn{\mathbf{C}(u,v)} according to Joe (2014, pp. 67--70). The \emph{tail order} is a concept for the strength of dependence in the joint tails of a multivariate distribution. The opposing tails can be compared to assess tail order or \emph{reflection symmetry} (term by Joe (2014) for Nelsen's (2006, p. 36) term \emph{radial symmetry}). Joe (2014) provides extensively analytical details but sufficient for the \pkg{copBasic} package, the tail orders can be numerically explored.

The \emph{lower-tail order} maybe numerically approximated by
\deqn{\kappa^L_\mathbf{C} = \frac{\log\bigl(\mathbf{C}(t,t)\bigr)}{\log(t)}\mbox{,}}
for some small positive values of \eqn{t}, and similarly the \emph{upper-tail order} maybe numerically approximated by
\deqn{\kappa^U_\mathbf{C} = \frac{\log\bigl(\hat{\mathbf{C}}(t,t)\bigr)}{\log(t)}\mbox{,}}
where \eqn{\hat{\mathbf{C}}(u,v)} is the \emph{survival copula} (\code{\link{surCOP}}). Joe (2014) has potentially(?) conflicting notation in the context of the upper-tail order; the term \dQuote{reflection} is used (p. 67) and \dQuote{lower tail order of the reflected copula is the same as the upper tail order of the original copula} (p. 69), but Joe (2014, p. 67) only uses the joint survival function (\code{\link{surfuncCOP}}) in the definition of \eqn{\kappa^U_\mathbf{C}}.

As a note, the author of this package was not able to get \code{tailordCOP} to function properly for the upper-tail order using the joint survival function as implied on the bottom of Joe (2014, p. 67) and fortunately the fact that \dQuote{reflection} is used in other contexts and used in analytical examples, the \code{tailordCOP} function uses the lower-tail order of the reflection (survival copula). Joe (2014) also defines \emph{tail order parameter} \eqn{\Psi} but that seems to be a result of analytics and not implemented in this package. Lastly, the tail orders are extendable into \eqn{d} dimensions, but only a bivariate (\eqn{d = 2}) is provided in \pkg{copBasic}. The tail orders have various classifications for \eqn{\kappa = \kappa_L = \kappa_U}:
\describe{
\item{\eqn{\bullet}}{\emph{Intermediate tail dependence} for \eqn{1 < \kappa < d} or \eqn{\kappa = 1, \Psi = 0};}
\item{\eqn{\bullet}}{\emph{Strong tail dependence} for \eqn{\kappa = 1} with \eqn{\Psi > 0}; and}
\item{\eqn{\bullet}}{\emph{Tail orthant independence} or \emph{tail quadrant independence} for \eqn{\kappa = d}.}
}
Joe (2014) provides additional properties:
\describe{
\item{\eqn{\bullet}}{\eqn{\kappa_L = \kappa_U = d} for the \eqn{d}-dimensional \emph{independence copula} (\code{\link{P}}; \emph{e.g.} \code{tailordCOP(cop=P)});}
\item{\eqn{\bullet}}{It is not possible for \eqn{\kappa_L < 1} or \eqn{\kappa_U < 1} but each can be \eqn{> 1} for a \eqn{\mathbf{C}(u,v)} having some negative dependence (\emph{e.g.} \code{tailordCOP(cop=PLACKETTcop, para=0.2)}; see \code{\link{PLACKETTcop}}); and}
\item{\eqn{\bullet}}{For the bivariate \emph{\enc{Fréchet}{Frechet}--Hoeffding lower-bound copula} (\code{\link{W}}; \emph{countermonotonicity copula}) the \eqn{\kappa_L = \kappa_U} and can be considered \eqn{+\infty}. (A special trap in the \code{tailordCOP} provides consistency on \code{\link{W}} but does not test that the copula is actually that function itself.)}
}
}
\usage{
tailordCOP(cop=NULL, para=NULL, tol=1e-6, plot=FALSE, verbose=FALSE, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{tol}{A tolerance on convergence;}
  \item{plot}{A logical plotting a diagnostic plot of the diagonal derivatives and label the limits;}
  \item{verbose}{Show incremental progress; and}
  \item{...}{Additional arguments to pass to the copula function.}
}
\value{
  An \R list is returned.
  \item{kappaL}{The rounded value of \eqn{\kappa^L_\mathbf{C}};}
  \item{kappaU}{The rounded value of \eqn{\kappa^U_\mathbf{C}};}
  \item{source}{An attribute identifying the computational source: \dQuote{tailordCOP}.}
}
\note{
The algorithm implemented for \code{tailordCOP} is based on halves (or alternatives by the setting of the \code{divisor} argument) and uses the copula function (not an analytical or even numerical derivative of the diagonal, \eqn{\delta_\mathbf{C}'(t)}). Starting from the median or \eqn{t = 0.5}, each limit is respectively computed by successive halving of the distance towards \eqn{0^{+}} and checking the change in computed value against the tolerance \code{tol} argument. After the change becomes less than the the \code{tol}erance, convergence is assumed. Other tests are made for \code{NaN} to aid in breaking the successive halving. The rounding for the numerical results for \eqn{\kappa^U_\mathbf{C}} and \eqn{\kappa^L_\mathbf{C}} is an order of magnitude larger than the tolerance.

Users are encouraged to plot the results and further verify whether the convergence makes sense. The plot produced when \code{plot=TRUE} shows the probability \eqn{t} transformed into standard normal variates by the \code{qnorm()} function in \R so that the distal reaches of each tail and thus limit are readily seen. The terminal points of each limit computation are shown by a small dot, and the letter \dQuote{L} and \dQuote{U} also are plotted at the terminal points.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{COP}}, \code{\link{tailconCOP}}, \code{\link{taildepCOP}}}
\examples{
\dontrun{
# Joe (2014, p. 5) names MTCJ = Mardia-Takahasi-Cook-Johnson copula
"MTCJ" <- function(u,v, para) { (u^(-para) + v^(-para) - 1)^(-1/para) }
# The results that follow match those reported by Joe (2014, p. 69) who
# analytically derives KappaL = 1 and KappaU = 2.
# TAIL ORDER:
tailordCOP(cop=MTCJ, para=3, plot=TRUE) # kappaL  = 1.00667, kappaU  = 1.96296
# TAIL DEPENDENCY:
taildepCOP(cop=MTCJ, para=3, plot=TRUE) # lambdaL = 0,       lambdaU = 0.7937
# Joe (2014) reports lambdaL = 2^(-1/para) = 2^(-1/3) = 0.7937005}
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{copula (tail characteristics)}
\keyword{copula (tail properties)}
\keyword{Joe (2014) Examples and Exercises}

\encoding{utf8}
\name{COP}
\alias{COP}
\title{The Copula}
\description{
Compute the \emph{copula} or \emph{joint distribution function} through a copula as shown by Nelsen (2006, p. 18) is the joint probability

\deqn{\mathrm{Pr}[U \le u, V \le v] = \mathbf{C}(u,v)\mbox{.}}

The copula is an expression of the joint probability that both \eqn{U \le u} and \eqn{V \le v}.

A copula is a type of \emph{dependence function} that permits straightforward characterization of dependence from independence.  Joe (2014, p. 8) comments that \dQuote{copula families are usually given as cdfs [cumulative distribution functions.]} A \emph{radially symmetric} or \emph{permutation symmetric copula} is one such that \eqn{\mathbf{C}(u,v) = \mathbf{C}(v,u)} otherwise the copula is \emph{asymmetric}.

The copula \emph{inversions} \eqn{t = \mathbf{C}(u{=}U, v)} or \eqn{t = \mathbf{C}(u, v{=}V)} for a given \eqn{t} and \eqn{U} or \eqn{V} are provided by \code{\link{COPinv}} and \code{\link{COPinv2}}, respectively. A copula exists in the domain of the unit square (\eqn{\mathcal{I}^2 = [0, 1]\times [0,1]}) and is a \emph{grounded} function meaning that
\deqn{\mathbf{C}(u,0) = 0 = \mathbf{C}(0,v) \mbox{\ and\ thus\ } \mathbf{C}(0,0) = 0\mbox{, }}
and other properties of a copula are that
\deqn{\mathbf{C}(u,1) = u \mbox{\ and\ } \mathbf{C}(1,v) = v\mbox{\ and}}
\deqn{\mathbf{C}(1,1) = 1\mbox{.}}

Copulas can be combined with each other (\code{\link{convexCOP}}, \code{\link{convex2COP}}, \code{\link{composite1COP}}, \cr \code{\link{composite2COP}}, \code{\link{composite3COP}}, and \code{\link{glueCOP}}) to form more complex and sophisticated dependence structures. Also copula multiplication---a special product of two copulas---yields another copula (see \code{\link{prod2COP}}).

Perhaps the one of the more useful features of this function is that in practical applications it can be used to take a copula formula and reflect or rotated it in fashions to attain association structures that the native definition of the copula can not acquire. The terminal demonstration in the \bold{Examples} demonstrates this for the \emph{Raftery copula} (\code{\link{RFcop}}).
}
\usage{
COP(u, v, cop=NULL, para=NULL,
          reflect=c("cop", "surv", "acute", "grave",
                      "1",    "2",     "3",     "4"), ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{cop}{A copula function with vectorization as in \code{asCOP};}
  \item{para}{Vector of parameters or other data structures, if needed, to pass to the copula;}
  \item{reflect}{The reflection of the copula form (see \bold{Note}) and the default \code{"cop"} or \code{"1"} is the usual copula definition (also see \code{\link{simCOPmicro}}). The numbered values correspond, respectively, to the named values; and}
  \item{...}{Additional arguments to pass to the copula.}
}
\value{
  Value(s) for the copula are returned.
}
\note{
\emph{REFLECTIONS OF VARIABLES (ROTATIONS OF THE COPULA)}---The copula of \eqn{(1-U, 1-V)} is the survival copula (\eqn{\hat{\mathbf{C}}(u,v)}; \code{\link{surCOP}}) and is defined as
\deqn{\mathrm{Pr}[\,U > u, V > v\,] = \hat{\mathbf{C}}(u,v) = u + v - 1 + \mathbf{C}(1-u,1-v)\:\rightarrow\mbox{\ \code{"surv"},}}
whereas, following the notation of Joe (2014, p. 271), the copula of \eqn{(1-U, V)} is defined as
\deqn{\mathrm{Pr}[\,U > u, V \le v\,] = \acute{\mathbf{C}}(u,v) = v - \mathbf{C}(1-u,v)\:\rightarrow\mbox{\ \code{"acute"}, and}}
 the copula of \eqn{(U, 1-V)} is defined as
\deqn{\mathrm{Pr}[\,U \le u, V > v\,] = \grave{\mathbf{C}}(u,v) = u - \mathbf{C}(u,1-v)\:\rightarrow\mbox{\ \code{"grave"}.}}
Here it is useful to stress the probability aspects that change with the reflections, but this section ends with the reflections themselves being graphically highlighted. The \bold{Examples} stress simple variations on the probability aspects.

To clarify the seemingly clunky nomenclature---Joe (2014) does not provide \dQuote{names} for \eqn{\acute{\mathbf{C}}(u,v)} or \eqn{\grave{\mathbf{C}}(u,v)}---the following guidance is informative where the numbers in the list align to those for the \code{reflect} argument:\cr
\eqn{\mbox{}\quad\mbox{}}(2) \code{"surv"} or \eqn{\hat{\mathbf{C}}(u,v)} is a reflection of \eqn{U} and \eqn{V} on the horizontal \emph{and} vertical axes, respectively,\cr
\eqn{\mbox{}\quad\mbox{}}(3) \code{"acute"} or \eqn{\acute{\mathbf{C}}(u,v)} is a reflection of \eqn{U} on the horizontal axis, and\cr
\eqn{\mbox{}\quad\mbox{}}(4) \code{"grave"} or \eqn{\grave{\mathbf{C}}(u,v)} is a reflection of \eqn{V} on the verical axis.\cr
The names \code{"acute"} and \code{"grave"} match those used in the \bold{Rd}-format math typesetting instructions. Users are directed to the documentation of \code{\link{simCOPmicro}} for further discussion because the \code{COP} function is expected to be an early entry point for new users interested in the \pkg{copBasic} API.

For the \pkg{copBasic} package and in order to keep some logic brief and code accessible for teaching and applied circumstances, reflections of copulas using analogs to the \code{reflect} argument are only natively supported in the \code{COP} and \code{\link{simCOPmicro}} functions. The interfaces of \pkg{copBasic} should already be flexible enough for users to adapt and (or) specially name reflections of copulas for deployment. A caveat is that some individual copula implementations might have some self-supporting infrastructure. The reflection can also be set within the \code{para} argument when it is a list (see \bold{Examples}).

An example is warranted. Although the Gumbel--Hougaard copula (\code{\link{GHcop}}) can be reflected by \code{COP} and \code{\link{simCOPmicro}} and testing is made in the \bold{Note} section of \code{\link{simCOPmicro}}, it is suggested that a user generally requiring say a horizontal reflection \code{ru} (or vertical reflection \code{rv}) of the Gumbel--Hougaard copula write a function named perhaps \code{ruGHcop} (or \code{rvGHcop}).

Such functions, consistent with the mathematics at the beginning of this \bold{Note}, can be used throughout functions of \pkg{copBasic} using the \code{cop} arguments. The author (Asquith) eschews implementing what is perceived as too much flexibility and overhead for the package to support the three reflection permutations universally across all copula functions of the package. This being said, \code{COP} can take an \R \code{list} for the \code{para} argument for rotation/reflection:
\preformatted{
  set.seed(14)
  UV3 <- simCOP(20, cop=COP, pch=16, col=3,
                para=list(cop=GLcop, para=pi+1, reflect="3"))
  set.seed(14)
  UV2 <- simCOP(20, cop=COP, pch=16, col=4, ploton=FALSE,
                para=list(cop=GLcop, para=pi+1, reflect="2"))
  arrows(x0=UV3[,1], y0=UV3[,2], x=UV2[,1], y=UV2[,2])
}
and this type of interface is similar to \code{\link{composite1COP}} as the following rotation and then asymmetric construction shows:
\preformatted{
  UV <- simCOP(1000, cop=composite1COP,
                     para=list(cop1=COP,
                               para1=c(cop=GHcop, para=pi+1, reflect="4"),
                               alpha=0.1, beta=0.3))
}
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{coCOP}}, \code{\link{duCOP}}, \code{\link{surCOP}}, \code{\link{surfuncCOP}}}
\examples{
u <- runif(1); v <- runif(1)
COP(cop=W,u,v); COP(cop=P,u,v); COP(cop=M,u,v); COP(cop=PSP,u,v)

FF <- 0.75 # 75th percentile, nonexceedance
GG <- 0.20 # 25th percentile, nonexceedance
bF <- 1 - FF; bG <- 1 - GG     # exceedance
# What is the probability that both X and Y are less than
# 75th and 20th percentiles, respectively?
COP(cop=P, FF, GG)    # 0.15
# What is the probability that both X and Y are greater than
# 75th and 20th percentiles, respectively?
surCOP(cop=P, bF, bG) # 0.20
# What is the probability that either X or Y are less than
# the 75th and 20th percentiles, respectively?
duCOP(cop=P, FF, GG)  # 0.8
# What is the probability that either X or Y are greater than
# the 75th and 20th percentiles, respectively?
coCOP(cop=P, bF, bG)  # 0.85

# Repeat for the PSP copula:
# What is the probability that both X and Y are less than
# 75th and 20th percentiles, respectively?
COP(cop=PSP, FF, GG)    # 0.1875
# What is the probability that both X and Y are greater than
# 75th and 20th percentiles, respectively?
surCOP(cop=PSP, bF, bG) # 0.2375
# What is the probability that either X or Y are less than
# the 75th and 20th percentiles, respectively?
duCOP(cop=PSP, FF, GG)  # 0.7625
# What is the probability that either X or Y are greater than
# the 75th and 20th percentiles, respectively?
coCOP(cop=PSP, bF, bG)  # 0.8125
# Both of these summations equal unity
   COP(cop=PSP, FF, GG) + coCOP(cop=PSP, bF, bG) # 1
surCOP(cop=PSP, bF, bG) + duCOP(cop=PSP, FF, GG) # 1

FF <- 0.99 # 99th percentile, nonexceedance
GG <- 0.50 # 50th percentile, nonexceedance
bF <- 1 - FF # nonexceedance
bG <- 1 - GG # nonexceedance
# What is the probability that both X and Y are less than
# 99th and 50th percentiles, respectively?
COP(cop=P, FF, GG)    # 0.495
# What is the probability that both X and Y are greater than
# 99th and 50th percentiles, respectively?
surCOP(cop=P, bF, bG) # 0.005
# What is the probability that either X or Y are less than
# the 99th and 50th percentiles, respectively?
duCOP(cop=P, FF, GG)  # 0.995
# What is the probability that either X or Y are greater than
# the 99th and 50th percentiles, respectively?
coCOP(cop=P, bF, bG)  # 0.505

\dontrun{
  # MAJOR EXAMPLE FOR QUICKLY MODIFYING INHERENT ASSOCIATION STRUCTURES
  p <- 0.5 # Reasonable strong positive association for the Raftery copula
  "RFcop1" <- function(u,v, para) COP(u,v, cop=RFcop, para=para, reflect="1")
  "RFcop2" <- function(u,v, para) COP(u,v, cop=RFcop, para=para, reflect="2")
  "RFcop3" <- function(u,v, para) COP(u,v, cop=RFcop, para=para, reflect="3")
  "RFcop4" <- function(u,v, para) COP(u,v, cop=RFcop, para=para, reflect="4")

  d <- 0.01 # Just to speed up the density plots a bit
  densityCOPplot(RFcop1, para=p, contour.col=1, deluv=d) # Raftery in the literature
  densityCOPplot(RFcop2, para=p, contour.col=1, deluv=d, ploton=FALSE)
  densityCOPplot(RFcop3, para=p, contour.col=1, deluv=d, ploton=FALSE)
  densityCOPplot(RFcop4, para=p, contour.col=1, deluv=d, ploton=FALSE)
  # Now some text into the converging tail to show the reflection used.
  text(-2,-2, "reflect=1", col=2); text(+2,+2, "reflect=2", col=2)
  text(+2,-2, "reflect=3", col=2); text(-2,+2, "reflect=4", col=2) #}

\dontrun{
  # ALTERNATIVE EXAMPLE FOR QUICKLY MODIFYING INHERENT ASSOCIATION STRUCTURES
  # To show how the reflection can be alternatively specified and avoid in this case
  # making four Raftery functions, pass by a list para argument. Also, demonstrate
  # that cop1 --> cop and para1 --> para are the same in use of the function. This
  # provides some nomenclature parallel to the other compositing functions.
  densityCOPplot(COP, para=list(reflect=1, cop1=RFcop, para=p ), deluv=d,
                            contour.col=1, drawlabels=FALSE)
  densityCOPplot(COP, para=list(reflect=2, cop= RFcop, para1=p), deluv=d,
                            contour.col=2, drawlabels=FALSE, ploton=FALSE)
  densityCOPplot(COP, para=list(reflect=3, cop1=RFcop, para1=p), deluv=d,
                            contour.col=3, drawlabels=FALSE, ploton=FALSE)
  densityCOPplot(COP, para=list(reflect=4, cop= RFcop, para=p ), deluv=d,
                            contour.col=4, drawlabels=FALSE, ploton=FALSE)
  # Now some text into the converging tail to show the reflection used.
  text(-2,-2, "reflect=1", col=2); text(+2,+2, "reflect=2", col=2)
  text(+2,-2, "reflect=3", col=2); text(-2,+2, "reflect=4", col=2) #}

\dontrun{
  # Similar example to previous, but COP() can handle the reflection within a
  # parameter list ,and the reflect, being numeric here, is converted to
  # character internally.
  T12 <- CLcop(tau=0.67)$para # Kendall Tau of 0.67
  T12 <- list(cop=CLcop, para=T12, reflect=2) # reflected to upper tail dependency
  UV  <- simCOP(n=1000, cop=COP, para=T12) # }
}
\keyword{copula (formulas)}
\keyword{copula operator}
\keyword{joint probability}
\keyword{copula (conditional distribution)}
\keyword{copula (conditional quantile function)}
\keyword{copula (reflection)}
\keyword{copula (rotation)}
\keyword{Theory Examples (reflection/rotation)}

\encoding{utf8}
\name{LCOMDIA_ManyCops}
\alias{LCOMDIA_ManyCops}
\title{L-comoment Ratio Diagrams for Many Copulas}
\description{
L-comoment ratio diagrams by computation example are shown for many copulas of the \pkg{copBasic} package.
}
\author{W.H. Asquith}
\seealso{\code{\link{lcomCOP}}, \code{\link{LCOMDIA_GH2cop}}, \code{\link{LCOMDIA_GH3cop}}, \code{\link{ORDSUMcop}}
}
\examples{
\dontrun{
  # Demonstration of an L-comoment ratio diagram as function of Spearman Rho for various
  # copula where each copula is fit to the Spearman Rho and the L-comoments computed.
  # ------------------------------------------------------------------------------------
  # The copulas here are permutation symmetric, which means that T*.12 and T*.21 are
  # identical, but plotting for the T*.21 are shown here to make it easier for the user
  # to insert permutation asymmetry say by blending in the breveCOP() or other
  # composition frameworks. L-cokurtosis is negated to have it simply plot away from
  # the L-coskew.
  # ------------------------------------------------------------------------------------
  LCMempty <- list(lcomUV=rep(NA, 5), lcomVU=rep(NA, 5)) # for comprehensive show of
  # features and handling for situations of a Spearman Rho being incompatible with a
  # copula, which is the case for the N4212cop that we have reversed here to put the
  # tail dependency for moderate Spearman Rho into the upper tail.
  rN4212cop <- function(u,v, para, ...) COP(u,v, cop=N4212cop, para=para, reflect=2)

  clfunc  <- function(k, rho=NA) rho - rhoCOP(cop=COP,
                           para=list(reflect=2, cop=CLcop,  para=exp(k)-1))
  gefunc  <- function(k, rho=NA) rho - rhoCOP(cop=gEVcop,    para=pnorm(k))
  ghfunc  <- function(k, rho=NA) rho - rhoCOP(cop=GHcop,     para=exp(k)+1)
  glfunc  <- function(k, rho=NA) rho - rhoCOP(cop=GLcop,     para=exp(k)  )
  hrfunc  <- function(k, rho=NA) rho - rhoCOP(cop=HRcop,     para=exp(k)  )
  jofunc  <- function(k, rho=NA) rho - rhoCOP(cop=JOcopB5,   para=exp(k)+1)
  mofunc  <- function(k, rho=NA) rho - rhoCOP(cop=MOcop,     para=pnorm(rep(k,2)))
  n12func <- function(k, rho=NA) rho - rhoCOP(cop=rN4212cop, para=exp(k)+1)
  n20func <- function(k, rho=NA) rho - rhoCOP(cop=rN4220cop, para=exp(k)  )
  rafunc  <- function(k, rho=NA) rho - rhoCOP(cop=RAYcop,    para=pnorm(k))
  rffunc  <- function(k, rho=NA) rho - rhoCOP(cop=COP,
                           para=list(reflect=2, cop=RFcop,  para=pnorm(k)))
  D <- NULL; rs <- c(0.0001, 0.0002, 0.0005, 0.0008, 0.001, 0.0013, 0.002, 0.003,
                     0.004, 0.005, 0.006, 0.007, 0.008, 0.009, 0.01, 0.015,
                     seq(0.02, 0.99, by=0.005), 0.99,
              0.991, 0.992, 0.993, 0.994, 0.995, 0.996, 0.997, 0.998)
  rs <- sort(unique(rs)) # to fix defects
  for(r in rs) {  message(r, "-", appendLF=FALSE) # Spearman Rhos,
    ga   <- 2*sin(pi*r/6)
    GA   <- lcomCOP(cop=NORMcop, para=ga)
    cl   <- exp(  uniroot(clfunc,  c(-9,+5), rho=r)$root)-1
    CL   <- lcomCOP(cop=COP, para=list(cop=CLcop, para=cl, reflect=2))
    fr   <- FRcop(rhotau=r, cortype="rho")$para
    FR   <- lcomCOP(cop=FRcop, para=fr)
    ge   <- pnorm(uniroot(gefunc,  c(-4,+4), rho=r)$root)
    GE   <- lcomCOP(cop=gEVcop, para=ge)
    gh   <- exp(  uniroot(ghfunc,  c(-10,+4), rho=r)$root)+1
    GH   <- lcomCOP(cop=GHcop, para=gh)
    gl   <- exp(  uniroot(glfunc,  c(-9,+4), rho=r)$root)
    GL   <- lcomCOP(cop=GLcop, para=gl)
    hr   <- exp(  uniroot(hrfunc,  c(-9,+4), rho=r)$root)
    HR   <- lcomCOP(cop=HRcop, para=hr)
    jo   <- exp(  uniroot(jofunc,  c(-10,+4), rho=r)$root)+1
    JO   <- lcomCOP(cop=JOcopB5, para=jo)
    mo   <- pnorm( uniroot(mofunc, c(-6,+6), rho=r)$root)
    mo   <- rep(mo, 2)
    MO   <- lcomCOP(cop=MOcop, para=mo)
    if(r > 0.4784176) { # rhoCOP(rN4212cop, para=1) = 0.4784176
      n12 <- exp(  uniroot(n12func, c(-9,+4), rho=r)$root)+1
      N12  <- lcomCOP(cop=rN4220cop, para=n12)
    } else {
      n12 <- NA; N12 <- LCMempty
    }
    n20  <- exp(  uniroot(n20func, c(-10,+4), rho=r)$root)
    N20  <- lcomCOP(cop=rN4220cop, para=n20)
    ra   <- RAYcop(rho=r)
    RA   <- lcomCOP(cop=RAYcop, para=ra)
    rf   <- pnorm(  uniroot(rffunc, c(-6,6), rho=r)$root)
    RF   <- lcomCOP(cop=COP, para=list(cop=RFcop, para=rf, reflect=2))
    TF   <- lcomCOP(cop=Tcop, para=c(ga,1))
    x <- data.frame(rho=r, CLpara=cl, FRpara=fr, GApara=ga, GEpara=ge, GHpara=gh,
                GLpara=gl, HRpara=hr, JOpara=jo, MOpara=mo[1],
                N12para=n12, N20para=n20, RApara=ra, RFpara=rf, TFpara=ga,
                 CLT2.12=CL$lcomUV[2],   CLT2.21=CL$lcomVU[2],
                 CLT3.12=CL$lcomUV[3],   CLT3.21=CL$lcomVU[3],
                 CLT4.12=CL$lcomUV[4],   CLT4.21=CL$lcomVU[4],
                 FRT2.12=FR$lcomUV[2],   FRT2.21=FR$lcomVU[2],
                 FRT3.12=FR$lcomUV[3],   FRT3.21=FR$lcomVU[3],
                 FRT4.12=FR$lcomUV[4],   FRT4.21=FR$lcomVU[4],
                 GAT2.12=GA$lcomUV[2],   GAT2.21=GA$lcomVU[2],
                 GAT3.12=GA$lcomUV[3],   GAT3.21=GA$lcomVU[3],
                 GAT4.12=GA$lcomUV[4],   GAT4.21=GA$lcomVU[4],
                 GET2.12=GE$lcomUV[2],   GET2.21=GE$lcomVU[2],
                 GET3.12=GE$lcomUV[3],   GET3.21=GE$lcomVU[3],
                 GET4.12=GE$lcomUV[4],   GET4.21=GE$lcomVU[4],
                 GHT2.12=GH$lcomUV[2],   GHT2.21=GH$lcomVU[2],
                 GHT3.12=GH$lcomUV[3],   GHT3.21=GH$lcomVU[3],
                 GHT4.12=GH$lcomUV[4],   GHT4.21=GH$lcomVU[4],
                 GLT2.12=GL$lcomUV[2],   GLT2.21=GL$lcomVU[2],
                 GLT3.12=GL$lcomUV[3],   GLT3.21=GL$lcomVU[3],
                 GLT4.12=GL$lcomUV[4],   GLT4.21=GL$lcomVU[4],
                 HRT2.12=HR$lcomUV[2],   HRT2.21=HR$lcomVU[2],
                 HRT3.12=HR$lcomUV[3],   HRT3.21=HR$lcomVU[3],
                 HRT4.12=HR$lcomUV[4],   HRT4.21=HR$lcomVU[4],
                 JOT2.12=JO$lcomUV[2],   JOT2.21=JO$lcomVU[2],
                 JOT3.12=JO$lcomUV[3],   JOT3.21=JO$lcomVU[3],
                 JOT4.12=JO$lcomUV[4],   JOT4.21=JO$lcomVU[4],
                 MOT2.12=MO$lcomUV[2],   MOT2.21=MO$lcomVU[2],
                 MOT3.12=MO$lcomUV[3],   MOT3.21=MO$lcomVU[3],
                 MOT4.12=MO$lcomUV[4],   MOT4.21=MO$lcomVU[4],
               N12T2.12=N12$lcomUV[2], N12T2.21=N12$lcomVU[2],
               N12T3.12=N12$lcomUV[3], N12T3.21=N12$lcomVU[3],
               N12T4.12=N12$lcomUV[4], N12T4.21=N12$lcomVU[4],
               N20T2.12=N20$lcomUV[2], N20T2.21=N20$lcomVU[2],
               N20T3.12=N20$lcomUV[3], N20T3.21=N20$lcomVU[3],
               N20T4.12=N20$lcomUV[4], N20T4.21=N20$lcomVU[4],
                 RAT2.12=RA$lcomUV[2],   RAT2.21=RA$lcomVU[2],
                 RAT3.12=RA$lcomUV[3],   RAT3.21=RA$lcomVU[3],
                 RAT4.12=RA$lcomUV[4],   RAT4.21=RA$lcomVU[4],
                 RFT2.12=RF$lcomUV[2],   RFT2.21=RF$lcomVU[2],
                 RFT3.12=RF$lcomUV[3],   RFT3.21=RF$lcomVU[3],
                 RFT4.12=RF$lcomUV[4],   RFT4.21=RF$lcomVU[4],
                 TFT2.12=TF$lcomUV[2],   TFT2.21=TF$lcomVU[2],
                 TFT3.12=TF$lcomUV[3],   TFT3.21=TF$lcomVU[3],
                 TFT4.12=TF$lcomUV[4],   TFT4.21=TF$lcomVU[4] )
    D <- rbind(D, x)
  }
  Z <- D[1,]
  Z$rho <- 0
  Z[,grep("[12]$", names(Z))] <- rep(0, length(grep("[12]$", names(Z))))
  Z[,grep("para$", names(Z))] <- floor(     Z[,grep("para$", names(Z))])
  D <- rbind(Z, D)
  Z <- D[1,]
  Z$rho <- 1
  Z[,grep("[12]$", names(Z))] <- rep(0,   length(grep("[12]$", names(Z))))
  Z[,grep("para$", names(Z))] <- rep(Inf, length(grep("para$", names(Z))))
  D <- rbind(   D, Z)
  message("done")
  row.names(D) <- NULL
  D$MOT3.12[is.na(D$MOT3.12)] <- D$MOT3.21[is.na(D$MOT3.12)]
  D$TFT3.12[      is.na(D$TFT3.12)] <- approx(D$rho[! is.na(D$TFT3.12)],
      D$TFT3.12[! is.na(D$TFT3.12)],     xout=D$rho[  is.na(D$TFT3.12)])$y
  # write.table(D, file="LCOMDIA_ManyCops.txt", sep=",", row.names=FALSE)
  # ------------------------------------------------------------------------------------
  ix <- c(9, 6, 3, 12, 10, 5, 11, 1, 14, 2, 7, 8, 4, 13)
  cols <- hcl.colors(length(ix), palette="Dark3")
  cols <- cols[ix]
  coll <- cols[unlist( sapply(seq_len(length(ix)), function(i) list(rep(i, 2))) )]
  rs <- D$rho
  plot(rs, D$GHT3.12, ylim=c(-0.20, 0.20), type="n", las=1,
      xlab="Spearman Rho", ylab="L-coskew (cT3) or -negated L-cokurtosis (cT4)")
  lines(rs, D$CLT3.12,  col=cols[ 1]); lines(rs, -D$CLT4.12,  col=cols[ 1], lty=2)
  lines(rs, D$CLT3.21,  col=cols[ 1]); lines(rs, -D$CLT4.21,  col=cols[ 1], lty=2)
  lines(rs, D$FRT3.12,  col=cols[ 2]); lines(rs, -D$FRT4.12,  col=cols[ 2], lty=4)
  lines(rs, D$FRT3.21,  col=cols[ 2]); lines(rs, -D$FRT4.21,  col=cols[ 2], lty=4)
  lines(rs, D$GAT3.12,  col=cols[ 3]); lines(rs, -D$GAT4.12,  col=cols[ 3], lty=2)
  lines(rs, D$GAT3.21,  col=cols[ 3]); lines(rs, -D$GAT4.21,  col=cols[ 3], lty=2)
  lines(rs, D$GET3.12,  col=cols[ 4]); lines(rs, -D$GET4.12,  col=cols[ 4], lty=4)
  lines(rs, D$GET3.21,  col=cols[ 4]); lines(rs, -D$GET4.21,  col=cols[ 4], lty=4)
  lines(rs, D$GHT3.12,  col=cols[ 5]); lines(rs, -D$GHT4.12,  col=cols[ 5], lty=2)
  lines(rs, D$GHT3.21,  col=cols[ 5]); lines(rs, -D$GHT4.21,  col=cols[ 5], lty=2)
  lines(rs, D$GLT3.12,  col=cols[ 6]); lines(rs, -D$GLT4.12,  col=cols[ 6], lty=4)
  lines(rs, D$GLT3.21,  col=cols[ 6]); lines(rs, -D$GLT4.21,  col=cols[ 6], lty=4)
  lines(rs, D$HRT3.12,  col=cols[ 7]); lines(rs, -D$HRT4.12,  col=cols[ 7], lty=2)
  lines(rs, D$HRT3.21,  col=cols[ 7]); lines(rs, -D$HRT4.21,  col=cols[ 7], lty=2)
  lines(rs, D$JOT3.12,  col=cols[ 8]); lines(rs, -D$JOT4.12,  col=cols[ 8], lty=4)
  lines(rs, D$JOT3.21,  col=cols[ 8]); lines(rs, -D$JOT4.21,  col=cols[ 8], lty=4)
  lines(rs, D$MOT3.21,  col=cols[ 9]); lines(rs, -D$MOT4.21,  col=cols[ 9], lty=2)
  lines(rs, D$MOT3.12,  col=cols[ 9]); lines(rs, -D$MOT4.12,  col=cols[ 9], lty=2)
  lines(rs, D$N12T3.12, col=cols[10]); lines(rs, -D$N12T4.12, col=cols[10], lty=4)
  lines(rs, D$N12T3.21, col=cols[10]); lines(rs, -D$N12T4.21, col=cols[10], lty=4)
  lines(rs, D$N20T3.12, col=cols[11]); lines(rs, -D$N20T4.12, col=cols[11], lty=2)
  lines(rs, D$N20T3.21, col=cols[11]); lines(rs, -D$N20T4.21, col=cols[11], lty=2)
  lines(rs, D$RAT3.12,  col=cols[12]); lines(rs, -D$RAT4.12,  col=cols[12], lty=4)
  lines(rs, D$RAT3.21,  col=cols[12]); lines(rs, -D$RAT4.21,  col=cols[12], lty=4)
  lines(rs, D$RFT3.12,  col=cols[13]); lines(rs, -D$RFT4.12,  col=cols[13], lty=2)
  lines(rs, D$RFT3.21,  col=cols[13]); lines(rs, -D$RFT4.21,  col=cols[13], lty=2)
  lines(rs, D$TFT3.12,  col=cols[14]); lines(rs, -D$TFT4.12,  col=cols[14], lty=4)
  lines(rs, D$TFT3.21,  col=cols[14]); lines(rs, -D$TFT4.21,  col=cols[14], lty=4)
  txt <- c("cT3 CLcop(reflect)",        "-cT4 CLcop(reflect)",
           "cT3 FRcop (equals zero)",   "-cT4 FRcop",
           "cT3 NORMcop (equals zero)", "-cT4 NORMcop",
           "cT3 gEVcop",                "-cT4 gEVcop",
           "cT3 GHcop",                 "-cT4 GHcop",
           "cT3 GLcop",                 "-cT4 GLcop",
           "cT3 HRcop",                 "-cT4 HRcop",
           "cT3 JOcopB5",               "-cT4 JOcopB5",
           "cT3 MOcop (a=b)",           "-cT4 MOcop (a=b)",
           "cT3 rN4212cop",             "-cT4 rN4212cop",
           "cT3 rN4220cop",             "-cT4 rN4220cop",
           "cT3 RAYcop",                "-cT4 RAYcop",
           "cT3 RFcop",                 "-cT4 RFcop",
           "cT3 Tcop (equals zero)",    "-cT4 Tcop")
  legend("bottomleft", txt, bty="n", cex=0.7, ncol=4, lty=c(1,2,1,4), col=coll)
  mtext("L-comoment ratio diagram for various copula", line=0.8)
  # ------------------------------------------------------------------------------------
  # Now, simulation, note that in samples T[2+].[12|21] are not perfectly equal even for
  # known permutation asymmetry. These could be average or used in isolation. Here, we
  # just plot the two separately. So, there are double the data points. It takes a lot
  # of samples for single sample to look authoritative.
  T2 <- T3 <- T4 <- NULL
  for(i in 1:60) {
    uv  <- simCOP(2000, cop=GLcop, para=0.95, graphics=FALSE)
    lcm <- lmomco::lcomoms2(uv, nmom=4); srho <- (lcm$T2[1,2] + lcm$T2[2,1]) / 2
    points(rep(srho, 2),  c(lcm$T3[1,2], lcm$T3[2,1]), cex=0.6, lwd=0.8, pch=24)
    points(rep(srho, 2), -c(lcm$T4[1,2], lcm$T4[2,1]), cex=0.6, lwd=0.8, pch=25)
    T2 <- c(T2, srho)
    T3 <- c(T3, mean(c(lcm$T3[1,2], lcm$T3[2,1])))
    T4 <- c(T4, mean(c(lcm$T4[1,2], lcm$T4[2,1])))
  } # BTW : compare the srho to srho <- cor(UV, method="spearman")
  mT2 <- mean(T2); mT3 <- mean(T3); mT4 <- mean(T4)
  points(mT2,  mT3, col=cols[6], lwd=2, bg=grey(0.9), pch=24, cex=2)
  points(mT2, -mT4, col=cols[6], lwd=2, bg=grey(0.9), pch=25, cex=2)
  # ------------------------------------------------------------------------------------
  # L-comoment ratio diagram of L-coskew and L-cokurtosis.
  # Again both 12 and 21 (UV and VU) are plotted but for permutation symmetry these
  # "with respect to's" have the same L-comoments. The GLcop was used in the simulation;
  # so, the lines for it are thicker to see that the general centrality of the sample
  # L-comoments provide evidence back to the GLcop copula.
  plot(D$GHT3.12, xlim=c(0, 0.20), ylim=c(-0.075, 0.075), type="n", las=1,
       xlab="L-coskew (cT3)", ylab="L-cokurtosis (cT4)")
  lines( D$CLT3.12,  D$CLT4.12, col=cols[ 1])
  lines( D$CLT3.21,  D$CLT4.21, col=cols[ 1], lty=2)
  lines( D$FRT3.12,  D$FRT4.12, col=cols[ 2])
  lines( D$FRT3.21,  D$FRT4.21, col=cols[ 2], lty=4)
  lines( D$GAT3.12,  D$GAT4.12, col=cols[ 3])
  lines( D$GAT3.21,  D$GAT4.21, col=cols[ 3], lty=2)
  lines( D$GET3.12,  D$GET4.12, col=cols[ 4])
  lines( D$GET3.21,  D$GET4.21, col=cols[ 4], lty=4)
  lines( D$GHT3.12,  D$GHT4.12, col=cols[ 5])
  lines( D$GHT3.21,  D$GHT4.21, col=cols[ 5], lty=2)
  lines( D$GLT3.12,  D$GLT4.12, col=cols[ 6],        lwd=2)
  lines( D$GLT3.21,  D$GLT4.21, col=cols[ 6], lty=4, lwd=2)
  lines( D$HRT3.12,  D$HRT4.12, col=cols[ 7])
  lines( D$HRT3.21,  D$HRT4.21, col=cols[ 7], lty=2)
  lines( D$JOT3.12,  D$JOT4.12, col=cols[ 8])
  lines( D$JOT3.21,  D$JOT4.21, col=cols[ 8], lty=4)
  lines( D$MOT3.12,  D$MOT4.12, col=cols[ 9])
  lines( D$MOT3.21,  D$MOT4.21, col=cols[ 9], lty=2)
  lines(D$N12T3.12, D$N12T4.12, col=cols[10])
  lines(D$N12T3.21, D$N12T4.21, col=cols[10], lty=2)
  lines(D$N20T3.12, D$N20T4.12, col=cols[11])
  lines(D$N20T3.21, D$N20T4.21, col=cols[11], lty=2)
  lines( D$RAT3.12,  D$RAT4.12, col=cols[12])
  lines( D$RAT3.21,  D$RAT4.21, col=cols[12], lty=4)
  lines( D$RFT3.12,  D$RFT4.12, col=cols[13])
  lines( D$RFT3.21,  D$RFT4.21, col=cols[13], lty=2)
  lines( D$TFT3.12,  D$TFT4.12, col=cols[14])
  lines( D$TFT3.21,  D$TFT4.21, col=cols[14], lty=4)
  points( T3,  T4, lwd=0.8, cex=0.8)
  points(mT3, mT4, lwd=2,   cex=2, pch=21, col=cols[6], bg=grey(0.9))
  txt <- c("CLcop(reflect)[UV]", "CLcop(reflect)[VU]",
             "FRcop[UV]",         "FRcop[VU]",
           "NORMcop[UV]",       "NORMcop[VU]",
            "gEVcop[UV]",        "gEVcop[VU]",
             "GHcop[UV]",         "GHcop[VU]",
             "GLcop[UV]",         "GLcop[VU]",
             "HRcop[UV]",         "HRcop[VU]",
             "JOcop[UV]",         "JOcop[VU]",
             "MOcop[UV] (a=b)",   "MOcop[VU] (a=b)",
           "rN4212cop[UV]",   "rN4212cop[VU]",
           "rN4220cop[UV]",   "rN4220cop[VU]",
             "RAcop[UV]",         "RAcop[VU]",
             "RFcop[UV]",         "RFcop[VU]",
              "Tcop[UV]",          "Tcop[VU]")
   legend("bottomright", txt, cex=0.7, ncol=4, lty=c(1,2), col=coll)
   mtext("L-comoment ratio diagram for various copula", line=0.8)
   # ------------------------------------------------------------------------------------}
}
\keyword{L-comoments (ratio diagram)}
\keyword{L-comoment ratio diagram}

\encoding{utf8}
\name{PSP}
\alias{PSP}
\title{The Ratio of the Product Copula to Summation minus Product Copula}
\description{
Compute \emph{PSP copula} (Nelsen, 2006, p. 23) is named by the author (Asquith) for the \pkg{copBasic} package and is
\deqn{\mathbf{PSP}(u,v) = \frac{\mathbf{\Pi}}{\mathbf{\Sigma} - \mathbf{\Pi}} = \frac{uv}{u + v - uv}\mbox{,}}
where \eqn{\mathbf{\Pi}} is the \emph{indpendence} or \emph{product copula} (\code{\link{P}}) and \eqn{\mathbf{\Sigma}} is the sum \eqn{\mathbf{\Sigma} = u + v}. The \eqn{\mathbf{PSP}(u,v)} copula is a special case of the \eqn{\mathbf{N4212}(u,v)} copula (\code{\link{N4212cop}}). The \eqn{\mathbf{PSP}} is included in \pkg{copBasic} because of its simplicity and for pedagogical purposes. The name \dQuote{PSP} comes from \dQuote{Product, Summation, Product} to loosely reflect the mathematical formula shown.  Nelsen (2006, p. 114) notes that the PSP copula shows up in several families and designates it as \dQuote{\eqn{\mathbf{\Pi}/(\mathbf{\Sigma}-\mathbf{\Pi})}.} The PSP is undefined for \eqn{u = v = 0} but no internal trapping is made; calling functions will have to intercept the \code{NaN} so produced for \eqn{\{0, 0\}}. The \eqn{\mathbf{PSP}} is left internally to not trap \code{NaN} so as to be available to stress other copula utility functions within the \pkg{copBasic} package.
}
\usage{
PSP(u, v, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction; and}
  \item{...}{Additional arguments to pass, which for this copula are not needed, but given here to support flexible implementation.}
}
\value{
  Value(s) for the copula are returned.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{P}}, \code{\link{N4212cop}}}
\examples{
PSP(0.4,0.6)
PSP(0,0)
PSP(1,1)
}
\keyword{copula (formulas)}
\keyword{copula}

\encoding{utf8}
\name{breveCOP}
\alias{breveCOP}
\title{Add Asymmetry to a Copula}
\description{
Adding \emph{permutation asymmetry} (Chang and Joe, 2020, p. 1596) (\code{\link{isCOP.permsym}}) is simple for a bivariate copula family. Let \eqn{\mathbf{C}} be a copula with respective vectors of parameters \eqn{\Theta_\mathbf{C}}, then the permutation asymmetry is added through an asymmetry parameter \eqn{\beta \in (-1, +1)} by

\deqn{\breve{\mathbf{C}}_{\beta;\Theta}(u,v) = v^{-\beta}\cdot\mathbf{C}\bigl(u, v^{(1+\beta)};\Theta\bigr)\mbox{, and}}

for \eqn{0 \le \beta \le +1} by

\deqn{\breve{\mathbf{C}}_{\beta;\Theta}(u,v) = u^{+\beta}\cdot\mathbf{C}\bigl(u^{(1-\beta)}, v;\Theta\bigr)\mbox{.}}
The parameter \eqn{\beta} clashes in name and symbology with a parameter used by functions \code{\link{composite1COP}}, \code{\link{composite2COP}}, and \code{\link{composite3COP}}. As a result, support for alternative naming is provided for compatibility. Lastly, an example under \code{\link{W}} demonstrates the span of the \emph{Kendall function} (\code{\link{kfuncCOP}}) based on \code{\link{M}} and \code{\link{W}} having permutation asymmetry inserted by \code{breveCOP()}. The demonstration shows that the Kendall function is a distribution unto itself with its own nonexceedance probabilities.
}
\usage{
breveCOP(u,v, para, breve=NULL, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{para}{A special parameter \code{list} (see \bold{Note});}
  \item{breve}{An alternative way from \code{para} to set the \eqn{\beta} for this function; and}
  \item{...}{Additional arguments to pass to the copula.}
}
\value{
  Value(s) for the copula are returned.
}
\note{
The following descriptions list in detail the structure and content of the \code{para} argument where \code{cop1} and \code{cop} and \code{para1} and \code{para} are respectively synonymous to have some structural similarity to the various copula constructors (compositors) of the \pkg{copBasic} package:
\describe{
  \item{\code{beta}}{--- The \eqn{\beta} asymmetry parameter;}
  \item{\code{breve}}{--- The \eqn{\beta} asymmetry parameter and presence of \code{breve} will not cause the non-use of \code{beta}; this feature is present so that \code{beta} remains accessible to the compositors that use \code{beta} (see \bold{Examples});}
  \item{\code{cop }}{--- Function of the copula \eqn{\mathbf{C}};}
  \item{\code{cop1}}{--- Alternative naming of the function of the coupla \eqn{\mathbf{C}};}
  \item{\code{para }}{--- Vector of parameters \eqn{\Theta_\mathbf{C}} for  \eqn{\mathbf{C}}; and}
  \item{\code{para1}}{--- Alternative naming of the vector of parameters \eqn{\Theta_\mathbf{C}} for  \eqn{\mathbf{C}}.}
}

The function silently restricts the \eqn{\beta} to its interval as defined, but parameter transform might be useful in some numerical optimization schemes. The following recipes might be useful for transform from a parameter in numerical optimization to the asymmetry parameter:
\preformatted{
   #    transform into space for optimization
   BREVEtfunc <- function(p) { return(  qnorm((p[1] + 1) / 2) ) } # [-Inf,+Inf]
   # re-transform back into space for the copula
   BREVErfunc <- function(p) { return(2*pnorm(p[1]) - 1)        } # [-1  ,+1  ]
}
}
\references{
Chang, B., and Joe, H., 2020, Copula diagnostics for asymmetries and conditional dependence: Journal of Applied Statistics, v. 47, no. 9, pp. 1587--1615, \doi{10.1080/02664763.2019.1685080}.
}
\author{W.H. Asquith}
\seealso{\code{\link{COP}}, \code{\link{convex2COP}}, \code{\link{convexCOP}}, \code{\link{composite1COP}}, \code{\link{composite2COP}}, \code{\link{composite3COP}}, \code{\link{FRECHETcop}}, \code{\link{glueCOP}}}
\examples{
para <- list(breve=0.24, cop1=FRECHETcop, para1=c(0.4, 0.56))
breveCOP(0.87, 0.35, para=para) # 0.282743

betas <- seq(-1,1, by=0.01)
bloms <- sapply(betas, function(b) {
             breveCOP(0.15, 0.25, para=list(cop=GLPMcop, para=c(2, 2), beta=b))
         } )
plot(betas, bloms, type="l", main="GLPMcop(u,v; 2,2) by breveCOP(beta)")

\dontrun{
  # Notice the argument cop and para name adjustments to show that
  # translation exists inside the function to have use flexibility.
  para <- list(beta=+0.44, cop1=FRECHETcop, para1=c(0.2, 0.56))
  UV   <- simCOP(1000, cop=breveCOP, para=para)
  para <- list(beta=-0.44, cop= FRECHETcop, para= c(0.2, 0.56))
  UV   <- simCOP(1000, cop=breveCOP, para=para) # }

\dontrun{
  # Testing on a comprehensive copula (Plackett)
  betas <- rhos <- thetas <- brhos <- NULL
  for(beta  in seq(-1, 1, by=0.1 )) {
    for(rho in seq(-1, 1, by=0.01))   {
       theta  <- PLACKETTpar(rho=rho, byrho=TRUE)
       thetas <- c(thetas, theta)
       para   <- list(  cop=PLcop,    para=theta, beta=beta)
       brho   <- rhoCOP(cop=breveCOP, para=para)
       betas  <- c(betas, beta); rhos <- c(rhos, rho)
       brhos  <- c(brhos, brho)
    }
  }
  df <- data.frame(beta=betas, theta=thetas, rho=rhos, brho=brhos)
  plot(df$theta, df$brho, log="x", pch=16, cex=0.9, col="seagreen",
       xlab="Plackett parameter", ylab="Spearman Rho")
  lines(df$theta[df$beta == 0], df$brho[df$beta == 0], col="red", lwd=2)
  # Red line is the Plackett in its permutation symmetric definition. #}

\dontrun{
  # Here is an example for a test using mleCOP() to estimate a 5-parameter asymmetric
  # copula model to "some data" on transition from yesterday to today data for a very
  # large daily time series. The purpose of example here is to demonstrate interfacing
  # to the breveCOP() for it to add asymmetry to composition of two copula.
  myASYMCOP <- function(u,v, para, ...) {
    subpara <- list(alpha=para$alpha, beta=para$beta, cop1=GHcop, para1=para$para1,
                                                      cop2=PLcop, para2=para$para2)
    breveCOP(u,v, cop=convex2COP, para=subpara)
  }
  para <- list(alpha=+0.16934027, cop1=GHcop, para1=c(1.11144148, 10.32292673),
                beta=-0.01923808, cop2=PLcop, para2=3721.82966727)
  UV <- simCOP(30000, cop=myASYMCOP, para=para, pch=16, col=grey(0, 0.1))
  abline(0,1, lwd=3, col="red") #}

\dontrun{
  # Here is a demonstration of the permutations of the passing of the
  # asymmetry parameter into the function and then by
  UV <- simCOP(1E3, cop=breveCOP, para=list(cop=HRcop, para=5), breve=+0.5)
  UV <- simCOP(1E3, cop=breveCOP, para=list(cop=HRcop, para=5), breve=-0.5)
  UV <- simCOP(1E3, cop=breveCOP, para=list(cop=HRcop, para=5,  beta =+0.5))
  UV <- simCOP(1E3, cop=breveCOP, para=list(cop=HRcop, para=5,  breve=+0.5))
  UV <- simCOP(1E3, cop=breveCOP, para=list(cop=HRcop, para=5,  beta=-0.4, breve=+0.5))

  para <- list(cop1=HRcop, para1=6, cop2=PSP, para2=NULL, alpha=1, beta=0.7)
  myCOP <- function(u,v, para, ...) breveCOP(u,v, cop=composite2COP, para=para)
  para$breve <- "here I am"
  UV <- simCOP(1E3, cop=composite2COP, para=para, seed=1) # breve is not used
  para$breve <- -0.16
  UV <- simCOP(1E3, cop=myCOP,         para=para, seed=1)
  para$breve <- +0.16
  UV <- simCOP(1E3, cop=myCOP,         para=para, seed=1) # }
}
\keyword{copula composition}
\keyword{copula composition (permutation asymmetric)}
\keyword{permutation asymmetry}

\encoding{utf8}
\name{PLACKETTpar}
\alias{PLACKETTpar}
\alias{PLpar}
\title{Estimate the Parameter of the Plackett Copula}
\description{
The parameter \eqn{\Theta} of the \emph{Plackett copula} (Nelsen, 2006, pp. 89--92) (\code{\link{PLACKETTcop}} or \code{\link{PLcop}}) is related to the \emph{Spearman Rho} (\eqn{\rho_S \ne 1}, see \code{\link{rhoCOP}})
\deqn{\rho_S(\Theta) = \frac{\Theta + 1}{\Theta - 1} - \frac{2\Theta\log(\Theta)}{(\Theta - 1)^2}\mbox{.}}
Alternatively, the parameter can be estimated using a \emph{median-split estimator}, which is best shown as an algorithm. First, compute the two medians:
\preformatted{
  medx <- median(x); medy <- median(y)
}
Second and third, compute the number of occurrences where both values are less than their medians and express that as a probability:
\preformatted{
  k <- length(x[x < medx & y < medy]); m <- k / length(x)
}
Finally, the median-split estimator of  \eqn{\Theta} is computed by
\deqn{\Theta = \frac{4m^2}{(1-2m)^2}\mbox{.}}
Nelsen (2006, p. 92) and Salvadori \emph{et al.} (2007, p. 247) provide further details. The input values \code{x} and \code{y} are \emph{not used} for the median-split estimator if \emph{Spearman Rho} (see \code{\link{rhoCOP}}) is provided by \code{rho}.
}
\usage{
PLACKETTpar(x, y, rho=NULL, byrho=FALSE, cor=NULL, ...)
      PLpar(x, y, rho=NULL, byrho=FALSE, cor=NULL, ...)
}
\arguments{
  \item{x}{Vector of values for random variable \eqn{X};}
  \item{y}{Vector of values for random variable \eqn{Y};}
  \item{rho}{Spearman Rho and \code{byrho} is set to \code{TRUE} automatically;}
  \item{byrho}{Should Spearman Rho be used instead of the median-split estimator;}
  \item{cor}{A \pkg{copBasic} syntax for \dQuote{the correlation coefficient} suitable for the copula---a synonym for \code{rho}; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  A value for the Plackett copula \eqn{\Theta} is returned.
}
\note{
Evidently there \dQuote{does not appear to be a closed form for \eqn{\tau(\Theta)}} (Fredricks and Nelsen, 2007, p. 2147),  but given \eqn{\rho(\Theta)}, the equivalent \eqn{\tau(\Theta)} can be computed by either the \code{\link{tauCOP}} function or by approximation. One of the Examples sweeps through \eqn{\rho \mapsto [0,1; \Delta\rho{=}\delta]}, fits the Plackett \eqn{\theta(\rho)}, and then solves for Kendall Tau \eqn{\tau(\theta)} using \code{\link{tauCOP}}. A polynomial is then fit between \eqn{\tau} and \eqn{\rho} to provide rapid conversion between \eqn{|\rho|} and \eqn{\tau}, where the residual standard error is 0.0005705, adjusted R-squared is \eqn{\approx 1}, the maximum residual is \eqn{\epsilon < 0.006}. Because of symmetry, it is only necessary to fit positive association and reflect the result by the sign of \eqn{\rho}. This polynomial is from the \code{Examples} is
\preformatted{
  rho <- 0.920698
  "getPLACKETTtau" <- function(rho) {
     taupoly <-   0.6229945*abs(rho)   +   1.1621854*abs(rho)^2 -
                 10.7424188*abs(rho)^3 +  48.9687845*abs(rho)^4 -
                119.0640264*abs(rho)^5 + 160.0438496*abs(rho)^6 -
                111.8403591*abs(rho)^7 +  31.8054602*abs(rho)^8
     return(sign(rho)*taupoly)
  }
  getPLACKETTtau(rho) # 0.7777726
}
The following code might be useful in some applications for the inversion of the polynomial for the \eqn{\rho} as a function of \eqn{\tau}:
\preformatted{
  "fun" <- function(rho, tau=NULL) {tp <- getPLACKETTtau(rho); return(tau-tp)}
   tau  <- 0.78
   rho  <- uniroot(fun, interval=c(0, 1), tau=tau)$root # 0.9220636
}
}
\references{
Fredricks, G.A, and Nelsen, R.B., 2007, On the relationship between Spearman's rho and Kendall's tau for pairs of continuous random variables: Journal of Statistical Planning and Inference, v. 137, pp. 2143--2150.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.

Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in Nature---An approach using copulas: Springer, 289 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{PLACKETTcop}}, \code{\link{PLcop}}, \code{\link{PLACKETTsim}}, \code{\link{rhoCOP}}
}
\examples{
\dontrun{
Q1 <- rnorm(1000); Q2 <- Q1 + rnorm(1000)
PLpar(Q1, Q2); PLpar(Q1, Q2, byrho=TRUE) # two estimates for same data
PLpar(rho= 0.76) # positive association
PLpar(rho=-0.76) # negative association
tauCOP(cop=PLcop, para=PLpar(rho=-0.15, by.rho=TRUE)) #}

\dontrun{
RHOS <- seq(0, 0.990, by=0.002); TAUS <- rep(NA, length(RHOS))
for(i in 1:length(RHOS)) {
   #message("Spearman Rho: ", RHOS[i])
   theta <- PLACKETTpar(rho=RHOS[i], by.rho=TRUE); tau <- NA
   try(tau <- tauCOP(cop=PLACKETTcop, para=theta), silent=TRUE)
   TAUS[i] <- ifelse(is.null(tau), NA, tau)
}
LM <- lm(TAUS~  RHOS    + I(RHOS^2) + I(RHOS^3) + I(RHOS^4) +
              I(RHOS^5) + I(RHOS^6) + I(RHOS^7) + I(RHOS^8) - 1)
plot(RHOS,TAUS, type="l", xlab="abs(Spearman Rho)", ylab="abs(Kendall Tau)")
lines(RHOS,fitted.values(LM), col=3)#}
}
\keyword{Plackett copula}
\keyword{copula (estimation)}

\encoding{utf8}
\name{LCOMDIA_GH3cop}
\alias{LCOMDIA_GH3cop}
\title{L-comoment Ratio Diagrams for 3-Parameter Gumbel--Hougaard Extreme Value Copula}
\description{
L-comoment ratio diagrams by computation example are shown for the 3-parameter \emph{Gumbel--Hougaard copula} (\code{\link{GHcop}}) with \emph{Marshall--Olkin copula} bounds (\code{\link{MOcop}}). Diagram computations for the 2-parameter version are found under \code{\link{LCOMDIA_GH2cop}}. Diagram computations for the 1-parameter version are found among many copulas in \code{\link{LCOMDIA_ManyCops}}.
}
\author{W.H. Asquith}
\seealso{\code{\link{lcomCOP}}, \code{\link{LCOMDIA_GH2cop}}, \code{\link{LCOMDIA_ManyCops}}
}
\examples{
\dontrun{
  # ------------------------------------------------------------------------------------
  # L-comoments of the GH3. Alert, there is a commented out write_feather below, so for
  # a "production" run, uncomment that. The reasoning for kicking-out output every nsim
  # so that another R session can already be "harvesting" output should the user seek
  # to study the output early. Later additions to these Examples are expected to hone
  # the results for polynomial boundary and polygonal domain of the relations.
  PLOT <- TRUE
  for(f in 1:20) {
    if(PLOT) {
      plot(c(0, 1), c(-0.2, +0.3), type="n", xaxs="i", yaxs="i", las=1,
           xlab="Spearman Rho", ylab="L-coskew (cT3) or L-cokurtosis (cT4)")
      legend("topright", c(    "L-coskew of GHcop(para=c(b1, p2, p3))",
                           "L-cokurtosis of GHcop(para=c(b1, p2, p3))"), bty="n",
            pch=c(3,4), pt.cex=0.5, col=c("seagreen4", "orchid3"))
    }
    fileGH3 <- tempfile(); message("Temp File = ", fileGH3)
    nsim <- 10000
    for(i in seq_len(nsim)) {
      if(length(grep("000$", as.character(i)))) message(i,"-", appendLF=FALSE)
      b1  <- 10^(runif(1, min=-5, max=3))+1
      p2  <- runif(1, min=0, max=1); p3 <- runif(1, min=0, max=1)
      GH3 <- lcomCOP(cop=GHcop, para=c(b1, p2, p3),       stop.on.error=FALSE)
      d3  <- data.frame(b1, p2, p3,
               T2.12=GH3$lcomUV[2], T3.12=GH3$lcomUV[3], T4.12=GH3$lcomUV[4],
               T2.21=GH3$lcomVU[2], T3.21=GH3$lcomVU[3], T4.21=GH3$lcomVU[4])
      row.names(d3) <- NULL; #print(d3)
      if(abs(d3$T2.12) < 1 & d3$T2.12 > 0) {
        if(PLOT) points(d3$T2.12, d3$T3.12, cex=0.5, pch=15, col="seagreen4")
        if(PLOT) points(d3$T2.12, d3$T4.12, cex=0.5, pch=16, col="orchid3"  )
      }
      if(abs(d3$T2.21) < 1 & d3$T2.21 > 0) {
        if(PLOT) points(d3$T2.21, d3$T3.21, cex=0.5, pch=15, col="seagreen4")
        if(PLOT) points(d3$T2.21, d3$T4.21, cex=0.5, pch=16, col="orchid3"  )
      }
      k <- i == 1 # triggering mechanism for write.table()
      write.table(d3, file=fileGH3, sep="\t", append=!k, col.names=k, row.names=FALSE)
    }
    message("done")
    fGH3 <- read.table(fileGH3, sep="\t", header=TRUE)
    file <- paste0("LCM_GH3_", f, ".feather")
    message("writing file '", file, "' to ", getwd())
    # feather::write_feather(fGH3, file)
  }
  # ------------------------------------------------------------------------------------
  files <- list.files(pattern=".feather")
  files <- files[grep("GH3_", files)]
  GH3 <- NULL
  for(file in files) GH3 <- rbind(GH3, as.data.frame(feather::read_feather(file)))
  # ------------------------------------------------------------------------------------
  plot(GH3$T2.12, GH3$T3.12, cex=0.2, pch=16, col=grey(0.3), las=1,
       xlim=c(0, 0.10), ylim=c(-0.07, 0.0))
  abline(0, -0.33) # }
}
\keyword{L-comoments (ratio diagram)}
\keyword{L-comoment ratio diagram}

\encoding{utf8}
\name{rhoCOP}
\alias{rhoCOP}
\title{The Spearman Rho of a Copula}
\description{
Compute the measure of association known as the \emph{Spearman Rho} \eqn{\rho_\mathbf{C}} of a copula according to Nelsen (2006, pp. 167--170, 189, 208) by
\deqn{\rho_\mathbf{C} = 12\int\!\!\int_{\mathcal{I}^2} \mathbf{C}(u,v)\, \mathrm{d}u\mathrm{d}v - 3\mbox{,}}
or
\deqn{\rho_\mathbf{C} = 12\int\!\!\int_{\mathcal{I}^2} \bigl[\mathbf{C}(u,v) - uv\bigr]\, \mathrm{d}u\mathrm{d}v\mbox{,}}
where the later equation is implemented by \code{rhoCOP} as the default method (\code{method="default"}). This equation, here having \eqn{p = 1} and \eqn{k_p(1) = 12}, is generalized under \code{\link{hoefCOP}}. The absence of the \eqn{12} in the above equation makes it equal to the \emph{covariance of a copula} defined by the \emph{Hoeffding Identity} (Joe, 2014, p. 54):
\deqn{\mathrm{cov}(U, V) = \int\!\!\int_{\mathcal{I}^2} \bigl[\mathbf{C}(u,v) - uv\bigr]\, \mathrm{d}u\mathrm{d}v\mbox{ or}}
\deqn{\mathrm{cov}(U, V) = \int\!\!\int_{\mathcal{I}^2} \bigl[\hat{\mathbf{C}}(u,v) - uv\bigr]\, \mathrm{d}u\mathrm{d}v\mbox{, which\ is}}
\deqn{\mathrm{cov}(U, V) = \int\!\!\int_{\mathcal{I}^2} \bigl[u+v-1+\mathbf{C}(1-u,1-v) - uv\bigr]\, \mathrm{d}u\mathrm{d}v\mbox{.}}

Depending on copula family (Joe, 2014, pp. 56 and 267), the alternative formulation for \eqn{\rho_\mathbf{C}} could be used
\deqn{\rho_\mathbf{C} = 3 - 12\int\!\!\int_{\mathcal{I}^2} u \frac{\delta\mathbf{C}(u,v)}{\delta u} \, \mathrm{d}u\mathrm{d}v = 3 - 12\int\!\!\int_{\mathcal{I}^2} v\frac{\delta\mathbf{C}(u,v)}{\delta v} \, \mathrm{d}u\mathrm{d}v\mbox{,}}
where the first integral form corresponds to Joe (2014, eq. 248, p. 56) and is the \code{method="joe21"}, and the second integral form is the \code{method="joe12"}.

The integral
\deqn{\int\!\!\int_{\mathcal{I}^2} \mathbf{C}(u,v)\,\mathrm{d}u\mathrm{d}v\mbox{,}}
represents the \dQuote{volume under the graph of the copula and over the unit square} (Nelsen, 2006, p. 170) and therefore \eqn{\rho_\mathbf{C}} is simple a rescaled volume under the copula. The second equation for \eqn{\rho_\mathbf{C}} expresses the \dQuote{average distance} between the joint distribution and statistical \emph{independence} \eqn{\mathbf{\Pi} = uv}. Nelsen (2006, pp. 175--176) shows that the following relation between \eqn{\rho_\mathbf{C}} and \eqn{\tau_\mathbf{C}} (\code{\link{tauCOP}}) exists
\deqn{-1 \le 3\tau - 2\rho \le 1\mbox{.}}
}
\usage{
rhoCOP(cop=NULL, para=NULL, method=c("default", "joe21", "joe12"),
                            as.sample=FALSE, brute=FALSE, delta=0.002, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{method}{The form of integration used to compute (see above);}
  \item{as.sample}{A logical controlling whether an optional \R \code{data.frame} in \code{para} is used to compute the \eqn{\hat\rho} by dispatch to \code{cor()} function in \R with \code{method = "spearman"};}
  \item{brute}{Should brute force be used instead of two nested \code{integrate()} functions in \R to perform the double integration;}
  \item{delta}{The \eqn{\mathrm{d}u} and \eqn{\mathrm{d}v} for the brute force integration using \code{brute}; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  The value for \eqn{\rho_\mathbf{C}} is returned.
}
\note{
Technically, Nelsen (2006) also shows that these definitions are a form of call to a \emph{concordance function} \eqn{\mathcal{Q}(\mathbf{C}_1,\mathbf{C}_2)} of two copulas that involve \eqn{\mathbf{C}_1{=}\mathbf{C}(u,v)} and \eqn{\mathbf{C}_2{=}\mathbf{\Pi}}. As such in order to keep \code{rhoCOP} a small function when \code{brute=TRUE}, \eqn{\rho_\mathbf{C}} is computed by a special call to \code{\link{tauCOP}}, which by itself and although titled for computation of \emph{Kendall Tau}, does support the concordance function \eqn{\mathcal{Q}(\mathbf{C}_1, \mathbf{C}_2)} [see Nelsen (2006, pp. 158--159)] when given two different copulas and respective parameters as arguments. The well-known \emph{Pearson correlation coefficient} equals Spearman rho value if random variables \eqn{X} and \eqn{Y} are both uniformly distributed on \eqn{[0,1]}.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{blomCOP}}, \code{\link{footCOP}}, \code{\link{giniCOP}},
         \code{\link{hoefCOP}}, \code{\link{tauCOP}},  \code{\link{wolfCOP}},
         \code{\link{joeskewCOP}}, \code{\link{uvlmoms}}
}
\examples{
rhoCOP(cop=PSP)              # 0.4784176

\dontrun{
  n <- 1E4; set.seed(4784)   # quick Monte Carlo integration
  12*mean(replicate(n/10, sum(PSP(runif(n),runif(n)))/n)) - 3 # 0.4779 (for instance) }

\dontrun{
  rhoCOP(cop=PSP, brute=TRUE) # 0.4684063
  # Heavy CPU example showing dual-integration (fast) results in Rho mimicking the
  # sample version
  do_rho <- function(n) {
    uv <- simCOP(n=n, cop=PSP, ploton=FALSE, points=FALSE)
    return(cor(uv$U, uv$V, method="spearman"))
  }
  set.seed(1)
  rhos <- replicate(200, do_rho(1000))
  rho_sample <- mean(rhos); print(rho_sample) # 0.4764914}

\dontrun{
  para <- list(cop1=PLACKETTcop,  cop2=PLACKETTcop,
               para1=0.00395,    para2=4.67,     alpha=0.9392, beta=0.5699)
  rhoCOP(cop=composite2COP, para=para) # -0.5924796

  para <- list(cop1=PLACKETTcop,  cop2=PLACKETTcop,
               para1=0.14147,    para2=20.96,    alpha=0.0411, beta=0.6873)
  rhoCOP(cop=composite2COP, para=para) # 0.2818874

  para <- list(cop1=PLACKETTcop,  cop2=PLACKETTcop,
               para1=0.10137,     para2=4492.87, alpha=0.0063, beta=0.0167)
  rhoCOP(cop=composite2COP, para=para)             # 0.9812919
  rhoCOP(cop=composite2COP, para=para, brute=TRUE) # 0.9752155}

\dontrun{
  # This is the same composited copula used in a highly asymmetric multi-modal
  # plotting example under densityCOPplot(). Let us use that copula as a means to
  # check on the Spearman Rho from alternative formulations by Joe (2014).
  para <- list(alpha=0.15, beta=0.90, kappa=0.06, gamma=0.96,
               cop1=GHcop, cop2=PLACKETTcop, para1=5.5, para2=0.07)
  "rhoCOPbyJoe21" <- function(cop=NULL, para=NULL, ...) { # Joe (2014, eq. 2.48)
     myint <- NULL
     try(myint <- integrate(function(u) {
         sapply(u,function(u) { integrate(function(v) {
         u * derCOP( u, v, cop=cop, para=para, ...)},  0, 1)$value })}, 0, 1))
     ifelse(is.null(myint), return(NA), return(3 - 12*myint$value))
  }
  "rhoCOPbyJoe12" <- function(cop=NULL, para=NULL, ...) { # Not in Joe (2014)
     myint <- NULL
     try(myint <- integrate(function(u) {
         sapply(u,function(u) { integrate(function(v) {
         v * derCOP2( u, v, cop=cop, para=para, ...)}, 0, 1)$value })}, 0, 1))
     ifelse(is.null(myint), return(NA), return(3 - 12*myint$value))
  }
  rhoCOP(       cop=composite2COP, para=para) # 0.1031758
  rhoCOPbyJoe21(cop=composite2COP, para=para) # 0.1031803
  rhoCOPbyJoe12(cop=composite2COP, para=para) # 0.1031532 }
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{copula covariance}
\keyword{Joe (2014) Examples and Exercises}
\keyword{Spearman Rho}
\keyword{Pearson correlation coefficient}

\encoding{utf8}
\name{uvlmoms}
\alias{uvlmoms}
\alias{uvskew}
\title{Bivariate Skewness after Joe (2014) or the Univariate L-moments of Combined U and V}
\description{
Joe (2014, pp. 65--66) suggests two quantile-based measures of \emph{bivariate skewness} defined for uniform random variables \eqn{U} and \eqn{V} combined as either \eqn{\psi_{u+v-1} = u + v - 1} or \eqn{\psi_{u-v} = u - v} for which the \eqn{\mathrm{E}[u] = \mathrm{E}[v] = 0}. The bivariate skewness is the quantity \eqn{\eta}:

\deqn{\eta(p; \psi) = \frac{x(1-p) - 2x(\frac{1}{2}) + x(p)}{x(1-p) - x(p)} \mbox{,}}

where \eqn{0 < p < \frac{1}{2}}, \eqn{x(F)} is the quantile function for nonexceedance probability \eqn{F} for either the quantities \eqn{X = \psi_{u+v-1}} or \eqn{X = \psi_{u-v}} using either the empirical quantile function or a fitted distribution. Joe (2014, p. 66) reports that \eqn{p = 0.05} to \dQuote{achieve some sensitivity to the tails.}  How these might be related (intuitively) to L-coskew (see function \code{lcomoms2()} of the \pkg{lmomco} package) of the L-comoments or bivariate L-moments (\code{\link{bilmoms}}) is unknown, but see the \bold{Examples} section of \code{\link{joeskewCOP}}.

Structurally the above definition for \eqn{\eta} based on quantiles is oft shown in comparative literature concerning L-moments. But why stop there? Why not compute the L-moments themselves to arbitrary order for \eqn{\eta} by either definition (the \code{uvlmoms} variation)? Why not fit a distribution to the computed L-moments for estimation of \eqn{x(F)}? Or simply compute \dQuote{skewness} according to the definition above (the \code{uvskew} variation).
}
\usage{
uvlmoms(u,v=NULL, umv=TRUE, p=NA,   type="gno", getlmoms=TRUE,  ...)

uvskew( u,v=NULL, umv=TRUE, p=0.05, type=6,     getlmoms=FALSE, ...)
}
\arguments{
  \item{u}{Nonexceedance probability \eqn{u} in the \eqn{X} direction;}
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction and if \code{NULL} then \code{u} is treated as a two column \R \code{data.frame};}
  \item{umv}{A logical controlling the computation of \eqn{\psi}: \eqn{\psi = u - v}  (\code{umv = TRUE}) or \eqn{\psi = u + v - 1} (\code{umv = FALSE}). The \code{"m"} is to read \dQuote{minus};}
  \item{p}{A suggested \eqn{p} value is \code{p = 0.05}. If \code{is.na(NA)}, then \code{getlmoms} is set to \code{TRUE} (see below);}
  \item{type}{The \code{type} argument is mutable, and is a syntax match to the canoncial use in package \pkg{lmomco}. Variation from that package however is permitted. Either \code{type} is an integer between 1 and 9 selecting one of the nine quantile algorithms described for the \code{quantile} function in \R. The default 6 uses the \emph{Weibull plotting positions} and differs from the \R default of 7. Otherwise \code{type} must be a valid distribution abbreviation for the \pkg{lmomco} package as in the abbreviation list \code{dist.list} function of that package. The \code{gno} shown as a default for the generalized normal distribution (see distribution type \code{"gno"} in package \pkg{lmomco});}
  \item{getlmoms}{A logical triggering whether the L-moments of either \eqn{\psi_{u+v-1}} or \eqn{\psi_{u - v}} are returned instead computing the above definition of \dQuote{skewness;} and}
  \item{...}{Additional arguments to pass to the \pkg{lmomco} function \code{lmoms}, such as the number of L-moments \code{nmoms}.}
}
\value{
  An \R \code{list} of the univariate L-moments of \eqn{\eta} is returned (see documentation for \code{lmoms} in the \pkg{lmomco} package). Or the skewness of \eqn{\eta} can be either (1) based on the empirical distribution based on plotting positions by the \code{quantile} function in \R using the \code{type} as described, or (2) based on the fitted quantile function for the parameters of a distribution for the \pkg{lmomco} package.
}
\references{
Asquith, W.H., 2011, Distributional analysis with L-moment statistics using the R environment for statistical computing: Createspace Independent Publishing Platform, ISBN 978--146350841--8.

Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{COP}}
}
\examples{
\dontrun{
set.seed(234)
UV <- simCOP(n=100, cop=GHcop, para=1.5, graphics=FALSE)
lmr <- uvlmoms(UV); print(lmr) # L-kurtosis = 0.16568268
uvskew(UV, p=0.10)             # -0.1271723
uvskew(UV, p=0.10, type="gno") # -0.1467011}

\dontrun{
pss <- seq(0.01,0.49, by=0.01)
ETA <- sapply(1:length(pss), function(i) uvskew(UV, p=pss[i], type=5, uvm1=FALSE) )
plot(pss, ETA, type="l", xlab="P FACTOR", ylab="BIVARIATE SKEWNESS") #}
}
\keyword{bivariate asymmetry (measures)}
\keyword{bivariate skewness}
\keyword{L-moments of combined U and V}
\keyword{L-moments}
\concept{Lmoments}

\encoding{utf8}
\name{EMPIRqua.regress2}
\alias{EMPIRqua.regress2}
\title{Quantile Regression of the Grid of the Bivariate Empirical Copula for U with respect to V}
\description{
Generate quantile regression from the gridded inversion of the \emph{bivariate empirical copula} of \eqn{U} with respect to \eqn{V}.
}
\usage{
EMPIRqua.regress2(f=0.5, v=seq(0.01,0.99, by=0.01), empinv=NULL,
                  lowess=FALSE, f.lowess=1/5, ...)
}
\arguments{
  \item{f}{The nonexceedance probability \eqn{F} to perform regression at and defaults to median regression \eqn{F=1/2};}
  \item{v}{A vector of \eqn{v} nonexceedance probabilities;}
  \item{empinv}{The grid from \code{\link{EMPIRgridderinv}};}
  \item{lowess}{Perform \code{lowess} smooth on the quantile regression using the smooth factor of \code{f=f.lowess};}
  \item{f.lowess}{Smooth factor of almost the same argument name fed to the \code{lowess()} function in \R;}
  \item{...}{Additional arguments to pass.}
}
\value{
  The gridded values of the quantile regression of \eqn{U} with respect to \eqn{V}.
}
\author{W.H. Asquith}
\seealso{\code{\link{EMPIRgridderinv2}}, \code{\link{EMPIRqua.regress}}, \code{\link{EMPIRmed.regress}}, \code{\link{EMPIRmed.regress2}}
}
\examples{
# See examples under EMPIRqua.regress
}
\keyword{empirical copula}
\keyword{empirical copula (quantile regression)}

\encoding{utf8}
\name{rhobevCOP}
\alias{rhobevCOP}
\title{A Dependence Measure for a Bivariate Extreme Value Copula based on the Expectation of the Product of Negated Log-Transformed Random Variables U and V}
\description{
Compute a dependence measure based on the expectation of the product of transformed random variables \eqn{U} and \eqn{V}, which unnamed by Joe (2014, pp. 383--384) but symbolically is \eqn{\rho_E}, having a \emph{bivariate extreme value copula} \eqn{\mathbf{C}_{BEV}(u,v)} by
\deqn{\rho_E = \mathrm{E}\bigl[(-\log U) \times (-\log V)\bigr] - 1 = \int_0^1 \bigl[B(w)\bigr]^{-2}\,\mathrm{d}w - 1\mbox{,}}
where \eqn{B(w) = A(w, 1-w)}, \eqn{B(0) = B(1) = 1}, \eqn{B(w) \ge 1/2}, and \eqn{0 \le w \le 1}, and where only bivariate extreme value copulas can be written as
\deqn{\mathbf{C}_{BEV}(u,v) = \mathrm{exp}[-A(-\log u, -\log v)]\mbox{,}}
and thus in terms of the coupla
\deqn{B(w) = -\log\bigl[\mathbf{C}_{BEV}(\mathrm{exp}[-w], \mathrm{exp}[w-1])\bigr]\mbox{.}}

Joe (2014, p. 383) states that \eqn{\rho_E} is the correlation of the \dQuote{survival function of a bivariate min-stable exponential distribution,} which can be assembled as a function of \eqn{B(w)}. Joe (2014, p. 383) also shows the following expression for \emph{Spearman Rho}
\deqn{\rho_S = 12 \int_0^1 \bigl[1 + B(w)\bigr]^{-2}\,\mathrm{d}w - 3\mbox{,}}
in terms of \eqn{B(w)}. This expression, in conjunction with \code{\link{rhoCOP}}, was used to confirm the prior expression shown here for \eqn{B(w)} in terms of \eqn{\mathbf{C}_{BEV}(u,v)}. Lastly, for \emph{independence} (\eqn{uv = \mathbf{\Pi}}; \code{\link{P}}), \eqn{\rho_E = 0} and for the \emph{\enc{Fréchet}{Frechet}--Hoeffding upper-bound copula} (perfect positive association), \eqn{\rho_E = 1}.
}
\usage{
rhobevCOP(cop=NULL, para=NULL, as.sample=FALSE, brute=FALSE, delta=0.002, ...)
}
\arguments{
  \item{cop}{A bivariate extreme value copula function---the function \code{rhobevCOP} makes \bold{no provision} for verifying whether the copula in \code{cop} is actually an \emph{extreme value copula};}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
    \item{as.sample}{A logical controlling whether an optional \R \code{data.frame} in \code{para} is used to compute a \eqn{\hat\rho_E} by \code{mean()} of the product of negated \code{log()}'s in \R. The user is required to cast \code{para} into estimated probabilities (see \bold{Examples});}
  \item{brute}{Should brute force be used instead of two nested \code{integrate()} functions in \R to perform the double integration;}
  \item{delta}{The \eqn{\mathrm{d}w} for the brute force (\code{brute=TRUE}) integration; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  The value for \eqn{\rho_E} is returned.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{rhoCOP}}, \code{\link{tauCOP}}}
\examples{
Theta <- GHcop(tau=1/3)$para     # Gumbel-Hougaard copula with Kendall Tau = 1/3
rhobevCOP(cop=GHcop, para=Theta) # 0.3689268 (RhoE after Joe [2014])
rhoCOP(   cop=GHcop, para=Theta) # 0.4766613 (Spearman Rho)

\dontrun{
set.seed(394)
Theta <- GHcop(tau=1/3)$para     # Gumbel-Hougaard copula with Kendall Tau = 1/3
simUV <- simCOP(n=30000, cop=GHcop, para=Theta, graphics=FALSE) # large simulation
samUV <- simUV * 150; n <- length(samUV[,1]) # convert to fake unit system
samUV[,1] <- rank(simUV[,1]-0.5)/n; samUV[,2] <- rank(simUV[,2]-0.5)/n # hazen
rhobevCOP(para=samUV, as.sample=TRUE) # 0.3708275}
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{copula (properties for extreme value)}

\encoding{utf8}
\name{lcomoms2.ABcop2parameter}
\alias{lcomoms2.ABcop2parameter}
\title{Convert L-comoments to Parameters of Alpha-Beta Compositions of Two One-Parameter Copulas}
\description{
\emph{EXPERIMENTAL}---This function converts the \emph{L-comoments} of a bivariate sample to the four parameters of a composition of two one-parameter copulas. Critical inputs are of course the first three dimensionless L-comoments: \emph{L-correlation}, \emph{L-coskew}, and \emph{L-cokurtosis}. The most complex input is the \code{solutionenvir}, which is an \code{environment} containing arbitrarily long, but individual tables, of L-comoment and parameter pairings. These pairings could be computed from the examples in \code{\link{simcompositeCOP}}.

The individual tables are prescanned for potentially acceptable solutions and the absolute additive error of both L-comoments for a given order is controlled by the \code{tNeps} arguments. The default values seem acceptable. The purpose of the prescanning is to reduce the computation space from perhaps millions of solutions to a few orders of magnitude. The computation of the solution error can be further controlled by \eqn{X} or \eqn{u} with respect to \eqn{Y} or \eqn{v} using the \code{comptNerrXY} arguments, but experiments thus far indicate that the defaults are likely the most desired. A solution \dQuote{matching} the L-correlation is always sought; thus there is no \code{uset2err} argument. The arguments \code{uset3err} and \code{uset4err} provide some level of granular control on addition error minimization; the defaults seek to \dQuote{match} L-coskew and ignore L-cokurtosis. The \code{setreturn} controls which rank of computed solution is returned; users might want to manually inspect a few of the most favorable solutions, which can be done by the \code{setreturn} or inspection of the returned object from the \code{lcomoms2.cop2parameter} function. The examples are detailed and self-contained to the \pkg{copBasic} package; curious users are asked to test these.
}
\usage{
lcomoms2.ABcop2parameter(solutionenvir=NULL,
                         T2.12=NULL, T2.21=NULL,
                         T3.12=NULL, T3.21=NULL,
                         T4.12=NULL, T4.21=NULL,
                         t2eps=0.1, t3eps=0.1, t4eps=0.1,
                         compt2erruv=TRUE, compt2errvu=TRUE,
                         compt3erruv=TRUE, compt3errvu=TRUE,
                         compt4erruv=TRUE, compt4errvu=TRUE,
                         uset3err=TRUE, uset4err=FALSE,
                         setreturn=1, maxtokeep=1e5)
}
\arguments{
  \item{solutionenvir}{The environment containing solutions;}
  \item{T2.12}{L-correlation \eqn{\tau_2^{[12]}};}
  \item{T2.21}{L-correlation \eqn{\tau_2^{[21]}};}
  \item{T3.12}{L-coskew \eqn{\tau_3^{[12]}};}
  \item{T3.21}{L-coskew \eqn{\tau_3^{[21]}};}
  \item{T4.12}{L-cokurtosis \eqn{\tau_4^{[12]}};}
  \item{T4.21}{L-cokurtosis \eqn{\tau_4^{[21]}};}
  \item{t2eps}{An error term in which to pick a potential solution as close enough on preliminary processing for \eqn{\tau_2^{[1 \leftrightarrow 2]}};}
  \item{t3eps}{An error term in which to pick a potential solution as close enough on preliminary processing for \eqn{\tau_3^{[1 \leftrightarrow 2]}};}
  \item{t4eps}{An error term in which to pick a potential solution as close enough on preliminary processing for \eqn{\tau_4^{[1 \leftrightarrow 2]}};}
  \item{compt2erruv}{Compute an L-correlation error using the 1 with respect to 2 (or \eqn{u} wrt \eqn{v});}
  \item{compt2errvu}{Compute an L-correlation error using the 2 with respect to 1 (or \eqn{v} wrt \eqn{u});}
  \item{compt3erruv}{Compute an L-coskew error using the 1 with respect to 2 (or \eqn{u} wrt \eqn{v});}
  \item{compt3errvu}{Compute an L-coskew error using the 2 with respect to 1 (or \eqn{v} wrt \eqn{u});}
  \item{compt4erruv}{Compute an L-cokurtosis error using the 1 with respect to 2 (or \eqn{u} wrt \eqn{v});}
  \item{compt4errvu}{Compute an L-cokurtosis error using the 2 with respect to 1 (or \eqn{v} wrt \eqn{u});}
  \item{uset3err}{Use the L-coskew error in the determination of the solution. The L-correlation error is always used;}
  \item{uset4err}{Use the L-cokurtosis error in the determination of the solution. The L-correlation error is always used;}
  \item{setreturn}{Set (index) number of the solution to return. The default of 1 returns the preferred solutions based on the controls for the minimization; and}
  \item{maxtokeep}{The value presets the number of rows in the solution matrix. This matrix is filled with potential solutions as the various subfiles of the \code{solutionenvir} are scanned. The matrix is trimmed of \code{NA}s and error trapping is in place for too small values of \code{maxtokeep}. The default value appears appropriate for the feeding of massively large simulated parameter spaces.}
}
\value{
  An \R \code{data.frame} is returned.
}
\references{
Asquith, W.H., 2011, Distributional analysis with L-moment statistics using the R environment for statistical computing: Createspace Independent Publishing Platform, ISBN 978--146350841--8.

Salvadori, G., De Michele, C., Kottegoda, N.T., and Rosso, R., 2007, Extremes in Nature---An approach using copulas: Springer, 289 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{simCOP}}, \code{\link{simcompositeCOP}}, \code{\link{composite2COP}}}
\examples{
\dontrun{
# Build an initial parameter to L-comoment mapping table.
  mainpara <- list(cop1=PLACKETTcop, cop2=PLACKETTcop,
                   para1gen=function() { return(10^runif(1, min=-5, max=0)) },
                   para2gen=function() { return(10^runif(1, min=0,  max=5)) })
  nsim <- 1E4
  sample.size.for.estimation <- 1000 # really use vastly larger sample size
  PlackettPlackettNP <-
      simcompositeCOP(n=sample.size.for.estimation, nsim=nsim, parent=mainpara)
  save(PlackettPlackettNP, file="PlackettPlackettNP.RData", compress="xz")

# Plackett-Plackett composited copula from the copBasic package
# Then create an environment to house the "table."
PlackettPlackett <- new.env()
assign("NeedToCreateForDemo", PlackettPlackettNP, envir=PlackettPlackett)
# Now that the table is assigned into the environment, the parameter
# estimation function can be used. In reality, a much much larger
# solution set is needed, but this effort is experimental.

# Now grab the closest Plackett-Plackett solution having the following six
# arbitrary L-comoments. Then simulate 1000 values and plot them to show
# the underlying bivariate distribution.
PPcop <- lcomoms2.ABcop2parameter(solutionenvir=PlackettPlackett,
                                  T2.12=-0.5059, T2.21=-0.5110,
                                  T3.12= 0.1500, T3.21= 0.1700,
                                  T4.12=-0.0500, T4.21= 0.0329,
                                  uset3err=TRUE, uset4err=TRUE)
# A user in encouraged to inspect the contents of PPcop to "assess" the
# solution by a method of L-comoments, we will now proceed with showing the
# copula via a simulation of the fitted version.
para <- list(cop1=PLACKETTcop, cop2=PLACKETTcop, alpha=PPcop$alpha, beta=PPcop$beta,
             para1=PPcop$Cop1Thetas, para2=PPcop$Cop2Thetas)

D <- simCOP(n=5000, cop=composite2COP, para=para, col=rgb(0,0,0,0.1), pch=16)
# The sample L-comoments of the fitted Plackett-Plackett may be found by
lmomco::lcomoms2(D, nmom=4) # from the lmomco package, and six sample values shown
T2.12 <- -0.5151547; T2.21 <- -0.5139863
T3.12 <-  0.1502336; T3.21 <-  0.1721355
T4.12 <- -0.0326277; T4.21 <-  0.0233979
PPcop <- lcomoms2.ABcop2parameter(solutionenvir=PlackettPlackett,
                                  T2.12=T2.12, T2.21=T2.21,
                                  T3.12=T3.12, T3.21=T3.21,
                                  T4.12=T4.12, T4.21=T4.21, uset4err=TRUE)
para <- list(cop1=PLACKETTcop, cop2=PLACKETTcop, alpha=PPcop$alpha, beta=PPcop$beta,
             para1=PPcop$Cop1Thetas, para2=PPcop$Cop2Thetas)
D <- simCOP(n=5000, cop=composite2COP, para=para, col=rgb(0,0,0,0.1), pch=16)
level.curvesCOP(cop=composite2COP, para=para, delt=.1, ploton=FALSE)
qua.regressCOP.draw(cop=composite2COP, para=para,
                    ploton=FALSE, f=seq(0.05, 0.95, by=0.05))
qua.regressCOP.draw(cop=composite2COP, para=para, wrtV=TRUE,
                    ploton=FALSE, f=seq(0.05, 0.95, by=0.05), col=c(3,2))
diag <- diagCOP(cop=composite2COP, para=para, ploton=FALSE, lwd=4)

image(gridCOP(cop=composite2COP, para=para), col=terrain.colors(20))
# One can inspect alternative solutions like this
# S <- PPcop$solutions$solutions[,1:16]
# B <- S[abs(S$t2.12res) < 0.02 & abs(S$t2.21res) < 0.02 &
#        abs(S$t3.12res) < 0.02 & abs(S$t3.21res) < 0.02, ]
#print(B)}
}
\keyword{L-comoments}
\keyword{copula composition}

\encoding{utf8}
\name{kfuncCOPlmoms}
\alias{kfuncCOPlmoms}
\alias{kfuncCOPlmom}
\title{The L-moments of the Kendall Function of a Copula}
\description{
Compute the L-moments of the \emph{Kendall Function} (\eqn{F_K(z; \mathbf{C})}) of a copula \eqn{\mathbf{C}(u,v)} where the \eqn{z} is the joint probability of the \eqn{\mathbf{C}(u,v)}. The Kendall Function (or \emph{Kendall Distribution Function}) is the cumulative distribution function (CDF) of the joint probability \eqn{Z} of the coupla. The expected value of the \eqn{z(F_K)} (mean, first L-moment \eqn{\lambda_1}), because \eqn{Z} has nonzero probability for \eqn{0 \le Z \le \infty}, is

\deqn{\mathrm{E}[Z] = \lambda_1 = \int_0^\infty \bigl[1 - F_K(t)\bigr]\,\mathrm{d}t = \int_0^1 \bigl[1 - F_K(t)\bigr] \,\mathrm{d}t\mbox{,}}

where for circumstances here \eqn{0 \le Z \le 1}. The \eqn{\infty} is mentioned only because expectations of such CDFs are usually shown using \eqn{(0,\infty)} limits, whereas integration of quantile functions (CDF inverses) use limits \eqn{(0, 1)}. Because the support of \eqn{Z} is \eqn{(0, 1)}, like the probability \eqn{F_K}, showing just it (\eqn{\infty}) as the upper limit could be confusing---statements such as \dQuote{probabilities of probabilities} are rhetorically complex. So, pursuit of word precision is made herein.

An expression for \eqn{\lambda_r} for \eqn{r \ge 2} in terms of the \eqn{F_K(z)} is
\deqn{
\lambda_r = \frac{1}{r}\sum_{j=0}^{r-2} (-1)^j {r-2 \choose j}{r \choose j+1} \int_{0}^{1} \! \bigl[F_K(t)\bigr]^{r-j-1}\times \bigl[1 - F_K(t)\bigr]^{j+1}\, \mathrm{d}t\mbox{,}
}
where because of these circumstances the limits of integration are \eqn{(0, 1)} and not \eqn{(-\infty, \infty)} as in the usual definition of L-moments in terms of a distribution's CDF. (Note, such expressions did not make it into Asquith (2011), which needs rectification if that monograph ever makes it to a 2nd edition.)

The mean, L-scale, coefficient of L-variation (\eqn{\tau_2}, LCV, L-scale/mean), L-skew (\eqn{\tau_3}, TAU3), L-kurtosis (\eqn{\tau_4}, TAU4), and \eqn{\tau_5} (TAU5) are computed. In usual nomenclature, the L-moments are
\eqn{ \lambda_1 = \mbox{mean,}}
\eqn{ \lambda_2 = \mbox{L-scale,}}
\eqn{ \lambda_3 = \mbox{third L-moment,}}
\eqn{ \lambda_4 = \mbox{fourth L-moment, and}}
\eqn{ \lambda_5 = \mbox{fifth L-moment,}}
whereas the L-moment ratios are
\eqn{ \tau_2 = \lambda_2/\lambda_1 = \mbox{coefficient of L-variation, }}
\eqn{ \tau_3 = \lambda_3/\lambda_2 = \mbox{L-skew, }}
\eqn{ \tau_4 = \lambda_4/\lambda_2 = \mbox{L-kurtosis, and}}
\eqn{ \tau_5 = \lambda_5/\lambda_2 = \mbox{not named.}}
It is common amongst practitioners to lump the L-moment ratios into the general term \dQuote{L-moments} and remain inclusive of the L-moment ratios. For example, L-skew then is referred to as the 3rd L-moment when it technically is the 3rd L-moment ratio. There is no first L-moment ratio (meaningless); so, results from \code{kfuncCOPlmoms} function will canoncially show a \code{NA} in that slot. The coefficient of L-variation is \eqn{\tau_2} (subscript 2) and not \emph{Kendall Tau} (\eqn{\tau}). Sample L-moments are readily computed by several packages in \R (\emph{e.g.} \pkg{lmomco}, \pkg{lmom}, \pkg{Lmoments}, \pkg{POT}).
}
\usage{
kfuncCOPlmom(r, cop=NULL, para=NULL, ...)

kfuncCOPlmoms(cop=NULL, para=NULL, nmom=5, begin.mom=1, ...)
}
\arguments{
  \item{r}{The \eqn{r}th order of a single L-moment to compute;}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{nmom}{The number of L-moments to compute;}
  \item{begin.mom}{The \eqn{r}th order to begin the sequence \code{lambegr:nmom} for L-moment computation. The rarely used argument is means to bypass the computation of the mean if the user has an alternative method for the mean or other central tendency characterization in which case \code{begin.mom = 2}; and}
  \item{...}{Additional arguments to pass.}
}
\value{
  An \R \code{list} is returned by \code{kfuncCOPlmoms} and only the scalar value of \eqn{\lambda_r} by \code{kfuncCOPlmom}.

  \item{lambdas}{Vector of the L-moments. First element is \eqn{\lambda_1}, second element is \eqn{\lambda_2}, and so on;}
  \item{ratios}{Vector of the L-moment ratios. Second element is \eqn{\tau}, third element is \eqn{\tau_3} and so on; and}
  \item{source}{An attribute identifying the computational source of the L-moments: \dQuote{kfuncCOPlmoms}.}
}
\note{
The L-moments of Kendall Functions appear to be not yet fully researched. An interesting research direction would be the trajectories of the L-moments or \emph{L-moment ratio diagrams} for the Kendall Function and the degree to which distinction between copulas becomes evident---such diagrams are in wide-spread use for distinguishing between univariate distributions. It is noted, however, that \emph{Kendall Function L-moment ratio diagrams} might be of less utility that in the univariate world---recalling that a univariate distribution is uniquely characterized by its L-moments---because different copulas can have the same \eqn{F_K(z)}, such as all bivariate extreme value copulas (see also \bold{Examples}).
\preformatted{
  Rhos <- c(0.001, 0.01, seq(0.05, 0.95, by=0.05), 0.99, 0.999)
  L1 <- T2 <- T3 <- T4 <- Thetas <- vector(mode="numeric", length(Rhos))
  for(i in 1:length(Thetas)) {
     Thetas[i] <- uniroot(function(p)
                 Rhos[i] - rhoCOP(cop=PARETOcop, para=p), c(0,200))$root
     message("Rho = ", Rhos[i], " and Pareto theta = ",
                                             round(Thetas[i], digits=4))
     lmr <- kfuncCOPlmoms(cop=PARETOcop, para=Thetas[i], nmom=4)
     L1[i] <- lmr$lambdas[1]; T2[i] <- lmr$ratios[2]
     T3[i] <- lmr$ratios[3];  T4[i] <- lmr$ratios[4]
  }
  LMR <- data.frame(Rho=Rhos, Theta=Thetas, L1=L1, T2=T2, T3=T3, T4=T4)
  plot(LMR$Rho, LMR$T2, ylim=c(-0.04, 0.5), xlim=c(0, 1),
       xlab="Spearman Rho or coefficient of L-variation",
       ylab="L-moment ratio", type="l", col="black")
  lines(LMR$Rho, LMR$T3, lty=1, col="red"         )
  lines(LMR$Rho, LMR$T4, lty=1, col="green"       )
  lines(LMR$T2,  LMR$T3, lty=2, col="blue"        )
  lines(LMR$T2,  LMR$T4, lty=2, col="deepskyblue2")
  lines(LMR$T3,  LMR$T4, lty=2, col="purple"      )
}
}
\references{
Asquith, W.H., 2011, Distributional analysis with L-moment statistics using the R environment for statistical computing: Createspace Independent Publishing Platform, ISBN 978--146350841--8.
}
\author{W.H. Asquith}
\seealso{\code{\link{kfuncCOP}}}
\examples{
\dontrun{
kfuncCOPlmom(1, cop=P) # 0.5 * 0.5 = 0.25 is expected joint prob. of independence
#[1] 0.2499999  (in agreement with theory)

ThetaGH <- 4.21
Rho <- rhoCOP(cop=GHcop, para=ThetaGH)
ThetaHR <- uniroot(function(p) Rho - rhoCOP(cop=HRcop, para=p), c(0, 100))$root
ThetaHR <- uniroot(function(p) Rho - rhoCOP(cop=HRcop, para=p), c(0, 100))$root
ThetaGL <- uniroot(function(p) Rho - rhoCOP(cop=GLcop, para=p), c(0, 100))$root
ls.str(kfuncCOPlmoms(cop=GHcop, para=ThetaGH)) # Gumbel-Hougaard copula
# lambdas :  num [1:5] 0.440617 0.169085 0.011228 -0.000797 0.000249
# ratios  :  num [1:5]       NA 0.383750 0.066400 -0.004720 0.001470
#                               L-skew = 0.066400
ls.str(kfuncCOPlmoms(cop=HRcop, para=ThetaHR)) # Husler-Reiss copula
# lambdas :  num [1:5] 0.439627 0.169052 0.011427 -0.000785 0.000249
# ratios  :  num [1:5]       NA 0.384540 0.067590 -0.004640 0.001470
#                               L-skew = 0.067590
ls.str(kfuncCOPlmoms(cop=GLcop, para=ThetaGL)) # Galambos copula
# lambdas :  num [1:5] 0.440415 0.169079 0.011268 -0.000795 0.000248
# ratios  :  num [1:5]       NA 0.383910 0.066650 -0.004700 0.001470
#                               L-skew = 0.066650
# These L-moments are extremely similar and within the numerics used.
# Extreme value copula all have the same Kendall Distribution function.}

\dontrun{
UV <- simCOP(200, cop=PLcop, para=1/pi, graphics=FALSE)
theta <- PLpar(UV[,1], UV[,2])
zs <- c(0.001, seq(0.01, 0.99, by=0.01), 0.999) # for later

# Take the sample estimated parameter and convert to joint probabilities Z
# Convert the Z to the Kendall Function estimates again with the sample parameter
Z  <- PLcop(UV[,1], UV[,2], para=theta); KF <- kfuncCOP(Z, cop=PLcop, para=theta)

# Compute L-moments of the "Kendall function" and the sample versions
# and again see that the L-moment are for the distribution of the Z!
KNFlmr <- kfuncCOPlmoms(cop=PLcop, para=theta); SAMlmr <- lmomco::lmoms(Z)
knftxt <- paste0("Kendall L-moments: ",
                 paste(round(KNFlmr$lambdas, digits=4), collapse=", "))
samtxt <- paste0("Sample L-moments: " ,
                 paste(round(SAMlmr$lambdas, digits=4), collapse=", "))

plot(Z, KF, xlim=c(0,1), ylim=c(0,1), col="black",
     xlab="COPULA(u,v) VALUE [JOINT PROBABILITY]",
     ylab="KENDALL DISTRIBUTION FUNCTION (KDF), AS NONEXCEEDANCE PROBABILITY")
rug(Z, side=1, col="red", lwd=0.5); rug(KF, side=2, col="red", lwd=0.5) # rug plots
lines(zs, kfuncCOP(zs, cop=PLcop, para=1/pi), col="darkgreen")
knf_meanZ <- KNFlmr$lambdas[1]; sam_meanZ <- SAMlmr$lambdas[1]
knf_mean  <- kfuncCOP(knf_meanZ, cop=PLcop, para=theta) # theo. Kendall function
sam_mean  <- kfuncCOP(sam_meanZ, cop=PLcop, para=theta) # sam. est. of Kendall func
points(knf_meanZ, knf_mean, pch=16, col="blue", cex=3)
points(sam_meanZ, sam_mean, pch=16, col="cyan", cex=2)
lines(zs, zs-zs*log(zs), lty=2, lwd=0.8) # dash ref line for independence
text(0.2, 0.30, knftxt, pos=4, cex=1); text(0.2, 0.25, samtxt, pos=4, cex=1)
text(0.2, 0.18, paste0("Notice uniform distribution of vertical axis rug.\n",
                       "A Critical remark with respect to to KDFs."), cex=1, pos=4)
legend("bottomright", c("Independence copula", "KDF of Plackett copula",
                        "Theoretical mean", "Sample mean"), bty="n", y.intersp=1.5,
       lwd=c(1, 1, NA, NA), lty=c(2, 1, NA, NA), pch=c(NA, NA, 16, 16),
       col=c("black", "darkgreen", "blue", "cyan"), pt.cex=c(NA, NA, 3, 2)) #}
}
\keyword{copula (characteristics)}
\keyword{Kendall Function (L-moments)}
\concept{Kendall Function Lmoments}
\concept{Kendall Function L-moments}
\keyword{L-moments}
\concept{Lmoments}
\concept{Kendall Function L-moment ratio diagram}
\concept{Kendall Function L-moment diagram}
\concept{Kendall Function Lmoment ratio diagram}
\concept{Kendall Function Lmoment diagram}
\keyword{L-moments by CDF}
\concept{cumulative distribution}

\encoding{utf8}
\name{EuvCOP}
\alias{EuvCOP}
\title{Expected value of U given V}
\description{
Compute the \emph{expected value} of \eqn{U} given a \eqn{V} (the \eqn{Y} direction) through the \emph{conditional distribution function} \eqn{G(Y)} using the appropriate \emph{partial derivative} of a copula (\eqn{\mathbf{C}(u,v)}) with respect to \eqn{V}. The inversion of the partial derivative is the \emph{conditional quantile function}. Basic principles provide the expectation for a \eqn{y \ge 0} is

\deqn{E[Y] = \int_0^\infty yf(y)\mathrm{d}y = \int_0^\infty \bigl(1-G_y(Y)\bigr)\mathrm{d}y\mbox{,}}

which for the setting here becomes

\deqn{E[U \mid V = v] = \int_0^1 \bigl(1 - \frac{\delta}{\delta v} \mathbf{C}(u,v)\bigr)\mathrm{d}u\mbox{.}}

This function solves the integral using the \code{\link{derCOP2}} function. This avoids a call of the \code{\link{derCOPinv2}} through its \code{uniroot()} inversion of the derivative. The example shown for \code{EuvCOP()} below does a validation check using conditional simulation, which is dependence (of course) of the design of the \pkg{copBasic} package, as part of simple isolation of a horizontal slice of the simulation and computing the mean of the \eqn{V} within the slice.
}
\usage{
EuvCOP(v=seq(0.01, 0.99, by=0.01), cop=NULL, para=NULL, asuv=FALSE, nsim=1E5,
   subdivisions=200L, rel.tol=.Machine$double.eps^0.25, abs.tol=rel.tol, ...)
}
\arguments{
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{cop}{A copula function with vectorization as in \code{asCOP};}
  \item{para}{Vector of parameters or other data structures, if needed, to pass to the copula;}
  \item{asuv}{Return a data frame of the \eqn{U} and \eqn{V};}
  \item{nsim}{Number of simulations for \emph{Monte Carlo integration} if numerical integration were to fail (refer to \bold{Note});}
  \item{subdivisions}{Argument of same name passed to \code{integrate()};}
  \item{rel.tol}{Argument of same name passed to \code{integrate()};}
  \item{abs.tol}{Argument of same name passed to \code{integrate()}; and}
  \item{...}{Additional arguments to pass to \code{\link{derCOP2}}.}
}
\value{
  Value(s) for the expectation are returned.
}
\note{
The author is well aware that the name of this function does not contain the number \code{2} as the family of functions also sharing this \emph{with respect to \eqn{v}} nature. It was a design mistake in 2008 to have used the \code{2}. The \code{uv} in the function name is the moniker for this \emph{with respect to \eqn{v}}.

There can be the rare examples of the numerical integration failing. In such circumstances, Monte Carlo integration is performed and the returned vector becomes a named vector with the \code{sim} identifying values stemming from the simulation.
\preformatted{
  para <- list(cop=RFcop, para=0.9)
  para <- list(cop=COP, para=para, reflect=1, alpha=0, beta=0.3)
  EuvCOP(c(0.0001, 0.0002, 0.001, 0.01, 0.1), cop=composite1COP, para=para)
  #            sim
  #[1] 0.001319395 0.002238238 0.006905300 0.034608078 0.173451788
}
}
\author{W.H. Asquith}
\seealso{\code{\link{EvuCOP}}, \code{\link{derCOP2}}}
\examples{
# We can show algorithmic validation using a highly asymmetric case of a
# copula having its parameter also nearly generating a singular component.
v <- c(0.2, 0.8); n <- 5E2; set.seed(1)
para <- list(cop=HRcop, para=120, alpha=0.4, beta=0.05)
UV   <- simCOP(n, cop=composite1COP, para=para, graphics=FALSE) # set TRUE to view

sapply(v, function(vv) EuvCOP(vv, cop=composite1COP, para=para))
# [1] 0.3051985 0.7059999

sapply(v, function(vv) mean( UV$U[UV$V > vv - 50/n & UV$V < vv + 50/n] ) )
# [1] 0.2796518 0.7092755 # general validation is thus shown as n-->large

# If visualized, we see in the lower corner than heuristics suggest a mean further
# to the right of the "singularity" for v=0.2 than v=0.80. For v=0.80, the
# "singularity" appears tighter given the upper tail dependency contrast of the
# coupla in the symmetrical case (alpha=0, beta=0) and changing the parameter to
# a Spearman Rho (say) similar to the para setting in this example. So, 0.70 for
# the mean given v=0.80 makes sense. Further notice that the two estimates of the
# mean are further apparent for v=0.2 relative to v=0.80. Again, this makes sense
# when the copula is visualized even at small n let alone large n.

# See additional Examples under EvuCOP().

\dontrun{
  set.seed(1)
  n <- 5000; Vlo <- rep(0.001, n); Vhi <- rep(0.95, n); Theta <- 3
  Ulo <- simCOPmicro(Vlo, cop=JOcopB5, para=Theta); dlo <- density(Ulo)
  Uhi <- simCOPmicro(Vhi, cop=JOcopB5, para=Theta); dhi <- density(Uhi)
  dlo$x[dlo$x < 0] <- 0; dhi$x[dhi$x < 0] <- 0
  dlo$x[dlo$x > 1] <- 1; dhi$x[dhi$x > 1] <- 1

  summary(Ulo)
  Ulomu <- EuvCOP(Vlo[1], cop=JOcopB5, para=Theta); print(Ulomu)
  #      Min.   1st Qu.    Median      Mean   3rd Qu.      Max.
  # 0.0000669 0.0887330 0.2006123 0.2504796 0.3802847 0.9589315
  #                  Ulomu -----> 0.2502145
  summary(Uhi)
  Uhimu <- EuvCOP(Vhi[1], cop=JOcopB5, para=Theta); print(Uhimu)
  #      Min.   1st Qu.    Median      Mean   3rd Qu.      Max.
  #   0.01399  0.90603    0.93919 0.9157600   0.95946   0.99411
  #                  Uhimu -----> 0.9154093

  UV <- simCOP(n, cop=JOcopB5, para=Theta,
                  cex=0.6, pch=21, bg="palegreen", col="darkgreen")
  abline(h=Vlo[1], col="salmon", lty=3) # near the bottom to form datum for density
  abline(h=Vhi[1], col="purple", lty=3) # near the   top  to form datum for density
  lines(dlo$x,   dlo$y/max(dlo$y)/2 +    Vlo[1],  col="salmon", lwd=2)
  # re-scaled density along the line already drawn near the bottom (Vlo)
  # think rug plotting to bottom the values plotting very close to the line
  lines(dhi$x, 1-dhi$y/max(dhi$y)/2 - (1-Vhi[1]), col="purple", lwd=2)
  # re-scaled  density along the line already drawn near the  top  (Vhi)
  # think rug plotting to bottom the values plotting very close to the line
  uv <- seq(0.001, 0.999, by=0.001) # for trajectory of E[U | V=v]
  lines(EuvCOP(uv, cop=JOcopB5, para=Theta), uv, col="blue", lwd=3.5, lty=2)
  points(Ulomu, Vlo[1], pch=16, col="salmon", cex=2)
  points(Uhimu, Vhi[1], pch=16, col="purple", cex=2) #}
}
\keyword{copula (utility)}
\keyword{copula operator}
\keyword{copula (conditional distribution)}
\keyword{conditional expectation}
\keyword{mean U given V}
\concept{expectation}
\keyword{Theory Examples (reflection/rotation)}

\encoding{utf8}
\name{kfuncCOPinv}
\alias{kfuncCOPinv}
\title{The Inverse Kendall Function of a Copula}
\description{
Compute the (numerical) inverse \eqn{F^{(-1)}_K(z) \equiv z(F_K)} of the \emph{Kendall Function} \eqn{F_K(z; \mathbf{C})} (\code{\link{kfuncCOP}}) of a copula \eqn{\mathbf{C}(u,v)} given nonexceedance probability \eqn{F_K}. The \eqn{z} is the joint probability of the random variables \eqn{U} and \eqn{V} coupled to each other through the copula \eqn{\mathbf{C}(u,v)} and the nonexceedance probability of the probability \eqn{z} is \eqn{F_K}---statements such as \dQuote{probabilities of probabilities} are rhetorically complex so pursuit of word precision is made herein.
}
\usage{
kfuncCOPinv(f, cop=NULL, para=NULL, subdivisions=100L,
               rel.tol=.Machine$double.eps^0.25, abs.tol=rel.tol, ...)
}
\arguments{
  \item{f}{Nonexceedance probability \eqn{(0 \le F_K \le 1)};}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{subdivisions}{Argument of same name passed to \code{integrate()} through \code{\link{kfuncCOP}},}
  \item{rel.tol}{Argument of same name passed to \code{integrate()} through \code{\link{kfuncCOP}},}
  \item{abs.tol}{Argument of same name passed to \code{integrate()} through \code{\link{kfuncCOP}}, and}
  \item{...}{Additional arguments to pass.}
}
\value{
  The value(s) for \eqn{z(F_K)} are returned.
}
\note{
The L-moments of Kendall Functions appear to be unresearched. Therefore, the \code{\link{kfuncCOPlmom}} and \code{\link{kfuncCOPlmoms}} functions were written. These compute L-moments on the CDF \eqn{F_K(z)} and not the quantile function \eqn{z(F_K)} and thus are much faster than trying to use \code{\link{kfuncCOPinv}} in the more common definitions of L-moments. A demonstration of the mean (first L-moment) of the Kendall Function numerical computation follows:
\preformatted{
  # First approach
  "afunc" <- function(f) kfuncCOPinv(f, cop=GHcop, para=pi)
  integrate(afunc, 0, 1) # 0.4204238 with absolute error < 2.5e-05
  # Second approach
  kfuncCOPlmom(1, cop=GHcop, para=pi)  # 0.4204222
}
where the first approach uses \eqn{z(F_K)}, whereas the second method uses integration for the mean on \eqn{F_K(z)}.
}
\references{
Asquith, W.H., 2011, Distributional analysis with L-moment statistics using the R environment for statistical computing: Createspace Independent Publishing Platform, ISBN 978--146350841--8.

Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{kfuncCOP}}}
\examples{
\dontrun{
Z <- c(0,0.25,0.50,0.75,1) # Joint probabilities of a N4212cop
kfuncCOPinv(kfuncCOP(Z, cop=N4212cop, para=4.3), cop=N4212cop, para=4.3)
# [1] 0.0000000 0.2499984 0.5000224 0.7500112 1.0000000}
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{Kendall inverse distribution function}
\keyword{Kendall quantile function}

\encoding{utf8}
\name{joint.curvesCOP2}
\alias{joint.curvesCOP2}
\title{Compute Coordinates of the Marginal Probabilities given joint AND or OR Probability}
\description{
Compute the coordinates of the bivariate marginal probabilities for variables \eqn{U} and \eqn{V} given selected probabilities levels \eqn{t} for a copula \eqn{\mathbf{C}(u,v)} for \eqn{u} with respect to \eqn{v}. For the case of a \bold{joint and} probability, symbolically the solution is
\deqn{\mathrm{Pr}[U \le v,\ V \le v] = t = \mathbf{C}(u,v)\mbox{,}}
where \eqn{V \mapsto [t_i, t_{j}, t_{j+1}, \cdots, 1; \Delta]} (an irregular sequence of \eqn{v} values from the \eqn{i}th value of \eqn{t_i} provided through to unity) and thus
\deqn{t_i \mapsto \mathbf{C}(u, v=V)\mbox{,}}
and solving for the sequence of \eqn{u}. The index \eqn{j} is to indicate that a separate loop is involved and is distinct from \eqn{i}. The pairings \eqn{\{u(t_i), v(t_i)\}} for each \eqn{t} are packaged as an \R \code{data.frame}. This operation is very similar to the plotting capabilities in \code{\link{level.curvesCOP2}} for \emph{level curves} (Nelsen, 2006, pp. 12--13) but implemented in the function \code{joint.curvesCOP2} for alternative utility.

For the case of a \bold{joint or} probability, the \emph{dual of a copula (function)} or \eqn{\tilde{\mathbf{C}}(u,v)} from a copula (Nelsen, 2006, pp. 33--34) is used and symbolically the solution is:
\deqn{\mathrm{Pr}[U \le v \mathrm{\ or\ } V \le v] = t = \tilde{\mathbf{C}}(u,v) = u + v - \mathbf{C}(u,v)\mbox{,}}
where \eqn{V \mapsto [0, v_j, v_{j+1}, \cdots, t_i; \Delta]} (an irregular sequence of \eqn{v} values from zero through to the \eqn{i}th value of \eqn{t}) and thus
\deqn{t_i \mapsto \tilde{\mathbf{C}}(u, v=V)\mbox{,}}
and solving for the sequence of \eqn{u}.  The index \eqn{j} is to indicate that a separate loop is involved and is distinct from \eqn{i}. The pairings \eqn{\{u(t_i), v(t_i)\}} for each \eqn{t} are packaged as an \R \code{data.frame}.
}
\usage{
joint.curvesCOP2(cop=NULL, para=NULL, type=c("and", "or"),
                 probs=c(0.5, 0.8, 0.90, 0.96, 0.98, 0.99, 0.995, 0.998),
                 zero2small=TRUE, small=1E-6, divisor=100, delv=0.001, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{type}{What type of joint probability is to be computed;}
  \item{probs}{The joint probabilities for which to compute the coordinates. The default values represent especially useful annual return period equivalents that are useful in hydrologic risk analyses;}
  \item{zero2small}{A logical controlling whether precise zero value for probability are converted to a \code{small} value and precise unity values for probability are converted to the value \code{1 - small}; this logical is useful if transformation from probability space into standard normal variates or \emph{Gumbel reduced variates} (GRV; see function \code{prob2grv()} in package \pkg{lmomco}) is later desired by the user for attendant graphics (see \bold{Examples} section);}
  \item{small}{The value for \emph{small} described for \code{zero2small};}
  \item{divisor}{A divisor on a computation of a \eqn{\Delta} for incrementing through the \eqn{v} domain as part of the coordinate computation (see source code);}
  \item{delv}{A \eqn{\Delta v} for setup of the incrementing through the \eqn{v} domain as part of the coordinate computation (see source code); and}
  \item{...}{Additional arguments to pass to the \code{\link{duCOP}} function of \pkg{copBasic} or \code{uniroot()} function.}
}
\value{
  An \R \code{list} is returned with elements each of the given \code{probs}.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{diagCOPatf}}, \code{\link{duCOP}}, \code{\link{jointCOP}}, \code{\link{joint.curvesCOP}}, \code{\link{level.curvesCOP2}}
}
\examples{
# See Note for joint.curvesCOP()
\dontrun{
# Approach the joint curves from both "with respect two" perspectives---results same.
JCvwrtu <- joint.curvesCOP( cop=PSP, prob=0.98)$"0.98"
JCuwrtv <- joint.curvesCOP2(cop=PSP, prob=0.98)$"0.98"; lim <- c(2,5)
plot(qnorm(JCvwrtu$U), qnorm(JCvwrtu$V), type="l", lwd=6, col=8, xlim=lim, ylim=lim,
     xlab="STANDARD NORMAL VARIATE OF U", ylab="STANDARD NORMAL VARIATE OF V")
lines(qnorm(JCuwrtv$U), qnorm(JCuwrtv$V), col=2, lwd=2)
mtext("98th Joint Percentile Level Curve for PSP Copula")#}
}
\keyword{joint probability}

\encoding{utf8}
\name{tailconCOP}
\alias{tailconCOP}
\title{The Tail Concentration Function of a Copula}
\description{
Compute the \emph{tail concentration function} (\eqn{q_\mathbf{C}}) of a copula \eqn{\mathbf{C}(u,v)} (\code{\link{COP}}) or diagonal (\code{\link{diagCOP}}) of a copula \eqn{\delta_\mathbf{C}(t) = \mathbf{C}(t,t)}  according to Durante and Semp (2015, p. 74):
\deqn{q_\mathbf{C}(t) = \frac{\mathbf{C}(t,t)}{t} \cdot \mathbf{1}_{[0,0.5)}  + \frac{1 - 2t + \mathbf{C}(t,t)}{1-t} \cdot \mathbf{1}_{[0.5, 1]}\mbox{\quad or}}
\deqn{q_\mathbf{C}(t) = \frac{\delta_\mathbf{C}(t)}{t} \cdot \mathbf{1}_{[0,0.5)} + \frac{1 - 2t + \delta_\mathbf{C}(t)}{1-t}   \cdot \mathbf{1}_{[0.5, 1]}\mbox{,}}
where \eqn{t} is a nonexceedance probability on the margins and \eqn{\mathbf{1}(.)} is an \emph{indicator function} scoring 1 if condition is true otherwise zero on what interval \eqn{t} resides: \eqn{t \in [0,0.5)} or \eqn{t \in [0.5,1]}. The \eqn{q_\mathbf{C}(t; \mathbf{M}) = 1} for all \eqn{t} for the \code{\link{M}} copula and  \eqn{q_\mathbf{C}(t; \mathbf{W}) = 0} for all \eqn{t} for the \code{\link{W}} copula. Lastly, the function is related to the \emph{Blomqvist Beta} (\eqn{\beta_\mathbf{C}}; \code{\link{blomCOP}}) by
\deqn{q_\mathbf{C}(0.5) = (1 + \beta_\mathbf{C})/2\mbox{,}}
where \eqn{\beta_\mathbf{C} = 4\mathbf{C}(0.5, 0.5) - 1}. Lastly, the \eqn{q_\mathbf{C}(t)} for \eqn{0,1 = t} is \code{NaN} and no provision for alternative return is made. Readers are asked to note some of the mathematical similarity in this function to Blomqvist Betas in \code{\link{blomCOPss}} in regards to tail dependency.
}
\usage{
tailconCOP(t, cop=NULL, para=NULL, ...)
}
\arguments{
  \item{t}{Nonexceedance probabilities \eqn{t};}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula; and}
  \item{...}{Additional arguments to pass to the copula function.}
}
\value{
  Value(s) for \eqn{q_\mathbf{C}} are returned.
}
\references{
Durante, F., and Sempi, C., 2015, Principles of copula theory: Boca Raton, CRC Press, 315 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{taildepCOP}}, \code{\link{tailordCOP}}}
\examples{
tailconCOP(0.5, cop=PSP) == (1 + blomCOP(cop=PSP)) / 2 # TRUE
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{copula (tail characteristics)}
\keyword{copula (tail properties)}

\encoding{utf8}
\name{isCOP.permsym}
\alias{isCOP.permsym}
\title{Is a Copula Permutation Symmetric}
\description{
Numerically set a logical whether a copula is \emph{symmetric} (Nelsen, 2006, p. 38), or has \emph{exchangeable} variables, or is \emph{permutation symmetric} (Joe, 2014, p. 66) (refer also to \emph{radial symmetric}, \code{\link{isCOP.radsym}}). A copula \eqn{\mathbf{C}(u,v)} is permutation symmetric if and only if for any \eqn{\{u,v\} \in [0,1]} the following holds
 \deqn{\mathbf{C}(u,v) = \mathbf{C}(v, u)\mbox{.}}
The computation is (can be) CPU intensive and refer also to the extensive documentation and computational examples within \code{\link{isCOP.radsym}} (secs. \bold{Note} and \bold{Examples}) and the conceptual contributions of the \emph{L-comoments} (\code{\link{lcomCOP}}) to symmetry.
}
\usage{
isCOP.permsym(cop=NULL, para=NULL, delta=0.005, tol=1e-4, ...)
}
\arguments{
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters, if needed, to pass to the copula;}
  \item{delta}{The increment of  \eqn{\{u,v\} \mapsto [0+\Delta\delta, 1-\Delta\delta, \Delta\delta]};}
  \item{tol}{A tolerance on the check for symmetry, default 1 part in 10,000, which is the test for the \eqn{\equiv 0} (zero equivalence, see source code); and}
  \item{...}{Additional arguments to pass to the copula.}
}
\value{
  A logical \code{TRUE} or \code{FALSE} is returned.
}
\references{
Joe, H., 2014, Dependence modeling with copulas: Boca Raton, CRC Press, 462 p.

Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{LzCOPpermsym}}, \code{\link{isCOP.radsym}}}
\examples{
\dontrun{
isCOP.permsym(cop=GHcop, para=1.3) # TRUE}
}
\keyword{copula (characteristics)}
\keyword{copula (properties)}
\keyword{copula (symmetry)}
\keyword{L-comoments}
\keyword{Lcomoments}

\encoding{utf8}
\name{med.regressCOP2}
\alias{med.regressCOP2}
\title{Perform Median Regression using a Copula by Numerical Derivative Method for U with respect to V}
\description{
Perform \emph{median regression} of a copula (Nelsen, 2006, pp. 217--218) by inversion of numerical derivatives of the copula (\code{\link{derCOPinv2}}). The documentation for \code{\link{qua.regressCOP2}} provides mathematical details.
}
\usage{
med.regressCOP2(v=seq(0.01,0.99, by=0.01), cop=NULL, para=NULL, level=NA, ...)
}
\arguments{
  \item{v}{Nonexceedance probability \eqn{v} in the \eqn{Y} direction;}
  \item{cop}{A copula function;}
  \item{para}{Vector of parameters or other data structure, if needed, to pass to the copula;}
  \item{level}{The level of the prediction interval to compute. For example, \code{level=0.95} will compute the 95-percent prediction interval as will \code{level=0.05} because internally a reflection check is made; and}
  \item{...}{Additional arguments to pass such \code{\link{qua.regressCOP2}} and \code{\link{derCOPinv2}} that are called in succession.}
}
\value{
  An \R \code{data.frame} of the median regressed probabilities of \eqn{U} and provided \eqn{V} values is returned. Note: if \code{level} is used, the column ordering of the returned \code{data.frame} changes---please access the columns by the named idiom. The lower- and upper-prediction interval bounds are contained in the columns respectively titled \code{Ulwr} and \code{Uupr} to mimic nomenclature somewhat of function \code{predict.lm()} in \R.
}
\references{
Nelsen, R.B., 2006, An introduction to copulas: New York, Springer, 269 p.
}
\author{W.H. Asquith}
\seealso{\code{\link{med.regressCOP}}, \code{\link{qua.regressCOP2}}, \code{\link{qua.regressCOP.draw}}
}
\examples{
# See examples under med.regressCOP
}
\keyword{median regression}
\keyword{quantile regression}
\concept{quantile regression}
